@@ -1,7 +1,7 @@
 /*
-OpenIO SDS cluster
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+OpenIO SDS cluster conscience
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -83,19 +83,19 @@ conscience_srv_compute_score(struct conscience_srv_s *service,
 	struct conscience_srvtype_s *srvtype;
 	gdouble d;
 
-	char *getField(char *b, char *f) {
+	gchar *getField(const char *b, const char *f) {
 		char str_name[128];
 		struct service_tag_s *pTag;
 
 		if (!f) {
-			DEBUG("[%s/%s/] NULL tag wanted", conscience->ns_info.name,
+			GRID_DEBUG("[%s/%s/] NULL tag wanted", conscience->ns_info.name,
 					srvtype->type_name);
 			return NULL;
 		}
 		g_snprintf(str_name,sizeof(str_name),"%s.%s", b, f);
 		pTag = conscience_srv_get_tag(service, str_name);
 		if (!pTag) {
-			DEBUG("[%s/%s/] Undefined tag wanted: %s",
+			GRID_DEBUG("[%s/%s/] Undefined tag wanted: %s",
 					conscience->ns_info.name, srvtype->type_name, f);
 			return NULL;
 		}
@@ -111,17 +111,17 @@ conscience_srv_compute_score(struct conscience_srv_s *service,
 		case STVT_BUF:
 			return g_strdup(pTag->value.buf);
 		}
-		DEBUG("[%s/%s/] invalid tag value: %s",
+		GRID_DEBUG("[%s/%s/] invalid tag value: %s",
 				conscience->ns_info.name, srvtype->type_name, f);
 		return NULL;
 	}
-	char *getStat(char *f) {
+	gchar *getStat(const char *f) {
 		return getField("stat", f);
 	}
-	char *getTag(char *f) {
+	gchar *getTag(const char *f) {
 		return getField("tag", f);
 	}
-	accessor_f *getAcc(char *b) {
+	accessor_f *getAcc(const char *b) {
 		if (!b)
 			return NULL;
 		switch (*b) {
