@@ -1,7 +1,7 @@
 /*
 OpenIO SDS cluster
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -26,33 +26,12 @@ License along with this library.
 #include <curl/curl.h>
 
 #include <metautils/lib/metautils.h>
+#include <metautils/lib/common_variables.h>
 #include <cluster/module/module.h>
 #include <core/http_internals.h>
 
 #include "gridcluster.h"
 
-#define MAX_REQ_LENGTH 1024
-#define CONNECT_TIMEOUT 5000
-#define SOCKET_TIMEOUT 5000
-#define BUF (wrkParam+writen)
-#define LEN (sizeof(wrkParam)-writen-1)
-#define MANAGE_ERROR(Req,Resp,Error) do {\
-	if (Resp.data_size > 0 && Resp.data)\
-		GSETERROR(Error, "Error from agent : %.*s", Resp.data_size, (char*)(Resp.data));\
-	else\
-		GSETERROR(Error, "Error from agent : (no response)");\
-} while (0)
-#define NS_WORM_OPT_NAME "worm"
-#define NS_CONTAINER_MAX_SIZE_NAME "container_max_size"
-#define NS_STORAGE_POLICY_NAME "storage_policy"
-#define NS_CHUNK_SIZE_NAME "chunk_size"
-#define NS_STATE_NAME "state"
-#define NS_WORM_OPT_VALUE_ON "on"
-#define NS_COMPRESS_OPT_NAME "compression"
-#define NS_COMPRESS_OPT_VALUE_ON "on"
-
-/* -------------------------------------------------------------------------- */
-
 GError *
 conscience_get_namespace (const char *ns, struct namespace_info_s **out)
 {
@@ -60,7 +39,7 @@ conscience_get_namespace (const char *ns, struct namespace_info_s **out)
 	g_assert (out != NULL);
 	*out = NULL;
 
-	GString *body = g_string_new ("");
+	GString *body = g_string_sized_new (2048);
 	CURL *h = _curl_get_handle_proxy ();
 	GError *err = oio_proxy_call_conscience_info (h, ns, body);
 	curl_easy_cleanup (h);
@@ -145,6 +124,9 @@ conscience_push_service (const char *ns, struct service_info_s *si)
 	g_assert (ns != NULL);
 	g_assert (si != NULL);
 
+	if (!*ns || !si->type[0] || !metautils_addr_valid_for_connect(&si->addr))
+		return BADREQ("Invalid service ns, type or address");
+
 	struct oio_cs_client_s *cs = oio_cs_client__create_proxied (ns);
 
 	/* convert the <service_info_t> into a <struct oio_cs_registration_s> */
@@ -185,6 +167,8 @@ conscience_remove_services(const char *ns, const char *type)
 {
 	g_assert (ns != NULL);
 	g_assert (type != NULL);
+	if (!*ns || !*type)
+		return BADREQ("Invalid type or NS");
 
 	struct oio_cs_client_s *cs = oio_cs_client__create_proxied (ns);
 	GError *err = oio_cs_client__flush_services (cs, type);
@@ -197,6 +181,8 @@ conscience_remove_services(const char *ns, const char *type)
 GError *
 register_namespace_service(const struct service_info_s *si)
 {
+	g_assert(si != NULL);
+
 	struct service_info_s *si_copy = service_info_dup(si);
 	si_copy->score.value = SCORE_UNSET;
 	si_copy->score.timestamp = oio_ext_real_time () / G_TIME_SPAN_SECOND;
@@ -211,140 +197,39 @@ register_namespace_service(const struct service_info_s *si)
 void
 metautils_srvinfo_ensure_tags (struct service_info_s *si)
 {
-	if (!si->tags)
-		return ;
+	if (!si || !si->tags)
+		return;
 
 	if (!service_info_get_tag (si->tags, "stat.cpu"))
 		service_tag_set_value_float(service_info_ensure_tag (
 					si->tags, "stat.cpu"), 100.0 * oio_sys_cpu_idle ());
 
-	gchar vol[512];
+	gchar vol[512] = "";
 	struct service_tag_s *tag = service_info_get_tag (si->tags, "tag.vol");
-	if (tag) {
-		if (service_tag_get_value_string (tag, vol, sizeof(vol), NULL)) {
-			if (!service_info_get_tag(si->tags, "stat.io"))
-				service_tag_set_value_float (service_info_ensure_tag(
-							si->tags, "stat.io"), 100.0 * oio_sys_io_idle (vol));
-			if (!service_info_get_tag(si->tags, "stat.space"))
-				service_tag_set_value_float (service_info_ensure_tag (
-							si->tags, "stat.space"), 100.0 * oio_sys_space_idle (vol));
-		}
-	}
+	if (!tag || !service_tag_get_value_string (tag, vol, sizeof(vol), NULL))
+		return;
+
+	if (!service_info_get_tag(si->tags, "stat.io"))
+		service_tag_set_value_float (service_info_ensure_tag(
+					si->tags, "stat.io"), 100.0 * oio_sys_io_idle (vol));
+	if (!service_info_get_tag(si->tags, "stat.space"))
+		service_tag_set_value_float (service_info_ensure_tag (
+					si->tags, "stat.space"), 100.0 * oio_sys_space_idle (vol));
 }
 
 /* -------------------------------------------------------------------------- */
 
-static gint64
-_gba_to_int64(GByteArray *gba, gboolean def)
-{
-	if (!gba)
-		return def;
-	gchar *str = g_alloca(gba->len + 1);
-	memset(str, 0, gba->len + 1);
-	memcpy(str, gba->data, gba->len);
-	return g_ascii_strtoll(str, NULL, 10);
-}
-
-static gboolean
-_gba_to_bool(GByteArray *gba, gboolean def)
-{
-	if (!gba || !gba->data || !gba->len)
-		return def;
-	if (!gba->data[ gba->len - 1 ])
-		return oio_str_parse_bool((gchar*)gba->data, def);
-	gchar *str = g_alloca(gba->len + 1);
-	memset(str, 0, gba->len + 1);
-	memcpy(str, gba->data, gba->len);
-	return oio_str_parse_bool(str, def);
-}
-
-static GByteArray *
-namespace_param_gba(const namespace_info_t* ns_info, const gchar *ns_name,
-		const gchar *param_name)
-{
-	return namespace_info_get_srv_param_gba(ns_info, ns_name, NULL, param_name);
-}
-
-gchar*
-gridcluster_get_nsinfo_strvalue(struct namespace_info_s *nsinfo,
-		const gchar *key, const gchar *def)
-{
-	GByteArray *value;
-
-	if (!nsinfo || !nsinfo->options)
-		return g_strdup(def);
-
-	value = g_hash_table_lookup(nsinfo->options, key);
-	if (!value)
-		return g_strdup(def);
-
-	return g_strndup((gchar*)value->data, value->len);
-}
-
-gint64
-gridcluster_get_nsinfo_int64(struct namespace_info_s *nsinfo,
-		const gchar* key, gint64 def)
-{
-	return namespace_info_get_srv_param_i64(nsinfo, NULL, NULL, key, def);
-}
-
-static gsize
-namespace_get_size(namespace_info_t *ns_info, const gchar *name, gsize def)
-{
-	return (gsize) gridcluster_get_nsinfo_int64(ns_info, name, def);
-}
-
-gboolean
-namespace_in_worm_mode(namespace_info_t* ns_info)
-{
-	GByteArray *val = namespace_param_gba(ns_info, NULL, NS_WORM_OPT_NAME);
-	return _gba_to_bool(val, FALSE);
-}
-
-gchar *
-namespace_get_state(namespace_info_t* ns_info)
-{
-	return gridcluster_get_nsinfo_strvalue(ns_info, NS_STATE_NAME,
-			NS_STATE_VALUE_STANDALONE);
-}
-
-gint64
-namespace_container_max_size(namespace_info_t* ns_info)
-{
-	GByteArray *val = namespace_param_gba(ns_info, NULL, NS_CONTAINER_MAX_SIZE_NAME);
-	return _gba_to_int64(val, -1);
-}
-
-gint64
-namespace_chunk_size(const namespace_info_t* ns_info, const char *ns_name)
-{
-	GByteArray *val = namespace_param_gba(ns_info, ns_name,
-			NS_CHUNK_SIZE_NAME);
-	return _gba_to_int64(val, ns_info->chunk_size);
-}
-
-gchar *
-namespace_storage_policy(const namespace_info_t* ns_info, const char *ns_name)
-{
-	GByteArray *gba = namespace_param_gba(ns_info, ns_name,
-			NS_STORAGE_POLICY_NAME);
-	return !gba ? NULL : g_strndup((gchar*)gba->data, gba->len);
-}
-
 gchar*
 namespace_storage_policy_value(const namespace_info_t *ns_info, const gchar *wanted_policy)
 {
-	const gchar *policy_to_lookup = wanted_policy ?
-			wanted_policy : namespace_storage_policy(ns_info, ns_info->name);
+	gchar *policy_to_lookup = wanted_policy
+		?  g_strdup(wanted_policy) : oio_var_get_string(oio_ns_storage_policy);
+	STRING_STACKIFY(policy_to_lookup);
 
 	if (!ns_info || ns_info->storage_policy)
 		return NULL;
 
 	GByteArray *gba = g_hash_table_lookup(ns_info->storage_policy, policy_to_lookup);
-
-	if (!wanted_policy)
-		g_free((gpointer)policy_to_lookup);
-
 	return !gba ? NULL : g_strndup((gchar*)gba->data, gba->len);
 }
 
@@ -355,7 +240,7 @@ _get_token(const gchar *colon_separated_tokens, const guint token_rank)
 	gchar *token_wanted = NULL;
 
 	if (g_strv_length(tokens) < token_rank) {
-		ERROR("Cannot split string [%s] into %i ':'-separated tokens.", colon_separated_tokens, token_rank);
+		GRID_ERROR("Cannot split string [%s] into %i ':'-separated tokens.", colon_separated_tokens, token_rank);
 		goto end;
 	}
 
@@ -374,7 +259,7 @@ _get_data_security_id(const gchar *storage_policy_value)
 	gchar *data_sec_id = _get_token(storage_policy_value, 1);
 
 	if (!data_sec_id) {
-		WARN("Storage policy configuration seems to be wrong: [%s]"
+		GRID_WARN("Storage policy configuration seems to be wrong: [%s]"
 				" Correct pattern is STG_CLASS:DATA_SEC:DATA_THREAT",
 				storage_policy_value ? storage_policy_value : "NULL");
 	}
@@ -395,7 +280,7 @@ namespace_data_security_value(const namespace_info_t *ns_info, const gchar *want
 	}
 
 	if (!data_sec_val) {
-		WARN("Cannot find data security with id [%s] (namespace [%s], wanted policy [%s])",
+		GRID_WARN("Cannot find data security with id [%s] (namespace [%s], wanted policy [%s])",
 				data_sec_id, ns_info->name, wanted_policy);
 	}
 
@@ -418,54 +303,3 @@ namespace_is_storage_policy_valid(const namespace_info_t* ns_info, const gchar *
 	return TRUE;
 }
 
-gboolean
-namespace_in_compression_mode(namespace_info_t* ns_info)
-{
-	if (!ns_info || !ns_info->options)
-		return FALSE;
-	GByteArray *val = namespace_param_gba(ns_info, NULL, NS_COMPRESS_OPT_NAME);
-	gboolean res = _gba_to_bool(val, FALSE);
-	return res;
-}
-
-gsize
-namespace_get_autocontainer_src_offset(namespace_info_t* ns_info)
-{
-	return namespace_get_size(ns_info, "flat_hash_offset", 0);
-}
-
-gsize
-namespace_get_autocontainer_src_size(namespace_info_t* ns_info)
-{
-	return namespace_get_size(ns_info, "flat_hash_size", 0);
-}
-
-gsize
-namespace_get_autocontainer_dst_bits(namespace_info_t* ns_info)
-{
-	return namespace_get_size(ns_info, "flat_bitlength", 17);
-}
-
-gint64
-gridcluster_get_container_max_versions(struct namespace_info_s *nsinfo)
-{
-	/* For backward compatibility, versioning is disabled by default */
-	return gridcluster_get_nsinfo_int64(nsinfo, "meta2_max_versions", 0);
-}
-
-gint64
-gridcluster_get_keep_deleted_delay(struct namespace_info_s *nsinfo)
-{
-	return gridcluster_get_nsinfo_int64(nsinfo, "meta2_keep_deleted_delay", -1);
-}
-
-gchar *
-gridcluster_get_service_update_policy (struct namespace_info_s *nsinfo)
-{
-	const gchar *def = "meta2=KEEP|1|1|;sqlx=KEEP|1|1|";
-
-	if (!nsinfo || !nsinfo->options)
-		return g_strdup(def);
-
-	return gridcluster_get_nsinfo_strvalue (nsinfo, "service_update_policy", def);
-}
