@@ -1,7 +1,7 @@
 /*
-OpenIO SDS cluster
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+OpenIO SDS cluster CLI
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -31,17 +31,16 @@ static void
 usage(void)
 {
 	g_printerr("Usage: oio-cluster [OPTION]... <NAMESPACE>...\n\n");
-	g_printerr("  %-20s\t%s\n", "--clear-services SERVICE    ", "Clear all local RAWX reference in cluster.");
-	g_printerr("  %-20s\t%s\n", "--full,                     ", "Show full services.");
-	g_printerr("  %-20s\t%s\n", "--lb-config,                ", "Prints to stdout the namespace LB configuration ");
+	g_printerr("  %-20s\t%s\n", "--verbose,                -v", "Increases the verbosity");
+	g_printerr("  %-20s\t%s\n", "--help,                   -h", "Display the current help section");
+	g_printerr("  %-20s\t%s\n", "--full,                   -f", "Show full services.");
+	g_printerr("  %-20s\t%s\n", "--raw,                    -r", "Output in parsable mode.");
 	g_printerr("  %-20s\t%s\n", "--local-cfg,              -A", "Prints to stdout the namespaces configuration values locally configured");
 	g_printerr("  %-20s\t%s\n", "--local-ns,               -L", "Prints to stdout the namespaces locally configured");
-	g_printerr("  %-20s\t%s\n", "--local-srv               -l", "List local services monitored on this server.");
-	g_printerr("  %-20s\t%s\n", "--raw,                    -r", "Output in parsable mode.");
 	g_printerr("  %-20s\t%s\n", "--service <service desc>, -S", "Select service described by desc.");
 	g_printerr("  %-20s\t%s\n", "--set-score <[0..100]>      ", "Set and lock score for the service specified by -S.");
 	g_printerr("  %-20s\t%s\n", "--unlock-score              ", "Unlock score for the service specified by -S.");
-	g_printerr("  %-20s\t%s\n", "--verbose,                -v", "Increases the verbosity");
+	g_printerr("  %-20s\t%s\n", "--clear-services SRVTYPE    ", "Clear all services of the given type from the conscience.");
 }
 
 static void
@@ -67,54 +66,10 @@ print_formated_namespace(namespace_info_t * ns)
 	g_print("NAMESPACE INFORMATION\n");
 	g_print("\n");
 	g_print("%20s : %s\n", "Name", ns->name);
-	g_print("%20s : %"G_GINT64_FORMAT" bytes\n", "Chunk size", ns->chunk_size);
 
-	print_formatted_hashtable(ns->options, "Option");
 	print_formatted_hashtable(ns->storage_policy, "Storage Policy");
 	print_formatted_hashtable(ns->data_security, "Data Security");
 	print_formatted_hashtable(ns->service_pools, "Service Pool");
-
-	GError *err = NULL;
-	GSList *types = NULL;
-	if (NULL != (err = conscience_get_types (ns->name, &types)))
-		g_clear_error (&err);
-
-	/* dump the directory load-balancing configuration */
-	gchar *cfg = gridcluster_get_service_update_policy(ns);
-	if (!cfg) {
-		g_printerr("Invalid NSINFO\n");
-	} else {
-		struct service_update_policies_s *pol = service_update_policies_create();
-		err = service_update_reconfigure(pol, cfg);
-		g_free(cfg); cfg=NULL;
-
-		if (err) {
-			g_printerr("Invalid namespace configuration : (%d) %s\n",
-					err->code, err->message);
-			service_update_policies_destroy(pol);
-			g_clear_error(&err);
-			return;
-		} else {
-			char *tmp = NULL;
-			tmp = service_update_policies_dump(pol);
-			g_print("%20s : %s\n", "LB(srv)", tmp);
-			g_free(tmp);
-
-			for (GSList *l=types; l ;l=l->next) {
-				const char *srvtype = l->data;
-				guint count = service_howmany_replicas(pol, srvtype);
-				guint dist = service_howmany_distance(pol, srvtype);
-				g_print("%20s : %s -> %s|%u|%u\n", "", srvtype,
-						service_update_policy_to_string(
-							service_howto_update (pol, srvtype)),
-						count ? count : 1, dist ? dist : 1);
-			}
-		}
-		service_update_policies_destroy(pol);
-	}
-
-	g_slist_free_full (types, g_free);
-	g_print("\n");
 }
 
 static void
@@ -235,16 +190,13 @@ main(int argc, char **argv)
 		/* long options only */
 		{"set-score",      1, 0, 4},
 		{"unlock-score",   0, 0, 5},
-		{"full",           0, 0, 7},
+		{"full",           0, 0, 'f'},
 
 		/* both long and short */
-		{"config",         0, 0, 'c'},
 		{"clear-services", 1, 0, 'C'},
 		{"service",        1, 0, 'S'},
 		{"local-cfg",      0, 0, 'A'},
 		{"local-ns",       0, 0, 'L'},
-		{"local-srv",      0, 0, 'l'},
-		{"show",           0, 0, 's'},
 		{"raw",            0, 0, 'r'},
 		{"help",           0, 0, 'h'},
 		{"verbose",        0, 0, 'v'},
@@ -257,7 +209,7 @@ main(int argc, char **argv)
 	memset(cid_str, 0x00, sizeof(cid_str));
 	enable_debug();
 
-	while ((c = getopt_long(argc, argv, "ALsvlrC:S:h", long_options, &option_index)) > -1) {
+	while ((c = getopt_long(argc, argv, "ALvrC:S:h", long_options, &option_index)) > -1) {
 
 		switch (c) {
 			case 'A':
@@ -282,9 +234,6 @@ main(int argc, char **argv)
 				has_set_score = TRUE;
 				has_unlock_score = TRUE;
 				break;
-			case 7:
-				has_flag_full = TRUE;
-				break;
 			case 'S':
 				has_service = TRUE;
 				if (!optarg) {
@@ -293,15 +242,19 @@ main(int argc, char **argv)
 				}
 				g_strlcpy(service_desc, optarg, sizeof(service_desc));
 				break;
+			case 'f':
+				has_flag_full = TRUE;
+				break;
+			case 'h':
+				rc = 0;
+				usage();
+				goto exit_label;
 			case 'r':
 				has_raw = TRUE;
 				break;
 			case 'v':
 				oio_log_verbose();
 				break;
-			case 'h':
-				rc = 0;
-				// FALLTHROUGH
 			case '?':
 			case 0:
 			default:
