@@ -1,6 +1,6 @@
 /*
 OpenIO SDS core library
-Copyright (C) 2015 OpenIO, original work as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -16,18 +16,19 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library.
 */
 
+#include <core/oiocfg.h>
+
 #include <string.h>
-#include <stdlib.h>
 
-#include <glib.h>
+#include <core/oiolog.h>
+#include <core/oiostr.h>
 
-#include "oio_core.h"
 #include "internals.h"
 
 static struct oio_cfg_handle_s *oio_cfg_handle_DEFAULT = NULL;
 
-static gchar *
-_build_key(const gchar *ns, const gchar *what)
+gchar *
+oio_cfg_build_key(const gchar *ns, const gchar *what)
 {
 	gchar *k, *result;
 
@@ -40,13 +41,12 @@ _build_key(const gchar *ns, const gchar *what)
 static void
 config_load_ns(GHashTable *h, GKeyFile *kf, const gchar *ns)
 {
-	gchar **pk, **keys, *v;
-
-	keys = g_key_file_get_keys(kf, ns, 0, NULL);
+	gchar **keys = g_key_file_get_keys(kf, ns, 0, NULL);
 	if (keys) {
-		for (pk=keys; *pk ;pk++) {
-			v = g_key_file_get_string(kf, ns, *pk, NULL);
-			g_hash_table_insert(h, _build_key(ns, *pk), v);
+		g_hash_table_insert(h, oio_cfg_build_key(ns, "known"), g_strdup("yes"));
+		for (gchar **pk=keys; *pk ;pk++) {
+			gchar *v = g_key_file_get_string(kf, ns, *pk, NULL);
+			g_hash_table_insert(h, oio_cfg_build_key(ns, *pk), v);
 		}
 		g_strfreev(keys);
 	}
@@ -106,7 +106,7 @@ oio_cfg_list_ns(void)
 		const gchar *sk = (gchar*)k;
 		if (g_str_has_prefix(sk, "default/"))
 			continue;
-		if (!g_str_has_suffix(sk, "/conscience"))
+		if (!g_str_has_suffix(sk, "/known"))
 			continue;
 		gchar *ns = g_strndup(sk, strrchr(sk,'/')- sk);
 		g_ptr_array_add(tmp, ns);
@@ -116,16 +116,27 @@ oio_cfg_list_ns(void)
 	return (gchar**) g_ptr_array_free(tmp, FALSE);
 }
 
+GHashTable*
+oio_cfg_parse_file(const char *path)
+{
+	GHashTable *ht = g_hash_table_new_full(g_str_hash, g_str_equal,
+			g_free, g_free);
+	config_load_file(ht, path);
+	return ht;
+}
+
 GHashTable*
 oio_cfg_parse(void)
 {
-	GHashTable *ht = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+	GHashTable *ht = g_hash_table_new_full(g_str_hash, g_str_equal,
+			g_free, g_free);
 
 	// Load the system configuration
 	if (g_file_test(OIO_CONFIG_FILE_PATH, G_FILE_TEST_IS_REGULAR))
 		config_load_file(ht, OIO_CONFIG_FILE_PATH);
 
-	if (g_file_test(OIO_CONFIG_DIR_PATH, G_FILE_TEST_IS_DIR|G_FILE_TEST_EXISTS)) {
+	if (g_file_test(OIO_CONFIG_DIR_PATH,
+			G_FILE_TEST_IS_DIR|G_FILE_TEST_EXISTS)) {
 		GDir *gdir = g_dir_open(OIO_CONFIG_DIR_PATH, 0, NULL);
 		if (gdir) {
 			config_load_dir(ht, OIO_CONFIG_DIR_PATH, gdir);
@@ -133,9 +144,10 @@ oio_cfg_parse(void)
 		}
 	}
 
-	// Overwrite with the user configuration (if any) 
+	// Overwrite with the user configuration (if any)
 	if (g_get_home_dir() && OIO_CONFIG_LOCAL_PATH) {
-		gchar *local = g_strdup_printf("%s/%s", g_get_home_dir(), OIO_CONFIG_LOCAL_PATH);
+		gchar *local = g_strdup_printf("%s/%s", g_get_home_dir(),
+				OIO_CONFIG_LOCAL_PATH);
 		config_load_file(ht, local);
 		g_free(local);
 	}
@@ -153,7 +165,7 @@ oio_cfg_get_value(const gchar *ns, const gchar *what)
 		ns = "default";
 
 	GHashTable *ht;
-	gchar *key = _build_key(ns, what);
+	gchar *key = oio_cfg_build_key(ns, what);
 	gchar *value = NULL;
 	if (NULL != (ht = oio_cfg_parse())) {
 		value = g_hash_table_lookup(ht, key);
@@ -228,6 +240,140 @@ oio_cfg_handle_get (struct oio_cfg_handle_s *self, const char *ns, const char *k
 void
 oio_cfg_set_handle (struct oio_cfg_handle_s *self)
 {
+	if (oio_cfg_handle_DEFAULT)
+		oio_cfg_handle_clean(oio_cfg_handle_DEFAULT);
 	oio_cfg_handle_DEFAULT = self;
 }
 
+/* -------------------------------------------------------------------------- */
+
+struct oio_cfg_cache_handle_s
+{
+	struct oio_cfg_handle_vtable_s *vtable;
+	GHashTable *cfg;
+	GRWLock lock;
+};
+
+static GHashTable*
+_cfg_cache_get_cfg_unlocked(gpointer self)
+{
+	struct oio_cfg_cache_handle_s *cache = self;
+
+	if (cache->cfg)
+		return cache->cfg;
+
+	g_rw_lock_reader_unlock(&cache->lock);
+	g_rw_lock_writer_lock(&cache->lock);
+	/* Check again in case another thread did the update
+	 * while we were waiting */
+	if (!cache->cfg) {
+		GHashTable *new_cfg = oio_cfg_parse();
+		if (new_cfg) {
+			if (cache->cfg)
+				g_hash_table_destroy(cache->cfg);
+			cache->cfg = new_cfg;
+		} else {
+			// Silent failure
+		}
+	}
+	g_rw_lock_writer_unlock(&cache->lock);
+	g_rw_lock_reader_lock(&cache->lock);
+
+	return cache->cfg;
+}
+
+static void
+_cfg_cache_clean(struct oio_cfg_handle_s *self)
+{
+	struct oio_cfg_cache_handle_s *cache = (gpointer)self;
+	g_rw_lock_writer_lock(&cache->lock);
+	if (cache->cfg)
+		g_hash_table_destroy(cache->cfg);
+	cache->cfg = NULL;
+	g_rw_lock_writer_unlock(&cache->lock);
+	g_rw_lock_clear(&cache->lock);
+	g_free(cache);
+}
+
+static gchar **
+_cfg_cache_namespaces(struct oio_cfg_handle_s *self)
+{
+	struct oio_cfg_cache_handle_s *cache = (gpointer)self;
+	GHashTableIter iter;
+	gpointer k, v;
+	GPtrArray *tmp = g_ptr_array_sized_new(4);
+
+	g_rw_lock_reader_lock(&cache->lock);
+	GHashTable *ht = _cfg_cache_get_cfg_unlocked(self);
+	g_hash_table_iter_init(&iter, ht);
+	while (g_hash_table_iter_next(&iter, &k, &v)) {
+		const gchar *sk = (gchar*)k;
+		if (g_str_has_prefix(sk, "default/"))
+			continue;
+		if (!g_str_has_suffix(sk, "/conscience"))
+			continue;
+		gchar *ns = g_strndup(sk, strrchr(sk, '/') - sk);
+		g_ptr_array_add(tmp, ns);
+	}
+	g_rw_lock_reader_unlock(&cache->lock);
+
+	g_ptr_array_add(tmp, NULL);
+	return (gchar**) g_ptr_array_free(tmp, FALSE);
+}
+
+static gchar*
+_cfg_cache_get(struct oio_cfg_handle_s *self, const char *ns, const char *what)
+{
+	struct oio_cfg_cache_handle_s *cache = (gpointer)self;
+
+	if (!ns || !strcasecmp(ns, "default"))
+		ns = "default";
+
+	gchar *key = oio_cfg_build_key(ns, what);
+	gchar *value = NULL;
+
+	g_rw_lock_reader_lock(&cache->lock);
+	GHashTable *ht = _cfg_cache_get_cfg_unlocked(self);
+	value = g_strdup(g_hash_table_lookup(ht, key));
+	g_rw_lock_reader_unlock(&cache->lock);
+
+	g_free(key);
+	return value;
+}
+
+static struct oio_cfg_handle_vtable_s VTABLE = {
+		.clean = _cfg_cache_clean,
+		.namespaces = _cfg_cache_namespaces,
+		.get = _cfg_cache_get,
+};
+
+struct oio_cfg_handle_s *
+oio_cfg_cache_create_fragment(const char *path)
+{
+	struct oio_cfg_cache_handle_s *cache =
+			g_malloc0(sizeof(struct oio_cfg_cache_handle_s));
+	cache->vtable = &VTABLE;
+	g_rw_lock_init(&cache->lock);
+	cache->cfg = oio_cfg_parse_file(path);
+	return (struct oio_cfg_handle_s *) cache;
+}
+
+struct oio_cfg_handle_s *
+oio_cfg_cache_create(void)
+{
+	struct oio_cfg_cache_handle_s *cache =
+			g_malloc0(sizeof(struct oio_cfg_cache_handle_s));
+	cache->vtable = &VTABLE;
+	g_rw_lock_init(&cache->lock);
+	// cache->cfg will be initialized at first call
+	return (struct oio_cfg_handle_s *) cache;
+}
+
+gboolean
+oio_cfg_handle_has_ns(struct oio_cfg_handle_s *self, const char *ns)
+{
+	gchar *v = oio_cfg_handle_get(self, ns, "known");
+	gboolean rc = oio_str_parse_bool(v, FALSE);
+	if (v) g_free(v);
+	return rc;
+}
