@@ -1,6 +1,6 @@
 /*
 OpenIO SDS core library
-Copyright (C) 2015 OpenIO, original work as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -16,15 +16,13 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library.
 */
 
-#include <glib.h>
-#include <json.h>
-#include <curl/curl.h>
+#include <core/oiodir.h>
 
-#include "oiourl.h"
-#include "oiostr.h"
-#include "oioext.h"
-#include "oiodir.h"
-#include "oiolog.h"
+#include <curl/curl.h>
+#include <json-c/json.h>
+
+#include <core/oiostr.h>
+#include <core/oioext.h>
 
 #include "internals.h"
 #include "http_internals.h"
@@ -74,6 +72,21 @@ oio_directory__set_properties(struct oio_directory_s *self,
 	DIR_CALL(self, set_prop)(self, url, values);
 }
 
+GError *
+oio_directory__force(struct oio_directory_s *self,
+		const struct oio_url_s *url, const char *srvtype,
+		const char* const *values, gint64 seq)
+{
+	DIR_CALL(self, force)(self, url, srvtype, values, seq);
+}
+
+GError *
+oio_directory__unlink(struct oio_directory_s *self,
+		const struct oio_url_s *url, const char *srvtype)
+{
+	DIR_CALL(self, unlink)(self, url, srvtype);
+}
+
 /* -------------------------------------------------------------------------- */
 
 struct oio_directory_PROXY_s {
@@ -100,6 +113,13 @@ static GError * _dir_proxy_get_prop(struct oio_directory_s *self,
 static GError * _dir_proxy_set_prop(struct oio_directory_s *self,
 		const struct oio_url_s *url, const char * const *values);
 
+static GError * _dir_proxy_force(struct oio_directory_s *self,
+		const struct oio_url_s *url, const char *srvtype,
+		const char* const *values, gint64 seq);
+
+static GError * _dir_proxy_unlink(struct oio_directory_s *self,
+		const struct oio_url_s *url, const char *srvtype);
+
 static struct oio_directory_vtable_s vtable_PROXY =
 {
 	_dir_proxy_destroy,
@@ -108,6 +128,8 @@ static struct oio_directory_vtable_s vtable_PROXY =
 	_dir_proxy_link,
 	_dir_proxy_get_prop,
 	_dir_proxy_set_prop,
+	_dir_proxy_force,
+	_dir_proxy_unlink,
 };
 
 struct oio_directory_s *
@@ -281,6 +303,7 @@ _dir_proxy_get_prop(struct oio_directory_s *self, const struct oio_url_s *url,
 				fct(ctx, key, json_object_get_string(val));
 			}
 		}
+		json_object_put(json);
 	}
 
 	if (out) {
@@ -307,3 +330,37 @@ _dir_proxy_set_prop(struct oio_directory_s *self, const struct oio_url_s *url,
 	oio_url_pclean(&u);
 	return err;
 }
+
+static GError *
+_dir_proxy_force(struct oio_directory_s *self, const struct oio_url_s *url,
+		const char *srvtype, const char* const *values, gint64 seq)
+{
+	_DIR_FUNC_INIT
+
+	GError *err = NULL;
+	struct oio_url_s *u = oio_url_dup(url);
+	CURL *h = _curl_get_handle_proxy();
+	err = oio_proxy_call_reference_force(h, u, srvtype, values, seq);
+
+	curl_easy_cleanup(h);
+	oio_url_pclean(&u);
+
+	return err;
+}
+
+static GError *
+_dir_proxy_unlink(struct oio_directory_s *self, const struct oio_url_s *url,
+		const char *srvtype)
+{
+	_DIR_FUNC_INIT
+
+	GError *err = NULL;
+	struct oio_url_s *u = oio_url_dup(url);
+	CURL *h = _curl_get_handle_proxy();
+	err = oio_proxy_call_reference_unlink(h, u, srvtype);
+
+	curl_easy_cleanup(h);
+	oio_url_pclean(&u);
+
+	return err;
+}
