@@ -1,6 +1,6 @@
 /*
 OpenIO SDS core library
-Copyright (C) 2015 OpenIO, original work as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -16,20 +16,22 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library.
 */
 
+#include <core/oioext.h>
+
 #include <unistd.h>
 #include <fcntl.h>
 #include <string.h>
 #include <stdio.h>
+#include <errno.h>
 #include <sys/stat.h>
 #include <sys/vfs.h>
-#include <sys/types.h>
+
+// DO NOT REMOVE! It is required for "major" and "minor" since glibc 2.24.
 #include <sys/sysmacros.h>
-#include <errno.h>
 
-#include <glib.h>
-#include <json.h>
+#include <core/oio_core.h>
+#include <core/client_variables.h>
 
-#include "oio_core.h"
 #include "internals.h"
 
 #define PREPEND(Result,List) do { \
@@ -41,11 +43,8 @@ License along with this library.
 
 time_hook_f oio_time_monotonic = NULL;
 time_hook_f oio_time_real = NULL;
-volatile int oio_sds_default_autocreate = 0;
-volatile int oio_sds_no_shuffle = 0;
-volatile int oio_dir_no_shuffle = 0;
 
-static GSList*gslist_merge_random (GSList *l1, GSList *l2) {
+static GSList* gslist_merge_random (GSList *l1, GSList *l2) {
 	GSList *next, *result = NULL;
 	GRand *r = oio_ext_local_prng ();
 	while (l1 || l2) {
@@ -54,8 +53,7 @@ static GSList*gslist_merge_random (GSList *l1, GSList *l2) {
 				PREPEND(result,l1);
 			else
 				PREPEND(result,l2);
-		}
-		else {
+		} else {
 			if (l1)
 				PREPEND(result,l1);
 			else
@@ -108,25 +106,31 @@ gsize oio_ext_array_partition (gpointer *array, gsize len,
 	/* qualify each item, so that we call the predicate only once */
 	guchar *good = g_malloc0 (len);
 
-	for (gsize i=0; i<len ;i++)
+	guchar any = 0;
+	for (gsize i=0; i<len; i++) {
 		good[i] = 0 != ((*predicate) (array[i]));
+		any |= good[i];
+	}
 
 	/* partition the items, the predicate==TRUE first */
-	for (gsize i=0; i<len ;i++) {
-		if (good[i])
-			continue;
-		/* swap the items */
-		gchar *tmp = array[len-1];
-		array[len-1] = array[i];
-		array[i] = tmp;
-		/* swap the qualities */
-		gboolean b = good[len-1];
-		good[len-1] = good[i];
-		good[i] = b;
+	if (any) {
+		for (gsize i=0; i<len; i++) {
+			if (good[i])
+				continue;
+			/* swap the items */
+			gchar *tmp = array[len-1];
+			array[len-1] = array[i];
+			array[i] = tmp;
+			/* swap the qualities */
+			gboolean b = good[len-1];
+			good[len-1] = good[i];
+			good[i] = b;
 
-		-- len;
-		-- i;
-	}
+			-- len;
+			-- i;
+		}
+	} else
+		len = 0;
 
 	g_free (good);
 	return len;
@@ -134,21 +138,20 @@ gsize oio_ext_array_partition (gpointer *array, gsize len,
 
 GError *oio_ext_extract_json (struct json_object *obj,
 		struct oio_ext_json_mapping_s *tab) {
-	EXTRA_ASSERT (obj != NULL);
 	EXTRA_ASSERT (tab != NULL);
 	for (struct oio_ext_json_mapping_s *p=tab; p->out ;p++)
 		*(p->out) = NULL;
-	if (!json_object_is_type(obj, json_type_object))
-		return NEWERROR(400, "Not an object");
+	if (!obj || !json_object_is_type(obj, json_type_object))
+		return BADREQ("Not an object");
 	for (struct oio_ext_json_mapping_s *p=tab; p->out ;p++) {
 		struct json_object *o = NULL;
 		if (!json_object_object_get_ex(obj, p->name, &o) || !o) {
 			if (!p->mandatory)
 				continue;
-			return NEWERROR(400, "Missing field [%s]", p->name);
+			return BADREQ("Missing field [%s]", p->name);
 		}
 		if (!json_object_is_type(o, p->type))
-			return NEWERROR(400, "Invalid type for field [%s]", p->name);
+			return BADREQ("Invalid type for field [%s]", p->name);
 		*(p->out) = o;
 	}
 	return NULL;
@@ -260,7 +263,10 @@ oio_ext_set_admin (const gboolean admin)
 /* -------------------------------------------------------------------------- */
 
 # ifdef HAVE_BACKTRACE
-#warn "Backtrace enabled"
+#pragma GCC diagnostic push
+#pragma GCC diagnostic warning "-Wcpp"
+#warning "Backtrace enabled"
+#pragma GCC diagnostic pop
 #include <execinfo.h>
 #define STACK_MAX 8
 
@@ -275,7 +281,7 @@ oio_error_debug (GQuark gq, int code, const char *fmt, ...)
 	if (strv) {
 		for (int i=1; i<nbframes ;i++) {
 			if (gs->len)
-				g_string_append (gs, ",");
+				g_string_append_c (gs, ',');
 			char *s, *start = strv[i];
 			if (NULL != (s = strchr(start, '(')))
 				start = s+1;
@@ -300,14 +306,18 @@ oio_error_debug (GQuark gq, int code, const char *fmt, ...)
 #endif
 
 gint64 oio_ext_real_time (void) {
+#ifdef HAVE_EXTRA_DEBUG
 	if (oio_time_real)
 		return (*oio_time_real)();
+#endif
 	return g_get_real_time();
 }
 
 gint64 oio_ext_monotonic_time (void) {
+#ifdef HAVE_EXTRA_DEBUG
 	if (oio_time_monotonic)
 		return (*oio_time_monotonic)();
+#endif
 	return g_get_monotonic_time();
 }
 
@@ -415,7 +425,7 @@ static gdouble _compute_io_idle (guint major, guint minor) {
 	}
 
 	/* check its validity and reload if necessary */
-	if (!out->last_update || (now - out->last_update) > G_TIME_SPAN_SECOND) {
+	if (!out->last_update || (now - out->last_update) > _refresh_io_idle) {
 		FILE *fst = fopen ("/proc/diskstats", "r");
 		while (fst && !feof(fst) && !ferror(fst)) {
 			char line[1024], name[256];
@@ -508,7 +518,7 @@ static int _get_major_minor (const gchar *path, guint *pmaj, guint *pmin) {
 	}
 
 	/* maybe refresh it */
-	if (!out->last_update || (now - out->last_update) > 30 * G_TIME_SPAN_SECOND) {
+	if (!out->last_update || (now - out->last_update) > _refresh_major_minor) {
 		struct stat st = {0};
 		if (0 != stat(out->path, &st)) {
 			out = NULL;
@@ -581,7 +591,7 @@ gdouble oio_sys_cpu_idle (void) {
 
 	g_mutex_lock (&lock);
 	gint64 now = oio_ext_monotonic_time ();
-	if (!last_update || ((now - last_update) > G_TIME_SPAN_SECOND)) {
+	if (!last_update || ((now - last_update) > _refresh_cpu_idle)) {
 		FILE *fst = fopen ("/proc/stat", "r");
 		while (fst && !feof(fst) && !ferror(fst)) {
 			char line[1024];
