@@ -1,6 +1,6 @@
 /*
 OpenIO SDS core library
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -18,14 +18,15 @@ License along with this library.
 
 #ifndef OIO_SDS__sdk__http_internals_h
 # define OIO_SDS__sdk__http_internals_h 1
+
+#include <glib.h>
+#include <curl/curl.h>
+#include <core/oiourl.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-# ifndef  OIOSDS_http_agent
-#  define OIOSDS_http_agent "OpenIO-SDS/SDK-2.0"
-# endif
-
 # ifdef M2V2_ADMIN_SIZE
 #  define OIO_SDS_CONTAINER_USAGE M2V2_ADMIN_SIZE
 # else
@@ -65,9 +66,6 @@ void oio_headers_clear (struct oio_headers_s *h);
 void oio_headers_add (struct oio_headers_s *h,
 		const char *k, const char *v);
 
-void oio_headers_add_int64 (struct oio_headers_s *h,
-		const char *k, gint64 i64);
-
 /* --------------------------------------------------------------------------
  * PROXY
  * Wrappers for CURL operations toward the proxy.
@@ -117,6 +115,12 @@ GError *oio_proxy_call_reference_get_properties(CURL *h, struct oio_url_s *u,
 GError * oio_proxy_call_reference_set_properties(CURL *h, struct oio_url_s *u,
 		const char* const *values);
 
+GError * oio_proxy_call_reference_force(CURL *h, struct oio_url_s *u,
+		const char *t, const char* const *values, gint64 seq);
+
+GError * oio_proxy_call_reference_unlink(CURL *h, struct oio_url_s *u,
+		const char *t);
+
 /* container */
 
 /* Links the meta2 then triggers container creation */
@@ -136,6 +140,8 @@ GError * oio_proxy_call_container_set_properties (CURL *h,
 GError * oio_proxy_call_content_show (CURL *h, struct oio_url_s *u,
 		GString *out, gchar ***hout);
 
+GError * oio_proxy_call_content_drain(CURL *h, struct oio_url_s *u);
+
 GError * oio_proxy_call_content_delete (CURL *h, struct oio_url_s *u);
 
 GError * oio_proxy_call_content_link (CURL *h, struct oio_url_s *u,
