@@ -1,7 +1,7 @@
 /*
 OpenIO SDS core library
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -17,27 +17,27 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library.
 */
 
+#include <core/http_put.h>
+
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 #include <poll.h>
-
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/ip.h>
 #include <netinet/tcp.h>
 
-#include <glib.h>
 #include <glib/gstdio.h>
-
 #include <curl/curl.h>
 #include <curl/multi.h>
 
+#include <core/client_variables.h>
+#include <core/oioext.h>
+#include <core/oiolog.h>
+
 #include "internals.h"
-#include "oioext.h"
-#include "oiolog.h"
-#include "http_put.h"
 #include "http_internals.h"
 
 enum http_single_put_e
@@ -87,8 +87,8 @@ struct http_put_s
 
 	CURLM *mhandle;
 
-	long timeout_cnx;
-	long timeout_op;
+	long timeout_cnx;  // milliseconds
+	long timeout_op;  // milliseconds
 
 	/* how many bytes are announced to the server
 	 *   <0 : streaming with transfer-encoding=chunked
@@ -159,8 +159,8 @@ http_put_create(gint64 content_length, gint64 soft_length)
 	p->dests = NULL;
 	p->mhandle = curl_multi_init();
 	p->buffer_tail = g_queue_new();
-	p->timeout_cnx = 60;
-	p->timeout_op = 60;
+	p->timeout_cnx = oio_client_rawx_timeout_cnx * 1000L;  // seconds to ms
+	p->timeout_op = oio_client_rawx_timeout_req * 1000L;  // seconds to ms
 	p->content_length = content_length;
 	p->remaining_length = p->soft_length = soft_length;
 	p->state = HTTP_WHOLE_BEGIN;
@@ -454,8 +454,8 @@ _start_upload(struct http_put_s *p)
 		dest->handle = _curl_get_handle_blob();
 		EXTRA_ASSERT(dest->handle != NULL);
 
-		curl_easy_setopt(dest->handle, CURLOPT_CONNECTTIMEOUT, p->timeout_cnx);
-		curl_easy_setopt(dest->handle, CURLOPT_TIMEOUT, p->timeout_op);
+		curl_easy_setopt(dest->handle, CURLOPT_CONNECTTIMEOUT_MS, p->timeout_cnx);
+		curl_easy_setopt(dest->handle, CURLOPT_TIMEOUT_MS, p->timeout_op);
 
 		curl_easy_setopt(dest->handle, CURLOPT_PRIVATE, dest);
 		curl_easy_setopt(dest->handle, CURLOPT_URL, dest->url);
@@ -617,26 +617,29 @@ http_put_step (struct http_put_s *p)
 			__FUNCTION__, count_dests, count_up, count_waiting_for_data);
 
 	if (count_up) {
-		fd_set fdread, fdwrite, fdexcep;
 		int maxfd = -1;
-		struct timeval tv = {1,0};
-		FD_ZERO(&fdread);
-		FD_ZERO(&fdwrite);
-		FD_ZERO(&fdexcep);
-
 		long timeout = 0;
+		struct timeval tv = {1,0};
+		fd_set fdread = {}, fdwrite = {}, fdexcep = {};
+
 		curl_multi_timeout (p->mhandle, &timeout);
-		if (timeout < 1000) {
-			tv.tv_sec = 0;
-			tv.tv_usec = timeout * 1000;
-		}
-		curl_multi_fdset(p->mhandle, &fdread, &fdwrite, &fdexcep, &maxfd);
+
+		/* timeout not set, libcurl recommend to wait for a few seconds */
+		if (timeout < 0)
+			timeout = 1000;
+
+		/* No need to wait if actions are ready */
+		if (timeout > 0) {
+			tv.tv_sec = timeout / 1000;
+			tv.tv_usec = (timeout * 1000) % 1000000;
+			curl_multi_fdset(p->mhandle, &fdread, &fdwrite, &fdexcep, &maxfd);
 
 retry:
-		rc = select(maxfd+1, &fdread, &fdwrite, &fdexcep, &tv);
-		if (rc < 0) {
-			if (errno == EINTR) goto retry;
-			return SYSERR("select() error: (%d) %s", errno, strerror(errno));
+			rc = select(maxfd+1, &fdread, &fdwrite, &fdexcep, &tv);
+			if (rc < 0) {
+				if (errno == EINTR) goto retry;
+				return SYSERR("select() error: (%d) %s", errno, strerror(errno));
+			}
 		}
 	}
 
@@ -716,7 +719,7 @@ _curl_set_sockopt_blob (void *u, curl_socket_t fd, curlsocktype event)
 }
 
 /* Overrides the default setsockopt() for proxy connections.
-   SO_LINGER is now set. */
+ * SO_LINGER is now set. */
 static int
 _curl_set_sockopt_proxy (void *u, curl_socket_t fd, curlsocktype event)
 {
@@ -737,11 +740,15 @@ _curl_get_handle_blob (void)
 	curl_easy_setopt (h, CURLOPT_FORBID_REUSE, 1L);
 	curl_easy_setopt (h, CURLOPT_NOSIGNAL, 1L);
 	curl_easy_setopt (h, CURLOPT_FRESH_CONNECT, 1L);
-	curl_easy_setopt (h, CURLOPT_USERAGENT, OIOSDS_http_agent);
+	curl_easy_setopt (h, CURLOPT_USERAGENT, oio_core_http_user_agent);
 	curl_easy_setopt (h, CURLOPT_NOPROGRESS, 1L);
 	curl_easy_setopt (h, CURLOPT_PROXY, "");
 	curl_easy_setopt (h, CURLOPT_SOCKOPTDATA, NULL);
 	curl_easy_setopt (h, CURLOPT_SOCKOPTFUNCTION, _curl_set_sockopt_blob);
+	if (oio_socket_rawx_buflen > 0) {
+		unsigned long opt = oio_socket_rawx_buflen;
+		curl_easy_setopt (h, CURLOPT_BUFFERSIZE, opt);
+	}
 	if (GRID_TRACE2_ENABLED()) {
 		curl_easy_setopt (h, CURLOPT_DEBUGFUNCTION, _trace);
 		curl_easy_setopt (h, CURLOPT_VERBOSE, 1L);
@@ -753,7 +760,7 @@ CURL *
 _curl_get_handle_proxy (void)
 {
 	CURL *h = curl_easy_init ();
-	curl_easy_setopt (h, CURLOPT_USERAGENT, OIOSDS_http_agent);
+	curl_easy_setopt (h, CURLOPT_USERAGENT, oio_core_http_user_agent);
 	curl_easy_setopt (h, CURLOPT_NOSIGNAL, 1L);
 	curl_easy_setopt (h, CURLOPT_TCP_NODELAY, 1L);
 	curl_easy_setopt (h, CURLOPT_NOPROGRESS, 1L);
@@ -762,6 +769,10 @@ _curl_get_handle_proxy (void)
 	curl_easy_setopt (h, CURLOPT_FRESH_CONNECT, 0L);
 	curl_easy_setopt (h, CURLOPT_SOCKOPTDATA, NULL);
 	curl_easy_setopt (h, CURLOPT_SOCKOPTFUNCTION, _curl_set_sockopt_proxy);
+	if (oio_socket_proxy_buflen > 0) {
+		unsigned long opt = oio_socket_proxy_buflen;
+		curl_easy_setopt (h, CURLOPT_BUFFERSIZE, opt);
+	}
 	if (GRID_TRACE2_ENABLED()) {
 		curl_easy_setopt (h, CURLOPT_DEBUGFUNCTION, _trace);
 		curl_easy_setopt (h, CURLOPT_VERBOSE, 1L);
