@@ -1,6 +1,6 @@
 /*
 OpenIO SDS core library
-Copyright (C) 2015 OpenIO, original work as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -29,6 +29,14 @@ extern "C" {
 #  define EXTRA_ASSERT(X)
 # endif
 
+# ifdef __GNUC__
+#  define likely(x)       __builtin_expect((x),1)
+#  define unlikely(x)     __builtin_expect((x),0)
+# else
+#  define likely(x)       (x)
+#  define unlikely(x)     (x)
+# endif
+
 #define LIMIT_LENGTH_REQID 64
 
 #define OLDEST(now,delay) (((now)>(delay)) ? ((now)-(delay)) : 0)
@@ -39,18 +47,25 @@ extern "C" {
 
 #define CODE_IS_NETWORK_ERROR(C) ((C) > ERRCODE_PARAM && (C) <= CODE_NETWORK_ERROR)
 
-#define CODE_IS_OK(C)            (((C) >= CODE_FINAL_OK) && ((C) < CODE_BEACON_REDIRECT))
-#define CODE_IS_TEMP(C)          (((C) >= CODE_TEMPORARY) && ((C) < CODE_FINAL_OK))
-#define CODE_IS_FINAL(C)         ((C) == CODE_FINAL_OK || (!CODE_IS_OK(C) && !CODE_IS_TEMP(C)))
+#define CODE_IS_OK(C)       (((C) >= CODE_FINAL_OK) && ((C) < CODE_BEACON_REDIRECT))
+#define CODE_IS_TEMP(C)     (((C) >= CODE_TEMPORARY) && ((C) < CODE_FINAL_OK))
+#define CODE_IS_FINAL(C)    ((C) == CODE_FINAL_OK || (!CODE_IS_OK(C) && !CODE_IS_TEMP(C)))
 #define CODE_IS_REDIRECT(C) (((C) > CODE_BEACON_REDIRECT) && ((C) < CODE_BEACON_ERROR))
 
-#define CODE_IS_NOTFOUND(C) ((C)==CODE_CONTAINER_NOTFOUND \
+#define CODE_IS_RETRY(C) ( \
+		   ((C) == CODE_EXCESSIVE_LOAD) \
+		|| ((C) == CODE_UNAVAILABLE) \
+		|| ((C) == CODE_TOOMANY_REDIRECT))
+
+#define CODE_IS_NOTFOUND(C) ( \
+		   (C)==CODE_CONTAINER_NOTFOUND \
 		|| (C)==CODE_SRV_NOLINK \
 		|| (C)==CODE_USER_NOTFOUND \
 		|| (C)==CODE_ACCOUNT_NOTFOUND \
 		|| (C)==CODE_CONTENT_NOTFOUND)
 
-#define CODE_IS_NSIMPOSSIBLE(C) ((C)==CODE_POLICY_NOT_SATISFIABLE \
+#define CODE_IS_NSIMPOSSIBLE(C) ( \
+		   (C)==CODE_POLICY_NOT_SATISFIABLE \
 		|| (C)==CODE_POLICY_NOT_SUPPORTED \
 		|| (C)==CODE_NAMESPACE_NOTMANAGED)
 
@@ -68,6 +83,12 @@ extern "C" {
 	VTABLE_CHECK(self,T,F); \
 	return VTABLE_CALL_NOCHECK(self,T,F)
 
+#define VARIABLE_PERIOD_DECLARE() \
+	static volatile guint tick = 0;
+
+#define VARIABLE_PERIOD_SKIP(period) \
+	((period <= 0) || (0 != ((tick++) % MAX(1,period))))
+
 #define ADAPTIVE_PERIOD_DECLARE() \
 	static volatile gboolean already_succeeded = FALSE; \
 	static volatile guint tick_reload = 0; \
@@ -118,8 +139,8 @@ enum {
 	CODE_NOT_ALLOWED = 403,
 	CODE_NOT_FOUND = 404,
 
-	/*  */
-	CODE_SERVICE_NOTFOUND,
+	/* Used when in WORM mode and trying to delete */
+	CODE_METHOD_NOTALLOWED = 405,
 
 	/*  */
 	CODE_USER_NOTFOUND,
@@ -137,6 +158,7 @@ enum {
 	CODE_CONTENT_PRECONDITION = 424,
 	CODE_CONTENT_CORRUPTED = 425,
 	CODE_CONTENT_DELETED = 426,
+	CODE_CONTENT_DRAINED = 427,
 	CODE_CONTAINER_MIGRATED = 430,
 	CODE_CONTAINER_NOTFOUND = 431,
 	CODE_CONTAINER_CLOSED = 432,
@@ -183,7 +205,8 @@ enum {
 	HTTP_CODE_NOT_FOUND          = 404,
 	HTTP_CODE_METHOD_NOT_ALLOWED = 405,
 	HTTP_CODE_CONFLICT           = 409,
-        HTTP_CODE_PAYLOAD_TO_LARGE   = 413,
+	HTTP_CODE_GONE               = 410,
+	HTTP_CODE_PAYLOAD_TO_LARGE   = 413,
 	HTTP_CODE_INTERNAL_ERROR     = 500,
 	HTTP_CODE_NOT_IMPLEMENTED    = 501,
 	HTTP_CODE_BAD_GATEWAY        = 502,
@@ -193,12 +216,6 @@ enum {
 
 typedef gint64 (*time_hook_f) (void);
 
-/* Must the service addresses be shuffled or not when uploading chunks */
-extern volatile int oio_sds_no_shuffle;
-
-/* Must the service addresses be shuffled or not (out from the directory) */
-extern volatile int oio_dir_no_shuffle;
-
 /* Let/Set it to NULL for the system time.
  * Microsecond precision */
 extern time_hook_f oio_time_monotonic;
@@ -215,7 +232,14 @@ enum oio_header_case_e
 	OIO_HDRCASE_RANDOM,
 };
 
-volatile enum oio_header_case_e oio_header_case;
+enum oio_header_case_e oio_header_case;
+
+/* http://www.cse.yorku.ca/~oz/hash.html */
+guint32 djb_hash_buf(const guint8 * b, register gsize bs);
+guint32 djb_hash_str0(const gchar *str);
+
+struct hash_len_s { guint32 h; guint32 l; };
+struct hash_len_s djb_hash_str(const gchar * b);
 
 /* -------------------------------------------------------------------------- */
 
@@ -238,35 +262,6 @@ GError * oio_error_debug (GQuark gq, int code, const char *fmt, ...);
 #define BADSRVTYPE()    NEWERROR(CODE_SRVTYPE_NOTMANAGED, "Unexpected service type")
 #define SYSERR(FMT,...) NEWERROR(CODE_INTERNAL_ERROR, FMT, ##__VA_ARGS__)
 
-/* -------------------------------------------------------------------------- */
-
-struct oio_cfg_handle_s;
-
-struct oio_cfg_handle_vtable_s
-{
-	void (*clean) (struct oio_cfg_handle_s *self);
-	gchar** (*namespaces) (struct oio_cfg_handle_s *self);
-	gchar* (*get) (struct oio_cfg_handle_s *self, const char *ns, const char *k);
-};
-
-struct oio_cfg_handle_abstract_s
-{
-	struct oio_cfg_handle_vtable_s *vtable;
-};
-
-/* wraps self->clean() */
-void oio_cfg_handle_clean (struct oio_cfg_handle_s *self);
-
-/* wraps self->namespaces() */
-gchar ** oio_cfg_handle_namespaces (struct oio_cfg_handle_s *self);
-
-/* wraps self->get(ns, k) */
-gchar * oio_cfg_handle_get (struct oio_cfg_handle_s *self,
-		const char *ns, const char *k);
-
-/* Replaces the default handle to manage configuration by yourself. */
-void oio_cfg_set_handle (struct oio_cfg_handle_s *self);
-
 #ifdef __cplusplus
 }
 #endif
