@@ -1,6 +1,6 @@
 /*
 OpenIO SDS load-balancing
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -16,21 +16,26 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library.
 */
 
+#include <core/oiolb.h>
+
 #include <string.h>
 
-#include <glib.h>
-#include <json.h>
+#include <json-c/json.h>
+
+#include <core/oiostr.h>
+#include <core/oioext.h>
+#include <core/oiolog.h>
 
-#include "oiolb.h"
-#include "oiostr.h"
-#include "oioext.h"
-#include "oiolog.h"
 #include "internals.h"
 
 typedef guint32 generation_t;
 
 #define OIO_LB_SHUFFLE_JUMP ((1UL << 31) - 1)
+#define OIO_LB_LOC_LEVELS 4
+#define OIO_LB_BITS_PER_LOC_LEVEL (sizeof(oio_location_t)*8/OIO_LB_LOC_LEVELS)
 
+/* This special target matches any service, any location. */
+#define OIO_LB_JOKER_SVC_TARGET "__any_slot"
 
 typedef guint16 oio_refcount_t;
 
@@ -38,14 +43,14 @@ struct oio_lb_pool_vtable_s
 {
 	void (*destroy) (struct oio_lb_pool_s *self);
 
-	guint (*poll) (struct oio_lb_pool_s *self,
+	GError* (*poll) (struct oio_lb_pool_s *self,
 			const oio_location_t * avoids,
-			oio_lb_on_id_f on_id);
+			oio_lb_on_id_f on_id, gboolean *flawed);
 
-	guint (*patch) (struct oio_lb_pool_s *self,
+	GError* (*patch) (struct oio_lb_pool_s *self,
 			const oio_location_t * avoids,
 			const oio_location_t * known,
-			oio_lb_on_id_f on_id);
+			oio_lb_on_id_f on_id, gboolean *flawed);
 
 	struct oio_lb_item_s* (*get_item) (struct oio_lb_pool_s *self,
 			const char *id);
@@ -65,22 +70,21 @@ oio_lb_pool__destroy (struct oio_lb_pool_s *self)
 	CFG_CALL(self,destroy)(self);
 }
 
-guint
+GError*
 oio_lb_pool__poll (struct oio_lb_pool_s *self,
 		const oio_location_t * avoids,
-		oio_lb_on_id_f on_id)
+		oio_lb_on_id_f on_id, gboolean *flawed)
 {
-	CFG_CALL(self,poll)(self, avoids, on_id);
+	CFG_CALL(self,poll)(self, avoids, on_id, flawed);
 }
 
-// FIXME: return a GError*
-guint
+GError*
 oio_lb_pool__patch(struct oio_lb_pool_s *self,
 		const oio_location_t * avoids,
 		const oio_location_t * known,
-		oio_lb_on_id_f on_id)
+		oio_lb_on_id_f on_id, gboolean *flawed)
 {
-	CFG_CALL(self, patch)(self, avoids, known, on_id);
+	CFG_CALL(self, patch)(self, avoids, known, on_id, flawed);
 }
 
 struct oio_lb_item_s *
@@ -116,24 +120,34 @@ struct _slot_item_s
  * criteria. */
 struct oio_lb_slot_s
 {
+	/* A pointer to the world owning this slot. */
+	struct oio_lb_world_s *world;
+
 	/* the sum of all the individual weights. */
 	oio_weight_acc_t sum_weight;
 
 	/* Array of inline <struct _slot_item_s> sorted by location. No real
-	   need to complexify by a secondary sort by ID, especially if the number
-	   of services at the same location is rather small.*/
+	 * need to complexify by a secondary sort by ID, especially if the number
+	 * of services at the same location is rather small.*/
 	GArray *items;
 
+	/* Total number of items per location, for each level.
+	 * We do not use level 0 at the moment. */
+	GData *items_by_loc[OIO_LB_LOC_LEVELS];
+
+	/* Total number of different locations for each level. */
+	guint locs_by_level[OIO_LB_LOC_LEVELS];
+
 	gchar *name;
 
 	generation_t generation;
 
 	/* Does the slot need to be re-handled to computed accumulated scores.
-	   When set, the slot cannot be used for polling elements. */
+	 * When set, the slot cannot be used for polling elements. */
 	guint8 flag_dirty_weights : 1;
 
 	/* Does the slot need to be re-sorted by location.
-	   When set, the slot cannot be used to be searched by location */
+	 * When set, the slot cannot be used to be searched by location */
 	guint8 flag_dirty_order : 1;
 
 	guint8 flag_rehash_on_update : 1;
@@ -142,6 +156,8 @@ struct oio_lb_slot_s
 /* All the load-balancing information:
  * - all the services in the 'world'
  * - all the slots that gather services with the same characteristics
+ * - generation of the services in the 'world' (incr by 1 at each reload)
+ * - absolute maximum distance between services in the 'world'
  */
 struct oio_lb_world_s
 {
@@ -149,14 +165,14 @@ struct oio_lb_world_s
 	GTree *slots;
 	GTree *items;
 	generation_t generation;
+	guint16 abs_max_dist;
 };
 
 /* A pool describes a preset configuration for the polling of several services.
  * So, a pool is composed of several members:
  * - the set of targets that must bring a service in the result set
  * - a pointer to the world to map the targets into LB slots.
- * - the bitwise mask that tells how close the service are, when the mask is
- *   applied to their locations.
+ * - operations to apply on locations to check how close the services are
  */
 struct oio_lb_pool_LOCAL_s
 {
@@ -170,28 +186,67 @@ struct oio_lb_pool_LOCAL_s
 	 * a coma-separated list of 'slot' names (the slots come from the 'world'). */
 	gchar ** targets;
 
-	oio_location_t location_mask;
-	guint location_mask_max_shift : 16;
+	/* Distance between services that the pool will try to ensure. */
+	guint16 initial_dist;
+
+	/* Distance between services that, when reached, will make the
+	 * poll functions set the 'flawed' parameter to true. */
+	guint16 warn_dist;
+
+	/* Absolute minimum distance between services returned by the pool.
+	 * Cannot be 0. */
+	guint16 min_dist;
+
+	/* If true, look for items close to each other. */
 	gboolean nearby_mode : 16;
 };
 
+struct polling_ctx_s
+{
+	void (*on_id) (oio_location_t location, const char *id);
+	const oio_location_t * avoids;
+	const oio_location_t * polled;
+	oio_location_t *next_polled;
+	guint n_targets;
+
+	/* Count how often each location has been chosen. */
+	GData *counters[OIO_LB_LOC_LEVELS];
+
+	/* Did we use a fallback slot while polling? */
+	gboolean fallback_used : 8;
+
+	/* Shall we check the distance requirements? This will be disabled
+	 * automagically if we detect a case where it is not possible to
+	 * strictly meet the requirements. */
+	gboolean check_distance : 8;
+
+	/* Shall we check the "popularity" of locations when picking
+	 * a new item? This is useful to ensure a good balancing on
+	 * platforms where there is less locations than targets
+	 * (for the specified distance). */
+	gboolean check_popularity : 8;
+};
+
 static void _local__destroy (struct oio_lb_pool_s *self);
 
-static guint _local__poll (struct oio_lb_pool_s *self,
+static GError *_local__poll (struct oio_lb_pool_s *self,
 		const oio_location_t * avoids,
-		oio_lb_on_id_f on_id);
+		oio_lb_on_id_f on_id, gboolean *flawed);
 
-static guint _local__patch(struct oio_lb_pool_s *self,
+static GError *_local__patch(struct oio_lb_pool_s *self,
 		const oio_location_t * avoids,
 		const oio_location_t * known,
-		oio_lb_on_id_f on_id);
+		oio_lb_on_id_f on_id, gboolean *flawed);
 
 static struct oio_lb_item_s *_local__get_item(struct oio_lb_pool_s *self,
 		const char *id);
 
 static struct oio_lb_pool_vtable_s vtable_LOCAL =
 {
-	_local__destroy, _local__poll, _local__patch, _local__get_item
+	.destroy = _local__destroy,
+	.poll = _local__poll,
+	.patch = _local__patch,
+	.get_item = _local__get_item
 };
 
 static struct _lb_item_s *
@@ -220,31 +275,74 @@ oio_lb_world__get_slot_unlocked(struct oio_lb_world_s *world, const char *name)
 static guint _search_first_at_location(GArray *tab, const oio_location_t needle,
 		const guint start, const guint end);
 
+guint32
+djb_hash_str0(const gchar *str)
+{
+	guint32 hash = 5381;
+	guint32 c = 0;
+	while ((c = *str++))
+		hash = ((hash << 5) + hash) + c;
+	return hash;
+}
+
+struct hash_len_s
+djb_hash_str(const gchar * b)
+{
+	struct hash_len_s hl = {.h = 5381,.l = 0 };
+	for (; b[hl.l]; ++hl.l)
+		hl.h = ((hl.h << 5) + hl.h) ^ (guint32) (b[hl.l]);
+	return hl;
+}
+
+guint32
+djb_hash_buf(const guint8 * b, register gsize bs)
+{
+	register guint32 h = 5381;
+	for (register gsize i = 0; i < bs; ++i)
+		h = ((h << 5) + h) ^ (guint32) (b[i]);
+	return h;
+}
+
+/** Gets the number of elements in a GData. */
+static guint
+oio_ext_gdatalist_length(GData **datalist)
+{
+	guint counter = 0;
+	void _datalist_count(GQuark key_id UNUSED,
+			gpointer data UNUSED, gpointer udata UNUSED) {
+		counter++;
+	}
+	g_datalist_foreach(datalist, _datalist_count, NULL);
+	return counter;
+}
+
 /* Take djb2 hash of each part of the '.'-separated string,
- * keep the 16 (or 8) LSB of each hash to build a 64 integer. */
+ * keep the 16 LSB of each hash to build a 64b integer. */
 oio_location_t
 location_from_dotted_string(const char *dotted)
 {
-	// http://www.cse.yorku.ca/~oz/hash.html
-	guint32 _djb2(const gchar *str) {
-		guint32 hash = 5381;
-		guint32 c = 0;
-		while ((c = *str++))
-			hash = ((hash << 5) + hash) + c;
-		return hash;
-	}
-	gchar **toks = g_strsplit(dotted, ".", 8);
-	unsigned int shift = (g_strv_length(toks) <= 4)? 16 : 8;
-	oio_location_t mask = (1u << shift) - 1u;
+	gchar **toks = g_strsplit(dotted, ".", OIO_LB_LOC_LEVELS);
 	oio_location_t location = 0;
+	int ntoks = 0;
 	// according to g_strsplit documentation, toks cannot be NULL
-	for (gchar **tok = toks; *tok; tok++) {
-		location = (location << shift) | (_djb2(*tok) & mask);
+	for (gchar **tok = toks; *tok; tok++, ntoks++) {
+		location = (location << OIO_LB_BITS_PER_LOC_LEVEL) |
+				(djb_hash_str0(*tok) & 0xFFFF);
 	}
 	g_strfreev(toks);
 	return location;
 }
 
+uint32_t
+key_from_loc_level(oio_location_t loc, int level)
+{
+	uint32_t key =
+			((loc >> (level * OIO_LB_BITS_PER_LOC_LEVEL)) + 1) & 0xFFFFFFFF;
+	if (level < 2)
+		key += (loc >> (2 * OIO_LB_BITS_PER_LOC_LEVEL)) * OIO_LB_SHUFFLE_JUMP;
+	return key;
+}
+
 static int
 _compare_stored_items_by_location (const void *k0, const void *k)
 {
@@ -254,33 +352,66 @@ _compare_stored_items_by_location (const void *k0, const void *k)
 }
 
 static gboolean
-_item_is_too_close (const oio_location_t * avoids,
-		const oio_location_t item, const oio_location_t mask)
+_item_is_too_close(const oio_location_t * avoids,
+		const oio_location_t item, const guint16 bit_shift)
 {
 	if (!avoids)
 		return FALSE;
-	const oio_location_t loc = mask & item;
+	const oio_location_t loc = item >> bit_shift;
 	for (const oio_location_t *pp=avoids; *pp; ++pp) {
-		if (loc == (mask & *pp))
+		if (loc == (*pp >> bit_shift))
 			return TRUE;
 	}
 	return FALSE;
 }
 
 static gboolean
-_item_is_too_far (const oio_location_t *known,
-		const oio_location_t item, const oio_location_t mask)
+_item_is_too_far(const oio_location_t *known,
+		const oio_location_t item, const guint16 bit_shift)
 {
 	if (!known)
 		return FALSE;
-	const oio_location_t loc = mask & item;
+	const oio_location_t loc = item >> bit_shift;
 	for (const oio_location_t *pp=known; *pp; ++pp) {
-		if (loc != (mask & *pp))
+		if (loc != (*pp >> bit_shift))
 			return TRUE;
 	}
 	return FALSE;
 }
 
+static gboolean
+_item_is_too_popular(struct polling_ctx_s *ctx, const oio_location_t item,
+		struct oio_lb_slot_s *slot)
+{
+	for (int level = 1; level <= 3; level++) {
+		GQuark key = key_from_loc_level(item, level);
+		// How many different items there is under this level
+		guint32 n_leafs = GPOINTER_TO_UINT(
+				g_datalist_id_get_data(&(slot->items_by_loc[level]), key));
+		if (unlikely(n_leafs == 0)) {
+			GRID_WARN("BUG: LB reload not followed by rehash");
+			n_leafs = 1;
+		}
+		// How often the location has been chosen
+		guint32 popularity = GPOINTER_TO_UINT(
+				g_datalist_id_get_data(ctx->counters + level, key));
+		// Maximum number of elements with this location that we can take
+		guint32 max = 1 + (ctx->n_targets - 1) / (slot->items->len / n_leafs);
+
+		// This gives better results on well balanced platforms,
+		// but is not resilient to service failures.
+		//guint32 max = 1 + (ctx->n_targets - 1) / slot->locs_by_level[level];
+
+		GRID_TRACE("At level %d, %08X has popularity: %u, leafs: %u, max: %u",
+				level, key, popularity, n_leafs, max);
+		if (popularity >= max) {
+			// TODO: return the level to optimize the next jump
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+
 static void
 _slot_flush(struct oio_lb_slot_s *slot)
 {
@@ -312,7 +443,11 @@ _slot_destroy (struct oio_lb_slot_s *slot)
 		g_array_free (slot->items, TRUE);
 		slot->items = NULL;
 	}
+	for (int level = 1; level < OIO_LB_LOC_LEVELS; level++) {
+		g_datalist_clear(&(slot->items_by_loc[level]));
+	}
 	oio_str_clean (&slot->name);
+	slot->world = NULL;
 	g_free (slot);
 }
 
@@ -331,13 +466,55 @@ _slot_needs_rehash (const struct oio_lb_slot_s * const slot)
 	return BOOL(slot->flag_dirty_order) || BOOL(slot->flag_dirty_weights);
 }
 
+static void
+_level_datalist_incr_loc(GData **counters, oio_location_t loc)
+{
+	for (int level = 1; level < OIO_LB_LOC_LEVELS; level++) {
+		GData **counter = counters + level;
+		GQuark key = key_from_loc_level(loc, level);
+		// Will be 0 (NULL) if key does not exist
+		guint32 count = GPOINTER_TO_UINT(
+				g_datalist_id_get_data(counter, key));
+		count++;
+		g_datalist_id_set_data(counter, key, GUINT_TO_POINTER(count));
+	}
+}
+
 static void
 _slot_rehash (struct oio_lb_slot_s *slot)
 {
 	if (slot->flag_dirty_order) {
 		slot->flag_dirty_order = 0;
 		slot->flag_dirty_weights = 1;
-		g_array_sort (slot->items, _compare_stored_items_by_location);
+		g_array_sort(slot->items, _compare_stored_items_by_location);
+
+		for (int level = 1; level < OIO_LB_LOC_LEVELS; level++) {
+			g_datalist_clear(&(slot->items_by_loc[level]));
+			// No need to call g_datalist_init()
+		}
+		for (guint i = 0; i < slot->items->len; i++) {
+			struct _slot_item_s *si = &SLOT_ITEM(slot, i);
+			_level_datalist_incr_loc(slot->items_by_loc, si->item->location);
+		}
+		for (int level = 1; level < OIO_LB_LOC_LEVELS; level++) {
+			slot->locs_by_level[level] =
+					oio_ext_gdatalist_length(&(slot->items_by_loc[level]));
+		}
+
+# ifdef HAVE_EXTRA_DEBUG
+		if (unlikely(GRID_TRACE_ENABLED())) {
+			void _display(GQuark k, gpointer data, gpointer u) {
+				guint level = GPOINTER_TO_UINT(u);
+				oio_location_t loc = (GPOINTER_TO_UINT(k) - 1);
+				GRID_TRACE("%0*lX prefix has %u services",
+						4 * (OIO_LB_LOC_LEVELS - level),
+						loc, GPOINTER_TO_UINT(data));
+			}
+			for (int i = 1; i < OIO_LB_LOC_LEVELS; i++)
+				g_datalist_foreach(
+						&slot->items_by_loc[i], _display, GUINT_TO_POINTER(i));
+		}
+#endif
 	}
 
 	if (slot->flag_dirty_weights) {
@@ -375,39 +552,38 @@ _search_closest_weight (GArray *tab, const guint32 needle,
 	return _search_closest_weight (tab, needle, i_pivot+1, end);
 }
 
-struct polling_ctx_s
-{
-	void (*on_id) (oio_location_t location, const char *id);
-	const oio_location_t * avoids;
-	const oio_location_t * polled;
-	oio_location_t *next_polled;
-};
-
 static gboolean
-_accept_item (struct oio_lb_slot_s *slot, oio_location_t mask,
+_accept_item(struct oio_lb_slot_s *slot, const guint16 bit_shift,
 		gboolean reversed, struct polling_ctx_s *ctx, guint i)
 {
 	const struct _lb_item_s *item = _slot_get (slot, i);
 	const oio_location_t loc = item->location;
 	// Check the item is not in "avoids" list
-	if (_item_is_too_close(ctx->avoids, loc, (oio_location_t)-1))
+	if (_item_is_too_close(ctx->avoids, loc, 0))
 		return FALSE;
 	if (reversed) {
-		// Check the item is not too far from alread polled items
-		if (_item_is_too_far(ctx->polled, loc, mask))
+		// Check the item is not too far from already polled items
+		if (_item_is_too_far(ctx->polled, loc, bit_shift))
 			return FALSE;
 		// Check item has not been already polled
-		if (_item_is_too_close(ctx->polled, loc, (oio_location_t)-1))
+		if (_item_is_too_close(ctx->polled, loc, 0))
 			return FALSE;
 	} else {
-		// Check the item is not too close to alread polled items
-		if (_item_is_too_close(ctx->polled, loc, mask))
+		// Check the item is not too close to already polled items
+		if (_item_is_too_close(ctx->polled, loc,
+				ctx->check_distance? bit_shift : 0))
+			return FALSE;
+		// Check the item has not been chosen too much already
+		if (ctx->check_popularity && _item_is_too_popular(ctx, loc, slot))
 			return FALSE;
 	}
-	GRID_TRACE("Accepting item %s (0x%016lX) from slot %s",
+	GRID_TRACE("Accepting item %s (0x%"OIO_LOC_FORMAT") from slot %s",
 			item->id, loc, slot->name);
 	ctx->on_id((oio_location_t)loc, item->id);
 	*(ctx->next_polled) = loc;
+
+	_level_datalist_incr_loc(ctx->counters, loc);
+
 	return TRUE;
 }
 
@@ -417,15 +593,17 @@ _accept_item (struct oio_lb_slot_s *slot, oio_location_t mask,
  * The purpose of the shuffled lookup is to jump to an item with
  * a distant location. */
 static gboolean
-_local_slot__poll (struct oio_lb_slot_s *slot, oio_location_t mask,
+_local_slot__poll(struct oio_lb_slot_s *slot, const guint16 bit_shift,
 		gboolean reversed, struct polling_ctx_s *ctx)
 {
-	if (_slot_needs_rehash (slot))
-		_slot_rehash (slot);
+	if (unlikely(_slot_needs_rehash(slot)))
+		GRID_WARN("BUG: LB reload not followed by rehash");
 
-	GRID_TRACE2("%s slot=%s sum=%"G_GUINT32_FORMAT" items=%d mask=%016lX",
+	GRID_TRACE2(
+			"%s slot=%s sum=%"G_GUINT32_FORMAT
+			" items=%d shift=%"G_GUINT16_FORMAT,
 			__FUNCTION__, slot->name, slot->sum_weight, slot->items->len,
-			mask);
+			bit_shift);
 
 	if (slot->items->len == 0) {
 		GRID_TRACE2("%s slot empty", __FUNCTION__);
@@ -436,6 +614,23 @@ _local_slot__poll (struct oio_lb_slot_s *slot, oio_location_t mask,
 		return FALSE;
 	}
 
+	/* If we need to pick more items than the number of different locations,
+	 * we can disable the distance checks, and rely only on the "popularity"
+	 * mechanism.
+	 * XXX: here we compare the overall number of targets to the number
+	 * of different locations IN THE CURRENT SLOT. We bet that in most
+	 * pools there will be only one targetted slot. */
+	if (!reversed && ctx->check_distance &&
+			bit_shift >= OIO_LB_BITS_PER_LOC_LEVEL) {
+		guint16 level = bit_shift / OIO_LB_BITS_PER_LOC_LEVEL;
+		if (ctx->n_targets > slot->locs_by_level[level]) {
+			GRID_TRACE("%u targets and %u locations at level %u: "
+					"disabling distance check",
+					ctx->n_targets, slot->locs_by_level[level], level);
+			ctx->check_distance = FALSE;
+		}
+	}
+
 	/* get the closest */
 	guint32 random_weight = g_random_int_range (0, slot->sum_weight);
 	int i = _search_closest_weight (slot->items, random_weight, 0,
@@ -447,7 +642,7 @@ _local_slot__poll (struct oio_lb_slot_s *slot, oio_location_t mask,
 
 	guint iter = 0;
 	while (iter++ < slot->items->len) {
-		if (_accept_item(slot, mask, reversed, ctx, i))
+		if (_accept_item(slot, bit_shift, reversed, ctx, i))
 			return TRUE;
 		i = (i + OIO_LB_SHUFFLE_JUMP) % slot->items->len;
 	}
@@ -457,40 +652,43 @@ _local_slot__poll (struct oio_lb_slot_s *slot, oio_location_t mask,
 }
 
 static gboolean
-_local_target__poll (struct oio_lb_pool_LOCAL_s *lb,
-		const char *target, guint mask_shift, struct polling_ctx_s *ctx)
+_local_target__poll(struct oio_lb_pool_LOCAL_s *lb,
+		const char *target, guint16 bit_shift, struct polling_ctx_s *ctx)
 {
-	oio_location_t mask = lb->location_mask;
-	if (lb->nearby_mode)
-		mask <<= mask_shift;  // Decrease number of differentiating bits
-	else
-		mask >>= mask_shift;  // Increase number of differentiating bits
-
-	GRID_TRACE2("%s pool=%s mask=%"OIO_LOC_FORMAT" target=%s",
-			__FUNCTION__, lb->name, mask, target);
+	GRID_TRACE2("%s pool=%s shift=%d target=%s",
+			__FUNCTION__, lb->name, bit_shift, target);
 	gboolean res = FALSE;
+	gboolean fallback = FALSE;
 
 	g_rw_lock_reader_lock(&lb->world->lock);
-	/* each target is a sequence of '\0'-separated strings, terminated with
-	 * an empty string. Each string is the name of a slot */
-	for (const char *name = target; *name && !res; name += 1+strlen(name)) {
+	/* Each target is a sequence of '\0'-separated strings, terminated with
+	 * an empty string. Each string is the name of a slot.
+	 * In most case we should not loop and consider only the first slot.
+	 * The other slots are fallbacks. */
+	for (const char *name = target; *name; name += 1+strlen(name)) {
 		struct oio_lb_slot_s *slot = oio_lb_world__get_slot_unlocked(
 				lb->world, name);
-		if (!slot)
+		if (!slot) {
 			GRID_DEBUG ("Slot [%s] not ready", name);
-		else if (_local_slot__poll (slot, mask, lb->nearby_mode, ctx))
+		} else if (_local_slot__poll(slot, bit_shift, lb->nearby_mode, ctx)) {
 			res = TRUE;
+			break;
+		}
+		fallback = TRUE;
 	}
 	g_rw_lock_reader_unlock(&lb->world->lock);
+	if (res && fallback)
+		ctx->fallback_used = TRUE;
 	return res;
 }
 
-static guint
+static GError*
 _local__poll (struct oio_lb_pool_s *self,
 		const oio_location_t * avoids,
-		void (*on_id) (oio_location_t location, const char *id))
+		void (*on_id) (oio_location_t, const char *),
+		gboolean *flawed)
 {
-	return _local__patch(self, avoids, NULL, on_id);
+	return _local__patch(self, avoids, NULL, on_id, flawed);
 }
 
 static gboolean
@@ -500,6 +698,12 @@ _local_target__is_satisfied(struct oio_lb_pool_LOCAL_s *lb,
 	if (!*(ctx->next_polled))
 		return FALSE;
 
+	/* Whatever the service is, the client provided it only for its location.
+	 * We don't check if it really exists, and consider this special target
+	 * as satisfied by default. */
+	if (!g_strcmp0(target, OIO_LB_JOKER_SVC_TARGET))
+		return TRUE;
+
 	/* Iterate over the slots of the target to find if one of the
 	** already known locations is inside, and thus satisfies the target. */
 	for (const char *name = target; *name; name += strlen(name)+1) {
@@ -509,8 +713,8 @@ _local_target__is_satisfied(struct oio_lb_pool_LOCAL_s *lb,
 			GRID_DEBUG ("Slot [%s] not ready", name);
 			continue;
 		}
-		if (_slot_needs_rehash(slot)) {
-			_slot_rehash(slot);
+		if (unlikely(_slot_needs_rehash(slot))) {
+			GRID_WARN("BUG: LB reload not followed by rehash");
 		}
 		oio_location_t *known = ctx->next_polled;
 		do {
@@ -532,27 +736,35 @@ _local_target__is_satisfied(struct oio_lb_pool_LOCAL_s *lb,
 	return FALSE;
 }
 
-static guint
+static inline guint16
+_dist_to_bit_shift(guint16 dist, gboolean nearby_mode)
+{
+	return (dist - (!nearby_mode)) * OIO_LB_BITS_PER_LOC_LEVEL;
+}
+
+static GError*
 _local__patch(struct oio_lb_pool_s *self,
 		const oio_location_t *avoids, const oio_location_t *known,
-		void (*on_id) (oio_location_t location, const char *id))
+		void (*on_id) (oio_location_t location, const char *id),
+		gboolean *flawed)
 {
 	struct oio_lb_pool_LOCAL_s *lb = (struct oio_lb_pool_LOCAL_s *) self;
 	EXTRA_ASSERT(lb != NULL);
 	EXTRA_ASSERT(lb->vtable == &vtable_LOCAL);
 	EXTRA_ASSERT(lb->world != NULL);
 	EXTRA_ASSERT(lb->targets != NULL);
+	EXTRA_ASSERT(lb->min_dist >= 1);
 
 	/* Count the expected targets to build a temp storage for
 	 * polled locations */
-	int count_targets = 0;
+	guint count_targets = 0;
 	for (gchar **ptarget = lb->targets; *ptarget; ++ptarget)
 		count_targets++;
 
 	/* Copy the array of known locations because we don't know
 	 * if its allocated length is big enough */
 	oio_location_t polled[count_targets+1];
-	int i = 0;
+	guint i = 0;
 	for (; known && known[i]; i++)
 		polled[i] = known[i];
 	for (; i < count_targets; i++)
@@ -563,25 +775,82 @@ _local__patch(struct oio_lb_pool_s *self,
 		.avoids = avoids,
 		.polled = (const oio_location_t *) polled,
 		.next_polled = polled,
+		.n_targets = count_targets,
+		.check_distance = TRUE,
+		.check_popularity = TRUE,
 	};
 
+	for (int level = 1; level < OIO_LB_LOC_LEVELS; level++) {
+		g_datalist_init(&ctx.counters[level]);
+	}
+
+	/* In normal mode (resp. nearby mode), bit shifts start high
+	 * (resp. low), because we want services with different
+	 * (resp. equal) most significant bits. Then we reduce (resp. increase)
+	 * the shifting so we compare more (resp. less) bits of the locations,
+	 * and thus have more chances to find differences (resp. similarities)
+	 * between service locations. */
+	guint16 max_dist = MIN(lb->world->abs_max_dist, lb->initial_dist);
+	guint16 start_dist = lb->nearby_mode? lb->min_dist : max_dist;
+	guint16 end_dist = (lb->nearby_mode? max_dist + 1 : lb->min_dist - 1);
+	guint16 reached_dist = start_dist;
+	gint16 incr = lb->nearby_mode? 1 : -1;
 	guint count = 0;
+	GError *err = NULL;
 	for (gchar **ptarget = lb->targets; *ptarget; ++ptarget) {
 		gboolean done = _local_target__is_satisfied(lb, *ptarget, &ctx);
-		guint mask_shift = 0u;
-		while (!done && mask_shift <= lb->location_mask_max_shift) {
-			done = _local_target__poll(lb, *ptarget, mask_shift, &ctx);
-			mask_shift += 8u;  // Degrade mask by 8 bits (two hex digit)
+		guint16 dist;
+		for (dist = start_dist; !done && dist != end_dist; dist += incr) {
+			done = _local_target__poll(lb, *ptarget,
+					_dist_to_bit_shift(dist, lb->nearby_mode), &ctx);
 		}
+		dist -= incr;
+		if ((lb->nearby_mode && dist > reached_dist) ||
+				(!lb->nearby_mode && dist < reached_dist))
+			reached_dist = dist;
 		if (!done) {
-			/* the strings is '\0' separated, printf won't display it */
-			GRID_WARN("No service polled from target [%s]", *ptarget);
-			return 0;
+			/* the strings are '\0' separated, printf won't display them */
+			err = NEWERROR(CODE_POLICY_NOT_SATISFIABLE, "no service polled "
+					"from [%s], %u/%d services polled, %u services in slot",
+					*ptarget, count, count_targets,
+					oio_lb_world__count_slot_items(lb->world, *ptarget));
+			break;
 		}
 		++ctx.next_polled;
 		++count;
 	}
-	return count;
+
+	if (unlikely(GRID_DEBUG_ENABLED())) {
+		// FIXME: there is similar code in _slot_rehash()
+		void _display(GQuark k, gpointer data, gpointer u) {
+			guint level = GPOINTER_TO_UINT(u);
+			oio_location_t loc = (GPOINTER_TO_UINT(k) - 1);
+			GRID_DEBUG("%0*" G_GINT64_MODIFIER "X selected %u times",
+					4 * (OIO_LB_LOC_LEVELS - level),
+					loc, GPOINTER_TO_UINT(data));
+		}
+		for (int level = 1; level < OIO_LB_LOC_LEVELS; level++)
+			g_datalist_foreach(&ctx.counters[level],
+					_display, GUINT_TO_POINTER(level));
+	}
+
+	for (int level = 1; level < OIO_LB_LOC_LEVELS; level++) {
+		g_datalist_clear(&ctx.counters[level]);
+	}
+	if (err != NULL) {
+		GRID_WARN("%s", err->message);
+		return err;
+	}
+	if (flawed) {
+		GRID_DEBUG(
+				"nearby_mode=%d, reached_dist=%u, warn_dist=%u, fallbacks=%d",
+				lb->nearby_mode, reached_dist, lb->warn_dist,
+				ctx.fallback_used);
+		*flawed = (lb->nearby_mode && reached_dist >= lb->warn_dist) ||
+				(!lb->nearby_mode && reached_dist <= lb->warn_dist) ||
+				ctx.fallback_used;
+	}
+	return NULL;
 }
 
 struct oio_lb_item_s *
@@ -638,32 +907,42 @@ oio_lb_world__set_pool_option(struct oio_lb_pool_s *self, const char *key,
 	struct oio_lb_pool_LOCAL_s *lb = (struct oio_lb_pool_LOCAL_s *) self;
 	if (!key || !*key)
 		return;
-	if (!strcmp(key, OIO_LB_OPT_MASK)) {
-		if (value[0] == '/') {
-			guint64 mask_bits = g_ascii_strtoull(value+1, &endptr, 10u);
-			if (endptr == value+1) {
-				GRID_WARN("Invalid location mask [%s] for pool [%s], ",
-						value, lb->name);
-			} else {
-				lb->location_mask = (~(oio_location_t)0) << (64 - mask_bits);
-				GRID_TRACE("pool [%s] mask=%s decoded as 0x%"OIO_LOC_FORMAT,
-						lb->name, value, lb->location_mask);
-			}
+	if (!strcmp(key, OIO_LB_OPT_MIN_DIST)) {
+		guint64 min = g_ascii_strtoull(value, &endptr, 10u);
+		if (endptr == value) {
+			GRID_WARN("Invalid %s [%s] for pool [%s]",
+					OIO_LB_OPT_MIN_DIST, value, lb->name);
+		} else if (min < 1 || min > OIO_LB_LOC_LEVELS) {
+			GRID_WARN("%s [%s] for pool [%s] out of range [1, %d]",
+					OIO_LB_OPT_MIN_DIST, value, lb->name,
+					OIO_LB_LOC_LEVELS);
 		} else {
-			guint64 new_mask = g_ascii_strtoull(value, &endptr, 16u);
-			if (new_mask == 0 && endptr == value)
-				GRID_WARN("Invalid location mask [%s] for pool [%s], "
-						"must be 64bit hex", value, lb->name);
-			else
-				lb->location_mask = new_mask;
+			lb->min_dist = min;
+		}
+	} else if (!strcmp(key, OIO_LB_OPT_MAX_DIST)) {
+		guint64 max = g_ascii_strtoull(value, &endptr, 10u);
+		if (endptr == value) {
+			GRID_WARN("Invalid %s [%s] for pool [%s]",
+					OIO_LB_OPT_MAX_DIST, value, lb->name);
+		} else if (max < 1 || max > OIO_LB_LOC_LEVELS) {
+			GRID_WARN("%s [%s] for pool [%s] out of range [1, %d]",
+					OIO_LB_OPT_MAX_DIST, value, lb->name,
+					OIO_LB_LOC_LEVELS);
+		} else {
+			lb->initial_dist = max;
+		}
+	} else if (!strcmp(key, OIO_LB_OPT_WARN_DIST)) {
+		guint64 warn = g_ascii_strtoull(value, &endptr, 10u);
+		if (endptr == value) {
+			GRID_WARN("Invalid %s [%s] for pool [%s]",
+					OIO_LB_OPT_WARN_DIST, value, lb->name);
+		} else if (warn > OIO_LB_LOC_LEVELS + 1) {
+			GRID_WARN("%s [%s] for pool [%s] out of range [0, %d]",
+					OIO_LB_OPT_WARN_DIST, value, lb->name,
+					OIO_LB_LOC_LEVELS + 1);
+		} else {
+			lb->warn_dist = warn;
 		}
-	} else if (!strcmp(key, OIO_LB_OPT_MASK_MAX_SHIFT)) {
-		guint64 shift = g_ascii_strtoull(value, &endptr, 10u);
-		if (endptr == value)
-			GRID_WARN("Invalid mask shift [%s] for pool [%s]",
-					value, lb->name);
-		else
-			lb->location_mask_max_shift = shift;
 	} else if (!strcmp(key, OIO_LB_OPT_NEARBY)) {
 		lb->nearby_mode = oio_str_parse_bool(value, FALSE);
 	} else {
@@ -712,6 +991,14 @@ oio_lb_local__create_world (void)
 			g_free, (GDestroyNotify) _slot_destroy);
 	self->items = g_tree_new_full (oio_str_cmp3, NULL,
 			g_free, g_free);
+
+	/* See at the end of oio_lb_world__feed_slot_unlocked()
+	 * for an explanation. */
+# ifndef __GNUC__
+	self->abs_max_dist = OIO_LB_LOC_LEVELS;
+# else
+	/* Keep it 0, we will increase it when adding items. */
+# endif
 	return self;
 }
 
@@ -771,8 +1058,8 @@ oio_lb_world__create_pool (struct oio_lb_world_s *world, const char *name)
 	lb->name = g_strdup (name);
 	lb->world = world;
 	lb->targets = g_malloc0 (4 * sizeof(gchar*));
-	lb->location_mask = ~0xFFFFUL;
-	lb->location_mask_max_shift = 16u;
+	lb->initial_dist = OIO_LB_LOC_LEVELS;
+	lb->min_dist = 1;
 	lb->nearby_mode = FALSE;
 	return (struct oio_lb_pool_s*) lb;
 }
@@ -786,8 +1073,12 @@ _world_create_slot (struct oio_lb_world_s *self, const char *name)
 	g_rw_lock_reader_unlock(&self->lock);
 	if (!slot) {
 		slot = g_malloc0(sizeof(*slot));
+		slot->world = self;
 		slot->name = g_strdup(name);
 		slot->items = g_array_new(FALSE, TRUE, sizeof(struct _slot_item_s));
+		for (int level = 1; level < OIO_LB_LOC_LEVELS; level++) {
+			g_datalist_init(&(slot->items_by_loc[level]));
+		}
 		g_rw_lock_writer_lock(&self->lock);
 		g_tree_replace(self->slots, g_strdup(name), slot);
 		g_rw_lock_writer_unlock(&self->lock);
@@ -978,7 +1269,21 @@ oio_lb_world__feed_slot_unlocked(struct oio_lb_world_s *self,
 	}
 
 	if (slot->flag_rehash_on_update && _slot_needs_rehash (slot))
-		_slot_rehash (slot);
+		_slot_rehash(slot);
+
+	/* This is an optimization to speedup the locations comparisons.
+	 * It needs __builtin_clzll which is a GCC builtin. */
+# ifdef __GNUC__
+	// Actual number of bits used by the location.
+	const int n_bits = sizeof(oio_location_t) * 8 -
+			__builtin_clzll(item->location? : 1u);
+	// Maximum distance between items with this number of bits.
+	guint16 max_dist = 1 + (n_bits - 1) / OIO_LB_BITS_PER_LOC_LEVEL;
+	if (self->abs_max_dist < max_dist) {
+		self->abs_max_dist = max_dist;
+		GRID_DEBUG("Absolute max_dist set to %u", max_dist);
+	}
+# endif
 }
 
 void
@@ -991,10 +1296,10 @@ oio_lb_world__feed_slot (struct oio_lb_world_s *self, const char *name,
 }
 
 static void
-_slot_debug (struct oio_lb_slot_s *slot, const char *name)
+_slot_debug (struct oio_lb_slot_s *slot)
 {
 	GRID_DEBUG("slot=%s num=%u sum=%"G_GUINT32_FORMAT" flags=%d content:",
-			name, slot->items->len, slot->sum_weight,
+			slot->name, slot->items->len, slot->sum_weight,
 			slot->flag_dirty_weights | slot->flag_dirty_order<<1 |
 			slot->flag_rehash_on_update<<2);
 	for (guint i = 0; i < slot->items->len; ++i) {
@@ -1008,8 +1313,9 @@ void
 oio_lb_world__debug (struct oio_lb_world_s *self)
 {
 	EXTRA_ASSERT (self != NULL);
-	gboolean _on_slot (gchar *name, struct oio_lb_slot_s *slot, void *i UNUSED) {
-		_slot_debug (slot, name);
+	gboolean _on_slot (gchar *name UNUSED, struct oio_lb_slot_s *slot,
+			void *i UNUSED) {
+		_slot_debug (slot);
 		return FALSE;
 	}
 	g_rw_lock_reader_lock(&self->lock);
@@ -1054,6 +1360,9 @@ _world_purge_slot_items(struct oio_lb_world_s *self, guint32 age)
 			slot->flag_dirty_order = 1;
 		}
 
+		if (_slot_needs_rehash(slot))
+			_slot_rehash(slot);
+
 		return FALSE;
 	}
 
@@ -1148,31 +1457,36 @@ oio_lb__has_pool(struct oio_lb_s *lb, const char *name)
 	return res;
 }
 
-guint
+GError*
 oio_lb__poll_pool(struct oio_lb_s *lb, const char *name,
-		const oio_location_t * avoids, oio_lb_on_id_f on_id)
+		const oio_location_t * avoids, oio_lb_on_id_f on_id,
+		gboolean *flawed)
 {
 	EXTRA_ASSERT(name != NULL);
-	guint res = 0;
+	GError *res = NULL;
 	g_rw_lock_reader_lock(&lb->lock);
 	struct oio_lb_pool_s *pool = g_hash_table_lookup(lb->pools, name);
 	if (pool)
-		res = oio_lb_pool__poll(pool, avoids, on_id);
+		res = oio_lb_pool__poll(pool, avoids, on_id, flawed);
+	else
+		res = BADREQ("pool [%s] not found", name);
 	g_rw_lock_reader_unlock(&lb->lock);
 	return res;
 }
 
-guint
+GError*
 oio_lb__patch_with_pool(struct oio_lb_s *lb, const char *name,
 		const oio_location_t *avoids, const oio_location_t *known,
-		oio_lb_on_id_f on_id)
+		oio_lb_on_id_f on_id, gboolean *flawed)
 {
 	EXTRA_ASSERT(name != NULL);
-	guint res = 0;
+	GError *res = NULL;
 	g_rw_lock_reader_lock(&lb->lock);
 	struct oio_lb_pool_s *pool = g_hash_table_lookup(lb->pools, name);
 	if (pool)
-		res = oio_lb_pool__patch(pool, avoids, known, on_id);
+		res = oio_lb_pool__patch(pool, avoids, known, on_id, flawed);
+	else
+		res = BADREQ("pool [%s] not found", name);
 	g_rw_lock_reader_unlock(&lb->lock);
 	return res;
 }
@@ -1190,4 +1504,3 @@ oio_lb__get_item_from_pool(struct oio_lb_s *lb, const char *name,
 	g_rw_lock_reader_unlock(&lb->lock);
 	return res;
 }
-
