@@ -1,6 +1,6 @@
 /*
 OpenIO SDS sqlx
-Copyright (C) 2015 OpenIO, original work as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -23,9 +23,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 extern "C" {
 #endif
 
-# include <glib.h>
+#include <glib.h>
 
-struct oio_url_s;
+#include <core/oiourl.h>
 
 struct oio_directory_s;
 
@@ -56,6 +56,15 @@ struct oio_directory_vtable_s
 	/* the OIOURL_TYPE in <url> will be ignored */
 	GError * (*set_prop)(struct oio_directory_s *self,
 			const struct oio_url_s *url, const char * const *values);
+
+	/* the OIOURL_TYPE in <url> will be ignored */
+	GError * (*force) (struct oio_directory_s *self,
+			const struct oio_url_s *url, const char *srvtype,
+			const char * const *values, gint64 seq);
+
+	/* the OIOURL_TYPE in <url> will be ignored */
+	GError * (*unlink) (struct oio_directory_s *self,
+			const struct oio_url_s *url, const char *srvtype);
 };
 
 struct oio_directory_abstract_s
@@ -84,6 +93,13 @@ GError * oio_directory__get_properties(struct oio_directory_s *self,
 GError * oio_directory__set_properties(struct oio_directory_s *self,
 		const struct oio_url_s *url, const char * const *values);
 
+GError * oio_directory__force(struct oio_directory_s *d,
+		const struct oio_url_s *url, const char *srvtype,
+		const char * const *values, gint64 seq);
+
+GError * oio_directory__unlink(struct oio_directory_s *d,
+		const struct oio_url_s *url, const char *srvtype);
+
 /* Implementation specifics ------------------------------------------------- */
 
 /* create a directory that perform direct access to the meta0 and meta1
