@@ -1,7 +1,6 @@
 /*
 OpenIO SDS core library
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -40,30 +39,41 @@ extern "C" {
 
 #define OIO_STRV_APPEND_COPY(Tab,S0) do { (Tab) = oio_strv_append((Tab), g_strdup(S0)); } while (0)
 
+#define g_string_append_static(out,Static) \
+	g_string_append_len(out, Static, sizeof(Static)-1) \
+
+#define oio_ptrv_length(v) oio_constptrv_length((const void * const *)(v))
+
+void oio_str_reuse(gchar **dst, gchar *src);
+
 /* Count the items in the array */
 size_t oio_strv_length (const char * const *v);
 
 /* Count the size of the concatenation of all the strings in <v> */
 size_t oio_strv_length_total (const char * const *v);
 
+gboolean oio_strv_has (const gchar * const * v, const gchar * needle);
+
 size_t oio_constptrv_length (const void * const *v);
 
-#define oio_ptrv_length(v) oio_constptrv_length((const void * const *)(v))
-
-void oio_str_reuse(gchar **dst, gchar *src);
-
 /* Reallocs <dst> and appends it <s>. <s> is reused and is not duplicated */
 gchar ** oio_strv_append(gchar **dst, gchar *s);
 
 /** frees *s and set it to NULL */
 void oio_str_clean(gchar **s);
 
+/** g_strfreev *p and set it to NULL */
+void oio_str_cleanv(gchar ***p);
+
 /** frees *dst and set it to src */
 void oio_str_replace(gchar **dst, const char *src);
 
 /** Returns FALSE if 's' is not 'slen' long and contains a non-hexa character. */
 gboolean oio_str_ishexa(const char *s, gsize slen);
 
+/** Returns TRUE if 's' contains only printable ascii characters. */
+gboolean oio_str_is_printable(const char *s, gsize slen);
+
 /** Returns is 's' is an even number of hexadecimal characters */
 gboolean oio_str_ishexa1(const char *s);
 
@@ -122,19 +132,80 @@ void oio_str_gstring_append_json_quote (GString *base, const char *s);
 void oio_str_gstring_append_json_blob(GString *base, const char *s, int len);
 
 /* appends "<k>":"<v>" where k and v are added with
-   oio_str_gstring_append_json_string() */
+ * oio_str_gstring_append_json_string() */
 void oio_str_gstring_append_json_pair (GString *base,
 		const char *k, const char *v);
 
+/* TODO add a macro version of oio_str_gstring_append_json_pair() taking a
+ * static string as the key, to avoid computing its length for trusted keys,
+ * as for oio_url, ns_info, etc. */
+
 void oio_str_gstring_append_json_pair_int (GString *base,
 		const char *k, gint64 v);
 
+void oio_str_gstring_append_json_pair_boolean (GString *base,
+		const char *k, gboolean v);
+
+#define OIO_JSON_append_bool(g,k,v) do { \
+	g_string_append_static(g, "\"" k "\":"); \
+	if (v) \
+		g_string_append_static(g, "true"); \
+	else \
+		g_string_append_static(g, "false"); \
+} while (0)
+
+#define OIO_JSON_append_int(g,k,v) do { \
+	g_string_append_static(g, "\"" k "\":"); \
+	g_string_append_printf(g, "%" G_GINT64_FORMAT, (gint64)v); \
+} while (0)
+
+#define OIO_JSON_append_gstr(g,k,v) do { \
+	if (v) { \
+		g_string_append_static(g, "\"" k "\":\""); \
+		oio_str_gstring_append_json_blob(g, v->str, v->len); \
+		g_string_append_c(g, '"'); \
+	} else { \
+		g_string_append_static(g, "\"" k "\":null"); \
+	} \
+} while (0)
+
+#define OIO_JSON_append_blob(g,k,S,L) do { \
+	if (S != NULL) { \
+		g_string_append_static(g, "\"" k "\":\""); \
+		oio_str_gstring_append_json_blob(g, S, L); \
+		g_string_append_c(g, '"'); \
+	} else { \
+		g_string_append_static(g, "\"" k "\":null"); \
+	} \
+} while (0)
+
+#define OIO_JSON_append_str(g,k,S) do { \
+	if (S != NULL) { \
+		g_string_append_static(g, "\"" k "\":"); \
+		oio_str_gstring_append_json_quote(g, S); \
+	} else { \
+		g_string_append_static(g, "\"" k "\":null"); \
+	} \
+} while (0)
+
+#define OIO_JSON_append_gba(g,k,gba) do { \
+	if (gba != NULL) { \
+		g_string_append_static(g, "\"" k "\":\""); \
+		metautils_gba_to_hexgstr(g, gba); \
+		g_string_append_c(g, '"'); \
+	} else { \
+		g_string_append_static(g, "\"" k "\":null"); \
+	} \
+} while (0)
+
+
 static inline int oio_str_is_set (const char *s) { return NULL!=s && 0!=*s; }
 
 int oio_str_prefixed (const char *s, const char *p, const char *sep);
 
 int oio_str_caseprefixed(const char *str, const char *prefix);
 
+/* pi64 is left untouched if the overall return is FALSE */
 gboolean oio_str_is_number (const char *s, gint64 *pi64);
 
 int oio_str_cmp3 (const void *a, const void *b, void *ignored);
@@ -168,6 +239,8 @@ gchar ** KV_extract_prefixed (gchar **kv, const char *prefix);
 /* @see KV_extract_prefixed(), with the keys left untouched */
 gchar ** KV_extract_not_prefixed (gchar **kv, const char *prefix);
 
+gchar * KV_get_value(gchar **kv, char *key);
+
 
 #ifdef __cplusplus
 }
