@@ -1,7 +1,7 @@
 /*
 OpenIO SDS core library
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -20,6 +20,7 @@ License along with this library.
 #ifndef OIO_SDS__core__oiourl_h
 # define OIO_SDS__core__oiourl_h 1
 
+#include <glib.h>
 #include <sys/types.h>
 
 #ifdef __cplusplus
@@ -42,6 +43,8 @@ enum oio_url_field_e
 
 	OIOURL_HEXID,     /* read-write */
 	OIOURL_CONTENTID, /* read-write */
+
+	OIOURL_FULLPATH
 };
 
 
@@ -128,6 +131,15 @@ int oio_url_has_fq_path (const struct oio_url_s *u);
 /** Returns wether all the mandatory components for a container are present */
 int oio_url_has_fq_container (const struct oio_url_s *u);
 
+/** Validate contains of oio_url
+ *
+ * @param u the URL
+ * @param n the namespace (optional)
+ * @param e will contains faulty field
+ * @return true if valid
+*/
+gboolean oio_url_check(const struct oio_url_s *u, const char *n, const gchar **e);
+
 #ifdef __cplusplus
 }
 #endif
