@@ -1,6 +1,6 @@
 /*
 OpenIO SDS core library
-Copyright (C) 2015-2016 OpenIO, work as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -16,19 +16,21 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library.
 */
 
+#include <core/http_internals.h>
+
 #include <string.h>
 
-#include <glib.h>
-#include <json.h>
+#include <json-c/json.h>
 #include <curl/curl.h>
 
-#include "oio_core.h"
-#include "oio_sds.h"
-#include "http_internals.h"
-#include "http_put.h"
-#include "internals.h"
+#include <core/oio_sds.h>
+#include <core/oiostr.h>
+#include <core/oiocfg.h>
+#include <core/oiolog.h>
+#include <core/oioext.h>
 
-/* -------------------------------------------------------------------------- */
+#include "internals.h"
+#include "core/http_put.h"
 
 
 static GString *
@@ -39,10 +41,10 @@ _build_json (const char* const *src, GString *json)
 	g_string_append_c(json, '{');
 	for (int i = 0; src [i] && src [i+1] ; i +=2 ) {
 		if (i != 0)
-			g_string_append (json,",");
+			g_string_append_c (json, ',');
 		oio_str_gstring_append_json_pair (json, src [i], src [i+1]);
 	}
-	g_string_append (json, "}");
+	g_string_append_c (json, '}');
 	return json;
 }
 
@@ -79,7 +81,8 @@ _curl_url_prefix (const char *ns, enum _prefix_e which)
 		return NULL;
 	}
 
-	GString *hu = g_string_new("http://");
+	GString *hu = g_string_sized_new(128);
+	g_string_append_static(hu, "http://");
 	g_string_append (hu, s);
 	g_free (s);
 	return hu;
@@ -258,6 +261,12 @@ struct http_ctx_s
 	GString *body;
 };
 
+struct http_reply_ctx_s
+{
+	guint retry_after;
+	struct http_ctx_s *out;
+};
+
 static int
 _has_prefix_len (char **pb, size_t *plen, const char *prefix)
 {
@@ -279,47 +288,66 @@ _has_prefix_len (char **pb, size_t *plen, const char *prefix)
 static size_t
 _header_callback(char *b, size_t s, size_t n, void *u)
 {
-	struct http_ctx_s *o = u;
-	size_t total = n*s;
+	struct http_reply_ctx_s *ctx = u;
+	const size_t total = n*s;
 
-	if (!o || !o->headers) /* caller not interested in headers */
-		return total;
-	if (!_has_prefix_len (&b, &total, "x-oio-"))
-		return total;
-	if (total > 8192) /* header too big */
+	EXTRA_ASSERT(ctx != NULL);
+
+	if (total > 2048) /* header too big */
 		return total;
 
-	gchar tmp[total+1];
-	memcpy (tmp, b, total);
-	tmp[total] = '\0';
+	size_t remaining = total;
 
-	char *colon = strchr(tmp, ':');
-	if (colon) {
-		*(colon++) = 0;
-
-		const gsize l = g_strv_length (o->headers);
-		o->headers = g_realloc (o->headers, (l+3) * sizeof(void*));
-		o->headers[l+0] = g_strdup (g_strstrip(tmp));
-		o->headers[l+1] = g_strdup (g_strstrip(colon));
-		o->headers[l+2] = NULL;
+	/* One special header is considered, would the caller app be interested in
+	 * headers or not */
+	if (_has_prefix_len(&b, &remaining, "Retry-After: ")) {
+		/* OSEF the value, let's do our own exponential back-off */
+		ctx->retry_after = 1;
+		return total;
 	}
 
-	return n*s;
+	/* Then only the OpenIO-related headers are considered if the caller app.
+	 * has an interest in them. */
+	if (ctx->out && ctx->out->headers
+			&& _has_prefix_len (&b, &remaining, "x-oio-")) {
+		gchar tmp[remaining+1];
+		memcpy (tmp, b, remaining);
+		tmp[remaining] = '\0';
+
+		char *colon = strchr(tmp, ':');
+		if (colon) {
+			*colon = 0;
+
+			const gsize l = g_strv_length (ctx->out->headers);
+			ctx->out->headers = g_realloc (ctx->out->headers, (l+3) * sizeof(void*));
+			ctx->out->headers[l+0] = g_strdup (g_strstrip(tmp));
+			ctx->out->headers[l+1] = g_strdup (g_strstrip(colon+1));
+			ctx->out->headers[l+2] = NULL;
+		}
+		return total;
+	}
+
+	return total;
 }
 
 static GError *
 _proxy_call_notime (CURL *h, const char *method, const char *url,
-		struct http_ctx_s *in, struct http_ctx_s *out)
+		struct http_ctx_s *in, struct http_ctx_s *out,
+		guint *p_retry_after)
 {
 	EXTRA_ASSERT (h != NULL);
 	EXTRA_ASSERT (method != NULL);
 	EXTRA_ASSERT (url != NULL);
+	EXTRA_ASSERT (p_retry_after != NULL);
 	struct view_GString_s view_input = {.data=NULL, .done=0};
 
 	GError *err = NULL;
 	curl_easy_setopt (h, CURLOPT_URL, url);
 	curl_easy_setopt (h, CURLOPT_CUSTOMREQUEST, method);
 
+	struct http_reply_ctx_s ctx = {0};
+	ctx.out = out;
+
 	/* Populate the request headers */
 	struct oio_headers_s headers = {NULL,NULL};
 	oio_headers_common (&headers);
@@ -330,13 +358,8 @@ _proxy_call_notime (CURL *h, const char *method, const char *url,
 	}
 
 	/* Intercept the headers from the response */
-	if (out) {
-		curl_easy_setopt (h, CURLOPT_HEADERDATA, out);
-		curl_easy_setopt (h, CURLOPT_HEADERFUNCTION, _header_callback);
-	} else {
-		curl_easy_setopt (h, CURLOPT_HEADERDATA, NULL);
-		curl_easy_setopt (h, CURLOPT_HEADERFUNCTION, NULL);
-	}
+	curl_easy_setopt (h, CURLOPT_HEADERDATA, &ctx);
+	curl_easy_setopt (h, CURLOPT_HEADERFUNCTION, _header_callback);
 
 	if (in && in->body) {
 		view_input.data = in->body;
@@ -377,7 +400,10 @@ _proxy_call_notime (CURL *h, const char *method, const char *url,
 			}
 		}
 	}
+
 	oio_headers_clear (&headers);
+
+	*p_retry_after = ctx.retry_after;
 	return err;
 }
 
@@ -388,10 +414,45 @@ _proxy_call (CURL *h, const char *method, const char *url,
 	if (!oio_ext_get_reqid ())
 		oio_ext_set_random_reqid();
 
-	gint64 t = g_get_monotonic_time ();
-	GError *err = _proxy_call_notime (h, method, url, in, out);
-	t = g_get_monotonic_time () - t;
-	GRID_TRACE("proxy: %s %s took %"G_GINT64_FORMAT"us", method, url, t);
+	GError *err = NULL;
+	guint retry_after = 0;
+
+	const guint max = 3;
+	for (guint i=0; i<max ;++i) {
+		const gint64 t = g_get_monotonic_time ();
+		err = _proxy_call_notime (h, method, url, in, out, &retry_after);
+		GRID_TRACE("proxy: %s %s took %"G_GINT64_FORMAT"us",
+				method, url, g_get_monotonic_time() - t);
+		(void) t;
+		if (!err) /* success */
+			break;
+#ifdef HAVE_ENBUG
+		if (err->code == 404 && i == 0) /* let's fake a retry */
+			err->code = 503, retry_after = 1;
+#endif
+		if (err->code != HTTP_CODE_SRV_UNAVAILABLE) /* not retryable */
+			break;
+		if (!retry_after)  /* not told to retry */
+			break;
+		/* Let's retry! */
+		if (i+1 != max) {
+			/* cleanup what has been allocated by the previous call */
+			g_clear_error(&err);
+			if (out) {
+				if (out->headers) {
+					g_strfreev(out->headers);
+					out->headers = g_malloc0(sizeof(void*));
+				}
+				if (out->body) {
+					g_string_set_size(out->body, 0);
+				}
+			}
+			/* randomize the sleep-time to avoid resonance effects */
+			const gulong sleep_base = (1 << i) * 200 * G_TIME_SPAN_MILLISECOND;
+			const gulong sleep_jitter = oio_ext_rand_int_range(0, 100 * G_TIME_SPAN_MILLISECOND);
+			g_usleep(sleep_base + sleep_jitter);
+		}
+	}
 	return err;
 }
 
@@ -525,11 +586,23 @@ oio_proxy_call_content_show (CURL *h, struct oio_url_s *u, GString *out,
 	return err;
 }
 
+GError *
+oio_proxy_call_content_drain(CURL *h, struct oio_url_s *u)
+{
+	GString *http_url = _curl_content_url(u, "drain");
+	if (!http_url) return BADNS();
+
+	GError *err = _proxy_call(h, "POST", http_url->str, NULL, NULL);
+	g_string_free(http_url, TRUE);
+	return err;
+}
+
 GError *
 oio_proxy_call_content_delete (CURL *h, struct oio_url_s *u)
 {
 	GString *http_url = _curl_content_url (u, "delete");
-	if (!http_url) return BADNS();
+	if (!http_url)
+		return BADNS();
 
 	GError *err = _proxy_call (h, "POST", http_url->str, NULL, NULL);
 	g_string_free (http_url, TRUE);
@@ -608,8 +681,7 @@ oio_proxy_call_content_prepare (CURL *h, struct oio_url_s *u,
 	}
 	g_string_free (http_url, TRUE);
 	g_string_free(i.body, TRUE);
-	if (o.headers)
-		g_strfreev (o.headers);
+	g_strfreev (o.headers);
 	return err;
 }
 
@@ -625,7 +697,7 @@ oio_proxy_call_content_create (CURL *h, struct oio_url_s *u,
 	if (!http_url) return BADNS();
 
 	if (in->content) {
-		g_string_append (http_url, "&id=");
+		g_string_append_static (http_url, "&id=");
 		g_string_append_uri_escaped (http_url, in->content, NULL, TRUE);
 	}
 
@@ -654,9 +726,10 @@ oio_proxy_call_content_create (CURL *h, struct oio_url_s *u,
 	}
 	GString *body = in? in->chunks : NULL;
 	if (in && !in->update && in->properties) {
-		GString *val = g_string_new("{'properties':");
+		GString *val = g_string_sized_new(1024);
+		g_string_append_static(val, "{\"properties\":");
 		val = _build_json(in->properties, val);
-		g_string_append(val, ",'chunks':");
+		g_string_append_static(val, ",\"chunks\":");
 		g_string_append(val, in->chunks->str);
 		g_string_append_c(val, '}');
 		body = val;
@@ -779,6 +852,55 @@ oio_proxy_call_reference_set_properties(CURL *h, struct oio_url_s *u,
 	return err;
 }
 
+GError *
+oio_proxy_call_reference_force(CURL *h, struct oio_url_s *u,
+		const char *t, const char* const *values, gint64 seq)
+{
+	GString *http_url = _curl_reference_url(u, "force");
+	if (!http_url) {
+		return BADNS();
+	}
+
+	if (t) {
+		_append(http_url, '&', "type", t);
+	}
+
+	gchar *hdrin[] = { PROXYD_HEADER_MODE, "autocreate", NULL };
+
+	GString *json = _build_json(values, NULL);
+	g_string_overwrite(json, json->len - 1, ",");
+	oio_str_gstring_append_json_pair_int(json, "seq", seq);
+	g_string_append_printf(json, "}");
+
+	struct http_ctx_s i = { .headers = hdrin, .body = json };
+	GError *err = _proxy_call(h, "POST", http_url->str, &i, NULL);
+
+	g_string_free(http_url, TRUE);
+	g_string_free(json, TRUE);
+
+	return err;
+}
+
+GError *
+oio_proxy_call_reference_unlink(CURL *h, struct oio_url_s *u,
+		const char *t)
+{
+	GString *http_url = _curl_reference_url(u, "unlink");
+	if (!http_url) {
+		return BADNS();
+	}
+
+	if (t) {
+		_append(http_url, '&', "type", t);
+	}
+
+	GError *err = _proxy_call(h, "POST", http_url->str, NULL, NULL);
+
+	g_string_free(http_url, TRUE);
+
+	return err;
+}
+
 /* -------------------------------------------------------------------------- */
 
 GError *
