@@ -1,6 +1,6 @@
 /*
 OpenIO SDS core library
-Copyright (C) 2015-2017 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -16,29 +16,26 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library.
 */
 
-#include <stdlib.h>
+#include <core/oio_sds.h>
+
 #include <string.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 
-#include <glib.h>
 #include <json.h>
 #include <curl/curl.h>
 #include <curl/curlver.h>
 
-#include "oio_core.h"
-#include "oio_sds.h"
-#include "oiostr.h"
+#include <core/client_variables.h>
+#include <metautils/lib/metautils.h>
 
 #include "http_put.h"
+#include "http_del.h"
 #include "http_internals.h"
 #include "internals.h"
 
-#include <metautils/lib/metautils.h>
-#include <metautils/lib/storage_policy.h>
 
 struct oio_sds_s
 {
@@ -57,6 +54,7 @@ struct oio_sds_s
 
 	GMutex curl_lock;
 	CURL *curl_handle;
+	gint64 chunk_size;
 };
 
 struct oio_error_s;
@@ -78,60 +76,6 @@ asm (".symver fmemopen, fmemopen@GLIBC_2.2.5");
 # endif
 #endif
 
-char **
-oio_sds_get_compile_options (void)
-{
-	GPtrArray *tmp = g_ptr_array_new ();
-	void _add (const gchar *k, const gchar *v) {
-		g_ptr_array_add (tmp, g_strdup(k));
-		g_ptr_array_add (tmp, g_strdup(v));
-	}
-	void _add_double (const gchar *k, gdouble v) {
-		gchar s[32];
-		_add (k, g_ascii_dtostr (s, sizeof(s), v));
-	}
-	void _add_integer (const gchar *k, gint64 v) {
-		gchar s[24];
-		g_snprintf (s, sizeof(s), "%"G_GINT64_FORMAT, v);
-		_add (k, s);
-	}
-#define _ADD_STR(S) _add(#S,S)
-#define _ADD_DBL(S) _add_double(#S,S)
-#define _ADD_INT(S) _add_integer(#S,S)
-	_ADD_STR (PROXYD_PREFIX);
-	_ADD_STR (PROXYD_HEADER_PREFIX);
-	_ADD_STR (PROXYD_HEADER_REQID);
-	_ADD_STR (PROXYD_HEADER_NOEMPTY);
-	_ADD_INT (PROXYD_PATH_MAXLEN);
-	_ADD_DBL (PROXYD_DEFAULT_TTL_CSM0);
-	_ADD_DBL (PROXYD_DEFAULT_TTL_SERVICES);
-	_ADD_INT (PROXYD_DEFAULT_MAX_CSM0);
-	_ADD_INT (PROXYD_DEFAULT_MAX_SERVICES);
-
-	_ADD_STR (GCLUSTER_RUN_DIR);
-	_ADD_STR (OIO_ETC_DIR);
-	_ADD_STR (OIO_CONFIG_FILE_PATH);
-	_ADD_STR (OIO_CONFIG_DIR_PATH);
-	_ADD_STR (OIO_CONFIG_LOCAL_PATH);
-	_ADD_STR (GCLUSTER_AGENT_SOCK_PATH);
-
-	_ADD_DBL (COMMON_CLIENT_TIMEOUT);
-	_ADD_DBL (CS_CLIENT_TIMEOUT);
-	_ADD_DBL (M0V2_CLIENT_TIMEOUT);
-	_ADD_DBL (M1V2_CLIENT_TIMEOUT);
-	_ADD_DBL (M2V2_CLIENT_TIMEOUT);
-	_ADD_DBL (M2V2_CLIENT_TIMEOUT_HUGE);
-	_ADD_DBL (SQLX_CLIENT_TIMEOUT);
-
-	char **out = calloc(1 + tmp->len, sizeof(void*));
-	for (guint i = 0; i < tmp->len; ++i)
-		out[i] = strdup((char*) tmp->pdata[i]);
-	for (guint i = 0; i < tmp->len; ++i)
-		g_free(tmp->pdata[i]);
-	g_ptr_array_free(tmp, TRUE);
-	return out;
-}
-
 static CURL *
 _get_proxy_handle (struct oio_sds_s *sds)
 {
@@ -251,7 +195,7 @@ _chunks_pack (GString *gs, GSList *chunks)
 {
 	gchar strpos[32];
 
-	g_string_append (gs, "[");
+	g_string_append_c (gs, '[');
 	for (GSList *l = chunks; l; l = l->next) {
 		struct chunk_s *c = l->data;
 		if (gs->str[gs->len - 1] != '[')
@@ -264,7 +208,7 @@ _chunks_pack (GString *gs, GSList *chunks)
 				"\"hash\":\"%s\"}",
 				c->url, c->size, strpos, c->hexhash);
 	}
-	g_string_append (gs, "]");
+	g_string_append_c (gs, ']');
 }
 
 static GError *
@@ -334,7 +278,7 @@ _get_meta_bound (GSList *lchunks)
 
 static GError *
 _organize_chunks (GSList *lchunks, struct metachunk_s ***result,
-		gboolean no_shuffle)
+		gboolean no_shuffle, gint64 k)
 {
 	*result = NULL;
 
@@ -379,6 +323,16 @@ _organize_chunks (GSList *lchunks, struct metachunk_s ***result,
 		out[i]->size = ((struct chunk_s*)(out[i]->chunks->data))->size;
 	}
 
+	/* patch each metachunk-size and multiply it by K */
+	for (guint i = 0; i < meta_bound; ++i) {
+		struct metachunk_s *mc = out[i];
+		mc->size = mc->size * k;
+		for (GSList *l=mc->chunks; l ;l=l->next) {
+			struct chunk_s *chunk = l->data;
+			chunk->size = mc->size;
+		}
+	}
+
 	/* Compute each metachunk's offset in the main content */
 	gsize offset = 0;
 	for (guint i = 0; i < meta_bound; ++i) {
@@ -475,6 +429,7 @@ oio_sds_init (struct oio_sds_s **out, const char *ns)
 	(*out)->no_shuffle = oio_sds_no_shuffle;
 	(*out)->admin = FALSE;
 	g_mutex_init(&((*out)->curl_lock));
+	(*out)->chunk_size = 0;
 
 	return NULL;
 }
@@ -533,6 +488,11 @@ oio_sds_configure (struct oio_sds_s *sds, enum oio_sds_config_e what,
 				return EINVAL;
 			sds->no_shuffle = BOOL(*(int*)pv);
 			return 0;
+		case OIOSDS_CFG_FLAG_CHUNKSIZE:
+			if (vlen != sizeof(int64_t))
+				return EINVAL;
+			sds->chunk_size = *(int64_t *)pv;
+			return 0;
 		default:
 			return EBADSLT;
 	}
@@ -565,7 +525,7 @@ _show_content (struct oio_sds_s *sds, struct oio_url_s *url, void *cb_data,
 
 	GError *err = NULL;
 	GSList *chunks = NULL;
-	GString *reply_body = g_string_new("");
+	GString *reply_body = g_string_sized_new(2048);
 	gchar **props = NULL;
 
 	/* Get the beans */
@@ -654,19 +614,19 @@ _dl_debug (const char *caller, struct oio_sds_dl_src_s *src,
 {
 	if (!GRID_DEBUG_ENABLED())
 		return;
-	GString *out = g_string_new("");
+	GString *out = g_string_sized_new(128);
 
 	g_string_append_printf (out, "SRC{%s", oio_url_get(src->url, OIOURL_WHOLE));
 	if (src->ranges && src->ranges[0]) {
-		g_string_append (out, ",[");
+		g_string_append_static (out, ",[");
 		for (struct oio_sds_dl_range_s **p = src->ranges; *p; ++p)
 			g_string_append_printf (out,
 					"[%"G_GSIZE_FORMAT",%"G_GSIZE_FORMAT"]",
 					(*p)->offset, (*p)->size);
-		g_string_append (out, "]}");
+		g_string_append_static (out, "]}");
 	}
 
-	g_string_append (out, " -> ");
+	g_string_append_static (out, " -> ");
 
 	if (dst->type == OIO_DL_DST_FILE)
 		g_string_append_printf (out, "DST{FILE,%s}", dst->data.file.path);
@@ -1004,7 +964,7 @@ _download_to_hook (struct oio_sds_s *sds, struct oio_sds_dl_src_s *src,
 			.sds = sds, .dst = dst, .src = src, .chunk_method = chunk_method,
 			.metachunks = NULL, .chunks = chunks,
 		};
-		err = _organize_chunks(chunks, &dl.metachunks, sds->no_shuffle);
+		err = _organize_chunks(chunks, &dl.metachunks, sds->no_shuffle, 1);
 		if (!err) {
 			EXTRA_ASSERT (dl.metachunks != NULL);
 			err = _download (&dl);
@@ -1237,6 +1197,7 @@ oio_sds_upload_init (struct oio_sds_s *sds, struct oio_sds_ul_dst_s *dst)
 	ul->checksum_chunk = NULL;
 	ul->buffer_tail = g_queue_new ();
 	ul->metachunk_ready = g_queue_new ();
+	ul->chunk_size = sds->chunk_size;
 
 	if (dst->content_id) {
 		EXTRA_ASSERT(oio_str_ishexa1 (dst->content_id));
@@ -1288,7 +1249,8 @@ oio_sds_upload_done (struct oio_sds_ul_s *ul)
 int
 oio_sds_upload_greedy (struct oio_sds_ul_s *ul)
 {
-	return NULL != ul && !ul->finished && ul->ready_for_data;
+	return NULL != ul && !ul->finished && ul->ready_for_data
+		&& g_queue_is_empty(ul->buffer_tail);
 }
 
 int
@@ -1303,8 +1265,8 @@ oio_sds_upload_prepare (struct oio_sds_ul_s *ul, size_t size)
 	EXTRA_ASSERT (ul != NULL);
 
 	GError *err = NULL;
-	GString *request_body = g_string_new("");
-	GString *reply_body = g_string_new ("");
+	GString *request_body = g_string_sized_new(128);
+	GString *reply_body = g_string_sized_new (1024);
 
 	/* get the beans from the proxy, for the size announced.
 	 * The reply will only carry the official chunk_size and
@@ -1364,23 +1326,30 @@ oio_sds_upload_prepare (struct oio_sds_ul_s *ul, size_t size)
 			g_prefix_error (&err, "Parsing: ");
 		json_object_put (jbody);
 		json_tokener_free (tok);
+	}
 
-		/* Verify we are not doing erasure coding.
-		 * TODO: implement erasure coding in C */
-		if (oio_sds_upload_needs_ecd(ul)) {
-			if (oio_str_is_set(ul->sds->ecd)) {
-				GRID_DEBUG("using ecd gateway");
-			} else {
-				err = NEWERROR(CODE_NOT_IMPLEMENTED,
-						"cannot upload this without ecd");
-			}
+	gint64 k = 1;
+
+	/* Verify either we are doing erasure coding or not */
+	if (!err && oio_sds_upload_needs_ecd(ul)) {
+		if (oio_str_is_set(ul->sds->ecd)) {
+			GRID_DEBUG("using ecd gateway");
+		} else {
+			err = NEWERROR(CODE_NOT_IMPLEMENTED,
+					"cannot upload this without ecd");
+		}
+
+		/* If we erasure-code, patch the metachunk-size */
+		if (oio_sds_client_patch_metachunk_size) {
+			k = data_security_decode_param_int64(ul->chunk_method, "k", 1);
+			ul->chunk_size = ul->chunk_size * k;
 		}
 	}
 
 	/* Organize the set of chunks into metachunks. */
 	if (!err) {
 		struct metachunk_s **out = NULL;
-		if ((err = _organize_chunks(_chunks, &out, ul->sds->no_shuffle)))
+		if ((err = _organize_chunks(_chunks, &out, ul->sds->no_shuffle, k)))
 			g_prefix_error (&err, "Logic: ");
 		else
 			for (struct metachunk_s **p = out; *p; ++p)
@@ -1406,6 +1375,14 @@ oio_sds_upload_prepare (struct oio_sds_ul_s *ul, size_t size)
 	return (struct oio_error_s*) err;
 }
 
+static void
+_upload_feed_bytes (struct oio_sds_ul_s *ul, GBytes *bytes)
+{
+	g_queue_push_tail (ul->buffer_tail, bytes);
+	if (!g_bytes_get_size(bytes))
+		ul->ready_for_data = FALSE;
+}
+
 struct oio_error_s *
 oio_sds_upload_feed (struct oio_sds_ul_s *ul,
 		const unsigned char *buf, size_t len)
@@ -1414,9 +1391,7 @@ oio_sds_upload_feed (struct oio_sds_ul_s *ul,
 	EXTRA_ASSERT (ul != NULL);
 	g_assert (!ul->finished);
 	g_assert (ul->ready_for_data);
-	g_queue_push_tail (ul->buffer_tail, g_bytes_new (buf, len));
-	if (!len)
-		ul->ready_for_data = FALSE;
+	_upload_feed_bytes(ul, g_bytes_new (buf, len));
 	return NULL;
 }
 
@@ -1492,8 +1467,11 @@ _sds_upload_add_headers(struct oio_sds_ul_s *ul, struct http_put_dest_s *dest)
 	http_put_dest_add_header (dest, PROXYD_HEADER_REQID,
 			"%s", oio_ext_get_reqid());
 
-	http_put_dest_add_header (dest, PROXYD_HEADER_ADMIN,
-			"%s", oio_ext_is_admin()?"1":"0");
+	if (oio_ext_is_admin()) {
+		/* TODO(jfs): melt these informatiion with the 'mode' (already
+		 * containing autocreate, etc */
+		http_put_dest_add_header (dest, PROXYD_HEADER_ADMIN, "yes");
+	}
 
 	http_put_dest_add_header (dest, RAWX_HEADER_PREFIX "container-id",
 			"%s", oio_url_get (ul->dst->url, OIOURL_HEXID));
@@ -1504,6 +1482,17 @@ _sds_upload_add_headers(struct oio_sds_ul_s *ul, struct http_put_dest_s *dest)
 			"%s", escaped);
 	g_free (escaped);
 
+	const gchar *full_path = oio_url_get(ul->dst->url, OIOURL_FULLPATH);
+	if (!oio_url_has(ul->dst->url, OIOURL_VERSION))
+		http_put_dest_add_header (dest, RAWX_HEADER_PREFIX "full-path", "%s/%"
+				G_GINT64_FORMAT, full_path, ul->version);
+	else
+		http_put_dest_add_header (dest, RAWX_HEADER_PREFIX "full-path", "%s",
+				full_path);
+
+	http_put_dest_add_header (dest, RAWX_HEADER_PREFIX "oio-version", "%s",
+			oio_sds_client_version);
+
 	http_put_dest_add_header (dest, RAWX_HEADER_PREFIX "content-version",
 			"%" G_GINT64_FORMAT, ul->version);
 	http_put_dest_add_header (dest, RAWX_HEADER_PREFIX "content-id",
@@ -1723,9 +1712,28 @@ oio_sds_upload_step (struct oio_sds_ul_s *ul)
 }
 
 static void
-_chunks_remove (CURL *h UNUSED, GSList *chunks UNUSED)
+_chunks_remove (GSList *failed, GSList *done)
 {
-	/* TODO JFS */
+	if (!failed && !done)
+		return;
+
+	/* Merge both lists */
+	GPtrArray *tmp = g_ptr_array_new();
+	for (GSList *l=failed; l ;l=l->next)
+		g_ptr_array_add(tmp, ((struct chunk_s*)(l->data))->url);
+	for (GSList *l=done; l ;l=l->next)
+		g_ptr_array_add(tmp, ((struct chunk_s*)(l->data))->url);
+	g_ptr_array_add(tmp, NULL);
+
+	GError *err = http_poly_delete((gchar**)(tmp->pdata));
+
+	g_ptr_array_free(tmp, TRUE);
+
+	if (err) {
+		GRID_WARN("Error rolling back some chunks: (%d) %s",
+				err->code, err->message);
+		g_clear_error(&err);
+	}
 }
 
 struct oio_error_s *
@@ -1741,8 +1749,8 @@ oio_sds_upload_commit (struct oio_sds_ul_s *ul)
 	for (GList *l = g_list_first(ul->metachunk_done); l; l = g_list_next(l))
 		size += ((struct metachunk_s*) l->data)->size;
 
-	GString *request_body = g_string_new("");
-	GString *reply_body = g_string_new ("");
+	GString *request_body = g_string_sized_new(2048);
+	GString *reply_body = g_string_sized_new (256);
 	_chunks_pack (request_body, ul->chunks_done);
 
 	gchar hash[STRLEN_CHUNKHASH];
@@ -1762,26 +1770,33 @@ oio_sds_upload_commit (struct oio_sds_ul_s *ul)
 	};
 
 	GRID_TRACE("%s (%p) Saving %s", __FUNCTION__, ul, request_body->str);
-	GError *err;
+	GError *err = NULL;
 	CURL_DO(ul->sds, H, err = oio_proxy_call_content_create (H, ul->dst->url, &in, reply_body));
-	if (ul->chunks_failed)
-		CURL_DO(ul->sds, H, _chunks_remove (H, ul->chunks_failed));
 
 	g_string_free (request_body, TRUE);
 	g_string_free (reply_body, TRUE);
-	if (err)
-		return (struct oio_error_s*) err;
-	return NULL;
+	return (struct oio_error_s*) err;
 }
 
 struct oio_error_s *
 oio_sds_upload_abort (struct oio_sds_ul_s *ul)
 {
-	GRID_TRACE("%s (%p)", __FUNCTION__, ul);
 	EXTRA_ASSERT (ul != NULL);
-	if (ul->chunks_failed)
-		CURL_DO(ul->sds, H, _chunks_remove (H, ul->chunks_failed));
-	return (struct oio_error_s *) NYI();
+	_chunks_remove(ul->chunks_failed, ul->chunks_done);
+	return NULL;
+}
+
+static void
+_upload_abort_no_error(struct oio_sds_ul_s *ul)
+{
+	EXTRA_ASSERT (ul != NULL);
+	GRID_WARN("Aborting...");
+	struct oio_error_s *e = oio_sds_upload_abort (ul);
+	if (e) {
+		GRID_WARN("Upload abort failed: (%d) %s",
+				oio_error_code (e), oio_error_message (e));
+		oio_error_free (e);
+	}
 }
 
 static void
@@ -1790,7 +1805,7 @@ _ul_debug (const char *caller, struct oio_sds_ul_src_s *src,
 {
 	if (!GRID_DEBUG_ENABLED())
 		return;
-	GString *out = g_string_new("");
+	GString *out = g_string_sized_new(128);
 
 	if (src->type == OIO_UL_SRC_HOOK_SEQUENTIAL)
 		g_string_append_printf (out, "SRC{HOOK,%p}", src->data.hook.cb);
@@ -1825,15 +1840,13 @@ _upload_sequential (struct oio_sds_s *sds, struct oio_sds_ul_dst_s *dst,
 		err = oio_sds_upload_prepare(ul, src->data.hook.size);
 
 	size_t sent = 0;
-	const size_t blen = 131072;
-	guint8 *b = g_malloc(blen);
 
 	while (!err && !oio_sds_upload_done (ul)) {
 		GRID_TRACE("%s (%p) not done yet", __FUNCTION__, ul);
 
 		/* feed the upload queue */
 		if (oio_sds_upload_greedy (ul)) {
-			size_t max = blen;
+			size_t max = 8 * 1024 * 1024;
 			if (src->data.hook.size > 0 && src->data.hook.size != (size_t)-1) {
 				const size_t remaining = src->data.hook.size - sent;
 				max = MIN(remaining, max);
@@ -1843,24 +1856,27 @@ _upload_sequential (struct oio_sds_s *sds, struct oio_sds_ul_dst_s *dst,
 			}
 
 			if (0 == max) {
-				err = oio_sds_upload_feed (ul, b, 0);
+				_upload_feed_bytes (ul, g_bytes_new_static((guint8*)"", 0));
 			} else {
+				guint8 *b = g_malloc(max);
 				size_t l = src->data.hook.cb (src->data.hook.ctx, b, max);
 				switch (l) {
 					case OIO_SDS_UL__ERROR:
 						err = (struct oio_error_s*) SYSERR("data hook error");
 						break;
 					case OIO_SDS_UL__DONE:
-						err = oio_sds_upload_feed (ul, b, 0);
+						_upload_feed_bytes (ul, g_bytes_new_static((guint8*)"", 0));
 						break;
 					case OIO_SDS_UL__NODATA:
 						GRID_INFO("%s No data ready from user's hook", __FUNCTION__);
 						break;
 					default:
-						err = oio_sds_upload_feed (ul, b, l);
+						_upload_feed_bytes (ul, g_bytes_new_take(b, l));
+						b = NULL;
 						sent += l;
 						break;
 				}
+				oio_pfree0(&b, NULL);
 			}
 		}
 
@@ -1869,17 +1885,19 @@ _upload_sequential (struct oio_sds_s *sds, struct oio_sds_ul_dst_s *dst,
 			err = oio_sds_upload_step (ul);
 	}
 
-	oio_pfree0(&b, NULL);
-
-	if (!err)
+	if (!err) {
 		err = oio_sds_upload_commit (ul);
-	else {
-		struct oio_error_s *e = oio_sds_upload_abort (ul);
-		if (e) {
-			GRID_WARN("Upload abort failed: (%d) %s",
-					oio_error_code (e), oio_error_message (e));
-			oio_error_free (e);
+		if (err) {
+			if (oio_error_code(err) == CODE_CONTENT_EXISTS ||
+					oio_error_code(err) == CODE_CONTENT_PRECONDITION) {
+				_upload_abort_no_error(ul);
+			} else {
+				GRID_WARN("Conditons unsafe to abort the upload: (%d) %s",
+						oio_error_code(err), oio_error_message(err));
+			}
 		}
+	} else {
+		_upload_abort_no_error(ul);
 	}
 
 	oio_sds_upload_clean (ul);
@@ -2012,7 +2030,7 @@ _notify_list_item (struct oio_sds_list_listener_s *listener,
 		{"name", &jn, json_type_string, 1},
 		{"hash", &jh, json_type_string, 1},
 		{"size", &js, json_type_int, 1},
-		{"ver",  &jv, json_type_int, 1},
+		{"version",  &jv, json_type_int, 1},
 		{"properties",  &jp, json_type_object, 0},
 		{NULL, NULL, 0, 0}
 	};
@@ -2063,9 +2081,10 @@ _notify_list_result (struct oio_sds_list_listener_s *listener,
 		return err;
 	}
 
-	GRID_TRACE2 ("Found %u items, %u prefixes",
-			json_object_array_length(jobjects),
-			json_object_array_length(jprefixes));
+	/* Type changed to size_t in json-c 0.13 */
+	GRID_TRACE2("Found %zu items, %zu prefixes",
+			(size_t)json_object_array_length(jobjects),
+			(size_t)json_object_array_length(jprefixes));
 
 	const int count_objects = json_object_array_length(jobjects);
 	if (count_objects >= 0)
@@ -2094,7 +2113,7 @@ _single_list (struct oio_sds_list_param_s *param,
 
 	listener->out_count = 0;
 	listener->out_truncated = FALSE;
-	GString *reply_body = g_string_new ("");
+	GString *reply_body = g_string_sized_new (2048);
 
 	// Query the proxy
 	GError *err = oio_proxy_call_content_list(h, param, reply_body);
@@ -2304,6 +2323,18 @@ oio_sds_truncate (struct oio_sds_s *sds, struct oio_url_s *url, size_t size)
 	return (struct oio_error_s*) err;
 }
 
+struct oio_error_s*
+oio_sds_drain(struct oio_sds_s *sds, struct oio_url_s *url)
+{
+	if (!sds || !url)
+		return (struct oio_error_s*) BADREQ("Missing argument");
+	oio_ext_set_reqid(sds->session_id);
+
+	GError *err;
+	CURL_DO(sds, H, err = oio_proxy_call_content_drain(H, url));
+	return (struct oio_error_s*) err;
+}
+
 struct oio_error_s*
 oio_sds_delete (struct oio_sds_s *sds, struct oio_url_s *url)
 {
