@@ -1,6 +1,6 @@
 /*
 OpenIO SDS core library
-Copyright (C) 2015 OpenIO, original work as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -16,15 +16,13 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library.
 */
 
+#include <core/oiostr.h>
+
 #include <string.h>
 #include <errno.h>
 
-#include <glib.h>
+#include <oioext.h>
 
-#include "oioext.h"
-#include "oiostr.h"
-#include "oiourl.h"
-#include "oiolog.h"
 #include "internals.h"
 
 static guint8 masks[] = {
@@ -130,6 +128,15 @@ gboolean oio_str_ishexa1(const char *s) {
 	return len > 0 && (len%2) == 0;
 }
 
+gboolean oio_str_is_printable(const char *s, gsize slen) {
+	gsize len = 0;
+	for (; len < slen && *s; ++s, ++len) {
+		if (!g_ascii_isprint(*s))
+			return FALSE;
+	}
+	return TRUE;
+}
+
 gboolean oio_str_hex2bin(const char *s0, guint8 *d, gsize dlen) {
 	const guint8 *s = (const guint8*) s0;
 	if (!s || !d)
@@ -394,7 +401,7 @@ void oio_str_gstring_append_json_pair (GString *base,
 	oio_str_gstring_append_json_quote(base, k);
 	g_string_append_c (base, ':');
 	if (v == NULL) {
-		g_string_append(base, "null");
+		g_string_append_static(base, "null");
 	} else {
 		oio_str_gstring_append_json_quote(base, v);
 	}
@@ -407,6 +414,17 @@ void oio_str_gstring_append_json_pair_int (GString *base,
 	g_string_append_printf(base, "%"G_GINT64_FORMAT, v);
 }
 
+void oio_str_gstring_append_json_pair_boolean (GString *base,
+		const char *k, gboolean v) {
+	oio_str_gstring_append_json_quote(base, k);
+	g_string_append_c (base, ':');
+	if (v) {
+		g_string_append_static(base, "True");
+	} else {
+		g_string_append_static(base, "False");
+	}
+}
+
 size_t oio_constptrv_length (const void * const *v) {
 	size_t count = 0;
 	if (v) while (*(v++)) { ++count; }
@@ -564,7 +582,7 @@ GError * STRV_decode_buffer (guint8 *buf, gsize len, gchar ***out) {
 void KV_encode_gstr2(GString *out, gchar **kv) {
 	gboolean first = TRUE;
 	if (!kv) {
-		g_string_append(out, "null");
+		g_string_append_static(out, "null");
 		return;
 	}
 	g_string_append_c(out, '{');
@@ -582,7 +600,7 @@ GString *KV_encode_gstr(gchar **kv) {
 		return g_string_new("");
 	if (!*kv)
 		return g_string_new("{}");
-	GString *gs = g_string_new("");
+	GString *gs = g_string_sized_new(128);
 	KV_encode_gstr2(gs, kv);
 	return gs;
 }
@@ -688,3 +706,36 @@ gchar ** KV_extract_not_prefixed (gchar **kv, const char *prefix) {
 	g_ptr_array_add(tmp, NULL);
 	return (gchar**) g_ptr_array_free(tmp, FALSE);
 }
+
+gchar *
+KV_get_value(gchar **kv, char *key)
+{
+	if (!kv)
+		return NULL;
+
+	for (gchar **p=kv; *p && *(p+1); p+=2) {
+		if (g_strcmp0(key, *p) == 0)
+			return *(p+1);
+	}
+	return NULL;
+}
+
+void oio_str_cleanv(gchar ***p) {
+	if (unlikely(p == NULL))
+		return;
+	if (*p) {
+		g_strfreev(*p);
+		*p = NULL;
+	}
+}
+
+gboolean
+oio_strv_has(register const gchar * const * p, register const gchar *needle)
+{
+	if (!p) return FALSE;
+	while (*p) {
+		if (!g_ascii_strcasecmp(needle, *(p++)))
+			return TRUE;
+	}
+	return FALSE;
+}
