@@ -1,6 +1,6 @@
 /*
 OpenIO SDS core library
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -22,10 +22,11 @@ License along with this library.
 #include <glib.h>
 #include <glib/gstdio.h>
 
+#include <core/oio_sds.h>
+#include <core/oio_core.h>
+#include <metautils/lib/storage_policy.h>
+
 #include "internals.h"
-#include "oio_core.h"
-#include "oio_sds.h"
-#include "oioext.h"
 
 #define NOERROR(E) g_assert_no_error((GError*)(E))
 
@@ -211,7 +212,7 @@ _checksum_file (const char *path, struct file_info_s *fi)
 	NOERROR(err); \
 } while (0)
 
-#define CHECK_PRESENT(ckient,url) do { \
+#define CHECK_PRESENT(client,url) do { \
 	int has = 0; \
 	err = oio_sds_has (client, url, &has); \
 	if (!err && !has) \
@@ -383,7 +384,11 @@ _roundtrip_tail (struct file_info_s *fi0, const char * content_id,
 		NOERROR(err);
 	}
 
-	/* Remove the roginal content from the container */
+	/* Drain the content */
+	err = oio_sds_drain(client, url);
+	NOERROR(err);
+
+	/* Remove the original content from the container */
 	err = oio_sds_delete (client, url);
 	NOERROR(err);
 
@@ -561,6 +566,83 @@ _roundtrip_put_from_file (const char * const * properties)
 	oio_error_pfree (&err);
 }
 
+/* Upload a content larger than the chunk size
+ * then check the chunks' size */
+static void
+check_chunksize (const char * const * properties)
+{
+	const gint64 mega = 1024 * 1024;
+	gint64 chunk_size = 5 * mega;
+	struct oio_error_s *err = NULL;
+
+	struct oio_url_s *url_random = oio_url_dup(url);
+	gchar path[32];
+	oio_str_randomize(path, oio_ext_rand_int_range(7,32), random_chars);
+	oio_url_set(url_random, OIOURL_PATH, path);
+
+	/* generate a buffer to work in. First we fill it of random bytes */
+	const gsize size = chunk_size + 1 * mega;
+	guint8 *buffer = g_malloc(size);
+	g_assert_nonnull(buffer);
+	oio_buf_randomize(buffer, size);
+	gchar *pre_hash =
+		g_compute_checksum_for_data(G_CHECKSUM_MD5, buffer, size);
+	g_assert_nonnull(pre_hash);
+	oio_str_upper(pre_hash);
+	if (GRID_TRACE2_ENABLED()) {
+		g_file_set_contents("/tmp/pre", (gchar *) buffer, size, (GError **) &err);
+		NOERROR(err);
+	}
+
+	g_assert_cmpint(oio_sds_configure(client, OIOSDS_CFG_FLAG_CHUNKSIZE,
+			&chunk_size, sizeof(chunk_size)), ==, 0);
+
+	struct oio_sds_ul_dst_s ul_dst = OIO_SDS_UPLOAD_DST_INIT;
+	ul_dst.url = url;
+	ul_dst.autocreate = 1;
+	ul_dst.append = 0;
+	ul_dst.out_size = 0;
+	ul_dst.properties = properties;
+	err = oio_sds_upload_from_buffer(client, &ul_dst, buffer, size);
+	NOERROR(err);
+
+	/* get details on the content */
+	gsize max_size[2] = {0};
+	guint k = 1;
+	void _on_info(void *i UNUSED, enum oio_sds_content_key_e key, const char *v) {
+		if (key != OIO_SDS_CONTENT_CHUNKMETHOD)
+			return;
+		k = data_security_decode_param_int64(v, DS_KEY_K, 1);
+	}
+	void _on_metachunk(void *i UNUSED, guint seq, gsize offt, gsize len) {
+		GRID_DEBUG("metachunk: %u, %"G_GSIZE_FORMAT" %"G_GSIZE_FORMAT,
+				seq, offt, len);
+		g_assert_cmpuint(seq, <, 2);
+		max_size[seq] = len;
+	}
+	err = oio_sds_show_content(client, url, NULL, _on_info, _on_metachunk, NULL);
+	NOERROR(err);
+	if (k == 1) {
+		g_assert_cmpuint(max_size[0], ==, chunk_size);
+		g_assert_cmpuint(max_size[1], ==, size - chunk_size);
+	} else {
+		g_assert_cmpuint(max_size[0], <=, k * chunk_size);
+	}
+
+	/* delete the content */
+	err = oio_sds_delete(client, url);
+	NOERROR(err);
+	/* Unset the chunk size */
+	int64_t default_chunk_size = 0;
+	g_assert_cmpint(oio_sds_configure(client, OIOSDS_CFG_FLAG_CHUNKSIZE,
+			&default_chunk_size, sizeof(default_chunk_size)), ==, 0);
+
+	oio_error_pfree(&err);
+	oio_pfree0 (&buffer, NULL);
+	oio_url_pclean(&url_random);
+	oio_str_clean(&pre_hash);
+}
+
 /* Perform subsequent appends then check the content is OK */
 static void
 _roundtrip_append (const char * const * properties)
@@ -682,6 +764,10 @@ main(int argc, char **argv)
 		g_printerr ("Usage: %s PATH\n", argv[0]);
 		return 1;
 	}
+	if (!oio_var_value_with_files(g_getenv("OIO_NS"), TRUE, NULL)) {
+		g_printerr("Unknown NS [%s]\n", g_getenv("OIO_NS"));
+		return 1;
+	}
 
 	source_path = argv[1];
 
@@ -739,6 +825,7 @@ main(int argc, char **argv)
 	ADD_TEST ("put/file/asis", OIO_HDRCASE_LOW,    _roundtrip_put_from_file);
 	ADD_TEST ("put/file/asis", OIO_HDRCASE_1CAP,   _roundtrip_put_from_file);
 	ADD_TEST ("put/file/asis", OIO_HDRCASE_RANDOM, _roundtrip_put_from_file);
+	ADD_TEST ("put/buffer/chunksize", OIO_HDRCASE_RANDOM, check_chunksize);
 
 	int rc = g_test_run();
 	oio_sds_pfree (&client);
