@@ -1,7 +1,7 @@
 /*
 OpenIO SDS core library
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -17,10 +17,13 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library.
 */
 
+#include <core/oiourl.h>
+
 #include <string.h>
 
-#include "oio_core.h"
-#include "url_ext.h"
+#include <core/oiostr.h>
+#include <core/oioext.h>
+#include <core/url_ext.h>
 
 struct oio_url_s
 {
@@ -36,6 +39,7 @@ struct oio_url_s
 
 	/* secondary */
 	gchar *whole;
+	gchar *fullpath;
 	guint8 id[32];
 	gchar hexid[65];
 	guint8 flags;
@@ -161,6 +165,7 @@ _clean_url (struct oio_url_s *u)
 	oio_str_clean(&u->version);
 	oio_str_clean(&u->content);
 	oio_str_clean(&u->whole);
+	oio_str_clean(&u->fullpath);
 	u->hexid[0] = '\0';
 	u->flags = 0;
 }
@@ -259,6 +264,7 @@ oio_url_dup(const struct oio_url_s *u)
 	STRDUP(result, u, whole);
 	STRDUP(result, u, version);
 	STRDUP(result, u, content);
+	STRDUP(result, u, fullpath);
 	return result;
 }
 
@@ -302,6 +308,9 @@ oio_url_set(struct oio_url_s *u, enum oio_url_field_e f, const char *v)
 		case OIOURL_WHOLE:
 			return NULL;
 
+		case OIOURL_FULLPATH:
+			return NULL;
+
 		case OIOURL_HEXID:
 			u->hexid[0] = 0;
 			if (!oio_str_ishexa(v,64) || !oio_str_hex2bin(v, u->id, 32))
@@ -342,6 +351,8 @@ oio_url_has(const struct oio_url_s *u, enum oio_url_field_e f)
 			return oio_str_is_set(u->version);
 		case OIOURL_WHOLE:
 			return TRUE;
+		case OIOURL_FULLPATH:
+			return TRUE;
 		case OIOURL_HEXID:
 			return (u->ns && u->hexid[0]) || (u->ns && u->user);
 		case OIOURL_CONTENTID:
@@ -383,9 +394,40 @@ _pack_url(struct oio_url_s *u)
 					g_string_append_c (gs, '/');
 					g_string_append_uri_escaped (gs, u->path, NULL, TRUE);
 				}
+				gboolean anyopt = FALSE;
 				if (u->content) {
-					g_string_append_c (gs, '?');
+					g_string_append_c (gs, anyopt ? '&' : '?');
+					g_string_append_len (gs, "id=", 3);
 					g_string_append_uri_escaped (gs, u->content, NULL, TRUE);
+					anyopt = TRUE;
+				}
+				if (u->version) {
+					g_string_append_c (gs, anyopt ? '&' : '?');
+					g_string_append_len (gs, "v=", 2);
+					g_string_append_uri_escaped (gs, u->version, NULL, TRUE);
+					anyopt = TRUE;
+				}
+			}
+		}
+	}
+	return gs;
+}
+
+static GString *
+_pack_fullpath(struct oio_url_s *u)
+{
+	GString *gs = g_string_new("");
+	if (u->account) {
+		g_string_append_uri_escaped(gs, u->account, NULL, TRUE);
+		if (u->user) {
+			g_string_append_c(gs, '/');
+			g_string_append_uri_escaped(gs, u->user, NULL, TRUE);
+			if (u->path) {
+				g_string_append_c(gs, '/');
+				g_string_append_uri_escaped(gs, u->path, NULL, TRUE);
+				if (u->version) {
+					g_string_append_c(gs, '/');
+					g_string_append_uri_escaped(gs, u->version, NULL, TRUE);
 				}
 			}
 		}
@@ -430,6 +472,11 @@ oio_url_get(struct oio_url_s *u, enum oio_url_field_e f)
 
 		case OIOURL_CONTENTID:
 			return u->content;
+
+		case OIOURL_FULLPATH:
+			if (!u->fullpath)
+				u->fullpath = g_string_free(_pack_fullpath(u), FALSE);
+			return u->fullpath;
 	}
 
 	g_assert_not_reached();
@@ -497,3 +544,29 @@ oio_url_to_json (GString *out, struct oio_url_s *u)
 	}
 }
 
+gboolean
+oio_url_check(const struct oio_url_s *u, const char *namespace, const gchar **err )
+{
+#define _ERR(v)  \
+    if (err) { \
+        *err = v; \
+    }
+
+    _ERR(NULL);
+    if (namespace && u->ns && strcmp(namespace, u->ns)) {
+        _ERR("namespace");
+        return 0;
+    }
+    if (u->version && !oio_str_is_number(u->version, NULL)) {
+        _ERR("version");
+        return 0;
+    }
+
+    if (u->path && !g_utf8_validate(u->path, -1, NULL)) {
+        _ERR("path");
+        return 0;
+    }
+    return 1;
+
+#undef _ERR
+}
