@@ -1,6 +1,6 @@
 /*
 OpenIO SDS event queue
-Copyright (C) 2016 OpenIO, original work as part of OpenIO Software Defined Storage
+Copyright (C) 2016-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -26,6 +26,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "oio_events_queue.h"
 #include "oio_events_queue_internals.h"
+#include "oio_events_queue_fanout.h"
 #include "oio_events_queue_zmq.h"
 #include "oio_events_queue_beanstalkd.h"
 
@@ -34,6 +35,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 void
 oio_events_queue__destroy (struct oio_events_queue_s *self)
 {
+	if (!self) return;
 	EVTQ_CALL(self,destroy)(self);
 }
 
@@ -64,16 +66,20 @@ oio_events_queue__is_stalled (struct oio_events_queue_s *self)
 	EVTQ_CALL(self,is_stalled)(self);
 }
 
-void
-oio_events_queue__set_max_pending (struct oio_events_queue_s *self, guint v)
+gint64
+oio_events_queue__get_health(struct oio_events_queue_s *self)
 {
-	EVTQ_CALL(self,set_max_pending)(self,v);
+	if (VTABLE_HAS(self,struct oio_events_queue_abstract_s*,get_health)) {
+		EVTQ_CALL(self,get_health)(self);
+	}
+	return 100;
 }
 
 void
-oio_events_queue__set_buffering(struct oio_events_queue_s *self, gint64 us)
+oio_events_queue__set_buffering (struct oio_events_queue_s *self,
+		gint64 delay)
 {
-	EVTQ_CALL(self,set_buffering)(self,us);
+	EVTQ_CALL(self,set_buffering)(self,delay);
 }
 
 GError *
@@ -91,6 +97,44 @@ _has_prefix (const char *cfg, const char *prefix)
 	return NULL;
 }
 
+static GError *
+_parse_and_create_multi(const char *cfg, struct oio_events_queue_s **out)
+{
+	gchar **tokens = g_strsplit(cfg, OIO_CSV_SEP2, -1);
+	if (!tokens)
+		return SYSERR("internal error");
+
+	GError *err = NULL;
+	GPtrArray *sub_queuev = g_ptr_array_new();
+
+	for (gchar **token = tokens; *token && !err ;++token) {
+		struct oio_events_queue_s *sub = NULL;
+		if (!(err = oio_events_queue_factory__create(*token, &sub)))
+			g_ptr_array_add(sub_queuev, sub);
+	}
+
+	if (!err) {
+		if (sub_queuev->len <= 0) {
+			err = BADREQ("empty connection string");
+		} else {
+			err = oio_events_queue_factory__create_fanout(
+					(struct oio_events_queue_s **)sub_queuev->pdata,
+					sub_queuev->len, out);
+		}
+	}
+
+	if (err) {
+		g_ptr_array_set_free_func(sub_queuev,
+				(GDestroyNotify)oio_events_queue__destroy);
+		g_ptr_array_free(sub_queuev, TRUE);
+	} else {
+		g_ptr_array_free(sub_queuev, FALSE);
+	}
+
+	g_strfreev(tokens);
+	return err;
+}
+
 GError *
 oio_events_queue_factory__create (const char *cfg, struct oio_events_queue_s **out)
 {
@@ -98,17 +142,22 @@ oio_events_queue_factory__create (const char *cfg, struct oio_events_queue_s **o
 	EXTRA_ASSERT (out != NULL);
 	*out = NULL;
 
-	const char *tmp;
+	if (NULL != strchr(cfg, OIO_CSV_SEP2_C)) {
+		// Sharding over several endpoints
+		return _parse_and_create_multi(cfg, out);
+	} else {
+		const char *tmp;
 
-	if (NULL != (tmp = _has_prefix (cfg, "beanstalk://")))
-		return oio_events_queue_factory__create_beanstalkd (tmp, out);
+		if (NULL != (tmp = _has_prefix (cfg, "beanstalk://")))
+			return oio_events_queue_factory__create_beanstalkd (tmp, out);
 
-	if (NULL != (tmp = _has_prefix (cfg, "ipc://"))
-			|| NULL != (tmp = _has_prefix (cfg, "tcp://"))
-			|| NULL != (tmp = _has_prefix (cfg, "inproc://")))
-		return oio_events_queue_factory__create_zmq (cfg, out);
+		if (NULL != (tmp = _has_prefix (cfg, "ipc://"))
+				|| NULL != (tmp = _has_prefix (cfg, "tcp://"))
+				|| NULL != (tmp = _has_prefix (cfg, "inproc://")))
+			return oio_events_queue_factory__create_zmq (cfg, out);
 
-	return BADREQ("implementation not recognized");
+		return BADREQ("implementation not recognized");
+	}
 }
 
 GError *
@@ -146,19 +195,30 @@ oio_event__init (GString *gs, const char *type, struct oio_url_s *url)
 	oio_str_gstring_append_json_pair (gs, "event", type);
 	g_string_append_printf (gs, ",\"when\":%"G_GINT64_FORMAT, oio_ext_real_time());
 	if (!url)
-		g_string_append (gs, ",\"url\":null");
+		g_string_append_static (gs, ",\"url\":null");
 	else {
-		g_string_append (gs, ",\"url\":{");
+		g_string_append_static (gs, ",\"url\":{");
 		oio_url_to_json (gs, url);
 		g_string_append_c (gs, '}');
 	}
 }
 
 GString*
-oio_event__create (const char *type, struct oio_url_s *url)
+oio_event__create(const char *type, struct oio_url_s *url)
+{
+	return oio_event__create_with_id(type, url, NULL);
+}
+
+GString*
+oio_event__create_with_id(const char *type, struct oio_url_s *url,
+		const char *request_id)
 {
 	GString *gs = g_string_sized_new(512);
 	g_string_append_c (gs, '{');
 	oio_event__init (gs, type, url);
+	if (request_id && *request_id) {
+		g_string_append_c(gs, ',');
+		oio_str_gstring_append_json_pair(gs, "request_id", request_id);
+	}
 	return gs;
 }
