@@ -1,6 +1,6 @@
 /*
 OpenIO SDS event queue
-Copyright (C) 2016 OpenIO, original work as part of OpenIO Software Defined Storage
+Copyright (C) 2016-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -35,14 +35,11 @@ void oio_events_queue__send_overwritable(struct oio_events_queue_s *self,
  * the queue won't deny events. */
 gboolean oio_events_queue__is_stalled (struct oio_events_queue_s *self);
 
-/* the queue might handle events asynchronously, and this is how to configure
- * the number of events waiting */
-void oio_events_queue__set_max_pending (struct oio_events_queue_s *self,
-		guint max);
+/* Get a health metric for the events queue, from 0 (bad) to 100 (good). */
+gint64 oio_events_queue__get_health(struct oio_events_queue_s *self);
 
-/* Set the overwritable events buffering delay */
 void oio_events_queue__set_buffering (struct oio_events_queue_s *self,
-		gint64 us);
+		gint64 delay);
 
 GError * oio_events_queue__run (struct oio_events_queue_s *self,
 		gboolean (*running) (gboolean pending));
@@ -55,6 +52,10 @@ void oio_event__init (GString *out, const char *type, struct oio_url_s *url);
 
 GString* oio_event__create (const char *type, struct oio_url_s *url);
 
+/* Create the base of a JSON formatted event, with the specified request ID. */
+GString* oio_event__create_with_id(const char *type, struct oio_url_s *url,
+		const char *request_id);
+
 /* -------------------------------------------------------------------------- */
 
 /* find the appropriate implementation of event queue for the configuration
