@@ -1,6 +1,6 @@
 /*
 OpenIO SDS event queue
-Copyright (C) 2016 OpenIO, original work as part of OpenIO Software Defined Storage
+Copyright (C) 2016-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -26,9 +26,10 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <sys/socket.h>
 
 #include <glib.h>
-#include <zmq.h>
+#include <math.h>
 
 #include <core/oio_core.h>
+#include <events/events_variables.h>
 #include <metautils/lib/metautils_sockets.h>
 #include <metautils/lib/metautils_resolv.h>
 #include <metautils/lib/metautils_syscall.h>
@@ -38,26 +39,31 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "oio_events_queue_beanstalkd.h"
 #include "oio_events_queue_buffer.h"
 
-#define INSERTED_PREFIX "INSERTED"
-#define USING_PREFIX "USING"
-
 #define EXPO_BACKOFF(DELAY,TRY,MAX_TRIES) \
 	g_usleep((1 << MIN(TRY, MAX_TRIES)) * DELAY); \
 	TRY++
 
+#define NETERR(FMT,...) NEWERROR(CODE_NETWORK_ERROR, FMT, ##__VA_ARGS__)
+
 static void _q_destroy (struct oio_events_queue_s *self);
 static void _q_send (struct oio_events_queue_s *self, gchar *msg);
 static void _q_send_overwritable(struct oio_events_queue_s *self, gchar *key, gchar *msg);
 static gboolean _q_is_stalled (struct oio_events_queue_s *self);
-static void _q_set_max_pending (struct oio_events_queue_s *self, guint v);
+static gint64 _q_get_health(struct oio_events_queue_s *self);
+
 static void _q_set_buffering (struct oio_events_queue_s *self, gint64 v);
 static GError * _q_run (struct oio_events_queue_s *self,
 		gboolean (*running) (gboolean pending));
 
 static struct oio_events_queue_vtable_s vtable_BEANSTALKD =
 {
-	_q_destroy, _q_send, _q_send_overwritable, _q_is_stalled,
-	_q_set_max_pending, _q_set_buffering, _q_run
+	.destroy = _q_destroy,
+	.send = _q_send,
+	.send_overwritable = _q_send_overwritable,
+	.is_stalled = _q_is_stalled,
+	.get_health = _q_get_health,
+	.set_buffering = _q_set_buffering,
+	.run = _q_run
 };
 
 /* Used by tests to intercept the result of the parsing of beanstalkd
@@ -72,7 +78,7 @@ struct _queue_BEANSTALKD_s
 	GAsyncQueue *queue;
 	gchar *endpoint;
 	gchar *tube;
-	guint max_events_in_queue;
+	gint64 pending_events;
 
 	struct oio_events_queue_buffer_s buffer;
 };
@@ -94,10 +100,9 @@ oio_events_queue_factory__create_beanstalkd (const char *endpoint,
 	self->vtable = &vtable_BEANSTALKD;
 	self->queue = g_async_queue_new ();
 	self->tube = g_strdup(OIO_EVT_BEANSTALKD_DEFAULT_TUBE);
-	self->max_events_in_queue = OIO_EVTQ_MAXPENDING;
 	self->endpoint = g_strdup (endpoint);
 
-	oio_events_queue_buffer_init(&(self->buffer), 1 * G_TIME_SPAN_SECOND);
+	oio_events_queue_buffer_init(&(self->buffer));
 
 	*out = (struct oio_events_queue_s*) self;
 	return NULL;
@@ -114,20 +119,22 @@ _q_set_buffering(struct oio_events_queue_s *self, gint64 v)
 	}
 }
 
-static void
-_q_set_max_pending (struct oio_events_queue_s *self, guint v)
+static GError *
+_match_common_error(gchar *buf)
 {
-	struct _queue_BEANSTALKD_s *q = (struct _queue_BEANSTALKD_s *)self;
-	EXTRA_ASSERT (q != NULL && q->vtable == &vtable_BEANSTALKD);
-	if (q->max_events_in_queue != v) {
-		GRID_INFO("max events in queue set to [%u]", v);
-		q->max_events_in_queue = v;
-	}
+	if (g_str_has_prefix(buf, "JOB_TOO_BIG"))
+		return SYSERR("Job too big");
+	if (g_str_has_prefix(buf, "OUT_OF_MEMORY"))
+		return BUSY("Beanstald is OOM");
+	return BADREQ("Invalid beanstalkd request/reply: %s", buf);
 }
 
 static int
 _send (int fd, struct iovec *iov, unsigned int iovcount)
 {
+	const int timeout =
+		oio_events_beanstalkd_timeout / G_TIME_SPAN_MILLISECOND;
+
 	int w;
 retry:
 	w = writev (fd, iov, iovcount);
@@ -135,14 +142,12 @@ retry:
 		if (errno == EINTR)
 			goto retry;
 		if (errno == EAGAIN || errno == EWOULDBLOCK) {
-			struct pollfd pfd = {0};
+			struct pollfd pfd = {};
 			pfd.fd = fd;
 			pfd.events = POLLOUT;
-			metautils_syscall_poll (&pfd, 1, 1000);
+			metautils_syscall_poll (&pfd, 1, timeout);
 			goto retry;
 		}
-		GRID_WARN("BEANSTALKD failed to put: [errno=%d] %s",
-				errno, strerror(errno));
 		return 0;
 	}
 
@@ -160,124 +165,240 @@ retry:
 	if (iovcount > 0)
 		goto retry;
 
-	GRID_TRACE("BEANSTALKD put sent!");
 	return 1;
 }
 
-static gboolean
-_is_success(gchar *buf)
-{
-	return g_str_has_prefix(buf, INSERTED_PREFIX)
-		|| g_str_has_prefix(buf, USING_PREFIX);
-}
-
 static GError *
-_send_and_read_reply (int fd, struct iovec *iov, unsigned int iovcount)
+_send_and_read_reply (int fd, struct iovec *iov, unsigned int iovcount,
+		gchar *dst, gsize dst_len)
 {
+	const int timeout =
+		oio_events_beanstalkd_timeout / G_TIME_SPAN_MILLISECOND;
+
 	if (!_send(fd, iov, iovcount))
-		return NEWERROR(CODE_NETWORK_ERROR,
-				"send error: (%d) %s",
-				errno, strerror(errno));
+		return NETERR("Send error: (%d) %s", errno, strerror(errno));
 
 	GError *err = NULL;
-	guint8 buf[256];
-	int r = sock_to_read (fd, 1000, buf, sizeof(buf)-1, &err);
-	if (r < 0)
-		return NEWERROR(CODE_NETWORK_ERROR,
-				"read error: (%d) %s", err->code, err->message);
+	int r = sock_to_read (fd, timeout, dst, dst_len - 1, &err);
+	if (r < 0) {
+		g_prefix_error(&err, "Read error: ");
+		return err;
+	}
+
 	if (r == 0)
-		return NEWERROR(CODE_NETWORK_ERROR,
-				"read error: closed by peer: (%d) %s",
-				errno, strerror(errno));
+		return NETERR("EOF");
 
-	buf[r+1] = 0;
-
-	if (!_is_success((gchar*) buf))
-		return NEWERROR(CODE_BAD_REQUEST, "reply error: unexpected");
+	dst[r] = 0;
 	return NULL;
 }
 
-static void
-_maybe_send_overwritable(struct oio_events_queue_s *self)
-{
-	struct _queue_BEANSTALKD_s *q = (struct _queue_BEANSTALKD_s *)self;
-	gboolean __send(gpointer key, gpointer msg, gpointer u UNUSED) {
-		g_free(key);
-		_q_send(self, (gchar*)msg);
-		return TRUE;
-	}
-
-	oio_events_queue_buffer_maybe_flush(&(q->buffer), __send, NULL);
-}
-
 static GError *
-_put (int fd, gchar *msg, size_t msglen)
+_put_job (int fd, gchar *msg, size_t msglen)
 {
 	gchar buf[256];
-	struct iovec iov[3];
 
+	/* send the request */
 	gsize len = g_snprintf (buf, sizeof(buf),
 			"put %u %u %u %"G_GSIZE_FORMAT"\r\n",
-			OIO_EVT_BEANSTALKD_DEFAULT_PRIO,
-			OIO_EVT_BEANSTALKD_DEFAULT_DELAY,
-			OIO_EVT_BEANSTALKD_DEFAULT_TTR,
+			oio_events_beanstalkd_default_prio,
+			(guint) oio_events_beanstalkd_default_delay,
+			(guint) oio_events_beanstalkd_default_ttr,
 			msglen);
+	struct iovec iov[3] = {};
 	iov[0].iov_base = buf;
 	iov[0].iov_len = len;
 	iov[1].iov_base = msg;
 	iov[1].iov_len = msglen;
 	iov[2].iov_base = "\r\n";
 	iov[2].iov_len = 2;
+	GError *err = _send_and_read_reply(fd, iov, 3, buf, sizeof(buf));
+	if (err) return err;
 
-	return _send_and_read_reply (fd, iov, 3);
+	/* No need to retry, the event has been saved ... or explicitely
+	 * dropped. */
+	const char * const replies_ok[] = { "INSERTED", "BURIED", "DRAINING", NULL };
+	for (const char * const *pmsg = replies_ok; *pmsg ;++pmsg) {
+		if (g_str_has_prefix(buf, *pmsg))
+			return NULL;
+	}
+
+	return _match_common_error(buf);
 }
 
 static GError *
 _use_tube (int fd, const char *name)
 {
+	gchar buf[256];
+
 	if (!oio_str_is_set(name))
 		return NULL;
 
-	gchar buf[256];
+	/* senf the request */
 	gsize len = g_snprintf(buf, sizeof(buf), "use %s\r\n", name);
-
 	if (len >= sizeof(buf))
-		return NEWERROR(CODE_INTERNAL_ERROR, "BUG: tube name too long");
-
-	struct iovec iov[1];
+		return SYSERR("BUG: tube name too long");
+	struct iovec iov[1] = {};
 	iov[0].iov_base = buf;
 	iov[0].iov_len = len;
-	return _send_and_read_reply (fd, iov, 1);
+	GError *err = _send_and_read_reply(fd, iov, 1, buf, sizeof(buf));
+	if (err) return err;
+
+	/* parse the reply */
+	if (g_str_has_prefix(buf, "USING"))
+		return NULL;
+	return _match_common_error(buf);
+}
+
+static GError *
+_check_server(struct _queue_BEANSTALKD_s *q, int fd)
+{
+	gchar buf[2048];
+
+	/* send the request */
+	strcpy(buf, "stats\r\n");
+	struct iovec iov[1] = {};
+	iov[0].iov_base = buf;
+	iov[0].iov_len = strlen(buf);
+	GError *err = _send_and_read_reply(fd, iov, 1, buf, sizeof(buf));
+	if (err) return err;
+
+	/* parse the reply */
+	if (!g_str_has_prefix(buf, "OK"))
+		return _match_common_error(buf);
+
+	gint64 total = 0;
+	gchar **lines = g_strsplit(buf, "\n", -1);
+	if (lines) {
+		for (gchar **pline = lines; *pline ;++pline) {
+			gchar *line = *pline;
+			gint64 count = 0;
+			if (!g_str_has_prefix(line, "current-jobs-"))
+				continue;
+			if (!(line = strchr(line, ':')))
+				continue;
+			if (!oio_str_is_number(g_strchug(line+1), &count))
+				continue;
+			total += count;
+		}
+		g_strfreev(lines);
+	}
+
+	q->pending_events = total;
+
+	const gint64 max_jobs = oio_events_beanstalkd_check_level_deny;
+	if (max_jobs > 0 && total > max_jobs) {
+		return BUSY("FULL (current=%" G_GINT64_FORMAT
+				" > limit=%" G_GINT64_FORMAT ")", total, max_jobs);
+	}
+
+	const gint64 alert = oio_events_beanstalkd_check_level_alert;
+	if (alert > 0 && total > alert) {
+		GRID_WARN("Beanstalkd load alert (current=%" G_GINT64_FORMAT
+				" > limit=%" G_GINT64_FORMAT ")", total, alert);
+	}
+	return NULL;
 }
 
 static GError *
 _poll_out (int fd)
 {
+	const int timeout =
+		oio_events_beanstalkd_timeout / G_TIME_SPAN_MILLISECOND;
+
 	int rc = 0;
-	struct pollfd pfd = {0};
+	struct pollfd pfd = {};
 	do {
 		pfd.fd = fd;
 		pfd.events = POLLOUT;
 		pfd.revents = 0;
-	} while (!(rc = metautils_syscall_poll(&pfd, 1, 1000)));
+	} while (!(rc = metautils_syscall_poll(&pfd, 1, timeout)));
 	if (pfd.revents != POLLOUT)
 		return socket_get_error(fd);
 	return NULL;
 }
 
+static void
+_flush_buffered(struct oio_events_queue_s *self, struct _queue_BEANSTALKD_s *q)
+{
+	gint avail =
+		oio_events_common_max_pending - g_async_queue_length(q->queue);
+	if (avail < (gint) oio_events_common_max_pending / 100) {
+		GRID_WARN("Pending events queue is reaching maximum: %d/%d",
+				g_async_queue_length(q->queue),
+				oio_events_common_max_pending);
+	}
+	/* This is not an else clause, we want to send the buffered events
+	 * (even if we do it slowly). */
+	oio_events_queue_send_buffered(self, &(q->buffer), MAX(1, avail / 2));
+}
+
 static GError *
 _q_run (struct oio_events_queue_s *self, gboolean (*running) (gboolean pending))
 {
 	struct _queue_BEANSTALKD_s *q = (struct _queue_BEANSTALKD_s *)self;
 	gchar *saved = NULL;
 	int fd = -1;
-	int try_count = 0;
+	guint attempts_connect = 0, attempts_check = 0, attempts_put = 0;
+	gint64 last_flush = 0, last_check = 0;
 
 	EXTRA_ASSERT (q != NULL && q->vtable == &vtable_BEANSTALKD);
 	EXTRA_ASSERT (running != NULL);
 
 	while ((*running)(0 < g_async_queue_length(q->queue))) {
-		_maybe_send_overwritable(self);
+
+		const gint64 now = oio_ext_monotonic_time();
+
+		/* Maybe reconnect to the beanstalkd */
+		if (fd < 0) {
+			GError *err = NULL;
+			fd = sock_connect(q->endpoint, &err);
+			if (!err)
+				err = _poll_out (fd);
+			if (err) {
+				g_prefix_error(&err, "Connection error: ");
+			} else {
+				err = _use_tube (fd, q->tube);
+				if (intercept_errors)
+					(*intercept_errors) (err);
+				if (err)
+					g_prefix_error(&err, "USE command error: ");
+			}
+			if (err) {
+				metautils_pclose(&fd);
+				GRID_WARN("BEANSTALK error to %s: (%d) %s", q->endpoint,
+						err->code, err->message);
+				g_clear_error (&err);
+
+				EXPO_BACKOFF(250 * G_TIME_SPAN_MILLISECOND, attempts_connect, 4);
+				continue;
+			} else {
+				GRID_INFO("BEANSTALKD: connected to %s", q->endpoint);
+				attempts_connect = 0;
+			}
+		}
+
+		/* Maybe do a periodic check of the beanstalkd tube */
+		if (oio_events_beanstalkd_check_period > 0 &&
+				last_check < OLDEST(now, oio_events_beanstalkd_check_period)) {
+			GError *err = _check_server(q, fd);
+			if (intercept_errors)
+				(*intercept_errors) (err);
+			if (err) {
+				GRID_WARN("Beanstalkd error [%s]: (%d) %s", q->endpoint, err->code, err->message);
+				g_clear_error(&err);
+				EXPO_BACKOFF(250 * G_TIME_SPAN_MILLISECOND, attempts_check, 4);
+				continue;
+			} else {
+				last_check = now;
+				attempts_check = 0;
+			}
+		}
+
+		/* Build the deadline for the timed wait */
+		if (now - last_flush > q->buffer.delay / 10) {
+			last_flush = now;
+			_flush_buffered(self, q);
+		}
 
 		/* find an event, prefering the last that failed */
 		gchar *msg = saved;
@@ -285,46 +406,29 @@ _q_run (struct oio_events_queue_s *self, gboolean (*running) (gboolean pending))
 		if (!msg) msg = g_async_queue_timeout_pop (q->queue, G_TIME_SPAN_SECOND);
 		if (!msg) continue;
 
-		/* forward the event */
+		/* forward the event as a beanstalkd job */
 		if (*msg) {
-			GError *err = NULL;
-
-			/* lazy-reconnection, with backoff sleeping to avoid crazy-looping */
-			if (fd < 0) {
-				fd = sock_connect(q->endpoint, &err);
-				if (!err)
-					err = _poll_out (fd);
-				if (!err) {
-					err = _use_tube (fd, q->tube);
-					if (intercept_errors)
-						(*intercept_errors) (err);
-				}
-				if (err) {
-					metautils_pclose(&fd);
-					GRID_WARN("BEANSTALKD: reconnection failed to %s: (%d) %s",
-							q->endpoint, err->code, err->message);
-					g_clear_error (&err);
-					saved = msg;
-					msg = NULL;
-
-					EXPO_BACKOFF(250 * G_TIME_SPAN_MILLISECOND, try_count, 4);
-					continue;
-				} else {
-					GRID_INFO("BEANSTALKD: connected to %s", q->endpoint);
-					try_count = 0;
-				}
-			}
-
-			/* prepare the header, and the buffers to be sent */
-			err = _put (fd, msg, strlen(msg));
+			const size_t msglen = strlen(msg);
+			GError *err = _put_job (fd, msg, msglen);
 			if (intercept_errors)
 				(*intercept_errors) (err);
-			if (err) {
+			if (!err) {
+				attempts_put = 0;
+			} else {
+				if (CODE_IS_RETRY(err->code) || CODE_IS_NETWORK_ERROR(err->code)) {
+					saved = msg;
+					msg = NULL;
+					EXPO_BACKOFF(250 * G_TIME_SPAN_MILLISECOND, attempts_put, 4);
+				} else {
+					GRID_WARN("Unrecoverable error with beanstalkd at [%s]: (%d) %s",
+							q->endpoint, err->code, err->message);
+					GRID_NOTICE("dropped %d %.*s",
+							(int)msglen, (int)MIN(msglen,2048), msg);
+					attempts_put = 0;
+				}
 				g_clear_error (&err);
 				sock_set_linger(fd, 1, 1);
 				metautils_pclose (&fd);
-				saved = msg;
-				msg = NULL;
 			}
 		}
 
@@ -378,5 +482,15 @@ _q_is_stalled (struct oio_events_queue_s *self)
 	const int l = g_async_queue_length (q->queue);
 	if (l <= 0)
 		return FALSE;
-	return ((guint)l) >= q->max_events_in_queue;
+	return ((guint)l) >= oio_events_common_max_pending;
+}
+
+static gint64
+_q_get_health(struct oio_events_queue_s *self)
+{
+	struct _queue_BEANSTALKD_s *q = (struct _queue_BEANSTALKD_s*) self;
+	EXTRA_ASSERT(q != NULL && q->vtable == &vtable_BEANSTALKD);
+
+	gint64 res = (gint64) (100.0 / (1.0 + log(1.0 + q->pending_events * 0.1)));
+	return MIN(SCORE_MAX, res);
 }
