@@ -1,7 +1,7 @@
 /*
 OpenIO SDS gridd
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -29,14 +29,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 static void
 _log_addr(GString *gs, int fd)
 {
-	struct sockaddr_storage ss;
-	socklen_t ss_len;
+	struct sockaddr_storage ss = {0};
+	socklen_t ss_len = sizeof(ss);
 	char buf_addr[STRLEN_ADDRINFO];
 	char buf_port[8];
 
-	memset(&ss, 0, sizeof(ss));
-	ss_len = sizeof(ss);
-
 	if (0 != getsockname(fd, (struct sockaddr*)&ss, &ss_len))
 		g_string_append_c(gs, '?');
 	else {
