@@ -1,7 +1,7 @@
 /*
 OpenIO SDS gridd
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -129,11 +129,11 @@ gridd_set_flag(enum gridd_flag_e flag, int onoff)
 		gridd_flags = gridd_flags & ~flag;
 
 	if (old_flags != gridd_flags) {
-		NOTICE("GRIDD flags changed (%08X) : [%08X] -> [%08X]",
+		GRID_NOTICE("GRIDD flags changed (%08X) : [%08X] -> [%08X]",
 				flag, old_flags, gridd_flags);
 	}
 	else {
-		DEBUG("GRIDD flags unchanged (%08X) : [%08X]", flag, gridd_flags);
+		GRID_DEBUG("GRIDD flags unchanged (%08X) : [%08X]", flag, gridd_flags);
 	}
 }
 
@@ -216,16 +216,10 @@ server_alert_possible( struct server_s *srv )
 		srv->mon.min_spare_workers, srv->mon.max_spare_workers)
 
 static void
-thread_monitoring_add (struct server_s *srv, gboolean locked)
+thread_monitoring_add (struct server_s *srv)
 {
-	if (!locked)
-		g_rec_mutex_lock (&(srv->recMutex));
-
 	srv->mon.creation ++;
 	srv->mon.nb_workers ++;
-
-	if (!locked)
-		g_rec_mutex_unlock (&(srv->recMutex));
 }
 
 static void
@@ -250,7 +244,7 @@ thread_start(struct server_s *srv)
 	if (NULL != (th = g_thread_try_new("main", main_thread, srv, &gErr)))
 		return TRUE;
 
-	ERROR("Cannot start a worker thread : %s", gErr?gErr->message:"?");
+	GRID_ERROR("Cannot start a worker thread : %s", gErr?gErr->message:"?");
 	if (gErr)
 		g_error_free(gErr);
 
@@ -282,7 +276,7 @@ thread_monitoring_reserve (struct server_s *srv)
 
 	if (too_few_workers || too_few_spare_workers) {
 		if (!too_many_spare_workers && !too_many_workers) {
-			thread_monitoring_add(srv, TRUE);
+			thread_monitoring_add(srv);
 			defer_creation = TRUE;
 			str_dbg = threads_debug(srv);
 		}
@@ -293,7 +287,7 @@ thread_monitoring_reserve (struct server_s *srv)
 
 	/* Deferred actions */
 	if (str_dbg) {
-		TRACE_DOMAIN(DOMAIN_THREADS, "%s TOO FEW", str_dbg);
+		GRID_TRACE("%s TOO FEW", str_dbg);
 		g_free(str_dbg);
 	}
 	if (defer_creation) {
@@ -309,7 +303,6 @@ static gboolean
 thread_monitoring_release (struct server_s *srv)
 {
 	gboolean rc = TRUE;
-	gchar *str_dbg = NULL;
 
 	/* XXX start of locked section */
 	g_rec_mutex_lock (&(srv->recMutex));
@@ -325,7 +318,6 @@ thread_monitoring_release (struct server_s *srv)
 	if ((srv->mon.nb_workers>1)) {
 		if (too_many_workers || too_many_spare_workers) {
 			if (!too_few_spare_workers && !too_few_workers) {
-				str_dbg = threads_debug(srv);
 				thread_monitoring_remove (srv, TRUE);
 				rc = FALSE;
 			}
@@ -335,56 +327,36 @@ thread_monitoring_release (struct server_s *srv)
 	g_rec_mutex_unlock (&(srv->recMutex));
 	/* XXX end of locked section */
 
-	if (str_dbg) {
-		TRACE_DOMAIN(DOMAIN_THREADS, "%s TOO MANY", str_dbg);
-		g_free(str_dbg);
-	}
 	return rc;
 }
 
 static void
 thread_monitoring_periodic_debug (struct server_s *srv)
 {
-	gchar *str_dbg = NULL;
 	struct thread_monitoring_s mon, mon0;
-	gchar str_pool[512];
-	gsize str_pool_size;
 
 	/* XXX locked section */
 	g_rec_mutex_lock (&(srv->recMutex));
 	memcpy(&mon, &(srv->mon), sizeof(struct thread_monitoring_s));
 	memcpy(&mon0, &(srv->mon0), sizeof(struct thread_monitoring_s));
 	memcpy(&(srv->mon0), &(srv->mon), sizeof(struct thread_monitoring_s));
-	str_dbg = threads_debug(srv);
 	g_rec_mutex_unlock (&(srv->recMutex));
 	/* XXX end of locked section */
 
 	gboolean too_many_workers = (mon.used_workers >= mon.max_workers);
 	gboolean max_reached = (mon.max_reached != mon0.max_reached);
 
-	/*and then print all*/
-	DEBUG_DOMAIN (DOMAIN_THREADS, "%s wake=%"G_GUINT64_FORMAT" variation=+%"G_GUINT64_FORMAT"-%"G_GUINT64_FORMAT,
-			str_dbg,
-			(mon.wake - mon0.wake),
-			(mon.creation - mon0.creation),
-			(mon.destruction - mon0.destruction));
-	g_free(str_dbg);
-
 	if (too_many_workers) {
-		WARN_DOMAIN (DOMAIN_THREADS, "%s ALL THREADS are currently used!", srv->name);
+		GRID_WARN ("%s ALL THREADS are currently used!", srv->name);
 		if (server_alert_possible(srv)) {
-			str_pool_size = accept_pool_to_string(srv->ap, str_pool, sizeof(str_pool));
-			SRV_SEND_ERROR(ALERTID_SRV_THREADS,"Server [%s] on [%.*s] currently uses all its threads",
-				srv->name, str_pool_size, str_pool);
+			SRV_SEND_ERROR(ALERTID_SRV_THREADS,"Server [%s] currently uses all its threads", srv->name);
 			server_notify_alert(srv);
 		}
 	}
 	else if (max_reached) {
-		WARN_DOMAIN (DOMAIN_THREADS, "%s ALL THREADS have been used!", srv->name);
+		GRID_WARN ("%s ALL THREADS have been used!", srv->name);
 		if (server_alert_possible(srv)) {
-			str_pool_size = accept_pool_to_string(srv->ap, str_pool, sizeof(str_pool));
-			SRV_SEND_WARNING(ALERTID_SRV_THREADS,"Server [%s] on [%.*s] used all its threads",
-				srv->name, str_pool_size, str_pool);
+			SRV_SEND_WARNING(ALERTID_SRV_THREADS,"Server [%s] used all its threads", srv->name);
 			server_notify_alert(srv);
 		}
 	}
@@ -407,40 +379,6 @@ thread_monitoring_periodic_stats (struct server_s *srv)
 
 /* ------------------------------------------------------------------------- */
 
-gint
-get_network_socket (message_handler_f h, char **addr, int *port, GError **error)
-{
-	SERVER s = NULL;
-	ACCEPT_POOL ap = NULL;
-	int id = 0, i = 0;
-	struct sockaddr_storage sock_name;
-	socklen_t sock_len = sizeof(struct sockaddr_storage);
-	char host[48], str_port[6];
-
-	for (s=BEACON_SRV.next; s && s!=&BEACON_SRV ;s=s->next) {
-		ap = s->ap;
-
-		for (i = 0; i < s->nbHandlers; i++) {
-			if (s->handlers[i]->handler == h) {
-				for (id = 0; id < ap->count; id++) {
-					memset(&sock_name, 0, sizeof(struct sockaddr));
-					if (!getsockname(ap->srv[id], (struct sockaddr*)&sock_name, &sock_len) &&
-					   (sock_name.ss_family == PF_INET || sock_name.ss_family == PF_INET6)) {
-						if (format_addr((struct sockaddr*)&sock_name, host, sizeof(host), str_port, sizeof(str_port), error)) {
-							*addr = strdup(host);
-							*port = atoi(str_port);
-							return(1);
-						}
-					}
-				}
-			}
-		}
-	}
-
-	GSETERROR(error, "No notwork socket found for this handler");
-	return(0);
-}
-
 static gint
 manage_message (SERVER srv, GByteArray *gba, struct request_context_s* ctx, GError **err)
 {
@@ -523,7 +461,7 @@ main_thread (gpointer arg)
 		/*accept a new connection*/
 		if (0 > (clt = accept_do(srv->ap, &clt_addr, &gErr))) {
 			if (gErr) {
-				ERROR("Cannot accept a new connection : %s", gErr->message);
+				GRID_ERROR("Cannot accept a new connection : %s", gErr->message);
 				g_clear_error(&gErr);
 			}
 			continue;
@@ -538,7 +476,7 @@ main_thread (gpointer arg)
 
 		memset(str_addr_src, 0, sizeof(str_addr_src));
 		grid_addrinfo_to_string(ctx->remote_addr, str_addr_src, sizeof(str_addr_src));
-		TRACE ("Connection NEW fd=%d [%s]", clt, str_addr_src);
+		GRID_TRACE ("Connection NEW fd=%d [%s]", clt, str_addr_src);
 
 		while (may_continue) {
 
@@ -572,25 +510,10 @@ main_thread (gpointer arg)
 
 		request_context_free(ctx);
 
-		if (gErr) {
-			switch (gErr->code) {
-				case ERRCODE_CONN_RESET:
-				case ERRCODE_CONN_CLOSED:
-					TRACE ("Connection CLOSED/RESET fd=%i [%s]", clt, str_addr_src);
-					break;
-				case ERRCODE_CONN_TIMEOUT:
-					DEBUG ("Connection TIMEOUT fd=%i [%s]", clt, str_addr_src);
-					break;
-				default:
-					DEBUG ("Connection ERROR fd=%i [%s]", clt, str_addr_src);
-					if (gErr->message)
-						DEBUG ("cause:\n\t%s", gErr->message);
-					break;
-			}
+		if (gErr)
 			g_clear_error (&gErr);
-		}
 
-		TRACE ("Connection CLOSING fd=%i [%s]", clt, str_addr_src);
+		GRID_TRACE ("Connection CLOSING fd=%i [%s]", clt, str_addr_src);
 		if (gridd_flags & GRIDD_FLAG_SHUTDOWN)
 			shutdown (clt, SHUT_RDWR);
 		metautils_pclose (&clt);
@@ -607,43 +530,25 @@ exit:
 }
 
 static GHashTable*
-extract_parameters (GKeyFile *kf, const char *s, const char *p, GError **err)
+extract_parameters (GKeyFile *kf, const char *s, const char *p)
 {
-	gchar **all_keys=NULL, **current_key=NULL;
+	const size_t pref_len = p ? strlen(p) : 0;
+
+	GHashTable *ht= g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+
 	gsize size=0;
-	GHashTable *ht=NULL;
-	size_t pref_len = p? strlen(p) : 0;
-
-	ht = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
-
-	all_keys = g_key_file_get_keys (kf, s, &size, err);
-	if (!all_keys)
-	{
-		GSETERROR(err, "cannot get the keys of the section '%s'", s);
-	}
-	for (current_key = all_keys; all_keys && *current_key; current_key++)
-	{
-		if (g_str_has_prefix(*current_key, p))
-		{
-			gchar *value = NULL;
-			value = g_key_file_get_value (kf, s, *current_key, err);
-			if (!value)
-			{
-				GSETERROR (err, "Cannot get the value");
-				goto error;
-			}
-			g_hash_table_insert(ht, g_strdup(*current_key + pref_len), value);
+	gchar **all_keys = g_key_file_get_keys (kf, s, &size, NULL);
+	for (gchar **current_key = all_keys; all_keys && *current_key; current_key++) {
+		if (g_str_has_prefix(*current_key, p)) {
+			gchar *value = g_key_file_get_value (kf, s, *current_key, NULL);
+			if (value)
+				g_hash_table_insert(ht, g_strdup(*current_key + pref_len), value);
 		}
 	}
 
-	g_strfreev(all_keys);
-	return ht;
-error:
-	if (ht)
-		g_hash_table_destroy(ht);
 	if (all_keys)
 		g_strfreev(all_keys);
-	return NULL;
+	return ht;
 }
 
 static int
@@ -654,13 +559,7 @@ preload_plugins (GKeyFile *cfgFile, GError **err)
 	gsize nbgroups=0, i;
 	gchar *fileName=NULL;
 
-	if (!cfgFile)
-	{
-		GSETERROR(err,"Invalid Parameter");
-		goto errorLabel;
-	}
-
-	DEBUG ("Start loading all the plugins found in the configuration");
+	EXTRA_ASSERT(cfgFile != NULL);
 
 	/*run the key's list and keep those mathing Plugin~*/
 	groups = g_key_file_get_groups (cfgFile, &nbgroups);
@@ -683,8 +582,6 @@ preload_plugins (GKeyFile *cfgFile, GError **err)
 
 		if (0 == fnmatch("Plugin.*", group, 0))
 		{
-			GModule *mod = NULL;
-
 			/*get the filename of the plugin*/
 			if (    !g_key_file_has_key (cfgFile, group, NAME_PATH, err)
 				||	!(fileName = g_key_file_get_string (cfgFile, group, NAME_PATH, err)))
@@ -694,28 +591,17 @@ preload_plugins (GKeyFile *cfgFile, GError **err)
 			}
 
 			/*get the parameter string*/
-			params = extract_parameters (cfgFile, group, NAME_PARAM"_", err);
-			if (!params)
-			{
-				GSETERROR(err,"Cannot load the parameters hash for the plugin (group %s in the configuration file)", group);
-				goto errorLabel;
-			}
+			params = extract_parameters (cfgFile, group, NAME_PARAM "_");
 
 			/*load the path*/
-			mod = g_module_open (fileName, 0);
-			if (!mod)
-			{
+			GModule *mod = g_module_open (fileName, 0);
+			if (!mod) {
 				GSETERROR(err, "Cannot load the plug-in from file %s (%s)", fileName, g_module_error());
 				goto errorLabel;
 			}
 
 			/*load the main exported symbol*/
-			if (plugin_holder_keep(mod, params, err))
-			{
-				DEBUG ("loaded %s", fileName);
-			}
-			else
-			{
+			if (!plugin_holder_keep(mod, params, err)) {
 				GSETERROR(err, "cannot load %s", fileName);
 				goto errorLabel;
 			}
@@ -758,12 +644,12 @@ set_srv_addr(const gchar* url)
 
 	newaddr = g_malloc0(sizeof(addr_info_t));
 	if (!grid_string_to_addrinfo(url, newaddr))
-		ERROR("Failed to init the server address to [%s] : %s", url, gerror_get_message(local_error));
+		GRID_ERROR("Failed to init the server address to [%s] : %s", url, gerror_get_message(local_error));
 	if (local_error)
 		g_clear_error(&local_error);
 
 	grid_addrinfo_to_string(newaddr, str_addr, sizeof(str_addr));
-	NOTICE("Saving [%s] as main server address", str_addr);
+	GRID_NOTICE("Saving [%s] as main server address", str_addr);
 }
 
 static int
@@ -773,13 +659,7 @@ load_servers (GKeyFile *cfgFile, GError **err)
 	gchar **groups=NULL;
 	gsize nbgroups=0, i;
 
-	if (!cfgFile)
-	{
-		GSETERROR(err,"Invalid Parameter");
-		goto errorLabel;
-	}
-
-	DEBUG ("Start loading all the servers found in the configuration");
+	EXTRA_ASSERT(cfgFile != NULL);
 
 	/*run the key's list and keep those mathing Plugin~*/
 	groups = g_key_file_get_groups (cfgFile, &nbgroups);
@@ -794,11 +674,7 @@ load_servers (GKeyFile *cfgFile, GError **err)
 		gchar *group;
 		group = groups[i];
 
-		if (!group)
-		{
-			GSETERROR (err, "Invalid group name");
-			return 0;
-		}
+		if (!group) continue;
 
 		if (0 == fnmatch("Server.*", group, 0))
 		{
@@ -905,7 +781,7 @@ load_servers (GKeyFile *cfgFile, GError **err)
 					if (0 == strcmp (h->name, pluginList [nbPlugins-1]))
 					{
 						BEACON_SRV.next->handlers[nbPlugins-1] = h;
-						DEBUG ("The message handler '%s' has been prepended to the server '%s'", pluginList [nbPlugins-1], BEACON_SRV.next->name);
+						GRID_DEBUG ("The message handler '%s' has been prepended to the server '%s'", pluginList [nbPlugins-1], BEACON_SRV.next->name);
 						break;
 					}
 				}
@@ -918,7 +794,7 @@ load_servers (GKeyFile *cfgFile, GError **err)
 			}
 
 			/*debug the server structure*/
-			DEBUG ("New server created SRV=%s POOL=%p WORKER={min:%d max:%d min_spare:%d max_spare:%d} TO_CNX=%i TO_OP=%i",
+			GRID_DEBUG ("New server created SRV=%s POOL=%p WORKER={min:%d max:%d min_spare:%d max_spare:%d} TO_CNX=%i TO_OP=%i",
 				srv->name, (void*)srv->ap,
 				srv->mon.min_workers, srv->mon.max_workers,
 				srv->mon.min_spare_workers, srv->mon.max_spare_workers,
@@ -1002,7 +878,7 @@ load_configuration (const char *cfg_path, GError **err)
 {
 	GKeyFile *cfgFile = NULL;
 
-	INFO("Logging facility configured!");
+	GRID_INFO("Logging facility configured!");
 
 	/*Parse the configuration*/
 	cfgFile = g_key_file_new ();
@@ -1031,7 +907,7 @@ load_configuration (const char *cfg_path, GError **err)
 		goto errorLabel;
 	}
 
-	INFO ("Plug-in's loaded!");
+	GRID_INFO ("Plug-in's loaded!");
 
 	/*then load the servers from the configuration file*/
 	if (!load_servers (cfgFile, err)) {
@@ -1039,7 +915,7 @@ load_configuration (const char *cfg_path, GError **err)
 		goto errorLabel;
 	}
 
-	INFO ("Server threads loaded!");
+	GRID_INFO ("Server threads loaded!");
 
 	g_key_file_free (cfgFile);
 	return 1;
@@ -1067,7 +943,7 @@ retry:
 	for (struct server_s *srv=BEACON_SRV.next ; srv ; srv=srv->next) {
 		gint64 i64;
 		if (0 < (i64 = server_has_thread(srv))) {
-			INFO("Waiting for workers : still %"G_GINT64_FORMAT, i64);
+			GRID_INFO("Waiting for workers : still %"G_GINT64_FORMAT, i64);
 			g_usleep(G_TIME_SPAN_SECOND);
 			goto retry;
 		}
@@ -1086,7 +962,7 @@ start_server_threads(struct server_s *srv)
 	g_rec_mutex_lock (&(srv->recMutex));
 	max = MAX(srv->mon.min_workers,srv->mon.min_spare_workers);
 	for (i=0; i<max ;i++)
-		thread_monitoring_add(srv, TRUE);
+		thread_monitoring_add(srv);
 	g_rec_mutex_unlock (&(srv->recMutex));
 	/* XXX end of locked section */
 
@@ -1096,7 +972,7 @@ start_server_threads(struct server_s *srv)
 		if (thread_start(srv)) {
 			nb++;
 		} else {
-			ERROR("Cannot start a worker thread for %s", srv->name);
+			GRID_ERROR("Cannot start a worker thread for %s", srv->name);
 			thread_monitoring_remove(srv, FALSE);
 		}
 	}
@@ -1112,9 +988,9 @@ start_threads (void)
 
 	for (srv=BEACON_SRV.next ; srv ; srv=srv->next) {
 		if (start_server_threads(srv))
-			NOTICE("Some threads started for %s",srv->name);
+			GRID_NOTICE("Some threads started for %s",srv->name);
 		else {
-			ERROR("No threads started for %s",srv->name);
+			GRID_ERROR("No threads started for %s",srv->name);
 			all_done = FALSE;
 		}
 	}
@@ -1131,7 +1007,7 @@ main_action (void)
 
 	/*start the threads*/
 	if (!start_threads()) {
-		ERROR("Cannot start threads for all the servers");
+		GRID_ERROR("Cannot start threads for all the servers");
 		grid_main_stop();
 		return;
 	}
@@ -1169,13 +1045,13 @@ main_configure(int argc, char **argv)
 	if (argc > 0) {
 		config_file = g_strdup(argv[0]);
 	} else {
-		ERROR("Missing argument\n");
+		GRID_ERROR("Missing argument\n");
 		return FALSE;
 	}
 
 	GError *err = NULL;
 	if (!load_configuration (config_file, &err)) {
-		ERROR("Failed to load the configuration : (%d) %s", err->code, err->message);
+		GRID_ERROR("Failed to load the configuration : (%d) %s", err->code, err->message);
 		return FALSE;
 	}
 
@@ -1195,19 +1071,13 @@ main_specific_fini (void)
 		threads_to_be_joined = NULL;
 	}
 
-	DEBUG("Closing the servers");
+	GRID_DEBUG("Closing the servers");
 	for (SERVER srv=BEACON_SRV.next; srv ;srv=srv->next) {
-		if (srv->ap) {
-			GError *error = NULL;
-			int nb = accept_close_servers( srv->ap, &error);
-			if (nb>0)
-				ERROR("%d server sockets could not be stopped : %s", nb, error?error->message:"?");
-			if (error)
-				g_clear_error( &error);
-		}
+		if (srv->ap)
+			accept_close_servers(srv->ap);
 	}
 
-	DEBUG("Stopping the servers");
+	GRID_DEBUG("Stopping the servers");
 	for (SERVER srv=BEACON_SRV.next; srv ;srv=srv->next) {
 		if (srv->ap) {
 			g_free(srv->ap);
@@ -1217,9 +1087,9 @@ main_specific_fini (void)
 
 	plugin_holder_close_all();
 
-	DEBUG("Unlinking the servers");
+	GRID_DEBUG("Unlinking the servers");
 	for (struct server_s *s=BEACON_SRV.next; s ;s=s->next) {
-		DEBUG("Stopping %p", s);
+		GRID_DEBUG("Stopping %p", s);
 		if (s->handlers) {
 			g_free(s->handlers);
 			s->handlers = NULL;
@@ -1227,19 +1097,19 @@ main_specific_fini (void)
 		g_rec_mutex_clear(&(s->recMutex));
 	}
 
-	DEBUG("Cleaning the servers");
+	GRID_DEBUG("Cleaning the servers");
 	for (struct server_s *s=BEACON_SRV.next; s ;) {
 		struct server_s *sTmp;
-		DEBUG("Cleaning %p", s);
+		GRID_DEBUG("Cleaning %p", s);
 		sTmp = s->next;
 		g_free(s);
 		s = sTmp;
 	}
 
-	DEBUG("Cleaning the message handlers");
+	GRID_DEBUG("Cleaning the message handlers");
 	for (struct message_handler_s *m=BEACON_MSGHANDLER.next; m ;) {
 		struct message_handler_s *mTmp;
-		DEBUG("Cleaning %p", m);
+		GRID_DEBUG("Cleaning %p", m);
 		mTmp = m->next;
 		g_free(m);
 		m = mTmp;
