@@ -1,7 +1,7 @@
 /*
 OpenIO SDS gridd
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -24,38 +24,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 typedef void (*srvstat_iterator_gvariant_f) (gpointer u, const gchar *name, GVariant *gv);
 
-/*!
- * @deprecated
- * @see srvstat_set_double()
- * @see srvstat_set_gvariant()
- */
-gboolean srvstat_set (const gchar *name, gdouble value);
-
-gboolean srvstat_set_double (const gchar *name, gdouble value);
-
 gboolean srvstat_set_u64 (const gchar *name, guint64 value);
 
 gboolean srvstat_set_gvariant(const gchar *name, GVariant* gv);
 
-/*!
- * @deprecated
- * @see srvstat_get_double()
- * @see srvstat_get_gvariant()
- */
-gboolean srvstat_get (const gchar *name, gdouble *value);
-
-gboolean srvstat_get_double (const gchar *name, gdouble *value);
-
-GVariant* srvstat_get_gvariant (const gchar *name);
-
-void srvstat_del (const gchar *name);
-
 void srvstat_init (void);
 
 void srvstat_fini (void);
 
-void srvstat_flush (void);
-
 void srvstat_foreach_gvariant (const gchar *pattern, srvstat_iterator_gvariant_f cb, void *udata);
 
 #endif /*OIO_SDS__gridd__main__srvstats_h*/
