@@ -1,7 +1,7 @@
 /*
 OpenIO SDS gridd
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -24,7 +24,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 typedef void (*srvtimer_f) (gpointer udata);
 
-gboolean srvtimer_register_regular (const char *name, srvtimer_f fire,
+void srvtimer_register_regular (const char *name, srvtimer_f fire,
 	srvtimer_f close, gpointer udata, guint64 freq);
 
 void srvtimer_init (void);
@@ -33,4 +33,4 @@ void srvtimer_fini (void);
 
 void srvtimer_fire (guint64 ticks);
 
-#endif /*OIO_SDS__gridd__main__srvtimer_h*/
\ No newline at end of file
+#endif /*OIO_SDS__gridd__main__srvtimer_h*/
