@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta0v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -17,20 +17,16 @@ You should have received a copy of the GNU Affero General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <stddef.h>
-#include <stdlib.h>
-#include <string.h>
 #include <errno.h>
 
 #include <json-c/json.h>
 
-#include <metautils/lib/metacomm.h>
+#include <metautils/lib/metautils.h>
 #include <sqliterepo/sqliterepo.h>
 #include <sqliterepo/version.h>
 
-#include "./internals.h"
-#include "./meta0_utils.h"
-#include "./meta0_backend.h"
+#include "meta0_utils.h"
+#include "meta0_backend.h"
 
 struct meta0_backend_s
 {
@@ -38,7 +34,6 @@ struct meta0_backend_s
 	gchar *ns;
 	GRWLock rwlock;
 	GPtrArray *array_by_prefix;
-	GPtrArray *array_meta1_ref;
 	struct sqlx_repository_s *repository;
 	gboolean reload_requested;
 };
@@ -50,21 +45,21 @@ static void _unlock_and_close(struct sqlx_sqlite3_s *sq3);
 
 /* ------------------------------------------------------------------------- */
 
-static int
+static enum sqlx_open_type_e
 m0_to_sqlx(enum m0v2_open_type_e t)
 {
 	switch (t & 0x03) {
 		case M0V2_OPENBASE_LOCAL:
-			return SQLX_OPEN_LOCAL;
+			return SQLX_OPEN_CREATE|SQLX_OPEN_LOCAL;
 		case M0V2_OPENBASE_MASTERONLY:
-			return SQLX_OPEN_MASTERONLY;
+			return SQLX_OPEN_CREATE|SQLX_OPEN_MASTERONLY;
 		case M0V2_OPENBASE_MASTERSLAVE:
-			return SQLX_OPEN_MASTERSLAVE;
+			return SQLX_OPEN_CREATE|SQLX_OPEN_MASTERSLAVE;
 		case M0V2_OPENBASE_SLAVEONLY:
-			return SQLX_OPEN_SLAVEONLY;
+			return SQLX_OPEN_CREATE|SQLX_OPEN_SLAVEONLY;
 	}
 	g_assert_not_reached();
-	return SQLX_OPEN_LOCAL;
+	return SQLX_OPEN_CREATE|SQLX_OPEN_LOCAL;
 }
 
 struct meta0_backend_s *
@@ -76,7 +71,6 @@ meta0_backend_init(const gchar *ns, const gchar *id,
 	m0->id = g_strdup(id);
 	m0->ns = g_strdup(ns);
 	m0->array_by_prefix = NULL;
-	m0->array_meta1_ref = NULL;
 	m0->repository = repo;
 	m0->reload_requested = FALSE;
 
@@ -92,8 +86,6 @@ meta0_backend_clean(struct meta0_backend_s *m0)
 	oio_str_clean (&m0->id);
 	if (m0->array_by_prefix)
 		meta0_utils_array_clean(m0->array_by_prefix);
-	if (m0->array_meta1_ref)
-		meta0_utils_array_meta1ref_clean(m0->array_meta1_ref);
 	g_rw_lock_clear (&m0->rwlock);
 	g_free(m0);
 }
@@ -145,8 +137,6 @@ _load_from_base(struct sqlx_sqlite3_s *sq3, GPtrArray **result)
 		}
 		else if (rc == SQLITE_DONE || rc == SQLITE_OK)
 			break;
-		else if (rc == SQLITE_BUSY)
-			sleep(1);
 		else {
 			err = SQLITE_GERROR(sq3->db, rc);
 			break;
@@ -154,71 +144,19 @@ _load_from_base(struct sqlx_sqlite3_s *sq3, GPtrArray **result)
 	}
 
 	sqlite3_finalize_debug(rc, stmt);
+	meta0_utils_array_finalize(array);
 
 	if (!err) {
 		*result = array;
 		GRID_INFO("Reloaded %u prefixes in %p (%u)",
 				count, array, array->len);
+	} else {
+		meta0_utils_array_clean(array);
 	}
 
 	return err;
 }
 
-static GError*
-_load_meta1ref_from_base(struct sqlx_sqlite3_s *sq3, GPtrArray **result)
-{
-        GError *err = NULL;
-        GPtrArray *array;
-        sqlite3_stmt *stmt;
-        int rc;
-        guint count = 0;
-
-	array = g_ptr_array_new();
-
-        sqlite3_prepare_debug(rc, sq3->db, "SELECT addr,state,prefixes FROM meta1_ref",
-			 -1, &stmt, NULL);
-        if (rc != SQLITE_OK && rc != SQLITE_DONE) {
-		if ( rc == SQLITE_ERROR ) {
-			GRID_DEBUG("Missing table meta1ref in DB");
-			*result = array;
-			return NULL;
-		}
-                return SQLITE_GERROR(sq3->db, rc);
-	}
-
-	for (;;) {
-                rc = sqlite3_step(stmt);
-                if (rc == SQLITE_ROW) {
-			const unsigned char *url,*prefix_nb,*ref;
-			url = sqlite3_column_text(stmt,0);
-			ref = sqlite3_column_text(stmt,1);
-			prefix_nb = sqlite3_column_text(stmt,2);
-
-			GRID_INFO("url %s, ref %s,prefix_nb %s ",url,ref,prefix_nb);
-			g_ptr_array_add(array,meta0_utils_pack_meta1ref((gchar *)url,(gchar *)ref,(gchar *)prefix_nb));
-			count++;
-		}
-		else if (rc == SQLITE_DONE || rc == SQLITE_OK)
-                        break;
-                else if (rc == SQLITE_BUSY)
-                        sleep(1);
-                else {
-                        err = SQLITE_GERROR(sq3->db, rc);
-                        break;
-                }
-
-	}
-	sqlite3_finalize_debug(rc, stmt);
-
-        if (!err) {
-                *result = array;
-                GRID_INFO("Reloaded %u meta1 in %p (%u)",
-                                count, array, array->len);
-        }
-
-        return err;
-}
-
 static GError*
 _load(struct meta0_backend_s *m0)
 {
@@ -236,10 +174,6 @@ _load(struct meta0_backend_s *m0)
 	if (err != NULL)
 		g_prefix_error(&err, "Query error: ");
 
-	err = _load_meta1ref_from_base(sq3, &(m0->array_meta1_ref));
-	if (err != NULL)
-                g_prefix_error(&err, "Query error: ");
-
 	_unlock_and_close(sq3);
 	return err;
 }
@@ -277,6 +211,9 @@ _json_to_meta0_mapping(const char *json_mapping, GPtrArray **result)
 		json_object_put(jbody);
 	json_tokener_free(parser);
 
+	if (urls_by_pfx)
+		meta0_utils_array_finalize(urls_by_pfx);
+
 	if (err)
 		meta0_utils_array_clean(urls_by_pfx);
 	else
@@ -313,15 +250,11 @@ _reload(struct meta0_backend_s *m0, gboolean lazy)
 
 	g_rw_lock_writer_lock(&(m0->rwlock));
 
-	if (!lazy || m0->reload_requested || !m0->array_by_prefix || !m0->array_meta1_ref) {
+	if (!lazy || m0->reload_requested || !m0->array_by_prefix) {
 		if (m0->array_by_prefix) {
 			meta0_utils_array_clean(m0->array_by_prefix);
 			m0->array_by_prefix = NULL;
 		}
-		if (m0->array_meta1_ref) {
-			meta0_utils_array_meta1ref_clean(m0->array_meta1_ref);
-			m0->array_meta1_ref = NULL;
-		}
 
 		err = _load(m0);
 		m0->reload_requested = FALSE;
@@ -343,7 +276,7 @@ _open_and_lock(struct meta0_backend_s *m0, enum m0v2_open_type_e how,
 	EXTRA_ASSERT(handle != NULL);
 
 	/* Now open/lock the base in a way suitable for our op */
-	guint flag = m0_to_sqlx(how);
+	enum sqlx_open_type_e flag = m0_to_sqlx(how);
 	struct sqlx_name_s n = {.base=m0->ns, .type=NAME_SRVTYPE_META0, .ns=m0->ns};
 	err = sqlx_repository_open_and_lock(m0->repository, &n, flag, handle, NULL);
 
@@ -371,7 +304,6 @@ static GError *
 _assign_prefixes(sqlite3 *db, const GPtrArray *new_assign_prefixes,
 		gboolean init)
 {
-	GError *err = NULL;
 	gint rc;
 	guint idx;
 	sqlite3_stmt *stmt = NULL;
@@ -380,15 +312,11 @@ _assign_prefixes(sqlite3 *db, const GPtrArray *new_assign_prefixes,
 		sqlite3_prepare_debug(rc, db, "DELETE FROM meta1", -1, &stmt, NULL);
 		if (rc != SQLITE_OK && rc != SQLITE_DONE)
 			return SQLITE_GERROR(db, rc);
-		while (!err) {
+		for (;;) {
 			rc = sqlite3_step(stmt);
 			if (rc == SQLITE_OK || rc == SQLITE_DONE)
 				break;
-			if (rc == SQLITE_BUSY)
-				sleep(1);
-			else {
-				return SQLITE_GERROR(db,rc);
-			}
+			return SQLITE_GERROR(db,rc);
 		}
 		sqlite3_finalize_debug(rc, stmt);
 	}
@@ -400,7 +328,8 @@ _assign_prefixes(sqlite3 *db, const GPtrArray *new_assign_prefixes,
 		return SQLITE_GERROR(db, rc);
 	}
 
-	for (idx = 0; idx < CID_PREFIX_COUNT; idx++) {
+	GError *err = NULL;
+	for (idx = 0; idx < CID_PREFIX_COUNT && !err; idx++) {
 		gchar **url = new_assign_prefixes->pdata[idx];
 		guint16 index16 = idx;
 
@@ -416,87 +345,14 @@ _assign_prefixes(sqlite3 *db, const GPtrArray *new_assign_prefixes,
 				rc = sqlite3_step(stmt);
 				if (rc == SQLITE_OK || rc == SQLITE_DONE)
 					break;
-				if (rc == SQLITE_BUSY)
-					sleep(1);
 				else {
 					err = SQLITE_GERROR(db, rc);
+					break;
 				}
 			}
 		}
 	}
 	sqlite3_finalize_debug(rc, stmt);
-
-	return err;
-}
-
-static GError *
-_record_meta1ref(sqlite3 *db, const GPtrArray *new_assign_meta1ref)
-{
-	GError *err = NULL;
-	gint rc;
-	guint idx;
-	sqlite3_stmt *stmt = NULL;
-
-	sqlite3_prepare_debug(rc, db, "REPLACE INTO meta1_ref"
-			" (addr,state,prefixes) VALUES (?,?,?)", -1, &stmt, NULL);
-	if (rc != SQLITE_OK && rc != SQLITE_DONE) {
-		if ( rc == SQLITE_ERROR ) {
-			GRID_DEBUG("Missing table meta1ref in DB");
-			return NULL;
-		}
-		return SQLITE_GERROR(db, rc);
-	}
-
-	for (idx=0; idx < new_assign_meta1ref->len; idx++) {
-		gchar *m1ref = new_assign_meta1ref->pdata[idx];
-		gchar *addr, *ref, *nb;
-		if (!meta0_utils_unpack_meta1ref(m1ref,&addr,&ref,&nb))
-			continue;
-		sqlite3_reset(stmt);
-		sqlite3_clear_bindings(stmt);
-		sqlite3_bind_text(stmt, 1, addr, -1, NULL);
-		sqlite3_bind_text(stmt, 2, ref, -1, NULL);
-		sqlite3_bind_text(stmt, 3, nb, -1, NULL);
-
-		while (!err) {
-			rc = sqlite3_step(stmt);
-			if (rc == SQLITE_OK || rc == SQLITE_DONE)
-				break;
-			if (rc == SQLITE_BUSY)
-				sleep(1);
-			else
-				err = SQLITE_GERROR(db, rc);
-		}
-
-		if (addr)
-			g_free(addr);
-		if (ref)
-			g_free(ref);
-		if (nb)
-			g_free(nb);
-	}
-	sqlite3_finalize_debug(rc, stmt);
-	return err;
-}
-
-static GError *
-_delete_meta1_ref(sqlite3 *db, gchar *meta1_ref)
-{
-	GError *err = NULL;
-	gint rc;
-	sqlite3_stmt *stmt = NULL;
-
-	sqlite3_prepare_debug(rc, db, "DELETE FROM meta1_ref where addr=?",
-			-1, &stmt, NULL);
-	if (rc != SQLITE_OK && rc != SQLITE_DONE)
-		return SQLITE_GERROR(db, rc);
-
-	(void) sqlite3_bind_text(stmt,1,meta1_ref, strlen(meta1_ref), NULL);
-	do { rc = sqlite3_step(stmt); } while (rc == SQLITE_ROW );
-	if (rc != SQLITE_OK && rc != SQLITE_DONE)
-		err = SQLITE_GERROR(db, rc);
-
-	sqlite3_finalize_debug(rc, stmt);
 	return err;
 }
 
@@ -591,9 +447,11 @@ meta0_backend_get_all(struct meta0_backend_s *m0, GPtrArray **result)
 		g_prefix_error(&err, "Reload error: ");
 		return err;
 	}
+	if (!m0->array_by_prefix) {
+		return NEWERROR(EINVAL, "Prefixes not ready");
+	}
 
 	g_rw_lock_reader_lock(&(m0->rwlock));
-	EXTRA_ASSERT(m0->array_by_prefix != NULL);
 	*result = meta0_utils_array_dup(m0->array_by_prefix);
 	g_rw_lock_reader_unlock(&(m0->rwlock));
 
@@ -617,115 +475,19 @@ meta0_backend_get_one(struct meta0_backend_s *m0, const guint8 *prefix,
 		return err;
 	}
 
-	g_rw_lock_reader_lock(&(m0->rwlock));
-	EXTRA_ASSERT(m0->array_by_prefix != NULL);
-	*u = meta0_utils_array_get_urlv(m0->array_by_prefix, prefix);
-	g_rw_lock_reader_unlock(&(m0->rwlock));
-
-	return *u ? NULL : NEWERROR(EINVAL, "META0 partially missing");
-}
-
-GError*
-meta0_backend_assign(struct meta0_backend_s *m0,
-		const GPtrArray *new_assign_prefixes,
-		const GPtrArray *new_assign_meta1ref, const gboolean init)
-{
-	GError *err;
-	struct sqlx_sqlite3_s *sq3 = NULL;
-	struct sqlx_repctx_s *repctx = NULL;
-
-	EXTRA_ASSERT(m0 != NULL);
-	EXTRA_ASSERT(new_assign_prefixes != NULL);
-	EXTRA_ASSERT(new_assign_meta1ref != NULL);
-
-	err = _open_and_lock(m0, M0V2_OPENBASE_MASTERONLY, &sq3);
-	if (NULL != err)
-		return err;
-
-	err = sqlx_transaction_begin(sq3, &repctx);
-	if (NULL == err) {
-		err = _assign_prefixes(sq3->db, new_assign_prefixes,init);
-		if (!err) {
-			err = _record_meta1ref(sq3->db, new_assign_meta1ref);
-			if (!err)
-				sqlx_transaction_notify_huge_changes(repctx);
-		}
-		err = sqlx_transaction_end(repctx, err);
+	if (!m0->array_by_prefix) {
+		*u = NULL;
+		return NEWERROR(CODE_UNAVAILABLE,
+				"The current META0 service is not ready yet, "
+				"it has not been initiated.");
+	} else {
+		g_rw_lock_reader_lock(&(m0->rwlock));
+		*u = meta0_utils_array_get_urlv(m0->array_by_prefix, prefix);
+		g_rw_lock_reader_unlock(&(m0->rwlock));
+		if (*u != NULL)
+			return NULL;
+		return NEWERROR(CODE_UNAVAILABLE,
+				"The current META0 service is not ready yet, "
+				"it has been partially initiated.");
 	}
-	_unlock_and_close(sq3);
-	return err;
 }
-
-GError*
-meta0_backend_get_all_meta1_ref(struct meta0_backend_s *m0, GPtrArray **result)
-{
-	GError *err;
-
-	EXTRA_ASSERT(m0 != NULL);
-	EXTRA_ASSERT(result != NULL);
-
-	if (NULL != (err = _reload(m0, TRUE))) {
-		g_prefix_error(&err, "Reload error: ");
-		return err;
-	}
-
-	g_rw_lock_reader_lock(&(m0->rwlock));
-	EXTRA_ASSERT(m0->array_meta1_ref != NULL);
-	*result = meta0_utils_array_meta1ref_dup(m0->array_meta1_ref);
-	g_rw_lock_reader_unlock(&(m0->rwlock));
-
-	return NULL;
-}
-
-GError*
-meta0_backend_destroy_meta1_ref(struct meta0_backend_s *m0, gchar *meta1)
-{
-	GError *err = NULL;
-	struct sqlx_sqlite3_s *sq3 = NULL;
-	struct sqlx_repctx_s *repctx = NULL;
-	GPtrArray *result;
-	gchar *v, *addr, *ref, *nb;
-	guint i, max, cmpaddr, cmpstate;
-
-	EXTRA_ASSERT(m0 != NULL);
-	EXTRA_ASSERT(meta1 != NULL);
-
-	/* check if meta1 is disable */
-	if (NULL != (err = _reload(m0, TRUE))) {
-		g_prefix_error(&err, "Reload error: ");
-		return err;
-	}
-
-	g_rw_lock_reader_lock(&(m0->rwlock));
-	EXTRA_ASSERT(m0->array_meta1_ref != NULL);
-	result = meta0_utils_array_meta1ref_dup(m0->array_meta1_ref);
-	g_rw_lock_reader_unlock(&(m0->rwlock));
-
-	for (i=0,max=result->len; i<max ;i++) {
-		if (!(v = result->pdata[i]))
-			continue;
-		meta0_utils_unpack_meta1ref(v,&addr,&ref,&nb);
-		cmpaddr = g_ascii_strcasecmp(addr,meta1);
-		cmpstate = g_ascii_strcasecmp(ref,"0");
-		g_free(addr);
-		g_free(ref);
-		g_free(nb);
-		if ( cmpaddr == 0) {
-			if (cmpstate != 0)
-				return NEWERROR(EINVAL, "meta1 always available to prefix allocation");
-			err = _open_and_lock(m0, M0V2_OPENBASE_MASTERONLY, &sq3);
-			if (NULL != err)
-				return err;
-
-			err = sqlx_transaction_begin(sq3, &repctx);
-			if (NULL == err) {
-				err = _delete_meta1_ref(sq3->db, meta1);
-				err = sqlx_transaction_end(repctx, err);
-			}
-			_unlock_and_close(sq3);
-			return err;
-		}
-	}
-	return NEWERROR(EINVAL, "UNKNOWN meta1");
-}
-
