@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta0v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -17,10 +17,6 @@ You should have received a copy of the GNU Affero General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <stddef.h>
-#include <string.h>
-#include <errno.h>
-
 #include <metautils/lib/metautils.h>
 #include <cluster/lib/gridcluster.h>
 
@@ -28,38 +24,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "meta0_utils.h"
 
 static addr_info_t addr = {{0}};
-static gchar *namespace = NULL;
+static gchar namespace[LIMIT_LENGTH_NSNAME] = {0};
 static gboolean flag_list = FALSE;
 static gboolean flag_get = FALSE;
 static gboolean flag_reload = FALSE;
 static gboolean flag_reset = FALSE;
-static gboolean flag_getmeta1info = FALSE;
 static gboolean flag_nocheck = FALSE;
-static gboolean flag_destroy_meta1ref = FALSE;
-static gboolean flag_destroy_zk_node = FALSE;
 static guint8 prefix[2] = {0,0};
-static gchar **urls = NULL;
-
-static gboolean
-url_check(const gchar *u)
-{
-	addr_info_t a = {{0}};
-	return grid_string_to_addrinfo(u, &a);
-}
-
-static gboolean
-urlv_check(gchar **urlv)
-{
-	if (!urlv)
-		return FALSE;
-	for (gchar **u = urlv; *u ;u++) {
-		if (!url_check(*u)) {
-			GRID_WARN("Bad address [%s]", *u);
-			return FALSE;
-		}
-	}
-	return TRUE;
-}
 
 static gboolean
 _is_usable_meta0(addr_info_t *m0addr, GSList *exclude)
@@ -98,7 +69,7 @@ meta0_utils_getMeta0addr(gchar *ns, GSList **m0_lst, GSList *exclude)
 static addr_info_t *
 _getMeta0addr(GSList **m0_lst, GSList *exclude)
 {
-	if (namespace)
+	if (namespace[0])
 		return  meta0_utils_getMeta0addr(namespace, m0_lst, exclude);
 	if (!exclude)
 		return &addr;
@@ -210,7 +181,7 @@ meta0_init_list(void)
 		err = meta0_remote_get_meta1_all(url, &list);
 		if (err != NULL) {
 			if (CODE_IS_NETWORK_ERROR(err->code)) {
-				if (DEBUG_ENABLED()) {
+				if (GRID_DEBUG_ENABLED()) {
 					GRID_DEBUG("Failed to reach meta0 [%s] : error (%d) : %s",
 							url, err->code, err->message);
 				}
@@ -268,117 +239,6 @@ exit:
 	g_slist_free_full(m0_lst, (GDestroyNotify)service_info_clean);
 }
 
-static void
-meta0_init_get_meta1_info(void)
-{
-	GError *err = NULL;
-	GSList *exclude = NULL;
-	GSList *m0_lst = NULL;
-	addr_info_t *m0addr;
-
-	GRID_INFO("GET all META1 information");
-
-	m0addr = _getMeta0addr(&m0_lst,exclude);
-	while (m0addr) {
-		gchar url[STRLEN_ADDRINFO];
-		grid_addrinfo_to_string(m0addr, url , sizeof(url));
-		gchar **result = NULL;
-		err = meta0_remote_get_meta1_info(url, &result);
-
-		if (err != NULL) {
-			GRID_WARN("META0 request error (%d) : %s", err->code, err->message);
-			if (CODE_IS_NETWORK_ERROR(err->code)) {
-				exclude = g_slist_prepend(exclude, m0addr);
-				m0addr = _getMeta0addr(&m0_lst, exclude);
-			} else {
-				m0addr = NULL;
-			}
-			g_clear_error(&err);
-		} else {
-			if (result != NULL) {
-				for (gchar **u = result; *u ;u++)
-					g_print("%s\n",*u);
-				g_free(result);
-			} else {
-				GRID_INFO("No meta1 referenced");
-			}
-			goto exit;
-		}
-	}
-
-	grid_main_set_status(1);
-exit:
-	g_slist_free_full(m0_lst, (GDestroyNotify)service_info_clean);
-}
-
-static void
-meta0_init_destroy_meta1ref(void)
-{
-	GError *err = NULL;
-	GSList *exclude = NULL;
-	GSList *m0_lst = NULL;
-	addr_info_t *m0addr;
-
-	GRID_INFO("Destroy META1 reference");
-
-	m0addr = _getMeta0addr(&m0_lst,exclude);
-	while (m0addr) {
-		gchar url[STRLEN_ADDRINFO];
-		grid_addrinfo_to_string(m0addr, url , sizeof(url));
-		err = meta0_remote_destroy_meta1ref(url, *urls);
-		if (err != NULL) {
-			GRID_WARN("META0 request error (%d) : %s", err->code, err->message);
-			if (CODE_IS_NETWORK_ERROR(err->code)) {
-				exclude = g_slist_prepend(exclude,m0addr);
-				m0addr = _getMeta0addr(&m0_lst,exclude);
-			} else {
-				m0addr = NULL;
-			}
-			g_clear_error(&err);
-		} else {
-			GRID_INFO("META1 reference removed!");
-			goto exit;
-		}
-	}
-
-	grid_main_set_status(1);
-exit:
-	g_slist_free_full(m0_lst, (GDestroyNotify)service_info_clean);
-}
-
-static void
-meta0_init_destroy_zk_node(void)
-{
-	GError *err = NULL;
-	GSList *exclude = NULL;
-	GSList *m0_lst = NULL;
-	addr_info_t *m0addr;
-
-	GRID_INFO("REMOVE META0 Zookeeper node");
-
-	m0addr = _getMeta0addr(&m0_lst,exclude);
-	while (m0addr) {
-		gchar url[STRLEN_ADDRINFO];
-		grid_addrinfo_to_string(m0addr, url , sizeof(url));
-		err = meta0_remote_destroy_meta0zknode(url, *urls);
-		if (err != NULL) {
-			GRID_WARN("META0 request error (%d) : %s", err->code, err->message);
-			if (CODE_IS_NETWORK_ERROR(err->code)) {
-				exclude = g_slist_prepend(exclude,m0addr);
-				m0addr = _getMeta0addr(&m0_lst,exclude);
-			} else {
-				m0addr = NULL;
-			}
-			g_clear_error(&err);
-		} else {
-			GRID_INFO("META0 Zookeeper node removed!");
-			return;
-		}
-	}
-
-	grid_main_set_status(1);
-}
-
 static void
 meta0_action(void)
 {
@@ -394,15 +254,6 @@ meta0_action(void)
 	else if (flag_reset) {
 		meta0_init_reset();
 	}
-	else if (flag_getmeta1info) {
-		meta0_init_get_meta1_info();
-	}
-	else if (flag_destroy_meta1ref) {
-		meta0_init_destroy_meta1ref();
-	}
-	else if (flag_destroy_zk_node) {
-		meta0_init_destroy_zk_node();
-	}
 	else {
 		GRID_INFO("No action specified");
 	}
@@ -420,7 +271,7 @@ meta0_get_options(void)
 	static struct grid_main_option_s meta0_options[] = {
 		{"NoCheck", OT_BOOL, {.b=&flag_nocheck},
 			"Disable checks to relaunch assign"},
-		{NULL, 0, {.i=0}, NULL}
+		{NULL, OT_INT, {.i=NULL}, NULL}
 	};
 	return meta0_options;
 }
@@ -446,9 +297,8 @@ meta0_configure(int argc, char **argv)
 		return FALSE;
 	}
 
-	if (!grid_string_to_addrinfo(argv[0], &addr)) {
-		namespace = strdup(argv[0]);
-	}
+	if (!grid_string_to_addrinfo(argv[0], &addr))
+		g_strlcpy(namespace, argv[0], sizeof(namespace));
 
 	command = argv[1];
 	if (!g_ascii_strcasecmp(command, "get")) {
@@ -481,40 +331,6 @@ meta0_configure(int argc, char **argv)
 		flag_list = TRUE;
 		return TRUE;
 	}
-	if (!g_ascii_strcasecmp(command, "get_meta1_info")) {
-		if (argc > 2)
-			GRID_DEBUG("Exceeding list arguments ignored.");
-		flag_getmeta1info = TRUE;
-		return TRUE;
-	}
-
-	if (!g_ascii_strcasecmp(command, "destroy_meta1ref")) {
-		if (argc != 3) {
-			GRID_WARN("Missing META1 address");
-			return FALSE;
-		}
-		if (!urlv_check(argv+2)) {
-			GRID_WARN("Invalid META1 address");
-			return FALSE;
-		}
-		urls = g_strdupv(argv+2);
-		flag_destroy_meta1ref = TRUE;
-		return TRUE;
-	}
-
-	if (!g_ascii_strcasecmp(command, "destroy_zknode")) {
-		if (argc != 3) {
-			GRID_WARN("Missing META1 address");
-			return FALSE;
-		}
-		if (!urlv_check(argv+2)) {
-			GRID_WARN("Invalid META1 address");
-			return FALSE;
-		}
-		urls = g_strdupv(argv+2);
-		flag_destroy_zk_node = TRUE;
-		return TRUE;
-	}
 
 	GRID_WARN("Invalid command, see usage.");
 	return FALSE;
