@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta0v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -17,12 +17,7 @@ You should have received a copy of the GNU Affero General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <stdlib.h>
-#include <string.h>
-#include <errno.h>
-
 #include <metautils/lib/metautils.h>
-#include <metautils/lib/metacomm.h>
 #include <server/transport_gridd.h>
 
 #include "meta0_backend.h"
@@ -32,7 +27,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 struct meta0_disp_s {
 	struct meta0_backend_s *m0;
-	struct zk_manager_s *m0zkmanager;
 	gchar *ns_name;
 	GByteArray *encoded;
 	GMutex lock;
@@ -110,29 +104,6 @@ static GByteArray* _get_encoded(struct meta0_disp_s *m0disp) {
 	return encoded;
 }
 
-static GByteArray* _encode_meta1ref(struct meta0_disp_s *m0disp) {
-	GError *err = NULL;
-	GByteArray *encoded = NULL;
-
-	g_mutex_lock(&m0disp->lock);
-	GPtrArray *array = NULL;
-	err = meta0_backend_get_all_meta1_ref(m0disp->m0, &array);
-	if (!err) {
-		if (array->len <= 0 || !array->pdata[array->len - 1])
-			g_ptr_array_add(array, NULL);
-		encoded = STRV_encode_gba((gchar**)array->pdata);
-	}
-	g_mutex_unlock(&m0disp->lock);
-
-	meta0_utils_array_meta1ref_clean(array);
-
-	if (err) {
-		GRID_WARN("META0 reload failed : (%d) %s", err->code, err->message);
-		g_clear_error(&err);
-	}
-	return encoded;
-}
-
 static void _reload(struct meta0_disp_s *m0disp) {
 	GError *err = NULL;
 	GPtrArray *array = NULL;
@@ -169,8 +140,7 @@ meta0_dispatch_v1_GETONE(struct gridd_reply_ctx_s *reply,
 		gchar **urlv = NULL;
 		err = meta0_backend_get_one(m0disp->m0, prefix, &urlv);
 		if (NULL != err) {
-			g_prefix_error(&err, "Backend error: ");
-			reply->send_error(CODE_INTERNAL_ERROR, err);
+			reply->send_error(0, err);
 		} else {
 			reply->add_body(_encode_meta0_tree(urlv_to_tree(prefix, urlv)));
 			reply->send_reply(CODE_FINAL_OK, "OK");
@@ -195,7 +165,6 @@ meta0_dispatch_v1_RELOAD(struct gridd_reply_ctx_s *reply,
 	GError *err;
 
 	if (NULL != (err = meta0_backend_reload(m0disp->m0))) {
-		g_prefix_error(&err, "Backend error: ");
 		reply->send_error(0, err);
 		return TRUE;
 	}
@@ -214,62 +183,6 @@ meta0_dispatch_v1_RESET(struct gridd_reply_ctx_s *reply,
 
 	GError *err = meta0_backend_reset(m0disp->m0, flag_local);
 	if (NULL != err) {
-		g_prefix_error(&err, "Backend error: ");
-		reply->send_error(0, err);
-		return TRUE;
-	}
-
-	reply->send_reply(CODE_FINAL_OK, "OK");
-	return TRUE;
-}
-
-static gboolean
-meta0_dispatch_v2_META1_INFO(struct gridd_reply_ctx_s *reply,
-		struct meta0_disp_s *m0disp, gpointer ignored UNUSED)
-{
-	reply->add_body(_encode_meta1ref(m0disp));
-	reply->send_reply(CODE_FINAL_OK, "OK");
-	return TRUE;
-}
-
-static gboolean
-meta0_dispatch_v2_DESTROY_META1REF(struct gridd_reply_ctx_s *reply,
-		struct meta0_disp_s *m0disp, gpointer ignored UNUSED)
-{
-	gchar meta1url[STRLEN_ADDRINFO];
-	GError *err = metautils_message_extract_string(reply->request,
-			NAME_MSGKEY_METAURL, meta1url, sizeof(meta1url));
-	if (err != NULL) {
-		reply->send_error(CODE_BAD_REQUEST, err);
-		return TRUE;
-	}
-
-	err = meta0_backend_destroy_meta1_ref(m0disp->m0, meta1url);
-	if (NULL != err) {
-		g_prefix_error(&err, "destroy meta1_ref error:");
-		reply->send_error(0, err);
-		return TRUE;
-	}
-
-	_reload(m0disp);
-	reply->send_reply(CODE_FINAL_OK, "OK");
-	return TRUE;
-}
-
-static gboolean
-meta0_dispatch_v2_DESTROY_ZKNODE(struct gridd_reply_ctx_s *reply,
-		struct meta0_disp_s *m0disp, gpointer ignored UNUSED)
-{
-	gchar meta0url[STRLEN_ADDRINFO];
-	GError *err = metautils_message_extract_string(reply->request,
-			NAME_MSGKEY_METAURL, meta0url, sizeof(meta0url));
-	if (err != NULL) {
-		reply->send_error(CODE_BAD_REQUEST, err);
-		return TRUE;
-	}
-
-	if ( NULL != (err = delete_zk_node(m0disp->m0zkmanager,NULL,meta0url))) {
-		g_prefix_error(&err, "Failed to destroy meta0 in zookeeper :");
 		reply->send_error(0, err);
 		return TRUE;
 	}
@@ -316,9 +229,6 @@ const struct gridd_request_descr_s *meta0_gridd_get_requests(void) {
 		{NAME_MSGNAME_M0_RELOAD,              (hook) meta0_dispatch_v1_RELOAD,  NULL},
 		{NAME_MSGNAME_M0_RESET,               (hook) meta0_dispatch_v1_RESET,   NULL},
 		{NAME_MSGNAME_M0_FORCE,               (hook) meta0_dispatch_v1_FORCE,   NULL},
-		{NAME_MSGNAME_M0_GET_META1_INFO,      (hook) meta0_dispatch_v2_META1_INFO, NULL},
-		{NAME_MSGNAME_M0_DESTROY_META1REF,    (hook) meta0_dispatch_v2_DESTROY_META1REF, NULL},
-		{NAME_MSGNAME_M0_DESTROY_META0ZKNODE, (hook) meta0_dispatch_v2_DESTROY_ZKNODE, NULL},
 		{NULL, NULL, NULL}
 	};
 
@@ -331,11 +241,10 @@ void meta0_gridd_requested_reload(struct meta0_disp_s *m0disp) {
 }
 
 struct meta0_disp_s* meta0_gridd_get_dispatcher(struct meta0_backend_s *m0,
-		struct zk_manager_s *m0zkmanager, gchar* ns_name) {
+		const char* ns_name) {
 	struct meta0_disp_s *result = g_malloc0(sizeof(*result));
 	result->ns_name = g_strdup(ns_name);
 	result->m0 = m0;
-	result->m0zkmanager = m0zkmanager;
 	g_mutex_init(&result->lock);
 
 	meta0_gridd_requested_reload(result);
