@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta0v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -17,14 +17,8 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library.
 */
 
-#include <errno.h>
-#include <netdb.h>
-#include <string.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <unistd.h>
-
 #include <metautils/lib/metautils.h>
+#include <metautils/lib/common_variables.h>
 
 #include "meta0_remote.h"
 #include "internals.h"
@@ -33,7 +27,7 @@ static GError *
 _m0_remote_no_return (const char *m0, GByteArray *req)
 {
 	EXTRA_ASSERT (m0 != NULL);
-	return gridd_client_exec (m0, M0V2_CLIENT_TIMEOUT, req);
+	return gridd_client_exec (m0, oio_m0_client_timeout_common, req);
 }
 
 static GError *
@@ -42,7 +36,7 @@ _m0_remote_m0info (const char *m0, GByteArray *req, GSList **out)
 	EXTRA_ASSERT (m0 != NULL);
 	EXTRA_ASSERT (out != NULL);
 	GSList *result = NULL;
-	GError *e = gridd_client_exec_and_decode (m0, M0V2_CLIENT_TIMEOUT,
+	GError *e = gridd_client_exec_and_decode (m0, oio_m0_client_timeout_common,
 			req, &result, meta0_info_unmarshall);
 	if (!e) {
 		*out = result;
@@ -98,41 +92,3 @@ meta0_remote_force(const char *m0, const guint8 *mapping, gsize mapping_len)
 	return _m0_remote_no_return(m0, message_marshall_gba_and_clean(request));
 }
 
-GError*
-meta0_remote_destroy_meta1ref(const char *m0, const char *urls)
-{
-	if (!urls || !*urls)
-		return NEWERROR(CODE_BAD_REQUEST, "Too few URL's");
-	MESSAGE request = metautils_message_create_named(NAME_MSGNAME_M0_DESTROY_META1REF);
-	metautils_message_add_field_str (request, NAME_MSGKEY_METAURL, urls);
-	return _m0_remote_no_return (m0, message_marshall_gba_and_clean(request));
-}
-
-GError*
-meta0_remote_destroy_meta0zknode(const char *m0, const char *urls)
-{
-	if (!urls || !*urls)
-		return NEWERROR(CODE_BAD_REQUEST, "Too few URL's");
-	MESSAGE request = metautils_message_create_named(NAME_MSGNAME_M0_DESTROY_META0ZKNODE);
-	metautils_message_add_field_str (request, NAME_MSGKEY_METAURL, urls);
-	return _m0_remote_no_return (m0, message_marshall_gba_and_clean(request));
-}
-
-GError*
-meta0_remote_get_meta1_info(const char *m0, gchar ***out)
-{
-	MESSAGE request = metautils_message_create_named(NAME_MSGNAME_M0_GET_META1_INFO);
-	GByteArray *packed = message_marshall_gba_and_clean(request);
-	GByteArray *tmp = NULL;
-	GError *err = gridd_client_exec_and_concat(m0, M0V2_CLIENT_TIMEOUT, packed, &tmp);
-	g_byte_array_free(packed, TRUE);
-
-	if (err) {
-		if (tmp)
-			g_byte_array_free(tmp, TRUE);
-		return err;
-	}
-	err = STRV_decode_buffer(tmp->data, tmp->len, out);
-	g_byte_array_free(tmp, TRUE);
-	return err;
-}
