@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta0v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -27,8 +27,5 @@ GError * meta0_remote_get_meta1_one(const char *m0, const guint8 *prefix, GSList
 GError * meta0_remote_cache_refresh(const char *m0);
 GError * meta0_remote_cache_reset(const char *m0, gboolean local);
 GError * meta0_remote_force(const char *m0, const guint8 *mapping, gsize mapping_len);
-GError * meta0_remote_get_meta1_info(const char *m0, gchar ***out);
-GError * meta0_remote_destroy_meta1ref(const char *m0, const char *urls);
-GError * meta0_remote_destroy_meta0zknode(const char *m0, const char *urls);
 
 #endif /*OIO_SDS__meta0v2__meta0_remote_h*/
