@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta0v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -26,12 +26,8 @@ License along with this library.
 
 #define CID_PREFIX_COUNT 65536
 
-GTree* meta0_utils_array_to_tree(const GPtrArray *array);
-
 GSList* meta0_utils_tree_to_list(GTree *tree);
 
-GTree* meta0_utils_list_to_tree(const GSList *list);
-
 GPtrArray* meta0_utils_list_to_array(GSList *list);
 
 GSList* meta0_utils_array_to_list(GPtrArray *array);
@@ -40,27 +36,17 @@ gchar ** meta0_utils_array_get_urlv(GPtrArray *array, const guint8 *bytes);
 
 GPtrArray* meta0_utils_array_dup(const GPtrArray *in);
 
-void meta0_utils_array_clean(GPtrArray *array);
-
 void meta0_utils_list_clean(GSList *list);
 
-void meta0_utils_array_add(GPtrArray *gpa, const guint8 *b, const gchar *s);
-
 GPtrArray * meta0_utils_array_create(void);
+void meta0_utils_array_add(GPtrArray *gpa, const guint8 *b, const gchar *s);
+void meta0_utils_array_finalize(GPtrArray *array);
+void meta0_utils_array_clean(GPtrArray *array);
 
 void meta0_utils_tree_add_url(GTree *tree, const guint8 *b, const gchar *url);
 
 GTree* meta0_utils_tree_create(void);
 
-void meta0_utils_array_meta1ref_clean(GPtrArray *array);
-
-GPtrArray* meta0_utils_array_meta1ref_dup(GPtrArray *in);
-
-gchar * meta0_utils_pack_meta1ref(gchar *addr, gchar *ref, gchar *nb);
-
-gboolean meta0_utils_unpack_meta1ref(const gchar *s_m1ref, gchar **addr,
-		gchar **ref, gchar **nb);
-
 /* @return TRUE to continue */
 typedef gboolean (*meta0_on_prefix)(gpointer u,
 		const guint8 *group, const guint8 *prefix);
