@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta1v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -135,6 +135,19 @@ GError* meta1_backend_services_set(struct meta1_backend_s *m1,
 		struct oio_url_s *url, const gchar *packedurl,
 		gboolean autocreate, gboolean force);
 
+/* Find replacements for some services linked to a reference.
+ *
+ * @param url URLs of the reference whose services must be replaced
+ * @param kept Packed list of service URLs that must be kept
+ * @param replaced Packed list of service URLs that must be replaced
+ * @param dryrun If true, find replacement services but do not save them
+ *
+ * @note
+ * The elements of `kept` and `replaced` must have the same sequence number.
+ *
+ * @note
+ * The union of `kept` and `replaced` must match exactly the list of services
+ * already known by the meta1 for the same sequence number. */
 GError* meta1_backend_services_relink(struct meta1_backend_s *m1,
 		struct oio_url_s *url, const char *kept, const char *replaced,
 		gboolean dryrun, gchar ***out);
@@ -153,8 +166,6 @@ GError* meta1_backend_get_container_properties(struct meta1_backend_s *m1,
 GError *meta1_backend_services_all(struct meta1_backend_s *m1,
 		struct oio_url_s *url, gchar ***result);
 
-typedef void (*m1b_ref_hook) (gpointer p, const gchar *ns, const gchar *ref);
-
 /* Send a notification (if enabled) with the services linked to a container. */
 GError *meta1_backend_notify_services(struct meta1_backend_s *m1,
 		struct oio_url_s *url);
