@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta1v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -33,8 +33,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "./meta1_backend.h"
 #include "./meta1_backend_internals.h"
 
-int meta1_backend_log_level = 0;
-
 static gboolean
 m1b_check_ns_url (struct meta1_backend_s *m1, struct oio_url_s *url)
 {
@@ -64,22 +62,23 @@ __exec_cid(sqlite3 *handle, const gchar *sql, const container_id_t cid)
 		sqlite3_finalize_debug(rc, stmt);
 	}
 }
+
 static int
 m1_to_sqlx(enum m1v2_open_type_e t)
 {
 	switch (t & 0x03) {
 		case M1V2_OPENBASE_LOCAL:
-			return SQLX_OPEN_LOCAL;
+			return SQLX_OPEN_CREATE|SQLX_OPEN_LOCAL;
 		case M1V2_OPENBASE_MASTERONLY:
-			return SQLX_OPEN_MASTERONLY;
+			return SQLX_OPEN_CREATE|SQLX_OPEN_MASTERONLY;
 		case M1V2_OPENBASE_SLAVEONLY:
-			return SQLX_OPEN_SLAVEONLY;
+			return SQLX_OPEN_CREATE|SQLX_OPEN_SLAVEONLY;
 		case M1V2_OPENBASE_MASTERSLAVE:
-			return SQLX_OPEN_MASTERSLAVE;
+			return SQLX_OPEN_CREATE|SQLX_OPEN_MASTERSLAVE;
 	}
 
 	g_assert_not_reached();
-	return SQLX_OPEN_LOCAL;
+	return SQLX_OPEN_CREATE|SQLX_OPEN_LOCAL;
 }
 
 GError *
@@ -100,7 +99,7 @@ _open_and_lock(struct meta1_backend_s *m1, struct oio_url_s *url,
 	EXTRA_ASSERT(url != NULL);
 	EXTRA_ASSERT(handle != NULL);
 
-	GRID_TRACE2("%s(%p,%p,%d,%p)", __FUNCTION__, (void*)m1,
+	GRID_TRACE2("%s(%p,%s,%d,%p)", __FUNCTION__, (void*)m1,
 			oio_url_get (url, OIOURL_HEXID), how, (void*)handle);
 
 	if (!oio_url_has (url, OIOURL_HEXID))
