@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta1v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -25,14 +25,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 # include <sqlx/sqlx_service.h>
 # include <events/oio_events_queue.h>
 
-#ifndef OIO_META1_DIGITS_KEY
-#define OIO_META1_DIGITS_KEY "meta1_digits"
-#endif
-
-#ifndef OIO_META1_DIGITS_DEFAULT
-#define OIO_META1_DIGITS_DEFAULT 4
-#endif
-
 #define M1_SQLITE_GERROR(db,RC) NEWERROR((RC), "(%s) %s", \
 		sqlite_strerror(RC), (db)?sqlite3_errmsg(db):"unkown error")
 
