@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta1v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -66,7 +66,7 @@ urlv_get_max_seq(struct meta1_service_url_s **uv)
 {
 	gint seq = G_MININT;
 	if (uv) {
-		for (; *uv ;++uv) {
+		for (; *uv; ++uv) {
 			if (seq < (*uv)->seq)
 				seq = (*uv)->seq;
 		}
@@ -119,10 +119,10 @@ expand_urlv(struct meta1_service_url_s **uv)
 
 	tmp = g_ptr_array_new();
 	if (uv) {
-		for (; *uv ;++uv) {
+		for (; *uv; ++uv) {
 			struct meta1_service_url_s **p, **utmp;
 			if (NULL != (utmp = expand_url(*uv))) {
-				for (p=utmp; *p ;++p)
+				for (p=utmp; *p; ++p)
 					g_ptr_array_add(tmp, *p);
 				g_free(utmp);
 			}
@@ -134,7 +134,7 @@ expand_urlv(struct meta1_service_url_s **uv)
 }
 
 static struct meta1_service_url_s*
-_ids_to_url(char **ids)
+_ids_to_url(gchar **ids)
 {
 	struct meta1_service_url_s *m1u = g_malloc0 (sizeof(*m1u));
 	meta1_urlv_shift_addr(ids);
@@ -240,7 +240,7 @@ __del_container_services(struct sqlx_sqlite3_s *sq3, struct oio_url_s *url,
 	if (!urlv || !*urlv)
 		err = __del_container_all_services(sq3, url, srvtype);
 	else {
-		for (; !err && *urlv ;urlv++,line++) {
+		for (; !err && *urlv; urlv++,line++) {
 			gchar *end = NULL;
 
 			errno = 0;
@@ -513,9 +513,10 @@ __poll_services(struct meta1_backend_s *m1, guint replicas,
 		(void)loc;
 		g_ptr_array_add(ids, g_strdup(id));
 	}
-	if (!oio_lb__patch_with_pool(m1->lb, ct->baretype, avoid, known, _on_id)) {
-		*err = NEWERROR(CODE_POLICY_NOT_SATISFIABLE,
-				"found only %u services matching the criteria",
+	*err = oio_lb__patch_with_pool(
+			m1->lb, ct->baretype, avoid, known, _on_id, NULL);
+	if (*err) {
+		g_prefix_error(err, "found only %u services matching the criteria: ",
 				ids->len);
 	}
 
@@ -721,14 +722,21 @@ __parse_and_expand (const char *packed)
 static gint
 _sorter (struct meta1_service_url_s **p0, struct meta1_service_url_s **p1)
 {
-	gint64 s0 = (*p0)->seq, s1 = (*p1)->seq;
+	const gint64 s0 = (*p0)->seq, s1 = (*p1)->seq;
 	if (s0 == s1)
 		return strcmp ((*p0)->host, (*p1)->host);
-	int one = s0 > s1;
+	const int one = s0 > s1;
 	return one ? one : -(s0 < s1);
 }
 
-/* check if two arrays of urls contains the same items. */
+static gboolean
+_idem (struct meta1_service_url_s *u0, struct meta1_service_url_s *u1)
+{
+	return u0->seq == u1->seq && !strcmp(u0->srvtype, u1->srvtype);
+}
+
+/* Check if `all` array contains exactly the same items as the union
+ * of `kept` and `replaced` arrays (order does not matter). */
 static gboolean
 __match_urlv (struct meta1_service_url_s **all, struct meta1_service_url_s **kept,
 		struct meta1_service_url_s **replaced)
@@ -739,8 +747,8 @@ __match_urlv (struct meta1_service_url_s **all, struct meta1_service_url_s **kep
 	GPtrArray *gpa_inplace = g_ptr_array_new ();
 	GPtrArray *gpa_told = g_ptr_array_new ();
 
-	/* build two array of URL we can safely sort */
-	for (; *all ;++all) {
+	/* build two arrays of URL we can safely sort */
+	for (; *all; ++all) {
 		if ((*all)->seq == ref->seq)
 			g_ptr_array_add (gpa_inplace, *all);
 	}
@@ -756,7 +764,7 @@ __match_urlv (struct meta1_service_url_s **all, struct meta1_service_url_s **kep
 	g_ptr_array_sort (gpa_told, (GCompareFunc)_sorter);
 
 	/* identical sorted arrays have equal items at each position */
-	for (guint i=0; i<gpa_told->len ;++i) {
+	for (guint i=0; i<gpa_told->len; ++i) {
 		if (0 != strcmp(M1U(gpa_told->pdata[i])->host, M1U(gpa_inplace->pdata[i])->host))
 			goto out;
 	}
@@ -771,17 +779,22 @@ static GError *
 __relink_container_services(struct m1v2_relink_input_s *in, gchar ***out)
 {
 	GError *err = NULL;
-	GPtrArray *ids = NULL;
 	struct meta1_service_url_s *packed = NULL;
 
 	struct meta1_service_url_s *ref = (in->kept && in->kept[0])
 			? in->kept[0] : in->replaced[0];
 
+	GPtrArray *ids = g_ptr_array_new_with_free_func(g_free);
+
 	/* check the services provided are those in place */
 	struct meta1_service_url_s **inplace = NULL;
 	err = __get_container_all_services (in->sq3, in->url, ref->srvtype, &inplace);
-	if (!err && !__match_urlv(inplace, in->kept, in->replaced))
-		err = NEWERROR(CODE_USER_INUSE, "services changed");
+	if (!err) {
+		struct meta1_service_url_s **newset = expand_url (*inplace);
+		if (!__match_urlv(newset, in->kept, in->replaced))
+			err = NEWERROR(CODE_USER_INUSE, "services changed");
+		meta1_service_url_cleanv (newset);
+	}
 	meta1_service_url_cleanv (inplace);
 	inplace = NULL;
 
@@ -802,16 +815,14 @@ __relink_container_services(struct m1v2_relink_input_s *in, gchar ***out)
 		if (g_strv_length((char**)known) >= max_svc) {
 			err = NEWERROR(CODE_POLICY_NOT_SATISFIABLE, "Too many services kept");
 		} else {
-			ids = g_ptr_array_new_with_free_func(g_free);
-			void _on_id(oio_location_t loc, const char *id)
-			{
-				(void)loc;
+			void _on_id(oio_location_t loc UNUSED, const char *id) {
 				g_ptr_array_add(ids, g_strdup(id));
 			}
-			if (!oio_lb__patch_with_pool(in->m1->lb, in->ct->baretype,
-					avoids, known, _on_id)) {
-				err = NEWERROR(CODE_POLICY_NOT_SATISFIABLE,
-						"found only %u services matching the criteria",
+			err = oio_lb__patch_with_pool(in->m1->lb, in->ct->baretype,
+					avoids, known, _on_id, NULL);
+			if (err) {
+				g_prefix_error(&err,
+						"found only %u services matching the criteria: ",
 						ids->len);
 			}
 			g_ptr_array_add(ids, NULL);
@@ -945,6 +956,8 @@ meta1_backend_services_link (struct meta1_backend_s *m1,
 		gboolean dryrun, gboolean autocreate,
 		gchar ***result)
 {
+	if (!result) return SYSERR("BUG: invalid output array");
+
 	GError *err = __check_backend_events (m1);
 	if (err) return err;
 
@@ -977,6 +990,8 @@ meta1_backend_services_poll(struct meta1_backend_s *m1,
 		gboolean dryrun, gboolean autocreate,
 		gchar ***result)
 {
+	if (!result) return SYSERR("BUG: invalid output array");
+
 	GError *err = __check_backend_events (m1);
 	if (err) return err;
 
@@ -1010,9 +1025,35 @@ GError *
 meta1_backend_services_list(struct meta1_backend_s *m1,
 		struct oio_url_s *url, const char *srvtype, gchar ***result)
 {
+	if (!result) return SYSERR("BUG: invalid output array");
+
+	gboolean retry = TRUE;
 	struct sqlx_sqlite3_s *sq3 = NULL;
-	GError *err = _open_and_lock(m1, url, M1V2_OPENBASE_MASTERSLAVE, &sq3);
-	if (err) return err;
+	GError *err = NULL;
+
+label_retry:
+	err = _open_and_lock(m1, url, M1V2_OPENBASE_MASTERSLAVE, &sq3);
+	if (err) {
+		if (retry && err->code == CODE_RANGE_NOTFOUND) {
+			retry = FALSE;
+			// Try to reload the prefixes
+			gboolean meta0_ok = FALSE;
+			GArray *updated_prefixes = NULL;
+			GError *err_load = meta1_prefixes_load(m1->prefixes,
+					oio_url_get(url, OIOURL_NS),
+					sqlx_repository_get_local_addr(m1->repo),
+					&updated_prefixes, &meta0_ok, m1->nb_digits);
+			if (err_load || !meta0_ok) {
+				if (err_load) g_error_free(err_load);
+				return err;
+			}
+			g_error_free(err);
+			if (updated_prefixes)
+				g_array_free(updated_prefixes, TRUE);
+			goto label_retry;
+		}
+		return err;
+	}
 
 	struct sqlx_repctx_s *repctx = NULL;
 	if (!(err = sqlx_transaction_begin(sq3, &repctx))) {
@@ -1078,30 +1119,41 @@ meta1_backend_services_relink(struct meta1_backend_s *m1,
 	ukept = __parse_and_expand (kept);
 	urepl = __parse_and_expand (replaced);
 
-	/* Sanity checks: we must receive at least one service */
+	/* prefetch the compound type (so it is parsed only once) */
+	if (!oio_url_has(url, OIOURL_TYPE)) {
+		err = BADREQ("Invalid OIOURL: missing service type");
+		goto out;
+	}
+	/* Sanity check: we must receive at least one service */
 	if ((!ukept || !*ukept) && (!urepl || !*urepl)) {
 		err = NEWERROR (CODE_BAD_REQUEST, "Missing URL set");
 		goto out;
 	}
-	/* Sanity check : all the services must have the same <seq,type> */
+
 	struct meta1_service_url_s *ref = ukept && *ukept ? *ukept : *urepl;
+	if (NULL != (err = compound_type_parse(&ct, ref->srvtype)))
+		goto out;
+
+	/* Sanity check: all the services must have the same <seq,type> */
 	for (struct meta1_service_url_s **p = ukept; ukept && *p; ++p) {
-		if (0 != _sorter(p, &ref)) {
-			err = NEWERROR(CODE_BAD_REQUEST, "Mismatch in URL set (%s)", "kept");
+		if (!_idem(*p, ref)) {
+			err = BADREQ("Mismatch in URL set (%s)", "kept");
 			goto out;
 		}
 	}
 	for (struct meta1_service_url_s **p = urepl; urepl && *p; ++p) {
-		if (0 != _sorter(p, &ref)) {
-			err = NEWERROR(CODE_BAD_REQUEST, "Mismatch in URL set (%s)", "kept");
+		if (!_idem(*p, ref)) {
+			err = BADREQ("Mismatch in URL set (%s)", "replaced");
 			goto out;
 		}
 	}
-
-	/* prefetch the compound type (so it is parsed only once) */
-	if (NULL != (err = compound_type_parse(&ct, ref->srvtype))) {
-		err = NEWERROR(CODE_BAD_REQUEST, "Invalid service type");
-		goto out;
+	/* Sanity check: all the kept/replaced services must have the type of
+	 * the oio_url */
+	for (struct meta1_service_url_s **p = urepl; urepl && *p; ++p) {
+		if (0 != strcmp((*p)->srvtype, ct.fulltype)) {
+			err = BADREQ("Service type mismatch (URL vs. kept/replaced)");
+			goto out;
+		}
 	}
 
 	/* Call the backend logic now */
@@ -1146,15 +1198,15 @@ __notify_services(struct meta1_backend_s *m1, struct sqlx_sqlite3_s *sq3,
 	if (!err) {
 		struct meta1_service_url_s **services2 = expand_urlv(services);
 		GString *notif = oio_event__create ("account.services", url);
-		g_string_append (notif, ",\"data\":[");
+		g_string_append_static (notif, ",\"data\":[");
 		if (services2) {
 			for (struct meta1_service_url_s **svc = services2; *svc ; svc++) {
 				if (svc != services2) // not at the beginning
-					g_string_append(notif, ",");
+					g_string_append_c(notif, ',');
 				meta1_service_url_encode_json(notif, *svc);
 			}
 		}
-		g_string_append(notif, "]}");
+		g_string_append_static(notif, "]}");
 
 		oio_events_queue__send (m1->notifier, g_string_free(notif, FALSE));
 
