@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta1v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -32,6 +32,9 @@ GError* meta1_prefixes_load(struct meta1_prefixes_set_s *m1ps,
 
 void meta1_prefixes_clean(struct meta1_prefixes_set_s *m1ps);
 
+/* For testing purposes */
+void meta1_prefixes_manage_all(struct meta1_prefixes_set_s *m1ps);
+
 gboolean meta1_prefixes_is_managed(struct meta1_prefixes_set_s *m1ps,
 		const guint8 *bytes);
 
