@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta1v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -18,6 +18,7 @@ License along with this library.
 */
 
 #include <metautils/lib/metautils.h>
+#include <metautils/lib/common_variables.h>
 
 #include "./internals.h"
 #include "./meta1_remote.h"
@@ -28,7 +29,7 @@ static GError *array_request(const char *to, GByteArray *req,
 	EXTRA_ASSERT(req != NULL);
 
 	GByteArray *gba = NULL;
-	GError *err = gridd_client_exec_and_concat(to, 30.0, req, out ? &gba : NULL);
+	GError *err = gridd_client_exec_and_concat(to, oio_m1_client_timeout_common, req, out ? &gba : NULL);
 
 	if (NULL != err) {
 		if (gba)
