@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta1v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -24,6 +24,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <sys/resource.h>
 
 #include <metautils/lib/metautils.h>
+#include <metautils/lib/common_variables.h>
 #include <cluster/lib/gridcluster.h>
 #include <server/network_server.h>
 #include <server/transport_gridd.h>
@@ -40,6 +41,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 static struct meta1_backend_s *m1 = NULL;
 static volatile gboolean already_succeeded = FALSE;
+static volatile gboolean decache_requested = FALSE;
 
 static GError*
 _reload_prefixes(struct sqlx_service_s *ss, gboolean init)
@@ -55,6 +57,7 @@ _reload_prefixes(struct sqlx_service_s *ss, gboolean init)
 			g_array_free(updated_prefixes, TRUE);
 		return err;
 	}
+
 	if (meta0_ok)
 		already_succeeded = TRUE;
 
@@ -89,6 +92,12 @@ _task_reload_prefixes(gpointer p)
 {
 	static volatile guint tick_reload = 0;
 
+	if (decache_requested) {
+		decache_requested = FALSE;
+		already_succeeded = FALSE;
+		tick_reload = 0;
+	}
+
 	if (already_succeeded && 0 != (tick_reload++ % 32))
 		return;
 
@@ -104,7 +113,7 @@ static void
 _task_reload_policies(gpointer p)
 {
 	GError *err = NULL;
-	gchar *cfg = gridcluster_get_service_update_policy (PSRV(p)->nsinfo);
+	gchar *cfg = oio_var_get_string(oio_ns_service_update_policy);
 	if (!cfg) {
 		err = NEWERROR(EINVAL, "Invalid parameter");
 	} else {
@@ -121,67 +130,31 @@ _task_reload_policies(gpointer p)
 	}
 }
 
-static gchar **
-filter_urls(gchar **src, const gchar *avoid)
-{
-	gboolean matched = FALSE;
-	GPtrArray *tmp = g_ptr_array_new();
-	for (; *src ;++src) {
-		if (!g_ascii_strcasecmp(avoid, *src))
-			matched = TRUE;
-		else
-			g_ptr_array_add(tmp, g_strdup(*src));
-	}
-
-	if (matched) {
-		g_ptr_array_add(tmp, NULL);
-		return (gchar**)g_ptr_array_free(tmp, FALSE);
-	}
-	else {
-		g_ptr_array_add(tmp, NULL);
-		g_strfreev((gchar**)g_ptr_array_free(tmp, FALSE));
-		return NULL;
-	}
-}
-
-static gchar **
-filter_urls_and_clean(gchar **src, const gchar *avoid)
-{
-	if (!src)
-		return NULL;
-	gchar **peers = filter_urls(src, avoid);
-	g_strfreev(src);
-	return peers;
-}
-
 static GError *
-_get_peers(struct sqlx_service_s *ss, const struct sqlx_name_s *n,
+_get_peers(struct sqlx_service_s *ss UNUSED, const struct sqlx_name_s *n,
 		gboolean nocache, gchar ***result)
 {
 	if (!n || !result)
-		return NEWERROR(CODE_INTERNAL_ERROR, "BUG [%s:%s:%d]",
-				__FUNCTION__, __FILE__, __LINE__);
+		return SYSERR("BUG [%s:%s:%d]", __FUNCTION__, __FILE__, __LINE__);
+
 	if (!g_str_has_prefix(n->type, NAME_SRVTYPE_META1))
-		return NEWERROR(CODE_BAD_REQUEST, "Invalid type name");
+		return BADREQ("Invalid type name");
 	if (!oio_str_ishexa(n->base,4))
-		return NEWERROR(CODE_BAD_REQUEST, "Invalid base name");
+		return BADREQ("Invalid base name");
 
 	/* normalizes the maybe-shortened base name: 4 xdigits, padded
 	 * with zeroes if necessary. */
 	guint8 cid[2] = {0,0};
 	oio_str_hex2bin(n->base, cid, 2);
 
-	if (nocache) {
-		_reload_prefixes(ss, FALSE);
-		if (!result)
-			return NULL;
-	}
+	if (nocache)
+		decache_requested = TRUE;
 
-	gchar **peers = meta1_prefixes_get_peers(
-			meta1_backend_get_prefixes(m1), cid);
-	if (!(*result = filter_urls_and_clean(peers, ss->url->str)))
-		return NEWERROR(CODE_CONTAINER_NOTFOUND, "Base not managed");
-	return NULL;
+	*result = meta1_prefixes_get_peers(meta1_backend_get_prefixes(m1), cid);
+	if (likely(*result != NULL))
+		return NULL;
+
+	return NEWERROR(CODE_CONTAINER_NOTFOUND, "Base not managed");
 }
 
 static gboolean
@@ -209,6 +182,7 @@ _post_config(struct sqlx_service_s *ss)
 		 * for this reason. */
 		if (!(err = _reload_prefixes(ss, TRUE))) {
 			done = TRUE;
+			GRID_DEBUG("Prefix reloaded");
 		} else {
 			GRID_WARN("PREFIXES reload failure : (%d) %s", err->code, err->message);
 			g_clear_error(&err);
@@ -229,10 +203,6 @@ _post_config(struct sqlx_service_s *ss)
 
 	m1->notifier = ss->events_queue;
 
-	/* Make the meta1 avoid meta services with known problems */
-	oio_cache_avoid_on_error = oio_cfg_get_bool (
-			ss->ns_name, OIO_CFG_AVOID_BADSRV, FALSE);
-
 	return TRUE;
 }
 
