@@ -1,7 +1,43 @@
 #!/usr/bin/env python
 
+# OpenIO SDS meta2v2
+# Copyright (C) 2014 Worldline, as part of Redcurrant
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 index = 0
 
+agpl = """/*
+Copyright (C) 2017-2017 OpenIO SAS, as part of OpenIO SDS
+
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU Affero General Public License as
+published by the Free Software Foundation, either version 3 of the
+License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU Affero General Public License for more details.
+
+You should have received a copy of the GNU Affero General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
+"""
+
+
 def bool2int(b):
     if b:
         return "1"
@@ -211,6 +247,7 @@ the list."""
         return [self.allbeans[n] for n in result]
 
     def dump_c_header(self, out):
+        out.write(agpl)
         out.write("/* AUTOGENERATED */\n")
         out.write("#ifndef GENERATED_H\n")
         out.write("# define GENERATED_H 1\n")
@@ -258,6 +295,7 @@ the list."""
         out.write("\n#endif /* GENERATED_H */\n")
 
     def dump_c_code(self, out):
+        out.write(agpl)
         out.write("/* !!!AUTOGENERATED!!! */\n")
         out.write("#include <metautils/lib/metautils.h>\n")
         out.write("#include <meta2v2/generic.h>\n")
@@ -462,7 +500,7 @@ the list."""
                     out.write("void\n"+t.c_name+"_set2_"+f.name+"(struct bean_"+t.c_name+"_s *bean, const guint8 *v, gsize vlen)\n{\n")
                     out.write("\tEXTRA_ASSERT(bean != NULL);\n")
                     out.write("\tEXTRA_ASSERT(v != NULL);\n")
-                    out.write("\tGByteArray *gba = g_byte_array_new();\n")
+                    out.write("\tGByteArray *gba = g_byte_array_sized_new(vlen);\n")
                     out.write("\tg_byte_array_append(gba, v, vlen);\n")
                     out.write("\t"+t.c_name+"_set_"+f.name+"(bean, gba);\n")
                     out.write("\tg_byte_array_free(gba, TRUE);\n")
