@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta2v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -20,6 +20,10 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <glib.h>
 
 #include <metautils/lib/metautils.h>
+#include <metautils/lib/common_variables.h>
+#include <server/server_variables.h>
+#include <meta2v2/meta2_variables.h>
+
 #include <cluster/lib/gridcluster.h>
 
 #include <sqliterepo/sqlite_utils.h>
@@ -30,7 +34,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <meta2v2/generic.h>
 #include <meta2v2/autogen.h>
-#include <meta2v2/meta2v2_remote.h>
 #include <meta2v2/meta2_macros.h>
 #include <meta2v2/meta2_utils_lb.h>
 #include <meta2v2/meta2_backend_internals.h>
@@ -46,7 +49,6 @@ enum m2v2_open_type_e
 #define M2V2_OPEN_REPLIMODE 0x00F
 
 	M2V2_OPEN_AUTOCREATE  = 0x010,
-	M2V2_OPEN_NOREFCHECK  = 0x020,
 #define M2V2_OPEN_FLAGS     0x0F0
 
 	// Set an OR'ed combination of the following flags to require
@@ -72,8 +74,11 @@ static void _meta2_backend_force_prepare_data(struct meta2_backend_s *m2b,
 static enum m2v2_open_type_e
 _mode_masterslave(guint32 flags)
 {
-	return (flags & M2V2_FLAG_MASTER)
-		? M2V2_OPEN_MASTERONLY : M2V2_OPEN_MASTERSLAVE;
+	if (flags & M2V2_FLAG_MASTER)
+		return M2V2_OPEN_MASTERONLY;
+	return (flags & M2V2_FLAG_LOCAL)
+		? M2V2_OPEN_LOCAL : M2V2_OPEN_MASTERSLAVE;
+
 }
 
 static enum m2v2_open_type_e
@@ -83,68 +88,22 @@ _mode_readonly(guint32 flags)
 }
 
 static gint64
-_quota(struct sqlx_sqlite3_s *sq3, struct meta2_backend_s *m2b)
+_maxvers(struct sqlx_sqlite3_s *sq3)
 {
-	gint64 quota = 0;
-
-	g_mutex_lock (&m2b->nsinfo_lock);
-	quota = namespace_container_max_size(m2b->nsinfo);
-	g_mutex_unlock (&m2b->nsinfo_lock);
-
-	return m2db_get_quota(sq3, quota);
+	return m2db_get_max_versions(sq3, meta2_max_versions);
 }
 
 static gint64
-m2b_max_versions(struct meta2_backend_s *m2b)
+_retention_delay(struct sqlx_sqlite3_s *sq3)
 {
-	gint64 max_versions = -1;
-
-	g_mutex_lock (&m2b->nsinfo_lock);
-	max_versions = gridcluster_get_container_max_versions(m2b->nsinfo);
-	g_mutex_unlock (&m2b->nsinfo_lock);
-
-	return max_versions;
-}
-
-static gint64
-m2b_keep_deleted_delay(struct meta2_backend_s *m2b)
-{
-	gint64 delay = -1;
-
-	g_mutex_lock (&m2b->nsinfo_lock);
-	delay = gridcluster_get_keep_deleted_delay(m2b->nsinfo);
-	g_mutex_unlock (&m2b->nsinfo_lock);
-
-	return delay;
+	return m2db_get_keep_deleted_delay(sq3, meta2_retention_period);
 }
 
 static gchar*
-m2b_storage_policy(struct meta2_backend_s *m2b)
-{
-	gchar *stgpol = NULL;
-	g_mutex_lock (&m2b->nsinfo_lock);
-	stgpol = namespace_storage_policy(m2b->nsinfo, NULL);
-	g_mutex_unlock (&m2b->nsinfo_lock);
-	return stgpol;
-}
-
-static gint64
-_maxvers(struct sqlx_sqlite3_s *sq3, struct meta2_backend_s *m2b)
-{
-	return m2db_get_max_versions(sq3, m2b_max_versions(m2b));
-}
-
-static gint64
-_retention_delay(struct sqlx_sqlite3_s *sq3, struct meta2_backend_s *m2b)
-{
-	return m2db_get_keep_deleted_delay(sq3, m2b_keep_deleted_delay(m2b));
-}
-
-static gchar*
-_stgpol(struct sqlx_sqlite3_s *sq3, struct meta2_backend_s *m2b)
+_stgpol(struct sqlx_sqlite3_s *sq3)
 {
 	gchar *stgpol = sqlx_admin_get_str(sq3, M2V2_ADMIN_STORAGE_POLICY);
-	return stgpol? stgpol : m2b_storage_policy(m2b);
+	return stgpol ?: oio_var_get_string(oio_ns_storage_policy);
 }
 
 /* Backend ------------------------------------------------------------------ */
@@ -195,7 +154,7 @@ meta2_backend_init(struct meta2_backend_s **result,
 	m2->lb = lb;
 	m2->policies = service_update_policies_create();
 	g_mutex_init(&m2->nsinfo_lock);
-	m2->flag_precheck_on_generate = TRUE;
+	m2->flag_precheck_on_generate = meta2_flag_precheck_on_generate;
 	// TODO: use a custom hash function
 	m2->prepare_data_cache = g_hash_table_new_full(g_str_hash, g_str_equal,
 			g_free, g_free);
@@ -291,8 +250,6 @@ m2_to_sqlx(enum m2v2_open_type_e t)
 
 	if (t & M2V2_OPEN_AUTOCREATE)
 		result |= SQLX_OPEN_CREATE;
-	if (t & M2V2_OPEN_NOREFCHECK)
-		result |= SQLX_OPEN_NOREFCHECK;
 
 	if (t & M2V2_OPEN_ENABLED)
 		result |= SQLX_OPEN_ENABLED;
@@ -307,7 +264,20 @@ m2_to_sqlx(enum m2v2_open_type_e t)
 static gboolean
 _is_container_initiated(struct sqlx_sqlite3_s *sq3)
 {
-	return sqlx_admin_has(sq3, META2_INIT_FLAG);
+	if (sqlx_admin_has(sq3, META2_INIT_FLAG))
+		return TRUE;
+
+	/* workaround for a known bug, when the container has no flag because
+	 * of some failed replication (yet to be determined) but it is used
+	 * because of a (now-fixed) inexistant check on the flag. */
+	if (sqlx_admin_has(sq3, M2V2_ADMIN_OBJ_COUNT)
+			|| sqlx_admin_has(sq3, M2V2_ADMIN_SIZE)) {
+		GRID_DEBUG("DB partially initiated: [%s][%.s]",
+				sq3->name.base, sq3->name.type);
+		return TRUE;
+	}
+
+	return FALSE;
 }
 
 static void
@@ -340,26 +310,22 @@ m2b_open(struct meta2_backend_s *m2, struct oio_url_s *url,
 	EXTRA_ASSERT(m2 != NULL);
 	EXTRA_ASSERT(m2->repo != NULL);
 
-	/* TODO */
-	gboolean no_peers = FALSE;
-	if (no_peers) {
-		how &= ~M2V2_OPEN_REPLIMODE;
-		how |= M2V2_OPEN_LOCAL|M2V2_OPEN_NOREFCHECK;
-	}
-
-	struct sqlx_name_mutable_s n;
-	sqlx_name_fill (&n, url, NAME_SRVTYPE_META2, 1);
-	err = sqlx_repository_open_and_lock(m2->repo,
-			sqlx_name_mutable_to_const(&n), m2_to_sqlx(how), &sq3, NULL);
-	sqlx_name_clean (&n);
+	struct sqlx_name_inline_s n0;
+	sqlx_inline_name_fill (&n0, url, NAME_SRVTYPE_META2, 1);
+	NAME2CONST(n,n0);
 
+	err = sqlx_repository_open_and_lock(m2->repo, &n, m2_to_sqlx(how), &sq3, NULL);
 	if (NULL != err) {
 		if (err->code == CODE_CONTAINER_NOTFOUND)
 			err->domain = GQ();
 		return err;
 	}
 
-	sq3->no_peers = how & (M2V2_OPEN_LOCAL|M2V2_OPEN_NOREFCHECK);
+	/* beware that LOCAL is maybe == 0 */
+	const gboolean _local = (M2V2_OPEN_LOCAL == (how & M2V2_OPEN_REPLIMODE));
+	const gboolean _create = (M2V2_OPEN_AUTOCREATE == (how & M2V2_OPEN_AUTOCREATE));
+
+	sq3->no_peers = _local;
 
 	// If the container is being deleted, this is sad ...
 	// This MIGHT happen if a cache is present (and this is the
@@ -373,12 +339,17 @@ m2b_open(struct meta2_backend_s *m2, struct oio_url_s *url,
 		return NEWERROR(CODE_CONTAINER_FROZEN, "destruction pending");
 	}
 
-	/* M2V2_OPEN_AUTOCREATE is only used when creating the container. If not
-	 * specified, we expect the container to also be "softly" initiated.
-	 * Non-LOCAL accesses are made by the functional requests, and by no
-	 * other, excepted by the DESTROY request. */
-	if (!(how & M2V2_OPEN_AUTOCREATE) &&
-			M2V2_OPEN_LOCAL == (how & M2V2_OPEN_REPLIMODE)) {
+	/* The kind of check we do depend of the kind of opening:
+	 * - creation : init not done
+	 * - local access : no check
+	 * - replicated access : int done */
+	if (_create) {
+		if (_is_container_initiated(sq3)) {
+			m2b_close(sq3);
+			return NEWERROR(CODE_CONTAINER_EXISTS,
+					"container already initiated");
+		}
+	} else if (!_local) {
 		if (!_is_container_initiated(sq3)) {
 			m2b_close(sq3);
 			return NEWERROR(CODE_CONTAINER_NOTFOUND,
@@ -386,7 +357,7 @@ m2b_open(struct meta2_backend_s *m2, struct oio_url_s *url,
 		}
 	}
 
-	// Complete URL with full VNS and container name
+	/* Complete URL with full VNS and container name */
 	void set(gchar *k, int f) {
 		if (oio_url_has(url, f))
 			return;
@@ -442,11 +413,11 @@ meta2_backend_has_container(struct meta2_backend_s *m2,
 	EXTRA_ASSERT(url != NULL);
 	GRID_DEBUG("HAS(%s)", oio_url_get(url, OIOURL_WHOLE));
 
-	struct sqlx_name_mutable_s n;
-	sqlx_name_fill (&n, url, NAME_SRVTYPE_META2, 1);
-	err = sqlx_repository_has_base(m2->repo, sqlx_name_mutable_to_const(&n));
-	sqlx_name_clean (&n);
+	struct sqlx_name_inline_s n0;
+	sqlx_inline_name_fill (&n0, url, NAME_SRVTYPE_META2, 1);
+	NAME2CONST(n,n0);
 
+	err = sqlx_repository_has_base(m2->repo, &n);
 	if (NULL != err) {
 		g_prefix_error(&err, "File error: ");
 		return err;
@@ -524,7 +495,7 @@ meta2_backend_get_max_versions(struct meta2_backend_s *m2b,
 
 	err = m2b_open(m2b, url, _mode_readonly(0), &sq3);
 	if (!err) {
-		*result = _maxvers(sq3, m2b);
+		*result = _maxvers(sq3);
 		m2b_close(sq3);
 	}
 
@@ -541,12 +512,6 @@ _init_container(struct sqlx_sqlite3_s *sq3,
 	if (!params->local && (err = _transaction_begin(sq3, url, &repctx)))
 		return err;
 
-	if (!err && params->storage_policy)
-		err = m2db_set_storage_policy(sq3, params->storage_policy, 0);
-	if (!err && params->version_policy) {
-		gint64 max = g_ascii_strtoll(params->version_policy, NULL, 10);
-		m2db_set_max_versions(sq3, max);
-	}
 	if (!err) {
 		m2db_set_ctime (sq3, oio_ext_real_time());
 		m2db_set_size(sq3, 0);
@@ -557,6 +522,12 @@ _init_container(struct sqlx_sqlite3_s *sq3,
 		for (gchar **p=params->properties; *p && *(p+1) ;p+=2)
 			sqlx_admin_set_str (sq3, *p, *(p+1));
 	}
+	if (!err && params->storage_policy)
+		err = m2db_set_storage_policy(sq3, params->storage_policy, TRUE);
+	if (!err && params->version_policy) {
+		gint64 max = g_ascii_strtoll(params->version_policy, NULL, 10);
+		m2db_set_max_versions(sq3, max);
+	}
 	if (!params->local)
 		err = sqlx_transaction_end(repctx, err);
 	return err;
@@ -567,7 +538,6 @@ meta2_backend_create_container(struct meta2_backend_s *m2,
 		struct oio_url_s *url, struct m2v2_create_params_s *params)
 {
 	GError *err = NULL;
-	enum m2v2_open_type_e open_mode = 0;
 	struct sqlx_sqlite3_s *sq3 = NULL;
 
 	GRID_DEBUG("CREATE(%s,%s,%s)%s", oio_url_get(url, OIOURL_WHOLE),
@@ -582,41 +552,36 @@ meta2_backend_create_container(struct meta2_backend_s *m2,
 			return err;
 	}
 
-	if (params->local) // NOREFCHECK: do not call get_peers()
-		open_mode = M2V2_OPEN_LOCAL|M2V2_OPEN_NOREFCHECK;
-	else
-		open_mode = M2V2_OPEN_MASTERONLY;
-
-	/* The AUTOCREATE flag disables the INIT check */
-	open_mode |= M2V2_OPEN_AUTOCREATE;
+	/* Defer the `m2.init` check to the m2b_open() */
+	enum m2v2_open_type_e open_mode = M2V2_OPEN_AUTOCREATE |
+		(params->local ? M2V2_OPEN_LOCAL : M2V2_OPEN_MASTERONLY);
 
 	err = m2b_open(m2, url, open_mode, &sq3);
-	if (sq3 && !err) {
-		if (sqlx_admin_has(sq3, META2_INIT_FLAG))
-			err = NEWERROR(CODE_CONTAINER_EXISTS, "Container already initiated");
-		else {
-			err = _init_container(sq3, url, params);
-			if (err) {
-				m2b_destroy(sq3);
-				return err;
-			}
+	EXTRA_ASSERT((sq3 != NULL) ^ (err != NULL));
+	if (err)
+		return err;
 
-			/* Fire an event to notify the world this container exists */
-			const enum election_status_e s = sq3->election;
-			if (!params->local && m2->notifier && (!s || s == ELECTION_LEADER)) {
-				GString *gs = oio_event__create (META2_EVENTS_PREFIX".container.new", url);
-				g_string_append (gs, ",\"data\":null}");
-				oio_events_queue__send (m2->notifier, g_string_free (gs, FALSE));
-			}
-
-			/* Reload any cache maybe already associated with the container.
-			 * It happens that the cache sometimes exists because created during a
-			 * M2_PREP that occured after a GETVERS (the meta1 is filled) but before
-			 * the CREATE. */
-			meta2_backend_change_callback(sq3, m2);
-		}
-		m2b_close(sq3);
+	/* At this point the base exist and it has nt been iniated yet */
+	err = _init_container(sq3, url, params);
+	if (err) {
+		m2b_destroy(sq3);
+		return err;
 	}
+
+	/* Fire an event to notify the world this container exists */
+	const enum election_status_e s = sq3->election;
+	if (!params->local && m2->notifier && (!s || s == ELECTION_LEADER)) {
+		GString *gs = oio_event__create (META2_EVENTS_PREFIX".container.new", url);
+		g_string_append_static (gs, ",\"data\":null}");
+		oio_events_queue__send (m2->notifier, g_string_free (gs, FALSE));
+	}
+
+	/* Reload any cache maybe already associated with the container.
+	 * It happens that the cache sometimes exists because created during a
+	 * M2_PREP that occured after a GETVERS (the meta1 is filled) but before
+	 * the CREATE. */
+	meta2_backend_change_callback(sq3, m2);
+	m2b_close(sq3);
 	return err;
 }
 
@@ -630,7 +595,7 @@ meta2_backend_destroy_container(struct meta2_backend_s *m2,
 	struct sqlx_sqlite3_s *sq3 = NULL;
 	GError *err = NULL;
 
-	err = m2b_open(m2, url, M2V2_OPEN_LOCAL|M2V2_OPEN_NOREFCHECK, &sq3);
+	err = m2b_open(m2, url, M2V2_OPEN_LOCAL, &sq3);
 	if (!err) {
 		EXTRA_ASSERT(sq3 != NULL);
 
@@ -649,11 +614,25 @@ meta2_backend_destroy_container(struct meta2_backend_s *m2,
 		/* TODO(jfs): manage base's subtype */
 		hc_decache_reference_service(m2->resolver, url, NAME_SRVTYPE_META2);
 
+#ifdef HAVE_ENBUG
+		if (event && !err) {
+			gint32 random = oio_ext_rand_int_range(0,3);
+			switch (random) {
+				case 0:
+					err = NEWERROR(CODE_BAD_REQUEST, "Fake error (meta2)");
+					break;
+				case 1:
+					sleep(100);
+					break;
+			}
+		}
+#endif
+
 		if (!err) {
 			GString *gs = NULL;
 			if (event && m2->notifier) {
 				gs = oio_event__create (META2_EVENTS_PREFIX ".container.deleted", url);
-				g_string_append (gs, ",\"data\":null}");
+				g_string_append_static (gs, ",\"data\":null}");
 			}
 			m2b_destroy(sq3);
 			if (gs) {
@@ -681,8 +660,7 @@ meta2_backend_flush_container(struct meta2_backend_s *m2, struct oio_url_s *url)
 		EXTRA_ASSERT(sq3 != NULL);
 		if (!(err = sqlx_transaction_begin(sq3, &repctx))) {
 			if (!(err = m2db_flush_container(sq3->db)))
-				err = m2db_purge(sq3, _maxvers(sq3, m2),
-						_retention_delay(sq3, m2));
+				err = m2db_purge(sq3, _maxvers(sq3), _retention_delay(sq3));
 			err = sqlx_transaction_end(repctx, err);
 		}
 		m2b_close(sq3);
@@ -702,7 +680,7 @@ meta2_backend_purge_container(struct meta2_backend_s *m2, struct oio_url_s *url)
 	if (!err) {
 		EXTRA_ASSERT(sq3 != NULL);
 		if (!(err = sqlx_transaction_begin(sq3, &repctx))) {
-			err = m2db_purge(sq3, _maxvers(sq3, m2), _retention_delay(sq3, m2));
+			err = m2db_purge(sq3, _maxvers(sq3), _retention_delay(sq3));
 			err = sqlx_transaction_end(repctx, err);
 		}
 		m2b_close(sq3);
@@ -725,7 +703,8 @@ meta2_backend_list_aliases(struct meta2_backend_s *m2b, struct oio_url_s *url,
 	EXTRA_ASSERT(url != NULL);
 	EXTRA_ASSERT(lp != NULL);
 
-	err = m2b_open(m2b, url, _mode_readonly(0), &sq3);
+	guint32 open_mode = lp->flag_local? M2V2_FLAG_LOCAL: 0;
+	err = m2b_open(m2b, url, _mode_readonly(open_mode), &sq3);
 	if (!err) {
 		err = m2db_list_aliases(sq3, lp, headers, cb, u0);
 		if (!err && out_properties)
@@ -786,12 +765,12 @@ _container_state (struct sqlx_sqlite3_s *sq3)
 		oio_url_set (u, OIOURL_TYPE, sq3->name.type + sizeof(NAME_SRVTYPE_META2));
 
 	GString *gs = oio_event__create (META2_EVENTS_PREFIX ".container.state", u);
-	g_string_append (gs, ",\"data\":{");
+	g_string_append_static (gs, ",\"data\":{");
 	append_const (gs, "policy", sqlx_admin_get_str(sq3, M2V2_ADMIN_STORAGE_POLICY));
 	append_int64 (gs, "ctime", m2db_get_ctime(sq3));
 	append_int64 (gs, "bytes-count", m2db_get_size(sq3));
 	append_int64 (gs, "object-count", m2db_get_obj_count(sq3));
-	g_string_append (gs, "}}");
+	g_string_append_static (gs, "}}");
 
 	oio_url_clean (u);
 	return g_string_free(gs, FALSE);
@@ -808,8 +787,8 @@ m2b_add_modified_container(struct meta2_backend_s *m2b,
 				_container_state (sq3));
 
 	gboolean has_peers = FALSE;
-	GError *err = election_has_peers(sq3->manager,
-			sqlx_name_mutable_to_const(&(sq3->name)), FALSE, &has_peers);
+	NAME2CONST(n, sq3->name);
+	GError *err = election_has_peers(sq3->manager, &n, FALSE, &has_peers);
 	if (!err && !has_peers) {
 		meta2_backend_change_callback(sq3, m2b);
 	}
@@ -856,6 +835,25 @@ meta2_backend_refresh_container_size(struct meta2_backend_s *m2b,
 	return err;
 }
 
+GError *
+meta2_backend_drain_content(struct meta2_backend_s *m2,
+		struct oio_url_s *url, m2_onbean_cb cb, gpointer u0)
+{
+	GError *err = NULL;
+	struct sqlx_sqlite3_s *sq3 = NULL;
+	struct sqlx_repctx_s *repctx = NULL;
+	err = m2b_open(m2, url, M2V2_OPEN_MASTERONLY | M2V2_OPEN_ENABLED, &sq3);
+	if (!err) {
+		EXTRA_ASSERT(sq3 != NULL);
+		if (!(err = sqlx_transaction_begin(sq3, &repctx))) {
+			err = m2db_drain_content(sq3, url, cb, u0);
+			err = sqlx_transaction_end(repctx, err);
+		}
+		m2b_close(sq3);
+	}
+	return err;
+}
+
 GError*
 meta2_backend_delete_alias(struct meta2_backend_s *m2b,
 		struct oio_url_s *url, m2_onbean_cb cb, gpointer u0)
@@ -869,7 +867,7 @@ meta2_backend_delete_alias(struct meta2_backend_s *m2b,
 	err = m2b_open(m2b, url, M2V2_OPEN_MASTERONLY|M2V2_OPEN_ENABLED, &sq3);
 	if (!err) {
 		struct sqlx_repctx_s *repctx = NULL;
-		const gint64 max_versions = _maxvers(sq3, m2b);
+		const gint64 max_versions = _maxvers(sq3);
 		if (!(err = _transaction_begin(sq3, url, &repctx))) {
 			if (!(err = m2db_delete_alias(sq3, max_versions, url, cb, u0))) {
 				m2db_increment_version(sq3);
@@ -904,7 +902,8 @@ meta2_backend_put_alias(struct meta2_backend_s *m2b, struct oio_url_s *url,
 		memset(&args, 0, sizeof(args));
 		args.sq3 = sq3;
 		args.url = url;
-		args.ns_max_versions = m2b_max_versions(m2b);
+		args.ns_max_versions = meta2_max_versions;
+		args.worm_mode = oio_ns_mode_worm && !oio_ext_is_admin();
 
 		if (!(err = _transaction_begin(sq3, url, &repctx))) {
 			if (!(err = m2db_put_alias(&args, in, out_deleted, out_added)))
@@ -937,7 +936,8 @@ meta2_backend_copy_alias(struct meta2_backend_s *m2b, struct oio_url_s *url,
 		memset(&args, 0, sizeof(args));
 		args.sq3 = sq3;
 		args.url = url;
-		args.ns_max_versions = m2b_max_versions(m2b);
+		args.ns_max_versions = meta2_max_versions;
+		args.worm_mode = oio_ns_mode_worm && !oio_ext_is_admin();
 
 		if (!(err = _transaction_begin(sq3, url, &repctx))) {
 			if (!(err = m2db_copy_alias(&args, src)))
@@ -962,6 +962,9 @@ meta2_backend_update_content(struct meta2_backend_s *m2b, struct oio_url_s *url,
 	EXTRA_ASSERT(url != NULL);
 	if (!in)
 		return NEWERROR(CODE_BAD_REQUEST, "No bean");
+	else if (oio_ns_mode_worm && !oio_ext_is_admin())
+		return NEWERROR(CODE_METHOD_NOTALLOWED,
+				"NS wormed! Cannot modify object.");
 
 	err = m2b_open(m2b, url, M2V2_OPEN_MASTERONLY|M2V2_OPEN_ENABLED, &sq3);
 	if (!err) {
@@ -1029,7 +1032,8 @@ meta2_backend_force_alias(struct meta2_backend_s *m2b, struct oio_url_s *url,
 		memset(&args, 0, sizeof(args));
 		args.sq3 = sq3;
 		args.url = url;
-		args.ns_max_versions = m2b_max_versions(m2b);
+		args.ns_max_versions = meta2_max_versions;
+		args.worm_mode = oio_ns_mode_worm && !oio_ext_is_admin();
 
 		if (!(err = _transaction_begin(sq3,url, &repctx))) {
 			if (!(err = m2db_force_alias(&args, in, out_deleted, out_added)))
@@ -1345,25 +1349,34 @@ _check_alias_doesnt_exist(struct sqlx_sqlite3_s *sq3, struct oio_url_s *url)
 	return err;
 }
 
-/* Create, save in cache, and possibly return m2_prepare_data */
+/* Create, save in cache, and possibly return m2_prepare_data.
+ * If container is disabled or frozen, decache m2_prepare_data
+ * and return nothing. */
 static void
 _meta2_backend_force_prepare_data_unlocked(struct meta2_backend_s *m2b,
 		const gchar *key, struct m2_prepare_data *pdata_out,
 		struct sqlx_sqlite3_s *sq3)
 {
-	GRID_DEBUG("Forcing M2_PREPARE data for %s", key);
 
 	struct m2_prepare_data *pdata = g_hash_table_lookup(
 			m2b->prepare_data_cache, key);
+
+	gint64 status = sqlx_admin_get_status(sq3);
+	if (status != ADMIN_STATUS_ENABLED) {
+		GRID_DEBUG("Decaching M2_PREP data for %s", key);
+		g_hash_table_remove(m2b->prepare_data_cache, key);
+		return;
+	}
+	GRID_DEBUG("Forcing M2_PREP data for %s", key);
 	if (!pdata) {
 		pdata = g_malloc0(sizeof(struct m2_prepare_data));
 		g_hash_table_insert(m2b->prepare_data_cache, g_strdup(key), pdata);
 	}
 
-	pdata->max_versions = _maxvers(sq3, m2b);
-	pdata->quota = _quota(sq3, m2b);
+	pdata->max_versions = _maxvers(sq3);
+	pdata->quota = m2db_get_quota(sq3, meta2_container_max_size);
 	pdata->size = m2db_get_size(sq3);
-	gchar *stgpol = _stgpol(sq3, m2b);
+	gchar *stgpol = _stgpol(sq3);
 	g_strlcpy(pdata->storage_policy, stgpol, LIMIT_LENGTH_STGPOLICY);
 	g_free(stgpol);
 
@@ -1372,8 +1385,9 @@ _meta2_backend_force_prepare_data_unlocked(struct meta2_backend_s *m2b,
 }
 
 /**
- * Update the data structure allowing to answer PREPARE requests
- * without taking the lock on the database file.
+ * Update the data structure allowing to answer M2_PREP requests
+ * without taking the lock on the database file. If the container
+ * is frozen or disabled, decache this data.
  */
 static void
 _meta2_backend_force_prepare_data(struct meta2_backend_s *m2b,
@@ -1392,7 +1406,7 @@ meta2_backend_change_callback(struct sqlx_sqlite3_s *sq3,
 	gchar *user = sqlx_admin_get_str(sq3, SQLX_ADMIN_USERNAME);
 	if (!account || !user) {
 		GRID_WARN("Missing "SQLX_ADMIN_ACCOUNT" or "SQLX_ADMIN_USERNAME
-				" in database %s", sq3->path);
+				" in database %s", sq3->path_inline);
 	} else {
 		struct oio_url_s *url = oio_url_empty();
 		oio_url_set(url, OIOURL_NS, m2b->ns_name);
@@ -1433,10 +1447,12 @@ m2b_get_prepare_data(struct meta2_backend_s *m2b,
 	g_rw_lock_reader_unlock(&(m2b->prepare_data_lock));
 
 	if (!pdata) {
-		// Prepare data is not available. Open the base, take the writer lock
-		// and check again, in case another thread did the job while we were
-		// waiting for the base or the writer lock.
-		err = m2b_open(m2b, url, _mode_readonly(0), sq3);
+		/* Prepare data is not available. Open the base, take the writer lock
+		 * and check again, in case another thread did the job while we were
+		 * waiting for the base or the writer lock.
+		 * The base must not be frozen or disabled
+		 * (we must refuse "prepare" operation in such cases). */
+		err = m2b_open(m2b, url, _mode_masterslave(0)|M2V2_OPEN_ENABLED, sq3);
 		if (!err) {
 			g_rw_lock_writer_lock(&(m2b->prepare_data_lock));
 			pdata = g_hash_table_lookup(m2b->prepare_data_cache, key);
@@ -1453,6 +1469,20 @@ m2b_get_prepare_data(struct meta2_backend_s *m2b,
 	return err;
 }
 
+GError *
+meta2_backend_check_content(struct meta2_backend_s *m2b,
+		GSList *beans, GString *message, gboolean update)
+{
+	GError *err = NULL;
+	struct namespace_info_s *nsinfo = NULL;
+	if (!(nsinfo = meta2_backend_get_nsinfo(m2b)))
+		return NEWERROR(CODE_INTERNAL_ERROR, "NS not ready");
+
+	err = m2db_check_content(beans, nsinfo, message, update);
+	namespace_info_free(nsinfo);
+	return err;
+}
+
 GError*
 meta2_backend_generate_beans(struct meta2_backend_s *m2b,
 		struct oio_url_s *url, gint64 size, const gchar *polname,
@@ -1477,13 +1507,17 @@ meta2_backend_generate_beans(struct meta2_backend_s *m2b,
 	 * This call may return an open database. */
 	m2b_get_prepare_data(m2b, url, &pdata, &sq3);
 
-	if (m2b->flag_precheck_on_generate &&
-			VERSIONS_DISABLED(pdata.max_versions)) {
+	gboolean must_check_alias = m2b->flag_precheck_on_generate && (
+			 VERSIONS_DISABLED(pdata.max_versions) ||
+			(VERSIONS_SUSPENDED(pdata.max_versions) &&
+			 oio_ns_mode_worm &&
+			 !oio_ext_is_admin()));
+	if (must_check_alias) {
 		err = m2b_open_if_needed(m2b, url,
 				_mode_masterslave(0)|M2V2_OPEN_ENABLED, &sq3);
 		if (!err) {
-			/* If the versioning is not supported, we check the content
-			 * is not present */
+			/* If the versioning is not supported, or the namespace is
+			 * is WORM mode, we check the content is not present */
 			err = _check_alias_doesnt_exist(sq3, url);
 			if (append) {
 				if (err) {
@@ -1508,14 +1542,13 @@ meta2_backend_generate_beans(struct meta2_backend_s *m2b,
 				err = m2db_get_storage_policy(sq3, url, nsinfo, append,
 						&policy);
 			if (err || !policy) {
-				gchar *def = namespace_storage_policy(nsinfo,
-						oio_url_get(url, OIOURL_NS));
-				if (NULL != def) {
+				gchar *def = oio_var_get_string(oio_ns_storage_policy);
+				if (oio_str_is_set(def)) {
 					if (!(policy = storage_policy_init(nsinfo, def)))
 						err = NEWERROR(CODE_POLICY_NOT_SUPPORTED,
 								"Invalid policy [%s]", def);
-					g_free(def);
 				}
+				g_free0(def);
 			}
 		} else {
 			if (!polname)
@@ -1538,8 +1571,7 @@ meta2_backend_generate_beans(struct meta2_backend_s *m2b,
 
 	/* Let's continue to generate the beans, no need for an open container for the moment */
 	if (!err) {
-		err = m2_generate_beans(url, size,
-				namespace_chunk_size(nsinfo, oio_url_get(url, OIOURL_NS)),
+		err = m2_generate_beans(url, size, oio_ns_chunk_size,
 				policy, m2b->lb, cb, cb_data);
 	}
 
@@ -1570,13 +1602,13 @@ _load_storage_policy(struct meta2_backend_s *m2b, struct oio_url_s *url,
 			/* check pol from container / ns */
 			err = m2db_get_storage_policy(sq3, url, nsinfo, FALSE, pol);
 			if (err || !*pol) {
-				gchar *def = namespace_storage_policy(nsinfo, oio_url_get(url, OIOURL_NS));
-				if (NULL != def) {
+				gchar *def = oio_var_get_string(oio_ns_storage_policy);
+				if (oio_str_is_set(def)) {
 					if (!(*pol = storage_policy_init(nsinfo, def)))
 						err = NEWERROR(CODE_POLICY_NOT_SUPPORTED,
 								"Invalid policy [%s]", def);
-					g_free(def);
 				}
+				g_free0(def);
 			}
 		}
 		m2b_close(sq3);
