@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta2v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -134,6 +134,9 @@ GError* meta2_backend_append_to_alias(struct meta2_backend_s *m2b,
 		struct oio_url_s *url, GSList *beans,
 		m2_onbean_cb cb, gpointer u0);
 
+GError* meta2_backend_check_content(struct meta2_backend_s *m2b,
+		GSList *beans, GString *message, gboolean update);
+
 /** Update a content with the given chunks replacing the existing chunks
  *  at the same position. */
 GError *meta2_backend_update_content(struct meta2_backend_s *m2b,
@@ -156,6 +159,9 @@ GError* meta2_backend_force_alias(struct meta2_backend_s *m2b,
 GError* meta2_backend_link_content (struct meta2_backend_s *m2b,
 		struct oio_url_s *url, GBytes *content_id);
 
+GError *meta2_backend_drain_content(struct meta2_backend_s *m2b,
+		struct oio_url_s *url, m2_onbean_cb cb, gpointer u0);
+
 GError* meta2_backend_delete_alias(struct meta2_backend_s *m2b,
 		struct oio_url_s *url, m2_onbean_cb cb, gpointer u0);
 
