@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta2v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -90,7 +90,9 @@ M2V2_DECLARE_FILTER(meta2_filter_action_list_by_header_id);
 M2V2_DECLARE_FILTER(meta2_filter_action_list_by_header_hash);
 M2V2_DECLARE_FILTER(meta2_filter_action_put_content);
 M2V2_DECLARE_FILTER(meta2_filter_action_append_content);
+M2V2_DECLARE_FILTER(meta2_filter_action_check_content);
 M2V2_DECLARE_FILTER(meta2_filter_action_get_content);
+M2V2_DECLARE_FILTER(meta2_filter_action_drain_content);
 M2V2_DECLARE_FILTER(meta2_filter_action_delete_content);
 M2V2_DECLARE_FILTER(meta2_filter_action_truncate_content);
 M2V2_DECLARE_FILTER(meta2_filter_action_link);
