@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta2v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -21,10 +21,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 #include <errno.h>
 
-#include <glib.h>
-
 #include <metautils/lib/metautils.h>
-#include <metautils/lib/metacomm.h>
+#include <meta2v2/meta2_variables.h>
+
 #include <cluster/lib/gridcluster.h>
 #include <sqliterepo/sqlite_utils.h>
 #include <server/transport_gridd.h>
@@ -34,7 +33,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <meta2v2/meta2_filters.h>
 #include <meta2v2/meta2_backend_internals.h>
 #include <meta2v2/meta2_bean.h>
-#include <meta2v2/meta2v2_remote.h>
 #include <meta2v2/generic.h>
 #include <meta2v2/autogen.h>
 #include <resolver/hc_resolver.h>
@@ -202,7 +200,7 @@ _list_S3(struct gridd_filter_ctx_s *ctx, struct gridd_reply_ctx_s *reply,
 	gchar **properties = NULL;
 
 	if (lp->maxkeys <= 0)
-		lp->maxkeys = OIO_M2V2_LISTRESULT_BATCH;
+		lp->maxkeys = meta2_batch_maxlen;
 
 	GRID_DEBUG("LP H:%d A:%d D:%d prefix:%s marker:%s end:%s max:%"G_GINT64_FORMAT,
 			lp->flag_headers, lp->flag_allversion, lp->flag_nodeleted,
@@ -280,6 +278,7 @@ _load_list_params (struct list_params_s *lp, struct gridd_filter_ctx_s *ctx)
 		lp->flag_nodeleted = BOOL(flags & M2V2_FLAG_NODELETED);
 		lp->flag_allversion = BOOL(flags & M2V2_FLAG_ALLVERSION);
 		lp->flag_properties = BOOL(flags & M2V2_FLAG_ALLPROPS);
+		lp->flag_local = BOOL(flags & M2V2_FLAG_LOCAL);
 	}
 
 	lp->prefix = meta2_filter_ctx_get_param(ctx, NAME_MSGKEY_PREFIX);
