@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta2v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -36,7 +36,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <meta2v2/meta2_filters.h>
 #include <meta2v2/meta2_backend_internals.h>
 #include <meta2v2/meta2_bean.h>
-#include <meta2v2/meta2v2_remote.h>
 #include <meta2v2/meta2_utils_json.h>
 #include <meta2v2/generic.h>
 #include <meta2v2/autogen.h>
@@ -50,29 +49,44 @@ enum content_action_e
 
 static void
 _m2b_notify_beans(struct meta2_backend_s *m2b, struct oio_url_s *url,
-		GSList *beans, const char *name)
+		GSList *beans, const char *name, gboolean send_chunks)
 {
-	void forward (GSList *list_of_beans) {
+	guint n_events = 1;
+	guint cur_event = 0;
+	void forward(GSList *list_of_beans) {
 		gchar tmp[256];
 		g_snprintf (tmp, sizeof(tmp), "%s.%s", META2_EVENTS_PREFIX, name);
 
-		GString *gs = oio_event__create (tmp, url);
-		g_string_append (gs, ",\"data\":[");
+		GString *gs = oio_event__create(tmp, url);
+		g_string_append_c(gs, ',');
+		oio_str_gstring_append_json_pair_int(gs, "part", cur_event++);
+		g_string_append_c(gs, ',');
+		oio_str_gstring_append_json_pair_int(gs, "parts", n_events);
+		g_string_append_static (gs, ",\"data\":[");
 		meta2_json_dump_all_xbeans (gs, list_of_beans);
-		g_string_append (gs, "]}");
+		g_string_append_static (gs, "]}");
 		oio_events_queue__send (m2b->notifier, g_string_free (gs, FALSE));
 	}
 
 	if (!m2b->notifier)
 		return;
 
-	if (g_slist_length (beans) < 16)
-		forward (beans);
-	else {
+	guint beans_len = g_slist_length(beans);
+	if (!send_chunks) {
+		GSList *non_chunks = NULL;
+		for (GSList *l = beans; l; l = l->next) {
+			if (DESCR(l->data) != &descr_struct_CHUNKS)
+				non_chunks = g_slist_prepend(non_chunks, l->data);
+		}
+		forward(non_chunks);
+		g_slist_free(non_chunks);
+	} else if (beans_len < 16) {
+		forward(beans);
+	} else {
 		/* first, notify everything but the chunks */
 		GSList *non_chunks = NULL;
 		struct bean_CONTENTS_HEADERS_s *header = NULL;
-		for (GSList *l=beans; l ;l=l->next) {
+		for (GSList *l = beans; l; l = l->next) {
 			if (DESCR(l->data) == &descr_struct_CONTENTS_HEADERS) {
 				if (header)
 					GRID_WARN("Several content headers in same event!");
@@ -83,6 +97,7 @@ _m2b_notify_beans(struct meta2_backend_s *m2b, struct oio_url_s *url,
 				non_chunks = g_slist_prepend (non_chunks, l->data);
 			}
 		}
+		n_events += 1 + (beans_len - g_slist_length(non_chunks)) / 16;
 		if (non_chunks) {
 			forward (non_chunks);
 			g_slist_free (non_chunks);
@@ -94,7 +109,7 @@ _m2b_notify_beans(struct meta2_backend_s *m2b, struct oio_url_s *url,
 		/* then notify each chunks by batches of 16 items */
 		GSList *batch = NULL;
 		guint count = 0;
-		for (GSList *l=beans; l ;l=l->next) {
+		for (GSList *l = beans; l; l = l->next) {
 			if (&descr_struct_CHUNKS != DESCR(l->data))
 				continue;
 			batch = g_slist_prepend (batch, l->data);
@@ -152,9 +167,9 @@ _put_alias(struct gridd_filter_ctx_s *ctx, struct gridd_reply_ctx_s *reply)
 		meta2_filter_ctx_set_error(ctx, e);
 		rc = FILTER_KO;
 	} else {
-		_m2b_notify_beans(m2b, url, added, "content.new");
+		_m2b_notify_beans(m2b, url, added, "content.new", FALSE);
 		if (deleted)
-			_m2b_notify_beans(m2b, url, deleted, "content.deleted");
+			_m2b_notify_beans(m2b, url, deleted, "content.deleted", TRUE);
 		_on_bean_ctx_send_list(obc);
 		rc = FILTER_OK;
 	}
@@ -193,6 +208,44 @@ _copy_alias(struct gridd_filter_ctx_s *ctx, struct gridd_reply_ctx_s *reply,
 	return FILTER_OK;
 }
 
+int
+meta2_filter_action_check_content(struct gridd_filter_ctx_s * ctx,
+		struct gridd_reply_ctx_s *reply) {
+	(void) reply;
+	GError *e = NULL;
+	int rc = FILTER_OK;
+	struct meta2_backend_s *m2b = meta2_filter_ctx_get_backend(ctx);
+	struct oio_url_s *url = meta2_filter_ctx_get_url(ctx);
+	const char *copy_source = meta2_filter_ctx_get_param(ctx, NAME_MSGKEY_COPY);
+	if (copy_source != NULL)
+		return rc;
+
+	gboolean is_update = FALSE;
+	const char *update = meta2_filter_ctx_get_param(ctx, NAME_MSGKEY_UPDATE);
+	if (update != NULL)
+		is_update=TRUE;
+
+	GSList *beans = meta2_filter_ctx_get_input_udata(ctx);
+	GString *message= g_string_new("");
+	e = meta2_backend_check_content(m2b, beans, message, is_update);
+	if (e) {
+		GString *gs = oio_event__create("storage.content.broken", url);
+		g_string_append(gs, ",\"data\":{");
+		g_string_append(gs, message->str);
+		g_string_append(gs, "}}");
+		oio_events_queue__send (m2b->notifier, g_string_free (gs, FALSE));
+		if (e->code == CODE_CONTENT_CORRUPTED) {
+			meta2_filter_ctx_set_error(ctx, e);
+			e = NULL;
+			rc = FILTER_KO;
+		} else {
+			g_clear_error(&e);
+		}
+	}
+	g_string_free(message, TRUE);
+	return rc;
+}
+
 int
 meta2_filter_action_put_content(struct gridd_filter_ctx_s *ctx,
 		struct gridd_reply_ctx_s *reply)
@@ -231,7 +284,7 @@ meta2_filter_action_append_content(struct gridd_filter_ctx_s *ctx,
 		return FILTER_KO;
 	}
 
-	_m2b_notify_beans(m2b, url, obc->l, "content.append");
+	_m2b_notify_beans(m2b, url, obc->l, "content.append", FALSE);
 	_on_bean_ctx_send_list(obc);
 	_on_bean_ctx_clean(obc);
 	return FILTER_OK;
@@ -263,6 +316,16 @@ meta2_filter_action_get_content(struct gridd_filter_ctx_s *ctx,
 		goto cleanup;
 	}
 
+	for (GSList *l = obc->l; l; l = l->next) {
+		if (DESCR(l->data) == &descr_struct_CONTENTS_HEADERS &&
+				!strcmp(CONTENTS_HEADERS_get_chunk_method(l->data)->str,
+						CHUNK_METHOD_DRAINED)) {
+			meta2_filter_ctx_set_error(ctx, NEWERROR(CODE_CONTENT_DRAINED,
+					"The content is drained"));
+			goto cleanup;
+		}
+	}
+
 	_on_bean_ctx_send_list(obc);
 	rc = FILTER_OK;
 
@@ -271,6 +334,28 @@ cleanup:
 	return rc;
 }
 
+int
+meta2_filter_action_drain_content(struct gridd_filter_ctx_s *ctx,
+		struct gridd_reply_ctx_s *reply)
+{
+	GError *e = NULL;
+	struct oio_url_s *url = meta2_filter_ctx_get_url(ctx);
+	struct meta2_backend_s *m2b = meta2_filter_ctx_get_backend(ctx);
+	struct on_bean_ctx_s *obc = _on_bean_ctx_init(ctx, reply);
+
+	e = meta2_backend_drain_content(m2b, url, _bean_list_cb, &obc->l);
+	if (e != NULL) {
+		meta2_filter_ctx_set_error(ctx, e);
+		_on_bean_ctx_clean(obc);
+		return FILTER_KO;
+	}
+
+	_m2b_notify_beans(m2b, url, obc->l, "content.drained", TRUE);
+	_on_bean_ctx_send_list(obc);
+	_on_bean_ctx_clean(obc);
+	return FILTER_OK;
+}
+
 int
 meta2_filter_action_delete_content(struct gridd_filter_ctx_s *ctx,
 		struct gridd_reply_ctx_s *reply)
@@ -290,7 +375,7 @@ meta2_filter_action_delete_content(struct gridd_filter_ctx_s *ctx,
 		return FILTER_KO;
 	}
 
-	_m2b_notify_beans(m2b, url, obc->l, "content.deleted");
+	_m2b_notify_beans(m2b, url, obc->l, "content.deleted", TRUE);
 	_on_bean_ctx_send_list(obc);
 	_on_bean_ctx_clean(obc);
 	return FILTER_OK;
@@ -319,9 +404,9 @@ meta2_filter_action_truncate_content(struct gridd_filter_ctx_s *ctx,
 	}
 
 	if (deleted)
-		_m2b_notify_beans(m2b, url, deleted, "content.deleted");
+		_m2b_notify_beans(m2b, url, deleted, "content.deleted", TRUE);
 	if (added)
-		_m2b_notify_beans(m2b, url, added, "content.new");
+		_m2b_notify_beans(m2b, url, added, "content.new", FALSE);
 
 	_bean_cleanl2(added);
 	_bean_cleanl2(deleted);
@@ -508,7 +593,7 @@ meta2_filter_action_touch_content(struct gridd_filter_ctx_s *ctx,
 			m2b, url, M2V2_FLAG_ALLPROPS|M2V2_FLAG_HEADERS,
 			_bean_list_cb, &beans);
     if (!err) {
-		_m2b_notify_beans(m2b, url, beans, "content.new");
+		_m2b_notify_beans(m2b, url, beans, "content.new", FALSE);
 		_bean_cleanl2(beans);
 		return FILTER_OK;
 	}
