@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta2v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -26,6 +26,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <metautils/lib/metautils.h>
 #include <metautils/lib/metacomm.h>
 #include <metautils/lib/metautils_strings.h>
+#include <metautils/lib/common_variables.h>
 #include <server/transport_gridd.h>
 #include <server/gridd_dispatcher_filters.h>
 #include <events/oio_events_queue.h>
@@ -34,7 +35,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <meta2v2/meta2_filters.h>
 #include <meta2v2/meta2_backend_internals.h>
 #include <meta2v2/meta2_bean.h>
-#include <meta2v2/meta2v2_remote.h>
 #include <meta2v2/generic.h>
 #include <meta2v2/autogen.h>
 #include <cluster/lib/gridcluster.h>
@@ -105,7 +105,6 @@ meta2_filter_check_ns_is_master(struct gridd_filter_ctx_s *ctx,
 {
 	(void) reply;
 	TRACE_FILTER();
-	struct meta2_backend_s *backend = meta2_filter_ctx_get_backend(ctx);
 
 	const char *admin = meta2_filter_ctx_get_param(ctx, NAME_MSGKEY_ADMIN_COMMAND);
 	if (oio_str_parse_bool(admin, FALSE)) {
@@ -113,17 +112,7 @@ meta2_filter_check_ns_is_master(struct gridd_filter_ctx_s *ctx,
 		return FILTER_OK;
 	}
 
-	gboolean master = TRUE;
-
-	g_mutex_lock(&backend->nsinfo_lock);
-	gchar *state = namespace_get_state(backend->nsinfo);
-	g_mutex_unlock(&backend->nsinfo_lock);
-	if (state) {
-		master = (0 != g_strcmp0(state, NS_STATE_VALUE_SLAVE));
-		g_free(state);
-	}
-
-	if (master)
+	if (oio_ns_master)
 		return FILTER_OK;
 	GRID_TRACE("NS is slave, operation failed");
 	meta2_filter_ctx_set_error(ctx, SYSERR("NS slave!"));
@@ -135,20 +124,17 @@ meta2_filter_check_ns_not_wormed(struct gridd_filter_ctx_s *ctx,
 		struct gridd_reply_ctx_s *reply)
 {
 	(void) reply;
-	struct meta2_backend_s *backend = meta2_filter_ctx_get_backend(ctx);
-	g_mutex_lock(&backend->nsinfo_lock);
-	const gboolean wormed = namespace_in_worm_mode(backend->nsinfo);
-	g_mutex_unlock(&backend->nsinfo_lock);
-	const char *admin = meta2_filter_ctx_get_param(ctx, NAME_MSGKEY_ADMIN_COMMAND);
-	if (oio_str_parse_bool(admin, FALSE)) {
+
+	if (oio_ext_is_admin()) {
 		if (GRID_DEBUG_ENABLED())
 			GRID_DEBUG("admin mode is on");
 		return FILTER_OK;
 	}
-	if (wormed) {
+	if (oio_ns_mode_worm) {
 		if (GRID_DEBUG_ENABLED())
 			GRID_DEBUG("NS wormed!");
-		meta2_filter_ctx_set_error(ctx, SYSERR("NS wormed!"));
+		meta2_filter_ctx_set_error(
+				ctx, NEWERROR(CODE_METHOD_NOTALLOWED, "NS wormed!"));
 		return FILTER_KO;
 	}
 	TRACE_FILTER();
