@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta2v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -34,7 +34,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <meta2v2/meta2_filters.h>
 #include <meta2v2/meta2_backend_internals.h>
 #include <meta2v2/meta2_bean.h>
-#include <meta2v2/meta2v2_remote.h>
 #include <meta2v2/generic.h>
 #include <meta2v2/autogen.h>
 
@@ -77,7 +76,14 @@ meta2_filter_extract_header_url(struct gridd_filter_ctx_s *ctx,
 		struct gridd_reply_ctx_s *reply)
 {
 	TRACE_FILTER();
+	const gchar *err = NULL;
 	struct oio_url_s *url = metautils_message_extract_url (reply->request);
+    if (!oio_url_check(url, NULL, &err)) {
+		meta2_filter_ctx_set_error(ctx, NEWERROR(CODE_BAD_REQUEST,
+					"Invalid request, Invalid %s", err));
+		oio_url_pclean(&url);
+		return FILTER_KO;
+    }
 	meta2_filter_ctx_set_url(ctx, url);
 	return FILTER_OK;
 }
@@ -268,6 +274,8 @@ meta2_filter_extract_admin(struct gridd_filter_ctx_s *ctx,
 	gchar buf[1024];
 	TRACE_FILTER();
 	EXTRACT_OPT(NAME_MSGKEY_ADMIN_COMMAND);
+	const char *admin = meta2_filter_ctx_get_param(ctx, NAME_MSGKEY_ADMIN_COMMAND);
+	oio_ext_set_admin(oio_str_parse_bool(admin, FALSE));
 	return FILTER_OK;
 }
 
