@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta2v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -30,7 +30,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <server/gridd_dispatcher_filters.h>
 
 #include <meta2v2/meta2_macros.h>
-#include <meta2v2/meta2v2_remote.h>
 #include <meta2v2/meta2_gridd_dispatcher.h>
 #include <meta2v2/meta2_filters.h>
 #include <meta2v2/meta2_filter_context.h>
@@ -47,6 +46,11 @@ meta2_dispatch_all(struct gridd_reply_ctx_s *reply,
 	guint loop;
 
 	fl = (gridd_filter*)hdata;
+
+	/* Another thread may have changed the thread-local storage,
+	 * ensure the admin-mode flag is clean. */
+	oio_ext_set_admin(FALSE);
+
 	ctx = meta2_filter_ctx_new();
 	meta2_filter_ctx_set_backend(ctx, (struct meta2_backend_s *) gdata);
 
@@ -75,6 +79,7 @@ meta2_dispatch_all(struct gridd_reply_ctx_s *reply,
 	}
 
 	meta2_filter_ctx_clean(ctx);
+	oio_ext_set_admin(FALSE);
 	return TRUE;
 }
 
@@ -118,6 +123,7 @@ static gridd_filter M2V2_EMPTY_FILTERS[] =
 {
 	meta2_filter_extract_header_url,
 	meta2_filter_extract_header_localflag,
+	meta2_filter_extract_admin,
 	meta2_filter_fill_subject,
 	meta2_filter_check_url_cid,
 	meta2_filter_check_backend,
@@ -149,6 +155,7 @@ static gridd_filter M2V2_PURGE_FILTERS[] =
 	meta2_filter_check_backend,
 	meta2_filter_check_ns_name,
 	meta2_filter_check_ns_is_master,
+	meta2_filter_check_ns_not_wormed,
 	meta2_filter_action_purge_container,
 	NULL
 };
@@ -156,6 +163,7 @@ static gridd_filter M2V2_PURGE_FILTERS[] =
 static gridd_filter M2V2_DEDUP_FILTERS[] =
 {
 	meta2_filter_extract_header_url,
+	meta2_filter_extract_admin,
 	meta2_filter_fill_subject,
 	meta2_filter_check_url_cid,
 	meta2_filter_check_backend,
@@ -168,10 +176,12 @@ static gridd_filter M2V2_DEDUP_FILTERS[] =
 static gridd_filter M2V2_FLUSH_FILTERS[] =
 {
 	meta2_filter_extract_header_url,
+	meta2_filter_extract_admin,
 	meta2_filter_fill_subject,
 	meta2_filter_check_url_cid,
 	meta2_filter_check_backend,
 	meta2_filter_check_ns_name,
+	meta2_filter_check_ns_not_wormed,
 	meta2_filter_action_flush_container,
 	meta2_filter_reply_success,
 	NULL
@@ -222,6 +232,7 @@ static gridd_filter M2V2_LINK_FILTERS[] =
 	meta2_filter_extract_header_url,
 	meta2_filter_extract_header_flags32,
 	meta2_filter_extract_list_params,
+	meta2_filter_extract_admin,
 	meta2_filter_fill_subject,
 	meta2_filter_check_url_cid,
 	meta2_filter_check_backend,
@@ -252,6 +263,7 @@ static gridd_filter M2V2_BEANS_FILTER[] =
 	meta2_filter_extract_header_storage_policy,
 	meta2_filter_extract_header_string_size,
 	meta2_filter_extract_header_localflag,
+	meta2_filter_extract_admin,
 	meta2_filter_fill_subject,
 	meta2_filter_check_url_cid,
 	meta2_filter_check_backend,
@@ -275,6 +287,7 @@ static gridd_filter M2V2_PUT_FILTERS[] =
 	meta2_filter_check_ns_is_master,
 	meta2_filter_check_events_not_stalled,
 	meta2_filter_extract_body_beans,
+	meta2_filter_action_check_content,
 	meta2_filter_action_put_content,
 	NULL
 };
@@ -308,6 +321,21 @@ static gridd_filter M2V2_GET_FILTERS[] =
 	NULL
 };
 
+static gridd_filter M2V2_DRAIN_FILTERS[] =
+{
+	meta2_filter_extract_header_url,
+	meta2_filter_extract_admin,
+	meta2_filter_fill_subject,
+	meta2_filter_check_url_cid,
+	meta2_filter_check_backend,
+	meta2_filter_check_ns_name,
+	meta2_filter_check_ns_is_master,
+	meta2_filter_check_ns_not_wormed,
+	meta2_filter_check_events_not_stalled,
+	meta2_filter_action_drain_content,
+	NULL,
+};
+
 static gridd_filter M2V2_DELETE_FILTERS[] =
 {
 	meta2_filter_extract_header_url,
@@ -372,6 +400,7 @@ static gridd_filter M2V2_PROPGET_FILTERS[] =
 static gridd_filter M2V2_PROPDEL_FILTERS[] =
 {
 	meta2_filter_extract_header_url,
+	meta2_filter_extract_admin,
 	meta2_filter_fill_subject,
 	meta2_filter_check_url_cid,
 	meta2_filter_check_backend,
@@ -487,6 +516,7 @@ meta2_gridd_get_v2_requests(void)
 		{NAME_MSGNAME_M2V2_PUT,     (hook) meta2_dispatch_all, M2V2_PUT_FILTERS},
 		{NAME_MSGNAME_M2V2_LINK,    (hook) meta2_dispatch_all, M2V2_LINK_FILTERS},
 		{NAME_MSGNAME_M2V2_APPEND,  (hook) meta2_dispatch_all, M2V2_APPEND_FILTERS},
+		{NAME_MSGNAME_M2V2_DRAIN,   (hook) meta2_dispatch_all, M2V2_DRAIN_FILTERS},
 		{NAME_MSGNAME_M2V2_DEL,     (hook) meta2_dispatch_all, M2V2_DELETE_FILTERS},
 		{NAME_MSGNAME_M2V2_TRUNC,   (hook) meta2_dispatch_all, M2V2_TRUNCATE_FILTERS},
 
