@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta2v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -30,6 +30,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <sqliterepo/sqliterepo.h>
 #include <sqliterepo/replication_dispatcher.h>
 #include <sqliterepo/upgrade.h>
+#include <meta2v2/meta2_variables.h>
 #include <meta2v2/meta2_gridd_dispatcher.h>
 #include <meta2v2/meta2_backend.h>
 #include <meta2v2/meta2_backend_internals.h>
@@ -46,101 +47,6 @@ _task_reconfigure_m2(gpointer p)
 	meta2_backend_configure_nsinfo(m2, PSRV(p)->nsinfo);
 }
 
-static gchar **
-filter_services(struct sqlx_service_s *ss,
-		gchar **s, gint64 seq, const gchar *type)
-{
-	gboolean matched = FALSE;
-	GPtrArray *tmp = g_ptr_array_new();
-	(void) seq;
-	for (; *s ;s++) {
-		struct meta1_service_url_s *u = meta1_unpack_url(*s);
-		if (0 == strcmp(type, u->srvtype)) {
-			if (!g_ascii_strcasecmp(u->host, ss->url->str))
-				matched = TRUE;
-			else
-				g_ptr_array_add(tmp, g_strdup(u->host));
-		}
-		meta1_service_url_clean(u);
-	}
-
-	gchar **out = (gchar**)metautils_gpa_to_array (tmp, TRUE);
-	if (matched)
-		return out;
-	g_strfreev (out);
-	return NULL;
-}
-
-static gchar **
-filter_services_and_clean(struct sqlx_service_s *ss,
-		gchar **src, gint64 seq, const gchar *type)
-{
-	if (!src)
-		return NULL;
-	gchar **result = filter_services(ss, src, seq, type);
-	g_strfreev(src);
-	return result;
-}
-
-static GError *
-_get_peers(struct sqlx_service_s *ss, const struct sqlx_name_s *n,
-		gboolean nocache, gchar ***result)
-{
-	EXTRA_ASSERT(ss != NULL);
-	EXTRA_ASSERT(result != NULL);
-
-	gint retries = 1;
-	gchar **peers = NULL;
-	GError *err = NULL;
-
-	gint64 seq = 1;
-	struct oio_url_s *u = oio_url_empty ();
-	oio_url_set(u, OIOURL_NS, ss->ns_name);
-	if (!sqlx_name_extract (n, u, NAME_SRVTYPE_META2, &seq)) {
-		oio_url_pclean (&u);
-		return BADREQ("Invalid type name: '%s'", n->type);
-	}
-
-retry:
-	if (nocache) {
-		hc_decache_reference_service(ss->resolver, u, n->type);
-		if (!result) {
-			oio_url_pclean (&u);
-			return NULL;
-		}
-	}
-
-	peers = NULL;
-	err = hc_resolve_reference_service(ss->resolver, u, n->type, &peers);
-
-	if (NULL != err) {
-		g_prefix_error(&err, "Peer resolution error: ");
-		oio_url_clean(u);
-		return err;
-	}
-
-	gchar **out = filter_services_and_clean(ss, peers, seq, n->type);
-
-	if (!out) {
-		if (retries-- > 0) {
-			nocache = TRUE;
-			goto retry;
-		}
-		err = NEWERROR(CODE_CONTAINER_NOTFOUND, "Base not managed");
-	}
-
-	if (err) {
-		if (out)
-			g_strfreev (out);
-		*result = NULL;
-	} else {
-		*result = out;
-	}
-
-	oio_url_clean(u);
-	return err;
-}
-
 static void
 meta2_on_close(struct sqlx_sqlite3_s *sq3, gboolean deleted, gpointer cb_data)
 {
@@ -152,7 +58,8 @@ meta2_on_close(struct sqlx_sqlite3_s *sq3, gboolean deleted, gpointer cb_data)
 
 	struct oio_url_s *u = oio_url_empty ();
 	oio_url_set (u, OIOURL_NS, PSRV(cb_data)->ns_name);
-	if (!sqlx_name_extract ((struct sqlx_name_s*)&sq3->name, u, NAME_SRVTYPE_META2, &seq)) {
+	NAME2CONST(n, sq3->name);
+	if (!sqlx_name_extract (&n, u, NAME_SRVTYPE_META2, &seq)) {
 		GRID_WARN("Invalid base name [%s]", sq3->name.base);
 		return;
 	}
@@ -189,22 +96,20 @@ _post_config(struct sqlx_service_s *ss)
 	sqlx_repository_configure_change_callback(ss->repository,
 			(sqlx_repo_change_hook)meta2_backend_change_callback, m2);
 
+	hc_resolver_configure(ss->resolver, HC_RESOLVER_DECACHEM0);
+
 	/* Register meta2 requests handlers */
 	transport_gridd_dispatcher_add_requests(ss->dispatcher,
 			meta2_gridd_get_v2_requests(), m2);
 
 	/* Register few meta2 tasks */
+	grid_task_queue_register(ss->gtq_reload, meta2_reload_nsinfo_period,
+			_task_reconfigure_m2, NULL, ss);
 	grid_task_queue_register(ss->gtq_reload, 1,
 			(GDestroyNotify)sqlx_task_reload_lb, NULL, ss);
-	grid_task_queue_register(ss->gtq_reload, 5,
-			_task_reconfigure_m2, NULL, ss);
 
 	m2->notifier = ss->events_queue;
 
-	/* Make the meta2 avoid meta services with known problems */
-	oio_cache_avoid_on_error = oio_cfg_get_bool (
-			ss->ns_name, OIO_CFG_AVOID_BADSRV, FALSE);
-
 	return TRUE;
 }
 
@@ -215,7 +120,7 @@ main(int argc, char **argv)
 		NAME_SRVTYPE_META2, "m2v2",
 		"el/" NAME_SRVTYPE_META2, 2, 2,
 		schema, 1, 3,
-		_get_peers, _post_config, NULL
+		sqlx_service_resolve_peers, _post_config, NULL
 	};
 
 	int rc = sqlite_service_main (argc, argv, &cfg);
