@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta2v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -25,10 +25,10 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <meta2v2/meta2_backend_internals.h>
 #include <meta2v2/meta2_utils.h>
 #include <meta2v2/meta2_macros.h>
-#include <meta2v2/meta2v2_remote.h>
 #include <meta2v2/meta2_dedup_utils.h>
 #include <meta2v2/generic.h>
 #include <meta2v2/autogen.h>
+#include <meta2v2/meta2_utils_json.h>
 
 #define RANDOM_UID(uid,uid_size) \
 	struct { guint64 now; guint32 r; guint16 pid; guint16 th; } uid; \
@@ -348,11 +348,9 @@ m2db_get_alias(struct sqlx_sqlite3_s *sq3, struct oio_url_s *u,
 {
 	/* sanity checks */
 	if (!oio_url_has(u, OIOURL_PATH) && !oio_url_has(u, OIOURL_CONTENTID))
-		return NEWERROR(CODE_BAD_REQUEST, "Missing path and content");
+		return BADREQ("Missing path and content");
 
-	GRID_TRACE("GET(%s)", oio_url_get(u, OIOURL_WHOLE));
-
-	/* query, and nevermind the snapshot */
+	/* query */
 	GError *err = NULL;
 	const gchar *sql = NULL;
 	GVariant *params[3] = {NULL, NULL, NULL};
@@ -366,7 +364,9 @@ m2db_get_alias(struct sqlx_sqlite3_s *sq3, struct oio_url_s *u,
 		} else {
 			if (oio_url_has(u, OIOURL_VERSION)) {
 				sql = "alias = ? AND version = ? LIMIT 1";
-				params[1] = g_variant_new_int64(atoi(oio_url_get(u, OIOURL_VERSION)));
+				gint64 version =
+					g_ascii_strtoll(oio_url_get(u, OIOURL_VERSION), NULL, 10);
+				params[1] = g_variant_new_int64(version);
 			} else {
 				sql = "alias = ? ORDER BY version DESC LIMIT 1";
 			}
@@ -374,7 +374,7 @@ m2db_get_alias(struct sqlx_sqlite3_s *sq3, struct oio_url_s *u,
 	} else {
 
 		do { /* get the content-id in its binary form */
-			/* XXX TODO this code is multiplicated, there is room for factorisation */
+			/* TODO factorize this */
 			const char *h = oio_url_get(u, OIOURL_CONTENTID);
 			gsize hl = strlen(h);
 			guint8 b[hl/2];
@@ -394,7 +394,9 @@ m2db_get_alias(struct sqlx_sqlite3_s *sq3, struct oio_url_s *u,
 		} else {
 			if (oio_url_has(u, OIOURL_VERSION)) {
 				sql = "content = ? AND version = ? LIMIT 1";
-				params[1] = g_variant_new_int64(atoi(oio_url_get(u, OIOURL_VERSION)));
+				gint64 version =
+					g_ascii_strtoll(oio_url_get(u, OIOURL_VERSION), NULL, 10);
+				params[1] = g_variant_new_int64(version);
 			} else {
 				sql = "content = ? ORDER BY version DESC LIMIT 1";
 			}
@@ -418,7 +420,7 @@ m2db_get_alias(struct sqlx_sqlite3_s *sq3, struct oio_url_s *u,
 	/* recurse on headers if allowed */
 	if (!err && cb && ((flags & M2V2_FLAG_HEADERS) ||
 			!(flags & M2V2_FLAG_NORECURSION))) {
-		for (guint i=0; !err && i<tmp->len ;i++) {
+		for (guint i = 0; !err && i < tmp->len; i++) {
 			struct bean_ALIASES_s *alias = tmp->pdata[i];
 			if (!alias)
 				continue;
@@ -430,14 +432,14 @@ m2db_get_alias(struct sqlx_sqlite3_s *sq3, struct oio_url_s *u,
 
 	/* recurse on properties if allowed */
 	if (!err && cb && !(flags & M2V2_FLAG_NOPROPS)) {
-		for (guint i=0; !err && i<tmp->len ;i++) {
+		for (guint i = 0; !err && i < tmp->len; i++) {
 			struct bean_ALIASES_s *alias = tmp->pdata[i];
 			if (!alias)
 				continue;
 			GPtrArray *props = g_ptr_array_new();
 			err = _db_get_FK_by_name_buffered(alias, "properties", sq3->db, props);
 			if (!err) {
-				for (guint j=0; j<props->len ;++j) {
+				for (guint j = 0; j < props->len; ++j) {
 					cb(u0, props->pdata[j]);
 					props->pdata[j] = NULL;
 				}
@@ -448,7 +450,7 @@ m2db_get_alias(struct sqlx_sqlite3_s *sq3, struct oio_url_s *u,
 
 	/* eventually manage the aliases */
 	if (!err && cb) {
-		for (guint i=0; i<tmp->len ;i++) {
+		for (guint i = 0; i < tmp->len; i++) {
 			struct bean_ALIASES_s *alias = tmp->pdata[i];
 			if (!alias)
 				continue;
@@ -525,50 +527,50 @@ _list_params_to_sql_clause(struct list_params_s *lp, GString *clause,
 		GSList *headers)
 {
 	void lazy_and () {
-		if (clause->len > 0) g_string_append(clause, " AND");
+		if (clause->len > 0) g_string_append_static(clause, " AND");
 	}
 	GPtrArray *params = g_ptr_array_new ();
 
 	if (lp->marker_start) {
 		lazy_and();
-		g_string_append (clause, " alias > ?");
+		g_string_append_static (clause, " alias > ?");
 		g_ptr_array_add (params, g_variant_new_string (lp->marker_start));
 	} else if (lp->prefix) {
 		lazy_and();
-		g_string_append (clause, " alias >= ?");
+		g_string_append_static (clause, " alias >= ?");
 		g_ptr_array_add (params, g_variant_new_string (lp->prefix));
 	}
 
 	if (lp->marker_end) {
 		lazy_and();
-		g_string_append (clause, " alias < ?");
+		g_string_append_static (clause, " alias < ?");
 		g_ptr_array_add (params, g_variant_new_string (lp->marker_end));
 	}
 
 	if (headers) {
 		lazy_and();
 		if (headers->next) {
-			g_string_append (clause, " content IN (");
-			for (GSList *l=headers; l; l=l->next) {
+			g_string_append_static (clause, " content IN (");
+			for (GSList *l = headers; l; l = l->next) {
 				if (l != headers)
 					g_string_append_c (clause, ',');
 				g_string_append_c (clause, '?');
 				GByteArray *gba = CONTENTS_HEADERS_get_id (l->data);
 				g_ptr_array_add (params, _gba_to_gvariant (gba));
 			}
-			g_string_append (clause, ")");
+			g_string_append_c (clause, ')');
 		} else {
-			g_string_append (clause, " content = ?");
+			g_string_append_static (clause, " content = ?");
 			GByteArray *gba = CONTENTS_HEADERS_get_id (headers->data);
 			g_ptr_array_add (params, _gba_to_gvariant (gba));
 		}
 	}
 
 	if (clause->len == 0)
-		clause = g_string_append(clause, " 1");
+		clause = g_string_append_static (clause, " 1");
 
 	if (!lp->flag_allversion || lp->maxkeys>0 || lp->marker_start || lp->marker_end)
-		g_string_append(clause, " ORDER BY alias ASC, version ASC");
+		g_string_append_static(clause, " ORDER BY alias ASC, version ASC");
 
 	if (lp->maxkeys > 0)
 		g_string_append_printf(clause, " LIMIT %"G_GINT64_FORMAT, lp->maxkeys);
@@ -623,7 +625,7 @@ m2db_list_aliases(struct sqlx_sqlite3_s *sq3, struct list_params_s *lp0,
 			lp.maxkeys -= count_aliases;
 
 		// List the next items
-		GString *clause = g_string_new("");
+		GString *clause = g_string_sized_new(128);
 		GVariant **params = _list_params_to_sql_clause (&lp, clause, headers);
 		err = ALIASES_load(sq3->db, clause->str, params, _bean_buffer_cb, tmp);
 		metautils_gvariant_unrefv (params);
@@ -634,7 +636,7 @@ m2db_list_aliases(struct sqlx_sqlite3_s *sq3, struct list_params_s *lp0,
 
 		metautils_gpa_reverse (tmp);
 
-		for (guint i=tmp->len; i>0 ;i--) {
+		for (guint i = tmp->len; i > 0; i--) {
 			struct bean_ALIASES_s *alias = tmp->pdata[i-1];
 			const gchar *name = ALIASES_get_alias(alias)->str;
 
@@ -668,12 +670,14 @@ label_ok:
 	aliases = g_slist_reverse(aliases);
 	for (GSList *l = aliases; l; l = l->next) {
 		struct bean_ALIASES_s *alias = l->data;
-		if (lp.flag_headers)
-			_load_fk_by_name(sq3, alias, "image", cb, u);
-		if (lp.flag_properties)
-			_load_fk_by_name(sq3, alias, "properties", cb, u);
-		cb(u, alias);
-		l->data = NULL;
+		if (!lp.flag_nodeleted || !ALIASES_get_deleted(alias) || lp.flag_allversion){
+			if (lp.flag_headers)
+				_load_fk_by_name(sq3, alias, "image", cb, u);
+			if (lp.flag_properties)
+				_load_fk_by_name(sq3, alias, "properties", cb, u);
+			cb(u, alias);
+			l->data = NULL;
+		}
 	}
 
 label_error:
@@ -693,7 +697,7 @@ m2db_get_properties(struct sqlx_sqlite3_s *sq3, struct oio_url_s *url,
 	if (err)
 		_bean_cleanv2(tmp);
 	else {
-		for (guint i=0; i<tmp->len; ++i)
+		for (guint i = 0; i < tmp->len; ++i)
 			cb(u0, tmp->pdata[i]);
 		g_ptr_array_free(tmp, TRUE);
 	}
@@ -724,7 +728,7 @@ m2db_set_properties(struct sqlx_sqlite3_s *sq3, struct oio_url_s *url,
 		metautils_gvariant_unrefv (params);
 	}
 
-	for (; !err && beans ;beans=beans->next) {
+	for (; !err && beans; beans = beans->next) {
 		struct bean_PROPERTIES_s *prop = beans->data;
 		if (DESCR(prop) != &descr_struct_PROPERTIES)
 			continue;
@@ -757,13 +761,13 @@ m2db_del_properties(struct sqlx_sqlite3_s *sq3, struct oio_url_s *url, gchar **n
 	tmp = g_ptr_array_new();
 	err = m2db_get_properties(sq3, url, _bean_buffer_cb, tmp);
 	if (!err) {
-		for (guint i=0; !err && i<tmp->len ;++i) {
+		for (guint i = 0; !err && i < tmp->len; ++i) {
 			struct bean_PROPERTIES_s *bean = tmp->pdata[i];
 			if (DESCR(bean) != &descr_struct_PROPERTIES)
 				continue;
 			if (namev && *namev) {
 				/* explicit properties to be deleted */
-				for (gchar **p=namev; *p ;++p) {
+				for (gchar **p = namev; *p; ++p) {
 					if (!strcmp(*p, PROPERTIES_get_key(bean)->str))
 						_db_delete_bean (sq3->db, bean);
 				}
@@ -796,7 +800,7 @@ m2db_purge_alias_being_deleted(struct sqlx_sqlite3_s *sq3, GSList *beans,
 
 	// First, mark the ALIAS for deletion, and check each header
 	gboolean header_deleted = FALSE;
-	for (GSList *l = beans; !err && l ;l=l->next) {
+	for (GSList *l = beans; !err && l; l = l->next) {
 		gpointer bean = l->data;
 		if (&descr_struct_ALIASES == DESCR(bean))
 			deleted = g_slist_prepend(deleted, bean);
@@ -810,7 +814,7 @@ m2db_purge_alias_being_deleted(struct sqlx_sqlite3_s *sq3, GSList *beans,
 
 	// Then, only if the HEADER is deleted, we can remove all the CHUNKS
 	if (header_deleted) {
-		for (GSList *l = beans; !err && l ;l=l->next) {
+		for (GSList *l = beans; !err && l; l = l->next) {
 			gpointer bean = l->data;
 			if (&descr_struct_CHUNKS == DESCR(bean))
 				deleted = g_slist_prepend(deleted, bean);
@@ -840,7 +844,7 @@ _real_delete(struct sqlx_sqlite3_s *sq3, GSList *beans, GSList **deleted_beans)
 
 	// Now really delete the beans, and notify them.
 	// But do not notify ALIAS already marked deleted (they have already been notified)
-	for (GSList *l = deleted; l ;l=l->next) {
+	for (GSList *l = deleted; l; l = l->next) {
 		if (DESCR(l->data) != &descr_struct_ALIASES || !ALIASES_get_deleted(l->data))
 			*deleted_beans = g_slist_prepend (*deleted_beans, l->data);
 		GError *e = _db_delete_bean(sq3->db, l->data);
@@ -852,7 +856,7 @@ _real_delete(struct sqlx_sqlite3_s *sq3, GSList *beans, GSList **deleted_beans)
 
 	// recompute container size and object count
 	gint64 obj_count = m2db_get_obj_count(sq3);
-	for (GSList *l = deleted; l ;l=l->next) {
+	for (GSList *l = deleted; l; l = l->next) {
 		if (&descr_struct_CONTENTS_HEADERS == DESCR(l->data)) {
 			gint64 decrement = CONTENTS_HEADERS_get_size(l->data);
 			gint64 size = m2db_get_size(sq3) - decrement;
@@ -869,6 +873,23 @@ _real_delete(struct sqlx_sqlite3_s *sq3, GSList *beans, GSList **deleted_beans)
 	return NULL;
 }
 
+GError *
+m2db_drain_content(struct sqlx_sqlite3_s *sq3, struct oio_url_s *url,
+		m2_onbean_cb cb, gpointer u0)
+{
+	GError *err = NULL;
+	err = m2db_get_alias(sq3, url, M2V2_FLAG_NOPROPS|M2V2_FLAG_HEADERS, cb, u0);
+	for (GSList *l = *(GSList **)u0; l; l = l->next) {
+		if (DESCR(l->data) == &descr_struct_CHUNKS) {
+			_db_delete_bean(sq3->db, l->data);
+		} else if (DESCR(l->data) == &descr_struct_CONTENTS_HEADERS) {
+			CONTENTS_HEADERS_set2_chunk_method(l->data, CHUNK_METHOD_DRAINED);
+			_db_save_bean(sq3->db, l->data);
+		}
+	}
+	return err;
+}
+
 GError*
 m2db_delete_alias(struct sqlx_sqlite3_s *sq3, gint64 max_versions,
 		struct oio_url_s *url, m2_onbean_cb cb, gpointer u0)
@@ -995,6 +1016,12 @@ m2db_truncate_content(struct sqlx_sqlite3_s *sq3, struct oio_url_s *url,
 		goto cleanup;
 	EXTRA_ASSERT(content.properties == NULL);
 
+	if (!strcmp(CONTENTS_HEADERS_get_chunk_method(content.header)->str,
+			CHUNK_METHOD_DRAINED)){
+		err = NEWERROR(CODE_CONTENT_DRAINED, "The content is drained");
+		goto cleanup;
+	}
+
 	if (truncate_size > CONTENTS_HEADERS_get_size(content.header)) {
 		err = BADREQ("truncate operation cannot grow contents");
 		goto cleanup;
@@ -1077,16 +1104,16 @@ cleanup:
 
 #if 0
 static void _patch_beans_with_stgpol(GSList *beans, const char *stgpol) {
-    for (GSList *l=beans; l ;l=l->next) {
-        gpointer bean = l->data;
-        if (DESCR(bean) == &descr_struct_CONTENTS_HEADERS) {
-            CONTENTS_HEADERS_set2_policy(bean, stgpol);
-        }
-    }
+	for (GSList *l = beans; l; l = l->next) {
+		gpointer bean = l->data;
+		if (DESCR(bean) == &descr_struct_CONTENTS_HEADERS) {
+			CONTENTS_HEADERS_set2_policy(bean, stgpol);
+		}
+	}
 }
 
 static gchar* _fetch_content_policy (GSList *beans) {
-	for (GSList *l=beans; l ;l=l->next) {
+	for (GSList *l = beans; l; l = l->next) {
 		gpointer bean = l->data;
 		if (DESCR(bean) == &descr_struct_CONTENTS_HEADERS) {
 			GString *gs = CONTENTS_HEADERS_get_policy(bean);
@@ -1098,99 +1125,99 @@ static gchar* _fetch_content_policy (GSList *beans) {
 #endif
 
 static void _patch_beans_with_version (GSList *beans, gint64 version) {
-    for (GSList *l=beans; l ;l=l->next) {
-        gpointer bean = l->data;
-        if (DESCR(bean) == &descr_struct_ALIASES) {
-            ALIASES_set_version(bean, version);
-        } else if (DESCR(bean) == &descr_struct_PROPERTIES) {
-            PROPERTIES_set_version (bean, version);
-        }
-    }
+	for (GSList *l = beans; l; l = l->next) {
+		gpointer bean = l->data;
+		if (DESCR(bean) == &descr_struct_ALIASES) {
+			ALIASES_set_version(bean, version);
+		} else if (DESCR(bean) == &descr_struct_PROPERTIES) {
+			PROPERTIES_set_version (bean, version);
+		}
+	}
 }
 
 static void _patch_beans_with_contentid (GSList *beans,
-        const guint8 *uid, gsize len) {
-    for (GSList *l=beans; l ;l=l->next) {
-        gpointer bean = l->data;
+		const guint8 *uid, gsize len) {
+	for (GSList *l = beans; l; l = l->next) {
+		gpointer bean = l->data;
 
-        if (DESCR(bean) == &descr_struct_ALIASES) {
-            ALIASES_set2_content(bean, uid, len);
-        } else if (DESCR(bean) == &descr_struct_CONTENTS_HEADERS) {
-            CONTENTS_HEADERS_set2_id(bean, uid, len);
-        } else if (DESCR(bean) == &descr_struct_CHUNKS) {
-            CHUNKS_set2_content(bean, uid, len);
-        }
-    }
+		if (DESCR(bean) == &descr_struct_ALIASES) {
+			ALIASES_set2_content(bean, uid, len);
+		} else if (DESCR(bean) == &descr_struct_CONTENTS_HEADERS) {
+			CONTENTS_HEADERS_set2_id(bean, uid, len);
+		} else if (DESCR(bean) == &descr_struct_CHUNKS) {
+			CHUNKS_set2_content(bean, uid, len);
+		}
+	}
 
 }
 
 static void _patch_beans_with_time (GSList *beans, gint64 now) {
-    for (GSList *l=beans; l ;l=l->next) {
-        gpointer bean = l->data;
-        if (DESCR(bean) == &descr_struct_ALIASES) {
-            ALIASES_set_ctime(bean, now);
-            ALIASES_set_mtime(bean, now);
-        } else if (DESCR(bean) == &descr_struct_CONTENTS_HEADERS) {
-            CONTENTS_HEADERS_set_ctime(bean, now);
-            CONTENTS_HEADERS_set_mtime(bean, now);
-        } else if (DESCR(bean) == &descr_struct_CHUNKS) {
-            CHUNKS_set_ctime(bean, now);
-        }
-    }
+	for (GSList *l = beans; l; l = l->next) {
+		gpointer bean = l->data;
+		if (DESCR(bean) == &descr_struct_ALIASES) {
+			ALIASES_set_ctime(bean, now);
+			ALIASES_set_mtime(bean, now);
+		} else if (DESCR(bean) == &descr_struct_CONTENTS_HEADERS) {
+			CONTENTS_HEADERS_set_ctime(bean, now);
+			CONTENTS_HEADERS_set_mtime(bean, now);
+		} else if (DESCR(bean) == &descr_struct_CHUNKS) {
+			CHUNKS_set_ctime(bean, now);
+		}
+	}
 }
 
 static void _patch_beans_defaults (GSList *beans) {
-    #define lazy_set_str(T,B,F,V) do { \
-	    GString *gs = T##_get_##F(B); \
-	    if (!gs || !gs->str || !gs->len) T##_set2_##F(B,V); \
-    } while (0)
-    for (GSList *l=beans; l ;l=l->next) {
-        gpointer bean = l->data;
-        if (!l->data)
-            continue;
-        if (DESCR(bean) == &descr_struct_ALIASES) {
-            ALIASES_set_deleted(bean, FALSE);
-        } else if (DESCR(bean) == &descr_struct_CONTENTS_HEADERS) {
-            lazy_set_str (CONTENTS_HEADERS, bean, chunk_method, OIO_DEFAULT_CHUNKMETHOD);
-            lazy_set_str (CONTENTS_HEADERS, bean, mime_type, OIO_DEFAULT_MIMETYPE);
-            lazy_set_str (CONTENTS_HEADERS, bean, policy, OIO_DEFAULT_STGPOL);
-        }
-    }
+#define lazy_set_str(T,B,F,V) do { \
+	GString *gs = T##_get_##F(B); \
+	if (!gs || !gs->str || !gs->len) T##_set2_##F(B,V); \
+} while (0)
+for (GSList *l = beans; l; l = l->next) {
+	gpointer bean = l->data;
+	if (!l->data)
+		continue;
+	if (DESCR(bean) == &descr_struct_ALIASES) {
+		ALIASES_set_deleted(bean, FALSE);
+	} else if (DESCR(bean) == &descr_struct_CONTENTS_HEADERS) {
+		lazy_set_str(CONTENTS_HEADERS, bean, chunk_method, OIO_DEFAULT_CHUNKMETHOD);
+		lazy_set_str(CONTENTS_HEADERS, bean, mime_type, OIO_DEFAULT_MIMETYPE);
+		lazy_set_str(CONTENTS_HEADERS, bean, policy, OIO_DEFAULT_STGPOL);
+	}
+}
 }
 
 static gint64 _fetch_alias_version (GSList *beans) {
-    for (GSList *l=beans; l ;l=l->next) {
-        gpointer bean = l->data;
-        if (DESCR(bean) == &descr_struct_ALIASES)
-            return ALIASES_get_version(bean);
-    }
-    return -1;
+	for (GSList *l = beans; l; l = l->next) {
+		gpointer bean = l->data;
+		if (DESCR(bean) == &descr_struct_ALIASES)
+			return ALIASES_get_version(bean);
+	}
+	return -1;
 }
 
 static gint64 _fetch_content_size (GSList *beans) {
-    for (GSList *l=beans; l ;l=l->next) {
-        gpointer bean = l->data;
-        if (DESCR(bean) == &descr_struct_CONTENTS_HEADERS)
-            return CONTENTS_HEADERS_get_size(bean);
-    }
-    return 0;
+	for (GSList *l = beans; l; l = l->next) {
+		gpointer bean = l->data;
+		if (DESCR(bean) == &descr_struct_CONTENTS_HEADERS)
+			return CONTENTS_HEADERS_get_size(bean);
+	}
+	return 0;
 }
 
 static void _extract_chunks_sizes_positions(GSList *beans,
-            GSList **chunks, gint64 *size, GTree *positions) {
-    for (GSList *l = beans; l; l = l->next) {
-        gpointer bean = l->data;
-        if (bean && &descr_struct_CHUNKS == DESCR(bean)) {
-            struct bean_CHUNKS_s *chunk = _bean_dup(bean);
-            *chunks = g_slist_prepend(*chunks, chunk);
-            gint64 pos = g_ascii_strtoll(
-                    CHUNKS_get_position(chunk)->str, NULL, 10);
-            if (!g_tree_lookup(positions, GINT_TO_POINTER(pos))) {
-                *size += CHUNKS_get_size(chunk);
-                g_tree_insert(positions, GINT_TO_POINTER(pos), GINT_TO_POINTER(1));
-            }
-        }
-    }
+		GSList **chunks, gint64 *size, GTree *positions) {
+	for (GSList *l = beans; l; l = l->next) {
+		gpointer bean = l->data;
+		if (bean && &descr_struct_CHUNKS == DESCR(bean)) {
+			struct bean_CHUNKS_s *chunk = _bean_dup(bean);
+			*chunks = g_slist_prepend(*chunks, chunk);
+			gint64 pos = g_ascii_strtoll(
+					CHUNKS_get_position(chunk)->str, NULL, 10);
+			if (!g_tree_lookup(positions, GINT_TO_POINTER(pos))) {
+				*size += CHUNKS_get_size(chunk);
+				g_tree_insert(positions, GINT_TO_POINTER(pos), GINT_TO_POINTER(1));
+			}
+		}
+	}
 }
 
 /* PUT ---------------------------------------------------------------------- */
@@ -1205,36 +1232,36 @@ static void m2db_purge_exceeding_versions(struct sqlx_sqlite3_s *sq3,
 }
 
 static GError* m2db_real_put_alias(struct sqlx_sqlite3_s *sq3, GSList *beans,
-        m2_onbean_cb cb, gpointer cb_data) {
-    GError *err = NULL;
-    for (GSList *l=beans; !err && l ;l=l->next)
-        err = _db_save_bean(sq3->db, l->data);
-    if (!err && cb) {
-        for (GSList *l=beans; l ;l=l->next)
-            cb(cb_data, _bean_dup(l->data));
-    }
-    return err;
+		m2_onbean_cb cb, gpointer cb_data) {
+	GError *err = NULL;
+	for (GSList *l = beans; !err && l; l = l->next)
+		err = _db_save_bean(sq3->db, l->data);
+	if (!err && cb) {
+		for (GSList *l = beans; l; l = l->next)
+			cb(cb_data, _bean_dup(l->data));
+	}
+	return err;
 }
 
 /* Returns NULL if the content is absent, an explicit error indicating it is
  * present or the error that occured while checking (if any) */
 static GError* m2db_check_content_absent(struct sqlx_sqlite3_s *sq3,
-        const guint8 *uid, const gsize len) {
-    GPtrArray *tmp = g_ptr_array_new ();
-    GVariant *params[2] = {NULL, NULL};
-    GBytes *id = g_bytes_new (uid, len);
-    params[0] = _gb_to_gvariant (id);
-    GError *err = CONTENTS_HEADERS_load (sq3->db, " id = ? LIMIT 1", params,
-                                         _bean_buffer_cb, tmp);
-    metautils_gvariant_unrefv(params);
-    guint count = tmp->len;
-    _bean_cleanv2 (tmp);
-    g_bytes_unref (id);
-    if (err)
-        return err;
-    if (count)
-        return NEWERROR(CODE_CONTENT_EXISTS, "A content exists with this ID");
-    return NULL;
+		const guint8 *uid, const gsize len) {
+	GPtrArray *tmp = g_ptr_array_new ();
+	GVariant *params[2] = {NULL, NULL};
+	GBytes *id = g_bytes_new (uid, len);
+	params[0] = _gb_to_gvariant (id);
+	GError *err = CONTENTS_HEADERS_load (sq3->db, " id = ? LIMIT 1", params,
+			_bean_buffer_cb, tmp);
+	metautils_gvariant_unrefv(params);
+	guint count = tmp->len;
+	_bean_cleanv2 (tmp);
+	g_bytes_unref (id);
+	if (err)
+		return err;
+	if (count)
+		return NEWERROR(CODE_CONTENT_EXISTS, "A content exists with this ID");
+	return NULL;
 }
 
 /* TODO(jfs): return the beans added/deleted */
@@ -1255,7 +1282,7 @@ GError* m2db_force_alias(struct m2db_put_args_s *args, GSList *beans,
 	else
 		err = m2db_latest_alias(args->sq3, args->url, &latest);
 
-	if (NULL != err) {
+	if (err) {
 		if (err->code != CODE_CONTENT_NOTFOUND) {
 			g_prefix_error(&err, "Version error: ");
 			return err;
@@ -1263,6 +1290,11 @@ GError* m2db_force_alias(struct m2db_put_args_s *args, GSList *beans,
 		g_clear_error(&err);
 	}
 
+	if (latest && args->worm_mode) {
+		err = NEWERROR(CODE_CONTENT_EXISTS, "NS wormed! Cannot overwrite.");
+		goto cleanup;
+	}
+
 	_patch_beans_defaults(beans);
 	_patch_beans_with_time(beans, oio_ext_real_seconds());
 
@@ -1304,6 +1336,7 @@ GError* m2db_force_alias(struct m2db_put_args_s *args, GSList *beans,
 		m2db_set_obj_count(args->sq3, obj_count);
 	}
 
+cleanup:
 	if (latest)
 		_bean_clean(latest);
 
@@ -1364,6 +1397,12 @@ GError* m2db_update_content(struct sqlx_sqlite3_s *sq3,
 	if (err)
 		goto cleanup;
 
+	if (!strcmp(CONTENTS_HEADERS_get_chunk_method(header)->str,
+			CHUNK_METHOD_DRAINED)){
+		err = NEWERROR(CODE_CONTENT_DRAINED, "The content is drained");
+		goto cleanup;
+	}
+
 	/* Update size (in header) and mtime (in alias and header) */
 	const gint64 now = oio_ext_real_time() / G_TIME_SPAN_SECOND;
 	struct bean_CONTENTS_HEADERS_s *new_header = _bean_dup(header);
@@ -1421,58 +1460,59 @@ GError* m2db_put_alias(struct m2db_put_args_s *args, GSList *beans,
 	EXTRA_ASSERT(args->url != NULL);
 	if (!oio_url_has_fq_path(args->url))
 		return NEWERROR(CODE_BAD_REQUEST, "Missing fully qualified path");
-    if (oio_url_has(args->url, OIOURL_VERSION))
-        return NEWERROR(CODE_BAD_REQUEST, "URL version is present");
+	if (oio_url_has(args->url, OIOURL_VERSION))
+		return NEWERROR(CODE_BAD_REQUEST, "URL version is present");
 
-    /* Needed later several times, we extract now the content-id if specified */
-    gint64 version = _fetch_alias_version(beans);
-    const char *content_hexid = oio_url_get (args->url, OIOURL_CONTENTID);
-    gsize content_idlen = 0;
-    guint8 *content_id = NULL;
-    if (content_hexid) {
-        content_idlen = strlen(content_hexid) / 2;
-        content_id = g_alloca(1 + strlen(content_hexid));
-        if (!oio_str_hex2bin(content_hexid, content_id, content_idlen))
-            return BADREQ("Invalid content ID (not hexa)");
-    }
-
-    /* The content-id has been specified, we MUST check it will be UNIQUE */
-    if (content_id) {
-        err = m2db_check_content_absent(args->sq3, content_id, content_idlen);
-        if (NULL != err)
-            return err;
-    }
-
-    /* Ensure the beans are all linked to the content (with their content-id) */
-    if (content_id) {
-		_patch_beans_with_contentid(beans, content_id, content_idlen);
-	} else {
-        RANDOM_UID(uid, uid_size);
-        _patch_beans_with_contentid(beans, (guint8*)&uid, uid_size);
+	/* Needed later several times, we extract now the content-id if specified */
+	gint64 version = _fetch_alias_version(beans);
+	const char *content_hexid = oio_url_get (args->url, OIOURL_CONTENTID);
+	gsize content_idlen = 0;
+	guint8 *content_id = NULL;
+	if (content_hexid) {
+		content_idlen = strlen(content_hexid) / 2;
+		content_id = g_alloca(1 + strlen(content_hexid));
+		if (!oio_str_hex2bin(content_hexid, content_id, content_idlen))
+			return BADREQ("Invalid content ID (not hexa)");
 	}
 
-    /* needed for later: the latest content in place. Fetch it once for all */
-    if (NULL != (err = m2db_latest_alias(args->sq3, args->url, &latest))) {
+	/* The content-id has been specified, we MUST check it will be UNIQUE */
+	if (content_id) {
+		err = m2db_check_content_absent(args->sq3, content_id, content_idlen);
+		if (NULL != err)
+			return err;
+	}
+
+	/* Ensure the beans are all linked to the content (with their content-id) */
+	if (content_id) {
+		_patch_beans_with_contentid(beans, content_id, content_idlen);
+	} else {
+		RANDOM_UID(uid, uid_size);
+		_patch_beans_with_contentid(beans, (guint8*)&uid, uid_size);
+	}
+
+	/* needed for later: the latest content in place. Fetch it once for all */
+	if (NULL != (err = m2db_latest_alias(args->sq3, args->url, &latest))) {
 		if (err->code != CODE_CONTENT_NOTFOUND) {
-            g_prefix_error(&err, "Version error: ");
-            return err;
-        }
-        GRID_TRACE("Alias not yet present (1)");
+			g_prefix_error(&err, "Version error: ");
+			return err;
+		}
+		GRID_TRACE("Alias not yet present (1)");
 		g_clear_error(&err);
 	}
 
 	/* Manage the potential conflict with the latest alias in place. */
 	if (version > 0) {
+		const gint64 latest_version = latest? ALIASES_get_version(latest) : 0;
 		/* version explicitely specified */
-		if (latest && version == ALIASES_get_version(latest)) {
-			/* TODO decide if it is better to alter the version to insert though */
+		if (version == latest_version) {
 			err = NEWERROR(CODE_CONTENT_EXISTS, "Alias already saved");
-		} else {
-			/* TODO check the PUT doesn't override an alias in place that is not the latest */
+		} else if (version < latest_version) {
+			err = NEWERROR(CODE_CONTENT_PRECONDITION,
+					"New object version is older than latest version");
 		}
 	} else {
 		/* version unset, let's deduce it from the alias in place */
-		version = oio_ext_real_seconds();
+		version = oio_ext_real_time();
 		if (latest && ALIASES_get_version(latest) == version)
 			version ++;
 	}
@@ -1492,6 +1532,11 @@ GError* m2db_put_alias(struct m2db_put_args_s *args, GSList *beans,
 		}
 		else if (VERSIONS_SUSPENDED(max_versions)) {
 suspended:
+			if (args->worm_mode) {
+				err = NEWERROR(CODE_CONTENT_EXISTS,
+						"NS wormed! Cannot overwrite.");
+			}
+
 			// JFS: do not alter the size to manage the alias being removed,
 			// this will be done by the real purge of the latest.
 			purge_latest = TRUE;
@@ -1503,19 +1548,19 @@ suspended:
 
 	/* Perform the insertion now and patch the URL with the version */
 	if (!err) {
-        /* Patch the beans, before inserting */
-        _patch_beans_defaults(beans);
-        _patch_beans_with_time(beans, oio_ext_real_seconds());
-        _patch_beans_with_version(beans, version);
+		/* Patch the beans, before inserting */
+		_patch_beans_defaults(beans);
+		_patch_beans_with_time(beans, oio_ext_real_seconds());
+		_patch_beans_with_version(beans, version);
 
-        err = m2db_real_put_alias(args->sq3, beans,
+		err = m2db_real_put_alias(args->sq3, beans,
 				out_added ? _bean_list_cb : NULL, out_added);
 	}
-    if (!err) {
-        m2db_set_size(args->sq3,
-                      m2db_get_size(args->sq3) + _fetch_content_size(beans));
+	if (!err) {
+		m2db_set_size(args->sq3,
+				m2db_get_size(args->sq3) + _fetch_content_size(beans));
 		m2db_set_obj_count(args->sq3, m2db_get_obj_count(args->sq3) + 1);
-    }
+	}
 
 	/* Purge the latest alias if the condition was met */
 	if (!err && purge_latest && latest) {
@@ -1526,11 +1571,11 @@ suspended:
 			GSList *deleted = NULL;
 			err = _real_delete (args->sq3, inplace, &deleted);
 			if (out_deleted) {
-                GSList *o = NULL;
-                for (GSList *l = deleted; l; l = l->next)
-                    o = g_slist_prepend(o, _bean_dup(l->data));
-                *out_deleted = o;
-            }
+				GSList *o = NULL;
+				for (GSList *l = deleted; l; l = l->next)
+					o = g_slist_prepend(o, _bean_dup(l->data));
+				*out_deleted = o;
+			}
 			/* <deleted> beans are direct pointer to <inplace> beans */
 			g_slist_free (deleted);
 		}
@@ -1669,10 +1714,15 @@ GError* m2db_append_to_alias(struct sqlx_sqlite3_s *sq3, struct oio_url_s *url,
 	GBytes *content_id = NULL;
 	struct bean_CONTENTS_HEADERS_s *header = NULL;
 	const gint64 now = oio_ext_real_time() / G_TIME_SPAN_SECOND;
-	for (guint i=0; i<tmp->len ;++i) {
+	for (guint i = 0; i < tmp->len; ++i) {
 		gpointer bean = tmp->pdata[i];
 		if (&descr_struct_CONTENTS_HEADERS == DESCR(bean)) {
 			header = bean;
+			if(!strcmp(CONTENTS_HEADERS_get_chunk_method(bean)->str,
+						CHUNK_METHOD_DRAINED)) {
+				err= NEWERROR(CODE_CONTENT_DRAINED, "The content is drained");
+				goto out;
+			}
 			GByteArray *gba = CONTENTS_HEADERS_get_id (header);
 			if (gba) {
 				if (content_id)
@@ -1698,7 +1748,7 @@ GError* m2db_append_to_alias(struct sqlx_sqlite3_s *sq3, struct oio_url_s *url,
 
 	/* update the position in each new chunk, and link it to the content
 	 * in place */
-	for (GSList *l=newchunks; l ;l=l->next) {
+	for (GSList *l = newchunks; l; l = l->next) {
 		struct bean_CHUNKS_s *chunk = l->data;
 		GString *gs = CHUNKS_get_position (chunk);
 		struct m2v2_position_s position = m2v2_position_decode (gs->str);
@@ -1717,7 +1767,7 @@ GError* m2db_append_to_alias(struct sqlx_sqlite3_s *sq3, struct oio_url_s *url,
 	/* Now insert each chunk bean */
 	if (!(err = _db_insert_beans_list (sq3->db, newchunks))) {
 		if (cb) {
-			for (GSList *l=newchunks; l ;l=l->next) {
+			for (GSList *l = newchunks; l; l = l->next) {
 				cb (u0, l->data);
 				l->data = NULL;  // prevent double free
 			}
@@ -1933,12 +1983,12 @@ _m2_generate_chunks(struct gen_ctx_s *ctx,
 			g_ptr_array_add(ids, shifted);
 		}
 		const char *pool = storage_policy_get_service_pool(ctx->pol);
-		if (!oio_lb__poll_pool(ctx->lb, pool, NULL, _on_id)) {
-			err = NEWERROR(CODE_POLICY_NOT_SATISFIABLE, "at position %u: "
-					"found only %u services matching the criteria (pool=%s)",
+		// FIXME(FVE): set last argument
+		if ((err = oio_lb__poll_pool(ctx->lb, pool, NULL, _on_id, NULL))) {
+			g_prefix_error(&err, "at position %u: "
+					"found only %u services matching the criteria (pool=%s): ",
 					pos, ids->len, pool);
-		}
-		if (!err) {
+		} else {
 			if (is_stgpol_backblaze(ctx->pol)) {
 				// Shortcut for backblaze
 				_gen_chunk(ctx, NULL, ctx->chunk_size, pos, -1);
@@ -1993,12 +2043,270 @@ m2_generate_beans(struct oio_url_s *url, gint64 size, gint64 chunk_size,
 			return _m2_generate_chunks(&ctx, chunk_size, 0);
 		case STGPOL_DS_EC:
 			k = _policy_parameter(pol, DS_KEY_K, 6);
-			return _m2_generate_chunks(&ctx, k*chunk_size, -1);
+			return _m2_generate_chunks(&ctx, k*chunk_size, TRUE);
 		default:
 			return NEWERROR(CODE_POLICY_NOT_SUPPORTED, "Invalid policy type");
 	}
 }
 
+enum _content_broken_state_e
+{
+	NONE,
+	REPARABLE,
+	IRREPARABLE
+};
+
+struct _check_content_s
+{
+	gint64 size;
+	gint last_pos;
+	GString *present_chunks;
+	GString *missing_pos;
+	guint nb_copy;
+	gint k;
+	gint m;
+	enum _content_broken_state_e ecb;
+	gboolean partial;
+};
+
+static void
+_prepare_message(struct _check_content_s *content, GString *message)
+{
+	g_string_append(message, "\"present_chunks\":[");
+	g_string_append(message, content->present_chunks->str);
+	g_string_append(message, "], \"missing_chunks\":[");
+	g_string_append(message, content->missing_pos->str);
+	g_string_append_c(message, ']');
+	g_string_free(content->missing_pos, TRUE);
+	g_string_free(content->present_chunks, TRUE);
+}
+
+static gboolean
+_check_metachunk_number(GSList *beans, struct _check_content_s *ec)
+{
+	gint last_pos = 0;
+	gint i = 0;
+
+	if (ec->present_chunks->len > 0)
+		g_string_append_c(ec->present_chunks, ',');
+	meta2_json_chunks_only(ec->present_chunks, beans, FALSE);
+
+	for (GSList *l = beans; l; l = l->next) {
+		i++;
+		gpointer bean = l->data;
+		if (DESCR(bean) != &descr_struct_CHUNKS) {
+			ec->ecb = IRREPARABLE;
+			return TRUE;
+		}
+
+		GString *gs = CHUNKS_get_position(bean);
+		char *subpos = g_strrstr(gs->str, ".");
+		if (subpos != NULL) {
+			subpos++;
+		} else {
+			ec->ecb = IRREPARABLE;
+			return TRUE;
+		}
+
+		gint64 p = g_ascii_strtoll(subpos, NULL, 10);
+		for (; last_pos < p - 1; last_pos++) {
+			if (ec->missing_pos->len)
+				g_string_append(ec->missing_pos, ",");
+			oio_str_gstring_append_json_string(ec->missing_pos,
+					CHUNKS_get_position(bean)->str);
+		}
+
+		last_pos++;
+		ec->size += CHUNKS_get_size(bean);
+	}
+
+	if (i < ec->k || i > ec->k + ec->m) {
+		ec->ecb = IRREPARABLE;
+		return TRUE;
+	} else if (i >= ec->k && i < ec->k + ec->m) {
+		ec->ecb = REPARABLE;
+	}
+
+	return FALSE;
+}
+
+static gboolean
+_foreach_check_plain_content(gpointer key, gpointer value, gpointer data)
+{
+	struct _check_content_s *content = data;
+	if (content->last_pos != GPOINTER_TO_INT(key) - 1 && content->last_pos != -1
+			&& !content->partial) {
+		content->ecb = IRREPARABLE;
+		return TRUE;
+	}
+
+	content->last_pos = GPOINTER_TO_INT(key);
+	guint nb_chunks = 0;
+
+	if (content->present_chunks->len > 0)
+		g_string_append_c(content->present_chunks, ',');
+	meta2_json_chunks_only (content->present_chunks, value, FALSE);
+
+	for (GSList *l = value; l; l = l->next) {
+		content->size += CHUNKS_get_size(l->data);
+		nb_chunks++;
+	}
+
+	if (content->nb_copy > nb_chunks) {
+		for (guint i = nb_chunks; i < content->nb_copy; i++) {
+			if (content->missing_pos->len)
+				g_string_append(content->missing_pos, ",");
+			oio_str_gstring_append_json_string(content->missing_pos,
+					CHUNKS_get_position(((GSList *)value)->data)->str);
+		}
+		content->ecb = REPARABLE;
+	}
+
+	return FALSE;
+}
+
+static gboolean
+_foreach_check_ec_content(gpointer key, gpointer value, gpointer data)
+{
+	struct _check_content_s *content = data;
+	if (content->last_pos != GPOINTER_TO_INT(key) - 1 && content->last_pos != -1
+		&& !content->partial) {
+		content->ecb = IRREPARABLE;
+		return TRUE;
+	}
+
+	content->last_pos = GPOINTER_TO_INT(key);
+	return _check_metachunk_number(value, content);
+}
+
+static enum _content_broken_state_e
+_check_plain_content(struct _sorted_content_s *content,
+		const struct data_security_s *dsec, GString *message, gboolean partial)
+{
+	gint nb_copy = data_security_get_int64_param(dsec, DS_KEY_COPY_COUNT, 1);
+	if (!content->header)
+		return IRREPARABLE;
+
+	gint64 size =  CONTENTS_HEADERS_get_size(content->header);
+	struct _check_content_s cp = {
+		.size = 0,
+		.last_pos = -1,
+		.missing_pos = g_string_new(""),
+		.present_chunks = g_string_new(""),
+		.nb_copy = nb_copy,
+		.partial = partial,
+	};
+
+	g_tree_foreach(content->metachunks, _foreach_check_plain_content, &cp);
+	_prepare_message(&cp, message);
+	/*If the size of all the chunks is inferior to the size indicate in the header
+	 *it means that some missing chunks could not be detected and so irreparable */
+	if (cp.size < size * nb_copy && cp.ecb == NONE && !cp.partial)
+		cp.ecb = IRREPARABLE;
+
+	return cp.ecb;
+}
+
+static enum _content_broken_state_e
+_check_ec_content(struct _sorted_content_s *content,
+		struct storage_policy_s *pol, GString *message, gboolean partial)
+{
+	if (!content->header)
+		return IRREPARABLE;
+
+	gint size = CONTENTS_HEADERS_get_size(content->header);
+	struct _check_content_s cec = {
+		.size = 0,
+		.last_pos = -1,
+		.present_chunks = g_string_new(""),
+		.missing_pos = g_string_new(""),
+		.k = _policy_parameter(pol, DS_KEY_K, 6),
+		.m = _policy_parameter(pol, DS_KEY_M, 3),
+		.partial = partial,
+	};
+
+	g_tree_foreach(content->metachunks, _foreach_check_ec_content, &cec);
+	_prepare_message(&cec, message);
+	/* Check if the size is at least superior to the minimum necessary to
+	 * size needed by the storage policy*/
+	if (cec.size < size * (cec.k + cec.m) / cec.k && cec.ecb == NONE
+			&& !cec.partial)
+		return IRREPARABLE;
+
+	return cec.ecb;
+}
+
+static gboolean
+_foreach_free_list(gpointer key, gpointer value, gpointer data){
+	(void) data;
+	(void) key;
+	if (value)
+		g_slist_free((GSList *)value);
+	return FALSE;
+}
+
+GError *m2db_check_content(GSList *beans, struct namespace_info_s *nsinfo,
+		GString *message, gboolean partial)
+{
+	GError *err = NULL;
+	struct _sorted_content_s sorted_content = {
+		.header = NULL,
+		.aliases = NULL,
+		.properties = NULL,
+		.metachunks = g_tree_new(_tree_compare_int),
+	};
+
+	struct storage_policy_s *pol = NULL;
+	GString *polname = NULL;
+	for (GSList *l = beans; l; l = l->next) {
+		gpointer bean = l->data;
+		_sort_content_cb(&sorted_content, bean);
+		if (DESCR(bean) == &descr_struct_CONTENTS_HEADERS) {
+			polname = CONTENTS_HEADERS_get_policy(bean);
+		}
+	}
+
+	if (polname && polname->str && polname->len)
+		pol = storage_policy_init(nsinfo, polname->str);
+
+	enum _content_broken_state_e  cbroken = NONE;
+	const struct data_security_s *dsec = storage_policy_get_data_security(pol);
+	switch (data_security_get_type(dsec)) {
+		case STGPOL_DS_BACKBLAZE:
+		case STGPOL_DS_PLAIN:
+			cbroken = _check_plain_content(&sorted_content, dsec, message, partial);
+			break;
+		case STGPOL_DS_EC:
+			cbroken = _check_ec_content(&sorted_content, pol, message, partial);
+			break;
+		default:
+			err = NEWERROR(CODE_POLICY_NOT_SUPPORTED, "Invalid policy type");
+	}
+
+	switch (cbroken) {
+		case NONE:
+			break;
+		case REPARABLE:
+			err = NEWERROR(CODE_CONTENT_UNCOMPLETE, "Content broken but reparable");
+			break;
+		case IRREPARABLE:
+			err = NEWERROR(CODE_CONTENT_CORRUPTED, "Content broken and irreparable");
+			break;
+	}
+
+	if (sorted_content.aliases)
+		g_slist_free(sorted_content.aliases);
+	if (sorted_content.properties)
+		g_slist_free(sorted_content.properties);
+
+	g_tree_foreach(sorted_content.metachunks, _foreach_free_list, NULL);
+	g_tree_destroy(sorted_content.metachunks);
+	if(pol)
+		storage_policy_clean(pol);
+
+	return err;
+}
+
 /* Storage Policy ----------------------------------------------------------- */
 
 static GError*
@@ -2102,7 +2410,7 @@ m2db_set_container_name(struct sqlx_sqlite3_s *sq3, struct oio_url_s *url)
 		{SQLX_ADMIN_USERTYPE, OIOURL_TYPE},
 		{NULL,0},
 	};
-	for (struct map_s *p = map; p->f ; ++p) {
+	for (struct map_s *p = map; p->f; ++p) {
 		const gchar *v = oio_url_get(url, p->k);
 		if (v != NULL)
 			sqlx_admin_init_str(sq3, p->f, v);
@@ -2171,7 +2479,7 @@ _purge_exceeding_aliases(struct sqlx_sqlite3_s *sq3, gint64 max_versions,
 		}
 	}
 
-	for (GSList *l=to_be_deleted; l ; l=l->next) {
+	for (GSList *l = to_be_deleted; l; l = l->next) {
 		GError *err2 = NULL;
 		GVariant *params[] = {NULL, NULL, NULL};
 		struct elt_s *elt = l->data;
@@ -2190,7 +2498,7 @@ _purge_exceeding_aliases(struct sqlx_sqlite3_s *sq3, gint64 max_versions,
 		metautils_gvariant_unrefv(params);
 	}
 
-	for (GSList *l=to_be_deleted; l ;l=l->next) {
+	for (GSList *l = to_be_deleted; l; l = l->next) {
 		struct elt_s *elt = l->data;
 		g_free(elt->alias);
 		g_free(elt);
