@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta2v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -32,6 +32,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define VERSIONS_ENABLED(V)   ((V) < 0 || (V) > 1)
 #define VERSIONS_LIMITED(V)   ((V) > 1)
 
+#define CHUNK_METHOD_DRAINED "drained"
+
 struct storage_policy_s;
 struct oio_url_s;
 struct sqlx_sqlite3_s;
@@ -46,6 +48,7 @@ struct list_params_s
 	guint8 flag_allversion:1;
 	guint8 flag_headers   :1;
 	guint8 flag_properties:1;
+	guint8 flag_local     :1;
 };
 
 struct dup_alias_params_s
@@ -145,12 +148,18 @@ GError* m2db_del_properties(struct sqlx_sqlite3_s *sq3, struct oio_url_s *url,
 GError* m2db_set_properties(struct sqlx_sqlite3_s *sq3, struct oio_url_s *url,
 		gboolean flush, GSList *beans, m2_onbean_cb cb, gpointer u0);
 
+GError* m2db_drain_content(struct sqlx_sqlite3_s *sq3, struct oio_url_s *url,
+		m2_onbean_cb cb, gpointer u0);
+
 GError* m2db_delete_alias(struct sqlx_sqlite3_s *sq3, gint64 max_versions,
 		struct oio_url_s *url, m2_onbean_cb cb, gpointer u0);
 
 GError* m2db_link_content(struct sqlx_sqlite3_s *sq3, struct oio_url_s *url,
 		GBytes *id);
 
+GError* m2db_check_content(GSList *beans, struct namespace_info_s *nsinfo,
+		GString* message, gboolean update);
+
 GError* m2db_update_content(struct sqlx_sqlite3_s *sq3, struct oio_url_s *url,
 		GSList *beans, GSList **out_deleted, GSList **out_added);
 
@@ -164,6 +173,8 @@ struct m2db_put_args_s
 	struct sqlx_sqlite3_s *sq3;
 	struct oio_url_s *url;
 	gint64 ns_max_versions;
+	// Should be true when in WORM mode and no admin flag
+	gboolean worm_mode;
 };
 
 GError* m2db_put_alias(struct m2db_put_args_s *args, GSList *in,
