@@ -1,7 +1,7 @@
 /*
 OpenIO SDS metautils
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -25,24 +25,34 @@ License along with this library.
 #include <errno.h>
 #include <sys/stat.h>
 
+#include <metautils/lib/common_variables.h>
+
 #include "metautils.h"
 #include "common_main.h"
 
 char syslog_id[64] = "";
+volatile gboolean main_signal_SIGHUP;
+
+static volatile gint64 main_log_level_update = 0;
 static int syslog_opened = 0;
-gint64 main_log_level_update = 0;
-
-
 static int grid_main_rc = 0;
 static volatile gboolean flag_running = FALSE;
 static volatile gboolean flag_daemon = FALSE;
 static volatile gboolean flag_quiet = FALSE;
+static volatile gboolean flag_dump_options = FALSE;
 
 static struct grid_main_callbacks *user_callbacks;
 static volatile gboolean pidfile_written = FALSE;
 static gchar pidfile_path[1024] = {0,0,0};
 static struct stat pidfile_stat;
 
+static gchar errbuff[1024];
+
+#define ERR(FMT,...) do { \
+	g_snprintf(errbuff, sizeof(errbuff), FMT, ##__VA_ARGS__); \
+	return errbuff; \
+} while (0)
+
 /* ------------------------------------------------------------------------- */
 
 void
@@ -55,78 +65,80 @@ logger_syslog_open (void)
 	g_log_set_default_handler(oio_log_syslog, NULL);
 }
 
-#define ERR(FMT,...) do { \
-	g_snprintf(errbuff, sizeof(errbuff), FMT, ##__VA_ARGS__); \
-	return errbuff; \
-} while (0)
-
 static const char*
-_set_opt(gchar **tokens)
+_set_config_option(struct grid_main_option_s *opt, gchar **tokens)
 {
-	static gchar errbuff[1024];
 	gint64 i64 = 0;
 
+	if (tokens[1] == NULL && opt->type != OT_BOOL)
+		ERR("Missing parameter, expected '%s=<Value>'", tokens[0]);
+
+	switch (opt->type) {
+		case OT_BOOL:
+			if (tokens[1] == NULL) {
+				*(opt->data.b) = TRUE;
+				return NULL;
+			}
+			*(opt->data.b) = oio_str_parse_bool(tokens[1], *(opt->data.b));
+			return NULL;
+		case OT_INT:
+			i64 = 0;
+			if (!oio_str_is_number(tokens[1], &i64))
+				ERR("Invalid integer parameter");
+			if (i64 < G_MININT || i64 > G_MAXINT)
+				ERR("Invalid parameter range");
+			*(opt->data.i) = i64;
+			return NULL;
+		case OT_UINT:
+			i64 = 0;
+			if (!oio_str_is_number(tokens[1], &i64))
+				ERR("Invalid integer parameter");
+			if (i64 < 0 || i64 > G_MAXUINT)
+				ERR("Invalid parameter range");
+			*(opt->data.u) = i64;
+			return NULL;
+		case OT_INT64:
+			i64 = g_ascii_strtoll(tokens[1], NULL, 10);
+			*(opt->data.i64) = i64;
+			return NULL;
+		case OT_TIME:
+			i64 = g_ascii_strtoll(tokens[1], NULL, 10);
+			*(opt->data.t) = i64;
+			return NULL;
+		case OT_DOUBLE:
+			*(opt->data.d) = g_ascii_strtod(tokens[1], NULL);
+			return NULL;
+		case OT_STRING:
+			if (!*(opt->data.str))
+				*(opt->data.str) = g_string_new("");
+			g_string_set_size(*(opt->data.str), 0);
+			g_string_append(*(opt->data.str), tokens[1]);
+			return NULL;
+		case OT_LIST:
+			*(opt->data.lst) = g_slist_prepend(*(opt->data.lst), g_strdup(tokens[1]));
+			return NULL;
+		default:
+			ERR("Invalid option type, possible corruption");
+	}
+}
+
+static const char*
+_set_fixed_opt(gchar **tokens)
+{
 	errbuff[0] = '\0';
 
 	if (!tokens || tokens[0] == NULL)
 		ERR("Invalid option format, expected 'Key=Value'");
 
-	for (struct grid_main_option_s *opt=user_callbacks->options();
-		 opt && opt->name;
-		 opt++) {
-		if (0 == g_ascii_strcasecmp(opt->name, tokens[0])) {
-			if (tokens[1] == NULL && opt->type != OT_BOOL)
-				ERR("Missing parameter, expected '%s=<Value>'", tokens[0]);
-			switch (opt->type) {
-				case OT_BOOL:
-					if (tokens[1] == NULL) {
-						*(opt->data.b) = TRUE;
-						return NULL;
-					}
-					*(opt->data.b) = oio_str_parse_bool(tokens[1], *(opt->data.b));
-					return NULL;
-				case OT_INT:
-					i64 = 0;
-					if (!oio_str_is_number(tokens[1], &i64))
-						ERR("Invalid integer parameter");
-					if (i64 < G_MININT || i64 > G_MAXINT)
-						ERR("Invalid parameter range");
-					*(opt->data.i) = i64;
-					return NULL;
-				case OT_UINT:
-					i64 = 0;
-					if (!oio_str_is_number(tokens[1], &i64))
-						ERR("Invalid integer parameter");
-					if (i64 < 0 || i64 > G_MAXUINT)
-						ERR("Invalid parameter range");
-					*(opt->data.u) = i64;
-					return NULL;
-				case OT_INT64:
-					i64 = g_ascii_strtoll(tokens[1], NULL, 10);
-					*(opt->data.i64) = i64;
-					return NULL;
-				case OT_TIME:
-					i64 = g_ascii_strtoll(tokens[1], NULL, 10);
-					*(opt->data.t) = i64;
-					return NULL;
-				case OT_DOUBLE:
-					*(opt->data.d) = g_ascii_strtod(tokens[1], NULL);
-					return NULL;
-				case OT_STRING:
-					if (!*(opt->data.str))
-						*(opt->data.str) = g_string_new("");
-					g_string_set_size(*(opt->data.str), 0);
-					g_string_append(*(opt->data.str), tokens[1]);
-					return NULL;
-				case OT_LIST:
-					*(opt->data.lst) = g_slist_prepend(*(opt->data.lst), g_strdup(tokens[1]));
-					return NULL;
-				default:
-					ERR("Invalid option type, possible corruption");
-			}
-		}
+	struct grid_main_option_s *opts = user_callbacks->options();
+	for (struct grid_main_option_s *opt = opts; opts && opt->name; opt++) {
+		if (!g_ascii_strcasecmp(opt->name, tokens[0]))
+			return _set_config_option(opt, tokens);
 	}
 
+	if (oio_var_value_one_with_option(tokens[0], tokens[1]))
+		return NULL;
+
 	ERR("Option '%s' not supported", tokens[0]);
 }
 
@@ -137,22 +149,22 @@ grid_main_set_option(const gchar *str_opt)
 	const gchar *result;
 
 	tokens = g_strsplit(str_opt, "=", 2);
-	result = _set_opt(tokens);
+	result = _set_fixed_opt(tokens);
 	if (tokens)
 		g_strfreev(tokens);
 	return result;
 }
 
 static void
-_dump_xopts()
+_dump_extra_options(void)
 {
-	gchar name[1024];
-	struct grid_main_option_s *o;
+	gchar name[256];
 
 	if (flag_quiet)
 		return;
 
-	for (o=user_callbacks->options(); o && o->name ;o++) {
+	struct grid_main_option_s *opts = user_callbacks->options();
+	for (struct grid_main_option_s *o = opts; opts && o->name; o++) {
 		switch (o->type) {
 			case OT_BOOL:
 				g_snprintf(name, sizeof(name), "%s=%s", o->name, (*(o->data.b)?"on":"off"));
@@ -186,6 +198,20 @@ _dump_xopts()
 	}
 }
 
+static void
+_on_option__dump_to_stdout(const char *k, const char *v)
+{
+	g_print("%s=%s\n", k, v);
+}
+
+static void
+_dump_config_options(void)
+{
+	if (flag_quiet)
+		return;
+	oio_var_list_all(_on_option__dump_to_stdout);
+}
+
 static void
 grid_main_usage(void)
 {
@@ -196,17 +222,18 @@ grid_main_usage(void)
 
 	g_printerr("\nOPTIONS:\n");
 	g_printerr("  -h         help, displays this section\n");
+	g_printerr("  -c         dump the current live central configurable variables (requires all positional arguments)\n");
 	g_printerr("  -d         daemonizes the process (default FALSE)\n");
 	g_printerr("  -q         quiet mode, supress output on stdout stderr \n");
 	g_printerr("  -v         verbose mode, this activates stderr traces (default FALSE)\n");
 	g_printerr("  -p PATH    pidfile path, no pidfile if unset\n");
-	g_printerr("  -l PATH    activates the log4c emulation and load PATH as a log4c file\n");
 	g_printerr("  -s TOKEN   activates syslog traces (default FALSE)\n"
 			   "             with the given identifier\n");
 	g_printerr("  -O XOPT    set extra options.\n");
 
 	g_printerr("\nXOPT'S with default value:\n");
-	_dump_xopts();
+	_dump_extra_options();
+
 	g_printerr("\nEXTRA_ARGS usage:\n%s\n", user_callbacks->usage());
 }
 
@@ -220,13 +247,13 @@ grid_main_cli_usage(void)
 
 	g_printerr("\nOPTIONS:\n");
 	g_printerr("  -h         help, displays this section\n");
+	g_printerr("  -c         dump the current live central configurable variables (requires all positional arguments)\n");
 	g_printerr("  -q         quiet mode, supress output on stdout stderr \n");
 	g_printerr("  -v         verbose mode, this activates stderr traces (default FALSE)\n");
-	g_printerr("  -l PATH    activates the log4c emulation and load PATH as a log4c file\n");
 	g_printerr("  -O XOPT    set extra options.\n");
 
 	g_printerr("\nXOPT'S with default value:\n");
-	_dump_xopts();
+	_dump_extra_options();
 
 	g_printerr("\nEXTRA_ARGS usage:\n%s\n", user_callbacks->usage());
 }
@@ -269,11 +296,18 @@ grid_main_sighandler_stop(int s)
 	signal(s, grid_main_sighandler_stop);
 }
 
+static void
+grid_main_sighandler_HUP(int s)
+{
+	main_signal_SIGHUP = TRUE;
+	signal(s, grid_main_sighandler_HUP);
+}
+
 static void
 grid_main_sighandler_USR1(int s)
 {
 	oio_log_verbose();
-	main_log_level_update = oio_ext_monotonic_time () / G_TIME_SPAN_SECOND;
+	main_log_level_update = oio_ext_monotonic_time ();
 	alarm(900);
 	signal(s, grid_main_sighandler_USR1);
 }
@@ -281,17 +315,18 @@ grid_main_sighandler_USR1(int s)
 static void
 grid_main_sighandler_USR2(int s)
 {
-	signal(s, grid_main_sighandler_USR2);
 	oio_log_reset_level();
 	main_log_level_update = 0;
+	signal(s, grid_main_sighandler_USR2);
 }
 
 static void
 grid_main_sighandler_ALRM(int s)
 {
-	gint64 now = oio_ext_monotonic_time () / G_TIME_SPAN_SECOND;
+	gint64 now = oio_ext_monotonic_time();
 	signal(s, grid_main_sighandler_ALRM);
-	if (!main_log_level_update || main_log_level_update + 299 < now) {
+	if (!main_log_level_update ||
+			now > (main_log_level_update + main_log_level_reset_delay)) {
 		oio_log_reset_level();
 		main_log_level_update = 0;
 	}
@@ -300,7 +335,8 @@ grid_main_sighandler_ALRM(int s)
 static void
 grid_main_install_sighandlers(void)
 {
-	signal(SIGHUP,  grid_main_sighandler_stop);
+	signal(SIGHUP,  grid_main_sighandler_HUP);
+
 	signal(SIGINT,  grid_main_sighandler_stop);
 	signal(SIGQUIT, grid_main_sighandler_stop);
 	signal(SIGTERM, grid_main_sighandler_stop);
@@ -342,7 +378,7 @@ grid_main_delete_pid_file(void)
 		return;
 	}
 	if (-1 == stat(pidfile_path, &current_pidfile_stat)) {
-		GRID_WARN("Unable to remove pidfile at [%s] : %s", pidfile_path, strerror(errno));
+		GRID_WARN("Unable to remove pidfile at [%s]: %s", pidfile_path, strerror(errno));
 		return;
 	}
 	if (current_pidfile_stat.st_ino != pidfile_stat.st_ino) {
@@ -351,7 +387,7 @@ grid_main_delete_pid_file(void)
 	}
 
 	if (-1 == metautils_syscall_unlink(pidfile_path))
-		GRID_WARN("Failed to unlink [%s] : %s", pidfile_path, strerror(errno));
+		GRID_WARN("Failed to unlink [%s]: %s", pidfile_path, strerror(errno));
 	else {
 		GRID_INFO("Deleted [%s]", pidfile_path);
 		pidfile_written = FALSE;
@@ -374,7 +410,7 @@ grid_main_init(int argc, char **args)
 	memset(pidfile_path, 0, sizeof(pidfile_path));
 
 	for (;;) {
-		int c = getopt(argc, args, "O:hdvqp:s:");
+		int c = getopt(argc, args, "O:hdvcqp:s:");
 		if (c == -1)
 			break;
 		switch (c) {
@@ -382,7 +418,7 @@ grid_main_init(int argc, char **args)
 				do {
 					const char *errmsg = grid_main_set_option(optarg);
 					if (errmsg) {
-						GRID_WARN("Invalid option : %s", errmsg);
+						GRID_WARN("Invalid option: %s", errmsg);
 						grid_main_usage();
 						return FALSE;
 					}
@@ -398,7 +434,7 @@ grid_main_init(int argc, char **args)
 			case 'p':
 				memset(pidfile_path, 0, sizeof(pidfile_path));
 				if (sizeof(pidfile_path) <= g_strlcpy(pidfile_path, optarg, sizeof(pidfile_path)-1)) {
-					GRID_WARN("Invalid '-p' argument : too long");
+					GRID_WARN("Invalid '-p' argument: too long");
 					grid_main_usage();
 					return FALSE;
 				}
@@ -411,16 +447,19 @@ grid_main_init(int argc, char **args)
 			case 's':
 				memset(syslog_id, 0, sizeof(syslog_id));
 				if (sizeof(syslog_id) <= g_strlcpy(syslog_id, optarg, sizeof(syslog_id)-1)) {
-					GRID_WARN("Invalid '-s' argument : too long");
+					GRID_WARN("Invalid '-s' argument: too long");
 					grid_main_usage();
 					return FALSE;
 				}
 				GRID_DEBUG("Explicitely configured syslog_id=[%s]", syslog_id);
 				break;
+			case 'c':
+				flag_dump_options = TRUE;
+				break;
 			case 'v':
 				if (!flag_quiet) {
 					oio_log_verbose_default();
-					main_log_level_update = oio_ext_monotonic_time () / G_TIME_SPAN_SECOND;
+					main_log_level_update = oio_ext_monotonic_time ();
 				}
 				break;
 			case ':':
@@ -496,24 +535,29 @@ grid_main(int argc, char ** argv, struct grid_main_callbacks * callbacks)
 	}
 	grid_main_install_sighandlers();
 
-	if (flag_daemon || *syslog_id) {
-		stdout = freopen("/dev/null", "w", stdout);
-		stderr = freopen("/dev/null", "w", stderr);
-	}
+	if (flag_dump_options) {
+		_dump_config_options();
+	} else {
 
-	if (flag_daemon) {
-		stdin = freopen("/dev/null", "r", stdin);
-		if (-1 == daemon(1,0)) {
-			GRID_WARN("daemonize error : %s", strerror(errno));
-			grid_main_fini();
-			return 1;
+		if (flag_daemon || *syslog_id) {
+			stdout = freopen("/dev/null", "w", stdout);
+			stderr = freopen("/dev/null", "w", stderr);
 		}
-		grid_main_write_pid_file();
-	}
 
-	grid_main_install_sighandlers();
-	if (flag_running)
-		user_callbacks->action();
+		if (flag_daemon) {
+			stdin = freopen("/dev/null", "r", stdin);
+			if (-1 == daemon(1,0)) {
+				GRID_WARN("daemonize error: %s", strerror(errno));
+				grid_main_fini();
+				return 1;
+			}
+			grid_main_write_pid_file();
+		}
+
+		grid_main_install_sighandlers();
+		if (flag_running)
+			user_callbacks->action();
+	}
 
 	grid_main_fini();
 	return grid_main_rc;
@@ -534,7 +578,7 @@ grid_main_cli_init(int argc, char **args)
 				do {
 					const char *errmsg = grid_main_set_option(optarg);
 					if (errmsg) {
-						GRID_WARN("Invalid option : %s", errmsg);
+						GRID_WARN("Invalid option: %s", errmsg);
 						grid_main_cli_usage();
 						return FALSE;
 					}
@@ -552,6 +596,9 @@ grid_main_cli_init(int argc, char **args)
 				if (!flag_quiet)
 					oio_log_verbose_default();
 				break;
+			case 'c':
+				flag_dump_options = TRUE;
+				break;
 			case ':':
 				GRID_WARN("Unexpected option at position %d ('%c')", optind, optopt);
 				return FALSE;
@@ -587,7 +634,9 @@ grid_main_cli(int argc, char ** argv, struct grid_main_callbacks * callbacks)
 	}
 	grid_main_install_sighandlers();
 
-	if (flag_running)
+	if (flag_dump_options)
+		_dump_config_options();
+	else if (flag_running)
 		user_callbacks->action();
 
 	metautils_ignore_signals();
@@ -602,11 +651,11 @@ metautils_ignore_signals(void)
 
 	sigfillset(&new_set);
 	if (0 != sigprocmask(SIG_BLOCK, &new_set, NULL))
-		g_message("LIBC Some signals could not be blocked : %s", strerror(errno));
+		g_message("LIBC Some signals could not be blocked: %s", strerror(errno));
 
 	sigfillset(&new_set);
 	if (0 != pthread_sigmask(SIG_BLOCK, &new_set, NULL))
-		g_message("PTHREAD Some signals could not be blocked : %s", strerror(errno));
+		g_message("PTHREAD Some signals could not be blocked: %s", strerror(errno));
 }
 
 void
