@@ -1,7 +1,7 @@
 /*
 OpenIO SDS metautils
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2017 OpenIO, modified as part of OpenIO SDS
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -28,6 +28,8 @@ License along with this library.
 
 #include "metautils.h"
 
+#include <metautils/lib/common_variables.h>
+
 #ifndef URL_MAXLEN
 # define URL_MAXLEN STRLEN_ADDRINFO
 #endif
@@ -36,10 +38,6 @@ License along with this library.
 # define EVENT_BUFFER_SIZE 2048
 #endif
 
-#ifdef HAVE_ENBUG
-gint32 oio_client_timeout_threshold = 10;
-#endif
-
 enum client_step_e
 {
 	NONE = 0,
@@ -51,14 +49,8 @@ enum client_step_e
 	STATUS_FAILED
 };
 
-struct gridd_client_factory_s
-{
-	struct abstract_client_factory_s abstract;
-};
-
 struct gridd_client_s
 {
-	struct abstract_client_s abstract;
 	GByteArray *request;
 	GByteArray *reply;
 	GError *error;
@@ -82,7 +74,7 @@ struct gridd_client_s
 	enum client_step_e step : 4;
 	guint8 keepalive : 1;
 	guint8 forbid_redirect : 1;
-	guint8 avoid_on_error : 1;
+	guint8 avoidance_onoff : 1;
 
 	gchar orig_url[URL_MAXLEN];
 	gchar url[URL_MAXLEN];
@@ -90,12 +82,11 @@ struct gridd_client_s
 
 /* Cache of network errors ------------------------------------------------- */
 
-guint64 oio_client_max_net_errors = 15;
-time_t oio_client_period_net_errors = 15;
-gboolean oio_cache_avoid_on_error = FALSE;
-
 static GMutex lock_errors;
-static GTree *tree_errors = NULL;
+static GTree *tree_errors = NULL;  /* <gchar*> -> <rrd*> */
+
+static GRWLock lock_down;
+static GTree *tree_down = NULL;  /* <gchar*> -> <constant> */
 
 void  _oio_cache_of_errors_contructor (void);
 
@@ -108,30 +99,39 @@ _oio_cache_of_errors_contructor (void)
 			g_mutex_init (&lock_errors);
 			tree_errors = g_tree_new_full(metautils_strcmp3, NULL,
 					g_free, (GDestroyNotify) grid_single_rrd_destroy);
+			g_rw_lock_init(&lock_down);
+			tree_down = g_tree_new_full(metautils_strcmp3, NULL, g_free, NULL);
 		}
 	}
 }
 
+static gboolean
+_is_peer_down (const char *url)
+{
+	g_rw_lock_reader_lock(&lock_down);
+	gpointer p = g_tree_lookup(tree_down, url);
+	g_rw_lock_reader_unlock(&lock_down);
+
+	return p != NULL;
+}
+
 static gboolean
 _has_too_many_errors (const char *url)
 {
-	if (!url || !*url)
-		return FALSE;
-
 	guint64 delta = 0;
 	g_mutex_lock(&lock_errors);
 
 	struct grid_single_rrd_s *rrd = g_tree_lookup(tree_errors, url);
 	if (rrd) {
 		const gint64 now = oio_ext_monotonic_seconds();
-		delta = grid_single_rrd_get_delta(rrd, now, oio_client_period_net_errors);
+		delta = grid_single_rrd_get_delta(rrd, now, oio_client_cache_errors_period);
 	}
 	g_mutex_unlock(&lock_errors);
 
-	gboolean rc = (oio_client_max_net_errors <= delta);
+	const gboolean rc = (oio_client_cache_errors_max <= delta);
 	if (rc) {
-		GRID_DEBUG("[%s] seems DOWN (%"G_GUINT64_FORMAT" fails in %lds)",
-				url, delta, oio_client_period_net_errors);
+		GRID_DEBUG("[%s] seems DOWN (%"G_GUINT64_FORMAT" fails in %" G_GINT64_FORMAT "s)",
+				url, delta, oio_client_cache_errors_period);
 	}
 	return rc;
 }
@@ -153,7 +153,7 @@ _count_network_error (const char *url, const GError *err)
 	struct grid_single_rrd_s *rrd = g_tree_lookup(tree_errors, url);
 	if (!rrd) {
 		rrd = grid_single_rrd_create(oio_ext_monotonic_seconds(),
-				oio_client_period_net_errors + 1);
+				oio_client_cache_errors_period + 1);
 		g_tree_replace(tree_errors, g_strdup(url), rrd);
 	}
 	const gint64 now = oio_ext_monotonic_seconds();
@@ -163,62 +163,26 @@ _count_network_error (const char *url, const GError *err)
 
 /* ------------------------------------------------------------------------- */
 
-static void _client_free(struct gridd_client_s *client);
-static GError* _client_connect_url(struct gridd_client_s *client, const gchar *url);
-static GError* _client_request(struct gridd_client_s *client, GByteArray *req,
-		gpointer ctx, client_on_reply cb);
-static gboolean _client_expired(struct gridd_client_s *client, gint64 now);
-static gboolean _client_finished(struct gridd_client_s *c);
-static const gchar* _client_url(struct gridd_client_s *client);
-static int _client_get_fd(struct gridd_client_s *client);
-static int _client_interest(struct gridd_client_s *client);
-static GError* _client_error(struct gridd_client_s *client);
-static gboolean _client_start(struct gridd_client_s *client);
-static GError* _client_set_fd(struct gridd_client_s *client, int fd);
-static void _client_set_timeout(struct gridd_client_s *client, gdouble seconds);
-static void _client_set_timeout_cnx(struct gridd_client_s *client, gdouble sec);
-static void _client_set_keepalive(struct gridd_client_s *client, gboolean on);
-static void _client_react(struct gridd_client_s *client);
-static gboolean _client_expire(struct gridd_client_s *client, gint64 now);
-static void _client_fail(struct gridd_client_s *client, GError *why);
-
-static void _factory_clean(struct gridd_client_factory_s *self);
-static struct gridd_client_s * _factory_create_client (
-		struct gridd_client_factory_s *self);
-
-struct gridd_client_factory_vtable_s VTABLE_FACTORY =
+static void
+_client_reset_reply(struct gridd_client_s *client)
 {
-	_factory_clean,
-	_factory_create_client
-};
-
-struct gridd_client_vtable_s VTABLE_CLIENT =
-{
-	_client_free,
-	_client_connect_url,
-	_client_request,
-	_client_error,
-	_client_interest,
-	_client_url,
-	_client_get_fd,
-	_client_set_fd,
-	_client_set_keepalive,
-	_client_set_timeout,
-	_client_set_timeout_cnx,
-	_client_expired,
-	_client_finished,
-	_client_start,
-	_client_react,
-	_client_expire,
-	_client_fail
-};
+	client->size = 0;
+	if (!client->reply)
+		client->reply = g_byte_array_new();
+	else if (client->reply->len > 0)
+		g_byte_array_set_size(client->reply, 0);
+}
 
+/* Benefit from the maybe-present TCP_FASTOPEN, and try to send a few bytes
+ * alongside with the initiation sequence.
+ */
 static GError*
 _client_connect(struct gridd_client_s *client)
 {
 	GError *err = NULL;
-	client->fd = sock_connect(client->url, &err);
-
+	gsize sent = client->request ? client->request->len : 0;
+	client->fd = sock_connect_and_send(client->url, &err,
+			sent ? client->request->data : NULL, &sent);
 	if (client->fd < 0) {
 		EXTRA_ASSERT(err != NULL);
 		g_prefix_error(&err, "Connect error: ");
@@ -227,7 +191,14 @@ _client_connect(struct gridd_client_s *client)
 
 	EXTRA_ASSERT(err == NULL);
 	client->tv_connect = oio_ext_monotonic_time ();
-	client->step = CONNECTING;
+	client->sent_bytes = sent;
+	if (client->sent_bytes >= client->request->len) {
+		_client_reset_reply(client);
+		client->step = REP_READING_SIZE;
+	} else {
+		client->step = CONNECTING;
+	}
+
 	return NULL;
 }
 
@@ -243,16 +214,6 @@ _client_reset_request(struct gridd_client_s *client)
 	client->nb_redirects = 0;
 }
 
-static void
-_client_reset_reply(struct gridd_client_s *client)
-{
-	client->size = 0;
-	if (!client->reply)
-		client->reply = g_byte_array_new();
-	else if (client->reply->len > 0)
-		g_byte_array_set_size(client->reply, 0);
-}
-
 static void
 _client_reset_cnx(struct gridd_client_s *client)
 {
@@ -297,6 +258,7 @@ _client_manage_reply(struct gridd_client_s *client, MESSAGE reply)
 	}
 
 	if (status == CODE_TEMPORARY) {
+		_client_reset_reply(client);
 		client->step = REP_READING_SIZE;
 		return NULL;
 	}
@@ -306,10 +268,12 @@ _client_manage_reply(struct gridd_client_s *client, MESSAGE reply)
 		if (client->step == STATUS_OK) {
 			if (!client->keepalive)
 				metautils_pclose(&(client->fd));
+		} else {
+			_client_reset_reply(client);
 		}
 		if (client->on_reply) {
 			if (!client->on_reply(client->ctx, reply))
-				return NEWERROR(CODE_INTERNAL_ERROR, "Handler error");
+				return SYSERR("Handler error");
 		}
 		return NULL;
 	}
@@ -367,106 +331,123 @@ static GError *
 _client_manage_event_in_buffer(struct gridd_client_s *client, guint8 *d, gsize ds)
 {
 	guint32 s32;
-	ssize_t rc;
 
 	switch (client->step) {
 
 		case CONNECTING:
 			EXTRA_ASSERT(client->fd >= 0);
-			EXTRA_ASSERT(client->request != NULL);
+			_client_reset_reply(client);
+			/* Do not reset client->sent_bytes, as we may have sent some bytes
+			 * along with the connection packets (thanks to TCP_FASTOPEN). */
 			client->step = REQ_SENDING;
 			return NULL;
 
 		case REQ_SENDING:
 
-			client->step = REQ_SENDING;
-
+			EXTRA_ASSERT(client->step == REQ_SENDING);
+			_client_reset_reply(client);
 			if (!client->request)
 				return NULL;
-			_client_reset_reply(client);
+			else {
+				/* Continue to send the request */
+				ssize_t rc = metautils_syscall_send(client->fd,
+						client->request->data + client->sent_bytes,
+						client->request->len - client->sent_bytes,
+						MSG_NOSIGNAL);
 
-			/* Continue to send the request */
-			rc = metautils_syscall_send(client->fd,
-					client->request->data + client->sent_bytes,
-					client->request->len - client->sent_bytes,
-					MSG_NOSIGNAL);
+				if (rc < 0)
+					return (errno == EINTR || errno == EAGAIN) ? NULL :
+						NEWERROR(CODE_NETWORK_ERROR,
+								"ERROR while requesting %s: (%d) %s",
+								gridd_client_url(client), errno, strerror(errno));
+				if (rc > 0)
+					client->sent_bytes += rc;
 
-			if (rc < 0)
-				return (errno == EINTR || errno == EAGAIN) ? NULL :
-					NEWERROR(errno, "write error (%s)", strerror(errno));
-			if (rc > 0)
-				client->sent_bytes += rc;
-
-			if (client->sent_bytes < client->request->len)
-				return NULL;
+				if (client->sent_bytes < client->request->len)
+					return NULL;
+			}
 
 			client->step = REP_READING_SIZE;
+			// FALLTHROUGH
 
 		case REP_READING_SIZE:
 
-			client->step = REP_READING_SIZE;
+			EXTRA_ASSERT(client->step == REP_READING_SIZE);
 
 			if (!client->reply)
-				client->reply = g_byte_array_new();
+				client->reply = g_byte_array_sized_new(256);
 
 			if (client->reply->len < 4) {
 				/* Continue reading the size */
-				rc = metautils_syscall_read(client->fd, d, (4 - client->reply->len));
+				ssize_t rc = metautils_syscall_read(
+						client->fd, d, (4 - client->reply->len));
+				if (rc == 0)
+					return NEWERROR(CODE_NETWORK_ERROR,
+							"EOF while reading response size from %s",
+							gridd_client_url(client));
 				if (rc < 0)
 					return (errno == EINTR || errno == EAGAIN) ? NULL :
-						NEWERROR(errno, "read error (%s)", strerror(errno));
-				if (rc > 0)
-					g_byte_array_append(client->reply, d, rc);
+						NEWERROR(CODE_NETWORK_ERROR,
+								"ERROR while reading response size from %s:"
+								" (%d) %s", gridd_client_url(client),
+								errno, strerror(errno));
 
-				if (client->reply->len < 4) {
-					if (!rc)
-						return NEWERROR(errno, "EOF!");
+				EXTRA_ASSERT(rc > 0);
+				g_byte_array_append(client->reply, d, rc);
+
+				if (client->reply->len < 4)  /* size still incomplete */
 					return NULL;
-				}
 			}
 
-			EXTRA_ASSERT (client->reply->len == 4);
+			EXTRA_ASSERT (client->reply->len >= 4);
 			s32 = *((guint32*)(client->reply->data));
 			client->size = g_ntohl(s32);
-
+			client->step = REP_READING_DATA;
+			// FALLTHROUGH
 		case REP_READING_DATA:
 
-			client->step = REP_READING_DATA;
-			rc = 0;
-
+			EXTRA_ASSERT(client->step == REP_READING_DATA);
 			EXTRA_ASSERT (client->reply->len <= client->size + 4);
+
+			/* If the reply is not complete, try to consume some bytes */
 			if (client->reply->len < client->size + 4) {
-				gsize remaiming = client->size + 4 - client->reply->len;
+				gsize remaining = client->size + 4 - client->reply->len;
 				gsize dmax = ds;
-				if (dmax > remaiming)
-					dmax = remaiming;
-				rc = metautils_syscall_read(client->fd, d, dmax);
+				if (dmax > remaining)
+					dmax = remaining;
+				ssize_t rc = metautils_syscall_read(client->fd, d, dmax);
+				if (rc == 0)
+					return NEWERROR(CODE_NETWORK_ERROR,
+							"EOF while reading response from %s "
+							"(got %u bytes, expected %"G_GUINT32_FORMAT")",
+							gridd_client_url(client), client->reply->len,
+							client->size + 4);
 				if (rc < 0)
 					return (errno == EINTR || errno == EAGAIN) ? NULL :
-						NEWERROR(errno, "read error (%s)", strerror(errno));
-				if (rc > 0)
-					g_byte_array_append(client->reply, d, rc);
+						NEWERROR(CODE_NETWORK_ERROR,
+								"ERROR while reading response from %s: (%d) %s",
+								gridd_client_url(client), errno, strerror(errno));
+
+				EXTRA_ASSERT(rc > 0);
+				g_byte_array_append(client->reply, d, rc);
 			}
 
 			EXTRA_ASSERT (client->reply->len <= client->size + 4);
-			if (client->reply->len == client->size + 4) {
+
+			if (client->reply->len < client->size + 4) {
+				/* the reply is not complete yet */
+				return NULL;
+			} else {
+				/* Ok, the reply is complete, ready to be parsed */
 				GError *err = _client_manage_reply_data(client);
 				if (err) {
 					client->step = STATUS_FAILED;
 					return err;
-				}
-				else {
-					if (client->step != CONNECTING && client->step != STATUS_FAILED
-							&& client->step != STATUS_OK) {
-						client->reply = g_byte_array_set_size(client->reply, 0);
-						client->step = REP_READING_SIZE;
-						client->size = 0;
-					}
+				} else {
+					/* `step` has been changed by _client_manage_reply_data() */
+					return NULL;
 				}
 			}
-			else if (!rc)
-				return NEWERROR(errno, "EOF!");
-			return NULL;
 
 		default:
 			g_assert_not_reached();
@@ -486,16 +467,11 @@ _client_manage_event(struct gridd_client_s *client)
 
 /* ------------------------------------------------------------------------- */
 
-static void
-_client_react(struct gridd_client_s *client)
+void
+gridd_client_react(struct gridd_client_s *client)
 {
-	EXTRA_ASSERT(client != NULL);
-	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
-
-	if (!client)
-		return;
 	GError *err = NULL;
-
+	EXTRA_ASSERT(client != NULL);
 retry:
 	if (!(err = _client_manage_event(client))) {
 		if (client->step == REP_READING_SIZE && client->reply
@@ -510,38 +486,25 @@ retry:
 	}
 }
 
-static const gchar*
-_client_url(struct gridd_client_s *client)
+const gchar*
+gridd_client_url(struct gridd_client_s *client)
 {
 	EXTRA_ASSERT(client != NULL);
-	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
-
-	if (!client)
-		return NULL;
-	else {
-		client->url[ sizeof(client->url)-1 ] = '\0';
-		return client->url;
-	}
+	client->url[ sizeof(client->url)-1 ] = '\0';
+	return client->url;
 }
 
-static int
-_client_get_fd(struct gridd_client_s *client)
+int
+gridd_client_fd(struct gridd_client_s *client)
 {
 	EXTRA_ASSERT(client != NULL);
-	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
-
-	return client ? client->fd : -1;
+	return client->fd;
 }
 
-static int
-_client_interest(struct gridd_client_s *client)
+int
+gridd_client_interest(struct gridd_client_s *client)
 {
 	EXTRA_ASSERT(client != NULL);
-	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
-
-	if (!client)
-		return 0;
-
 	switch (client->step) {
 		case NONE:
 			return 0;
@@ -562,24 +525,21 @@ _client_interest(struct gridd_client_s *client)
 	}
 }
 
-static GError *
-_client_error(struct gridd_client_s *client)
+GError *
+gridd_client_error(struct gridd_client_s *client)
 {
 	EXTRA_ASSERT(client != NULL);
-	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
-
-	if (!client || !client->error)
+	if (!client->error)
 		return NULL;
 	return NEWERROR(client->error->code, "%s", client->error->message);
 }
 
-static void
-_client_free(struct gridd_client_s *client)
+void
+gridd_client_free(struct gridd_client_s *client)
 {
 	EXTRA_ASSERT(client != NULL);
-	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
 
-	if (client->avoid_on_error)
+	if (oio_client_cache_errors)
 		_count_network_error(client->url, client->error);
 
 	_client_reset_reply(client);
@@ -593,39 +553,25 @@ _client_free(struct gridd_client_s *client)
 	SLICE_FREE (struct gridd_client_s, client);
 }
 
-static void
-_client_set_keepalive(struct gridd_client_s *client, gboolean on)
+void
+gridd_client_set_timeout(struct gridd_client_s *client, gdouble seconds)
 {
 	EXTRA_ASSERT(client != NULL);
-	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
-
-	client->keepalive = BOOL(on);
-}
-
-static void
-_client_set_timeout(struct gridd_client_s *client, gdouble seconds)
-{
-	EXTRA_ASSERT(client != NULL);
-	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
-
 	client->delay_single = seconds * (gdouble) G_TIME_SPAN_SECOND;
 	client->delay_overall = seconds * (gdouble) G_TIME_SPAN_SECOND;
 }
 
-static void
-_client_set_timeout_cnx(struct gridd_client_s *client, gdouble seconds)
+void
+gridd_client_set_timeout_cnx(struct gridd_client_s *client, gdouble seconds)
 {
 	EXTRA_ASSERT(client != NULL);
-	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
-
 	client->delay_connect = seconds * (gdouble) G_TIME_SPAN_SECOND;
 }
 
-static GError*
-_client_set_fd(struct gridd_client_s *client, int fd)
+GError*
+gridd_client_set_fd(struct gridd_client_s *client, int fd)
 {
 	EXTRA_ASSERT(client != NULL);
-	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
 
 	if (fd >= 0) {
 		switch (client->step) {
@@ -661,11 +607,10 @@ _client_set_fd(struct gridd_client_s *client, int fd)
 	return NULL;
 }
 
-static GError*
-_client_connect_url(struct gridd_client_s *client, const gchar *url)
+GError*
+gridd_client_connect_url(struct gridd_client_s *client, const gchar *url)
 {
 	EXTRA_ASSERT(client != NULL);
-	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
 
 	if (NULL == url || !url[0])
 		return NEWERROR(CODE_INTERNAL_ERROR, "Bad address");
@@ -685,14 +630,13 @@ _client_connect_url(struct gridd_client_s *client, const gchar *url)
 	return NULL;
 }
 
-static GError*
-_client_request(struct gridd_client_s *client, GByteArray *req,
+GError*
+gridd_client_request(struct gridd_client_s *client, GByteArray *req,
 		gpointer ctx, client_on_reply cb)
 {
 	EXTRA_ASSERT(client != NULL);
-	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
 
-	if ( NULL == req)
+	if (NULL == req)
 		return NEWERROR(CODE_INTERNAL_ERROR, "Invalid parameter");
 
 	switch (client->step) {
@@ -728,11 +672,11 @@ _client_request(struct gridd_client_s *client, GByteArray *req,
 	return NULL;
 }
 
-static gboolean
-_client_expired(struct gridd_client_s *client, gint64 now)
+gboolean
+gridd_client_expired(struct gridd_client_s *client, gint64 now)
 {
 	EXTRA_ASSERT(client != NULL);
-	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
+
 	switch (client->step) {
 		case NONE:
 			return FALSE;
@@ -776,18 +720,17 @@ _client_react_timeout(struct gridd_client_s *client)
 	client->step = STATUS_FAILED;
 }
 
-static gboolean
-_client_expire(struct gridd_client_s *client, gint64 now)
+gboolean
+gridd_client_expire(struct gridd_client_s *client, gint64 now)
 {
 	EXTRA_ASSERT(client != NULL);
-	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
 
-	if (_client_finished(client))
+	if (gridd_client_finished(client))
 		return FALSE;
 
-	if (!_client_expired(client, now)) {
+	if (!gridd_client_expired(client, now)) {
 #ifdef HAVE_ENBUG
-		if (oio_client_timeout_threshold >= oio_ext_rand_int_range(1,100)) {
+		if (oio_client_fake_timeout_threshold >= oio_ext_rand_int_range(1,100)) {
 			_client_react_timeout(client);
 			return TRUE;
 		}
@@ -799,11 +742,10 @@ _client_expire(struct gridd_client_s *client, gint64 now)
 	return TRUE;
 }
 
-static gboolean
-_client_finished(struct gridd_client_s *client)
+gboolean
+gridd_client_finished(struct gridd_client_s *client)
 {
 	EXTRA_ASSERT(client != NULL);
-	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
 
 	if (client->error != NULL)
 		return TRUE;
@@ -829,28 +771,43 @@ _client_finished(struct gridd_client_s *client)
 	return FALSE;
 }
 
-static gboolean
-_client_start(struct gridd_client_s *client)
+gboolean
+gridd_client_start(struct gridd_client_s *client)
 {
 	EXTRA_ASSERT(client != NULL);
-	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
 
 	client->tv_start = client->tv_connect = oio_ext_monotonic_time ();
 
-	if (client->step != NONE)
-		return FALSE;
-
-	if (!client->url[0]) {
-		_client_replace_error(client, NEWERROR(EINVAL, "No target"));
+	if (client->step != NONE) {
+		_client_replace_error(client, SYSERR("bug: invalid client state"));
 		return FALSE;
 	}
 
-	if (oio_cache_avoid_on_error && client->avoid_on_error) {
-		if (_has_too_many_errors(client->url)) {
+	if (!client->url[0]) {
+		_client_replace_error(client, SYSERR("bug: no client target"));
+		return FALSE;
+	}
+
+	if (client->avoidance_onoff) {
+		if (oio_client_cache_errors && _has_too_many_errors(client->url)) {
 			_client_replace_error(client, NEWERROR(CODE_AVOIDED,
 						"Request avoided, service probably down"));
 			return FALSE;
 		}
+
+		if (oio_client_down_avoid || oio_client_down_shorten) {
+			const gboolean down = _is_peer_down(client->url);
+			if (down) {
+				if (oio_client_down_avoid) {
+					_client_replace_error(client, NEWERROR(CODE_AVOIDED,
+								"Request avoided, service marked down"));
+					return FALSE;
+				}
+				if (oio_client_down_shorten) {
+					client->delay_connect = 25 * G_TIME_SPAN_MILLISECOND;
+				}
+			}
+		}
 	}
 
 	GError *err = _client_connect(client);
@@ -862,48 +819,50 @@ _client_start(struct gridd_client_s *client)
 	return FALSE;
 }
 
-static void
-_client_fail(struct gridd_client_s *client, GError *why)
+void
+gridd_client_fail(struct gridd_client_s *client, GError *why)
 {
 	EXTRA_ASSERT(client != NULL);
-	EXTRA_ASSERT(client->abstract.vtable == &VTABLE_CLIENT);
 	_client_replace_error(client, why);
 }
 
-static void
-_factory_clean(struct gridd_client_factory_s *self)
-{
-	EXTRA_ASSERT(self != NULL);
-	EXTRA_ASSERT(self->abstract.vtable == &VTABLE_FACTORY);
-	SLICE_FREE(struct gridd_client_factory_s, self);
-}
-
-static struct gridd_client_s *
-_factory_create_client (struct gridd_client_factory_s *factory)
-{
-	EXTRA_ASSERT(factory != NULL);
-	EXTRA_ASSERT(factory->abstract.vtable == &VTABLE_FACTORY);
-	(void) factory;
-	return gridd_client_create_empty();
-}
-
 /* ------------------------------------------------------------------------- */
 
+void
+gridd_client_learn_peers_down(const char * const * peers)
+{
+	GTree *new_down = NULL, *old_down = NULL;
+
+	new_down = g_tree_new_full(metautils_strcmp3, NULL, g_free, NULL);
+	for (const char * const * ppeer = peers; peers && *ppeer ;++ppeer)
+		g_tree_replace(new_down, g_strdup(*ppeer), (void*)0xDEADBEAF);
+
+	g_rw_lock_writer_lock(&lock_down);
+	old_down = tree_down;
+	tree_down = new_down;
+	new_down = NULL;
+	g_rw_lock_writer_unlock(&lock_down);
+
+	g_tree_destroy(old_down);
+}
+
 struct gridd_client_s *
 gridd_client_create_empty(void)
 {
 	struct gridd_client_s *client = SLICE_NEW0(struct gridd_client_s);
-	if (unlikely(NULL == client))
+	if (unlikely(!client))
 		return NULL;
 
-	client->abstract.vtable = &VTABLE_CLIENT;
 	client->fd = -1;
 	client->step = NONE;
-	client->delay_overall = COMMON_CLIENT_TIMEOUT * (gdouble)G_TIME_SPAN_SECOND;
-	client->delay_single = COMMON_CLIENT_TIMEOUT * (gdouble)G_TIME_SPAN_SECOND;
-	client->delay_connect = COMMON_CNX_TIMEOUT * (gdouble)G_TIME_SPAN_SECOND;
+	client->delay_overall = oio_client_timeout_whole * (gdouble)G_TIME_SPAN_SECOND;
+	client->delay_single = oio_client_timeout_single * (gdouble)G_TIME_SPAN_SECOND;
+	client->delay_connect = oio_client_timeout_connect * (gdouble)G_TIME_SPAN_SECOND;
 	client->tv_start = client->tv_connect = oio_ext_monotonic_time ();
-	client->avoid_on_error = BOOL(oio_cache_avoid_on_error);
+
+	client->keepalive = 0;
+	client->forbid_redirect = 0;
+	client->avoidance_onoff = 1;
 
 	return client;
 }
@@ -911,121 +870,21 @@ gridd_client_create_empty(void)
 void
 gridd_client_no_redirect (struct gridd_client_s *c)
 {
-	if (!c) return;
-	EXTRA_ASSERT(c->abstract.vtable == &VTABLE_CLIENT);
+	if (unlikely(!c)) return;
 	c->forbid_redirect = 1;
 }
 
-struct gridd_client_factory_s *
-gridd_client_factory_create(void)
-{
-	struct gridd_client_factory_s *factory = SLICE_NEW0(struct gridd_client_factory_s);
-	factory->abstract.vtable = &VTABLE_FACTORY;
-	return factory;
-}
-
-#define GRIDD_CALL(self,F) VTABLE_CALL(self,struct abstract_client_s*,F)
-
 void
-gridd_client_free (struct gridd_client_s *self)
+gridd_client_set_keepalive(struct gridd_client_s *c, gboolean onoff)
 {
-	GRIDD_CALL(self,clean)(self);
-}
-
-GError *
-gridd_client_connect_url (struct gridd_client_s *self, const gchar *u)
-{
-	GRIDD_CALL(self,connect_url)(self,u);
-}
-
-GError *
-gridd_client_request (struct gridd_client_s *self, GByteArray *req,
-		gpointer ctx, client_on_reply cb)
-{
-	GRIDD_CALL(self,request)(self,req,ctx,cb);
-}
-
-GError *
-gridd_client_error (struct gridd_client_s *self)
-{
-	GRIDD_CALL(self,error)(self);
-}
-
-int
-gridd_client_interest (struct gridd_client_s *self)
-{
-	GRIDD_CALL(self,interest)(self);
-}
-
-const gchar *
-gridd_client_url (struct gridd_client_s *self)
-{
-	GRIDD_CALL(self,get_url)(self);
-}
-
-int
-gridd_client_fd (struct gridd_client_s *self)
-{
-	GRIDD_CALL(self,get_fd)(self);
-}
-
-GError *
-gridd_client_set_fd(struct gridd_client_s *self, int fd)
-{
-	GRIDD_CALL(self,set_fd)(self,fd);
+	if (unlikely(!c)) return;
+	c->keepalive = BOOL(onoff);
 }
 
 void
-gridd_client_set_keepalive(struct gridd_client_s *self, gboolean on)
+gridd_client_set_avoidance (struct gridd_client_s *c, gboolean onoff)
 {
-	GRIDD_CALL(self,set_keepalive)(self,on);
-}
-
-void
-gridd_client_set_timeout (struct gridd_client_s *self, gdouble seconds)
-{
-	GRIDD_CALL(self,set_timeout)(self,seconds);
-}
-
-void
-gridd_client_set_timeout_cnx (struct gridd_client_s *self, gdouble seconds)
-{
-	GRIDD_CALL(self,set_timeout_cnx)(self,seconds);
-}
-
-gboolean
-gridd_client_expired(struct gridd_client_s *self, gint64 now)
-{
-	GRIDD_CALL(self,expired)(self,now);
-}
-
-gboolean
-gridd_client_finished (struct gridd_client_s *self)
-{
-	GRIDD_CALL(self,finished)(self);
-}
-
-gboolean
-gridd_client_start (struct gridd_client_s *self)
-{
-	GRIDD_CALL(self,start)(self);
-}
-
-gboolean
-gridd_client_expire (struct gridd_client_s *self, gint64 now)
-{
-	GRIDD_CALL(self,expire)(self,now);
-}
-
-void
-gridd_client_react (struct gridd_client_s *self)
-{
-	GRIDD_CALL(self,react)(self);
-}
-
-void
-gridd_client_fail (struct gridd_client_s *self, GError *why)
-{
-	GRIDD_CALL(self,fail)(self,why);
+	if (unlikely(!c)) return;
+	c->avoidance_onoff = BOOL(onoff);
 }
 
