@@ -1,7 +1,7 @@
 /*
 OpenIO SDS metautils
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -48,7 +48,7 @@ gridd_client_create_idle(const gchar *target)
 
 struct gridd_client_s *
 gridd_client_create(const gchar *target, GByteArray *req, gpointer ctx,
-                client_on_reply cb)
+		client_on_reply cb)
 {
 	EXTRA_ASSERT(req != NULL);
 
@@ -80,7 +80,7 @@ gridd_client_loop(struct gridd_client_s *client)
 
 struct gridd_client_s **
 gridd_client_create_many(gchar **targets, GByteArray *req, gpointer ctx,
-                client_on_reply cb)
+		client_on_reply cb)
 {
 	EXTRA_ASSERT(targets != NULL);
 	EXTRA_ASSERT(req != NULL);
@@ -324,13 +324,17 @@ GError *
 gridd_client_run (struct gridd_client_s *self)
 {
 	if (!self)
-		return NEWERROR(CODE_INTERNAL_ERROR, "creation error");
-	if (!gridd_client_start(self))
-		return NEWERROR(CODE_INTERNAL_ERROR, "starting error");
+		return SYSERR("creation error");
 	GError *err;
-	if (NULL != (err = gridd_client_loop (self)))
+	if (!gridd_client_start(self)) {
+		if ((err = gridd_client_error(self)))
+			return err;
+		/* TODO good place for a g_assert_not_reached(); */
+		return SYSERR("client startup: unknown error");
+	}
+	if ((err = gridd_client_loop(self)))
 		return err;
-	if (NULL != (err = gridd_client_error (self)))
+	if ((err = gridd_client_error(self)))
 		return err;
 	return NULL;
 }
@@ -415,7 +419,7 @@ gridd_client_exec_and_concat (const gchar *to, gdouble seconds, GByteArray *req,
 
 	GByteArray *tmp = NULL;
 	if (out)
-		tmp = g_byte_array_new();
+		tmp = g_byte_array_sized_new(512);
 
 	struct gridd_client_s *client = gridd_client_create(to, req,
 			out ? tmp : NULL, out ? (client_on_reply)_cb_exec_and_concat : NULL);
@@ -438,24 +442,6 @@ gridd_client_exec_and_concat (const gchar *to, gdouble seconds, GByteArray *req,
 	return err;
 }
 
-GError *
-gridd_client_exec_and_concat_string (const gchar *to, gdouble seconds, GByteArray *req,
-		gchar **out)
-{
-	GByteArray *tmp = NULL;
-	GError *err = gridd_client_exec_and_concat (to, seconds, req, out ? &tmp : NULL);
-
-	if (err) {
-		if (tmp) g_byte_array_unref (tmp);
-		return err;
-	}
-	if (out) {
-		g_byte_array_append (tmp, (guint8*)"", 1);
-		*out = (gchar*) g_byte_array_free (tmp, FALSE);
-	}
-	return NULL;
-}
-
 GError *
 gridd_client_exec_and_decode (const gchar *to, gdouble seconds,
 		GByteArray *req, GSList **out, body_decoder_f decode)
