@@ -1,7 +1,7 @@
 /*
 OpenIO SDS metautils
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -58,10 +58,6 @@ GError * gridd_client_exec_and_decode (const char *to, gdouble timeout,
 GError * gridd_client_exec_and_concat (const char *to, gdouble timeout,
 		GByteArray *req, GByteArray **out);
 
-/* wraps gridd_client_exec_and_concat() */
-GError * gridd_client_exec_and_concat_string (const char *to, gdouble timeout, GByteArray *req,
-		gchar **out);
-
 /* Implementation specifics / array of structures -------------------------- */
 
 // @return NULL if one of the subsequent client creation fails
