@@ -1,7 +1,7 @@
 /*
 OpenIO SDS metautils
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -115,6 +115,10 @@ GError* metautils_message_extract_flags32(MESSAGE msg, const gchar *n,
 GError* metautils_message_extract_string(MESSAGE msg, const gchar *n, gchar *dst,
 		gsize dst_size);
 
+/** Returns TRUE on success, FALSE otherwise */
+gboolean metautils_message_extract_string_noerror(MESSAGE msg, const gchar *n,
+		gchar *dst, gsize dst_size);
+
 gchar* metautils_message_extract_string_copy(MESSAGE msg, const gchar *n);
 
 GError* metautils_message_extract_strint64(MESSAGE msg, const gchar *n,
