@@ -1,7 +1,7 @@
 /*
 OpenIO SDS metautils
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -22,44 +22,57 @@ License along with this library.
 
 # include <glib.h>
 
-/**
- * @defgroup metautils_types Metatypes
- * @ingroup metautils
- * @{
- */
-
-/** The maximum length of a volume name */
+/** One plus the maximum length of a volume name
+ * i.e. a size enough to store the C string */
 #define LIMIT_LENGTH_VOLUMENAME 256
 
-/** The maximum length of a namespace name */
+/** One plus the maximum length of a namespace name
+ * i.e. a size enough to store the C string */
 #define LIMIT_LENGTH_NSNAME 64
 
-/** The maximum length of a content name */
+/** One plus the maximum length of an account name
+ * i.e. a size enough to store the C string */
+#define LIMIT_LENGTH_ACCOUNTNAME 64
+
+/** One plus the maximum length of a content name
+ * i.e. a size enough to store the C string */
 #define LIMIT_LENGTH_CONTENTPATH 1024
 
-/** The maximum length of a storage policy name */
+/** One plus the maximum length of the string representation of a content version
+ * i.e. a size enough to store the C string */
+#define LIMIT_LENGTH_VERSION 24
+
+/** One plus the maximum length of a storage policy name
+ * i.e. a size enough to store the C string */
 #define LIMIT_LENGTH_STGPOLICY 32
 
-/** The maximum length of a service tag name */
+/** One plus the maximum length of a service tag name
+ * i.e. a size enough to store the C string */
 #define LIMIT_LENGTH_TAGNAME 32
 
+/** One plus the maximum length of a content name
+ * i.e. a size enough to store the C string */
 #define LIMIT_LENGTH_CHUNKURL 512
 
+/** One plus the maximum length of a database name
+ * i.e. a size enough to store the C string */
 #define LIMIT_LENGTH_BASENAME 256
 
-#define LIMIT_LENGTH_BASETYPE 32
-
-/** The maximum length of a service type name */
-#define LIMIT_LENGTH_SRVID 64
-
-/** The maximum length of a service id */
+/** One plus the maximum length of a service type name, with all its subtypes
+ * i.e. a size enough to store the C string */
 #define LIMIT_LENGTH_SRVTYPE 32
 
-/** The maximum length of service options */
-#define LIMIT_LENGTH_SRVARGS 256
+/** One plus the maximum length of a database type, including all its subtypes
+ * i.e. a size enough to store the C string */
+#define LIMIT_LENGTH_BASETYPE 32
 
-/** The maximum length for values of 'admin' table */
-#define LIMIT_LENGTH_ADMIN_VALUE 1024
+/** One plus the maximum length of the textual representation of a service id
+ * i.e. a size enough to store the C string */
+#define LIMIT_LENGTH_SRVID 64
+
+/** One plus the maximum length of a single service option
+ * i.e. a size enough to store the C string */
+#define LIMIT_LENGTH_SRVARGS 256
 
 #define TYPE_TO_STRLEN(T)  ((sizeof(T)*2)+1)
 #define STRLEN_CHUNKID     TYPE_TO_STRLEN(hash_sha256_t)
@@ -139,8 +152,6 @@ typedef struct addr_info_s
 typedef struct namespace_info_s
 {
 	gchar name[LIMIT_LENGTH_NSNAME]; /**< The namespace name */
-	chunk_size_t chunk_size;	 /**< The chunk size in the namespace */
-	GHashTable* options;         /**< A hash of namespace options (gchar*) -> (GByteArray*) */
 	GHashTable* storage_policy;	 /**< Storage policies definitions name = STG_CLASS:DATA_SEC:DATA_THREAT */
 	GHashTable* data_security;	 /**< Data security definitions name = TYPE:OTHER_INFO */
 	GHashTable* service_pools;   /**< Service policies definitions name = GSList<char*> */
@@ -174,6 +185,15 @@ typedef struct key_value_pair_s
 	GByteArray *value;	/**< The value */
 } key_value_pair_t;
 
+enum service_tag_value_type_e
+{
+	STVT_I64 = 1,	/**< An int64 */
+	STVT_REAL = 2,	/**< A double */
+	STVT_BOOL = 3,	/**< A boolean */
+	STVT_STR = 4,	/**< A string */
+	STVT_BUF = 5,	/**< A bin buffer */
+};
+
 /**
  * Type to store a service tag
  */
@@ -181,20 +201,8 @@ typedef struct service_tag_s
 {
 	char name[LIMIT_LENGTH_TAGNAME];	/**< The tag name */
 
-	/**
-	 * The list of tag value types
-	 */
-	enum service_tag_value_type_e
-	{
-		STVT_I64 = 1,	/**< An int64 */
-		STVT_REAL = 2,	/**< A double */
-		STVT_BOOL = 3,	/**< A boolean */
-		STVT_STR = 4,	/**< A string */
-		STVT_BUF = 5,	/**< A bin buffer */
-	} type;					/**< The tag type */
-	/**
-	 * Type to store a tag value
-	 */
+	enum service_tag_value_type_e type;
+
 	union
 	{
 		gint64 i;	/**< The int64 representation */
@@ -227,6 +235,4 @@ typedef struct addr_rule_s
 	gboolean authorize;  /**< Allow (TRUE) or deny (FALSE) */
 } addr_rule_t;
 
-/** @} */
-
 #endif /*OIO_SDS__metautils__lib__metatypes_h*/
