@@ -1,7 +1,7 @@
 /*
 OpenIO SDS metautils
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -34,7 +34,6 @@ License along with this library.
 
 # include <metautils/lib/metatypes.h>
 
-# include <metautils/lib/metautils_bits.h>
 # include <metautils/lib/metautils_errors.h>
 # include <metautils/lib/metautils_strings.h>
 # include <metautils/lib/metautils_sockets.h>
@@ -54,7 +53,6 @@ License along with this library.
 # include <metautils/lib/metatype_addrinfo.h>
 # include <metautils/lib/metatype_kv.h>
 # include <metautils/lib/metatype_metadata.h>
-# include <metautils/lib/metatype_acl.h>
 
 # include <metautils/lib/lrutree.h>
 # include <metautils/lib/storage_policy.h>
