@@ -1,7 +1,7 @@
 /*
 OpenIO SDS metautils
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -38,7 +38,7 @@ License along with this library.
 #define STRINGV_STACKIFY(V) do { \
 	if (!(V)) break; \
 	BUFFER_STACKIFY((V), (1+g_strv_length(V))*sizeof(void*)); \
-	for (gchar **p=(V); *p ;++p) { STRING_STACKIFY(*p); } \
+	for (gchar **__p=(V); *__p ;++__p) { STRING_STACKIFY(*__p); } \
 } while (0)
 
 #define metautils_str_upper oio_str_upper
