@@ -1,7 +1,7 @@
 /*
 OpenIO SDS metautils
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -69,6 +69,9 @@ const char * storage_policy_get_name(const struct storage_policy_s *sp);
  * (nb_copy for plain, k+m for EC). */
 gint64 storage_policy_get_nb_chunks(const struct storage_policy_s *sp);
 
+/** Get the minimum distance between chunks required by the storage policy. */
+gint64 storage_policy_get_distance(const struct storage_policy_s *sp);
+
 const struct data_security_s *storage_policy_get_data_security(
 		const struct storage_policy_s *sp);
 
@@ -83,4 +86,24 @@ const char * data_security_get_param(const struct data_security_s *ds,
 gint64 data_security_get_int64_param(const struct data_security_s *ds,
 		const char *key, gint64 def);
 
+static inline gint64
+data_security_decode_param_int64 (const char *encoded, const char *k, gint64 def)
+{
+	gint64 rc = def;
+	const char *params = strchr(encoded, '/') + 1;
+	gchar **tokens = g_strsplit(params, OIO_CSV_SEP, -1);
+	gchar *prefix = g_strdup_printf("%s=", k);
+	for (gchar **pt=tokens; tokens && *pt ;++pt) {
+		if (g_str_has_prefix(*pt, prefix)) {
+			const char *value = strchr(*pt, '=') + 1;
+			if (oio_str_is_number(value, &rc))
+				break;
+		}
+	}
+	if (tokens)
+		g_strfreev(tokens);
+	g_free(prefix);
+	return rc;
+}
+
 #endif /*OIO_SDS__metautils__lib__storage_policy_h*/
