@@ -1,7 +1,7 @@
 /*
 OpenIO SDS metautils
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -20,6 +20,7 @@ License along with this library.
 #include <errno.h>
 
 #include "metautils.h"
+#include <metautils/lib/common_variables.h>
 
 #include <json.h>
 
@@ -47,7 +48,6 @@ namespace_info_copy(namespace_info_t* src, namespace_info_t* dst)
 	EXTRA_ASSERT(dst != NULL);
 
 	memcpy(dst->name, src->name, sizeof(src->name));
-	dst->chunk_size = src->chunk_size;
 
 #define NSI_COPY_TABLE_REF(SRC, DST) \
 	if ((SRC) != NULL) {\
@@ -56,7 +56,6 @@ namespace_info_copy(namespace_info_t* src, namespace_info_t* dst)
 		if (old) g_hash_table_unref(old);\
 	}
 
-	NSI_COPY_TABLE_REF(src->options, dst->options);
 	NSI_COPY_TABLE_REF(src->storage_policy, dst->storage_policy);
 	NSI_COPY_TABLE_REF(src->data_security, dst->data_security);
 	NSI_COPY_TABLE_REF(src->service_pools, dst->service_pools);
@@ -69,9 +68,7 @@ namespace_info_dup(namespace_info_t* src)
 {
 	namespace_info_t *dst = g_malloc0(sizeof(namespace_info_t));
 	memcpy(dst->name, src->name, sizeof(src->name));
-	dst->chunk_size = src->chunk_size;
 
-	dst->options = _copy_hash(src->options);
 	dst->storage_policy = _copy_hash(src->storage_policy);
 	dst->data_security = _copy_hash(src->data_security);
 	dst->service_pools = _copy_hash(src->service_pools);
@@ -83,8 +80,6 @@ namespace_info_clear(namespace_info_t* ns_info)
 {
 	if (ns_info == NULL)
 		return;
-	if (ns_info->options != NULL)
-		g_hash_table_unref(ns_info->options);
 	if (ns_info->storage_policy != NULL)
 		g_hash_table_unref(ns_info->storage_policy);
 	if (ns_info->data_security != NULL)
@@ -100,9 +95,7 @@ namespace_info_init(namespace_info_t *ni)
 {
 	if (!ni)
 		return;
-	ni->chunk_size = 0;
 	memset(ni->name, 0, sizeof(ni->name));
-	ni->options = _copy_hash(NULL);
 	ni->storage_policy = _copy_hash(NULL);
 	ni->data_security = _copy_hash(NULL);
 	ni->service_pools = _copy_hash(NULL);
@@ -140,57 +133,6 @@ namespace_info_get_data_security(namespace_info_t *ni, const gchar *data_sec_key
 	return NULL;
 }
 
-GByteArray *
-namespace_info_get_srv_param_gba(const namespace_info_t *ni,
-		const gchar *ns_name, const gchar *srv_type, const gchar *param_name)
-{
-	(void) ns_name;  // not used anymore
-
-	gchar key[128] = {0};
-	GByteArray *res = NULL;
-
-	if (!ni || !ni->options)
-		return NULL;
-
-	if (srv_type != NULL) {
-		// Prefix the param name with the service type
-		g_snprintf(key, sizeof(key), "%s.%s", srv_type, param_name);
-		res = g_hash_table_lookup(ni->options, key);
-	}
-
-	if (!res) {
-		// Try with unprefixed param name
-		res = g_hash_table_lookup(ni->options, param_name);
-	}
-
-	return res;
-}
-
-gint64
-namespace_info_get_srv_param_i64(const namespace_info_t *ni,
-		const gchar *ns_name, const gchar *srv_type,
-		const gchar *param_name, gint64 def)
-{
-	gint64 res;
-	gchar *end = NULL;
-	GByteArray *value;
-
-	if (!ni || !ni->options)
-		return def;
-
-	value = namespace_info_get_srv_param_gba(ni, ns_name, srv_type, param_name);
-	if (!value)
-		return def;
-
-	gchar *v = g_strndup((gchar*)value->data, value->len);
-	res = g_ascii_strtoll(v, &end, 10);
-	if (end == v) // Conversion failed
-		res = def;
-	g_free(v);
-
-	return res;
-}
-
 //------------------------------------------------------------------------------
 
 static GError *
@@ -217,20 +159,17 @@ namespace_info_init_json_object(struct json_object *obj,
 {
 	EXTRA_ASSERT(ni != NULL);
 
-	struct json_object *ns=NULL, *sz=NULL;
+	struct json_object *ns=NULL;
 	struct oio_ext_json_mapping_s mapping[] = {
 		{"ns",        &ns, json_type_string, 1},
-		{"chunksize", &sz, json_type_int,    1},
 		{NULL, NULL, 0, 0}
 	};
 	GError *err = oio_ext_extract_json (obj, mapping);
 	if (err) return err;
 
 	g_strlcpy(ni->name, json_object_get_string(ns), sizeof(ni->name));
-	ni->chunk_size = json_object_get_int64(sz);
 
-	if (NULL != (err = _load_hash(obj, "options", ni->options))
-			|| NULL != (err = _load_hash(obj, "storage_policy", ni->storage_policy))
+	if (NULL != (err = _load_hash(obj, "storage_policy", ni->storage_policy))
 			|| NULL != (err = _load_hash(obj, "data_security", ni->data_security))
 			|| NULL != (err = _load_hash(obj, "service_pools", ni->service_pools)))
 		return err;
@@ -267,7 +206,9 @@ namespace_info_init_json(const gchar *encoded, struct namespace_info_s *ni)
 static void
 _encode_json_properties (GString *out, GHashTable *ht, const gchar *tag)
 {
-	g_string_append_printf(out, "\"%s\":{", tag);
+	g_string_append_c(out, '"');
+	g_string_append(out, tag);
+	g_string_append_static(out, "\":{");
 	if (ht && g_hash_table_size(ht) > 0) {
 		GHashTableIter iter;
 		gpointer k, v;
@@ -278,22 +219,41 @@ _encode_json_properties (GString *out, GHashTable *ht, const gchar *tag)
 			if (!first)
 				g_string_append_c(out, ',');
 			first = FALSE;
-			g_string_append_printf(out, "\"%s\":\"%.*s\"", (gchar*)k,
-					gba->len, (gchar*)(gba->data));
+			oio_str_gstring_append_json_quote(out, (gchar*)k);
+			g_string_append_static(out, ":\"");
+			/* If we do not trust 'by default' what has been configured
+			 * for the namespace, we cannot encode the values as is because
+			 * they often contain a final '\0' that has been added in earlier
+			 * times for more robustness */
+			guint len = gba->len;
+			while (len && !gba->data[len-1]) { len --; }
+			oio_str_gstring_append_json_blob(out, (gchar*)(gba->data), len);
+			g_string_append_c(out, '"');
 		}
 	}
-	g_string_append(out, "}");
+	g_string_append_c(out, '}');
 }
 
 void
 namespace_info_encode_json(GString *out, struct namespace_info_s *ni)
 {
 	g_string_append_c(out, '{');
-	g_string_append_printf(out, "\"ns\":\"%s\",", ni->name);
-	g_string_append_printf(out, "\"chunksize\":%"G_GINT64_FORMAT",",
-			ni->chunk_size);
+	OIO_JSON_append_str(out, "ns", ni->name);
+
+	/* for a smooth transition, also send the centralized values */
+	g_string_append_c(out, ',');
+	oio_str_gstring_append_json_pair_int(out, "chunksize", oio_ns_chunk_size);
+	g_string_append(out, ",\"options\":{");
+	oio_str_gstring_append_json_pair_int(out,
+			"flat_bitlength", oio_ns_flat_bits);
+	g_string_append_c(out, ',');
+	oio_str_gstring_append_json_pair(out,
+			"storage_policy", oio_ns_storage_policy);
+	g_string_append_c(out, ',');
+	oio_str_gstring_append_json_pair(out,
+			"service_update_policy", oio_ns_service_update_policy);
+	g_string_append_c(out, '}');
 
-	_encode_json_properties(out, ni->options, "options");
 	g_string_append_c(out, ',');
 	_encode_json_properties(out, ni->storage_policy, "storage_policy");
 	g_string_append_c(out, ',');
