@@ -1,7 +1,7 @@
 /*
 OpenIO SDS metautils
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -28,13 +28,7 @@ License along with this library.
 #include "metautils.h"
 #include "metautils_syscall.h"
 
-#ifndef SOCK_DEFAULT_LINGER_ONOFF
-# define SOCK_DEFAULT_LINGER_ONOFF 1
-#endif
-
-#ifndef SOCK_DEFAULT_LINGER_DELAY
-# define SOCK_DEFAULT_LINGER_DELAY 0
-#endif
+#include <metautils/lib/common_variables.h>
 
 static gint
 errno_to_errcode(int e)
@@ -82,8 +76,10 @@ metautils_get_vtable_sockets(void)
 int
 socket_nonblock(int domain, int type, int protocol)
 {
+#ifdef HAVE_EXTRA_DEBUG
 	if (VTABLE.socket_nonblock)
 		return VTABLE.socket_nonblock(domain, type, protocol);
+#endif
 #ifdef HAVE_SOCKET3
 	return metautils_syscall_socket(domain, type|SOCK_NONBLOCK, protocol);
 #else
@@ -100,8 +96,10 @@ socket_nonblock(int domain, int type, int protocol)
 int
 accept_nonblock(int srv, struct sockaddr *sa, socklen_t *sa_len)
 {
+#ifdef HAVE_EXTRA_DEBUG
 	if (VTABLE.accept_nonblock)
 		return VTABLE.accept_nonblock(srv, sa, sa_len);
+#endif
 #ifdef HAVE_ACCEPT4
 	return metautils_syscall_accept4(srv, sa, sa_len, SOCK_NONBLOCK);
 #else
@@ -115,8 +113,10 @@ accept_nonblock(int srv, struct sockaddr *sa, socklen_t *sa_len)
 gint
 sock_to_write(int fd, gint ms, void *buf, gsize bufSize, GError ** err)
 {
+#ifdef HAVE_EXTRA_DEBUG
 	if (VTABLE.to_write)
 		return VTABLE.to_write(fd, ms, buf, bufSize, err);
+#endif
 
 #define WRITE() do { \
 		written = metautils_syscall_write(fd, ((guint8 *)buf) + nbSent, bufSize - nbSent); \
@@ -165,7 +165,7 @@ sock_to_write(int fd, gint ms, void *buf, gsize bufSize, GError ** err)
 				return (-1);
 			}
 			else {
-				TRACE("poll interrupted (%s)", strerror(errno));
+				GRID_TRACE("poll interrupted (%s)", strerror(errno));
 				continue;
 			}
 		}
@@ -194,8 +194,10 @@ sock_to_write(int fd, gint ms, void *buf, gsize bufSize, GError ** err)
 gint
 sock_to_read(int fd, gint ms, void *buf, gsize bufSize, GError ** err)
 {
+#ifdef HAVE_EXTRA_DEBUG
 	if (VTABLE.to_read)
 		return VTABLE.to_read(fd, ms, buf, bufSize, err);
+#endif
 
 #define READ() do { \
 		rc = metautils_syscall_read(fd, buf, bufSize); \
@@ -258,8 +260,10 @@ sock_to_read(int fd, gint ms, void *buf, gsize bufSize, GError ** err)
 gint
 sock_to_read_size(int fd, gint ms, void *buf, gsize bufSize, GError ** err)
 {
+#ifdef HAVE_EXTRA_DEBUG
 	if (VTABLE.to_read_size)
 		return VTABLE.to_read_size(fd, ms, buf, bufSize, err);
+#endif
 
 	gsize nbRead = 0;
 
@@ -284,8 +288,10 @@ sock_to_read_size(int fd, gint ms, void *buf, gsize bufSize, GError ** err)
 gint
 socket_get_errcode(int fd)
 {
+#ifdef HAVE_EXTRA_DEBUG
 	if (VTABLE.get_error)
 		return VTABLE.get_error(fd);
+#endif
 
 	int rc, sock_err = 0;
 	socklen_t sock_err_size = sizeof(sock_err);
@@ -308,8 +314,10 @@ socket_get_error(int fd)
 gboolean
 sock_set_tcpquickack(int fd, gboolean enabled)
 {
+#ifdef HAVE_EXTRA_DEBUG
 	if (VTABLE.set_tcpquickack)
 		return VTABLE.set_tcpquickack(fd, enabled);
+#endif
 
 	int opt = BOOL(enabled);
 	if (!metautils_syscall_setsockopt(fd, IPPROTO_TCP, TCP_QUICKACK, (void*)&opt, sizeof(opt)))
@@ -322,8 +330,10 @@ sock_set_tcpquickack(int fd, gboolean enabled)
 gboolean
 sock_set_non_blocking(int fd, gboolean enabled)
 {
+#ifdef HAVE_EXTRA_DEBUG
 	if (VTABLE.set_non_blocking)
 		return VTABLE.set_non_blocking(fd, enabled);
+#endif
 
 	if (fd < 0) {
 		errno = EAGAIN;
@@ -344,8 +354,10 @@ sock_set_non_blocking(int fd, gboolean enabled)
 gboolean
 sock_set_reuseaddr(int fd, gboolean enabled)
 {
+#ifdef HAVE_EXTRA_DEBUG
 	if (VTABLE.set_reuseaddr)
 		return VTABLE.set_reuseaddr(fd, enabled);
+#endif
 
 	int opt = BOOL(enabled);
 	if (!metautils_syscall_setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (void*)&opt, sizeof(opt)))
@@ -358,8 +370,10 @@ sock_set_reuseaddr(int fd, gboolean enabled)
 gboolean
 sock_set_keepalive(int fd, gboolean enabled)
 {
+#ifdef HAVE_EXTRA_DEBUG
 	if (VTABLE.set_keepalive)
 		return VTABLE.set_keepalive(fd, enabled);
+#endif
 
 	int opt = BOOL(enabled);
 	if (!metautils_syscall_setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, (void*)&opt, sizeof(opt)))
@@ -372,8 +386,10 @@ sock_set_keepalive(int fd, gboolean enabled)
 gboolean
 sock_set_nodelay(int fd, gboolean enabled)
 {
+#ifdef HAVE_EXTRA_DEBUG
 	if (VTABLE.set_nodelay)
 		return VTABLE.set_nodelay(fd, enabled);
+#endif
 
 	int opt = BOOL(enabled);
 	if (!metautils_syscall_setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, (void*)&opt, sizeof(opt)))
@@ -386,8 +402,10 @@ sock_set_nodelay(int fd, gboolean enabled)
 gboolean
 sock_set_cork(int fd, gboolean enabled)
 {
+#ifdef HAVE_EXTRA_DEBUG
 	if (VTABLE.set_cork)
 		return VTABLE.set_cork(fd, enabled);
+#endif
 
 	int opt = BOOL(enabled);
 	if (!metautils_syscall_setsockopt(fd, IPPROTO_TCP, TCP_CORK, (void*)&opt, sizeof(opt)))
@@ -397,11 +415,27 @@ sock_set_cork(int fd, gboolean enabled)
 	return FALSE;
 }
 
+gboolean
+sock_set_fastopen(int fd)
+{
+	int syndata_backlog = 16;
+	int rc = metautils_syscall_setsockopt(fd, SOL_TCP, TCP_FASTOPEN,
+			&syndata_backlog, sizeof(syndata_backlog));
+	if (!rc)
+		return TRUE;
+
+	GRID_DEBUG("fd=%i set(TCP_FASTOPEN,%d): (%d) %s",
+			fd, syndata_backlog, errno, strerror(errno));
+	return FALSE;
+}
+
 gboolean
 sock_set_linger(int fd, int onoff, int linger)
 {
+#ifdef HAVE_EXTRA_DEBUG
 	if (VTABLE.set_linger)
 		return VTABLE.set_linger(fd, onoff, linger);
+#endif
 
 	struct linger ls;
 	ls.l_onoff = BOOL(onoff);
@@ -417,16 +451,17 @@ sock_set_linger(int fd, int onoff, int linger)
 gboolean
 sock_set_linger_default(int fd)
 {
-	return sock_set_linger(fd, SOCK_DEFAULT_LINGER_ONOFF,
-			SOCK_DEFAULT_LINGER_DELAY);
+	return sock_set_linger(fd,
+			oio_socket_linger_onoff, oio_socket_linger_delay);
 }
 
 void
 sock_set_client_default(int fd)
 {
+	sock_set_reuseaddr(fd, TRUE);
 	sock_set_linger_default(fd);
-	sock_set_nodelay(fd, TRUE);
-	sock_set_tcpquickack(fd, TRUE);
+	sock_set_nodelay(fd, oio_socket_quickack);
+	sock_set_tcpquickack(fd, oio_socket_nodelay);
 }
 
 int
@@ -448,23 +483,35 @@ metautils_pclose(int *pfd)
 }
 
 int
-sock_connect (const char *url, GError **err)
+sock_build_for_url(const char *url, GError **err,
+		struct sockaddr_storage *sas, size_t *sas_len)
 {
-	struct sockaddr_storage sas;
-	gsize sas_len = sizeof(sas);
+	*sas_len = sizeof(*sas);
 
-	if (!grid_string_to_sockaddr (url, (struct sockaddr*) &sas, &sas_len)) {
+	if (!grid_string_to_sockaddr (url, (struct sockaddr*) sas, sas_len)) {
 		g_error_transmit(err, NEWERROR(EINVAL, "invalid URL"));
 		return -1;
 	}
 
-	int fd = socket_nonblock(sas.ss_family, SOCK_STREAM, 0);
+	int fd = socket_nonblock(sas->ss_family, SOCK_STREAM, 0);
 	if (0 > fd) {
 		g_error_transmit(err, NEWERROR(EINVAL, "socket error: (%d) %s", errno, strerror(errno)));
 		return -1;
 	}
 
-	sock_set_reuseaddr(fd, TRUE);
+	sock_set_client_default(fd);
+	*err = NULL;
+	return fd;
+}
+
+int
+sock_connect (const char *url, GError **err)
+{
+	gsize sas_len = 0;
+	struct sockaddr_storage sas;
+	int fd = sock_build_for_url(url, err, &sas, &sas_len);
+	if (fd < 0)
+		return -1;
 
 	if (0 != metautils_syscall_connect (fd, (struct sockaddr*)&sas, sas_len)) {
 		if (errno != EINPROGRESS && errno != 0) {
@@ -474,9 +521,103 @@ sock_connect (const char *url, GError **err)
 		}
 	}
 
-	sock_set_linger_default(fd);
-	sock_set_nodelay(fd, TRUE);
-	sock_set_tcpquickack(fd, TRUE);
+	sock_set_client_default(fd);
 	*err = NULL;
 	return fd;
 }
+
+/* Set buffer sizes for small RPC */
+static void
+sock_setopt_buflen(int fd)
+{
+	int rc, opt;
+
+	if (oio_socket_gridd_sndbuf > 0) {
+		opt = oio_socket_gridd_sndbuf;
+		rc = metautils_syscall_setsockopt(fd, SOL_SOCKET, SO_SNDBUF, &opt, sizeof(opt));
+		(void) rc;
+	}
+
+	if (oio_socket_gridd_rcvbuf > 0) {
+		opt = oio_socket_gridd_rcvbuf;
+		rc = metautils_syscall_setsockopt(fd, SOL_SOCKET, SO_RCVBUF, &opt, sizeof(opt));
+		(void) rc;
+	}
+}
+
+static volatile gint64 _fastopen_last_error = 0;
+
+gboolean oio_allow_tcp_fastopen = FALSE;
+
+int
+sock_connect_and_send (const char *url, GError **err,
+		const uint8_t *buf, gsize *len)
+{
+	gsize sas_len = 0;
+	struct sockaddr_storage sas;
+	int fd = sock_build_for_url(url, err, &sas, &sas_len);
+	if (fd < 0)
+		return -1;
+
+	sock_setopt_buflen(fd);
+
+#if defined(MSG_FASTOPEN) && defined(TCP_FASTOPEN)
+
+	const gint64 now = oio_ext_monotonic_time();
+
+	if (!oio_allow_tcp_fastopen || !buf || !len ||
+			(_fastopen_last_error != 0 &&
+			 _fastopen_last_error > OLDEST(now,G_TIME_SPAN_MINUTE)))
+		goto label_simple_connect;
+
+#ifdef HAVE_ENBUG
+	if (len > 1)
+		*len = *len / 2;
+#endif
+
+	ssize_t rc;
+retry:
+	rc = sendto(fd, buf, *len,
+			MSG_FASTOPEN|MSG_NOSIGNAL, (struct sockaddr*) &sas, sas_len);
+	if (rc < 0) {
+		if (errno == EINTR)
+			goto retry;
+		if (errno == ENOTSUP || errno == EOPNOTSUPP) {
+			/* the TCP_FASTOPEN is not accepted, let's continue with a
+			 * regular connect/send sequence */
+			_fastopen_last_error = now;
+label_simple_connect:
+#endif
+			*len = 0;
+			if (0 != metautils_syscall_connect (fd, (struct sockaddr*)&sas, sas_len)) {
+				if (errno != EINPROGRESS && errno != 0) {
+					g_error_transmit(err,
+							SYSERR("connect error: (%d) %s", errno, strerror(errno)));
+					metautils_pclose (&fd);
+					return -1;
+				}
+			}
+			*err = NULL;
+			return fd;
+#if defined(MSG_FASTOPEN) && defined(TCP_FASTOPEN)
+		} else if (errno == EINPROGRESS) {
+			/* syn-cookie not ready, so the kernel will proceed internally with
+			 * traditional connect() SYN-SYN/ACK-ACK sequence */
+			*len = 0;
+			*err = NULL;
+			return fd;
+		} else {
+			g_error_transmit(err, NEWERROR(CODE_NETWORK_ERROR,
+						"connect error: (%d) %s", errno, strerror(errno)));
+			metautils_pclose (&fd);
+			return -1;
+		}
+	} else {
+		/* syn-cookie ready, the number of bytes packed in the SYN-cookie is
+		 * returned. */
+		*len = rc;
+		*err = NULL;
+		return fd;
+	}
+#endif
+}
