@@ -1,5 +1,4 @@
-# Copyright (C) 2015 OpenIO, original work as part of
-# OpenIO Software Defined Storage
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -15,16 +14,31 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
-from werkzeug.wrappers import Request, Response
+from werkzeug.wrappers import Response
 from werkzeug.routing import Map, Rule
-from werkzeug.exceptions import HTTPException, NotFound, BadRequest, \
-    Conflict, InternalServerError
+from werkzeug.exceptions import NotFound, BadRequest, Conflict
+from functools import wraps
 
 from oio.account.backend import AccountBackend
 from oio.common.utils import json, get_logger
+from oio.common.wsgi import WerkzeugApp
 
 
-class Account(object):
+def access_log(func):
+
+    @wraps(func)
+    def _access_log_wrapper(self, *args, **kwargs):
+        from time import time
+        pre = time()
+        rc = func(self, *args, **kwargs)
+        post = time()
+        self.logger.info("%s %0.6f", func.__name__, post - pre)
+        return rc
+
+    return _access_log_wrapper
+
+
+class Account(WerkzeugApp):
     def __init__(self, conf, backend, logger=None):
         self.conf = conf
         self.backend = backend
@@ -34,12 +48,18 @@ class Account(object):
             Rule('/status', endpoint='status'),
             Rule('/v1.0/account/create', endpoint='account_create'),
             Rule('/v1.0/account/delete', endpoint='account_delete'),
+            Rule('/v1.0/account/list', endpoint='account_list'),
             Rule('/v1.0/account/update', endpoint='account_update'),
             Rule('/v1.0/account/show', endpoint='account_show'),
             Rule('/v1.0/account/containers', endpoint='account_containers'),
+            Rule('/v1.0/account/refresh', endpoint='account_refresh'),
+            Rule('/v1.0/account/flush', endpoint='account_flush'),
             Rule('/v1.0/account/container/update',
-                 endpoint='account_container_update')
+                 endpoint='account_container_update'),
+            Rule('/v1.0/account/container/reset',
+                 endpoint='account_container_reset')
         ])
+        super(Account, self).__init__(self.url_map, self.logger)
 
     def _get_account_id(self, req):
         account_id = req.args.get('id')
@@ -59,6 +79,13 @@ class Account(object):
         else:
             return Response(status=202)
 
+    @access_log
+    def on_account_list(self, req):
+        accounts = self.backend.list_account()
+        if accounts is None:
+            return NotFound('No account found')
+        return Response(json.dumps(accounts), mimetype='text/json')
+
     def on_account_delete(self, req):
         account_id = self._get_account_id(req)
         result = self.backend.delete_account(account_id)
@@ -80,6 +107,7 @@ class Account(object):
             return Response(status=204)
         return NotFound('Account not found')
 
+    @access_log
     def on_account_show(self, req):
         account_id = self._get_account_id(req)
         raw = self.backend.info_account(account_id)
@@ -87,6 +115,7 @@ class Account(object):
             return Response(json.dumps(raw), mimetype='text/json')
         return NotFound('Account not found')
 
+    @access_log
     def on_account_containers(self, req):
         account_id = self._get_account_id(req)
 
@@ -116,31 +145,35 @@ class Account(object):
         dtime = d.get('dtime')
         object_count = d.get('objects')
         bytes_used = d.get('bytes')
+        # Exceptions are catched by dispatch_request
         info = self.backend.update_container(
             account_id, name, mtime, dtime, object_count, bytes_used)
         result = json.dumps(info)
         return Response(result)
 
-    def dispatch_request(self, req):
-        adapter = self.url_map.bind_to_environ(req.environ)
-        try:
-            endpoint, values = adapter.match()
-            return getattr(self, 'on_' + endpoint)(req)
-        except NotFound:
-            return BadRequest()
-        except HTTPException as e:
-            return e
-        except Exception:
-            self.logger.exception('ERROR Unhandled exception in request')
-            return InternalServerError()
+    def on_account_container_reset(self, req):
+        account_id = self._get_account_id(req)
+        data = json.loads(req.get_data())
+        name = data.get('name')
+        mtime = data.get('mtime')
+        dtime = None
+        object_count = 0
+        bytes_used = 0
+        # Exceptions are catched by dispatch_request
+        self.backend.update_container(
+            account_id, name, mtime, dtime, object_count, bytes_used,
+            autocreate_container=False)
+        return Response(status=204)
 
-    def wsgi_app(self, environ, start_response):
-        req = Request(environ)
-        resp = self.dispatch_request(req)
-        return resp(environ, start_response)
+    def on_account_refresh(self, req):
+        account_id = self._get_account_id(req)
+        self.backend.refresh_account(account_id)
+        return Response(status=204)
 
-    def __call__(self, environ, start_response):
-        return self.wsgi_app(environ, start_response)
+    def on_account_flush(self, req):
+        account_id = self._get_account_id(req)
+        self.backend.flush_account(account_id)
+        return Response(status=204)
 
 
 def create_app(conf, **kwargs):
