@@ -1,13 +1,15 @@
-# Copyright (C) 2016 OpenIO SAS
-
+# Copyright (C) 2015-2018 OpenIO SAS, as part of OpenIO SDS
+#
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 3.0 of the License, or (at your option) any later version.
+#
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
+#
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library.
 
@@ -21,7 +23,7 @@ from greenlet import GreenletExit
 from oio.common import exceptions
 from oio.common.exceptions import SourceReadError
 from oio.common.http import HeadersDict, parse_content_range, \
-    ranges_from_http_header
+    ranges_from_http_header, headers_from_object_metadata
 from oio.common.utils import fix_ranges
 from oio.api import io
 from oio.common.constants import chunk_headers
@@ -55,61 +57,6 @@ def segment_range_to_fragment_range(segment_start, segment_end, segment_size,
     return (fragment_start, fragment_end)
 
 
-def obj_range_to_meta_chunk_range(obj_start, obj_end, meta_sizes):
-    """
-    Converts a requested object range into a list of meta_chunk ranges.
-
-    :returns: a tuple list (pos, meta_chunk_start, meta_chunk_end)
-
-        * pos is the meta chunk position
-
-        * meta_chunk_start is the first byte of the meta chunk,
-          or None if this is a suffix byte range
-
-        * meta_chunk_end is the last byte of the meta_chunk,
-          or None if this is a prefix byte range
-    """
-
-    offset = 0
-    found_start = False
-    found_end = False
-    total_size = 0
-
-    for meta_size in meta_sizes:
-        total_size += meta_size
-    # suffix byte range handling
-    if obj_start is None and obj_end is not None:
-        obj_start = total_size - min(total_size, obj_end)
-        obj_end = total_size - 1
-
-    meta_chunk_ranges = {}
-    for pos, meta_size in enumerate(meta_sizes):
-        if meta_size <= 0:
-            continue
-        if found_start:
-            meta_chunk_start = 0
-        elif obj_start is not None and obj_start >= offset + meta_size:
-            offset += meta_size
-            continue
-        elif obj_start is not None and obj_start < offset + meta_size:
-            meta_chunk_start = obj_start - offset
-            found_start = True
-        else:
-            meta_chunk_start = 0
-        if obj_end is not None and offset + meta_size > obj_end:
-            meta_chunk_end = obj_end - offset
-            # found end
-            found_end = True
-        elif meta_size > 0:
-            meta_chunk_end = meta_size - 1
-        meta_chunk_ranges[pos] = (meta_chunk_start, meta_chunk_end)
-        if found_end:
-            break
-        offset += meta_size
-
-    return meta_chunk_ranges
-
-
 def meta_chunk_range_to_segment_range(meta_start, meta_end, segment_size):
     """
     Converts a meta chunk range to a segment range.
@@ -145,11 +92,10 @@ class ECChunkDownloadHandler(object):
     """
 
     def __init__(self, storage_method, chunks, meta_start, meta_end, headers,
-                 connection_timeout=None, response_timeout=None,
-                 read_timeout=None):
+                 connection_timeout=None, read_timeout=None,
+                 **_kwargs):
         """
         :param connection_timeout: timeout to establish the connections
-        :param response_timeout: timeout to receive the headers
         :param read_timeout: timeout to read a buffer of data
         """
         self.storage_method = storage_method
@@ -161,7 +107,6 @@ class ECChunkDownloadHandler(object):
         self.meta_length = self.chunks[0]['size']
         self.headers = headers
         self.connection_timeout = connection_timeout
-        self.response_timeout = response_timeout
         self.read_timeout = read_timeout
 
     def _get_range_infos(self):
@@ -207,7 +152,7 @@ class ECChunkDownloadHandler(object):
                     range_info['req_fragment_end'])
         reader = io.ChunkReader(chunk_iter, storage_method.ec_fragment_size,
                                 headers, self.connection_timeout,
-                                self.response_timeout, self.read_timeout,
+                                self.read_timeout,
                                 align=True)
         return (reader, reader.get_iter())
 
@@ -337,9 +282,9 @@ class ECStream(object):
             except GreenletExit:
                 # ignore
                 pass
-            except green.ChunkReadTimeout:
-                logger.error("Timeout on reading")
-            except:
+            except green.ChunkReadTimeout as err:
+                logger.error('%s', err)
+            except Exception:
                 logger.exception("Exception on reading")
             finally:
                 queue.resize(2)
@@ -574,16 +519,20 @@ def ec_encode(storage_method, n):
     yield last_fragments
 
 
-class ECWriter(object):
+class EcChunkWriter(object):
     """
     Writes an EC chunk
     """
-    def __init__(self, chunk, conn, write_timeout=None, **_kwargs):
+    def __init__(self, chunk, conn, write_timeout=None,
+                 chunk_checksum_algo='md5', **_kwargs):
         self._chunk = chunk
         self._conn = conn
         self.failed = False
         self.bytes_transferred = 0
-        self.checksum = hashlib.md5()
+        if chunk_checksum_algo:
+            self.checksum = hashlib.new(chunk_checksum_algo)
+        else:
+            self.checksum = None
         self.write_timeout = write_timeout or io.CHUNK_TIMEOUT
         # we use eventlet Queue to pass data to the send coroutine
         self.queue = Queue(io.PUT_QUEUE_DEPTH)
@@ -598,34 +547,28 @@ class ECWriter(object):
 
     @classmethod
     def connect(cls, chunk, sysmeta, reqid=None,
-                connection_timeout=None, write_timeout=None, **_kwargs):
+                connection_timeout=None, write_timeout=None, **kwargs):
         raw_url = chunk["url"]
         parsed = urlparse(raw_url)
         chunk_path = parsed.path.split('/')[-1]
-        h = {}
-        h["transfer-encoding"] = "chunked"
-        h[chunk_headers["content_id"]] = sysmeta['id']
-        h[chunk_headers["content_path"]] = sysmeta['content_path']
-        h[chunk_headers["content_chunkmethod"]] = sysmeta['chunk_method']
-        h[chunk_headers["container_id"]] = sysmeta['container_id']
-        h[chunk_headers["chunk_pos"]] = chunk["pos"]
-        h[chunk_headers["chunk_id"]] = chunk_path
-        h[chunk_headers["content_policy"]] = sysmeta['policy']
-        h[chunk_headers["content_version"]] = sysmeta['version']
+        hdrs = headers_from_object_metadata(sysmeta)
         if reqid:
-            h['X-oio-req-id'] = reqid
+            hdrs['X-oio-req-id'] = reqid
+
+        hdrs[chunk_headers["chunk_pos"]] = chunk["pos"]
+        hdrs[chunk_headers["chunk_id"]] = chunk_path
 
         # in the trailer
         # metachunk_size & metachunk_hash
-        h["Trailer"] = ', '.join((chunk_headers["metachunk_size"],
-                                  chunk_headers["metachunk_hash"],
-                                  chunk_headers["chunk_hash"]))
+        hdrs["Trailer"] = ', '.join((chunk_headers["metachunk_size"],
+                                     chunk_headers["metachunk_hash"],
+                                     chunk_headers["chunk_hash"]))
         with green.ConnectionTimeout(
                 connection_timeout or io.CONNECTION_TIMEOUT):
             conn = io.http_connect(
-                parsed.netloc, 'PUT', parsed.path, h)
+                parsed.netloc, 'PUT', parsed.path, hdrs)
             conn.chunk = chunk
-        return cls(chunk, conn, write_timeout=write_timeout)
+        return cls(chunk, conn, write_timeout=write_timeout, **kwargs)
 
     def start(self, pool):
         """Spawn the send coroutine"""
@@ -677,11 +620,12 @@ class ECWriter(object):
             '%s: %s\r\n' % (chunk_headers['metachunk_size'],
                             metachunk_size),
             '%s: %s\r\n' % (chunk_headers['metachunk_hash'],
-                            metachunk_hash),
-            '%s: %s\r\n' % (chunk_headers['chunk_hash'],
-                            self.checksum.hexdigest()),
-            '\r\n'
+                            metachunk_hash)
         ]
+        if self.checksum:
+            parts.append('%s: %s\r\n' % (chunk_headers['chunk_hash'],
+                                         self.checksum.hexdigest()))
+        parts.append('\r\n')
         to_send = "".join(parts)
         self.conn.send(to_send)
 
@@ -694,17 +638,18 @@ class ECWriter(object):
             return self.conn.getresponse()
 
 
-# FIXME: the name of this class is misleading
-# since it does not extend io.WriteHandler
-class ECChunkWriteHandler(object):
+class EcMetachunkWriter(io.MetachunkWriter):
     def __init__(self, sysmeta, meta_chunk, global_checksum, storage_method,
                  reqid=None, connection_timeout=None, write_timeout=None,
-                 read_timeout=None):
+                 read_timeout=None, **kwargs):
+        super(EcMetachunkWriter, self).__init__(
+            storage_method=storage_method, **kwargs)
         self.sysmeta = sysmeta
         self.meta_chunk = meta_chunk
         self.global_checksum = global_checksum
-        self.checksum = hashlib.md5()
-        self.storage_method = storage_method
+        # Unlike plain replication, we cannot use the checksum returned
+        # by rawx services, whe have to compute the checksum client-side.
+        self.checksum = hashlib.new(self.chunk_checksum_algo or 'md5')
         self.reqid = reqid
         self.connection_timeout = connection_timeout or io.CONNECTION_TIMEOUT
         self.write_timeout = write_timeout or io.CHUNK_TIMEOUT
@@ -724,10 +669,7 @@ class ECChunkWriteHandler(object):
         bytes_transferred = self._stream(source, size, current_writers)
 
         # get the chunks from writers
-        chunks, quorum = self._get_results(current_writers)
-
-        if not quorum:
-            raise exceptions.OioException('Write failure: quorum not reached')
+        chunks = self._get_results(current_writers)
 
         meta_checksum = self.checksum.hexdigest()
 
@@ -753,15 +695,18 @@ class ECChunkWriteHandler(object):
                 return
 
             current_writers = list(writers)
+            failed_chunks = list()
             for writer in current_writers:
                 fragment = fragments[chunk_index[writer]]
                 if not writer.failed:
-                    writer.checksum.update(fragment)
+                    if writer.checksum:
+                        writer.checksum.update(fragment)
                     writer.send(fragment)
                 else:
                     current_writers.remove(writer)
-            self._check_quorum(current_writers)
-            # TODO handle no quorum
+                    failed_chunks.append(writer.chunk)
+            self.quorum_or_fail([w.chunk for w in current_writers],
+                                failed_chunks)
 
         try:
             # we use eventlet GreenPool to manage writers
@@ -774,22 +719,34 @@ class ECChunkWriteHandler(object):
                 for writer in writers:
                     writer.start(pool)
 
-                # the main write loop
-                while True:
-                    remaining_bytes = size - bytes_transferred
-                    if io.WRITE_CHUNK_SIZE < remaining_bytes:
-                        read_size = io.WRITE_CHUNK_SIZE
-                    else:
-                        read_size = remaining_bytes
+                def read(read_size):
                     with green.SourceReadTimeout(self.read_timeout):
                         try:
                             data = source.read(read_size)
                         except (ValueError, IOError) as exc:
                             raise SourceReadError(str(exc))
-                    if len(data) == 0:
-                        break
-                    bytes_transferred += len(data)
-                    send(data)
+                    return data
+
+                # the main write loop
+                if size:
+                    while True:
+                        remaining_bytes = size - bytes_transferred
+                        if io.WRITE_CHUNK_SIZE < remaining_bytes:
+                            read_size = io.WRITE_CHUNK_SIZE
+                        else:
+                            read_size = remaining_bytes
+                        data = read(read_size)
+                        bytes_transferred += len(data)
+                        if len(data) == 0:
+                            break
+                        send(data)
+                else:
+                    while True:
+                        data = read(io.WRITE_CHUNK_SIZE)
+                        bytes_transferred += len(data)
+                        if len(data) == 0:
+                            break
+                        send(data)
 
                 # flush out buffered data
                 send('')
@@ -809,15 +766,15 @@ class ECChunkWriteHandler(object):
 
                 return bytes_transferred
 
-        except green.SourceReadTimeout:
-            logger.warn('Source read timeout')
+        except green.SourceReadTimeout as exc:
+            logger.warn('%s', exc)
+            raise exceptions.SourceReadTimeout(exc)
+        except SourceReadError as exc:
+            logger.warn('Source read error: %s', exc)
             raise
-        except SourceReadError:
-            logger.warn('Source read error')
-            raise
-        except Timeout:
+        except Timeout as to:
             logger.exception('Timeout writing data')
-            raise
+            raise exceptions.OioTimeout(to)
         except Exception:
             logger.exception('Exception writing data')
             raise
@@ -838,18 +795,16 @@ class ECChunkWriteHandler(object):
     def _get_writer(self, chunk):
         """Spawn a writer for the chunk and connect it"""
         try:
-            writer = ECWriter.connect(
+            writer = EcChunkWriter.connect(
                 chunk, self.sysmeta, self.reqid,
                 connection_timeout=self.connection_timeout,
-                write_timeout=self.write_timeout)
+                write_timeout=self.write_timeout,
+                chunk_checksum_algo=self.chunk_checksum_algo)
             return writer, chunk
         except (Exception, Timeout) as exc:
             msg = str(exc)
             logger.error("Failed to connect to %s (%s)", chunk, msg)
-            if isinstance(exc, Timeout):
-                chunk['error'] = 'connect: Timeout %s' % msg
-            else:
-                chunk['error'] = 'connect: %s' % msg
+            chunk['error'] = 'connect: %s' % msg
             return None, chunk
 
     def _get_results(self, writers):
@@ -870,7 +825,7 @@ class ECChunkWriteHandler(object):
             if resp:
                 if resp.status == 201:
                     checksum = resp.getheader(chunk_headers['chunk_hash'])
-                    if checksum and \
+                    if checksum and writer.checksum and \
                             checksum.lower() != writer.checksum.hexdigest():
                         writer.chunk['error'] = \
                             "checksum mismatch: %s (local), %s (rawx)" % \
@@ -879,8 +834,8 @@ class ECChunkWriteHandler(object):
                     else:
                         success_chunks.append(writer.chunk)
                 else:
-                    logger.error("Wrong status code from %s (%s)",
-                                 writer.chunk, resp.status)
+                    logger.error("Wrong status code from %s (%s) %s",
+                                 writer.chunk, resp.status, resp.reason)
                     writer.chunk['error'] = 'resp: HTTP %s' % resp.status
                     failed_chunks.append(writer.chunk)
             else:
@@ -889,9 +844,9 @@ class ECChunkWriteHandler(object):
         for (writer, resp) in pile:
             _handle_resp(writer, resp)
 
-        quorum = self._check_quorum(success_chunks)
+        self.quorum_or_fail(success_chunks, failed_chunks)
 
-        return success_chunks + failed_chunks, quorum
+        return success_chunks + failed_chunks
 
     def _get_response(self, writer):
         # spawned in a coroutine to read the HTTP response
@@ -903,7 +858,7 @@ class ECChunkWriteHandler(object):
             if isinstance(exc, Timeout):
                 logger.warn("Timeout (%s) while writing %s",
                             msg, writer.chunk)
-                writer.chunk['error'] = 'resp: Timeout %s' % msg
+                writer.chunk['error'] = 'resp: %s' % msg
             else:
                 logger.warn("Failed to read response for %s (%s)",
                             writer.chunk, msg)
@@ -916,9 +871,6 @@ class ECChunkWriteHandler(object):
             chunk_index[w] = w.chunk['num']
         return chunk_index
 
-    def _check_quorum(self, writers):
-        return len(writers) >= self.storage_method.quorum
-
 
 class ECWriteHandler(io.WriteHandler):
     """
@@ -950,13 +902,14 @@ class ECWriteHandler(io.WriteHandler):
         # iterate through the meta chunks
         bytes_transferred = -1
         for meta_chunk in self.chunk_prep():
-            handler = ECChunkWriteHandler(
+            handler = EcMetachunkWriter(
                 self.sysmeta, meta_chunk,
                 global_checksum, self.storage_method,
                 reqid=self.headers.get('X-oio-req-id'),
                 connection_timeout=self.connection_timeout,
                 write_timeout=self.write_timeout,
-                read_timeout=self.read_timeout)
+                read_timeout=self.read_timeout,
+                chunk_checksum_algo=self.chunk_checksum_algo)
             bytes_transferred, checksum, chunks = handler.stream(self.source,
                                                                  max_size)
 
@@ -965,10 +918,12 @@ class ECWriteHandler(io.WriteHandler):
             for chunk in chunks:
                 chunk['hash'] = checksum
                 chunk['size'] = bytes_transferred
+                # add the chunks whose upload succeeded
+                # to the content chunk list
+                if not chunk.get('error'):
+                    content_chunks.append(chunk)
 
             total_bytes_transferred += bytes_transferred
-            # add the chunks to the content chunk list
-            content_chunks += chunks
             if bytes_transferred < max_size:
                 break
             if len(self.source.peek()) == 0:
@@ -982,13 +937,12 @@ class ECWriteHandler(io.WriteHandler):
 
 class ECRebuildHandler(object):
     def __init__(self, meta_chunk, missing, storage_method,
-                 connection_timeout=None, response_timeout=None,
-                 read_timeout=None):
+                 connection_timeout=None, read_timeout=None,
+                 **_kwargs):
         self.meta_chunk = meta_chunk
         self.missing = missing
         self.storage_method = storage_method
         self.connection_timeout = connection_timeout or io.CONNECTION_TIMEOUT
-        self.response_timeout = response_timeout or io.CHUNK_TIMEOUT
         self.read_timeout = read_timeout or io.CHUNK_TIMEOUT
 
     def _get_response(self, chunk, headers):
@@ -999,10 +953,11 @@ class ECRebuildHandler(object):
                 conn = io.http_connect(
                     parsed.netloc, 'GET', parsed.path, headers)
 
-            with Timeout(self.response_timeout):
+            with Timeout(self.read_timeout):
                 resp = conn.getresponse()
             if resp.status != 200:
-                logger.warning('Invalid GET response from %s', chunk)
+                logger.warning('Invalid GET response from %s: %s %s',
+                               chunk, resp.status, resp.reason)
                 resp = None
         except (Exception, Timeout):
             logger.exception('ERROR fetching %s', chunk)
