@@ -1,13 +1,15 @@
-# Copyright (C) 2016-2017 OpenIO SAS
-
+# Copyright (C) 2015-2018 OpenIO SAS, as part of OpenIO SDS
+#
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 3.0 of the License, or (at your option) any later version.
+#
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
+#
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library.
 
@@ -29,11 +31,11 @@ WRITE_CHUNK_SIZE = 65536
 READ_CHUNK_SIZE = 65536
 
 # RAWX connection timeout
-CONNECTION_TIMEOUT = 2.0
+CONNECTION_TIMEOUT = 10.0
 # chunk operations timeout
-CHUNK_TIMEOUT = 5.0
+CHUNK_TIMEOUT = 60.0
 # client read timeout
-CLIENT_TIMEOUT = 5.0
+CLIENT_TIMEOUT = 60.0
 
 PUT_QUEUE_DEPTH = 10
 
@@ -71,13 +73,17 @@ class IOBaseWrapper(RawIOBase):
 
 class WriteHandler(object):
     def __init__(self, source, sysmeta, chunk_preparer,
-                 storage_method, headers,
+                 storage_method, headers=None,
                  connection_timeout=None, write_timeout=None,
-                 read_timeout=None):
+                 read_timeout=None, chunk_checksum_algo='md5',
+                 **_kwargs):
         """
         :param connection_timeout: timeout to establish the connection
         :param write_timeout: timeout to send a buffer of data
         :param read_timeout: timeout to read a buffer of data from source
+        :param chunk_checksum_algo: algorithm to use to compute chunk
+            checksums locally. Can be `None` to disable local checksum
+            computation and let the rawx compute it (will be md5).
         """
         if isinstance(source, IOBase):
             self.source = BufferedReader(source)
@@ -92,10 +98,11 @@ class WriteHandler(object):
             self.chunk_prep = chunk_preparer
         self.sysmeta = sysmeta
         self.storage_method = storage_method
-        self.headers = headers
+        self.headers = headers or dict()
         self.connection_timeout = connection_timeout or CONNECTION_TIMEOUT
         self.write_timeout = write_timeout or CHUNK_TIMEOUT
         self.read_timeout = read_timeout or CLIENT_TIMEOUT
+        self.chunk_checksum_algo = chunk_checksum_algo
 
     def stream(self):
         """
@@ -168,14 +175,13 @@ class ChunkReader(object):
     """
 
     def __init__(self, chunk_iter, buf_size, headers,
-                 connection_timeout=None, response_timeout=None,
-                 read_timeout=None, align=False):
+                 connection_timeout=None, read_timeout=None,
+                 align=False, **_kwargs):
         """
         :param chunk_iter:
         :param buf_size: size of the read buffer
         :param headers:
         :param connection_timeout: timeout to establish the connection
-        :param response_timeout: timeout to receive the headers
         :param read_timeout: timeout to read a buffer of data
         :param align: if True, the reader will skip some bytes to align
                       on `buf_size`
@@ -192,10 +198,16 @@ class ChunkReader(object):
         self.discard_bytes = 0
         self.align = align
         self.connection_timeout = connection_timeout or CONNECTION_TIMEOUT
-        self.response_timeout = response_timeout or CHUNK_TIMEOUT
         self.read_timeout = read_timeout or CHUNK_TIMEOUT
         self._resp_by_chunk = dict()
 
+    @property
+    def reqid(self):
+        """:returns: the request ID or None"""
+        if not self.request_headers:
+            return None
+        return self.request_headers.get('X-oio-req-id')
+
     def recover(self, nb_bytes):
         """
         Recover the request.
@@ -203,9 +215,9 @@ class ChunkReader(object):
         :param nb_bytes: number of bytes already consumed that we need to
                          discard if we perform a recovery from another source.
 
-        :raises ValueError: if range header is not valid
-        :raises UnsatisfiableRange
-        :raises EmptyByteRange
+        :raises `ValueError`: if range header is not valid
+        :raises `oio.common.exceptions.UnsatisfiableRange`:
+        :raises `oio.common.exceptions.EmptyByteRange`:
         """
         if 'Range' in self.request_headers:
             request_range = ranges_from_http_header(
@@ -250,11 +262,12 @@ class ChunkReader(object):
                 parsed = urlparse(raw_url)
                 conn = http_connect(parsed.netloc, 'GET', parsed.path,
                                     self.request_headers)
-            with green.OioTimeout(self.response_timeout):
+            with green.OioTimeout(self.read_timeout):
                 source = conn.getresponse()
                 source.conn = conn
         except (Exception, Timeout) as error:
-            logger.exception('Connection failed to %s', chunk)
+            logger.exception('Connection failed to %s (reqid=%s)',
+                             chunk, self.reqid)
             self._resp_by_chunk[chunk["url"]] = (0, str(error))
             return False
 
@@ -264,8 +277,8 @@ class ChunkReader(object):
             self.sources.append((source, chunk))
             return True
         else:
-            logger.warn("Invalid response from %s: %d %s",
-                        chunk, source.status, source.reason)
+            logger.warn("Invalid response from %s (reqid=%s): %d %s",
+                        chunk, self.reqid, source.status, source.reason)
             self._resp_by_chunk[chunk["url"]] = (source.status,
                                                  str(source.reason))
         return False
@@ -298,8 +311,11 @@ class ChunkReader(object):
                                      self._resp_by_chunk)
 
     def stream(self):
-        # Calling that right now will make `headers` field available
-        # before the caller starts reading the stream
+        """
+        Get a generator over chunk data.
+        After calling this method, the `headers` field will be available
+        (even if no data is read from the generator).
+        """
         parts_iter = self.get_iter()
 
         def _iter():
@@ -336,111 +352,118 @@ class ChunkReader(object):
         self.request_headers['Range'] = http_header_from_ranges(
             new_ranges)
 
+    @staticmethod
+    def get_next_part(parts_iter):
+        """
+        Gets next part of the body
+
+        NOTE: for the moment only return one part
+              (single range only)
+        """
+        while True:
+            try:
+                with green.ChunkReadTimeout(CHUNK_TIMEOUT):
+                    start, end, length, headers, part = next(
+                        parts_iter[0])
+                return (start, end, length, headers, part)
+            except green.ChunkReadTimeout:
+                # TODO recover
+                raise StopIteration
+
+    def iter_from_resp(self, source, parts_iter, part, chunk):
+        bytes_consumed = 0
+        count = 0
+        buf = ''
+        while True:
+            try:
+                with green.ChunkReadTimeout(self.read_timeout):
+                    data = part.read(READ_CHUNK_SIZE)
+                    count += 1
+                    buf += data
+            except green.ChunkReadTimeout as crto:
+                try:
+                    self.recover(bytes_consumed)
+                except (exc.UnsatisfiableRange, ValueError):
+                    raise
+                except exc.EmptyByteRange:
+                    # we are done already
+                    break
+                buf = ''
+                # find a new source to perform recovery
+                new_source, new_chunk = self._get_source()
+                if new_source:
+                    logger.warn(
+                        "Failed to read from %s (%s), "
+                        "retrying from %s (reqid=%s)",
+                        chunk, crto, new_chunk, self.reqid)
+                    close_source(source[0])
+                    # switch source
+                    source[0] = new_source
+                    chunk = new_chunk
+                    parts_iter[0] = make_iter_from_resp(source[0])
+                    try:
+                        _j, _j, _j, _j, part = \
+                            self.get_next_part(parts_iter)
+                    except StopIteration:
+                        # failed to recover
+                        # we did our best
+                        return
+
+                else:
+                    logger.warn("Failed to read from %s (%s, reqid=%s)",
+                                chunk, crto, self.reqid)
+                    # no valid source found to recover
+                    raise
+            else:
+                # discard bytes
+                if buf and self.discard_bytes:
+                    if self.discard_bytes < len(buf):
+                        buf = buf[self.discard_bytes:]
+                        bytes_consumed += self.discard_bytes
+                        self.discard_bytes = 0
+                    else:
+                        self.discard_bytes -= len(buf)
+                        bytes_consumed += len(buf)
+                        buf = ''
+
+                # no data returned
+                # flush out buffer
+                if not data:
+                    if buf:
+                        bytes_consumed += len(buf)
+                        yield buf
+                    buf = ''
+                    break
+
+                # If buf_size is defined, yield bounded data buffers
+                if self.buf_size is not None:
+                    while len(buf) >= self.buf_size:
+                        read_d = buf[:self.buf_size]
+                        buf = buf[self.buf_size:]
+                        yield read_d
+                        bytes_consumed += len(read_d)
+                else:
+                    yield buf
+                    bytes_consumed += len(buf)
+                    buf = ''
+
+                # avoid starvation by forcing sleep()
+                # every once in a while
+                if count % 10 == 0:
+                    sleep()
+
     def _get_iter(self, chunk, source):
         source = [source]
-
         try:
-            read_size = self.buf_size
             parts_iter = [make_iter_from_resp(source[0])]
-
-            def get_next_part():
-                """
-                Gets next part of the body
-
-                NOTE: for the moment only return one part
-                      (single range only)
-                """
-                while True:
-                    try:
-                        with green.ChunkReadTimeout(CHUNK_TIMEOUT):
-                            start, end, length, headers, part = next(
-                                parts_iter[0])
-                        return (start, end, length, headers, part)
-                    except green.ChunkReadTimeout:
-                        # TODO recover
-                        raise StopIteration
-
-            def iter_from_resp(part):
-                bytes_consumed = 0
-                count = 0
-                buf = ''
-                while True:
-                    try:
-                        with green.ChunkReadTimeout(self.read_timeout):
-                            data = part.read(READ_CHUNK_SIZE)
-                            count += 1
-                            buf += data
-                    except green.ChunkReadTimeout:
-                        try:
-                            self.recover(bytes_consumed)
-                        except (exc.UnsatisfiableRange, ValueError):
-                            raise
-                        except exc.EmptyByteRange:
-                            # we are done already
-                            break
-                        buf = ''
-                        # find a new source to perform recovery
-                        new_source, new_chunk = self._get_source()
-                        if new_source:
-                            logger.warn("Retrying from another source")
-                            close_source(source[0])
-                            # switch source
-                            source[0] = new_source
-                            parts_iter[0] = make_iter_from_resp(source[0])
-                            try:
-                                _j, _j, _j, _j, part = get_next_part()
-                            except StopIteration:
-                                # failed to recover
-                                # we did our best
-                                return
-
-                        else:
-                            # no valid source found to recover
-                            raise
-                    else:
-                        # discard bytes
-                        if buf and self.discard_bytes:
-                            if self.discard_bytes < len(buf):
-                                buf = buf[self.discard_bytes:]
-                                bytes_consumed += self.discard_bytes
-                                self.discard_bytes = 0
-                            else:
-                                self.discard_bytes -= len(buf)
-                                bytes_consumed += len(buf)
-                                buf = ''
-
-                        # no data returned
-                        # flush out buffer
-                        if not data:
-                            if buf:
-                                bytes_consumed += len(buf)
-                                yield buf
-                            buf = ''
-                            break
-
-                        # buffer to read_size
-                        if read_size is not None:
-                            while len(buf) >= read_size:
-                                read_d = buf[:read_size]
-                                buf = buf[read_size:]
-                                yield read_d
-                                bytes_consumed += len(read_d)
-                        else:
-                            yield buf
-                            bytes_consumed += len(buf)
-                            buf = ''
-
-                        # avoid starvation by forcing sleep()
-                        # every once in a while
-                        if count % 10 == 0:
-                            sleep()
-
             body_iter = None
             try:
                 while True:
-                    start, end, length, headers, part = get_next_part()
+                    start, end, length, headers, part = \
+                        self.get_next_part(parts_iter)
                     self.fill_ranges(start, end, length)
-                    body_iter = iter_from_resp(part)
+                    body_iter = self.iter_from_resp(
+                        source, parts_iter, part, chunk)
                     result = {'start': start, 'end': end, 'length': length,
                               'iter': body_iter, 'headers': headers}
                     yield result
@@ -451,7 +474,8 @@ class ChunkReader(object):
                     body_iter.close()
 
         except green.ChunkReadTimeout:
-            logger.exception("Failure during chunk read")
+            logger.exception("Failure during chunk read (reqid=%s)",
+                             self.reqid)
             raise
         except GeneratorExit:
             pass
@@ -483,6 +507,58 @@ class ChunkReader(object):
         raise StopIteration
 
 
+class MetachunkWriter(object):
+    """Base class for metachunk writers"""
+
+    def __init__(self, storage_method=None, quorum=None,
+                 chunk_checksum_algo='md5', **_kwargs):
+        self.storage_method = storage_method
+        self._quorum = quorum
+        if storage_method is None and quorum is None:
+            raise ValueError('Missing storage_method or quorum')
+        self.chunk_checksum_algo = chunk_checksum_algo
+
+    @property
+    def quorum(self):
+        """Minimum number of chunks required to validate an upload"""
+        if self._quorum is None:
+            return self.storage_method.quorum
+        return self._quorum
+
+    def quorum_or_fail(self, successes, failures):
+        """
+        Compare the number of uploads against the quorum.
+
+        :param successes: a list of chunk objects whose upload succeded
+        :type successes: `list` or `tuple`
+        :param failures: a list of chunk objects whose upload failed
+        :type failures: `list` or `tuple`
+        :raises `exc.SourceReadError`: if there is an error while reading
+            data from the client
+        :raises `exc.SourceReadTimeout`: if there is a timeout while reading
+            data from the client
+        :raises `exc.OioTimeout`: if there is a timeout among the errors
+        :raises `exc.OioException`: if quorum has not been reached
+            for any other reason
+        """
+        if len(successes) < self.quorum:
+            errors = group_chunk_errors(
+                ((chunk["url"], chunk.get("error", "success"))
+                 for chunk in successes + failures))
+            new_exc = exc.OioException(
+                "RAWX write failure, quorum not reached (%d/%d): %s" %
+                (len(successes), self.quorum, errors))
+            for err in [x.get('error') for x in failures]:
+                if isinstance(err, exc.SourceReadError):
+                    raise exc.SourceReadError(new_exc)
+                elif isinstance(err, green.SourceReadTimeout):
+                    # Never raise 'green' timeouts out of our API
+                    raise exc.SourceReadTimeout(new_exc)
+                elif isinstance(err, (exc.OioTimeout, green.OioTimeout)):
+                    raise exc.OioTimeout(new_exc)
+            raise new_exc
+
+
 def make_iter_from_resp(resp):
     """
     Makes a part iterator from a HTTP response
