@@ -1,3 +1,19 @@
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+
 from contextlib import closing
 import hashlib
 import time
@@ -36,7 +52,7 @@ class BlobAuditorWorker(object):
             conf.get('chunks_per_second'), 30)
         self.max_bytes_per_second = int_value(
             conf.get('bytes_per_second'), 10000000)
-        self.container_client = ContainerClient(conf)
+        self.container_client = ContainerClient(conf, logger=self.logger)
 
     def audit_pass(self):
         self.namespace, self.address = check_volume(self.volume)
@@ -164,8 +180,8 @@ class BlobAuditorWorker(object):
             try:
                 container_id = meta['container_id']
                 content_path = meta['content_path']
-                content_attr, data = self.container_client.content_show(
-                    cid=container_id, path=content_path)
+                _obj_meta, data = self.container_client.content_locate(
+                    cid=container_id, path=content_path, properties=False)
 
                 # Check chunk data
                 chunk_data = None
