@@ -1,3 +1,18 @@
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 import time
 
 from oio.blob.client import BlobClient
@@ -41,7 +56,7 @@ class BlobMoverWorker(object):
         self.max_bytes_per_second = int_value(
             conf.get('bytes_per_second'), 10000000)
         self.blob_client = BlobClient()
-        self.container_client = ContainerClient(conf)
+        self.container_client = ContainerClient(conf, logger=self.logger)
         self.content_factory = ContentFactory(conf)
 
     def mover_pass(self):
