@@ -1,3 +1,19 @@
+# Copyright (C) 2015-2018 OpenIO SAS, as part of OpenIO SDS
+#
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3.0 of the License, or (at your option) any later version.
+#
+# This library is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public
+# License along with this library.
+
+
 import time
 import logging
 import eventlet
@@ -12,28 +28,29 @@ logging._lock = logging.threading.RLock()
 
 
 class OioTimeout(Timeout):
+    """Wrapper over eventlet.Timeout with better __str__."""
+
+    msg_prefix = ''
+
     def __str__(self):
-        return "Timeout %s" % super(OioTimeout, self).__str__()
-
-
-class ClientReadTimeout(OioTimeout):
-    pass
+        return "%stimeout %s" % (self.__class__.msg_prefix,
+                                 super(OioTimeout, self).__str__())
 
 
 class ConnectionTimeout(OioTimeout):
-    pass
+    msg_prefix = 'Connection '
 
 
 class SourceReadTimeout(OioTimeout):
-    pass
+    msg_prefix = 'Source read '
 
 
 class ChunkWriteTimeout(OioTimeout):
-    pass
+    msg_prefix = 'Chunk write '
 
 
 class ChunkReadTimeout(OioTimeout):
-    pass
+    msg_prefix = 'Chunk read '
 
 
 def get_hub():
