@@ -1,18 +1,42 @@
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
+#
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3.0 of the License, or (at your option) any later version.
+#
+# This library is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public
+# License along with this library.
+
+
 import logging
 import re
 import socket
-from urllib import quote
+from urllib import quote, quote_plus
+from urlparse import urlparse
 
 from eventlet import patcher
 from eventlet.green.httplib import HTTPConnection, HTTPResponse, _UNKNOWN, \
         CONTINUE, HTTPMessage
-from oio.common.utils import json
 
-requests = patcher.import_patched('requests.__init__')
-requests_adapters = patcher.import_patched('requests.adapters')
+from oio.common.constants import chunk_headers, OIO_VERSION
+from oio.common.utils import oio_reraise
+from oio.common import exceptions
 
-CONNECTION_TIMEOUT = 2.0
-READ_TIMEOUT = 30.0
+urllib3 = patcher.import_patched('urllib3.__init__')
+
+
+CONNECTION_TIMEOUT = 10.0
+READ_TIMEOUT = 60.0
+
+DEFAULT_POOLSIZE = 32
+DEFAULT_RETRIES = 0
+DEFAULT_BACKOFF = 0
 
 
 class CustomHTTPResponse(HTTPResponse):
@@ -87,24 +111,6 @@ class CustomHttpConnection(HTTPConnection):
         return response
 
 
-def http_request(host, method, path, headers=None, query_string=None,
-                 body=None):
-    headers = headers or {}
-
-    if isinstance(body, dict):
-        body = json.dumps(body)
-        headers['Content-Type'] = 'application/json'
-    headers['Content-Length'] = len(body)
-    conn = http_connect(host, method, path, headers=headers,
-                        query_string=query_string)
-    conn.send(body)
-    resp = conn.getresponse()
-    body = resp.read()
-    resp.close()
-    conn.close()
-    return resp, body
-
-
 def http_connect(host, method, path, headers=None, query_string=None):
     if isinstance(path, unicode):
         try:
@@ -204,6 +210,36 @@ def ranges_from_http_header(val):
     return ranges
 
 
+def headers_from_object_metadata(metadata):
+    """
+    Generate chunk PUT request headers from object metadata.
+    """
+    out = dict()
+    out["transfer-encoding"] = "chunked"
+    # FIXME: remove key incoherencies
+    out[chunk_headers["content_id"]] = metadata['id']
+    out[chunk_headers["content_version"]] = metadata['version']
+    out[chunk_headers["content_path"]] = metadata['content_path']
+    out[chunk_headers["content_chunkmethod"]] = metadata['chunk_method']
+    out[chunk_headers["content_policy"]] = metadata['policy']
+    out[chunk_headers["container_id"]] = metadata['container_id']
+    out[chunk_headers["oio_version"]] = metadata.get('oio_version',
+                                                     OIO_VERSION)
+
+    for key in ['metachunk_hash', 'metachunk_size', 'chunk_hash']:
+        val = metadata.get(key)
+        if val is not None:
+            out[chunk_headers[key]] = metadata[key]
+
+    header = {k: quote_plus(str(v)) for (k, v) in out.iteritems()}
+    full_path = metadata['full_path']
+    if isinstance(full_path, basestring):
+        header[chunk_headers['full_path']] = full_path
+    else:
+        header[chunk_headers['full_path']] = ','.join(full_path)
+    return header
+
+
 class HeadersDict(dict):
     def __init__(self, headers, **kwargs):
         if headers:
@@ -228,3 +264,57 @@ class HeadersDict(dict):
 
     def pop(self, k, default=None):
         return dict.pop(self, k.title(), default)
+
+
+def get_pool_manager(pool_connections=DEFAULT_POOLSIZE,
+                     pool_maxsize=DEFAULT_POOLSIZE,
+                     max_retries=DEFAULT_RETRIES,
+                     backoff_factor=DEFAULT_BACKOFF):
+    """
+    Get `urllib3.PoolManager` to manage pools of connections
+
+    :param pool_connections: number of connection pools
+    :type pool_connections: `int`
+    :param pool_maxsize: number of connections per connection pool
+    :type pool_maxsize: `int`
+    :param max_retries: number of retries per request
+    :type max_retries: `int`
+    :param backoff_factor: backoff factor to apply between attemps after
+        second try
+    :type backoff_factor: `float`
+    """
+    if max_retries == DEFAULT_RETRIES:
+        max_retries = urllib3.Retry(0, read=False)
+    else:
+        max_retries = urllib3.Retry(total=max_retries,
+                                    backoff_factor=backoff_factor)
+    return urllib3.PoolManager(num_pools=pool_connections,
+                               maxsize=pool_maxsize, retries=max_retries,
+                               block=False)
+
+
+def oio_exception_from_httperror(exc, reqid=None, url=None):
+    """
+    Convert an HTTPError from urllib3 to an OioException,
+    and re-raise it.
+    """
+    extra_dict = dict()
+    if reqid:
+        extra_dict['reqid'] = reqid
+    if url:
+        extra_dict['host'] = urlparse(url).netloc
+    extra = ', '.join('%s=%s' % x for x in extra_dict.items())
+    if isinstance(exc, urllib3.exceptions.MaxRetryError):
+        if isinstance(exc.reason, urllib3.exceptions.NewConnectionError):
+            oio_reraise(exceptions.OioNetworkException, exc.reason, extra)
+        if isinstance(exc.reason, urllib3.exceptions.TimeoutError):
+            oio_reraise(exceptions.OioTimeout, exc.reason, extra)
+        oio_reraise(exceptions.OioNetworkException, exc, extra)
+    elif isinstance(exc, (urllib3.exceptions.ProtocolError,
+                          urllib3.exceptions.ProxyError,
+                          urllib3.exceptions.ClosedPoolError)):
+        oio_reraise(exceptions.OioNetworkException, exc, extra)
+    elif isinstance(exc, urllib3.exceptions.TimeoutError):
+        oio_reraise(exceptions.OioTimeout, exc, extra)
+    else:
+        oio_reraise(exceptions.OioException, exc, extra)
