@@ -1,3 +1,18 @@
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
+#
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3.0 of the License, or (at your option) any later version.
+#
+# This library is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public
+# License along with this library.
+
 import os
 import socket
 import errno
@@ -20,10 +35,13 @@ from optparse import OptionParser
 from ConfigParser import SafeConfigParser
 
 from itertools import islice
+from functools import wraps
 
 import codecs
 from oio.common.exceptions import OioException
 
+import math
+
 xattr = None
 try:
     # try python-pyxattr
@@ -86,10 +104,7 @@ def quote(value, safe='/'):
 
 
 def name2cid(account, ref):
-    h = hashlib.sha256()
-    for v in [account, '\0', ref]:
-        h.update(v)
-    return h.hexdigest()
+    return cid_from_name(account, ref)
 
 
 def env(*vars, **kwargs):
@@ -462,7 +477,7 @@ def cid_from_name(account, ref):
     h = hashlib.sha256()
     for v in [account, '\0', ref]:
         h.update(v)
-    return h.hexdigest()
+    return h.hexdigest().upper()
 
 
 def fix_ranges(ranges, length):
@@ -552,9 +567,55 @@ class GeneratorIO(RawIOBase):
 
 
 def group_chunk_errors(chunk_err_iter):
+    """
+    Group errors in a dictionary of lists.
+    The keys are errors, the values are lists of chunk IDs.
+    """
     errors = dict()
     for chunk, err in chunk_err_iter:
         err_list = errors.get(err) or list()
         err_list.append(chunk)
         errors[err] = err_list
     return errors
+
+
+def ensure_headers(func):
+    @wraps(func)
+    def ensure_headers_wrapper(*args, **kwargs):
+        kwargs['headers'] = kwargs.get('headers') or dict()
+        return func(*args, **kwargs)
+    return ensure_headers_wrapper
+
+
+def ensure_request_id(func):
+    @wraps(func)
+    def ensure_request_id_wrapper(*args, **kwargs):
+        headers = kwargs['headers']
+        if 'X-oio-req-id' not in headers:
+            headers['X-oio-req-id'] = request_id()
+        return func(*args, **kwargs)
+    return ensure_request_id_wrapper
+
+
+METRIC_SYMBOLS = ("", "K", "M", "G", "T", "P", "E", "Z", "Y")
+
+
+def convert_size(size, unit=""):
+    if size == 0:
+        return "0%s" % unit
+    i = int(math.log(size, 1000))
+    if i != 0:
+        p = math.pow(1000, i)
+        size = round(size / p, 3)
+    return "%s%s%s" % (size, METRIC_SYMBOLS[i], unit)
+
+
+def oio_reraise(exc_type, exc_value, extra_message=None):
+    """
+    Raise an exception of type `exc_type` with arguments of `exc_value`
+    plus maybe `extra_message` at the beginning.
+    """
+    args = exc_value.args
+    if extra_message:
+        args = (extra_message, ) + args
+    raise exc_type(*args), None, sys.exc_info()[2]
