@@ -1,5 +1,23 @@
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
+#
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3.0 of the License, or (at your option) any later version.
+#
+# This library is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public
+# License along with this library.
+
 from gunicorn.app.base import BaseApplication
 from gunicorn.glogging import Logger
+from werkzeug.wrappers import Request, Response
+from werkzeug.utils import escape
+from werkzeug.exceptions import HTTPException, InternalServerError
 
 from oio.common.utils import get_logger, read_conf, CPU_COUNT
 
@@ -19,14 +37,14 @@ class Application(BaseApplication):
                           self.conf.get('bind_port', '8000'))
         self.cfg.set('bind', bind)
         self.cfg.set('backlog', self.conf.get('backlog', 2048))
-        self.cfg.set('workers', self.conf.get('workers', 1))
-        self.cfg.set('threads', self.conf.get('threads', CPU_COUNT))
+        self.cfg.set('workers', self.conf.get('workers', CPU_COUNT))
         self.cfg.set('worker_class', self.conf.get('worker_class', 'eventlet'))
         self.cfg.set('worker_connections', self.conf.get(
             'worker_connections', 1000))
         self.cfg.set('syslog_prefix', self.conf.get('syslog_prefix', ''))
         self.cfg.set('syslog_addr', self.conf.get('log_address', '/dev/log'))
         self.cfg.set('accesslog', None)
+        self.cfg.set('keepalive', 30)
         self.cfg.set('access_log_format', self.conf.get('access_log_format',
                                                         self.access_log_fmt))
         if self.logger_class:
@@ -76,6 +94,36 @@ class ServiceLogger(Logger):
             super(ServiceLogger, self).access(resp, req, environ, request_time)
 
 
+class WerkzeugApp(object):
+
+    def __init__(self, url_map=None, logger=None):
+        self.url_map = url_map
+        self.logger = logger or get_logger(None)
+
+    def dispatch_request(self, req):
+        adapter = self.url_map.bind_to_environ(req.environ)
+        try:
+            endpoint, _ = adapter.match()
+            resp = getattr(self, 'on_' + endpoint)(req)
+        except HTTPException as exc:
+            resp = exc
+        except Exception as exc:
+            if self.logger:
+                self.logger.exception('ERROR Unhandled exception in request')
+            resp = InternalServerError('Unmanaged error: %s' % exc)
+        if isinstance(resp, HTTPException) and not resp.response:
+            resp.response = Response(escape(resp.description), resp.code)
+        return resp
+
+    def wsgi_app(self, environ, start_response):
+        req = Request(environ)
+        resp = self.dispatch_request(req)
+        return resp(environ, start_response)
+
+    def __call__(self, environ, start_response):
+        return self.wsgi_app(environ, start_response)
+
+
 def init_request_processor(conf_file, app_name, app_factory, *args, **kwargs):
     conf = read_conf(conf_file, app_name)
     if 'logger' in kwargs:
