@@ -1,3 +1,18 @@
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 import re
 import os
 
@@ -5,11 +20,12 @@ import pkg_resources
 from eventlet import GreenPool, sleep
 
 from oio.common.daemon import Daemon
-from oio.common.http import requests
+from oio.common.http import get_pool_manager
 from oio.common.utils import get_logger, float_value, validate_service_conf, \
     int_value, parse_config, true_value
-from oio.common.client import Client
+from oio.common.client import ProxyClient
 from oio.conscience.client import ConscienceClient
+from oio.common.exceptions import OioException
 
 
 def load_modules(group_name):
@@ -42,10 +58,14 @@ class ServiceWatcher(object):
                 self._load_item_config('deregister_on_exit', False))
 
         self.logger = get_logger(self.conf)
-        self.session = requests.Session()
-        self.cs = ConscienceClient(self.conf, session=self.session)
-        self.client = Client(self.conf, session=self.session)
+        self.pool_manager = get_pool_manager()
+        self.cs = ConscienceClient(self.conf, pool_manager=self.pool_manager,
+                                   logger=self.logger)
+        # FIXME: explain that
+        self.client = ProxyClient(self.conf, pool_manager=self.pool_manager,
+                                  no_ns_in_url=True, logger=self.logger)
         self.last_status = False
+        self.status = False
         self.failed = False
         self.service_definition = {
             'ns': self.conf['namespace'],
@@ -71,12 +91,14 @@ class ServiceWatcher(object):
         self.logger.info('watcher "%s" starting', self.name)
         self.running = True
         self.watch()
+        self.running = False
 
     def stop(self):
         self.logger.info('watcher "%s" stopping', self.name)
         if self.deregister_on_exit:
             self.logger.info('watcher "%s" deregister service', self.name)
             try:
+                self.status = False
                 self.last_status = False
                 self.register()
             except Exception as e:
@@ -84,32 +106,45 @@ class ServiceWatcher(object):
         self.running = False
 
     def check(self):
-        status = True
-        for service_check in self.service_checks:
+        """Perform the registered checks on the service until any of
+        them fails of the end of the list is reached."""
+        self.status = True
+        for service_check in (x for x in self.service_checks if self.running):
             if not service_check.service_status():
-                status = False
-
-        if status != self.last_status:
-            if status:
-                self.logger.info('service "%s" is now up', self.name)
-            else:
-                self.logger.warn('service "%s" is now down', self.name)
-            self.last_status = status
+                self.status = False
+                return
 
     def get_stats(self):
         """Update service definition with all configured stats"""
-        if not self.last_status:
+        if not self.status:
             return
-        for stat in self.service_stats:
-            stats = stat.get_stats()
-            self.service_definition['tags'].update(stats)
+        try:
+            for stat in (x for x in self.service_stats if self.running):
+                stats = stat.get_stats()
+                self.service_definition['tags'].update(stats)
+        except Exception as ex:
+            self.logger.debug("get_stats error: %s", ex)
+            self.status = False
 
     def register(self):
+        # only accept a final zero/down-registration when exiting
+        if not self.running and self.status:
+            return
+
+        # Alert when the status changes
+        if self.status != self.last_status:
+            if self.status:
+                self.logger.info('service "%s" is now up', self.name)
+            else:
+                self.logger.warn('service "%s" is now down', self.name)
+            self.last_status = self.status
+
         # Use a boolean so we can easily convert it to a number in conscience
-        self.service_definition['tags']['tag.up'] = self.last_status
+        self.service_definition['tags']['tag.up'] = self.status
         try:
-            self.cs.register(self.service['type'], self.service_definition)
-        except requests.RequestException as rqe:
+            self.cs.register(self.service['type'], self.service_definition,
+                             retries=False)
+        except OioException as rqe:
             self.logger.warn("Failed to register service %s: %s",
                              self.service_definition["addr"], rqe)
 
@@ -164,11 +199,15 @@ class ServiceWatcher(object):
 class ConscienceAgent(Daemon):
     def __init__(self, conf):
         validate_service_conf(conf)
+        self.running = True
         self.conf = conf
         self.logger = get_logger(conf)
         self.load_services()
         self.init_watchers(self.conf['services'])
 
+    def stop(self):
+        self.running = False
+
     def run(self, *args, **kwargs):
         try:
             self.logger.info('conscience agent: starting')
@@ -177,7 +216,8 @@ class ConscienceAgent(Daemon):
             for watcher in self.watchers:
                 pool.spawn(watcher.start)
 
-            while True:
+            self.running = True
+            while self.running:
                 sleep(1)
                 for w in self.watchers:
                     if w.failed:
@@ -192,6 +232,7 @@ class ConscienceAgent(Daemon):
             raise e
         finally:
             self.logger.warn('conscience agent: stopping')
+            self.running = False
             self.stop_watchers()
 
     def init_watchers(self, services):
