@@ -1,3 +1,18 @@
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 from oio.common.http import http_connect
 from oio.common import exceptions as exc
 from oio.conscience.checker.base import BaseChecker
@@ -18,7 +33,7 @@ class HttpChecker(BaseChecker):
         self.name = '%s|http|%s|%s|%s' % \
             (self.srv_type, self.host, self.port, self.path)
         self.netloc = '%s:%s' % (self.host, self.port)
-        self.session = self.agent.session
+        self.pool_manager = self.agent.pool_manager
 
     def check(self):
         success = False
