@@ -1,5 +1,4 @@
-# Copyright (C) 2015-2017 OpenIO, original work as part of
-# OpenIO Software Defined Storage
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -18,29 +17,70 @@ from oio.common import exceptions as exc
 from oio.common.exceptions import ClientException, OrphanChunk
 from oio.common.utils import get_logger
 from oio.blob.client import BlobClient
-from oio.conscience.client import ConscienceClient
 from oio.container.client import ContainerClient
+from urllib import quote_plus
+from oio.common.constants import OIO_VERSION
 
 
 class Content(object):
-    def __init__(self, conf, container_id, metadata, chunks, storage_method):
+
+    def __init__(self, conf, container_id, metadata, chunks, storage_method,
+                 account, container_name, blob_client=None,
+                 container_client=None, logger=None):
         self.conf = conf
         self.container_id = container_id
         self.metadata = metadata
         self.chunks = ChunksHelper(chunks)
         self.storage_method = storage_method
-        self.logger = get_logger(self.conf)
-        self.cs_client = ConscienceClient(conf)
-        self.blob_client = BlobClient()
-        self.container_client = ContainerClient(self.conf)
+        self.logger = logger or get_logger(self.conf)
+        self.blob_client = (blob_client or BlobClient())
+        self.container_client = (container_client
+                                 or ContainerClient(self.conf,
+                                                    logger=self.logger))
+
+        # FIXME: all these may be properties
         self.content_id = self.metadata["id"]
-        self.stgpol = self.metadata["policy"]
         self.path = self.metadata["name"]
         self.length = int(self.metadata["length"])
         self.version = self.metadata["version"]
         self.checksum = self.metadata["hash"]
-        self.mime_type = self.metadata["mime_type"]
         self.chunk_method = self.metadata["chunk_method"]
+        self.account = account
+        self.container_name = container_name
+        if 'full_path' in self.metadata:
+            self.full_path = metadata['full_path']
+        else:
+            self.full_path = ['{0}/{1}/{2}/{3}'.
+                              format(quote_plus(self.account),
+                                     quote_plus(self.container_name),
+                                     quote_plus(self.path),
+                                     self.version)]
+
+    @property
+    def mime_type(self):
+        return self.metadata["mime_type"]
+
+    @mime_type.setter
+    def mime_type(self, value):
+        self.metadata["mime_type"] = value
+
+    @property
+    def policy(self):
+        return self.metadata["policy"]
+
+    @policy.setter
+    def policy(self, value):
+        self.metadata["policy"] = value
+
+    @property
+    def properties(self):
+        return self.metadata.get('properties')
+
+    @properties.setter
+    def properties(self, value):
+        if not isinstance(value, dict):
+            raise ValueError("'value' must be a dict")
+        self.metadata['properties'] = value
 
     def _get_spare_chunk(self, chunks_notin, chunks_broken):
         spare_data = {
@@ -49,8 +89,8 @@ class Content(object):
         }
         try:
             spare_resp = self.container_client.content_spare(
-                cid=self.container_id, content=self.content_id,
-                data=spare_data, stgpol=self.stgpol)
+                cid=self.container_id, path=self.content_id,
+                data=spare_data, stgpol=self.policy)
         except ClientException as e:
             raise exc.SpareChunkException("No spare chunk (%s)" % e.message)
 
@@ -60,6 +100,17 @@ class Content(object):
 
         return url_list
 
+    def _add_raw_chunk(self, current_chunk, url):
+        data = {'type': 'chunk',
+                'id': url,
+                'hash': current_chunk.checksum,
+                'size': current_chunk.size,
+                'pos': current_chunk.pos,
+                'content': self.content_id}
+
+        self.container_client.container_raw_insert(
+            data, cid=self.container_id)
+
     def _update_spare_chunk(self, current_chunk, new_url):
         old = {'type': 'chunk',
                'id': current_chunk.url,
@@ -76,25 +127,42 @@ class Content(object):
         self.container_client.container_raw_update(
             old, new, cid=self.container_id)
 
-    def _create_object(self):
+    def _generate_sysmeta(self):
+        sysmeta = dict()
+        sysmeta['id'] = self.content_id
+        sysmeta['version'] = self.version
+        sysmeta['policy'] = self.policy
+        sysmeta['mime_type'] = self.mime_type
+        sysmeta['chunk_method'] = self.chunk_method
+        sysmeta['chunk_size'] = self.metadata['chunk_size']
+        sysmeta['oio_version'] = OIO_VERSION
+        sysmeta['full_path'] = self.full_path
+        sysmeta['content_path'] = self.path
+        sysmeta['container_id'] = self.container_id
+        return sysmeta
+
+    def _create_object(self, **kwargs):
+        data = {'chunks': self.chunks.raw(),
+                'properties': self.properties}
         self.container_client.content_create(
             cid=self.container_id, path=self.path, content_id=self.content_id,
-            stgpol=self.stgpol, size=self.length, checksum=self.checksum,
+            stgpol=self.policy, size=self.length, checksum=self.checksum,
             version=self.version, chunk_method=self.chunk_method,
-            mime_type=self.mime_type, data=self.chunks.raw())
+            mime_type=self.mime_type, data=data,
+            **kwargs)
 
-    def rebuild_chunk(self, chunk_id, allow_same_rawx=False):
+    def rebuild_chunk(self, chunk_id, allow_same_rawx=False, chunk_pos=None):
         raise NotImplementedError()
 
-    def create(self, stream):
+    def create(self, stream, **kwargs):
         raise NotImplementedError()
 
     def fetch(self):
         raise NotImplementedError()
 
-    def delete(self):
-        self.container_client.content_delete(cid=self.container_id,
-                                             path=self.path)
+    def delete(self, **kwargs):
+        self.container_client.content_delete(
+            cid=self.container_id, path=self.path, **kwargs)
 
     def move_chunk(self, chunk_id):
         current_chunk = self.chunks.filter(id=chunk_id).one()
@@ -114,7 +182,7 @@ class Content(object):
 
         try:
             self.blob_client.chunk_delete(current_chunk.url)
-        except:
+        except Exception:
             self.logger.warn("Failed to delete chunk %s" % current_chunk.url)
 
         current_chunk.url = spare_urls[0]
