@@ -1,3 +1,19 @@
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+
 from __future__ import print_function
 import sys
 import random
@@ -167,7 +183,7 @@ class Runner(object):
             sys.exit(0)
         except SystemExit:
             raise
-        except:
+        except Exception:
             self.logger.exception("Exception in worker process")
             if not worker.started:
                 sys.exit(self.WORKER_START_ERROR)
