@@ -1,3 +1,19 @@
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
+#
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3.0 of the License, or (at your option) any later version.
+#
+# This library is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public
+# License along with this library.
+
+
 import os
 import sys
 from oio.common.utils import yaml
@@ -114,7 +130,7 @@ class Reader(object):
         try:
             self.purge()
             self._buffer.close()
-        except:
+        except Exception:
             pass
 
         self._buffer = None
@@ -162,21 +178,23 @@ class Connection(object):
     @classmethod
     def from_url(cls, url, **kwargs):
         url = urlparse(url)
+        if not url.netloc:
+            raise ConnectionError('Invalid URL')
         url_options = {}
         url_options.update({
             'host': url.hostname,
-            'port': int(url.port or 11300)})
+            'port': int(url.port)})
         kwargs.update(url_options)
         return cls(**kwargs)
 
-    def __init__(self, host='localhost', port=11300, use_tubes=None,
+    def __init__(self, host=None, port=None, use_tubes=None,
                  watch_tubes=None, socket_timeout=None,
                  socket_connect_timeout=None, socket_keepalive=False,
                  socket_keepalive_options=None, encoding='utf-8',
                  socket_read_size=65536):
         self.pid = os.getpid()
         self.host = host
-        self.port = port
+        self.port = int(port)
         self.encoding = encoding
         self.socket_timeout = socket_timeout
         self.socket_connect_timeout = socket_connect_timeout
@@ -313,14 +331,14 @@ class Connection(object):
                 errmsg = e.args[1]
             raise ConnectionError("Error %s while writing to socket. %s." %
                                   (errno, errmsg))
-        except:
+        except Exception:
             self.disconnect()
             raise
 
     def read_response(self):
         try:
             response = self._parser.read_response()
-        except:
+        except Exception:
             self.disconnect()
             raise
         if isinstance(response, ResponseError):
@@ -330,7 +348,7 @@ class Connection(object):
     def read_body(self, size):
         try:
             response = self._parser.read(size)
-        except:
+        except Exception:
             self.disconnect()
             raise
         if isinstance(response, ResponseError):
@@ -385,22 +403,29 @@ class Beanstalk(object):
     EXPECTED_ERR = dict_merge(
         {'reserve': ['DEADLINE_SOON', 'TIMED_OUT'],
          'delete': ['NOT_FOUND'],
-         'release': ['BURIED', 'NOT_FOUND'],
-         'bury': ['NOT_FOUND'],
+         'release': ['BURIED', 'NOT_FOUND', 'OUT_OF_MEMORY'],
+         'bury': ['NOT_FOUND', 'OUT_OF_MEMORY'],
          'stats-tube': ['NOT_FOUND'],
          'use': [],
          'watch': [],
-         'put': ['JOB_TOO_BIG', 'BURIED', 'DRAINING'],
-         'kick': [],
-         'kick-job': ['NOT_FOUND']}
+         'put': ['JOB_TOO_BIG', 'BURIED', 'DRAINING', 'OUT_OF_MEMORY'],
+         'kick': ['OUT_OF_MEMORY'],
+         'kick-job': ['NOT_FOUND', 'OUT_OF_MEMORY']}
     )
 
     @classmethod
     def from_url(cls, url, **kwargs):
+        if url is None or not url:
+            raise ConnectionError('Empty URL')
+        if not url.startswith('beanstalk://'):
+            import warnings
+            warnings.warn(
+                    'Invalid URL scheme, expecting beanstalk',
+                    DeprecationWarning)
         connection = Connection.from_url(url, **kwargs)
         return cls(connection=connection)
 
-    def __init__(self, host='localhost', port=11300, socket_timeout=None,
+    def __init__(self, host=None, port=None, socket_timeout=None,
                  socket_connect_timeout=None, socket_keepalive=None,
                  retry_on_timeout=False, socket_keepalive_options=None,
                  max_connections=None, connection=None):
@@ -408,7 +433,7 @@ class Beanstalk(object):
             self.socket_timeout = socket_timeout
             kwargs = {
                 'host': host,
-                'port': port,
+                'port': int(port),
                 'socket_connect_timeout': socket_connect_timeout,
                 'socket_keepalive': socket_keepalive,
                 'socket_keepalive_options': socket_keepalive_options,
