@@ -1,3 +1,19 @@
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+
 import time
 import signal
 import os
@@ -6,11 +22,11 @@ import sys
 import greenlet
 import eventlet
 from eventlet import Timeout, greenthread
+from oio.common.exceptions import OioNetworkException, ClientException
 
 from oio.conscience.client import ConscienceClient
 from oio.rdir.client import RdirClient
 from oio.event.beanstalk import Beanstalk, ConnectionError
-from oio.common.http import requests
 from oio.common.utils import true_value, drop_privileges, \
         json, int_value
 from oio.event.evob import is_success, is_error
@@ -95,11 +111,13 @@ class Worker(object):
 
 
 class EventTypes(object):
+    ACCOUNT_SERVICES = 'account.services'
     CHUNK_NEW = 'storage.chunk.new'
     CHUNK_DELETED = 'storage.chunk.deleted'
     CONTAINER_NEW = 'storage.container.new'
     CONTAINER_DELETED = 'storage.container.deleted'
     CONTAINER_STATE = 'storage.container.state'
+    CONTENT_BROKEN = 'storage.content.broken'
     CONTENT_NEW = 'storage.content.new'
     CONTENT_DELETED = 'storage.content.deleted'
 
@@ -114,12 +132,15 @@ def _stop(client, server):
 
 
 class EventWorker(Worker):
+    def __init__(self, *args, **kwargs):
+        super(EventWorker, self).__init__(*args, **kwargs)
+        self.app_env = dict()
+
     def init(self):
         eventlet.monkey_patch(os=False)
         self.tube = self.conf.get("tube", DEFAULT_TUBE)
-        self.session = requests.Session()
-        self.cs = ConscienceClient(self.conf)
-        self.rdir = RdirClient(self.conf)
+        self.cs = ConscienceClient(self.conf, logger=self.logger)
+        self.rdir = RdirClient(self.conf, logger=self.logger)
         self._acct_addr = None
         self.acct_update = 0
         self.graceful_timeout = 1
@@ -128,6 +149,7 @@ class EventWorker(Worker):
         )
         self.acct_update = true_value(self.conf.get('acct_update', True))
         self.rdir_update = true_value(self.conf.get('rdir_update', True))
+        self.app_env['acct_addr'] = self.acct_addr
         if 'handlers_conf' not in self.conf:
             raise ValueError("'handlers_conf' path not defined in conf")
         self.handlers = loadhandlers(self.conf.get('handlers_conf'),
@@ -145,22 +167,24 @@ class EventWorker(Worker):
             env['job_id'] = job_id
             return env
         except Exception as exc:
-            self.logger.warn('decoding job "%s"', str(exc.message))
+            self.logger.warn('Failed to decode job %s: "%s"',
+                             job_id, str(exc.message))
             return None
 
     def run(self):
         coros = []
-        queue_url = self.conf.get('queue_url', '127.0.0.1:11300')
+        queue_url = self.conf.get('queue_url', 'beanstalk://127.0.0.1:11300')
         concurrency = int_value(self.conf.get('concurrency'), 10)
 
         server_gt = greenthread.getcurrent()
 
-        for i in range(concurrency):
-            beanstalk = Beanstalk.from_url(queue_url)
-            gt = eventlet.spawn(self.handle, beanstalk)
-            gt.link(_eventlet_stop, server_gt, beanstalk)
-            coros.append(gt)
-            beanstalk, gt = None, None
+        for url in queue_url.split(';'):
+            for i in range(concurrency):
+                beanstalk = Beanstalk.from_url(url)
+                gt = eventlet.spawn(self.handle, beanstalk)
+                gt.link(_eventlet_stop, server_gt, beanstalk)
+                coros.append(gt)
+                beanstalk, gt = None, None
 
         while self.alive:
             self.notify()
@@ -198,12 +222,22 @@ class EventWorker(Worker):
                         conn_error = True
                     eventlet.sleep(BEANSTALK_RECONNECTION)
                     continue
-                try:
-                    event = self.safe_decode_job(job_id, data)
-                    self.process_event(job_id, event, beanstalk)
-                except Exception:
-                    self.logger.exception("handling event %s (bury)", job_id)
+                event = self.safe_decode_job(job_id, data)
+                if not event:
+                    self.logger.warn("Burying event %s: %s",
+                                     job_id, "malformed")
                     beanstalk.bury(job_id)
+                else:
+                    try:
+                        self.process_event(job_id, event, beanstalk)
+                    except (ClientException, OioNetworkException) as exc:
+                        self.logger.warn("Burying event %s (%s): %s",
+                                         job_id, event.get('event'), exc)
+                        beanstalk.bury(job_id)
+                    except Exception:
+                        self.logger.exception("Burying event %s: %s",
+                                              job_id, event)
+                        beanstalk.bury(job_id)
         except StopServe:
             pass
 
@@ -228,7 +262,6 @@ class EventWorker(Worker):
     def get_handler(self, event):
         return self.handlers.get(event.get('event'), None)
 
-    @property
     def acct_addr(self):
         if not self._acct_addr or self.acct_refresh():
             acct_instance = self.cs.next_instance(ACCOUNT_SERVICE)
