@@ -1,3 +1,19 @@
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU Affero General Public License as
+# published by the Free Software Foundation, either version 3 of the
+# License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU Affero General Public License for more details.
+#
+# You should have received a copy of the GNU Affero General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+
 from functools import partial
 
 
@@ -23,13 +39,14 @@ class Event(object):
     job_id = _event_env_property('job_id')
     event_type = _event_env_property('event')
     data = _event_env_property('data')
+    reqid = _event_env_property('request_id')
     when = _event_env_property('when')
 
     def __init__(self, env):
         self.env = env
 
     def __repr__(self):
-        return "Event [%s](%s)" % (self.job_id, self.event_type)
+        return "Event [%s,%s](%s)" % (self.job_id, self.reqid, self.event_type)
 
 
 class Response(object):
