@@ -1,7 +1,7 @@
 /*
 OpenIO SDS proxy
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -20,15 +20,17 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #ifndef OIO_SDS__proxy__actions_h
 # define OIO_SDS__proxy__actions_h 1
 
-#define OIO_CONTAINER_CREATE_MANY_NB_MAX_REQ 100
-#define OIO_CONTENT_DELETE_MANY_NB_MAX_REQ 100
-
 struct req_args_s;
 
-enum http_rc_e action_forward (struct req_args_s *args);
-enum http_rc_e action_forward_stats (struct req_args_s *args);
-
 enum http_rc_e action_cache_status (struct req_args_s *args);
+enum http_rc_e action_get_config (struct req_args_s *args);
+enum http_rc_e action_set_config (struct req_args_s *args);
+
+enum http_rc_e action_forward_get_config (struct req_args_s *args);
+enum http_rc_e action_forward_set_config (struct req_args_s *args);
+enum http_rc_e action_forward_stats (struct req_args_s *args);
+enum http_rc_e action_forward (struct req_args_s *args);
+
 enum http_rc_e action_cache_flush_local (struct req_args_s *args);
 enum http_rc_e action_cache_flush_high (struct req_args_s *args);
 enum http_rc_e action_cache_flush_low (struct req_args_s *args);
@@ -40,6 +42,7 @@ enum http_rc_e action_cache_set_max_high (struct req_args_s *args);
 enum http_rc_e action_lb_reload (struct req_args_s *args);
 enum http_rc_e action_lb_choose (struct req_args_s *args);
 enum http_rc_e action_lb_poll (struct req_args_s *args);
+enum http_rc_e action_lb_create_pool(struct req_args_s *args);
 
 enum http_rc_e action_local_list (struct req_args_s *args);
 
@@ -80,6 +83,7 @@ enum http_rc_e action_container_raw_update (struct req_args_s *args);
 enum http_rc_e action_container_raw_delete (struct req_args_s *args);
 
 enum http_rc_e action_content_put (struct req_args_s *args);
+enum http_rc_e action_content_drain(struct req_args_s *args);
 enum http_rc_e action_content_delete (struct req_args_s *args);
 enum http_rc_e action_content_delete_many (struct req_args_s *args);
 enum http_rc_e action_content_show (struct req_args_s *args);
