@@ -1,6 +1,6 @@
 /*
 OpenIO SDS proxy
-Copyright (C) 2015 OpenIO, original work as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -23,13 +23,14 @@ gint32 oio_proxy_request_failure_threshold_first = 100;
 gint32 oio_proxy_request_failure_threshold_middle = 50;
 gint32 oio_proxy_request_failure_threshold_last = 0;
 
-gchar *
-proxy_get_csurl (void)
+gchar **
+proxy_get_cs_urlv (void)
 {
 	g_rw_lock_reader_lock (&csurl_rwlock);
-	const guint i = oio_ext_rand_int_range(0, csurl_count);
-	gchar *cs = g_strdup(csurl[i]);
+	gchar **cs = g_strdupv(csurl);
 	g_rw_lock_reader_unlock (&csurl_rwlock);
+
+	oio_ext_array_shuffle((void**)cs, g_strv_length(cs));
 	return cs;
 }
 
@@ -157,7 +158,8 @@ static void _sort_services (struct client_ctx_s *ctx,
 		const char *k, gchar **m1uv) {
 	_debug_services ("PRE sort: ", m1uv);
 
-	gsize pivot = g_strv_length (m1uv);
+	gsize len = g_strv_length (m1uv);
+	gsize pivot = len;
 
 	/* prefer services recently available */
 	if (pivot > 1)
@@ -180,9 +182,22 @@ static void _sort_services (struct client_ctx_s *ctx,
 		}
 	}
 
-	/* If multiple available & preferred services, shuffle them */
-	if (pivot > 1)
-		oio_ext_array_shuffle ((void**)m1uv, pivot);
+	if (oio_proxy_srv_shuffle) {
+		switch (pivot) {
+			case 1:
+				break;
+			case 0:
+				/* If no available & preferred service, shuffle them */
+				if (len > 1)
+					pivot = len;
+				else
+					break;
+				// FALLTHROUGH
+			default:
+				/* If multiple available & preferred services, shuffle them */
+				oio_ext_array_shuffle((void**)m1uv, pivot);
+		}
+	}
 
 	_debug_services ("POST sort: ", m1uv);
 }
@@ -214,16 +229,19 @@ static gboolean _on_reply (gpointer p, MESSAGE reply) {
 	return TRUE;
 }
 
-GError *gridd_request_replicated (struct client_ctx_s *ctx,
-		request_packer_f pack) {
+GError *
+gridd_request_replicated (struct client_ctx_s *ctx, request_packer_f pack)
+{
+	gboolean retry = TRUE;
 	GError *err = NULL;
+	gchar **m1uv = NULL;
 	EXTRA_ASSERT (ctx != NULL);
 
 	gchar *election_key = g_strconcat (ctx->name.base, "/", ctx->name.type, NULL);
 	STRING_STACKIFY(election_key);
 
 	/* Locate the services */
-	gchar **m1uv = NULL;
+label_retry:
 	if (*ctx->type == '#')
 		err = hc_resolve_reference_directory (resolver, ctx->url, &m1uv);
 	else
@@ -231,21 +249,32 @@ GError *gridd_request_replicated (struct client_ctx_s *ctx,
 
 	if (err) {
 		EXTRA_ASSERT(m1uv == NULL);
-		g_prefix_error (&err, "Directory error: ");
-		return err;
+		if (retry && err->code == CODE_RANGE_NOTFOUND) {
+			retry = FALSE;
+			hc_decache_reference_service(resolver, ctx->url, NAME_SRVTYPE_META1);
+			hc_decache_reference(resolver, ctx->url);
+			goto label_retry;
+		} else {
+			g_prefix_error (&err, "Directory error: ");
+			return err;
+		}
 	} else {
 		EXTRA_ASSERT(m1uv != NULL);
+
 		if (*ctx->type == '#') {
+			/* when looking for a directory service, the resolver always replies
+			 * all the services involved. Let's keep only the services with the
+			 * targeted type */
 			gchar **tmp = meta1_url_filter_typed (
 					(const char * const *)m1uv, ctx->type+1);
 			if (m1uv)
 				g_strfreev(m1uv);
 			m1uv = tmp;
-		}
-		if (!*m1uv) {
+		} else if (!*m1uv) {
 			g_strfreev (m1uv);
 			return NEWERROR (CODE_CONTAINER_NOTFOUND, "No service located");
 		}
+
 		/* We found some locations, let's keep only the URL part */
 		meta1_urlv_shift_addr (m1uv);
 		/* let's prefer the services requested (master, slave, etc) */
@@ -258,47 +287,63 @@ GError *gridd_request_replicated (struct client_ctx_s *ctx,
 		*errorv = g_ptr_array_new (), /* <GError*> */
 		*bodyv = g_ptr_array_new (); /* <GByteArray*> */
 
-	GByteArray *packed = pack(sqlx_name_mutable_to_const(&ctx->name));
+	NAME2CONST(n,ctx->name);
+	GByteArray *packed = pack(&n);
 
 	gboolean stop = FALSE;
 	for (gchar **pu=m1uv; *pu && !stop ;++pu) {
 
 		const char *url = pu[0];
 		const char *next_url = pu[1];
-		struct gridd_client_s *client = NULL;
 		GByteArray *body = NULL;
 
-		/* TODO ensure the service match the expected TYPE and SEQ */
-		if (!ctx->decoder) {
-			client = gridd_client_create (url, packed, &body, _on_reply);
+		struct gridd_client_s *client = gridd_client_create_empty();
+		if (!client) {
+			err = SYSERR("Memory allocation error");
 		} else {
-			client = gridd_client_create (url, packed, ctx->decoder_data, ctx->decoder);
+			err = gridd_client_connect_url(client, url);
+			if (err) {
+				GRID_WARN("Invalid peer [%s]", url);
+				err->code = ERRCODE_CONN_NOROUTE;
+			}
 		}
 
-#ifdef HAVE_ENBUG
-		gint32 threshold = 0;
-		if (url == m1uv[0] && !next_url)
-			threshold = oio_proxy_request_failure_threshold_alone;
-		else if (url == m1uv[0])
-			threshold = oio_proxy_request_failure_threshold_first;
-		else if (next_url == NULL)
-			threshold = oio_proxy_request_failure_threshold_last;
-		else
-			threshold = oio_proxy_request_failure_threshold_middle;
-		if (threshold >= oio_ext_rand_int_range(1, 100)) {
-			err = NEWERROR(CODE_AVOIDED, "FAKE ERROR");
-		} else {
-#endif /* HAVE_ENBUG */
-			/* Send a unitary request */
-			if (ctx->which == CLIENT_RUN_ALL)
-				gridd_client_no_redirect (client);
-			gridd_client_start (client);
-			gridd_client_set_timeout (client, ctx->timeout);
-			if (!(err = gridd_client_loop (client)))
-				err = gridd_client_error (client);
-#ifdef HAVE_ENBUG
+		if (!err) {
+			/* TODO ensure the service match the expected TYPE and SEQ */
+			if (!ctx->decoder) {
+				err = gridd_client_request(client, packed, &body, _on_reply);
+			} else {
+				err = gridd_client_request(client, packed, ctx->decoder_data, ctx->decoder);
+			}
 		}
+
+		if (!err) {
+#ifdef HAVE_ENBUG
+			gint32 threshold = 0;
+			if (url == m1uv[0] && !next_url)
+				threshold = oio_proxy_request_failure_threshold_alone;
+			else if (url == m1uv[0])
+				threshold = oio_proxy_request_failure_threshold_first;
+			else if (next_url == NULL)
+				threshold = oio_proxy_request_failure_threshold_last;
+			else
+				threshold = oio_proxy_request_failure_threshold_middle;
+			if (threshold >= oio_ext_rand_int_range(1, 100)) {
+				err = NEWERROR(CODE_AVOIDED, "FAKE ERROR");
+			} else {
+#endif /* HAVE_ENBUG */
+				/* Send a unitary request */
+				if (ctx->which == CLIENT_RUN_ALL)
+					gridd_client_no_redirect (client);
+				gridd_client_start (client);
+				gridd_client_set_timeout (client, ctx->timeout);
+				if (!(err = gridd_client_loop (client)))
+					err = gridd_client_error (client);
+#ifdef HAVE_ENBUG
+			}
 #endif
+		}
+
 		/* ensure an output for that request: each array (url, body, error)
 		 * must contain the correspondinng item. */
 		if (err) {
@@ -319,7 +364,7 @@ GError *gridd_request_replicated (struct client_ctx_s *ctx,
 		/* Check for a possible redirection */
 		if (flag_prefer_master_for_read || flag_prefer_slave_for_read
 				|| flag_prefer_master_for_write) {
-			const char *actual = gridd_client_url(client);
+			const char *actual = client ? gridd_client_url(client) : NULL;
 			if (actual && 0 != strcmp(actual, url)) {
 				gchar *k = g_strdup(election_key);
 				gchar *v = g_strdup(actual);
@@ -330,18 +375,35 @@ GError *gridd_request_replicated (struct client_ctx_s *ctx,
 
 		if (err) {
 			if (CODE_IS_NETWORK_ERROR(err->code)) {
+				/* the target service is in bad shape, let's avoid it for
+				 * the subsequent requests. */
 				service_invalidate(url);
-				/* that error is not strong enoough to stop the iteration, we
+
+				/* TODO(jfs): should we let the client retry or occupy a
+				 * thread in the proxy to make all the necessary retries ? */
+
+				/* that error is not strong enough to stop the iteration, we
 				 * just try with another service */
 				g_clear_error (&err);
-				/* But if we expected at least one service to responnd (!ALL),
+
+				/* But if we expected at least one service to respond,
 				 * and we still encounter that error with the last URL of the
 				 * array (!pu[1]), then this is an overall error that we should return. */
 				if (ctx->which != CLIENT_RUN_ALL && !next_url) {
 					err = BUSY("No service replied");
 					stop = TRUE;
 				}
+			} else if (CODE_IS_RETRY(err->code)) {
+				/* the target service is in bad shape, let's avoid it for
+				 * the subsequent requests. And we currently we choose to
+				 * stop the iteration and let the retry be achieved in the
+				 * client SDK. This error is a clue that the other replicas
+				 * will also be overloaded. */
+				service_invalidate(url);
+				stop = TRUE;
 			} else if (ctx->which == CLIENT_RUN_ALL) {
+				/* All the services must be reached, let's just remind the
+				 * error (already done) and continue to the next service */
 				g_clear_error (&err);
 			} else {
 				stop = TRUE;
@@ -351,8 +413,10 @@ GError *gridd_request_replicated (struct client_ctx_s *ctx,
 				stop = TRUE;
 		}
 
-		gridd_client_free (client);
-		client = NULL;
+		if (client) {
+			gridd_client_free (client);
+			client = NULL;
+		}
 	}
 
 	EXTRA_ASSERT(urlv->len == bodyv->len);
@@ -455,19 +519,18 @@ GBytes* service_is_wanted (const char *type) {
 }
 
 void client_init (struct client_ctx_s *ctx, struct req_args_s *args,
-		const char *srvtype, gint seq) {
+		const char *srvtype, gint64 seq) {
 	memset(ctx, 0, sizeof(*ctx));
 	ctx->url = args->url;
 	ctx->type = srvtype;
 	ctx->seq = seq;
-	sqlx_name_fill_type_asis (&ctx->name, args->url,
+	sqlx_inline_name_fill_type_asis (&ctx->name, args->url,
 			*srvtype == '#' ? srvtype+1 : srvtype, ctx->seq);
-	ctx->timeout = COMMON_CLIENT_TIMEOUT;
+	ctx->timeout = proxy_timeout_common;
 	ctx->which = CLIENT_PREFER_NONE;
 }
 
 void client_clean (struct client_ctx_s *ctx) {
-	sqlx_name_clean(&ctx->name);
 	if (ctx->urlv)
 		g_strfreev (ctx->urlv);
 	if (ctx->errorv) {
@@ -529,3 +592,50 @@ GError * KV_read_usersys_properties (struct json_object *j, gchar ***out) {
 	*out = kv;
 	return NULL;
 }
+
+static gboolean
+_cb_exec_and_concat (GByteArray *tmp, MESSAGE reply)
+{
+	gsize bsize = 0;
+	void *b = metautils_message_get_BODY(reply, &bsize);
+	if (b && bsize)
+		g_byte_array_append(tmp, b, bsize);
+	return TRUE;
+}
+
+GError *
+gridd_client_exec_and_concat_string (const gchar *to, gdouble seconds,
+		GByteArray *req, gchar **out)
+{
+	EXTRA_ASSERT(to != NULL);
+	EXTRA_ASSERT(out == NULL || *out == NULL);
+
+	GError *err = NULL;
+	GByteArray *tmp = g_byte_array_sized_new(512);
+
+	struct gridd_client_s *client = gridd_client_create(
+			to, req, tmp, (client_on_reply)_cb_exec_and_concat);
+	g_byte_array_unref (req);
+	req = NULL;
+
+	if (!client) {
+		return SYSERR("client creation");
+	} else {
+		if (seconds > 0.0)
+			gridd_client_set_timeout (client, seconds);
+		gridd_client_set_avoidance(client, FALSE);
+		err = gridd_client_run (client);
+		gridd_client_free (client);
+	}
+
+	if (!err && out) {
+		g_byte_array_append (tmp, (guint8*)"", 1);
+		*out = (gchar*) g_byte_array_free (tmp, FALSE);
+		tmp = NULL;
+	}
+	if (tmp)
+		g_byte_array_free (tmp, TRUE);
+
+	return err;
+}
+
