@@ -1,7 +1,7 @@
 /*
 OpenIO SDS proxy
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -32,19 +32,25 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <json-c/json.h>
 
 #include <core/url_ext.h>
+#include <core/client_variables.h>
+
 #include <metautils/lib/metautils.h>
+#include <metautils/lib/common_variables.h>
+#include <proxy/proxy_variables.h>
+
 #include <cluster/lib/gridcluster.h>
 #include <server/network_server.h>
+#include <server/server_variables.h>
 #include <resolver/hc_resolver.h>
 #include <meta1v2/meta1_remote.h>
 #include <meta2v2/meta2_macros.h>
-#include <meta2v2/meta2v2_remote.h>
 #include <meta2v2/meta2_utils.h>
 #include <meta2v2/autogen.h>
 #include <meta2v2/generic.h>
 #include <sqliterepo/sqlx_remote.h>
 #include <sqliterepo/sqlite_utils.h>
 
+#include "meta2v2_remote.h"
 #include "path_parser.h"
 #include "transport_http.h"
 
@@ -92,9 +98,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define MASTER_READ(Action)  GUARDED_READ(master_rwlock,Action)
 #define MASTER_WRITE(Action) GUARDED_WRITE(master_rwlock,Action)
 
-#define CSURL(C) gchar *C = NULL; do { \
-	C = proxy_get_csurl(); \
-	STRING_STACKIFY(C); \
+#define CSURL(C) gchar **C = NULL; do { \
+	C = proxy_get_cs_urlv(); \
+	STRINGV_STACKIFY(C); \
 } while (0)
 
 #define COMA(gs,first) do { \
@@ -104,6 +110,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 		first = FALSE; \
 } while (0)
 
+/* ------------------------------------------------------------------------- */
+
 enum proxy_preference_e {
 	CLIENT_PREFER_NONE = 0,
 	CLIENT_RUN_ALL,
@@ -112,38 +120,6 @@ enum proxy_preference_e {
 };
 
 extern gchar *ns_name;
-extern gboolean flag_cache_enabled;
-extern gboolean flag_local_scores;
-
-/* Should the proxy prefer the master it knows, even when a SLAVE is enough
- * (read-only requests) */
-extern gboolean flag_prefer_master_for_read;
-
-/* Should the proxy prefer the master it knows, when a MASTER is required
- * (write requests) */
-extern gboolean flag_prefer_master_for_write;
-
-/* Should the proxy prefer the SLAVE it knows, when a SLAVE is enough (read-only
- * requests) */
-extern gboolean flag_prefer_slave_for_read;
-
-/* Should the proxy tell the service to accept the requests only if the base has
- * the MASTER status. Currently only managed by the meta2 services. */
-extern gboolean flag_force_master;
-
-/* how long the proxy remembers the srv it registered ino the conscience */
-extern gint64 ttl_expire_local_services;
-
-/* how long the proxy remembers dead services */
-extern gint64 ttl_down_services;
-
-/* how long the proxy remembers services from the conscience */
-extern gint64 ttl_known_services;
-
-/* how long the proxy remembers which service is the master for a given
- * election */
-extern gint64 ttl_expire_master_services;
-
 extern struct oio_lb_world_s *lb_world;
 extern struct oio_lb_s *lb;
 extern struct hc_resolver_s *resolver;
@@ -152,7 +128,6 @@ extern struct hc_resolver_s *resolver;
 extern GRWLock nsinfo_rwlock;
 extern gchar **srvtypes;
 extern struct namespace_info_s nsinfo;
-extern gint64 ns_chunk_size;
 gboolean validate_namespace (const char * ns);
 gboolean validate_srvtype (const char * n);
 
@@ -161,7 +136,7 @@ gboolean validate_srvtype (const char * n);
 extern GRWLock csurl_rwlock;
 extern gchar **csurl;
 extern gsize csurl_count;
-gchar * proxy_get_csurl (void);
+gchar ** proxy_get_cs_urlv (void);
 
 /* Periodically loads lists of services from the conscience, and keep this
  * in cache. */
@@ -181,25 +156,23 @@ extern struct lru_tree_s *srv_registered; /* registered srv seen within 5s */
 
 extern GRWLock srv_rwlock;
 extern struct lru_tree_s *srv_down; /* "IP:PORT" that had a problem */
+extern struct lru_tree_s *srv_known; /* services seen since 'ever' */
+
 gboolean service_is_ok (gconstpointer p);
 void service_invalidate (gconstpointer n);
-extern struct lru_tree_s *srv_known; /* services seen since 'ever' */
+
 void service_learn (const char *key);
 gboolean service_is_known (const char *key);
 
 /* Set of items requiring an election, associated to the latest known master */
 extern GRWLock master_rwlock;
 extern struct lru_tree_s *srv_master;
+
 gboolean service_is_slave (const char *obj, const char *master);
 gboolean service_is_master (const char *obj, const char *master);
 void service_learn_master (const char *obj, const char *master);
 guint service_expire_masters (gint64 oldest);
 
-typedef GByteArray * (request_packer_f) (const struct sqlx_name_s *);
-
-#define PACKER_VOID(N) GByteArray * N (const struct sqlx_name_s *u UNUSED)
-#define PACKER(N)      GByteArray * N (const struct sqlx_name_s *n)
-
 struct req_args_s
 {
 	struct oio_requri_s *req_uri; // parsed URI
@@ -208,12 +181,8 @@ struct req_args_s
 
 	struct http_request_s *rq;
 	struct http_reply_ctx_s *rp;
-};
 
-struct sub_action_s
-{
-	const char *verb;
-	enum http_rc_e (*handler) (struct req_args_s *, struct json_object *);
+	gint64 version;
 };
 
 typedef enum http_rc_e (*req_handler_f) (struct req_args_s *);
@@ -235,7 +204,7 @@ struct client_ctx_s {
 	gpointer decoder_data;
 
 	/* input */
-	struct sqlx_name_mutable_s name;
+	struct sqlx_name_inline_s name;
 	struct oio_url_s *url;
 	const char *type;
 	gint64 seq;
@@ -250,7 +219,7 @@ struct client_ctx_s {
 };
 
 void client_init (struct client_ctx_s *ctx, struct req_args_s *args,
-	   const char *srvtype, gint seq);
+	   const char *srvtype, gint64 seq);
 
 void client_clean (struct client_ctx_s *ctx);
 
@@ -276,7 +245,12 @@ enum proxy_preference_e _prefer_master (void);
 	CLIENT_CTX(ctx,args,type,seq); \
 	ctx.which = _prefer_slave()
 
-GError * _m1_locate_and_action (struct oio_url_s *url, GError * (*hook) ());
+GError * _m1_locate_and_action (struct oio_url_s *url,
+		GError * (*hook) (const char *m1addr));
+
+typedef GByteArray * (request_packer_f) (const struct sqlx_name_s *);
+
+#define PACKER_VOID(N) GByteArray * N (const struct sqlx_name_s *_u UNUSED)
 
 GError * gridd_request_replicated (struct client_ctx_s *, request_packer_f);
 
@@ -297,15 +271,16 @@ gboolean lb_cache_reload(void);
 enum http_rc_e _reply_json (struct req_args_s *args, int code, const char * msg, GString * gstr);
 
 enum http_rc_e _reply_format_error (struct req_args_s *args, GError *err);
-enum http_rc_e _reply_system_error (struct req_args_s *args, GError *err);
 enum http_rc_e _reply_bad_gateway (struct req_args_s *args, GError *err);
 enum http_rc_e _reply_srv_unavailable (struct req_args_s *args, GError *err);
+enum http_rc_e _reply_retry (struct req_args_s *args, GError *err);
 enum http_rc_e _reply_gateway_timeout (struct req_args_s *args, GError * err);
 enum http_rc_e _reply_not_implemented (struct req_args_s *args);
 enum http_rc_e _reply_notfound_error (struct req_args_s *args, GError * err);
 enum http_rc_e _reply_forbidden_error (struct req_args_s *args, GError * err);
-enum http_rc_e _reply_method_error (struct req_args_s *args);
+enum http_rc_e _reply_method_error (struct req_args_s *args, GError *err, gchar *allowed);
 enum http_rc_e _reply_conflict_error (struct req_args_s *args, GError * err);
+enum http_rc_e _reply_gone_error(struct req_args_s *args, GError *err);
 enum http_rc_e _reply_too_large (struct req_args_s *args, GError * err);
 enum http_rc_e _reply_nocontent (struct req_args_s *args);
 enum http_rc_e _reply_accepted (struct req_args_s *args);
@@ -322,12 +297,12 @@ enum http_rc_e _reply_common_error (struct req_args_s *args, GError *err);
 
 /* -------------------------------------------------------------------------- */
 
-GError * conscience_remote_get_namespace (const char *cs, namespace_info_t **out);
-GError * conscience_remote_get_services(const char *cs, const char *type,
+GError * conscience_remote_get_namespace (gchar **cs, namespace_info_t **out);
+GError * conscience_remote_get_services(gchar **cs, const char *type,
 		gboolean full, GSList **out);
-GError * conscience_remote_get_types(const char *cs, gchar ***out);
-GError * conscience_remote_push_services(const char *cs, GSList *ls);
-GError* conscience_remote_remove_services(const char *cs, const char *type,
+GError * conscience_remote_get_types(gchar **cs, gchar ***out);
+GError * conscience_remote_push_services(gchar **cs, GSList *ls);
+GError* conscience_remote_remove_services(gchar **cs, const char *type,
 		GSList *ls);
 
 /* -------------------------------------------------------------------------- */
@@ -347,4 +322,15 @@ extern gint32 oio_proxy_request_failure_threshold_last;
 extern gint32 oio_proxy_request_failure_threshold_middle;
 #endif
 
+/* ------------------------------------------------------------------------- */
+
+/* Execute the request without avoidance, and discard the output */
+#define CLIENT_EXEC(Url,Timeout,Req) \
+	gridd_client_exec_and_concat_string(Url,Timeout,Req,NULL)
+
+/* wraps gridd_client_exec_and_concat() but disable the avoidance of peers
+ * seen faulty or down. */
+GError * gridd_client_exec_and_concat_string (const char *to, gdouble timeout,
+		GByteArray *req, gchar **out);
+
 #endif /*OIO_SDS__proxy__common_h*/
