@@ -1,7 +1,7 @@
 /*
 OpenIO SDS proxy
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -17,78 +17,114 @@ You should have received a copy of the GNU Affero General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <cluster/module/module.h>
+
 #include "common.h"
 #include "actions.h"
 
-#include <cluster/module/module.h>
-
-GError *
-conscience_remote_get_namespace (const char *cs, namespace_info_t **out)
+static GError *
+_loop_on_allcs_while_neterror(gchar **allcs, GError* (action)(const char *cs))
 {
-	MESSAGE req = metautils_message_create_named(NAME_MSGNAME_CS_GET_NSINFO);
-	GByteArray *gba = NULL;
-	GError *err = gridd_client_exec_and_concat (cs, CS_CLIENT_TIMEOUT,
-			message_marshall_gba_and_clean(req), &gba);
-	if (err) {
-		EXTRA_ASSERT (gba == NULL);
+	EXTRA_ASSERT(allcs != NULL);
+
+	for (gchar **pcs=allcs; *pcs ;++pcs) {
+		GError *err = action(*pcs);
+		if (!err)
+			return NULL;
+		if (CODE_IS_NETWORK_ERROR(err->code)) {
+			GRID_DEBUG("Error toward [%s]: (%d) %s", *pcs, err->code, err->message);
+			g_clear_error(&err);
+			continue;
+		}
 		g_prefix_error(&err, "request: ");
 		return err;
 	}
 
-	*out = namespace_info_unmarshall(gba->data, gba->len, &err);
-	g_byte_array_unref (gba);
-	if (*out) return NULL;
-	GSETERROR(&err, "Decoding error");
-	return err;
+	return BUSY("No conscience replied");
 }
 
 GError *
-conscience_remote_get_services(const char *cs, const char *type, gboolean full,
+conscience_remote_get_namespace (gchar **allcs, namespace_info_t **out)
+{
+	GError * action (const char *cs) {
+		GByteArray *gba = NULL;
+		MESSAGE req = metautils_message_create_named(NAME_MSGNAME_CS_GET_NSINFO);
+		GError *err = gridd_client_exec_and_concat (cs, proxy_timeout_conscience,
+				message_marshall_gba_and_clean(req), &gba);
+		EXTRA_ASSERT ((gba != NULL) ^ (err != NULL));
+		if (err)
+			return err;
+		*out = namespace_info_unmarshall(gba->data, gba->len, &err);
+		g_byte_array_unref (gba);
+		if (*out) return NULL;
+		g_prefix_error(&err, "Decoding error: ");
+		return err;
+	}
+	return _loop_on_allcs_while_neterror(allcs, action);
+}
+
+GError *
+conscience_remote_get_services(gchar **allcs, const char *type, gboolean full,
 		GSList **out)
 {
 	EXTRA_ASSERT(type != NULL);
-	MESSAGE req = metautils_message_create_named(NAME_MSGNAME_CS_GET_SRV);
-	metautils_message_add_field_str (req, NAME_MSGKEY_TYPENAME, type);
-	if (full)
-		metautils_message_add_field_str(req, NAME_MSGKEY_FULL, "1");
-	return gridd_client_exec_and_decode (cs, CS_CLIENT_TIMEOUT,
-			message_marshall_gba_and_clean(req), out, service_info_unmarshall);
-}
-
-GError * conscience_remote_get_types(const char *cs, gchar ***out) {
-	MESSAGE req = metautils_message_create_named (NAME_MSGNAME_CS_GET_SRVNAMES);
-	gchar *json = NULL;
-	GError *err = gridd_client_exec_and_concat_string (cs, CS_CLIENT_TIMEOUT,
-			message_marshall_gba_and_clean(req), &json);
-	EXTRA_ASSERT((err != NULL) ^ (json != NULL));
-	if (!err) {
-		err = STRV_decode_buffer((guint8*)json, strlen(json), out);
-		if (out) {
-			EXTRA_ASSERT((err != NULL) ^ (*out != NULL));
-		}
-		g_free(json);
+	GError * action (const char *cs) {
+		MESSAGE req = metautils_message_create_named(NAME_MSGNAME_CS_GET_SRV);
+		metautils_message_add_field_str (req, NAME_MSGKEY_TYPENAME, type);
+		if (full)
+			metautils_message_add_field_str(req, NAME_MSGKEY_FULL, "1");
+		return gridd_client_exec_and_decode (cs, proxy_timeout_conscience,
+				message_marshall_gba_and_clean(req), out, service_info_unmarshall);
 	}
-	return err;
+	return _loop_on_allcs_while_neterror(allcs, action);
 }
 
 GError *
-conscience_remote_push_services(const char *cs, GSList *ls)
+conscience_remote_get_types(gchar **allcs, gchar ***out)
 {
-	MESSAGE req = metautils_message_create_named (NAME_MSGNAME_CS_PUSH_SRV);
-	metautils_message_add_body_unref (req, service_info_marshall_gba (ls, NULL));
-	return gridd_client_exec (cs, CS_CLIENT_TIMEOUT,
-			message_marshall_gba_and_clean(req));
+	GError * action(const char *cs) {
+		MESSAGE req = metautils_message_create_named (NAME_MSGNAME_CS_GET_SRVNAMES);
+		gchar *json = NULL;
+		GError *err = gridd_client_exec_and_concat_string (cs, proxy_timeout_conscience,
+				message_marshall_gba_and_clean(req), &json);
+		EXTRA_ASSERT((err != NULL) ^ (json != NULL));
+		if (!err) {
+			err = STRV_decode_buffer((guint8*)json, strlen(json), out);
+			if (out) {
+				EXTRA_ASSERT((err != NULL) ^ (*out != NULL));
+			}
+			g_free(json);
+		}
+		return err;
+	}
+	return _loop_on_allcs_while_neterror(allcs, action);
+}
+
+GError *
+conscience_remote_push_services(gchar **allcs, GSList *ls)
+{
+	GError * action(const char *cs) {
+		MESSAGE req = metautils_message_create_named (NAME_MSGNAME_CS_PUSH_SRV);
+		metautils_message_add_body_unref (req, service_info_marshall_gba (ls, NULL));
+		return gridd_client_exec (cs, proxy_timeout_conscience,
+				message_marshall_gba_and_clean(req));
+	}
+	return _loop_on_allcs_while_neterror(allcs, action);
 }
 
 GError*
-conscience_remote_remove_services(const char *cs, const char *type, GSList *ls)
+conscience_remote_remove_services(gchar **allcs, const char *type, GSList *ls)
 {
-	MESSAGE req = metautils_message_create_named (NAME_MSGNAME_CS_RM_SRV);
-	if (ls)
-		metautils_message_add_body_unref (req, service_info_marshall_gba (ls, NULL));
-	if (type) metautils_message_add_field_str (req, NAME_MSGKEY_TYPENAME, type);
-	return gridd_client_exec (cs, CS_CLIENT_TIMEOUT,
-			message_marshall_gba_and_clean(req));
+	GError * action(const char *cs) {
+		MESSAGE req = metautils_message_create_named (NAME_MSGNAME_CS_RM_SRV);
+		if (ls)
+			metautils_message_add_body_unref (req, service_info_marshall_gba (ls, NULL));
+		if (type)
+			metautils_message_add_field_str (req, NAME_MSGKEY_TYPENAME, type);
+		return gridd_client_exec (cs, proxy_timeout_conscience,
+				message_marshall_gba_and_clean(req));
+	}
+	return _loop_on_allcs_while_neterror(allcs, action);
 }
 
 /* -------------------------------------------------------------------------- */
@@ -109,13 +145,14 @@ _cs_check_tokens (struct req_args_s *args)
 static GString *
 _cs_pack_and_free_srvinfo_list (GSList * svc)
 {
-	GString *gstr = g_string_new ("[");
+	GString *gstr = g_string_sized_new (2048);
+	g_string_append_c (gstr, '[');
 	for (GSList * l = svc; l; l = l->next) {
 		if (l != svc)
 			g_string_append_c (gstr, ',');
 		service_info_encode_json (gstr, l->data, FALSE);
 	}
-	g_string_append (gstr, "]");
+	g_string_append_c (gstr, ']');
 	g_slist_free_full (svc, (GDestroyNotify) service_info_clean);
 	return gstr;
 }
@@ -126,129 +163,130 @@ enum reg_op_e {
 	REGOP_UNLOCK,
 };
 
-#ifdef HAVE_EXTRA_DEBUG
-static const char*
-_regop_2str (const enum reg_op_e op)
+static GError *
+_registration_batch (enum reg_op_e op, GSList *services)
 {
-	switch (op) {
-		ON_ENUM(REGOP_,PUSH);
-		ON_ENUM(REGOP_,LOCK);
-		ON_ENUM(REGOP_,UNLOCK);
+	const gint64 now = oio_ext_real_seconds();
+
+	/* Sanity checks and patch of each score */
+	for (GSList *l=services; l ;l=l->next) {
+		struct service_info_s *si = l->data;
+
+		if (!metautils_addr_valid_for_connect(&si->addr))
+			return BADREQ("Invalid service address");
+		if (!si->type[0] && !service_info_get_tag(si->tags, "tag.id"))
+			return BADREQ("Service type not specified");
+		if (!validate_srvtype(si->type))
+			return BADREQ("Service type currently unknown");
+
+		si->score.timestamp = now;
+		switch (op) {
+			case REGOP_PUSH:
+				si->score.value = SCORE_UNSET;
+				continue;
+			case REGOP_LOCK:
+				si->score.value = CLAMP(si->score.value, SCORE_DOWN, SCORE_MAX);
+				continue;
+			case REGOP_UNLOCK:
+				si->score.value = SCORE_UNLOCK;
+				continue;
+		}
+	}
+
+	/* Patch the various caches where services are identified by the
+	 * "service ID" key: the cache of known services, ad the cache of local
+	 * services (if configured).  */
+	for (GSList *l=services; l ;l=l->next) {
+		struct service_info_s *si = l->data;
+		gchar *k = service_info_key (si);
+
+		if (!service_is_known (k)) {
+			service_learn (k);
+			service_tag_set_value_boolean (service_info_ensure_tag (
+						si->tags, NAME_TAGNAME_RAWX_FIRST), TRUE);
+		}
+
+		if (ttl_expire_local_services > 0 && op != REGOP_UNLOCK) {
+			struct service_info_s *v = service_info_dup (si);
+			v->score.timestamp = oio_ext_monotonic_seconds ();
+			REG_WRITE(
+					const struct service_info_s *si0 = lru_tree_get(srv_registered, k);
+					if (si0)
+						v->score.value = si0->score.value;
+					lru_tree_insert (srv_registered, g_strdup(k), v);
+					);
+		}
+
+		g_free(k);
+	}
+
+/* if we receive a simple registration and if a special action
+ * is already pending (lock or unlock), we should not lose the
+ * special action, so merge the old score (i.e. the action code)
+ * in the new services description */
+#define ENQUEUE_SERVICE() { \
+	if (op == REGOP_PUSH) { \
+		struct service_info_s *si0 = lru_tree_get(push_queue, key); \
+		if (si0 && si0->score.value != SCORE_UNSET) \
+			si->score.value = si0->score.value; \
+	} \
+	lru_tree_insert(push_queue, key, si); \
+}
+	if (flag_cache_enabled) {
+		for (GSList *l=services; l ;l=l->next) {
+			struct service_info_s *si = l->data;
+			gchar *key = service_info_key(si);
+			PUSH_WRITE(ENQUEUE_SERVICE());
+			/* Prevent future 'services' list free from freeing element data.
+			 * It will be freed when leaving 'push_queue'. */
+			l->data = NULL;
+		}
+		return NULL;
+	} else {
+		CSURL(cs);
+		return conscience_remote_push_services (cs, services);
 	}
-	g_assert_not_reached ();
-	return "?";
 }
-#endif
 
 static enum http_rc_e
 _registration (struct req_args_s *args, enum reg_op_e op, struct json_object *jsrv)
 {
 	GError *err;
 
-	if (!jsrv || !json_object_is_type (jsrv, json_type_object))
-		return _reply_common_error (args, BADREQ("Expected: json object"));
-
 	if (!push_queue)
 		return _reply_bad_gateway(args, SYSERR("Service upstream disabled"));
 
 	if (NULL != (err = _cs_check_tokens(args)))
 		return _reply_notfound_error (args, err);
 
-	struct service_info_s *si = NULL;
-	err = service_info_load_json_object (jsrv, &si, TRUE);
-
-	if (err) {
-		g_prefix_error (&err, "JSON error: ");
-		if (err->code == CODE_BAD_REQUEST)
-			return _reply_format_error (args, err);
-		else
-			return _reply_system_error (args, err);
-	}
-
-	if (!si->type[0] && !service_info_get_tag(si->tags, "tag.id")) {
-		service_info_clean (si);
-		return _reply_format_error (args, BADREQ("Service type not specified"));
-	}
-
-	if (!validate_srvtype(si->type)) {
-		service_info_clean (si);
-		return _reply_format_error (args, BADREQ("Service type currently unknown"));
-	}
-
-	if (!si->ns_name[0]) {
-		GRID_TRACE2("%s NS forced to %s", __FUNCTION__, si->ns_name);
-		g_strlcpy (si->ns_name, ns_name, sizeof(si->ns_name));
-	} else if (!validate_namespace (si->ns_name)) {
-		service_info_clean (si);
-		return _reply_format_error (args, BADNS());
-	}
-
-	gchar *k = service_info_key (si);
-	STRING_STACKIFY(k);
-	GRID_TRACE2("%s op=%s score=%d key=[%s]", __FUNCTION__,
-			_regop_2str(op), si->score.value, k);
-
-	switch (op) {
-		case REGOP_PUSH:
-			si->score.value = SCORE_UNSET;
-			if (!service_is_known (k)) {
-				service_learn (k);
-				service_tag_set_value_boolean (service_info_ensure_tag (
-							si->tags, NAME_TAGNAME_RAWX_FIRST), TRUE);
-			}
-			break;
-		case REGOP_LOCK:
-			si->score.value = CLAMP(si->score.value, SCORE_DOWN, SCORE_MAX);
-			break;
-		case REGOP_UNLOCK:
-			si->score.value = SCORE_UNLOCK;
-			break;
-		default:
-			g_assert_not_reached();
-	}
-
-	if (ttl_expire_local_services > 0 && op != REGOP_UNLOCK) {
-		struct service_info_s *v = service_info_dup (si);
-		v->score.timestamp = oio_ext_monotonic_seconds ();
-		REG_WRITE(
-			const struct service_info_s *si0 = lru_tree_get(srv_registered, k);
-			if (si0) v->score.value = si0->score.value;
-			lru_tree_insert (srv_registered, g_strdup(k), v);
-		);
-	}
-
-	si->score.timestamp = oio_ext_real_seconds ();
-
-	if (flag_cache_enabled) {
-		GString *gstr = g_string_new ("");
-		service_info_encode_json (gstr, si, TRUE);
-		gchar *key = service_info_key(si);
-		PUSH_WRITE(
-			/* if we receive a simple registration and if a special action
-			 * is already pending (lock or unlock), we should not lose the
-			 * special action, so merge the old score (i.e. the action code)
-			 * in the new services description */
-			if (op == REGOP_PUSH) {
-				struct service_info_s *si0 = lru_tree_get(push_queue, key);
-				if (si0 && si0->score.value != SCORE_UNSET)
-					si->score.value = si0->score.value;
-			}
-			lru_tree_insert(push_queue, key, si);
-		);
-		return _reply_success_json (args, gstr);
-	} else {
-		CSURL(cs);
-		GSList l = {.data = si, .next = NULL};
-		if (NULL != (err = conscience_remote_push_services (cs, &l))) {
-			service_info_clean (si);
-			return _reply_common_error (args, err);
-		} else {
-			GString *gstr = g_string_new ("");
-			service_info_encode_json (gstr, si, TRUE);
-			service_info_clean (si);
-			return _reply_success_json (args, gstr);
+	/* Manage a single service as well as a list of services */
+	GSList *services = NULL;
+	if (json_object_is_type (jsrv, json_type_array)) {
+		const gint max = json_object_array_length(jsrv);
+		for (gint i = 0; i < max; ++i) {
+			struct json_object *jitem = json_object_array_get_idx(jsrv, i);
+			struct service_info_s *si = NULL;
+			err = service_info_load_json_object (jitem, &si, TRUE);
+			if (err)
+				break;
+			services = g_slist_prepend(services, si);
 		}
+	} else if (json_object_is_type (jsrv, json_type_object)) {
+		struct service_info_s *si = NULL;
+		if (!(err = service_info_load_json_object (jsrv, &si, TRUE)))
+			services = g_slist_prepend(services, si);
+	} else {
+		err = BADREQ("Expected: json object");
 	}
+
+	/* Register the whole batch */
+	if (!err)
+		err = _registration_batch(op, services);
+	g_slist_free_full(services, (GDestroyNotify)service_info_clean);
+
+	if (!err)
+		return _reply_success_json (args, NULL);
+	return _reply_common_error (args, err);
 }
 
 /* -------------------------------------------------------------------------- */
@@ -260,11 +298,16 @@ action_conscience_info (struct req_args_s *args)
 	GError *err;
 	const char *v = OPT("what");
 
+#ifdef HAVE_ENBUG
+	if (10 >= oio_ext_rand_int_range(1,100))
+		return _reply_retry(args, NEWERROR(CODE_UNAVAILABLE, "FAKE"));
+#endif
+
 	if (v && !strcmp(v, "types")) {
 		if (NULL != (err = _cs_check_tokens(args)))
 			return _reply_notfound_error (args, err);
 
-		GString *out = g_string_new("");
+		GString *out = g_string_sized_new(128);
 		g_string_append_c(out, '[');
 		NSINFO_READ(if (srvtypes && *srvtypes) {
 			g_string_append_c(out, '"');
@@ -287,7 +330,7 @@ action_conscience_info (struct req_args_s *args)
 	struct namespace_info_s ni = {{0}};
 	NSINFO_READ(namespace_info_copy (&nsinfo, &ni));
 
-	GString *gstr = g_string_new ("");
+	GString *gstr = g_string_sized_new (2048);
 	namespace_info_encode_json (gstr, &ni);
 	namespace_info_clear (&ni);
 	return _reply_success_json (args, gstr);
@@ -304,7 +347,8 @@ action_local_list (struct req_args_s *args)
 	if (NULL != (err = _cs_check_tokens(args)))
 		return _reply_notfound_error(args, err);
 
-	GString *gs = g_string_new ("[");
+	GString *gs = g_string_sized_new (2048);
+	g_string_append_c (gs, '[');
 	gboolean _on_service (gpointer k, gpointer v, gpointer u) {
 		(void) k, (void) u;
 		struct service_info_s *si = v;
@@ -331,6 +375,11 @@ action_conscience_list (struct req_args_s *args)
 	if (!type)
 		return _reply_format_error (args, BADREQ("Missing type"));
 
+#ifdef HAVE_ENBUG
+	if (10 >= oio_ext_rand_int_range(1,100))
+		return _reply_retry(args, NEWERROR(CODE_UNAVAILABLE, "FAKE"));
+#endif
+
 	gboolean full = _request_get_flag (args, "full");
 
 	GError *err;
@@ -365,7 +414,7 @@ action_conscience_list (struct req_args_s *args)
 	if (NULL != err) {
 		g_slist_free_full (sl, (GDestroyNotify) service_info_clean);
 		g_prefix_error (&err, "Conscience error: ");
-		return _reply_system_error (args, err);
+		return _reply_common_error (args, err);
 	}
 
 	args->rp->access_tail ("%s=%u", type, g_slist_length(sl));
@@ -379,6 +428,11 @@ action_conscience_flush (struct req_args_s *args)
 	if (NULL != (err = _cs_check_tokens(args)))
 		return _reply_notfound_error (args, err);
 
+#ifdef HAVE_ENBUG
+	if (10 >= oio_ext_rand_int_range(1,100))
+		return _reply_retry(args, NEWERROR(CODE_UNAVAILABLE, "FAKE"));
+#endif
+
 	const char *srvtype = TYPE();
 	if (!srvtype)
 		return _reply_format_error (args, BADREQ("Missing type"));
@@ -388,7 +442,7 @@ action_conscience_flush (struct req_args_s *args)
 
 	if (err) {
 		g_prefix_error (&err, "Conscience error: ");
-		return _reply_system_error (args, err);
+		return _reply_common_error (args, err);
 	}
 	return _reply_success_json (args, _create_status (CODE_FINAL_OK, "OK"));
 }
