@@ -1,7 +1,7 @@
 /*
 OpenIO SDS proxy
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -22,7 +22,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 static GString * _pack_m1url_list (GString *gstr, gchar ** urlv) {
 	if (!gstr)
-		gstr = g_string_new ("");
+		gstr = g_string_sized_new (128);
 	g_string_append_c (gstr, '[');
 	for (gchar ** v = urlv; urlv && *v; v++) {
 		struct meta1_service_url_s *m1 = meta1_unpack_url (*v);
@@ -42,7 +42,7 @@ static GString * _pack_and_freev_m1url_list (GString *gstr, gchar ** urlv) {
 }
 
 static GError * _m1_action (struct oio_url_s *url, gchar ** m1v,
-		GError * (*hook) (const char * m1)) {
+		GError * (*hook) (const char * m1addr)) {
 	if (m1v && *m1v) {
 		gboolean _wrap (gconstpointer p) {
 			gchar *m1u = meta1_strurl_get_address ((const char*)p);
@@ -51,7 +51,7 @@ static GError * _m1_action (struct oio_url_s *url, gchar ** m1v,
 		}
 		gsize len = oio_ext_array_partition ((void**)m1v,
 				g_strv_length (m1v), _wrap);
-		if (len > 0 && !oio_dir_no_shuffle)
+		if (len > 1 && oio_proxy_dir_shuffle)
 			oio_ext_array_shuffle ((void**)m1v, len);
 	}
 
@@ -93,7 +93,8 @@ static GError * _m1_action (struct oio_url_s *url, gchar ** m1v,
 	return NEWERROR (CODE_UNAVAILABLE, "No meta1 answered");
 }
 
-GError * _m1_locate_and_action (struct oio_url_s *url, GError * (*hook) ()) {
+GError * _m1_locate_and_action (struct oio_url_s *url,
+		GError * (*hook) (const char * m1addr)) {
 	gchar **m1v = NULL;
 	GError *err = hc_resolve_reference_directory (resolver, url, &m1v);
 	if (NULL != err) {
@@ -231,7 +232,8 @@ action_dir_srv_renew (struct req_args_s *args, struct json_object *jargs)
 
 	gchar **urlv = NULL;
 	GError *hook (const char * m1) {
-		return meta1v2_remote_renew_reference_service (m1, args->url, type, dryrun, autocreate, &urlv);
+		return meta1v2_remote_renew_reference_service (
+				m1, args->url, type, dryrun, autocreate, &urlv);
 	}
 
 	GError *err = _m1_locate_and_action (args->url, hook);
@@ -251,6 +253,32 @@ action_dir_srv_renew (struct req_args_s *args, struct json_object *jargs)
 	return _reply_success_json (args, _pack_and_freev_m1url_list (NULL, urlv));
 }
 
+/**
+ * Find replacements for some services linked to a reference.
+ *
+ * Expects a JSON object like:
+ *
+ *  {
+ *    "kept": {
+ *      "seq": 1,
+ *      "type": "meta2",
+ *      "host": "192.168.56.1:6005,192.168.56.2:6005",
+ *      "args": ""
+ *    },
+ *    "replaced": {
+ *      "seq": 1,
+ *      "type": "meta2",
+ *      "host": "192.168.56.3:6005",
+ *      "args": ""
+ *    }
+ *  }
+ *
+ * "kept" is the description of services that must be kept.
+ * "replaced" is the description of services that must be replaced.
+ * Note that the meta1 expects that the union of "kept" and "replaced"
+ * matches exactly the list of services it knows for the specified
+ * sequence number.
+ */
 static enum http_rc_e
 action_dir_srv_relink (struct req_args_s *args, struct json_object *jargs)
 {
@@ -271,12 +299,16 @@ action_dir_srv_relink (struct req_args_s *args, struct json_object *jargs)
 		{NULL, NULL, 0, 0}
 	};
 	err = oio_ext_extract_json (jargs, mapping);
-	if (!err && jkept && json_object_is_type(jkept, json_type_object)) {
-		if (NULL != (err = meta1_service_url_load_json_object (jkept, &m1u_kept)))
+	if (!err && !jkept && !jrepl)
+		err = BADREQ("Missing [kept] and [replaced]");
+	if (!err && jkept) {
+		err = meta1_service_url_load_json_object (jkept, &m1u_kept);
+		if (err)
 			g_prefix_error (&err, "invalid service in [%s]: ", "kept");
 	}
-	if (!err && jrepl && json_object_is_type(jrepl, json_type_object)) {
-		if (NULL != (err = meta1_service_url_load_json_object (jrepl, &m1u_repl)))
+	if (!err && jrepl) {
+		err = meta1_service_url_load_json_object (jrepl, &m1u_repl);
+		if (err)
 			g_prefix_error (&err, "invalid service in [%s]: ", "replaced");
 	}
 
@@ -285,7 +317,8 @@ action_dir_srv_relink (struct req_args_s *args, struct json_object *jargs)
 		GError *hook (const gchar * m1) {
 			if (newset)
 				g_strfreev (newset);
-			return meta1v2_remote_relink_service (m1, args->url, kept, repl, dryrun, &newset);
+			return meta1v2_remote_relink_service (
+					m1, args->url, kept, repl, dryrun, &newset);
 		}
 		kept = meta1_pack_url (m1u_kept);
 		repl = m1u_repl ? meta1_pack_url (m1u_repl) : NULL;
@@ -328,7 +361,8 @@ action_dir_prop_get (struct req_args_s *args, struct json_object *jargs)
 		keys = NULL;
 	}
 	if (!err) {
-		GString *gs = g_string_new("{\"properties\":");
+		GString *gs = g_string_sized_new(256);
+		g_string_append_static(gs, "{\"properties\":");
 		KV_encode_gstr2(gs, pairs);
 		g_string_append_c(gs, '}');
 		g_strfreev(pairs);
@@ -437,15 +471,16 @@ enum http_rc_e action_ref_show (struct req_args_s *args) {
 	if (!err) {
 		gchar **dirv = NULL;
 		err = hc_resolve_reference_directory (resolver, args->url, &dirv);
-		GString *out = g_string_new ("{");
-		g_string_append (out, "\"dir\":");
+		GString *out = g_string_sized_new (512);
+		g_string_append_c (out, '{');
+		g_string_append_static (out, "\"dir\":");
 		if (dirv)
 			out = _pack_and_freev_m1url_list (out, dirv);
 		else
-			g_string_append (out, "null");
-		g_string_append (out, ",\"srv\":");
+			g_string_append_static (out, "null");
+		g_string_append_static (out, ",\"srv\":");
 		out = _pack_and_freev_m1url_list (out, urlv);
-		g_string_append (out, "}");
+		g_string_append_static (out, "}");
 		return _reply_success_json (args, out);
 	}
 	return _reply_common_error (args, err);
