@@ -1,7 +1,7 @@
 /*
 OpenIO SDS proxy
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -43,8 +43,7 @@ _lb_pack_srvid_tab(const char **ids)
 
 		gchar *straddr = NULL;
 		oio_parse_service_key(*pp, NULL, NULL, &straddr);
-		g_string_append_printf(gstr,
-				"{\"addr\":\"%s\",\"id\":\"%s\"}",
+		g_string_append_printf(gstr, "{\"addr\":\"%s\",\"id\":\"%s\"}",
 				straddr, *pp);
 		g_free(straddr);
 	}
@@ -56,14 +55,20 @@ static enum http_rc_e
 _lb(struct req_args_s *args, const char *srvtype)
 {
 	enum http_rc_e code;
-	const char *slot, *sz;
 
-	slot = OPT("slot");
-	sz = OPT("size");
-	guint howmany = sz ? atoi(sz) : 1;
+	const char *slot = OPT("slot");
+	const char *sz = OPT("size");
+
+	gint64 howmany = 1;
+	if (sz && !oio_str_is_number(sz, &howmany))
+		return _reply_format_error(args, BADREQ("Invalid size"));
 
 	struct oio_lb_pool_s *pool = oio_lb_world__create_pool(lb_world, srvtype);
 	GString *targets = g_string_sized_new(64);
+	if (howmany > 1) {
+		g_string_append(targets, sz);
+		g_string_append_c(targets, OIO_CSV_SEP_C);
+	}
 	if (slot) {
 		if (g_str_has_prefix(slot, srvtype)) {
 			g_string_append(targets, slot);
@@ -72,11 +77,9 @@ _lb(struct req_args_s *args, const char *srvtype)
 			g_string_append_printf(targets, "%s-%s"OIO_CSV_SEP, srvtype, slot);
 		}
 	}
-	g_string_append_printf(targets, "%s", srvtype);
-	GRID_DEBUG("Temporary pool [%s] will target [%s] %u times",
-			srvtype, targets->str, howmany);
-	for (; howmany > 0; howmany--)
-		oio_lb_world__add_pool_target(pool, targets->str);
+	g_string_append(targets, srvtype);
+	GRID_DEBUG("Temporary pool [%s] will target [%s]", srvtype, targets->str);
+	oio_lb_world__add_pool_targets(pool, targets->str);
 	g_string_free(targets, TRUE);
 
 	GPtrArray *ids = g_ptr_array_new_with_free_func(g_free);
@@ -84,14 +87,18 @@ _lb(struct req_args_s *args, const char *srvtype)
 		(void)loc;
 		g_ptr_array_add(ids, g_strdup(id));
 	}
-	if (!oio_lb_pool__poll(pool, NULL, _on_id)) {
-		code = _reply_common_error(args,
-				NEWERROR(CODE_POLICY_NOT_SATISFIABLE,
-					"found only %u services matching the criteria",
-					ids->len));
+	gboolean flawed = FALSE;
+	GError *err = oio_lb_pool__poll(pool, NULL, _on_id, &flawed);
+	if (err) {
+		g_prefix_error(&err,
+				"found only %u services matching the criteria: ", ids->len);
+		code = _reply_common_error(args, err);
 	} else {
 		g_ptr_array_add(ids, NULL);
 		GString *gstr = _lb_pack_srvid_tab((const char**)ids->pdata);
+		if (flawed)
+			args->rp->add_header(
+					PROXYD_HEADER_PREFIX "lb-flawed", g_strdup("1"));
 		code = _reply_success_json(args, gstr);
 	}
 
@@ -131,15 +138,16 @@ _json_ids_to_locations(struct json_object *arr, oio_location_t *prev)
 	if (!arr)
 		return prev;
 	GArray *out = g_array_new(TRUE, TRUE, sizeof(oio_location_t));
-	int len = 0;
 	if (prev) {
+		guint len = 0;
 		while (prev[len] != 0)
 			len++;
 		g_array_append_vals(out, prev, len);
 		g_free(prev);
 	}
-	len = json_object_array_length(arr);
-	for (int i = 0; i < len; i++) {
+
+	const guint max = json_object_array_length(arr);
+	for (guint i = 0; i < max; i++) {
 		const char *id = json_object_get_string(
 				json_object_array_get_idx(arr, i));
 		struct oio_lb_item_s *item = oio_lb_world__get_item(lb_world, id);
@@ -151,16 +159,12 @@ _json_ids_to_locations(struct json_object *arr, oio_location_t *prev)
 	return (oio_location_t*) g_array_free(out, FALSE);
 }
 
-static enum http_rc_e
-_poll(struct req_args_s *args, struct json_object *body)
+static GError*
+_decode_lb_body(struct json_object *body,
+		oio_location_t **avoid, oio_location_t **known)
 {
-	GError *err = NULL;
-	enum http_rc_e code;
-	const gchar *policy = OPT("policy");
-	const gchar *pool = OPT("pool");
-
 	if (body && !json_object_is_type(body, json_type_object))
-		return _reply_format_error(args, BADREQ("Expected: json object"));
+		return BADREQ("Expected: json object");
 
 	struct json_object *javoid = NULL, *javoid_locs = NULL;
 	struct json_object *jknown = NULL, *jknown_locs = NULL;
@@ -171,7 +175,27 @@ _poll(struct req_args_s *args, struct json_object *body)
 		{"known_locations", &jknown_locs,  json_type_array, 0},
 		{NULL, NULL, 0, 0}
 	};
+	GError *err = NULL;
 	if (body && (err = oio_ext_extract_json(body, mapping)))
+		return err;
+	*avoid = _json_to_locations(javoid_locs);
+	*known = _json_to_locations(jknown_locs);
+	*avoid = _json_ids_to_locations(javoid, *avoid);
+	*known = _json_ids_to_locations(jknown, *known);
+	return NULL;
+}
+
+static enum http_rc_e
+_poll(struct req_args_s *args, struct json_object *body)
+{
+	GError *err = NULL;
+	enum http_rc_e code;
+	const gchar *policy = OPT("policy");
+	const gchar *pool = OPT("pool");
+	oio_location_t *avoid = NULL, *known = NULL;
+
+	err = _decode_lb_body(body, &avoid, &known);
+	if (err)
 		return _reply_common_error(args, err);
 
 	if (pool) {
@@ -189,24 +213,23 @@ _poll(struct req_args_s *args, struct json_object *body)
 		storage_policy_clean(sp);
 	}
 
-	oio_location_t *avoid = _json_to_locations(javoid_locs);
-	oio_location_t *known = _json_to_locations(jknown_locs);
-	avoid = _json_ids_to_locations(javoid, avoid);
-	known = _json_ids_to_locations(jknown, known);
-
 	GPtrArray *ids = g_ptr_array_new_with_free_func(g_free);
 	void _on_id(oio_location_t loc, const char *id) {
 		(void)loc;
 		g_ptr_array_add(ids, g_strdup(id));
 	}
-	if (!oio_lb__patch_with_pool(lb, pool, avoid, known, _on_id)) {
-		code = _reply_common_error(args,
-				NEWERROR(CODE_POLICY_NOT_SATISFIABLE,
-					"found only %u services matching the criteria",
-					ids->len));
+	gboolean flawed = FALSE;
+	err = oio_lb__patch_with_pool(lb, pool, avoid, known, _on_id, &flawed);
+	if (err) {
+		g_prefix_error(&err,
+				"found only %u services matching the criteria: ", ids->len);
+		code = _reply_common_error(args, err);
 	} else {
 		g_ptr_array_add(ids, NULL);
 		GString *gstr = _lb_pack_srvid_tab((const char**)ids->pdata);
+		if (flawed)
+			args->rp->add_header(
+					PROXYD_HEADER_PREFIX "lb-flawed", g_strdup("1"));
 		code = _reply_json(args, CODE_FINAL_OK, "OK", gstr);
 	}
 
@@ -239,3 +262,59 @@ action_lb_reload (struct req_args_s *args)
 	return _reply_success_json (args, NULL);
 }
 
+static enum http_rc_e
+_create_pool(struct req_args_s *args, struct json_object *body)
+{
+	if (!body || !json_object_is_type(body, json_type_object))
+		return _reply_common_error(args, BADREQ("Expected: json object"));
+
+	const gchar *name = OPT("name");
+
+	struct json_object *jtargets = NULL, *jopts = NULL;
+	struct oio_ext_json_mapping_s mapping[] = {
+		{"targets", &jtargets, json_type_string, 1},
+		{"options", &jopts,    json_type_object, 0},
+		{NULL, NULL, 0, 0}
+	};
+	GError *err = NULL;
+	if (body && (err = oio_ext_extract_json(body, mapping))) {
+		return _reply_format_error(args, err);
+	}
+
+	struct oio_lb_pool_s *pool = oio_lb_world__create_pool(lb_world, name);
+	oio_lb_world__add_pool_targets(pool, json_object_get_string(jtargets));
+
+	if (jopts) {
+		json_object_object_foreach(jopts, key, val) {
+			if (json_object_is_type(val, json_type_array) ||
+					json_object_is_type(val, json_type_object)) {
+				oio_lb_pool__destroy(pool);
+				err = BADREQ("Bad value for option %s: "
+						"should be a string, integer or boolean", key);
+				return _reply_format_error(args, err);
+			}
+			oio_lb_world__set_pool_option(pool, key,
+					json_object_get_string(val));
+		}
+	}
+
+	oio_lb__force_pool(lb, pool);
+
+	return _reply_created(args);
+}
+
+enum http_rc_e
+action_lb_create_pool(struct req_args_s *args)
+{
+	GError *err = NULL;
+	if (!validate_namespace(NS()))
+		err = NEWERROR(CODE_NAMESPACE_NOTMANAGED, "Invalid NS");
+	else if	(!OPT("name"))
+		err = BADREQ("Missing name parameter");
+	else if (oio_lb__has_pool(lb, OPT("name")))
+		err = NEWERROR(CODE_CONTENT_EXISTS, "A pool named [%s] already exists",
+				OPT("name"));
+	if (err)
+		return _reply_common_error(args, err);
+	return rest_action(args, _create_pool);
+}
