@@ -1,7 +1,7 @@
 /*
 OpenIO SDS proxy
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -19,7 +19,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <errno.h>
 
-#include <metautils/lib/metautils.h>
 #include <meta2v2/meta2_utils_json.h>
 #include <meta2v2/meta2_bean.h>
 #include <meta2v2/autogen.h>
@@ -108,6 +107,8 @@ _reply_m2_error (struct req_args_s *args, GError * err)
 	if (err->code == CODE_CONTAINER_NOTEMPTY ||
 			err->code == CODE_CONTENT_EXISTS)
 		return _reply_conflict_error (args, err);
+	if (err->code == CODE_CONTENT_DRAINED)
+		return _reply_gone_error(args, err);
 	return _reply_common_error (args, err);
 }
 
@@ -184,7 +185,7 @@ static void
 _dump_json_aliases_and_headers(GString *gstr, GSList *aliases,
 		GTree *headers, GTree *props)
 {
-	g_string_append (gstr, "\"objects\":[");
+	g_string_append_static (gstr, "\"objects\":[");
 	gboolean first = TRUE;
 	for (; aliases ; aliases=aliases->next) {
 		COMA(gstr,first);
@@ -198,21 +199,18 @@ _dump_json_aliases_and_headers(GString *gstr, GSList *aliases,
 		GSList *prop_list = g_tree_lookup(props, prop_key);
 		g_free(prop_key);
 
-		g_string_append(gstr, "{");
-		oio_str_gstring_append_json_pair(gstr, "name", ALIASES_get_alias(a)->str);
-		g_string_append_printf(gstr,
-				",\"ver\":%"G_GINT64_FORMAT
-				",\"ctime\":%"G_GINT64_FORMAT
-				",\"mtime\":%"G_GINT64_FORMAT
-				",\"deleted\":%s"
-				",\"content\":\"",
-				ALIASES_get_version(a),
-				ALIASES_get_ctime(a),
-				ALIASES_get_mtime(a),
-				ALIASES_get_deleted(a) ? "true" : "false");
-
-		metautils_gba_to_hexgstr(gstr, ALIASES_get_content(a));
-		g_string_append_c (gstr, '"');
+		g_string_append_c(gstr, '{');
+		OIO_JSON_append_gstr(gstr, "name", ALIASES_get_alias(a));
+		g_string_append_c(gstr, ',');
+		OIO_JSON_append_int(gstr, "version", ALIASES_get_version(a));
+		g_string_append_c(gstr, ',');
+		OIO_JSON_append_int(gstr, "ctime", ALIASES_get_ctime(a));
+		g_string_append_c(gstr, ',');
+		OIO_JSON_append_int(gstr, "mtime", ALIASES_get_mtime(a));
+		g_string_append_c(gstr, ',');
+		OIO_JSON_append_bool(gstr, "deleted", ALIASES_get_deleted(a));
+		g_string_append_c(gstr, ',');
+		OIO_JSON_append_gba(gstr, "content", ALIASES_get_content(a));
 
 		if (h) {
 			g_string_append_c (gstr, ',');
@@ -229,7 +227,7 @@ _dump_json_aliases_and_headers(GString *gstr, GSList *aliases,
 				metautils_gba_to_hexgstr(gstr, hh);
 				g_string_append_c (gstr, '"');
 			} else {
-				g_string_append(gstr, "null");
+				g_string_append_static(gstr, "null");
 			}
 
 			g_string_append_printf(gstr, ",\"size\":%"G_GINT64_FORMAT,
@@ -238,7 +236,7 @@ _dump_json_aliases_and_headers(GString *gstr, GSList *aliases,
 					CONTENTS_HEADERS_get_mime_type(h)->str);
 		}
 		if (prop_list) {
-			g_string_append(gstr, ",\"properties\":{");
+			g_string_append_static(gstr, ",\"properties\":{");
 			gboolean inner_first = TRUE;
 			for (GSList *prop = prop_list;
 					prop && prop->data;
@@ -305,7 +303,7 @@ static void
 _dump_json_prefixes (GString *gstr, GTree *tree_prefixes)
 {
 	gchar **prefixes = gtree_string_keys (tree_prefixes);
-	g_string_append (gstr, "\"prefixes\":[");
+	g_string_append_static (gstr, "\"prefixes\":[");
 	if (prefixes) {
 		gboolean first = TRUE;
 		for (gchar **pp=prefixes; *pp ;++pp) {
@@ -320,18 +318,28 @@ _dump_json_prefixes (GString *gstr, GTree *tree_prefixes)
 static void
 _dump_json_properties (GString *gstr, GTree *properties)
 {
-	g_string_append (gstr, "\"properties\":{");
-	if (properties) {
-		gboolean first = TRUE;
-		gboolean _func (gpointer k, gpointer v, gpointer i) {
-			(void) i;
-			COMA(gstr,first);
-			oio_str_gstring_append_json_pair (gstr, (const char *)k, (const char *)v);
-			return FALSE;
+	gboolean first = TRUE;
+	gboolean _func (gpointer k, gpointer v, gpointer i) {
+		gboolean want_system = GPOINTER_TO_INT(i);
+		gboolean is_user = g_str_has_prefix((gchar*)k, "user.");
+		if (want_system ^ is_user) {
+			COMA(gstr, first);
+			if (is_user)
+				k += sizeof("user.") - 1;
+			oio_str_gstring_append_json_pair(gstr, (const char *)k, (const char *)v);
 		}
-		g_tree_foreach (properties, _func, NULL);
+		return FALSE;
 	}
-	g_string_append (gstr, "}");
+	g_string_append_static(gstr, "\"properties\":{");
+	if (properties) {
+		g_tree_foreach(properties, _func, GINT_TO_POINTER(0));
+	}
+	g_string_append_static(gstr, "},\"system\":{");
+	if (properties) {
+		first = TRUE;
+		g_tree_foreach(properties, _func, GINT_TO_POINTER(1));
+	}
+	g_string_append_c(gstr, '}');
 }
 
 static enum http_rc_e
@@ -345,7 +353,8 @@ _reply_list_result (struct req_args_s *args, GError * err,
 	 * in the headers. */
 	_container_new_props_to_headers (args, out->props);
 
-	GString *gstr = g_string_new ("{");
+	GString *gstr = g_string_sized_new (4096);
+	g_string_append_c (gstr, '{');
 	_dump_json_prefixes (gstr, tree_prefixes);
 	g_string_append_c (gstr, ',');
 	_dump_json_properties (gstr, out->props);
@@ -362,7 +371,7 @@ _reply_beans (struct req_args_s *args, GError * err, GSList * beans)
 	if (err)
 		return _reply_m2_error (args, err);
 
-	GString *gstr = g_string_new ("");
+	GString *gstr = g_string_sized_new (2048);
 	_json_dump_all_beans (gstr, beans);
 	_bean_cleanl2 (beans);
 	return _reply_success_json (args, gstr);
@@ -447,7 +456,11 @@ _reply_simplified_beans (struct req_args_s *args, GError *err,
 	struct bean_ALIASES_s *alias = NULL;
 	struct bean_CONTENTS_HEADERS_s *header = NULL;
 	gboolean first = TRUE;
-	GString *gstr = body ? g_string_new ("[") : NULL;
+	GString *gstr = NULL;
+	if (body) {
+		gstr = g_string_sized_new (2048);
+		g_string_append_c (gstr, '[');
+	}
 
 	beans = g_slist_sort(beans, _bean_compare_kind);
 
@@ -457,7 +470,7 @@ _reply_simplified_beans (struct req_args_s *args, GError *err,
 
 		if (&descr_struct_CHUNKS == DESCR(l0->data) && gstr) {
 			if (!first)
-				g_string_append (gstr, ",\n");
+				g_string_append_static (gstr, ",\n");
 			first = FALSE;
 
 			// Serialize the chunk
@@ -466,7 +479,7 @@ _reply_simplified_beans (struct req_args_s *args, GError *err,
 			g_string_append_printf (gstr, "{\"url\":\"%s\"", CHUNKS_get_id (chunk)->str);
 			g_string_append_printf (gstr, ",\"pos\":\"%s\"", CHUNKS_get_position (chunk)->str);
 			g_string_append_printf (gstr, ",\"size\":%"G_GINT64_FORMAT, CHUNKS_get_size (chunk));
-			g_string_append (gstr, ",\"hash\":\"");
+			g_string_append_static (gstr, ",\"hash\":\"");
 			metautils_gba_to_hexgstr (gstr, CHUNKS_get_hash (chunk));
 			g_string_append_printf(gstr, "\",\"score\":%d}", score);
 		}
@@ -484,10 +497,12 @@ _reply_simplified_beans (struct req_args_s *args, GError *err,
 		}
 		else if (&descr_struct_PROPERTIES == DESCR(l0->data)) {
 			struct bean_PROPERTIES_s *prop = l0->data;
-			gchar *k = g_strdup_printf (PROXYD_HEADER_PREFIX "content-meta-x-%s",
+			gchar *k = g_strdup_printf (PROXYD_HEADER_PREFIX "content-meta-%s",
 					PROPERTIES_get_key(prop)->str);
 			GByteArray *v = PROPERTIES_get_value (prop);
-			args->rp->add_header(k, g_strndup ((gchar*)v->data, v->len));
+			g_byte_array_append(v, (guint8*)"", 1);  // Ensure nul-terminated
+			args->rp->add_header(k,
+					g_uri_escape_string((gchar*)v->data, NULL, FALSE));
 			g_free (k);
 		}
 	}
@@ -513,7 +528,7 @@ _get_hash (const char *s, GByteArray **out)
 	*out = NULL;
 	GByteArray *h = metautils_gba_from_hexstring (s);
 	if (!h)
-		return BADREQ("JSON: invalid hash: not hexa");
+		return BADREQ("JSON: invalid hash: not hexa: '%s'", s);
 
 	const gssize len = h->len;
 	if (len != g_checksum_type_get_length(G_CHECKSUM_MD5)
@@ -744,7 +759,7 @@ static GError * _load_content_from_json_object(struct req_args_s *args,
 			 * header. Then if there is no error, complete it with properties */
 			if (!(err = _load_content_from_json_array(args, jchunks, out))) {
 				GSList *beans = _load_properties_from_strv(props);
-				for (GSList *l=beans; l ;l=l->next)
+				for (GSList *l = beans; l; l = l->next)
 					PROPERTIES_set2_alias(l->data, oio_url_get(args->url, OIOURL_PATH));
 				*out = metautils_gslist_precat(*out, beans);
 			}
@@ -758,13 +773,8 @@ static GError * _load_content_from_json_object(struct req_args_s *args,
 static enum http_rc_e
 _reply_properties (struct req_args_s *args, GError * err, GSList * beans)
 {
-	if (err) {
-		if (err->code == CODE_BAD_REQUEST)
-			return _reply_format_error (args, err);
-		if (CODE_IS_NOTFOUND(err->code))
-			return _reply_notfound_error (args, err);
-		return _reply_system_error (args, err);
-	}
+	if (err)
+		return _reply_common_error (args, err);
 
 	for (GSList *l=beans; l ;l=l->next) {
 		if (DESCR(l->data) == &descr_struct_ALIASES)
@@ -774,7 +784,8 @@ _reply_properties (struct req_args_s *args, GError * err, GSList * beans)
 	}
 
 	gboolean first = TRUE;
-	GString *gs = g_string_new("{\"properties\":{");
+	GString *gs = g_string_sized_new(1024);
+	g_string_append_static(gs, "{\"properties\":{");
 	for (GSList *l=beans; l ;l=l->next) {
 		if (DESCR(l->data) != &descr_struct_PROPERTIES)
 			continue;
@@ -825,17 +836,18 @@ struct filter_ctx_s
 	GTree *prefixes;
 	guint count; // aliases in <beans>
 	const char *prefix;
+	const char *marker;
 	char delimiter;
 };
 
 static void
-_filter (struct filter_ctx_s *ctx, GSList *l)
+_filter_list_result(struct filter_ctx_s *ctx, GSList *l)
 {
 	void forget (GSList *p) { if (p->data) _bean_clean (p->data); g_slist_free1 (p); }
 	void prepend (GSList *p) { p->next = ctx->beans; ctx->beans = p; }
 
 	gsize prefix_len = ctx->prefix ? strlen(ctx->prefix) : 0;
-	for (GSList *tmp; l ;l=tmp) {
+	for (GSList *tmp; l; l = tmp) {
 		tmp = l->next;
 		l->next = NULL;
 
@@ -855,9 +867,15 @@ _filter (struct filter_ctx_s *ctx, GSList *l)
 
 		const char *name = ALIASES_get_alias(l->data)->str;
 		if (ctx->delimiter) {
-			const char *p = strchr(name+prefix_len, ctx->delimiter);
+			const char *p = strchr(name + prefix_len, ctx->delimiter);
 			if (p) {
-				g_tree_insert(ctx->prefixes, g_strndup(name, (p-name)+1), GINT_TO_POINTER(1));
+				// We must not respond a prefix equal to the marker.
+				if (!ctx->marker ||
+						strncmp(name, ctx->marker, (p - name) + 1)) {
+					g_tree_insert(ctx->prefixes,
+							g_strndup(name, (p - name) + 1),
+							GINT_TO_POINTER(1));
+				}
 				forget (l);
 			} else {
 				ctx->count ++;
@@ -871,12 +889,19 @@ _filter (struct filter_ctx_s *ctx, GSList *l)
 }
 
 static GError *
-_m2_container_create_with_properties (struct req_args_s *args, char **props)
+_m2_container_create_with_properties (struct req_args_s *args, char **props,
+		const char *container_stgpol, const char *container_verpol)
 {
 	gboolean autocreate = TRUE;
+
+	/* JFS: don't lookup for default verpol and stgpol, we must left they unset
+	 * so that they will follow the default values of the namespace and (later)
+	 * of the account. This is how we do NOW, by letting the meta2 find the best
+	 * value when necessary. */
 	struct m2v2_create_params_s param = {
-			OPT("stgpol"), OPT("verpol"), props, FALSE
+			container_stgpol, container_verpol, props, FALSE
 	};
+
 	GError *err = NULL;
 	PACKER_VOID (_pack) { return m2v2_remote_pack_CREATE (args->url, &param); }
 
@@ -889,12 +914,11 @@ retry:
 					   err->code, err->message);
 			autocreate = FALSE; /* autocreate just once */
 			g_clear_error (&err);
-			GError *hook_dir (const gchar *m1) {
-				gchar **urlv = NULL;
-				gchar realtype[64];
+			GError *hook_dir (const char *m1) {
+				gchar **urlv = NULL, realtype[64];
 				_get_meta2_realtype (args, realtype, sizeof(realtype));
-				GError *e = meta1v2_remote_link_service (m1, args->url,
-														 realtype, FALSE, TRUE, &urlv);
+				GError *e = meta1v2_remote_link_service (
+						m1, args->url, realtype, FALSE, TRUE, &urlv);
 				if (!e && urlv && *urlv) {
 					/* Explicitely feeding the meta1 avoids a subsequent
 					   call to meta1 to locate the meta2 */
@@ -913,14 +937,22 @@ retry:
 	return err;
 }
 
-static void
-_re_enable (struct req_args_s *args, struct sqlx_name_s *name)
+static GError *
+_m2_container_create_with_defaults (struct req_args_s *args)
 {
-	GByteArray* _pack_enable () { return sqlx_pack_ENABLE (name); }
-	GError *e = _resolve_meta2 (args, _prefer_master(), _pack_enable, NULL);
+	return _m2_container_create_with_properties(args, NULL, NULL, NULL);
+}
+
+static void
+_re_enable (struct req_args_s *args)
+{
+	GError *e = _resolve_meta2(args, CLIENT_PREFER_MASTER, sqlx_pack_ENABLE, NULL);
+	if (e && e->code == CODE_CONTAINER_ENABLED)
+		g_clear_error(&e);
 	if (e) {
-		GRID_INFO("Failed to un-freeze [%s]", oio_url_get(args->url, OIOURL_WHOLE));
-		g_clear_error (&e);
+		GRID_INFO("Failed to un-freeze [%s]: (%d) %s",
+				oio_url_get(args->url, OIOURL_WHOLE), e->code, e->message);
+		g_clear_error(&e);
 	}
 }
 
@@ -929,44 +961,43 @@ action_m2_container_destroy (struct req_args_s *args)
 {
 	GError *err = NULL;
 	gchar **urlv = NULL;
-	struct sqlx_name_mutable_s n = {NULL,NULL,NULL};
 
-	const gboolean flush = _request_get_flag (args, "flush");
 	const gboolean force = _request_get_flag (args, "force");
 
-	/* TODO make this const! */
-	struct sqlx_name_s *name = sqlx_name_mutable_to_const(&n);
 	/* TODO FIXME manage container subtype */
-	sqlx_name_fill (&n, args->url, NAME_SRVTYPE_META2, 1);
+	struct sqlx_name_inline_s n0;
+	sqlx_inline_name_fill (&n0, args->url, NAME_SRVTYPE_META2, 1);
+	NAME2CONST(n, n0);
 
-	/* pre-loads the locations of the container. We will need this at the
+	/* 0. Pre-loads the locations of the container. We will need this at the
 	 * destroy step. */
 	err = hc_resolve_reference_service (resolver, args->url, n.type, &urlv);
+	if (!err && (!urlv || !*urlv))
+		err = NEWERROR(CODE_CONTAINER_NOTFOUND, "No service located");
 
 	/* 1. FREEZE the base to avoid writings during the operation */
 	if (!err) {
-		PACKER_VOID (_pack) { return sqlx_pack_FREEZE (name); }
-		if (NULL != (err = _resolve_meta2 (args, _prefer_master(), _pack, NULL))) {
-			/* rollback! */
-			_re_enable (args, name);
+		err = _resolve_meta2 (args, CLIENT_PREFER_MASTER,
+							  sqlx_pack_FREEZE, NULL);
+		if (err != NULL && err->code == CODE_CONTAINER_FROZEN)
+			g_clear_error(&err);
+		if (err != NULL && CODE_IS_NETWORK_ERROR(err->code)) {
+			/* rollback! There are chances the request made a timeout
+			 * but was actually managed by the server. */
+			_re_enable (args);
 			goto clean_and_exit;
 		}
 	}
 
 	/* 2. FLUSH the base on the MASTER, so events are generated for all the
 	   contents removed. */
-	if (!err) {
-		if (flush) {
-			PACKER_VOID(_pack) { return m2v2_remote_pack_FLUSH (args->url); }
-			err = _resolve_meta2 (args, _prefer_master(), _pack, NULL);
-		} else if (!force) {
-			guint32 flags = flag_force_master ? M2V2_FLAG_MASTER : 0;
-			PACKER_VOID(_pack) { return m2v2_remote_pack_ISEMPTY (args->url, flags); }
-			err = _resolve_meta2 (args, _prefer_master(), _pack, NULL);
-		}
-		if (NULL != err) {
+	if (!err && !force) {
+		guint32 flags = flag_force_master ? M2V2_FLAG_MASTER : 0;
+		PACKER_VOID(_pack) { return m2v2_remote_pack_ISEMPTY (args->url, flags); }
+		err = _resolve_meta2 (args, _prefer_master(), _pack, NULL);
+		if (err != NULL) {
 			/* rollback! */
-			_re_enable (args, name);
+			_re_enable (args);
 			goto clean_and_exit;
 		}
 	}
@@ -976,33 +1007,58 @@ action_m2_container_destroy (struct req_args_s *args)
 		GError * _unlink (const char * m1) {
 			return meta1v2_remote_unlink_service (m1, args->url, n.type);
 		}
-		if (NULL != (err = _m1_locate_and_action (args->url, _unlink))) {
-			_re_enable (args, name);
+		err = _m1_locate_and_action (args->url, _unlink);
+		hc_decache_reference_service (resolver, args->url, n.type);
+		if (err != NULL) {
+			/* Rolling back will be hard if there is any chance the UNLINK has
+			 * been managed by the server, despite a time-out that occured. */
+			_re_enable (args);
 			goto clean_and_exit;
 		}
 	}
 
-	hc_decache_reference_service (resolver, args->url, n.type);
-
 	/* 4. DESTROY each local base */
 	if (!err && urlv && *urlv) {
 		const guint32 flag_force = (force) ? M2V2_DESTROY_FORCE : 0;
-		const guint32 flag_flush = (flush) ? M2V2_DESTROY_FLUSH : 0;
 
 		meta1_urlv_shift_addr(urlv);
-		err = m2v2_remote_execute_DESTROY (urlv[0], args->url,
-				M2V2_DESTROY_EVENT|flag_force|flag_flush);
-		if (!err && urlv[1]) {
+		err = m2v2_remote_execute_DESTROY(urlv[0], args->url,
+				M2V2_DESTROY_EVENT|flag_force);
+		if (err != NULL) {
+			/* rollback! */
+			struct meta1_service_url_s m1u;
+			m1u.seq = 1;
+			g_strlcpy(m1u.srvtype, n.type, LIMIT_LENGTH_SRVTYPE);
+			gchar *hosts = g_strjoinv(OIO_CSV_SEP, urlv);
+			g_strlcpy(m1u.host, hosts, 256);
+			g_free(hosts);
+			m1u.args[0] = 0;
+			GError * _link (const char * m1) {
+				gchar *packed = meta1_pack_url(&m1u);
+				GError *e = meta1v2_remote_force_reference_service(
+						m1, args->url, packed, FALSE, FALSE);
+				g_free(packed);
+				return e;
+			}
+			GError *_err = _m1_locate_and_action(args->url, _link);
+			if (_err) {
+				GRID_ERROR("Failed to re-link the meta2 services for %s: "
+						"(%d) %s", oio_url_get(args->url, OIOURL_HEXID),
+						_err->code, _err->message);
+				g_error_free(_err);
+			}
+			_re_enable(args);
+			goto clean_and_exit;
+		} else if (urlv[1]) {
 			err = m2v2_remote_execute_DESTROY_many(urlv+1, args->url,
-					flag_force|flag_flush);
+					flag_force);
 		}
 	}
 
 clean_and_exit:
 	if (urlv)
 		g_strfreev (urlv);
-	sqlx_name_clean (&n);
-	if (NULL != err)
+	if (err != NULL)
 		return _reply_m2_error(args, err);
 	return _reply_nocontent (args);
 }
@@ -1181,13 +1237,21 @@ _m2_container_create (struct req_args_s *args, struct json_object *jbody)
 	if (err)
 		return _reply_m2_error(args, err);
 
-	err = _m2_container_create_with_properties(args, properties);
+	err = _m2_container_create_with_properties(args, properties,
+			KV_get_value(properties, M2V2_ADMIN_STORAGE_POLICY),
+			KV_get_value(properties, M2V2_ADMIN_VERSIONING_POLICY));
 	g_strfreev (properties);
 
 	if (err && CODE_IS_NOTFOUND(err->code))
 		return _reply_forbidden_error (args, err);
 	if (err && err->code == CODE_CONTAINER_EXISTS) {
-		g_clear_error (&err);
+		/* We did not create it, thus we cannot _reply_created() */
+		g_clear_error(&err);
+		return _reply_nocontent(args);
+	}
+	if (!err || err->code == CODE_FINAL_OK) {
+		/* No error means we actually created it. */
+		g_clear_error(&err);
 		return _reply_created(args);
 	}
 	return _reply_m2_error (args, err);
@@ -1211,10 +1275,8 @@ _bulk_item_result(GString *gresponse,
 }
 
 static enum http_rc_e
-_m2_container_create_many (struct req_args_s *args, struct json_object *jbody)
+_m2_container_create_many(struct req_args_s *args, struct json_object *jbody)
 {
-	guint max_request = OIO_CONTAINER_CREATE_MANY_NB_MAX_REQ;
-
 	if (!oio_url_get(args->url, OIOURL_ACCOUNT)
 			|| !oio_url_get(args->url, OIOURL_NS))
 		return _reply_format_error(args,
@@ -1227,9 +1289,9 @@ _m2_container_create_many (struct req_args_s *args, struct json_object *jbody)
 				BADREQ("Invalid array of containers"));
 
 	const guint jarray_len = json_object_array_length(jarray);
-	if (jarray_len > max_request)
+	if (jarray_len > proxy_bulk_max_create_many)
 		return _reply_too_large(args, NEWERROR(HTTP_CODE_PAYLOAD_TO_LARGE,
-					"More than %u requested", max_request));
+					"More than %u requested", proxy_bulk_max_create_many));
 
 	/* A final sanity check on the format of the payload */
 	for (guint i = 0; i < jarray_len; i++) {
@@ -1265,7 +1327,9 @@ _m2_container_create_many (struct req_args_s *args, struct json_object *jbody)
 		}
 
 		oio_url_set(args->url, OIOURL_USER, name);
-		err = _m2_container_create_with_properties(args, properties);
+		err = _m2_container_create_with_properties(args, properties,
+				KV_get_value(properties, M2V2_ADMIN_STORAGE_POLICY),
+				KV_get_value(properties, M2V2_ADMIN_VERSIONING_POLICY));
 		g_strfreev(properties);
 		_bulk_item_result(gresponse, i, name, err, HTTP_CODE_CREATED);
 		if (err) g_clear_error(&err);
@@ -1306,7 +1370,7 @@ static GError * _list_loop (struct req_args_s *args,
 
 		/* Action */
 		err = _resolve_meta2_for_list (args, _pack, &out);
-		if (NULL != err) {
+		if (err) {
 			m2v2_list_result_clean (&out);
 			break;
 		}
@@ -1314,7 +1378,7 @@ static GError * _list_loop (struct req_args_s *args,
 		/* Manage the properties */
 		gchar **keys = gtree_string_keys (out.props);
 		if (keys) {
-			for (gchar **pk=keys; *pk ;++pk) {
+			for (gchar **pk = keys; *pk; ++pk) {
 				gchar *v = g_tree_lookup (out.props, *pk);
 				g_tree_steal (out.props, *pk);
 				g_tree_replace (out0->props, *pk, v);
@@ -1331,14 +1395,16 @@ static GError * _list_loop (struct req_args_s *args,
 			ctx.prefixes = tree_prefixes;
 			ctx.count = count;
 			ctx.prefix = in0->prefix;
+			ctx.marker = in0->marker_start;
 			ctx.delimiter = delimiter;
-			_filter (&ctx, out.beans);
+			_filter_list_result(&ctx, out.beans);
 			out.beans = NULL;
 			count = ctx.count;
 			out0->beans = ctx.beans;
 		}
 
-		if (in0->maxkeys > 0 && in0->maxkeys <= (count + g_tree_nnodes(tree_prefixes))) {
+		if (in0->maxkeys > 0 &&
+				(count + g_tree_nnodes(tree_prefixes)) >= in0->maxkeys) {
 			/* enough elements received */
 			out0->truncated = out.truncated;
 			stop = TRUE;
@@ -1392,17 +1458,19 @@ enum http_rc_e action_container_list (struct req_args_s *args) {
 	}
 
 	/* Init the listing options common to all the modes */
-	list_in.flag_headers = ~0;
-	list_in.flag_nodeleted = ~0;
+	list_in.flag_headers = 1;
+	list_in.flag_nodeleted = 1;
 	list_in.prefix = OPT("prefix");
 	list_in.marker_start = OPT("marker");
-	list_in.marker_end = OPT("marker_end");
+	list_in.marker_end = OPT("end_marker");
+	if (!list_in.marker_end)
+		list_in.marker_end = OPT("marker_end");  // backward compatibility
 	if (OPT("deleted"))
 		list_in.flag_nodeleted = 0;
 	if (OPT("all"))
-		list_in.flag_allversion = ~0;
+		list_in.flag_allversion = 1;
 	if (oio_str_parse_bool(OPT("properties"), FALSE))
-		list_in.flag_properties = ~0;
+		list_in.flag_properties = 1;
 	if (!err)
 		err = _max (args, &list_in.maxkeys);
 	if (!err) {
@@ -1447,10 +1515,7 @@ enum http_rc_e action_container_show (struct req_args_s *args) {
 
 	CLIENT_CTX(ctx,args,NAME_SRVTYPE_META2,1);
 
-	GByteArray* packer () {
-		return sqlx_pack_PROPGET (sqlx_name_mutable_to_const(&ctx.name));
-	}
-	err = gridd_request_replicated (&ctx, packer);
+	err = gridd_request_replicated (&ctx, sqlx_pack_PROPGET);
 	if (err) {
 		client_clean (&ctx);
 		return _reply_m2_error (args, err);
@@ -1473,7 +1538,7 @@ enum http_rc_e action_container_show (struct req_args_s *args) {
 	if (err) {
 		/* TODO(jfs): maybe a good place for an assert */
 		if (pairs) g_strfreev(pairs);
-		return _reply_system_error(args, err);
+		return _reply_common_error(args, err);
 	}
 	if (!pairs) {
 		pairs = g_malloc0(sizeof(void*));
@@ -1486,15 +1551,15 @@ enum http_rc_e action_container_show (struct req_args_s *args) {
 	_container_old_props_to_headers (args, sys);
 	g_free(sys);
 
-	GString *body = g_string_new("");
+	GString *body = g_string_sized_new(1024);
 
 	/* In the reply's body, we then store only the "user." related properties
 	 * without the implicit prefix. For the sake of uniformity, we store these
 	 * properties under a json sub-object named "properties" */
 	gchar **user = KV_extract_prefixed(pairs, "user.");
-	g_string_append(body, "{\"properties\":");
+	g_string_append_static(body, "{\"properties\":");
 	KV_encode_gstr2(body, user);
-	g_string_append(body, "}");
+	g_string_append_c(body, '}');
 	g_free(user);
 
 	g_strfreev(pairs);
@@ -1545,7 +1610,7 @@ enum http_rc_e action_container_raw_delete (struct req_args_s *args) {
 
 /* CONTENT action resource -------------------------------------------------- */
 
-static enum http_rc_e action_m2_content_beans (struct req_args_s *args,
+static enum http_rc_e action_m2_content_prepare (struct req_args_s *args,
 		struct json_object *jargs) {
 	struct json_object *jsize = NULL, *jpol = NULL;
 	json_object_object_get_ex(jargs, "size", &jsize);
@@ -1579,7 +1644,7 @@ retry:
 					err->code, err->message);
 			autocreate = FALSE;
 			g_clear_error (&err);
-			err = _m2_container_create_with_properties (args, NULL);
+			err = _m2_container_create_with_defaults (args);
 			if (!err)
 				goto retry;
 			if (err->code == CODE_CONTAINER_EXISTS
@@ -1592,14 +1657,7 @@ retry:
 
 	// Patch the chunk size to ease putting contents with unknown size.
 	if (!err) {
-
-		/* 'ns_chunk_size' should be read in a critical section ... but
-		   nevermind 'cos it won't change often and will always be allocated,
-		   so I (jfs) accept to just read without protection. We thus save a
-		   lot of mutex operations on this. */
-		gint64 chunk_size = ns_chunk_size;
-
-		chunk_size = MAX(chunk_size,1);
+		gint64 chunk_size = oio_ns_chunk_size;
 		for (GSList *l=beans; l ;l=l->next) {
 			if (l->data && (DESCR(l->data) == &descr_struct_CHUNKS)) {
 				struct bean_CHUNKS_s *bean = l->data;
@@ -1662,24 +1720,23 @@ static enum http_rc_e action_m2_content_touch (struct req_args_s *args,
 		struct json_object *jargs) {
 	(void) jargs;
 
-	if ((!oio_url_has_fq_container(args->url) &&
-		 !oio_url_has(args->url, OIOURL_HEXID)) ||
-			(!oio_url_has(args->url, OIOURL_PATH) &&
-			 !oio_url_has(args->url, OIOURL_CONTENTID)))
-		return _reply_format_error(args, BADREQ("Missing content path or ID"));
+	if (!oio_url_has_fq_container(args->url))
+		return _reply_format_error(args, BADREQ("container unspecified"));
+	if (!oio_url_has(args->url, OIOURL_PATH) &&
+			!oio_url_has(args->url, OIOURL_CONTENTID))
+		return _reply_format_error(args, BADREQ("missing content path of ID"));
 
 	PACKER_VOID(_pack) { return m2v2_remote_pack_TOUCHC (args->url); }
 	GError *err = _resolve_meta2 (args, _prefer_master(), _pack, NULL);
-	if (err && CODE_IS_NOTFOUND(err->code))
-		return _reply_forbidden_error (args, err);
 	return _reply_m2_error (args, err);
 }
 
 static enum http_rc_e action_m2_content_link (struct req_args_s *args,
 		struct json_object *jargs) {
+	if (!oio_url_has_fq_container(args->url))
+		return _reply_m2_error (args, BADREQ("no container identified"));
 	if (NULL != CONTENT())
 		return _reply_m2_error (args, BADREQ("No content allowed in the URL"));
-
 	if (!jargs || !json_object_is_type (jargs, json_type_object))
 		return _reply_m2_error (args, BADREQ("Expected: json object"));
 
@@ -1698,8 +1755,6 @@ static enum http_rc_e action_m2_content_link (struct req_args_s *args,
 
 	PACKER_VOID(_pack) { return m2v2_remote_pack_LINK (args->url); }
 	err = _resolve_meta2 (args, _prefer_master(), _pack, NULL);
-	if (err && CODE_IS_NOTFOUND(err->code))
-		return _reply_forbidden_error (args, err);
 	return _reply_m2_error (args, err);
 }
 
@@ -1708,8 +1763,6 @@ static enum http_rc_e action_m2_content_propset (struct req_args_s *args,
 	if (CONTENT())
 		return _reply_m2_error (args, BADREQ("Content. not allowed in the URL"));
 
-	// TODO manage the version of the content
-	gint64 version = 0;
 	GSList *beans = NULL;
 
 	if (jargs) {
@@ -1722,7 +1775,7 @@ static enum http_rc_e action_m2_content_propset (struct req_args_s *args,
 			PROPERTIES_set2_key (prop, *p);
 			PROPERTIES_set2_value (prop, (guint8*)*(p+1), strlen(*(p+1)));
 			PROPERTIES_set2_alias (prop, oio_url_get (args->url, OIOURL_PATH));
-			PROPERTIES_set_version (prop, version);
+			PROPERTIES_set_version (prop, args->version);
 			beans = g_slist_prepend (beans, prop);
 		}
 		g_strfreev(kv);
@@ -1735,8 +1788,6 @@ static enum http_rc_e action_m2_content_propset (struct req_args_s *args,
 	PACKER_VOID(_pack) { return m2v2_remote_pack_PROP_SET (args->url, flags, beans); }
 	GError *err = _resolve_meta2 (args, _prefer_master(), _pack, NULL);
 	_bean_cleanl2 (beans);
-	if (err && CODE_IS_NOTFOUND(err->code))
-		return _reply_forbidden_error (args, err);
 	return _reply_m2_error (args, err);
 }
 
@@ -1745,11 +1796,6 @@ static enum http_rc_e action_m2_content_propdel (struct req_args_s *args,
 	if (!json_object_is_type(jargs, json_type_array))
 		return _reply_format_error (args, BADREQ("Array argument expected"));
 
-	// TODO manage the version of the content
-	gint64 version = 0;
-	(void) version;
-
-	// build the payload
 	gchar **namev = NULL;
 	GError *err = STRV_decode_object(jargs, &namev);
 	EXTRA_ASSERT((err != NULL) ^ (namev != NULL));
@@ -1759,16 +1805,12 @@ static enum http_rc_e action_m2_content_propdel (struct req_args_s *args,
 	PACKER_VOID(_pack) { return m2v2_remote_pack_PROP_DEL (args->url, namev); }
 	err = _resolve_meta2 (args, _prefer_master(), _pack, NULL);
 	g_strfreev(namev);
-	if (err && CODE_IS_NOTFOUND(err->code))
-		return _reply_forbidden_error (args, err);
 	return _reply_m2_error (args, err);
 }
 
 static enum http_rc_e action_m2_content_propget (struct req_args_s *args,
 		struct json_object *jargs UNUSED) {
-	/* TODO manage the version of the content */
-
-	guint32 flags = flag_force_master ? M2V2_FLAG_MASTER : 0;
+	const guint32 flags = flag_force_master ? M2V2_FLAG_MASTER : 0;
 
 	GSList *beans = NULL;
 	PACKER_VOID(_pack) { return m2v2_remote_pack_PROP_GET (args->url, flags); }
@@ -1781,8 +1823,8 @@ static GError *_m2_json_put (struct req_args_s *args,
 	if (!jbody)
 		return BADREQ("Invalid JSON body");
 
-	gboolean append = _request_get_flag (args, "append");
-	gboolean force = _request_get_flag (args, "force");
+	const gboolean append = _request_get_flag (args, "append");
+	const gboolean force = _request_get_flag (args, "force");
 	GSList *ibeans = NULL, *obeans = NULL;
 	GError *err;
 
@@ -1822,7 +1864,7 @@ retry:
 			GRID_DEBUG("Resource not found, autocreation");
 			autocreate = FALSE;
 			g_clear_error (&err);
-			err = _m2_container_create_with_properties (args, NULL);
+			err = _m2_container_create_with_defaults (args);
 			if (!err)
 				goto retry;
 			if (err->code == CODE_CONTAINER_EXISTS
@@ -1879,12 +1921,16 @@ enum http_rc_e action_content_truncate(struct req_args_s *args) {
 }
 
 enum http_rc_e action_content_prepare (struct req_args_s *args) {
-	return rest_action (args, action_m2_content_beans);
+	return rest_action (args, action_m2_content_prepare);
 }
 
 enum http_rc_e action_content_show (struct req_args_s *args) {
 	GSList *beans = NULL;
 	guint32 flags = flag_force_master ? M2V2_FLAG_MASTER : 0;
+	/* Historical behaviour is to return properties as headers, but
+	 * if there are too many, the client will fail decoding the request. */
+	if (!oio_str_parse_bool(OPT("properties"), TRUE))
+		flags |= M2V2_FLAG_NOPROPS;
 	PACKER_VOID(_pack) { return m2v2_remote_pack_GET (args->url, flags); }
 	GError *err = _resolve_meta2 (args, _prefer_slave(), _pack, &beans);
 	return _reply_simplified_beans (args, err, beans, TRUE);
@@ -1898,7 +1944,6 @@ enum http_rc_e action_content_delete (struct req_args_s *args) {
 
 static enum http_rc_e
 _m2_content_delete_many (struct req_args_s *args, struct json_object * jbody) {
-	guint max_requests = OIO_CONTENT_DELETE_MANY_NB_MAX_REQ;
 	json_object *jarray = NULL;
 	PACKER_VOID(_pack) { return m2v2_remote_pack_DEL (args->url); }
 
@@ -1916,7 +1961,7 @@ _m2_content_delete_many (struct req_args_s *args, struct json_object * jbody) {
 		return _reply_format_error(args,
 				BADREQ("At least one element is needed"));
 
-	if (jarray_len > max_requests)
+	if (jarray_len > proxy_bulk_max_create_many)
 		return _reply_too_large(args, NEWERROR(HTTP_CODE_PAYLOAD_TO_LARGE,
 				"Payload Too Large"));
 
@@ -1978,20 +2023,31 @@ enum http_rc_e action_content_prop_del (struct req_args_s *args) {
 	return rest_action (args, action_m2_content_propdel);
 }
 
+enum http_rc_e action_content_drain(struct req_args_s *args) {
+	PACKER_VOID(_pack) {return m2v2_remote_pack_DRAIN(args->url);}
+	GError *err = _resolve_meta2(args, _prefer_master(), _pack, NULL);
+	return _reply_m2_error(args, err);
+}
+
 enum http_rc_e action_content_copy (struct req_args_s *args) {
 	const gchar *target = g_tree_lookup (args->rq->tree_headers, "destination");
+	const gchar *_err = NULL;
+
 	if (!target)
 		return _reply_format_error(args, BADREQ("Missing target header"));
 
 	struct oio_url_s *target_url = oio_url_init(target);
-	if (!target_url)
-		return _reply_format_error(args, BADREQ("Invalid URL in target header"));
+	if (!target_url || !oio_url_check(target_url, NULL, &_err))
+		return _reply_format_error(args, BADREQ("Invalid %s in target header", _err));
 
 	// Check the namespace and container match between both URLs
 	if (!oio_url_has(target_url, OIOURL_HEXID)
 			|| !oio_url_has(target_url, OIOURL_NS)
-			|| !oio_url_has(target_url, OIOURL_PATH)
-			|| !oio_url_has(args->url, OIOURL_HEXID)
+			|| !oio_url_has(target_url, OIOURL_PATH)) {
+		oio_url_pclean(&target_url);
+		return _reply_format_error(args, BADREQ("Invalid source URL"));
+	}
+	if (!oio_url_has(args->url, OIOURL_HEXID)
 			|| !oio_url_has(args->url, OIOURL_NS)
 			|| strcmp(oio_url_get(target_url, OIOURL_NS), oio_url_get(args->url, OIOURL_NS))
 			|| strcmp(oio_url_get(target_url, OIOURL_HEXID), oio_url_get(args->url, OIOURL_HEXID))) {
@@ -2004,7 +2060,5 @@ enum http_rc_e action_content_copy (struct req_args_s *args) {
 	}
 	GError *err = _resolve_meta2 (args, _prefer_master(), _pack, NULL);
 	oio_url_pclean(&target_url);
-	if (err && CODE_IS_NOTFOUND(err->code))
-		return _reply_forbidden_error (args, err);
 	return _reply_m2_error (args, err);
 }
