@@ -1,7 +1,7 @@
 /*
 OpenIO SDS proxy
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -31,34 +31,11 @@ static GThread *admin_thread = NULL;
 static GThread *upstream_thread = NULL;
 static GThread *downstream_thread = NULL;
 
-guint csurl_refresh_delay = PROXYD_PERIOD_RELOAD_CSURL;
-guint srvtypes_refresh_delay = PROXYD_PERIOD_RELOAD_SRVTYPES;
-guint nsinfo_refresh_delay = PROXYD_PERIOD_RELOAD_NSINFO;
-time_t nsinfo_refresh_m0 = PROXYD_PERIOD_RELOAD_M0INFO;
-
-gint64 ttl_expire_local_services = PROXYD_TTL_DEAD_LOCAL_SERVICES;
-gint64 ttl_down_services = PROXYD_TTL_DOWN_SERVICES;
-gint64 ttl_known_services = PROXYD_TTL_KNOWN_SERVICES;
-gint64 ttl_expire_master_services = PROXYD_TTL_MASTER_SERVICES;
-
-// Configuration
-
+static gboolean config_system = TRUE;
+static GSList *config_paths = NULL;
 static GSList *config_urlv = NULL;
 
-static guint lb_downstream_delay = PROXYD_DEFAULT_PERIOD_DOWNSTREAM;
-static guint lb_upstream_delay = PROXYD_DEFAULT_PERIOD_UPSTREAM;
-static guint dir_low_ttl = PROXYD_DEFAULT_TTL_SERVICES / G_TIME_SPAN_SECOND;
-static guint dir_low_max = PROXYD_DEFAULT_MAX_SERVICES;
-static guint dir_high_ttl = PROXYD_DEFAULT_TTL_CSM0 / G_TIME_SPAN_SECOND;
-static guint dir_high_max = PROXYD_DEFAULT_MAX_CSM0;
-
 gchar *ns_name = NULL;
-gboolean flag_cache_enabled = TRUE;
-gboolean flag_local_scores = FALSE;
-gboolean flag_prefer_master_for_write = TRUE;
-gboolean flag_prefer_master_for_read = FALSE;
-gboolean flag_prefer_slave_for_read = FALSE;
-gboolean flag_force_master = FALSE;
 
 struct oio_lb_world_s *lb_world = NULL;
 struct oio_lb_s *lb = NULL;
@@ -77,7 +54,6 @@ struct lru_tree_s *srv_registered = NULL;
 GRWLock nsinfo_rwlock = {0};
 struct namespace_info_s nsinfo = {{0}};
 gchar **srvtypes = NULL;
-gint64 ns_chunk_size = 10*1024*1024;
 
 GRWLock master_rwlock = {0};
 struct lru_tree_s *srv_master = NULL;
@@ -98,7 +74,7 @@ action_status(struct req_args_s *args)
 	if (0 == strcasecmp("HEAD", args->rq->cmd))
 		return _reply_success_json(args, NULL);
 	if (0 != strcasecmp("GET", args->rq->cmd))
-		return _reply_method_error(args);
+		return _reply_method_error(args, NULL, "GET, HEAD");
 
 	GString *gstr = g_string_sized_new (128);
 
@@ -112,7 +88,7 @@ action_status(struct req_args_s *args)
 	g_array_free (array, TRUE);
 
 	/* some stats about the internal cache */
-	struct hc_resolver_stats_s s = {0};
+	struct hc_resolver_stats_s s = {{0}};
 	hc_resolver_info(resolver, &s);
 
 	g_string_append_printf(gstr, "gauge cache.dir.count = %"G_GINT64_FORMAT"\n", s.csm0.count);
@@ -139,7 +115,7 @@ static struct path_matching_s **
 _metacd_match (const gchar *method, const gchar *path)
 {
 	gsize lp = strlen(path), lm = strlen(method);
-	if (lp > PROXYD_PATH_MAXLEN || lm > 64)
+	if (lp > proxy_url_path_maxlen || lm > 64)
 		return g_malloc0(sizeof(struct path_matching_s*));
 
 	gchar *key = g_alloca (lp + 2 + lm + 1);
@@ -177,11 +153,10 @@ _metacd_match (const gchar *method, const gchar *path)
 	return result;
 }
 
-static struct oio_url_s *
-_metacd_load_url (struct req_args_s *args)
+static gboolean
+_metacd_load_url (struct req_args_s *args, struct oio_url_s *url)
 {
 	const gchar *s;
-	struct oio_url_s *url = oio_url_empty();
 
 	if (NULL != (s = NS()))
 		oio_url_set (url, OIOURL_NS, s);
@@ -201,13 +176,29 @@ _metacd_load_url (struct req_args_s *args)
 			oio_url_set (url, OIOURL_VERSION, s);
 	}
 
-	if (NULL != (s = CID()))
-		oio_url_set (url, OIOURL_HEXID, s);
+	if ((s = CID())) {
+		/* Tolerate the base name (with a dot and sequence number)
+		 * passed as a container ID. Also tolerate an hexadecimal prefx. */
+		char cid[STRLEN_CONTAINERID] =
+			"0000000000000000000000000000000000000000000000000000000000000000";
+		const size_t len = strlen(s);
+		if (len >= STRLEN_CONTAINERID && s[STRLEN_CONTAINERID-1] == '.') {
+			g_strlcpy(cid, s, sizeof(cid));
+			oio_url_set(url, OIOURL_HEXID, cid);
+		} else if (0 < len && len <= 4) {
+			memcpy(cid, s, len);
+			oio_url_set(url, OIOURL_HEXID, cid);
+		} else {
+			oio_url_set(url, OIOURL_HEXID, s);
+		}
+		if (!oio_url_get(url, OIOURL_HEXID))
+			return FALSE;
+	}
 
 	if (NULL != (s = CONTENT()))
 		oio_url_set (url, OIOURL_CONTENTID, s);
 
-	return url;
+	return TRUE;
 }
 
 static enum http_rc_e
@@ -245,26 +236,38 @@ handler_action (struct http_request_s *rq, struct http_reply_ctx_s *rp)
 		rp->finalize ();
 		rc = HTTPRC_DONE;
 	} else {
+		const char *err;
 		struct req_args_s args = {0};
 		args.req_uri = &ruri;
 		args.matchings = matchings;
 		args.rq = rq;
 		args.rp = rp;
+		args.url = url = oio_url_empty();
 
-		args.url = url = _metacd_load_url (&args);
-		rp->subject(oio_url_get(url, OIOURL_HEXID));
-		gq_count = (*matchings)->last->gq_count;
-		gq_time = (*matchings)->last->gq_time;
-		GRID_TRACE("%s %s URL %s", __FUNCTION__, ruri.path, oio_url_get(args.url, OIOURL_WHOLE));
-		req_handler_f handler = (*matchings)->last->u;
-		rc = (*handler) (&args);
+		if (!_metacd_load_url (&args, url)) {
+			rc = _reply_format_error(&args, BADREQ("Invalid oio url"));
+		} else {
+			rp->subject(oio_url_get(url, OIOURL_HEXID));
+			gq_count = (*matchings)->last->gq_count;
+			gq_time = (*matchings)->last->gq_time;
+
+			GRID_TRACE("%s %s URL %s", __FUNCTION__,
+					ruri.path, oio_url_get(args.url, OIOURL_WHOLE));
+
+			if (!oio_url_check(url, ns_name, &err)) {
+				rc = _reply_format_error(&args, BADREQ("Invalid parameter %s", err));
+			} else {
+				req_handler_f handler = (*matchings)->last->u;
+				rc = (*handler) (&args);
+			}
+		}
 	}
 
-	gint64 spent = oio_ext_monotonic_time () - rq->client->time.evt_in;
+	const gint64 spent = oio_ext_monotonic_time () - rq->client->time.evt_in;
 
 	network_server_stat_push4 (rq->client->server, TRUE,
 			gq_count, 1, gq_count_all, 1,
-			gq_time, spent, gq_time_all, spent);
+			gq_time, (guint64)spent, gq_time_all, (guint64)spent);
 
 	path_matching_cleanv (matchings);
 	oio_requri_clear (&ruri);
@@ -370,7 +373,8 @@ _NOLOCK_precache_list_of_services (const char *type, GBytes *encoded)
 static GBytes *
 _encode_wanted_services (const char *type, GSList *list)
 {
-	GString *encoded = g_string_new(type);
+	GString *encoded = g_string_sized_new(4096);
+	g_string_append (encoded, type);
 	g_string_append_c (encoded, '\0');
 	g_string_append_c (encoded, '[');
 	for (GSList *l=list; l ;l=l->next) {
@@ -386,7 +390,7 @@ static void
 _reload_srvtype(const char *type, GSList *list)
 {
 	/* reloads the known services */
-	gulong now = oio_ext_monotonic_seconds ();
+	time_t now = oio_ext_monotonic_seconds ();
 	SRV_WRITE(for (GSList *l=list; l ;l=l->next) {
 		gchar *k = service_info_key (l->data);
 		lru_tree_insert (srv_known, k, (void*)now);
@@ -413,11 +417,10 @@ _reload_srvtype(const char *type, GSList *list)
 static void
 _reload_lb_service_types(
 		struct oio_lb_world_s *lbw, struct oio_lb_s *lb_,
-		struct namespace_info_s *nsi,
 		gchar **tabtypes, GPtrArray *tabsrv, GPtrArray *taberr)
 {
 	struct service_update_policies_s *pols = service_update_policies_create();
-	gchar *pols_cfg = gridcluster_get_service_update_policy(nsi);
+	gchar *pols_cfg = oio_var_get_string(oio_ns_service_update_policy);
 	service_update_reconfigure(pols, pols_cfg);
 	g_free(pols_cfg);
 
@@ -452,6 +455,28 @@ _free_error(gpointer p)
 	g_error_free((GError*)p);
 }
 
+static GSList *
+_filter_good_services(GSList *src, GSList **out_garbage)
+{
+	GSList *good = NULL, *bad = NULL;
+
+	while (src) {
+		GSList *next = src->next;
+		struct service_info_s *si = src->data;
+		if (metautils_addr_valid_for_connect(&si->addr)) {
+			src->next = good;
+			good = src;
+		} else {
+			src->next = bad;
+			bad = src;
+		}
+		src = next;
+	}
+
+	*out_garbage = bad;
+	return good;
+}
+
 /* If you ever plan to factorize this code with the similar part in
  * sqlx/sqlx_service.c be carefull that a lot of context is expected on both
  * sides, and that even the function used to fetch the services cannot be the
@@ -486,6 +511,11 @@ lb_cache_reload (void)
 			GRID_WARN("Failed to load the list of [%s] in NS=%s", *pt, ns_name);
 			any_loading_error = TRUE;
 		}
+
+		GSList *bad = NULL;
+		srv = _filter_good_services(srv, &bad);
+		g_slist_free_full(bad, (GDestroyNotify)service_info_clean);
+
 		g_ptr_array_add(tabsrv, srv);
 		g_ptr_array_add(taberr, e);
 	}
@@ -494,7 +524,7 @@ lb_cache_reload (void)
 	if (!any_loading_error)
 		oio_lb_world__increment_generation(lb_world);
 	oio_lb_world__reload_pools(lb_world, lb, nsi);
-	_reload_lb_service_types(lb_world, lb, nsi, tabtypes, tabsrv, taberr);
+	_reload_lb_service_types(lb_world, lb, tabtypes, tabsrv, taberr);
 	oio_lb_world__reload_storage_policies(lb_world, lb, nsi);
 	if (!any_loading_error)
 		oio_lb_world__purge_old_generations(lb_world);
@@ -534,13 +564,12 @@ _task_reload_csurl (gpointer p UNUSED)
 		ADAPTIVE_PERIOD_ONSUCCESS(csurl_refresh_delay);
 		gchar **newcs = g_strsplit(s, ",", -1);
 		if (newcs) {
+			const guint newcs_count = g_strv_length(newcs);
 			g_rw_lock_writer_lock (&csurl_rwlock);
-			do {
-				gchar **tmp = csurl;
-				csurl = newcs;
-				newcs = tmp;
-				csurl_count = g_strv_length (csurl);
-			} while (0);
+			gchar **tmp = csurl;
+			csurl = newcs;
+			newcs = tmp;
+			csurl_count = newcs_count;
 			g_rw_lock_writer_unlock (&csurl_rwlock);
 			g_strfreev (newcs);
 		}
@@ -562,13 +591,14 @@ _task_reload_nsinfo (gpointer p UNUSED)
 	struct namespace_info_s *ni = NULL;
 	GError *err = conscience_remote_get_namespace (cs, &ni);
 	if (err) {
+		gchar *tmp = g_strjoinv("/", cs);
+		STRING_STACKIFY(tmp);
 		GRID_WARN ("NSINFO reload error [%s] from [%s]: (%d) %s",
-			ns_name, cs, err->code, err->message);
+			ns_name, tmp, err->code, err->message);
 		g_clear_error (&err);
 	} else {
 		ADAPTIVE_PERIOD_ONSUCCESS(nsinfo_refresh_delay);
-		NSINFO_WRITE(namespace_info_copy (ni, &nsinfo);
-					 ns_chunk_size = nsinfo.chunk_size);
+		NSINFO_WRITE(namespace_info_copy (ni, &nsinfo));
 		namespace_info_free (ni);
 	}
 }
@@ -590,8 +620,10 @@ _task_reload_srvtypes (gpointer p UNUSED)
 	EXTRA_ASSERT((err != NULL) ^ (types != NULL));
 
 	if (err != NULL) {
+		gchar *tmp = g_strjoinv("/", cs);
+		STRING_STACKIFY(tmp);
 		GRID_WARN ("SRVTYPES reload error [%s] from [%s] : (%d) %s",
-			ns_name, cs, err->code, err->message);
+			ns_name, tmp, err->code, err->message);
 		g_clear_error (&err);
 		return;
 	} else {
@@ -639,6 +671,37 @@ _task_push (gpointer p UNUSED)
 	lru_tree_destroy(lru);
 }
 
+static void
+_patch_configuration_fd(void)
+{
+	if (server_fd_max_passive <= 0) {
+		const guint maxfd = metautils_syscall_count_maxfd();
+
+		/* The purpose of the oio-proxy is ... to proxy connections to the
+		 * backend. It seems a good idea to reserve several connections for the
+		 * background tasks and the internals, then to reserve 2 FD for each
+		 * cnx. */
+		server_fd_max_passive = (maxfd -20) / 2;
+	}
+
+}
+
+static void
+_patch_and_apply_configuration(void)
+{
+	_patch_configuration_fd();
+	network_server_reconfigure(server);
+}
+
+static void
+_reconfigure_on_SIGHUP(void)
+{
+	GRID_NOTICE("SIGHUP! Reconfiguring...");
+	oio_var_reset_all();
+	oio_var_value_with_files(ns_name, config_system, config_paths);
+	_patch_and_apply_configuration();
+}
+
 // MAIN callbacks --------------------------------------------------------------
 
 static void
@@ -681,7 +744,7 @@ grid_main_action (void)
 		return;
 	}
 
-	if (NULL != (err = network_server_run (server))) {
+	if (NULL != (err = network_server_run (server, _reconfigure_on_SIGHUP))) {
 		_main_error (err);
 		return;
 	}
@@ -696,48 +759,11 @@ grid_main_get_options (void)
 			"An additional URL to bind to (might be used several time).\n"
 			"\t\tAccepts UNIX and INET sockets." },
 
-		{"LbRefresh", OT_INT, {.u = &lb_downstream_delay},
-			"Interval between load-balancer service refreshes (seconds)\n"},
+		{"SysConfig", OT_BOOL, {.b = &config_system},
+			"Load the system configuration and overload the central variables"},
 
-		{"RefreshNamespace", OT_TIME, {.u = &nsinfo_refresh_delay},
-			"Interval between NS configuration's refreshes (seconds)"},
-		{"RefreshConscience", OT_TIME, {.u = &csurl_refresh_delay},
-			"Interval between CS urls refreshes (seconds)"},
-		{"RefreshTypes", OT_TIME, {.u = &srvtypes_refresh_delay},
-			"Interval between service types refreshes (seconds)"},
-
-		{"SrvPush", OT_INT, {.u = &lb_upstream_delay},
-			"Interval between load-balancer service refreshes (seconds)\n"
-			"\t\tMust be >= 1"},
-
-		{"Cache", OT_BOOL, {.b = &flag_cache_enabled},
-			"Disable caching for the conscience: services are pushed\n"
-			"synchronously and no cache is kept on a GET."},
-
-		{"LocalScores", OT_BOOL, {.b = &flag_local_scores},
-			"Enables the updates of the list of local services, so that "
-			"their score is displayed. This gives a pretty output but "
-			"requires CPU spent in critical sections."},
-
-		{"DirLowTtl", OT_UINT, {.u = &dir_low_ttl},
-			"Directory 'low' (meta1) TTL for cache elements"},
-		{"DirLowMax", OT_UINT, {.u = &dir_low_max},
-			"Directory 'low' (meta1) MAX cached elements"},
-		{"DirHighTtl", OT_UINT, {.u = &dir_high_ttl},
-			"Directory 'high' (cs+meta0) TTL for cache elements"},
-		{"DirHighMax", OT_UINT, {.u = &dir_high_max},
-			"Directory 'high' (cs+meta0) MAX cached elements"},
-
-		{"PreferMaster", OT_BOOL, {.b = &flag_prefer_master_for_read},
-		        "Prefer to join the Master, even foor read-only"},
-		{"PreferSlave", OT_BOOL, {.b = &flag_prefer_slave_for_read},
-		        "Prefer to join a Slave, for read-only"},
-		{"PreferMasterForWrites", OT_BOOL, {.b = &flag_prefer_master_for_write},
-		        "Prefer to join a Master, for write operations"},
-		{"ForceMaster", OT_BOOL, {.b = &flag_force_master},
-		        "Force the service to redirect if not master "
-					"(even on read-only requests). Only effective on meta2 "
-					"requests."},
+		{"Config", OT_LIST, {.lst = &config_paths},
+			"Load the given file and overload the central variables"},
 
 		{NULL, 0, {.i = 0}, NULL}
 	};
@@ -834,12 +860,18 @@ grid_main_specific_fini (void)
 	g_rw_lock_clear(&wanted_rwlock);
 	g_rw_lock_clear(&master_rwlock);
 
-	if (csurl) g_strfreev (csurl);
+	if (csurl)
+		g_strfreev(csurl);
 	csurl = NULL;
 	csurl_count = 0;
 
 	g_slist_free_full (config_urlv, g_free);
 	config_urlv = NULL;
+
+	g_slist_free_full (config_paths, g_free);
+	config_paths = NULL;
+
+	oio_cfg_set_handle(NULL);
 }
 
 static void
@@ -848,8 +880,15 @@ configure_request_handlers (void)
 #define SET(Url,Cb) path_parser_configure (path_parser, PROXYD_PREFIX Url, Cb)
 
 	SET("/status/#GET", action_status);
+	SET("/config/#GET", action_get_config);
+	SET("/config/#POST", action_set_config);
 
+	SET("/forward/config/#POST", action_forward_set_config);
+	SET("/forward/config/#GET", action_forward_get_config);
+
+	SET("/forward/stats/#GET", action_forward_stats);
 	SET("/forward/stats/#POST", action_forward_stats);
+
 	SET("/forward/$ACTION/#POST", action_forward);
 
 	SET("/cache/status/#GET", action_cache_status);
@@ -867,6 +906,7 @@ configure_request_handlers (void)
 	SET("/$NS/lb/reload/#POST", action_lb_reload);
 	SET("/$NS/lb/choose/#GET", action_lb_choose);
 	SET("/$NS/lb/poll/#POST", action_lb_poll);
+	SET("/$NS/lb/create_pool/#POST", action_lb_create_pool);
 
 	// Local services
 	SET ("/$NS/local/list/#GET", action_local_list);
@@ -888,6 +928,7 @@ configure_request_handlers (void)
 	SET("/$NS/reference/set_properties/#POST", action_ref_prop_set);
 	SET("/$NS/reference/del_properties/#POST", action_ref_prop_del);
 	SET("/$NS/reference/link/#POST", action_ref_link);
+	SET("/$NS/reference/relink/#POST", action_ref_relink);
 	SET("/$NS/reference/unlink/#POST", action_ref_unlink);
 	SET("/$NS/reference/force/#POST", action_ref_force);
 	SET("/$NS/reference/renew/#POST", action_ref_renew);
@@ -913,6 +954,7 @@ configure_request_handlers (void)
 	// Content
 	SET("/$NS/content/create/#POST", action_content_put);
 	SET("/$NS/content/link/#POST", action_content_link);
+	SET("/$NS/content/drain/#POST", action_content_drain);
 	SET("/$NS/content/delete/#POST", action_content_delete);
 	SET("/$NS/content/delete_many/#POST", action_content_delete_many);
 	SET("/$NS/content/show/#GET", action_content_show);
@@ -966,28 +1008,36 @@ grid_main_configure (int argc, char **argv)
 	g_rw_lock_init (&wanted_rwlock);
 	g_rw_lock_init (&master_rwlock);
 
-	ns_name = g_strdup (cfg_namespace);
-	g_strlcpy (nsinfo.name, cfg_namespace, sizeof (nsinfo.name));
-	nsinfo.chunk_size = 1;
+	ns_name = g_strdup(cfg_namespace);
+	g_strlcpy(nsinfo.name, cfg_namespace, sizeof(nsinfo.name));
 
-	_task_reload_csurl (NULL);
+	/* Load the central configuration facility, it will tell us our
+	 * NS is locally known. */
+	if (!oio_var_value_with_files(ns_name, config_system, config_paths)) {
+		GRID_ERROR("NS [%s] unknown in the configuration", ns_name);
+		return FALSE;
+	}
+
+	/* Ensure we will cache the system config */
+	struct oio_cfg_handle_s *ns_conf = oio_cfg_cache_create();
+	oio_cfg_set_handle(ns_conf);
+
+	/* To work properly, the PROXY needs to know the URL of the conscience */
+	_task_reload_csurl(NULL);
 	if (!csurl || !csurl_count) {
 		GRID_ERROR("No conscience URL configured");
 		return FALSE;
 	}
 
+	_patch_and_apply_configuration();
+
+	/* init the networking capability of the processus. The server will use
+	 * the actual value of `server_fd_max_passive` that we (maybe) patched
+	 * earlier. */
+	server = network_server_init ();
 	path_parser = path_parser_init ();
 	configure_request_handlers ();
 
-	/* Make the proxy avoid meta services with known problems */
-	oio_cache_avoid_on_error = oio_cfg_get_bool (
-			ns_name, OIO_CFG_AVOID_BADSRV, FALSE);
-	if (oio_cache_avoid_on_error)
-		GRID_NOTICE("Faulty peers avoidance: ENABLED");
-
-	/* init the networking capability of the processus */
-	server = network_server_init ();
-
 	/* ensure each Route as a pair of count/time stats */
 	void _runner (const struct trie_node_s *n) {
 		network_server_stat_push2 (server, FALSE,
@@ -1004,19 +1054,13 @@ grid_main_configure (int argc, char **argv)
 	srv_known = lru_tree_create((GCompareFunc)g_strcmp0, g_free, NULL, LTO_NOATIME);
 	srv_master = lru_tree_create((GCompareFunc)g_strcmp0, g_free, g_free, LTO_NOATIME);
 
+	enum hc_resolver_flags_e f =
+		flag_cache_enabled ? HC_RESOLVER_DECACHEM0 : HC_RESOLVER_NOCACHE;
+
 	resolver = hc_resolver_create ();
-	enum hc_resolver_flags_e f = 0;
-	if (!flag_cache_enabled)
-		f |= HC_RESOLVER_NOCACHE;
 	hc_resolver_configure (resolver, f);
 	hc_resolver_qualify (resolver, service_is_ok);
 	hc_resolver_notify (resolver, service_invalidate);
-	hc_resolver_set_ttl_csm0 (resolver, dir_high_ttl * G_TIME_SPAN_SECOND);
-	hc_resolver_set_max_csm0 (resolver, dir_high_max);
-	hc_resolver_set_ttl_services (resolver, dir_low_ttl * G_TIME_SPAN_SECOND);
-	hc_resolver_set_max_services (resolver, dir_low_max);
-	GRID_INFO ("RESOLVER limits HIGH[%u/%u] LOW[%u/%u]",
-		dir_high_max, dir_high_ttl, dir_low_max, dir_low_ttl);
 
 	srv_registered = _push_queue_create ();
 
@@ -1034,7 +1078,7 @@ grid_main_configure (int argc, char **argv)
 	// Prepare a queue responsible for the downstream from the conscience
 	downstream_gtq = grid_task_queue_create ("downstream");
 
-	grid_task_queue_register (downstream_gtq, (guint) 1,
+	grid_task_queue_register (downstream_gtq, 1,
 		(GDestroyNotify) _task_reload_lb, NULL, NULL);
 
 	// Now prepare a queue for administrative tasks, such as cache expiration,
@@ -1066,6 +1110,10 @@ grid_main_configure (int argc, char **argv)
 		network_server_bind_host (server, lu->data, handler_action,
 				(network_transport_factory) transport_http_factory0);
 
+	/* Quick abstract of the meaningful options */
+	GRID_NOTICE("Faulty peers avoidance: %s", oio_client_cache_errors ? "ON" : "OFF");
+	GRID_NOTICE("TCP_FASTOPEN %s", oio_socket_fastopen ? "ON" : "OFF");
+
 	return TRUE;
 }
 
