@@ -1,7 +1,7 @@
 /*
 OpenIO SDS proxy
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -17,15 +17,18 @@ You should have received a copy of the GNU Affero General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+// For strcasestr
+#include <string.h>
+
 #include "common.h"
 
 void
 _append_status (GString *out, gint code, const char * msg)
 {
-	EXTRA_ASSERT (out != NULL);
-	oio_str_gstring_append_json_pair_int(out, "status", code);
-	g_string_append_c (out, ',');
-	oio_str_gstring_append_json_pair (out, "message", msg);
+	EXTRA_ASSERT(out != NULL);
+	OIO_JSON_append_int(out, "status", code);
+	g_string_append_c(out, ',');
+	OIO_JSON_append_str(out, "message", msg);
 }
 
 GString *
@@ -51,16 +54,6 @@ _create_status_error (GError * e)
 	return gstr;
 }
 
-enum http_rc_e
-_reply_common_error (struct req_args_s *args, GError *err)
-{
-	if (CODE_IS_NOTFOUND(err->code))
-		return _reply_notfound_error (args, err);
-	if (err->code == CODE_BAD_REQUEST)
-		return _reply_format_error (args, err);
-	return _reply_system_error (args, err);
-}
-
 static enum http_rc_e
 _reply_bytes (struct req_args_s *args, int code, const gchar * msg,
 		GBytes * bytes)
@@ -105,13 +98,6 @@ _reply_format_error (struct req_args_s *args, GError * err)
 			"Bad request", _create_status_error (err));
 }
 
-enum http_rc_e
-_reply_system_error (struct req_args_s *args, GError * err)
-{
-	return _reply_json_error (args, HTTP_CODE_INTERNAL_ERROR,
-			"Internal error", _create_status_error (err));
-}
-
 enum http_rc_e
 _reply_bad_gateway (struct req_args_s *args, GError * err)
 {
@@ -126,6 +112,46 @@ _reply_srv_unavailable (struct req_args_s *args, GError *err)
 			"Service unavailable", _create_status_error (err));
 }
 
+enum http_rc_e
+_reply_retry (struct req_args_s *args, GError *err)
+{
+	args->rp->add_header("Retry-After", g_strdup("1"));
+	return _reply_srv_unavailable (args, err);
+}
+
+static enum http_rc_e
+_reply_system_error (struct req_args_s *args, GError * err)
+{
+	return _reply_json (args, HTTP_CODE_INTERNAL_ERROR,
+			"Internal error", _create_status_error (err));
+}
+
+enum http_rc_e
+_reply_common_error (struct req_args_s *args, GError *err)
+{
+	if (CODE_IS_NOTFOUND(err->code))
+		return _reply_notfound_error (args, err);
+	switch (err->code) {
+		case CODE_BAD_REQUEST:
+			return _reply_format_error (args, err);
+		case CODE_TOOMANY_REDIRECT:
+		case CODE_UNAVAILABLE:
+		case CODE_EXCESSIVE_LOAD:
+		case CODE_CONTAINER_FROZEN:
+			return _reply_retry(args, err);
+		case CODE_CONTAINER_EXISTS:
+		case CODE_CONTENT_EXISTS:
+		case CODE_CONTENT_PRECONDITION:
+			return _reply_conflict_error(args, err);
+		case CODE_NOT_ALLOWED:
+			return _reply_forbidden_error(args, err);
+		case CODE_METHOD_NOTALLOWED:
+			return _reply_method_error(args, err, NULL);
+	}
+
+	return _reply_system_error (args, err);
+}
+
 enum http_rc_e
 _reply_gateway_timeout (struct req_args_s *args, GError * err)
 {
@@ -155,10 +181,17 @@ _reply_forbidden_error (struct req_args_s *args, GError * err)
 }
 
 enum http_rc_e
-_reply_method_error (struct req_args_s *args)
+_reply_method_error (struct req_args_s *args, GError *err, char *allowed)
 {
-	return _reply_json_error (args, HTTP_CODE_METHOD_NOT_ALLOWED,
-			"Method not allowed", NULL);
+	if (!allowed && err && strcasestr(err->message, "worm")) {
+		// Namespace in WORM mode, do not allow DELETE
+		allowed = "GET, HEAD, PUT";
+	}
+	// RFC 7231 requires the Allow header, but say it can be empty
+	EXTRA_ASSERT(allowed != NULL);
+	args->rp->add_header("Allow", g_strdup(allowed));
+	return _reply_json_error(args, HTTP_CODE_METHOD_NOT_ALLOWED,
+			"Method not allowed", _create_status_error(err));
 }
 
 enum http_rc_e
@@ -168,6 +201,13 @@ _reply_conflict_error (struct req_args_s *args, GError * err)
 			"Conflict", _create_status_error (err));
 }
 
+enum http_rc_e
+_reply_gone_error(struct req_args_s *args, GError *err)
+{
+	return _reply_json_error(args, HTTP_CODE_GONE,
+			"Gone", _create_status_error(err));
+}
+
 enum http_rc_e
 _reply_too_large (struct req_args_s *args, GError * err)
 {
