@@ -1,6 +1,6 @@
 /*
 OpenIO SDS proxy
-Copyright (C) 2015 OpenIO, original work as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -41,7 +41,8 @@ _sqlx_action_noreturn_TAIL (struct req_args_s *args, struct client_ctx_s *ctx,
 		return _reply_common_error (args, err);
 	}
 
-	GString *out = g_string_new ("{");
+	GString *out = g_string_sized_new (256);
+	g_string_append_c(out, '{');
 
 	/* Pack the output */
 	if (ctx->which == CLIENT_RUN_ALL && ctx->count) {
@@ -51,19 +52,19 @@ _sqlx_action_noreturn_TAIL (struct req_args_s *args, struct client_ctx_s *ctx,
 			g_string_append_printf (out, "\"%s\":{", ctx->urlv[i]);
 			if (ctx->errorv[i]) {
 				GError *e = ctx->errorv[i];
-				g_string_append (out, "\"status\":{");
+				g_string_append_static (out, "\"status\":{");
 				_append_status (out, e->code, e->message);
-				g_string_append (out, "}");
+				g_string_append_c (out, '}');
 			}
 			if (ctx->bodyv[i]) {
 				GByteArray *b = ctx->bodyv[i];
-				g_string_append_printf (out, ",\"body\":");
+				g_string_append_static (out, ",\"body\":");
 				if (b && b->data && b->len)
 					g_string_append_len (out, (const char*)(b->data), b->len);
 				else
-					g_string_append_len (out, "\"\"", 2);
+					g_string_append_static (out, "\"\"");
 			} else {
-				g_string_append_printf (out, ",\"body\":null");
+				g_string_append_static (out, ",\"body\":null");
 			}
 			g_string_append_c (out, '}');
 		}
@@ -86,6 +87,13 @@ _sqlx_action_noreturn (struct req_args_s *args, enum proxy_preference_e how,
 	gint64 seq = 1;
 
 	CLIENT_CTX (ctx, args, dirtype, seq);
+
+	if (*dirtype == '#' && !strcmp(dirtype+1, NAME_SRVTYPE_META1)) {
+		const guint nb_digits = MIN(oio_ns_meta1_digits, 4);
+		for (guint i=nb_digits; i<4 ;i++)
+			ctx.name.base[i] = '0';
+	}
+
 	ctx.which = how;
 	enum http_rc_e rc = _sqlx_action_noreturn_TAIL (args, &ctx, pack);
 	client_clean(&ctx);
@@ -99,9 +107,9 @@ _load_stringv (struct json_object *jargs)
 {
 	if (!json_object_is_type (jargs, json_type_array))
 		return NULL;
-	int max = json_object_array_length (jargs);
+	const guint max = json_object_array_length (jargs);
 	GPtrArray *tmp = g_ptr_array_sized_new(max);
-	for (int i=max; i>0 ;i--) {
+	for (guint i=max; i>0 ;i--) {
 		struct json_object *item = json_object_array_get_idx (jargs, i-1);
 		if (!json_object_is_type (item, json_type_string)) {
 			g_ptr_array_set_free_func (tmp, g_free0);
@@ -135,7 +143,9 @@ action_sqlx_copyto (struct req_args_s *args, struct json_object *jargs)
 
 	CLIENT_CTX (ctx, args, dirtype, seq);
 	ctx.which = CLIENT_PREFER_MASTER;
-	PACKER(_pack) { return sqlx_pack_PIPEFROM(n, to); }
+	GByteArray * _pack (const struct sqlx_name_s *n) {
+		return sqlx_pack_PIPEFROM(n, to);
+	}
 
 	enum http_rc_e rc = _sqlx_action_noreturn_TAIL (args, &ctx, _pack);
 	client_clean(&ctx);
@@ -145,21 +155,23 @@ action_sqlx_copyto (struct req_args_s *args, struct json_object *jargs)
 enum http_rc_e
 action_sqlx_propset (struct req_args_s *args, struct json_object *jargs)
 {
-	enum http_rc_e rc;
-	GError *err = NULL;
+	enum http_rc_e rc = HTTPRC_ABORT;
 
 	gchar **kv = NULL;
-	err = KV_read_usersys_properties(jargs, &kv);
+	GError *err = KV_read_usersys_properties(jargs, &kv);
 	if (!err && !*kv)
 		err = BADREQ("No properties found in JSON object");
 	if (!err) {
 		gboolean flush = _request_get_flag(args, "flush");
-		PACKER(_pack) { return sqlx_pack_PROPSET_tab(n, flush, kv); }
+		GByteArray * _pack (const struct sqlx_name_s *n) {
+			return sqlx_pack_PROPSET_tab(n, flush, kv);
+		}
 		rc = _sqlx_action_noreturn(args, CLIENT_PREFER_MASTER, _pack);
-		g_strfreev(kv);
-	}
-	if (err)
+	} else {
 		rc = _reply_common_error (args, err);
+	}
+	if (kv)
+		g_strfreev(kv);
 	return rc;
 }
 
@@ -179,8 +191,7 @@ action_sqlx_propget (struct req_args_s *args, struct json_object *jargs)
 	gint64 seq = 1;
 	CLIENT_CTX(ctx, args, dirtype, seq);
 
-	PACKER(_pack) { return sqlx_pack_PROPGET (n); }
-	err = gridd_request_replicated (&ctx, _pack);
+	err = gridd_request_replicated (&ctx, sqlx_pack_PROPGET);
 
 	if (err) {
 		client_clean (&ctx);
@@ -202,7 +213,7 @@ action_sqlx_propget (struct req_args_s *args, struct json_object *jargs)
 	if (err) {
 		/* TODO(jfs): maybe a good place for an assert */
 		if (pairs) g_strfreev(pairs);
-		return _reply_system_error(args, err);
+		return _reply_common_error(args, err);
 	}
 	if (!pairs) {
 		pairs = g_malloc0(sizeof(void*));
@@ -212,12 +223,12 @@ action_sqlx_propget (struct req_args_s *args, struct json_object *jargs)
 	gchar **user = KV_extract_prefixed(pairs, SQLX_ADMIN_PREFIX_USER);
 	gchar **nonuser = KV_extract_not_prefixed(pairs, SQLX_ADMIN_PREFIX_USER);
 
-	GString *out = g_string_new("");
-	g_string_append(out, "{\"properties\":");
+	GString *out = g_string_sized_new(512);
+	g_string_append_static(out, "{\"properties\":");
 	KV_encode_gstr2(out, user);
-	g_string_append(out, ",\"system\":");
+	g_string_append_static(out, ",\"system\":");
 	KV_encode_gstr2(out, nonuser);
-	g_string_append(out, "}");
+	g_string_append_c(out, '}');
 
 	g_free(user);
 	g_free(nonuser);
@@ -237,8 +248,10 @@ action_sqlx_propdel (struct req_args_s *args, struct json_object *jargs)
 	for (gchar **p = namev; namev && *p; p++)
 		oio_str_reuse(p, g_strconcat("user.", *p, NULL));
 
-	PACKER(packer) { return sqlx_pack_PROPDEL (n, (const gchar * const * )namev); }
-	enum http_rc_e rc = _sqlx_action_noreturn (args, CLIENT_PREFER_MASTER, packer);
+	GByteArray * _pack(const struct sqlx_name_s *n) {
+		return sqlx_pack_PROPDEL (n, (const gchar * const * )namev);
+	}
+	enum http_rc_e rc = _sqlx_action_noreturn (args, CLIENT_PREFER_MASTER, _pack);
 	g_strfreev (namev);
 	return rc;
 }
@@ -272,22 +285,19 @@ action_admin_drop_cache (struct req_args_s *args)
 enum http_rc_e
 action_admin_sync (struct req_args_s *args)
 {
-	PACKER(_pack) { return sqlx_pack_RESYNC (n); }
-	return _sqlx_action_noreturn (args, CLIENT_RUN_ALL, _pack);
+	return _sqlx_action_noreturn (args, CLIENT_RUN_ALL, sqlx_pack_RESYNC);
 }
 
 enum http_rc_e
 action_admin_leave (struct req_args_s *args)
 {
-	PACKER(_pack) { return sqlx_pack_EXITELECTION (n); }
-	return _sqlx_action_noreturn (args, CLIENT_RUN_ALL, _pack);
+	return _sqlx_action_noreturn (args, CLIENT_RUN_ALL, sqlx_pack_EXITELECTION);
 }
 
 enum http_rc_e
 action_admin_debug (struct req_args_s *args)
 {
-	PACKER(_pack) { return sqlx_pack_DESCR (n); }
-	return _sqlx_action_noreturn (args, CLIENT_RUN_ALL, _pack);
+	return _sqlx_action_noreturn (args, CLIENT_RUN_ALL, sqlx_pack_DESCR);
 }
 
 enum http_rc_e
