@@ -1,7 +1,7 @@
 /*
 OpenIO SDS proxy
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -17,9 +17,7 @@ You should have received a copy of the GNU Affero General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <stddef.h>
 #include <string.h>
-#include <sys/stat.h>
 
 #include <metautils/lib/metautils.h>
 #include <server/slab.h>
@@ -275,7 +273,7 @@ http_request_create(struct network_client_s *client)
 	req = g_malloc0(sizeof(*req));
 	req->client = client;
 	req->tree_headers = g_tree_new_full(metautils_strcmp3, NULL, g_free, g_free);
-	req->body = g_byte_array_new();
+	req->body = g_byte_array_sized_new(512);
 	return req;
 }
 
@@ -337,16 +335,16 @@ static gboolean
 sender(gpointer k, gpointer v, gpointer u)
 {
 	g_string_append((GString*)u, (gchar*)k);
-	g_string_append((GString*)u, ": ");
+	g_string_append_static((GString*)u, ": ");
 	g_string_append((GString*)u, (gchar*)v);
-	g_string_append((GString*)u, "\r\n");
+	g_string_append_static((GString*)u, "\r\n");
 	return FALSE;
 }
 
 static void
 _access_log(struct req_ctx_s *r, gint status, gsize out_len, const gchar *tail)
 {
-	if (r->access_disabled && 2 == (status / 100))
+	if (r->access_disabled && 2 == (status / 100) && !GRID_DEBUG_ENABLED())
 		return;
 
 	const char *reqid = g_tree_lookup(r->request->tree_headers, PROXYD_HEADER_REQID);
@@ -447,17 +445,16 @@ http_manage_request(struct req_ctx_s *r)
 
 		// Set the status line
 		g_string_append_printf(buf, "%s %d %s\r\n", r->request->version, code, msg);
-		g_string_append_printf(buf, "Server: oio-proxy/%s\r\n", OIOSDS_PROJECT_VERSION);
 
 		if (0 == g_ascii_strcasecmp("HTTP/1.1", r->request->version)) {
 			// Manage the "Connection" header of http/1.1
 			gchar *v = g_tree_lookup(r->request->tree_headers, "connection");
 			if (v && 0 == g_ascii_strcasecmp("Keep-Alive", v)) {
-				g_string_append(buf, "Connection: Keep-Alive\r\n");
+				g_string_append_static(buf, "Connection: Keep-Alive\r\n");
 				r->close_after_request = FALSE;
 			}
 			else {
-				g_string_append(buf, "Connection: Close\r\n");
+				g_string_append_static(buf, "Connection: Close\r\n");
 				r->close_after_request = TRUE;
 			}
 		}
@@ -466,9 +463,12 @@ http_manage_request(struct req_ctx_s *r)
 
 		// Add body-related headers
 		if (body_len) {
-			if (content_type)
-				g_string_append_printf(buf, "Content-Type: %s\r\n", content_type);
-			g_string_append(buf, "Transfer-Encoding: identity\r\n");
+			if (content_type) {
+				g_string_append_static(buf, "Content-Type: ");
+				/* TODO url-encode the header */
+				g_string_append(buf, content_type);
+				g_string_append_static(buf, "\r\n");
+			}
 		}
 		g_string_append_printf(buf, "Content-Length: %"G_GSIZE_FORMAT"\r\n", body_len);
 
@@ -476,7 +476,7 @@ http_manage_request(struct req_ctx_s *r)
 		g_tree_foreach(headers, sender, buf);
 
 		// Finalize and send the headers
-		g_string_append(buf, "\r\n");
+		g_string_append_static(buf, "\r\n");
 		network_client_send_slab(r->client, data_slab_make_gstr(buf));
 
 		// Now send the body
@@ -568,7 +568,7 @@ http_notify_input(struct network_client_s *clt)
 		g_tree_replace(r.request->tree_headers, g_strdup(k), g_strdup(v));
 	}
 	void body_provider(const guint8 *data, gsize data_len) {
-		g_byte_array_append(r.request->body, data, data_len);
+		g_byte_array_append(r.request->body, data, (guint)data_len);
 	}
 
 	r.close_after_request = TRUE;
@@ -656,4 +656,3 @@ http_notify_input(struct network_client_s *clt)
 	oio_str_clean (&r.uid);
 	return clt->transport.waiting_for_close ? RC_NODATA : RC_PROCESSED;
 }
-
