@@ -1,7 +1,7 @@
 /*
 OpenIO SDS rawx-apache2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -32,9 +32,10 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <mod_dav.h>
 
 #include <core/oio_sds.h>
+#include <core/oiovar.h>
 #include <metautils/lib/metautils.h>
-#include <cluster/lib/gridcluster.h>
 #include <rawx-lib/src/rawx.h>
+#include <rawx-apache2/src/rawx_variables.h>
 
 #include "mod_dav_rawx.h"
 #include "rawx_internals.h"
@@ -54,9 +55,6 @@ _cleanup_master(dav_rawx_server_conf *conf)
 	server_master_stat_fini(conf, conf->pool);
 }
 
-/**
- * Invoked whatever the context
- */
 static apr_status_t
 _cleanup_to_register(void *udata)
 {
@@ -71,14 +69,9 @@ _cleanup_to_register(void *udata)
 static unsigned int i = 0;
 
 static void *
-dav_rawx_create_server_config(apr_pool_t *p, server_rec *s)
+dav_rawx_create_server_config(apr_pool_t *p, server_rec *s UNUSED)
 {
-	dav_rawx_server_conf *conf = NULL;
-
-	(void) s;
-	DAV_XDEBUG_POOL(p, 0, "%s()", __FUNCTION__);
-
-	conf = apr_pcalloc(p, sizeof(dav_rawx_server_conf));
+	dav_rawx_server_conf *conf = apr_pcalloc(p, sizeof(dav_rawx_server_conf));
 	conf->pool = p;
 	conf->cleanup = NULL;
 	conf->hash_depth = 1;
@@ -86,22 +79,16 @@ dav_rawx_create_server_config(apr_pool_t *p, server_rec *s)
 	conf->fsync_on_close = FSYNC_ON_CHUNK_DIR;
 	conf->fallocate = 1;
 	conf->checksum_mode = CHECKSUM_ALWAYS;
-
 	return conf;
 }
 
 static void *
-dav_rawx_merge_server_config(apr_pool_t *p, void *base, void *overrides)
+dav_rawx_merge_server_config(apr_pool_t *p, void *base UNUSED, void *overrides)
 {
-	dav_rawx_server_conf *child;
-	dav_rawx_server_conf *newconf;
-
-	DAV_XDEBUG_POOL(p, 0, "%s()", __FUNCTION__);
-	(void) base;
-	child = overrides;
-
-	newconf = apr_pcalloc(p, sizeof(*newconf));
+	dav_rawx_server_conf *child = overrides;
+	dav_rawx_server_conf *newconf = apr_pcalloc(p, sizeof(*newconf));
 	newconf->pool = p;
+	newconf->enabled_compression = child->enabled_compression;
 	newconf->cleanup = NULL;
 	newconf->hash_depth = child->hash_depth;
 	newconf->hash_width = child->hash_width;
@@ -110,132 +97,66 @@ dav_rawx_merge_server_config(apr_pool_t *p, void *base, void *overrides)
 	newconf->checksum_mode = child->checksum_mode;
 	memcpy(newconf->docroot, child->docroot, sizeof(newconf->docroot));
 	memcpy(newconf->ns_name, child->ns_name, sizeof(newconf->ns_name));
-	update_rawx_conf(p, &(newconf->rawx_conf), newconf->ns_name);
-
-	DAV_DEBUG_POOL(p, 0, "Configuration merged!");
 	return newconf;
 }
 
 static const char *
-dav_rawx_cmd_gridconfig_hash_width(cmd_parms *cmd, void *config, const char *arg1)
+dav_rawx_cmd_gridconfig_hash_width(cmd_parms *cmd, void *config UNUSED, const char *arg1)
 {
-	dav_rawx_server_conf *conf;
-
-	(void) config;
-	DAV_XDEBUG_POOL(cmd->pool, 0, "%s()", __FUNCTION__);
-
-	conf = ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
+	dav_rawx_server_conf *conf =
+		ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
 	conf->hash_width = atoi(arg1);
-
-	DAV_DEBUG_POOL(cmd->pool, 0, "hash_width=[%d]", conf->hash_width);
 	return NULL;
 }
 
 static const char *
-dav_rawx_cmd_gridconfig_hash_depth(cmd_parms *cmd, void *config, const char *arg1)
+dav_rawx_cmd_gridconfig_hash_depth(cmd_parms *cmd, void *config UNUSED, const char *arg1)
 {
-	dav_rawx_server_conf *conf;
-
-	(void) config;
-	DAV_XDEBUG_POOL(cmd->pool, 0, "%s()", __FUNCTION__);
-
-	conf = ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
+	dav_rawx_server_conf *conf =
+		ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
 	conf->hash_depth = atoi(arg1);
-
-	DAV_DEBUG_POOL(cmd->pool, 0, "hash_depth=[%d]", conf->hash_depth);
 	return NULL;
 }
 
 static const char *
-dav_rawx_cmd_gridconfig_docroot(cmd_parms *cmd, void *config, const char *arg1)
+dav_rawx_cmd_gridconfig_docroot(cmd_parms *cmd, void *config UNUSED, const char *arg1)
 {
-	apr_finfo_t finfo;
-	dav_rawx_server_conf *conf;
-
-	(void) config;
-
 	/* Check the directory exists */
-	do {
-		apr_status_t status = apr_stat(&(finfo), arg1, APR_FINFO_NORM, cmd->pool);
-		if (status != APR_SUCCESS) {
-			DAV_DEBUG_POOL(cmd->temp_pool, 0,
-					"Invalid docroot for GridStorage chunks: %s", arg1);
-			return apr_pstrcat(cmd->temp_pool,
-					"Invalid docroot for GridStorage chunks: ", arg1, NULL);
-		}
-		if (finfo.filetype != APR_DIR) {
-			DAV_DEBUG_POOL(cmd->temp_pool, 0,
-					"Docroot for GridStorage chunks must be a directory: %s", arg1);
-			return apr_pstrcat(cmd->temp_pool,
-					"Docroot for GridStorage chunks must be a directory: ", arg1, NULL);
-		}
-	} while (0);
-
-	conf = ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
-	memset(conf->docroot, 0x00, sizeof(conf->docroot));
-	apr_cpystrn(conf->docroot, arg1, sizeof(conf->docroot)-1);
-
-	DAV_DEBUG_POOL(cmd->pool, 0, "DOCROOT=[%s]", conf->docroot);
-
-	return NULL;
-}
-
-static apr_status_t
-_cleanup (void *p)
-{
-	rawx_conf_t *conf = p;
-	if (NULL != conf->ni) {
-		namespace_info_free (conf->ni);
-		conf->ni = NULL;
-	}
-	return APR_SUCCESS;
-}
-
-static const char *
-dav_rawx_cmd_gridconfig_namespace(cmd_parms *cmd, void *config, const char *arg1)
-{
-	dav_rawx_server_conf *conf;
-	(void) config;
-
-	DAV_XDEBUG_POOL(cmd->pool, 0, "%s()", __FUNCTION__);
-
-	conf = ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
-	memset(conf->ns_name, 0x00, sizeof(conf->ns_name));
-	apr_cpystrn(conf->ns_name, arg1, sizeof(conf->ns_name)-1);
-
-	DAV_DEBUG_POOL(cmd->pool, 0, "NS=[%s]", conf->ns_name);
-
-	/* Prepare COMPRESSION / ACL CONF when we get ns name */
-	namespace_info_t* ns_info;
-	GError *local_error = conscience_get_namespace(conf->ns_name, &ns_info);
-	if (!ns_info) {
+	apr_finfo_t finfo;
+	apr_status_t status = apr_stat(&(finfo), arg1, APR_FINFO_NORM, cmd->pool);
+	if (status != APR_SUCCESS && status != APR_INCOMPLETE) {
 		DAV_DEBUG_POOL(cmd->temp_pool, 0,
-				"Failed to get namespace info from ns [%s]", conf->ns_name);
-		return apr_pstrcat(cmd->temp_pool, "Failed to get namespace info from ns: ",
-				conf->ns_name, NULL);
+				"Invalid docroot for GridStorage chunks: %s", arg1);
+		return apr_pstrcat(cmd->temp_pool,
+				"Invalid docroot for GridStorage chunks: ", arg1, NULL);
+	}
+	if (finfo.filetype != APR_DIR) {
+		DAV_DEBUG_POOL(cmd->temp_pool, 0,
+				"Docroot for GridStorage chunks must be a directory: %s", arg1);
+		return apr_pstrcat(cmd->temp_pool,
+				"Docroot for GridStorage chunks must be a directory: ", arg1, NULL);
 	}
 
-	conf->rawx_conf = apr_palloc(cmd->pool, sizeof(rawx_conf_t));
-	apr_pool_cleanup_register (cmd->pool, conf->rawx_conf, _cleanup, _cleanup);
-
-	conf->rawx_conf->ni = ns_info;
-	conf->rawx_conf->acl = _get_acl(cmd->pool, ns_info);
-	conf->rawx_conf->last_update = oio_ext_monotonic_seconds();
-
-	if(local_error)
-		g_clear_error(&local_error);
+	dav_rawx_server_conf *conf =
+		ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
+	g_strlcpy(conf->docroot, arg1, sizeof(conf->docroot));
 	return NULL;
 }
 
 static const char *
-dav_rawx_cmd_gridconfig_fsync(cmd_parms *cmd, void *config, const char *arg1)
+dav_rawx_cmd_gridconfig_namespace(cmd_parms *cmd, void *config UNUSED, const char *arg1)
 {
-	dav_rawx_server_conf *conf;
-	(void) config;
+	dav_rawx_server_conf *conf =
+		ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
+	g_strlcpy(conf->ns_name, arg1, sizeof(conf->ns_name));
+	return NULL;
+}
 
-	DAV_XDEBUG_POOL(cmd->pool, 0, "%s()", __FUNCTION__);
-
-	conf = ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
+static const char *
+dav_rawx_cmd_gridconfig_fsync(cmd_parms *cmd, void *config UNUSED, const char *arg1)
+{
+	dav_rawx_server_conf *conf =
+		ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
 	if (oio_str_parse_bool(arg1, FALSE))
 		conf->fsync_on_close |= FSYNC_ON_CHUNK;
 	else
@@ -245,82 +166,48 @@ dav_rawx_cmd_gridconfig_fsync(cmd_parms *cmd, void *config, const char *arg1)
 }
 
 static const char *
-dav_rawx_cmd_gridconfig_fsync_dir(cmd_parms *cmd, void *config, const char *arg1)
+dav_rawx_cmd_gridconfig_fsync_dir(cmd_parms *cmd, void *config UNUSED, const char *arg1)
 {
-	dav_rawx_server_conf *conf;
-	(void) config;
-
-	DAV_XDEBUG_POOL(cmd->pool, 0, "%s()", __FUNCTION__);
-
-	conf = ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
+	dav_rawx_server_conf *conf =
+		ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
 	if (oio_str_parse_bool(arg1, FALSE))
 		conf->fsync_on_close |= FSYNC_ON_CHUNK_DIR;
 	else
 		conf->fsync_on_close &= ~FSYNC_ON_CHUNK_DIR;
-
 	return NULL;
 }
 
 static const char *
-dav_rawx_cmd_gridconfig_fallocate(cmd_parms *cmd, void *config, const char *arg1)
+dav_rawx_cmd_gridconfig_fallocate(cmd_parms *cmd, void *config UNUSED, const char *arg1)
 {
-	dav_rawx_server_conf *conf;
-	(void) config;
-
-	DAV_XDEBUG_POOL(cmd->pool, 0, "%s()", __FUNCTION__);
-
-	conf = ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
+	dav_rawx_server_conf *conf =
+		ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
 	conf->fallocate = oio_str_parse_bool(arg1, FALSE);
-
 	return NULL;
 }
 
 static const char *
-dav_rawx_cmd_gridconfig_dirrun(cmd_parms *cmd, void *config, const char *arg1)
+dav_rawx_cmd_gridconfig_dirrun(cmd_parms *cmd, void *config UNUSED, const char *arg1)
 {
-	dav_rawx_server_conf *conf;
-	(void) config;
-
-	DAV_XDEBUG_POOL(cmd->pool, 0, "%s()", __FUNCTION__);
-
-	conf = ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
-
+	dav_rawx_server_conf *conf =
+		ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
 	apr_snprintf(conf->shm.path, sizeof(conf->shm.path),
 		"%s/httpd-shm.%d", arg1, getpid());
-
-	DAV_DEBUG_POOL(cmd->pool, 0, "mutex_key=[%s]", conf->shm.path);
-	DAV_DEBUG_POOL(cmd->pool, 0, "shm_key=[%s]", conf->shm.path);
-
 	return NULL;
 }
 
 static const char *
-dav_rawx_cmd_gridconfig_acl(cmd_parms *cmd, void *config, const char *arg1)
+dav_rawx_cmd_gridconfig_acl(cmd_parms *cmd, void *config UNUSED, const char *arg1 UNUSED)
 {
-	dav_rawx_server_conf *conf;
-	(void) config;
-
-	DAV_XDEBUG_POOL(cmd->pool, 0, "%s()", __FUNCTION__);
-
-	conf = ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
-	conf->enabled_acl = 0;
-	conf->enabled_acl |= (0 == apr_strnatcasecmp(arg1,"on"));
-	conf->enabled_acl |= (0 == apr_strnatcasecmp(arg1,"true"));
-	conf->enabled_acl |= (0 == apr_strnatcasecmp(arg1,"yes"));
-	conf->enabled_acl |= (0 == apr_strnatcasecmp(arg1,"enabled"));
-
+	DAV_ERROR_POOL(cmd->pool, 0, "IGNORED OPTION: %s", "grid_acl");
 	return NULL;
 }
 
 static const char *
-dav_rawx_cmd_gridconfig_checksum(cmd_parms *cmd, void *config, const char *arg1)
+dav_rawx_cmd_gridconfig_checksum(cmd_parms *cmd, void *config UNUSED, const char *arg1)
 {
-	dav_rawx_server_conf *conf;
-	(void) config;
-
-	DAV_DEBUG_POOL(cmd->pool, 0, "%s(%s)", __FUNCTION__, arg1);
-
-	conf = ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
+	dav_rawx_server_conf *conf =
+		ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
 	if (!oio_str_is_set(arg1)) {
 		conf->checksum_mode = CHECKSUM_ALWAYS;
 	} else if (0 == apr_strnatcasecmp(arg1, "smart")) {
@@ -335,20 +222,11 @@ dav_rawx_cmd_gridconfig_checksum(cmd_parms *cmd, void *config, const char *arg1)
 }
 
 static const char *
-dav_rawx_cmd_gridconfig_compression(cmd_parms *cmd, void *config, const char *arg1)
+dav_rawx_cmd_gridconfig_compression(cmd_parms *cmd, void *config UNUSED, const char *arg1)
 {
-	dav_rawx_server_conf *conf;
-	(void) config;
-
-	DAV_XDEBUG_POOL(cmd->pool, 0, "%s()", __FUNCTION__);
-
-	conf = ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
-	if (apr_strnatcasecmp(arg1, "off") &&
-			apr_strnatcasecmp(arg1, "disabled") &&
-			apr_strnatcasecmp(arg1, "false") &&
-			apr_strnatcasecmp(arg1, "0"))
-		strncpy(conf->compression_algo, arg1, sizeof(conf->compression_algo));
-
+	dav_rawx_server_conf *conf =
+		ap_get_module_config(cmd->server->module_config, &dav_rawx_module);
+	conf->enabled_compression = oio_str_parse_bool(arg1, FALSE);
 	return NULL;
 }
 
@@ -368,7 +246,16 @@ rawx_hook_child_init(apr_pool_t *pchild, server_rec *s)
 
 	conf->cleanup = _cleanup_child;
 
-	if (oio_cfg_get_bool(conf->ns_name, OIO_CFG_RAWX_EVENTS, TRUE)) {
+	/* Load the system configuration in the central config system */
+	do {
+		struct oio_cfg_handle_s *ns_conf = oio_cfg_cache_create();
+		if (ns_conf) {
+			oio_var_value_all_with_config(ns_conf, conf->ns_name);
+			oio_cfg_handle_clean(ns_conf);
+		}
+	} while (0);
+
+	if (oio_rawx_events) {
 		gchar *event_agent_addr = oio_cfg_get_eventagent(conf->ns_name);
 		GError *err = rawx_event_init(event_agent_addr);
 		if (NULL != err) {
@@ -446,7 +333,7 @@ _create_shm_if_needed(char *shm_path, server_rec *server, apr_pool_t *plog)
 }
 
 static int
-rawx_hook_post_config(apr_pool_t *pconf, apr_pool_t *plog, apr_pool_t *ptemp,
+rawx_hook_post_config(apr_pool_t *pconf, apr_pool_t *plog, apr_pool_t *ptemp UNUSED,
 		server_rec *server)
 {
 	apr_status_t status;
@@ -456,9 +343,6 @@ rawx_hook_post_config(apr_pool_t *pconf, apr_pool_t *plog, apr_pool_t *ptemp,
 	GError *gerr;
 	int volume_validated = 0;
 
-	(void) ptemp;
-	DAV_XDEBUG_POOL(plog, 0, "%s(%lx)", __FUNCTION__, (long)server);
-
 	if (__rawx_is_first_call(server)) {
 		DAV_DEBUG_POOL(plog, 0, "First call detected");
 		return OK;
@@ -558,8 +442,8 @@ static const command_rec dav_rawx_cmds[] =
     AP_INIT_TAKE1("grid_fsync",       dav_rawx_cmd_gridconfig_fsync,       NULL, RSRC_CONF, "do fsync on file close"),
     AP_INIT_TAKE1("grid_fsync_dir",   dav_rawx_cmd_gridconfig_fsync_dir,   NULL, RSRC_CONF, "do fsync on chunk direcory after renaming .pending"),
     AP_INIT_TAKE1("grid_fallocate",   dav_rawx_cmd_gridconfig_fallocate,   NULL, RSRC_CONF, "call fallocate when receiving a chunk"),
-    AP_INIT_TAKE1("grid_acl",         dav_rawx_cmd_gridconfig_acl,         NULL, RSRC_CONF, "enabled acl"),
-    AP_INIT_TAKE1("grid_compression", dav_rawx_cmd_gridconfig_compression, NULL, RSRC_CONF, "enable compression ('zlib' or 'lzo')'"),
+    AP_INIT_TAKE1("grid_acl",         dav_rawx_cmd_gridconfig_acl,         NULL, RSRC_CONF, "enable acl (ignored)"),
+    AP_INIT_TAKE1("grid_compression", dav_rawx_cmd_gridconfig_compression, NULL, RSRC_CONF, "enable compression ('yes', 'no')'"),
     AP_INIT_TAKE1("grid_checksum",    dav_rawx_cmd_gridconfig_checksum,    NULL, RSRC_CONF, "enable checksuming the body of PUT ('yes', 'no', 'smart')'"),
     AP_INIT_TAKE1(NULL,  NULL,  NULL, RSRC_CONF, NULL)
 };
