@@ -1,7 +1,7 @@
 /*
 OpenIO SDS rawx-apache2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -160,8 +160,8 @@ __build_chunk_full_path(const dav_resource *resource, char **full_path)
 
 	p[0] = '/';
 
-	for (int i = 0; i < conf->hash_depth ; i++) {
-		for (int j = 0; j < conf->hash_width ; j++)
+	for (unsigned int i = 0; i < conf->hash_depth ; i++) {
+		for (unsigned int j = 0; j < conf->hash_width ; j++)
 			p[i_p++] = r->uri[i_uri++];
 		p[i_p++] = '/';
 	}
@@ -230,14 +230,18 @@ _load_in_place_chunk_info(const dav_resource *r, const char *path, struct chunk_
 	str_replace_by_pooled_str(p, &(chunk->chunk_position));
 	str_replace_by_pooled_str(p, &(chunk->chunk_hash));
 
-	if(!get_compression_info_in_attr(path, &ge, comp_opt)){
+	str_replace_by_pooled_str(p, &(chunk->oio_version));
+	str_replace_by_pooled_str(p, &(chunk->oio_full_path));
+
+
+	if (!get_compression_info_in_attr(path, &ge, comp_opt)){
 		if(NULL != ge) {
 			e = server_create_and_stat_error(conf, p, HTTP_CONFLICT,
 				0, apr_pstrcat(p, "Failed to get chunk compression attributes: ", ge->message, NULL));
 			g_clear_error(&ge);
 		} else {
 			e = server_create_and_stat_error(conf, p, HTTP_CONFLICT,
-			                                0, "Failed to get chunk compression attributes: No error specified");
+					0, "Failed to get chunk compression attributes: No error specified");
 		}
 		return e;
 	}
