@@ -1,7 +1,7 @@
 /*
 OpenIO SDS rawx-apache2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -111,13 +111,13 @@ struct dav_rawx_server_conf_s {
 	apr_pool_t *pool;
 	char docroot[1024];
 	char ns_name[LIMIT_LENGTH_NSNAME];
-	int hash_depth;
-	int hash_width;
-	int fsync_on_close;
-	int fallocate;
-	char event_agent_addr[RAWX_EVENT_ADDR_SIZE];
+	unsigned int hash_depth;
+	unsigned int hash_width;
+	unsigned int fsync_on_close;
+	unsigned int fallocate;
+	unsigned int enabled_compression;
 
-	char compression_algo[64];
+	char event_agent_addr[RAWX_EVENT_ADDR_SIZE];
 
 	enum rawx_checksum_mode_e checksum_mode;
 
@@ -128,9 +128,6 @@ struct dav_rawx_server_conf_s {
 	} shm;
 
 	void (*cleanup)(dav_rawx_server_conf *conf);
-
-	int enabled_acl;
-	rawx_conf_t* rawx_conf;
 };
 
 apr_status_t server_init_master_stat(dav_rawx_server_conf *conf, apr_pool_t *pool, apr_pool_t *plog);
@@ -155,12 +152,4 @@ apr_uint64_t server_get_reqperseq(dav_rawx_server_conf *conf);
 
 apr_uint64_t server_get_reqavgtime(dav_rawx_server_conf *conf);
 
-char * _get_compression_algorithm(apr_pool_t *p, namespace_info_t *ns_info);
-
-apr_int64_t _get_compression_block_size(apr_pool_t *p, namespace_info_t *ns_info);
-
-GSList * _get_acl(apr_pool_t *p, namespace_info_t *ns_info);
-
-gboolean update_rawx_conf(apr_pool_t* p, rawx_conf_t **rawx_conf, const gchar* ns_name);
-
 #endif /*OIO_SDS__rawx_apache2__src__rawx_config_h*/
