@@ -1,11 +1,27 @@
+/*
+OpenIO SDS rawx-apache2
+Copyright (C) 2016-2017 OpenIO SAS, as part of OpenIO SDS
+
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU Affero General Public License as
+published by the Free Software Foundation, either version 3 of the
+License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU Affero General Public License for more details.
+
+You should have received a copy of the GNU Affero General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 #include <stdlib.h>
 #include <assert.h>
 
 #include <glib.h>
 
 #include <events/oio_events_queue.h>
-#include <mod_dav.h>
-#include "mod_dav_rawx.h"
+#include <core/internals.h>
 #include "rawx_event.h"
 
 struct oio_events_queue_s *q = NULL;
@@ -76,10 +92,12 @@ rawx_event_destroy (void)
 }
 
 GError *
-rawx_event_send (const char *event_type, GString *data_json)
+rawx_event_send(const char *event_type, const char *request_id,
+		GString *data_json)
 {
 	if (q != NULL && th_queue != NULL) {
-		GString *json = oio_event__create (event_type, NULL);
+		GString *json = oio_event__create_with_id(
+				event_type, NULL, request_id);
 		g_string_append_printf(json, ",\"data\":%.*s}",
 				(int) data_json->len, data_json->str);
 		oio_events_queue__send (q, g_string_free (json, FALSE));
