@@ -1,7 +1,7 @@
 /*
 OpenIO SDS rawx-apache2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -147,16 +147,26 @@ send_chunk_event(const char *type, const dav_resource *resource)
 	_PAIR_AND_COMMA("chunk_position", resource->info->chunk.chunk_position);
 	_PAIR_AND_COMMA("chunk_hash", resource->info->chunk.chunk_hash);
 
-	g_string_append_printf(json, "}");
+	_PAIR_AND_COMMA("oio_version", resource->info->chunk.oio_version);
 
+	_PAIR_AND_COMMA("full_path", resource->info->chunk.oio_full_path);
+
+	g_string_append_c(json, '}');
+
+	const char *reqid = apr_table_get(
+			resource->info->request->headers_in, PROXYD_HEADER_REQID);
 	const gint64 pre = oio_ext_monotonic_time ();
-	GError *err = rawx_event_send(type, json);
+	GError *err = rawx_event_send(type, reqid, json);
 	const gint64 post = oio_ext_monotonic_time ();
 	if (!err) {
-		if (post - pre > 5 * G_TIME_SPAN_SECOND) {
+		gint64 limit = 5 * G_TIME_SPAN_SECOND;
+		if (post - pre > limit) {
 			DAV_ERROR_REQ(resource->info->request, 0,
-					"Event LONG %s t=%"G_GINT64_FORMAT"ms", type,
-					(post-pre)/G_TIME_SPAN_MILLISECOND);
+					"Sending event %s took %"G_GINT64_FORMAT
+					"ms (warning limit is %"G_GINT64_FORMAT"ms)",
+					type,
+					(post-pre) / G_TIME_SPAN_MILLISECOND,
+					limit / G_TIME_SPAN_MILLISECOND);
 		} else {
 			DAV_DEBUG_REQ(resource->info->request, 0, "Event OK %s", type);
 		}
