@@ -1,7 +1,7 @@
 /*
 OpenIO SDS rawx-apache2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -244,6 +244,16 @@ _load_field(apr_pool_t *pool, apr_table_t *table, const char *name, char **dst)
 	return 1;
 }
 
+static int
+_load_raw_field(apr_pool_t *pool, apr_table_t *table, const char *name, char ** dst)
+{
+	const char *value = apr_table_get(table, name);
+	if (!value)
+		return 0;
+	*dst = apr_pstrdup (pool, value);
+	return 1;
+}
+
 #define REPLACE_FIELD(S) str_replace_by_pooled_str(pool, &(cti->S))
 
 #define LAZY_LOAD_FIELD(Where,Name) do { \
@@ -279,6 +289,10 @@ chunk_info_fields__glib2apr (apr_pool_t *pool, struct chunk_textinfo_s *cti)
 	REPLACE_FIELD(chunk_size);
 	REPLACE_FIELD(chunk_position);
 	REPLACE_FIELD(chunk_hash);
+
+	REPLACE_FIELD(oio_version);
+
+	REPLACE_FIELD(oio_full_path);
 }
 
 void
@@ -316,6 +330,11 @@ request_load_chunk_info_from_headers(request_rec *request,
 	LAZY_LOAD_FIELD(chunk_size,             "chunk-size");
 	LAZY_LOAD_FIELD(chunk_position,         "chunk-pos");
 	LAZY_LOAD_FIELD(chunk_hash,             "chunk-hash");
+	LAZY_LOAD_FIELD(oio_version,            "oio-version");
+
+	if (!cti->oio_full_path) {
+		_load_raw_field(pool, src, RAWX_HEADER_PREFIX  "full-path", &(cti->oio_full_path));
+	}
 }
 
 const char *
@@ -391,13 +410,13 @@ resource_stat_chunk(dav_resource *resource, int flags)
 				APR_FINFO_NORM, resource->pool);
 	}
 
-	if (status != APR_SUCCESS)
+	if (status != APR_SUCCESS && status != APR_INCOMPLETE)
 		status = apr_stat(&(resource->info->finfo),
 				resource_get_pathname(resource),
 				APR_FINFO_NORM, resource->pool);
 
 	resource->collection = 0;
-	resource->exists = (status == APR_SUCCESS);
+	resource->exists = (status == APR_SUCCESS || status == APR_INCOMPLETE);
 
 	if (!resource->exists)
 		DAV_DEBUG_RES(resource, 0, "Resource does not exist [%s]", resource_get_pathname(resource));
@@ -424,44 +443,6 @@ resource_stat_chunk(dav_resource *resource, int flags)
 	}
 }
 
-void
-request_parse_query(request_rec *r, dav_resource *resource)
-{
-	if (!r->parsed_uri.query)
-		return;
-
-	char *query = NULL;
-	query = apr_pstrdup(r->pool, r->parsed_uri.query);
-
-	/* Expected comp=true&algo=XXXX&bs=XXXX */
-	char *k = NULL;
-	char *v = NULL;
-	char *last = NULL;
-
-	k = apr_strtok(query, "=&", &last);
-	v = apr_strtok(NULL, "=&", &last);
-
-	if (!k || !v)
-		goto end;
-
-	do {
-		if (!apr_strnatcasecmp(k, "comp"))
-			resource->info->forced_cp = apr_pstrdup(r->pool, v);
-		if (!apr_strnatcasecmp(k, "algo"))
-			resource->info->forced_cp_algo = apr_pstrdup(r->pool, v);
-		if (!apr_strnatcasecmp(k, "bs"))
-			resource->info->forced_cp_bs = apr_pstrdup(r->pool, v);
-	} while ((k = apr_strtok(NULL, "=&", &last)) &&
-			(v = apr_strtok(NULL, "=&", &last)));
-
-end:
-	DAV_DEBUG_REQ(r, 0, "forced_cp=%s, forced_cp_algo=%s, forced_cp_bs=%s",
-			resource->info->forced_cp, resource->info->forced_cp_algo,
-			resource->info->forced_cp_bs);
-	if (!resource->info->forced_cp)
-		resource->info->forced_cp = apr_pstrdup(r->pool, "false");
-}
-
 void
 request_fill_headers(request_rec *r, struct chunk_textinfo_s *c)
 {
@@ -489,6 +470,11 @@ request_fill_headers(request_rec *r, struct chunk_textinfo_s *c)
 	__set_header(r, "chunk-size", c->chunk_size);
 	__set_header(r, "chunk-hash", c->chunk_hash);
 	__set_header(r, "chunk-pos",  c->chunk_position);
+
+	__set_header(r, "oio-version", c->oio_version);
+
+	__set_header(r, "full-path", c->oio_full_path);
+
 }
 
 /*************************************************************************/
@@ -533,7 +519,7 @@ rawx_repo_check_request(request_rec *req, const char *root_dir, const char * lab
 dav_error *
 rawx_repo_configure_hash_dir(request_rec *req, dav_resource_private *ctx)
 {
-	int i_width, i_depth, i_src, i_dst;
+	unsigned int i_width, i_depth, i_src, i_dst;
 	int dst_maxlen;
 	const char *src;
 	char *dst;
@@ -647,8 +633,10 @@ rawx_repo_commit_upload(dav_stream *stream)
 	DUP(chunk_size);
 	DUP(chunk_hash);
 	DUP(chunk_position);
+	DUP(oio_version);
 	DUP(compression_metadata);
 	DUP(compression_size);
+	DUP(oio_full_path);
 
 	/* Load the metadata located in the Trailers of the request */
 	request_overload_chunk_info_from_trailers (stream->r->info->request, &fake);
@@ -750,11 +738,8 @@ rawx_repo_stream_create(const dav_resource *resource, dav_stream **result)
 	apr_pool_t *p = resource->info->pool;
 	dav_resource_private *ctx = resource->info;
 	dav_rawx_server_conf *conf = resource_get_server_config(resource);
-	char * metadata_compress = NULL;
 	int retryable = 1;
 
-	int should_compress = 0;
-
 	dav_stream *ds = apr_pcalloc(p, sizeof(*ds));
 	ds->fsync_on_close = conf->fsync_on_close;
 	ds->p = p;
@@ -813,57 +798,17 @@ retry:
 		return err;
 	}
 
-	if (conf->compression_algo && *conf->compression_algo) {
-		should_compress = TRUE;
-		ctx->forced_cp_algo = apr_pstrdup(p, conf->compression_algo);
-	} else if (ctx->forced_cp) {
-		should_compress = !g_ascii_strncasecmp(ctx->forced_cp, "true", 4);
-	}
-
-	if (!should_compress ||
-			!namespace_in_compression_mode(conf->rawx_conf->ni)) {
-		ds->buffer_size = DEFAULT_BLOCK_SIZE;
-		ds->buffer = apr_pcalloc(p, ds->buffer_size);
-		ds->buffer_offset = 0;
-	} else {
-		ds->compression = TRUE;
-		if (ctx->forced_cp_bs)
-			ds->buffer_size = strtol(ctx->forced_cp_bs, NULL, 10);
-		else
-			ds->buffer_size = DEFAULT_BLOCK_SIZE;
-
-		if (!ctx->forced_cp_algo)
-			ctx->forced_cp_algo = DEFAULT_COMPRESSION_ALGO;
-
-		metadata_compress = apr_pstrcat(p,
-				NS_COMPRESSION_OPTION, "=", NS_COMPRESSION_ON, ";",
-				NS_COMPRESS_ALGO_OPTION,"=", ctx->forced_cp_algo, ";",
-				NS_COMPRESS_BLOCKSIZE_OPTION, "=", ctx->forced_cp_bs, NULL);
-
-		DAV_DEBUG_REQ(resource->info->request, 0 , "%s", metadata_compress);
-		init_compression_ctx(&(ds->comp_ctx), ctx->forced_cp_algo);
-
-		ds->buffer = apr_pcalloc(p, ds->buffer_size);
-		ds->buffer_offset = 0;
-
-		gulong checksum = 0;
-
-		if(!(ds->comp_ctx.checksum_initiator)(&checksum)){
-			WARN("Failed to init compression checksum");
-		}
-
-		ds->metadata_compress = apr_pstrndup(p, metadata_compress, strlen(metadata_compress));
-
-		/* writting compression header in busy file */
-		guint32 bsize32 = ds->buffer_size;
-		if(0 != ds->comp_ctx.header_writer(ds->f, bsize32, &checksum, &(ds->compressed_size))){
-			return server_create_and_stat_error(resource_get_server_config(resource), p,
-				HTTP_INTERNAL_SERVER_ERROR, 0,
-				apr_pstrcat(p, "Failed to write compression headers", NULL));
-		}
-		ds->compress_checksum = checksum;
+	/* Trigger the compression of the chunk */
+	if (conf->enabled_compression) {
+		DAV_ERROR_REQ(resource->info->request, 0, "compression ignored on [%s]",
+				ds->pathname);
 	}
+	ds->compression = FALSE;
+	ds->buffer_size = DEFAULT_BLOCK_SIZE;
+	ds->buffer = apr_pcalloc(p, ds->buffer_size);
+	ds->buffer_offset = 0;
 
+	/* Trigger the checksum on the chunk */
 	ds->md5 = NULL;
 	if (conf->checksum_mode == CHECKSUM_ALWAYS) {
 		ds->md5 = g_checksum_new (G_CHECKSUM_MD5);
