@@ -1,7 +1,7 @@
 /*
 OpenIO SDS rawx-apache2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -150,19 +150,6 @@ dav_rawx_get_resource(request_rec *r, const char *root_dir, const char *label,
 
 	dav_rawx_server_conf *conf = request_get_server_config(r);
 
-	/* Check if client allowed to work with us */
-	if (conf->enabled_acl) {
-#if MODULE_MAGIC_COOKIE == 0x41503234UL /* "AP24" */
-		if (!authorized_personal_only(r->connection->client_ip, conf->rawx_conf->acl))
-#else
-		if (!authorized_personal_only(r->connection->remote_ip, conf->rawx_conf->acl))
-#endif
-		{
-			return server_create_and_stat_error(conf, r->pool,
-					HTTP_UNAUTHORIZED, 0, "Permission Denied (APO)");
-		}
-	}
-
 	/* Create private resource context descriptor */
 	dav_resource_private ctx = {0};
 	ctx.pool = r->pool;
@@ -202,21 +189,29 @@ dav_rawx_get_resource(request_rec *r, const char *root_dir, const char *label,
 		resource->info->file_extension,
 		NULL);
 
-	/* init compression context structure if we are in get method */
-	if (r->method_number == M_GET && !ctx.update_only) {
-		resource_init_decompression(resource, conf);
-	}
-
 	/* Check the chunk's existence */
-	int flags = (r->method_number == M_GET ||
-			r->method_number == M_OPTIONS ||
-			r->method_number == M_DELETE)?
-				 RESOURCE_STAT_CHUNK_READ_ATTRS : 0;
-	if (r->method_number == M_PUT || r->method_number == M_POST)
-		flags |= RESOURCE_STAT_CHUNK_PENDING;
-
+	int flags = 0;
+	switch (r->method_number) {
+		case M_GET:
+			if (oio_str_parse_bool(apr_table_get(r->headers_in, "X-oio-xattr"), TRUE))
+				flags |= RESOURCE_STAT_CHUNK_READ_ATTRS;
+			break;
+		case M_OPTIONS:
+		case M_DELETE:
+		case M_MOVE:
+			flags |= RESOURCE_STAT_CHUNK_READ_ATTRS;
+			break;
+		case M_PUT:
+		case M_POST:
+			flags |= RESOURCE_STAT_CHUNK_PENDING;
+			break;
+	}
 	resource_stat_chunk(resource, flags);
 
+	if (r->method_number == M_COPY) {
+		request_load_chunk_info_from_headers(r, &(resource->info->chunk));
+	}
+
 	if (r->method_number == M_PUT || r->method_number == M_POST ||
 			r->method_number == M_MOVE ||
 			(r->method_number == M_GET && ctx.update_only)) {
@@ -237,7 +232,6 @@ dav_rawx_get_resource(request_rec *r, const char *root_dir, const char *label,
 		if (resource->exists)
 			return server_create_and_stat_error(request_get_server_config(r), r->pool,
 				HTTP_CONFLICT, 0, "Resource busy or already exists");
-		request_parse_query(r, resource);
 	}
 
 	*result_resource = resource;
@@ -445,7 +439,6 @@ static dav_error *
 dav_rawx_seek_stream(dav_stream *stream, apr_off_t abs_pos)
 {
 	DAV_XDEBUG_POOL(stream->p, 0, "%s(%s)", __FUNCTION__, stream->pathname);
-	TRACE("Seek stream: START please contact CDR if you get this TRACE");
 
 	if (fseek(stream->f, abs_pos, SEEK_SET) != 0) {
 		/* ### should check whether apr_file_seek set abs_pos was set to the
@@ -483,9 +476,9 @@ dav_rawx_set_headers(request_rec *r, const dav_resource *resource)
 
 	/* compute metadata_compress if compressed content */
 	if (resource->info->compression) {
-		char *buf = apr_pstrcat(r->pool, "compression=on;compression_algorithm=", resource->info->compress_algo,
-				";compression_blocksize=", apr_psprintf(r->pool, "%d", resource->info->cp_chunk.block_size), ";", NULL);
-		apr_table_setn(r->headers_out, apr_pstrdup(r->pool, "metadatacompress"), buf);
+		apr_table_setn(r->headers_out,
+				apr_pstrdup(r->pool, "compression"),
+				apr_pstrdup(r->pool, "on"));
 	}
 
 	return NULL;
@@ -527,7 +520,7 @@ dav_rawx_deliver(const dav_resource *resource, ap_filter_t *output)
 		char *pending_file = apr_pstrcat(pool,
 				resource_get_pathname(resource), ".pending", NULL);
 		status = apr_stat(&info, pending_file, APR_FINFO_ATIME, pool);
-		if (status == APR_SUCCESS) {
+		if (status == APR_SUCCESS || status == APR_INCOMPLETE) {
 			e = server_create_and_stat_error(conf, pool, HTTP_FORBIDDEN,
 					0, "File in pending mode.");
 			goto end_deliver;
@@ -560,7 +553,7 @@ dav_rawx_deliver(const dav_resource *resource, ap_filter_t *output)
 					APR_FOPEN_BUFFERED|
 					APR_FOPEN_SENDFILE_ENABLED,
 					0, pool);
-			if (APR_SUCCESS != status) {
+			if (status != APR_SUCCESS) {
 				e = server_create_and_stat_error(conf, pool, HTTP_FORBIDDEN,
 						0, "File permissions deny server access.");
 				goto end_deliver;
@@ -632,6 +625,74 @@ end_deliver:
 	return e;
 }
 
+static dav_error *
+dav_rawx_copy_resource(const dav_resource *src, dav_resource *dst, int depth,
+		dav_response **response)
+{
+	(void) depth;
+	char buff[128];
+	apr_pool_t *pool;
+	pool = dst->pool;
+	apr_status_t status;
+	dav_error *e = NULL;
+	dav_rawx_server_conf * srv_conf = resource_get_server_config(src);
+
+	*response = NULL;
+
+	if (DAV_RESOURCE_TYPE_REGULAR != src->type) {
+		e = server_create_and_stat_error(srv_conf, pool, HTTP_CONFLICT,
+				0, "Cannot COPY this type of resource");
+		goto end_copy;
+	}
+	if (src->collection) {
+		e = server_create_and_stat_error(srv_conf, pool, HTTP_CONFLICT,
+				0, "No COPY on collections");
+		goto end_copy;
+	}
+	if (!apr_strnatcasecmp(src->info->hex_chunkid, dst->info->hex_chunkid)) {
+		e = server_create_and_stat_error(srv_conf, pool, HTTP_FORBIDDEN, 0,
+				"Source and destination should not have the same id");
+		goto end_copy;
+	}
+
+	DAV_DEBUG_RES(src, 0, "Copying %s to %s", resource_get_pathname(src),
+			resource_get_pathname(dst));
+	status = apr_file_link(resource_get_pathname(src),
+			resource_get_pathname(dst));
+
+	if (status != APR_SUCCESS) {
+		e = server_create_and_stat_error(srv_conf,
+				pool, HTTP_INTERNAL_SERVER_ERROR, status,
+				apr_pstrcat(pool, "Failed to COPY this chunk: ",
+					apr_strerror(status, buff, sizeof(buff)), NULL));
+		goto end_copy;
+	}
+
+	if (!src->info->chunk.oio_full_path) {
+		e = server_create_and_stat_error(srv_conf, pool,
+					HTTP_FORBIDDEN, 0,
+					apr_pstrdup(pool, "Missing fullpath"));
+				goto end_copy;
+	}
+
+	GError *local_error = NULL;
+	if (!set_rawx_info_to_file(resource_get_pathname(dst), &local_error,
+				&(src->info->chunk))) {
+			e = server_create_and_stat_error(srv_conf, pool,
+					HTTP_FORBIDDEN, 0,
+					apr_pstrdup(pool, gerror_get_message(local_error)));
+				goto end_copy;
+	}
+
+	server_inc_stat(srv_conf, RAWX_STATNAME_REP_2XX, 0);
+
+end_copy:
+	server_inc_request_stat(srv_conf, RAWX_STATNAME_REQ_OTHER,
+			request_get_duration(src->info->request));
+
+	return e;
+}
+
 static dav_error *
 dav_rawx_move_resource(dav_resource *src_res, dav_resource *dst_res,
 		dav_response **response)
@@ -806,7 +867,7 @@ static const dav_hooks_repository dav_hooks_repository_rawx =
 	dav_rawx_set_headers,
 	dav_rawx_deliver,
 	NULL /* no collection creation */,
-	NULL /* no copy of resources allowed */,
+	dav_rawx_copy_resource,
 	dav_rawx_move_resource /* only for regular resources */,
 	dav_rawx_remove_resource /* only for regular resources */,
 	dav_rawx_walk /* no walk across the chunks */,
