@@ -1,7 +1,7 @@
 /*
 OpenIO SDS rawx-lib
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -31,8 +31,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "rawx.h"
 
-static volatile ssize_t longest_xattr = 1024;
-static volatile ssize_t longest_xattr_list = 2048;
+static volatile ssize_t longest_xattr = 2048;
+static volatile ssize_t longest_xattr_list = 16384;
 
 static gchar *
 _getxattr_from_fd(int fd, const char *attrname)
@@ -70,6 +70,36 @@ retry:
 		goto error_set_attr; \
 }
 
+static gboolean
+_set_xattr_list(int fd, gchar* strkeys)
+{
+	if (!strkeys)
+		return FALSE;
+
+	gchar **keys = g_strsplit(strkeys, ",", -1);
+	if (!keys)
+		return FALSE;
+
+	gboolean rc = FALSE;
+	GChecksum *checksum = g_checksum_new(G_CHECKSUM_SHA256);
+	for (gchar **key = keys; *key; ++key) {
+		g_checksum_reset(checksum);
+		g_checksum_update(checksum, (guint8*)*key, strlen(*key));
+		gchar *hexup = g_ascii_strup(g_checksum_get_string(checksum), -1);
+		gchar *xname = g_strconcat(ATTR_DOMAIN_OIO, ":", hexup, NULL);
+		const int xrc = fsetxattr(fd, xname, *key, strlen(*key), 0);
+		g_free(xname);
+		g_free(hexup);
+		if (xrc == -1)
+			goto error_label;
+	}
+	rc = TRUE;
+error_label:
+	g_checksum_free(checksum);
+	g_strfreev(keys);
+	return rc;
+}
+
 gboolean
 set_rawx_info_to_fd (int fd, GError **error, struct chunk_textinfo_s *cti)
 {
@@ -109,6 +139,11 @@ set_rawx_info_to_fd (int fd, GError **error, struct chunk_textinfo_s *cti)
 	SET(ATTR_NAME_CHUNK_METADATA_COMPRESS, cti->compression_metadata);
 	SET(ATTR_NAME_CHUNK_COMPRESSED_SIZE,   cti->compression_size);
 
+	SET(ATTR_NAME_OIO_VERSION, cti->oio_version);
+
+	if (cti->oio_full_path && !_set_xattr_list(fd, cti->oio_full_path))
+		goto error_set_attr;
+
 	return TRUE;
 
 error_set_attr:
@@ -168,6 +203,68 @@ _get (int fd, const char *k, gchar **pv)
 
 #define GET(K,R) _get(fd, ATTR_DOMAIN "." K, &(R))
 
+static GString *
+_append_path(GString *gs, const char *path)
+{
+	if (gs->len > 0)
+		gs = g_string_append_c(gs, ',');
+	return g_string_append(gs, path);
+}
+
+static GString*
+_get_fullpaths_from_listxattr(int fd, gchar * xattrlist, ssize_t size)
+{
+	GString* user_list = g_string_sized_new(8192);
+
+	for (gint i = 0; i < size; i+= strlen(&xattrlist[i]) + 1) {
+		const char *k = &xattrlist[i];
+		if (g_str_has_prefix(k, ATTR_DOMAIN_OIO ":")) {
+			if (strchr(k, '/')) {
+				/* old-style with a path in the key */
+				k += sizeof(ATTR_DOMAIN_OIO ":") - 1;
+				user_list = _append_path(user_list, k);
+			} else {
+				/* new-style SHA256 identifiers */
+				gchar *path = NULL;
+				if (_get(fd, k, &path)) {
+					user_list = _append_path(user_list, path);
+					g_free(path);
+				}
+			}
+		}
+	}
+
+	return user_list;
+}
+
+static gboolean
+_get_fullpaths_from_fd(int fd, gchar **list_path)
+{
+	ssize_t s = longest_xattr_list;
+	gchar *buf = g_malloc0(s);
+	ssize_t rc;
+retry:
+	rc = flistxattr(fd, buf, longest_xattr_list);
+	if (rc < 0) {
+		if (errno == ERANGE) {
+			s = s*2;
+			if (s > longest_xattr_list)
+				longest_xattr_list = s;
+			buf = g_realloc(buf, s);
+			memset(buf, 0, s);
+			goto retry;
+		} else {
+			g_free(buf);
+			return FALSE;
+		}
+	}
+	GString *full_path = _get_fullpaths_from_listxattr(fd, buf, rc);
+	*list_path = full_path->str;
+	g_free(buf);
+	g_string_free(full_path, FALSE);
+	return TRUE;
+}
+
 gboolean
 get_rawx_info_from_fd (int fd, GError **error, struct chunk_textinfo_s *cti)
 {
@@ -180,7 +277,7 @@ get_rawx_info_from_fd (int fd, GError **error, struct chunk_textinfo_s *cti)
 		gchar *v = NULL;
 		if (!GET(ATTR_NAME_CONTENT_CONTAINER, v)) {
 			if (errno == ENOTSUP) {
-				GSETCODE(error, errno, "xatr not supported");
+				GSETCODE(error, errno, "xattr not supported");
 				return FALSE;
 			}
 		} else {
@@ -192,11 +289,13 @@ get_rawx_info_from_fd (int fd, GError **error, struct chunk_textinfo_s *cti)
 	if (!GET(ATTR_NAME_CONTENT_CONTAINER, cti->container_id)) {
 		/* just one check to detect unsupported xattr */
 		if (errno == ENOTSUP) {
-			GSETCODE(error, errno, "xatr not supported");
+			GSETCODE(error, errno, "xattr not supported");
 			return FALSE;
 		}
 	}
 
+	_get_fullpaths_from_fd(fd, &(cti->oio_full_path));
+
 	GET(ATTR_NAME_CONTENT_ID,      cti->content_id);
 	GET(ATTR_NAME_CONTENT_PATH,    cti->content_path);
 	GET(ATTR_NAME_CONTENT_VERSION, cti->content_version);
@@ -218,6 +317,8 @@ get_rawx_info_from_fd (int fd, GError **error, struct chunk_textinfo_s *cti)
 	GET(ATTR_NAME_CHUNK_METADATA_COMPRESS, cti->compression_metadata);
 	GET(ATTR_NAME_CHUNK_COMPRESSED_SIZE,   cti->compression_size);
 
+	GET(ATTR_NAME_OIO_VERSION, cti->oio_version);
+
 	return TRUE;
 }
 
@@ -263,39 +364,6 @@ get_compression_info_in_attr(const char *p, GError ** error, GHashTable *table)
 	return TRUE;
 }
 
-/* -------------------------------------------------------------------------- */
-
-static void
-_rawx_acl_clean(gpointer data, gpointer udata)
-{
-	(void) udata;
-	addr_rule_g_free(data);
-}
-
-void
-rawx_conf_gclean(rawx_conf_t* c)
-{
-	rawx_conf_clean(c);
-	g_free(c);
-}
-
-void
-rawx_conf_clean(rawx_conf_t* c)
-{
-	if(!c)
-		return;
-
-	if(c->ni) {
-		namespace_info_free(c->ni);
-		c->ni = NULL;
-	}
-	if(c->acl) {
-		g_slist_foreach(c->acl, _rawx_acl_clean, NULL);
-		g_slist_free(c->acl);
-		c->acl = NULL;
-	}
-}
-
 void
 chunk_textinfo_free_content(struct chunk_textinfo_s *cti)
 {
@@ -323,4 +391,8 @@ chunk_textinfo_free_content(struct chunk_textinfo_s *cti)
 
 	oio_str_clean (&cti->compression_metadata);
 	oio_str_clean (&cti->compression_size);
+
+	oio_str_clean (&cti->oio_version);
+
+	oio_str_clean (&cti->oio_full_path);
 }
