@@ -1,7 +1,7 @@
 /*
 OpenIO SDS resolver
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -23,23 +23,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 # include <resolver/hc_resolver.h>
 # include <glib.h>
 
-#ifndef  HC_RESOLVER_DEFAULT_MAX_SERVICES
-# define HC_RESOLVER_DEFAULT_MAX_SERVICES 1048576
-#endif
-
-#ifndef  HC_RESOLVER_DEFAULT_TTL_SERVICES
-# define HC_RESOLVER_DEFAULT_TTL_SERVICES 0
-#endif
-
-// No expiration and no max for content of META0 & Conscience
-#ifndef  HC_RESOLVER_DEFAULT_MAX_CSM0
-# define HC_RESOLVER_DEFAULT_MAX_CSM0 0
-#endif
-
-#ifndef  HC_RESOLVER_DEFAULT_TTL_CSM0
-# define HC_RESOLVER_DEFAULT_TTL_CSM0 0
-#endif
-
 struct lru_tree_s;
 
 struct cached_element_s
