@@ -1,7 +1,7 @@
 /*
 OpenIO SDS server
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2017 OpenIO, modified as part of OpenIO SDS
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -40,16 +40,19 @@ License along with this library.
 #include <arpa/inet.h>
 
 #include <metautils/lib/metautils.h>
+#include <metautils/lib/common_variables.h>
+#include <server/server_variables.h>
 
 #include "internals.h"
-#include "network_server.h"
-#include "resolv.h"
-#include "slab.h"
 
 GQuark gq_count_all = 0;
 GQuark gq_time_all = 0;
+GQuark gq_count_ioerror = 0;
+GQuark gq_time_ioerror = 0;
 GQuark gq_count_unexpected = 0;
 GQuark gq_time_unexpected = 0;
+GQuark gq_count_overloaded = 0;
+GQuark gq_time_overloaded = 0;
 
 static gboolean _endpoint_is_UNIX (struct endpoint_s *u);
 static gboolean _endpoint_is_INET6 (struct endpoint_s *u);
@@ -77,18 +80,15 @@ static void _client_remove_from_monitored(struct network_server_s *srv,
 static void _client_add_to_monitored(struct network_server_s *srv,
 		struct network_client_s *clt);
 
-static void _cb_worker(struct network_client_s *clt,
+static void _cb_tcp_worker(struct network_client_s *clt,
 		struct network_server_s *srv);
 
 static void _cb_stats(struct server_stat_msg_s *msg,
 		struct network_server_s *srv);
 
-static void _cb_ping(struct network_client_s *clt,
+static void _manage_udp_task(struct network_client_s *clt,
 		struct network_server_s *srv);
 
-/* Returns the number of max file descriptors for this process */
-static guint _server_get_maxfd(void);
-
 static void
 _client_sock_name(int fd, gchar *dst, gsize dst_size)
 {
@@ -138,6 +138,11 @@ _cnx_notify_close(struct network_server_s *srv)
 static void __attribute__ ((constructor))
 _constructor (void)
 {
+	gq_count_ioerror = g_quark_from_static_string (OIO_STAT_PREFIX_REQ ".IOERROR");
+	gq_time_ioerror = g_quark_from_static_string (OIO_STAT_PREFIX_TIME ".IOERROR");
+	gq_count_overloaded = g_quark_from_static_string (OIO_STAT_PREFIX_REQ ".OVERLOADED");
+	gq_time_overloaded = g_quark_from_static_string (OIO_STAT_PREFIX_TIME ".OVERLOADED");
+
 	gq_count_unexpected = g_quark_from_static_string (OIO_STAT_PREFIX_REQ ".UNEXPECTED");
 	gq_time_unexpected = g_quark_from_static_string (OIO_STAT_PREFIX_TIME ".UNEXPECTED");
 	gq_count_all = g_quark_from_static_string (OIO_STAT_PREFIX_REQ);
@@ -181,19 +186,7 @@ network_server_stat_push4 (struct network_server_s *srv, gboolean increment,
 	m->which[0] = k1, m->which[1] = k2, m->which[2] = k3, m->which[3] = k4;
 	m->value[0] = v1, m->value[1] = v2, m->value[2] = v3, m->value[3] = v4;
 	m->increment = BOOL(increment);
-	g_thread_pool_push (srv->pool_stats, m, NULL);
-}
-
-guint64
-network_server_stat_getone (struct network_server_s *srv, GQuark which)
-{
-	EXTRA_ASSERT (srv != NULL);
-	guint64 *p, value = 0;
-	g_mutex_lock (&srv->lock_stats);
-	if (NULL != (p = _stat_locate (srv, which)))
-		value = *p;
-	g_mutex_unlock (&srv->lock_stats);
-	return value;
+	metautils_gthreadpool_push("STAT", srv->pool_stats, m);
 }
 
 GArray*
@@ -207,55 +200,80 @@ network_server_stat_getall (struct network_server_s *srv)
 	return out;
 }
 
+void
+network_server_reconfigure(struct network_server_s *srv)
+{
+	if (!srv)
+		return;
+
+	/* Reconfigure the server */
+	guint emax = server_fd_max_passive;
+	if (!emax) {
+		emax = metautils_syscall_count_maxfd();
+		if (emax <= 10) {
+			GRID_WARN("Not enough max file descriptors (%u)", srv->cnx_max);
+			emax = 0;
+		} else {
+			emax -= 10;
+		}
+	}
+	if (emax > 0)
+		srv->cnx_max = emax;
+
+	/* Reconfigure the thread pools */
+	g_thread_pool_set_max_unused_threads(server_threadpool_max_unused);
+	g_thread_pool_set_max_idle_time(
+			server_threadpool_max_idle / G_TIME_SPAN_MILLISECOND);
+
+	gint _map(const guint i) {
+		return (i<=0 || i>G_MAXINT) ? -1 : (gint)i;
+	}
+
+	g_thread_pool_set_max_threads(
+			srv->pool_stats, _map(server_threadpool_max_stat), NULL);
+
+	g_thread_pool_set_max_threads(
+			srv->pool_tcp, _map(server_threadpool_max_tcp), NULL);
+
+	g_thread_pool_set_max_threads(
+			srv->pool_udp, _map(server_threadpool_max_udp), NULL);
+}
+
 struct network_server_s *
 network_server_init(void)
 {
-	int efd = -1;
+	int efd;
 	if ((efd = eventfd(0, EFD_NONBLOCK)) < 0) {
 		GRID_ERROR("eventfd creation failure: (%d) %s",
 				errno, strerror(errno));
 		return NULL;
 	}
 
-	guint maxfd = _server_get_maxfd();
-
 	struct network_server_s *result = g_malloc0(sizeof(struct network_server_s));
 	result->flag_continue = ~0;
-
+	result->cnx_max = metautils_syscall_count_maxfd();
 	g_mutex_init(&result->lock_stats);
 	result->stats = g_array_new (FALSE, TRUE, sizeof(struct server_stat_s));
-
 	result->queue_monitor = g_async_queue_new();
-
 	result->endpointv = g_malloc0(sizeof(struct endpoint_s*));
 	g_mutex_init(&result->lock_threads);
-
-	result->cnx_max_sys = maxfd;
-	result->cnx_max = (result->cnx_max_sys * 99) / 100;
 	result->eventfd = efd;
 	result->epollfd = epoll_create(1024);
-
-	result->atexit_max_open_never_input = SERVER_DEFAULT_CNX_INACTIVE;
-	result->atexit_max_idle = SERVER_DEFAULT_CNX_IDLE;
-	result->atexit_max_open_persist = SERVER_DEFAULT_CNX_LIFETIME;
-
 	result->gq_gauge_threads =      g_quark_from_static_string ("gauge thread.active");
 	result->gq_gauge_cnx_current =  g_quark_from_static_string ("gauge cnx.client");
-	result->gq_gauge_cnx_max =      g_quark_from_static_string ("gauge cnx.max");
-	result->gq_gauge_cnx_maxsys =   g_quark_from_static_string ("gauge cnx.max_sys");
 	result->gq_counter_cnx_accept = g_quark_from_static_string ("counter cnx.accept");
 	result->gq_counter_cnx_close =  g_quark_from_static_string ("counter cnx.close");
 
-	result->pool_stats = g_thread_pool_new ((GFunc)_cb_stats, result,
-			1, TRUE, NULL);
-	result->pool_tcp = g_thread_pool_new ((GFunc)_cb_worker, result,
-			SERVER_DEFAULT_THP_MAXWORKERS, FALSE, NULL);
-	/* Even if UDP is eventually not allowed, a pool with shared threads
-	 * doesn't consume much resources. */
-	result->pool_udp = g_thread_pool_new ((GFunc)_cb_ping, result,
-			8, FALSE, NULL);
-	g_thread_pool_set_max_unused_threads (SERVER_DEFAULT_THP_MAXUNUSED);
-	g_thread_pool_set_max_idle_time (SERVER_DEFAULT_THP_IDLE);
+	/* no limit at the creation ... */
+	result->pool_stats = g_thread_pool_new(
+			(GFunc)_cb_stats, result, 0, FALSE, NULL);
+	result->pool_tcp = g_thread_pool_new(
+			(GFunc)_cb_tcp_worker, result, 0, FALSE, NULL);
+	result->pool_udp = g_thread_pool_new(
+			(GFunc)_manage_udp_task, result, 0, FALSE, NULL);
+
+	/* ... and then supersedes the limits now */
+	network_server_reconfigure(result);
 
 	GRID_DEBUG("SERVER ready with epollfd[%d] eventfd[%d]",
 			result->epollfd, result->eventfd);
@@ -387,13 +405,6 @@ network_server_bind_host_lowlatency(struct network_server_s *srv,
 	_srv_bind_host(srv, url, u, factory, NETSERVER_LATENCY);
 }
 
-void
-network_server_bind_host_throughput(struct network_server_s *srv, const gchar *url, gpointer u,
-		network_transport_factory factory)
-{
-	_srv_bind_host(srv, url, u, factory, NETSERVER_THROUGHPUT);
-}
-
 gchar**
 network_server_endpoints (struct network_server_s *srv)
 {
@@ -531,13 +542,13 @@ _manage_client_event(struct network_server_s *srv,
 
 	if (clt->events & CLT_ERROR)
 		ARM_CLIENT(srv, clt, EPOLL_CTL_DEL);
-	g_thread_pool_push(srv->pool_tcp, clt, NULL);
+	metautils_gthreadpool_push("TCP", srv->pool_tcp, clt);
 }
 
 static void
 _manage_endpoint_event (struct network_server_s *srv, struct endpoint_s *e)
 {
-	for (guint i=0; i<SERVER_DEFAULT_ACCEPT_MAX ;++i) {
+	for (guint i=0; i<server_accept_batch_size ;++i) {
 		struct network_client_s *clt = _endpoint_accept_one(srv, e);
 		if (!clt) break;
 		if (clt->current_error)
@@ -553,9 +564,9 @@ static void
 _manage_events(struct network_server_s *srv)
 {
 	int erc;
-	struct epoll_event *pev, allev[SERVER_DEFAULT_EPOLL_MAXEV];
+	struct epoll_event *pev, allev[server_event_batch_size];
 
-	erc = epoll_wait(srv->epollfd, allev, SERVER_DEFAULT_EPOLL_MAXEV, 500);
+	erc = epoll_wait(srv->epollfd, allev, server_event_batch_size, 500);
 	if (erc > 0) {
 		while (erc-- > 0) {
 			pev = allev+erc;
@@ -582,9 +593,9 @@ _server_shutdown_inactive_connections(struct network_server_s *srv)
 {
 	guint count = 0;
 	gint64 now = oio_ext_monotonic_time ();
-	gint64 ti = now - srv->atexit_max_idle;
-	gint64 tc = now - srv->atexit_max_open_never_input;
-	gint64 tp = now - srv->atexit_max_open_persist;
+	const gint64 ti = now - server_cnx_ttl_idle;
+	const gint64 tc = now - server_cnx_ttl_never;
+	const gint64 tp = now - server_cnx_ttl_persist;
 
 	struct network_client_s *clt, *n;
 	for (clt=srv->first ; clt ; clt=n) {
@@ -630,9 +641,9 @@ _thread_cb_events(gpointer d)
 	 * the epoll pool.*/
 
 	GRID_DEBUG("Server %p waiting for its connections", srv);
-	srv->atexit_max_open_never_input = 5 * G_TIME_SPAN_SECOND;
-	srv->atexit_max_open_persist = 5 * G_TIME_SPAN_SECOND;
-	srv->atexit_max_idle = 1 * G_TIME_SPAN_SECOND;
+	server_cnx_ttl_never = 5 * G_TIME_SPAN_SECOND;
+	server_cnx_ttl_persist = 5 * G_TIME_SPAN_SECOND;
+	server_cnx_ttl_idle = 1 * G_TIME_SPAN_SECOND;
 
 	for (gint64 next = 0; 0 < srv->cnx_clients ;) {
 		_manage_events(srv);
@@ -673,7 +684,7 @@ _endpoint_monitor_udp (struct endpoint_s **pu, struct pollfd *pfd)
 }
 
 static void
-_cb_ping(struct network_client_s *clt, struct network_server_s *srv)
+_manage_udp_task(struct network_client_s *clt, struct network_server_s *srv)
 {
 	EXTRA_ASSERT(clt != NULL);
 	EXTRA_ASSERT(clt->server == srv);
@@ -687,7 +698,7 @@ _cb_ping(struct network_client_s *clt, struct network_server_s *srv)
 
 		/* OIO_SERVER_UDP_QUEUE_MAXAGE is arbitrary but it avoids managing
 		 * pings that have probably been retried by the emitter. */
-		if (now - clt->time.evt_in > OIO_SERVER_UDP_QUEUE_MAXAGE) {
+		if (now - clt->time.evt_in > server_udp_queue_ttl) {
 			GRID_DEBUG("PING %s -> %s queued for too long",
 					clt->peer_name, clt->local_name);
 		} else {
@@ -702,7 +713,7 @@ _cb_ping(struct network_client_s *clt, struct network_server_s *srv)
 }
 
 static void
-_manage_ping_event(struct network_server_s *srv, struct endpoint_s *e,
+_manage_udp_event(struct network_server_s *srv, struct endpoint_s *e,
 		struct pollfd *pfd)
 {
 	/* destined for little notifications, there is currently no clue ping
@@ -739,35 +750,15 @@ _manage_ping_event(struct network_server_s *srv, struct endpoint_s *e,
 				data_slab_make_buffer(g_memdup(buf, r), r));
 
 		/* notify the transport layer, and manage this in another thread */
-		if (NULL != clt->transport.notify_input) {
-			GError *err = NULL;
+		EXTRA_ASSERT(NULL != clt->transport.notify_input);
 
-			/* OIO_SERVER_UDP_QUEUE_MAXLEN is arbitrary, but it is only used to
-			 * avoid a memory leak */
-			const guint unprocessed = g_thread_pool_unprocessed(srv->pool_udp);
-			if (unprocessed > OIO_SERVER_UDP_QUEUE_MAXLEN) {
-				GRID_DEBUG("PING %s -> %s (%"G_GSIZE_FORMAT") dropped",
-						clt->peer_name, clt->local_name,
-						data_slab_sequence_size(&clt->input));
-			} else {
-				if (!g_thread_pool_push(srv->pool_udp, clt, &err)) {
-					GRID_WARN("PING %s -> %s (%"G_GSIZE_FORMAT") discarded: (%d) %s",
-							clt->peer_name, clt->local_name,
-							data_slab_sequence_size(&clt->input),
-							err->code, err->message);
-					_client_clean(srv, clt);
-				} else {
-					GRID_TRACE("PING %s -> %s (%"G_GSIZE_FORMAT") defered",
-							clt->peer_name, clt->local_name,
-							data_slab_sequence_size(&clt->input));
-				}
-			}
+		/* `server_udp_queue_maxlen` is arbitrary, but it is only used to
+		 * avoid a memory leak */
+		const guint unprocessed = g_thread_pool_unprocessed(srv->pool_udp);
+		if (unprocessed > server_udp_queue_maxlen) {
+			GRID_DEBUG("UDP dropped %s -> %s", clt->peer_name, clt->local_name);
 		} else {
-			GRID_DEBUG("PING %s -> %s (%"G_GSIZE_FORMAT") discarded: %s",
-					clt->peer_name, clt->local_name,
-					data_slab_sequence_size(&clt->input),
-					"no transport");
-			_client_clean(srv, clt);
+			metautils_gthreadpool_push("UDP", srv->pool_udp, clt);
 		}
 	}
 }
@@ -794,7 +785,7 @@ _thread_cb_ping(gpointer d)
 		} else if (rc > 0) {
 			for (guint i=0; i<count_structs ;++i) {
 				if (pfd[i].revents & POLLIN)
-					_manage_ping_event(srv, srv->endpointv[i], pfd+i);
+					_manage_udp_event(srv, srv->endpointv[i], pfd+i);
 				pfd[i].revents = 0;
 			}
 		}
@@ -804,7 +795,7 @@ _thread_cb_ping(gpointer d)
 }
 
 GError *
-network_server_run(struct network_server_s *srv)
+network_server_run(struct network_server_s *srv, void (*on_reload)(void))
 {
 	struct endpoint_s **pu, *u;
 	GError *err = NULL;
@@ -831,10 +822,6 @@ network_server_run(struct network_server_s *srv)
 		srv->thread_udp = g_thread_new("udp", _thread_cb_ping, srv);
 	srv->thread_tcp = g_thread_new("tcp", _thread_cb_events, srv);
 
-	network_server_stat_push2 (srv, FALSE,
-			srv->gq_gauge_cnx_max, srv->cnx_max,
-			srv->gq_gauge_cnx_maxsys, srv->cnx_max_sys);
-
 	while (srv->flag_continue) {
 		g_usleep(1 * G_TIME_SPAN_SECOND);
 		network_server_stat_push4 (srv, FALSE,
@@ -842,6 +829,11 @@ network_server_run(struct network_server_s *srv)
 				srv->gq_gauge_cnx_current, srv->cnx_clients,
 				srv->gq_counter_cnx_accept, srv->cnx_accept,
 				srv->gq_counter_cnx_close, srv->cnx_close);
+		if (main_signal_SIGHUP) {
+			main_signal_SIGHUP = FALSE;
+			if (on_reload)
+				(*on_reload)();
+		}
 	}
 
 	network_server_close_servers(srv);
@@ -907,7 +899,7 @@ _endpoint_open(struct endpoint_s *u, gboolean udp_allowed)
 	EXTRA_ASSERT(u != NULL);
 
 	struct sockaddr_storage ss = {0};
-	socklen_t ss_len = sizeof(ss);
+	socklen_t ss_len;
 
 	/* patch some socket preferences that make sense only for INET sockets */
 	if (_endpoint_is_UNIX(u)) {
@@ -973,8 +965,8 @@ _endpoint_open(struct endpoint_s *u, gboolean udp_allowed)
 		return NEWERROR(errsave, "bind(udp,%s) = '%s'", u->url, strerror(errsave));
 	}
 
-	/* for INET sockets, get the port really used */
 	if (_endpoint_is_INET(u)) {
+		/* for INET sockets, get the port really used */
 		memset(&ss, 0, sizeof(ss));
 		ss_len = sizeof(ss);
 		getsockname(u->fd, (struct sockaddr*)&ss, &ss_len);
@@ -982,6 +974,9 @@ _endpoint_open(struct endpoint_s *u, gboolean udp_allowed)
 			u->port_real = ntohs(((struct sockaddr_in*)&ss)->sin_port);
 		else
 			u->port_real = ntohs(((struct sockaddr_in6*)&ss)->sin6_port);
+
+		/* and benefit from the TCP_FASTOPEN support */
+		sock_set_fastopen(u->fd);
 	}
 
 	if (0 > listen(u->fd, 32768))
@@ -1015,9 +1010,7 @@ retry:
 			sock_set_cork(fd, TRUE);
 			break;
 		case NETSERVER_LATENCY:
-			sock_set_linger_default(fd);
-			sock_set_nodelay(fd, TRUE);
-			sock_set_tcpquickack(fd, TRUE);
+			sock_set_client_default(fd);
 			break;
 		default:
 			break;
@@ -1037,7 +1030,7 @@ retry:
 			SLICE_FREE(struct network_client_s, clt);
 			metautils_pclose(&fd);
 			_cnx_notify_close(srv);
-			GRID_WARN("Too many inbound connections! (cnx_max=%u)",
+			GRID_WARN("Too many inbound connections! (max=%u)",
 					srv->cnx_max);
 			return NULL;
 	}
@@ -1060,31 +1053,6 @@ retry:
 
 /* Server features ---------------------------------------------------------- */
 
-void
-network_server_set_max_workers(struct network_server_s *srv, guint max)
-{
-	EXTRA_ASSERT(srv != NULL);
-	if (!grid_main_is_running() || !srv->pool_tcp)
-		return;
-	g_thread_pool_set_max_threads (srv->pool_tcp, CLAMP(max, 1, G_MAXUINT16), NULL);
-}
-
-void
-network_server_set_maxcnx(struct network_server_s *srv, guint max)
-{
-	EXTRA_ASSERT(srv != NULL);
-
-	guint emax = CLAMP(max, 2, srv->cnx_max_sys);
-
-	if (emax != max)
-		GRID_WARN("MAXCNX [%u] clamped to [%u]", max, emax);
-
-	if (srv->cnx_max != emax) {
-		GRID_INFO("MAXCNX [%u] changed to [%u]", srv->cnx_max, emax);
-		srv->cnx_max = emax;
-	}
-}
-
 static void
 _cb_stats(struct server_stat_msg_s *msg, struct network_server_s *srv)
 {
@@ -1112,7 +1080,7 @@ _cb_stats(struct server_stat_msg_s *msg, struct network_server_s *srv)
 }
 
 static void
-_cb_worker(struct network_client_s *clt, struct network_server_s *srv)
+_cb_tcp_worker(struct network_client_s *clt, struct network_server_s *srv)
 {
 	EXTRA_ASSERT(clt != NULL);
 	EXTRA_ASSERT(clt->server == srv);
@@ -1122,6 +1090,30 @@ _cb_worker(struct network_client_s *clt, struct network_server_s *srv)
 		return;
 	}
 
+	/* The event stayed *really* long in the queue of the thread pool.
+	 * Let's close the connection, and let the client retry it's request. */
+	if (clt->events & CLT_READ) {
+#ifdef HAVE_ENBUG
+		if (oio_server_request_failure_threshold >= oio_ext_rand_int_range(1,100)) {
+			_client_clean(srv, clt);
+			return;
+		}
+#endif
+		const gint64 now = oio_ext_monotonic_time();
+		if (clt->time.evt_in < OLDEST(now, server_queue_max_delay)) {
+			GRID_INFO("STARVING fd %d peer %s delay %"G_GINT64_FORMAT"ms",
+					clt->fd, clt->peer_name,
+					(now - clt->time.evt_in) / G_TIME_SPAN_MILLISECOND);
+			_client_clean(srv, clt);
+			return;
+		}
+		if (clt->time.evt_in < OLDEST(now, server_queue_warn_delay)) {
+			GRID_INFO("CLOGGED fd %d peer %s delay %"G_GINT64_FORMAT"ms",
+					clt->fd, clt->peer_name,
+					(now - clt->time.evt_in) / G_TIME_SPAN_MILLISECOND);
+		}
+	}
+
 	_client_manage_event(clt, clt->events);
 
 	/* re Monitor the socket */
@@ -1144,21 +1136,6 @@ _cb_worker(struct network_client_s *clt, struct network_server_s *srv)
 	}
 }
 
-static guint
-_server_get_maxfd(void)
-{
-	struct rlimit limit;
-
-	if (0 != getrlimit(RLIMIT_NOFILE, &limit)) {
-		GRID_WARN("getrlimit() error: (%d) %s", errno, strerror(errno));
-		return 512;
-	}
-	else {
-		guint u = limit.rlim_cur;
-		return u;
-	}
-}
-
 /* Client functions --------------------------------------------------------- */
 
 static void
@@ -1273,22 +1250,17 @@ _client_manage_input(struct network_client_s *client)
 	guint total, size;
 
 	int _notify(void) {
-		if (!client->transport.notify_input)
-			return RC_PROCESSED;
 		if (!data_slab_sequence_has_data(&(client->input))) {
 			/* drain the data */
 			data_slab_sequence_clean_data(&(client->input));
 			return RC_PROCESSED;
 		}
-		GRID_TRACE2("fd=%d passing %u/%"G_GSIZE_FORMAT" to transport %p",
-				client->fd, total,
-				data_slab_sequence_size(&(client->input)),
-				client->transport.notify_input);
 		return client->transport.notify_input(client);
 	}
 
 	EXTRA_ASSERT(client != NULL);
 	EXTRA_ASSERT(client->fd >= 0);
+	EXTRA_ASSERT(client->transport.notify_input != NULL);
 
 	for (size=SLAB_STARTSIZE, total=0; total < ROUND_MAXSIZE ;) {
 
