@@ -1,7 +1,7 @@
 /*
 OpenIO SDS server
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -22,14 +22,6 @@ License along with this library.
 
 # include <server/slab.h>
 
-# define timespec_sub(a, b, result) do { \
-	(result)->tv_sec = (a)->tv_sec - (b)->tv_sec, \
-	(result)->tv_nsec = (a)->tv_nsec - (b)->tv_nsec; \
-	if ((result)->tv_nsec < 0) { \
-		--(result)->tv_sec, (result)->tv_nsec += 1000000000; \
-	} \
-} while (0)
-
 struct network_server_s;
 struct grid_stats_holder_s;
 struct network_client_s;
@@ -75,10 +67,16 @@ struct network_transport_s
 	gboolean waiting_for_close;
 };
 
+enum network_client_event_e {
+	CLT_READ=0X01,
+	CLT_WRITE=0X02,
+	CLT_ERROR=0X04
+};
+
 struct network_client_s
 {
 	int fd;
-	enum { CLT_READ=0X01, CLT_WRITE=0X02, CLT_ERROR=0X04 } events;
+	enum network_client_event_e events;
 	struct network_server_s *server;
 
 	int flags;
@@ -96,8 +94,8 @@ struct network_client_s
 	struct network_transport_s transport;
 	GError *current_error;
 
-	struct network_client_s *prev; /*!< XXX DO NOT USE */
-	struct network_client_s *next; /*!< XXX DO NOT USE */
+	struct network_client_s *prev; /*!< DO NOT USE */
+	struct network_client_s *next; /*!< DO NOT USE */
 
 	gchar local_name[128];
 	gchar peer_name[128];
@@ -105,18 +103,22 @@ struct network_client_s
 
 extern GQuark gq_count_all;
 extern GQuark gq_time_all;
+extern GQuark gq_count_ioerror;
+extern GQuark gq_time_ioerror;
 extern GQuark gq_count_unexpected;
 extern GQuark gq_time_unexpected;
+extern GQuark gq_count_overloaded;
+extern GQuark gq_time_overloaded;
 
 struct network_server_s * network_server_init(void);
 
+/* Re-set the limits of the server with the values stored in the central
+ * configuration facility */
+void network_server_reconfigure(struct network_server_s *srv);
+
 /* must be called PRIOR to network_server_open_servers */
 void network_server_allow_udp(struct network_server_s *srv);
 
-void network_server_set_max_workers(struct network_server_s *srv, guint max);
-
-void network_server_set_maxcnx(struct network_server_s *srv, guint max);
-
 typedef void (*network_transport_factory) (gpointer u,
 		struct network_client_s *clt);
 
@@ -124,10 +126,6 @@ void network_server_bind_host(struct network_server_s *srv,
 		const gchar *url, gpointer factory_udata,
 		network_transport_factory factory);
 
-void network_server_bind_host_throughput(struct network_server_s *srv,
-		const gchar *url, gpointer factory_udata,
-		network_transport_factory factory);
-
 /* returns a NULL-terminated array of strings, containing the actual IP:PORT
  * the server has been bond to, in the order they have been declared.
  * @param srv MUST be a valid server
@@ -145,7 +143,8 @@ void network_server_close_servers(struct network_server_s *srv);
 
 GError * network_server_open_servers(struct network_server_s *srv);
 
-GError * network_server_run(struct network_server_s *srv);
+GError * network_server_run(struct network_server_s *srv,
+		void (*on_reload)(void));
 
 void network_server_stop(struct network_server_s *srv);
 
@@ -158,9 +157,6 @@ void network_server_stat_push4 (struct network_server_s *srv, gboolean inc,
 		GQuark k1, guint64 v1, GQuark k2, guint64 v2,
 		GQuark k3, guint64 v3, GQuark k4, guint64 v4);
 
-/* Synchronosly get the current value of the stat named <which> */
-guint64 network_server_stat_getone (struct network_server_s *srv, GQuark which);
-
 GArray* network_server_stat_getall (struct network_server_s *srv);
 
 /* -------------------------------------------------------------------------- */
