@@ -1,7 +1,7 @@
 /*
 OpenIO SDS server
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -24,14 +24,16 @@ License along with this library.
 # include <string.h>
 # include <sys/types.h>
 
+enum data_slab_type_e {
+	STYPE_BUFFER=1,
+	STYPE_BUFFER_STATIC,
+	STYPE_GBYTES,
+	STYPE_EOF
+};
+
 struct data_slab_s
 {
-	enum {
-		STYPE_BUFFER=1,
-		STYPE_BUFFER_STATIC,
-		STYPE_GBYTES,
-		STYPE_EOF
-	} type;
+	enum data_slab_type_e type;
 	union {
 		GBytes *gbytes;
 		struct {
@@ -66,8 +68,6 @@ gboolean data_slab_send(struct data_slab_s *ds, int fd);
 gboolean data_slab_consume(struct data_slab_s *ds, guint8 **p_data,
 		gsize *p_size);
 
-void data_slab_trace(const gchar *tag, struct data_slab_s *ds);
-
 gsize data_slab_size(struct data_slab_s *ds);
 
 /* Slab-sequence features --------------------------------------------------- */
@@ -89,10 +89,6 @@ struct data_slab_s* data_slab_sequence_shift(
 void data_slab_sequence_unshift(struct data_slab_sequence_s *dss,
 		struct data_slab_s *ds);
 
-void data_slab_sequence_trace(struct data_slab_sequence_s *dss);
-
-gsize data_slab_sequence_size(struct data_slab_sequence_s *dss);
-
 /* Slab constructors -------------------------------------------------------- */
 
 struct data_slab_s * data_slab_make_empty(gsize alloc);
@@ -105,12 +101,6 @@ struct data_slab_s *
 data_slab_make_buffer2(guint8 *buff, gboolean tobefreed, gsize start,
 		gsize end, gsize alloc);
 
-static inline struct data_slab_s *
-data_slab_make_static_buffer(guint8 *buff, gsize bs)
-{
-	return data_slab_make_buffer2(buff, FALSE, 0, bs, bs);
-}
-
 static inline struct data_slab_s *
 data_slab_make_buffer(guint8 *buff, gsize bs)
 {
@@ -131,13 +121,6 @@ data_slab_make_gba(GByteArray *gba)
 	return data_slab_make_buffer2(g_byte_array_free(gba, FALSE), TRUE, 0, l, l);
 }
 
-static inline struct data_slab_s *
-data_slab_make_string(const gchar *s)
-{
-	gsize l = strlen(s);
-	return data_slab_make_buffer2((guint8*)g_strndup(s, l), TRUE, 0, l, l);
-}
-
 static inline struct data_slab_s *
 data_slab_make_static_string(const gchar *s)
 {
