@@ -1,7 +1,7 @@
 /*
 OpenIO SDS server
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -24,11 +24,13 @@ License along with this library.
 #include <malloc.h>
 
 #include <metautils/lib/metautils.h>
+#include <metautils/lib/common_variables.h>
+#include <server/server_variables.h>
 #include <core/internals.h>
 
-#include "internals.h"
 #include "network_server.h"
 #include "transport_gridd.h"
+#include "internals.h"
 
 struct cnx_data_s
 {
@@ -59,6 +61,11 @@ struct gridd_request_handler_s
 struct gridd_request_dispatcher_s
 {
 	GTree *tree_requests;
+
+	/* By default to 0, set to a monotonic time value when an I/O
+	 * error occurs, periodically checked for recent acctivity */
+	gint64 last_io_error;
+	gint64 last_io_success;
 };
 
 struct req_ctx_s
@@ -89,10 +96,12 @@ static void transport_gridd_clean_context(struct transport_client_context_s *);
 static gboolean _client_manage_l4v(struct network_client_s *clt, GByteArray *gba);
 
 /* XXX(jfs): ugly quirk, ok, but helpful to keep simple the stats support in
-   the server but allow it to reply "config volume /path/to/docroot" in its
-   stats. */
+ * the server but allow it to reply "config volume /path/to/docroot" in its
+ * stats. */
 const char *oio_server_volume = NULL;
 
+static int _local_variable = 0;
+
 /* -------------------------------------------------------------------------- */
 
 void
@@ -158,18 +167,6 @@ transport_gridd_build_empty_dispatcher(void)
 	return dispatcher;
 }
 
-struct gridd_request_dispatcher_s *
-transport_gridd_build_dispatcher( const struct gridd_request_descr_s *descr,
-		gpointer context)
-{
-	struct gridd_request_dispatcher_s *dispatcher;
-
-	dispatcher = transport_gridd_build_empty_dispatcher();
-	if (descr)
-		(void) transport_gridd_dispatcher_add_requests(dispatcher, descr, context);
-	return dispatcher;
-}
-
 void
 transport_gridd_factory0(struct gridd_request_dispatcher_s *dispatcher,
 		struct network_client_s *client)
@@ -224,7 +221,8 @@ network_client_log_access(struct log_item_s *item)
 	g_string_append_c(gstr, ' ');
 	g_string_append(gstr, ensure(r->client->peer_name));
 	g_string_append_c(gstr, ' ');
-	g_string_append(gstr, ensure(hashstr_str(r->reqname)));
+	g_string_append_len(gstr, ensure(hashstr_str(r->reqname)),
+			hashstr_len(r->reqname)?:1);
 	g_string_append_printf(gstr, " %d %"G_GINT64_FORMAT" %"G_GSIZE_FORMAT" ",
 			item->code, diff_total, item->out_len);
 	g_string_append(gstr, ensure(r->uid));
@@ -265,17 +263,16 @@ _request_get_name(MESSAGE req)
 }
 
 static gchar *
-_req_get_hex_ID(MESSAGE req, gchar *d, gsize dsize)
+_req_get_ID(MESSAGE req, gchar *d, gsize dsize)
 {
 	memset(d, 0, dsize);
 
 	gsize flen = 0;
 	guint8 *f = metautils_message_get_ID(req, &flen);
-	if (!f || !flen)
+	if (!f || !flen) {
 		*d = '-';
-	else if (oio_str_ishexa((gchar*)f, flen)) {
-		for (gchar *p=d; flen-- > 0 && dsize-- > 0;)
-			*(p++) = *(f++);
+	} else if (oio_str_is_printable((gchar*)f, flen)) {
+		g_strlcpy(d, (gchar*)f, MIN(dsize, flen + 1));
 	} else {
 		oio_str_bin2hex(f, MIN(flen,dsize/2), d, dsize);
 	}
@@ -416,14 +413,13 @@ _ctx_get_cnx_data(struct transport_client_context_s *ctx, const gchar *key)
 
 /* ------------------------------------------------------------------------- */
 
-static void _client_send_error(struct network_client_s *);
-
 static void
 transport_gridd_notify_error(struct network_client_s *clt)
 {
 	EXTRA_ASSERT(clt != NULL);
-	// @todo TODO write an access log trace
-	_client_send_error(clt);
+	/* No access log must be written here, for an unknown network error.
+	 * This always happens, periodically, for monitoring purposes (TCP hits
+	 * without data, connect() and close() */
 	_ctx_reset_cnx_data(clt->transport.client_context);
 }
 
@@ -441,7 +437,7 @@ transport_gridd_notify_input(struct network_client_s *clt)
 		struct data_slab_s *ds;
 
 		if (!ctx->gba_l4v)
-			ctx->gba_l4v = g_byte_array_new();
+			ctx->gba_l4v = g_byte_array_sized_new(256);
 
 		if (!(ds = data_slab_sequence_shift(&(clt->input))))
 			break;
@@ -475,8 +471,6 @@ transport_gridd_notify_input(struct network_client_s *clt)
 
 		gba_read(ctx->gba_l4v, ds, payload_size + 4);
 		data_slab_sequence_unshift(&(clt->input), ds);
-		ds = NULL;
-		/*data_slab_sequence_trace(&(clt->input));*/
 
 		if (ctx->gba_l4v->len >= 4 + payload_size) { /* complete */
 			if (!_client_manage_l4v(clt, ctx->gba_l4v)) {
@@ -537,7 +531,7 @@ _client_reply_fixed(struct req_ctx_s *req_ctx, gint code, const gchar *msg)
 	gsize answer_size = _reply_message(req_ctx->client, reply);
 
 	if ((req_ctx->final_sent = is_code_final(code))) {
-		struct log_item_s item;
+		struct log_item_s item = {0};
 		item.req_ctx = req_ctx;
 		item.code = code;
 		item.msg = msg;
@@ -547,19 +541,6 @@ _client_reply_fixed(struct req_ctx_s *req_ctx, gint code, const gchar *msg)
 	return (gboolean) answer_size;
 }
 
-static void
-_client_send_error(struct network_client_s *clt)
-{
-	GError *err = clt->current_error;
-	if (!err)
-		return;
-	/* TODO FIXME WTF!? */
-	MESSAGE request = metautils_message_create ();
-	MESSAGE reply = metaXServer_reply_simple(request, err->code, err->message);
-	(void) _reply_message(clt, reply);
-	metautils_message_destroy(request);
-}
-
 static gboolean
 _client_call_handler(struct req_ctx_s *req_ctx)
 {
@@ -580,26 +561,19 @@ _client_call_handler(struct req_ctx_s *req_ctx)
 	}
 	void _add_header(const gchar *n, GByteArray *v) {
 		EXTRA_ASSERT(!req_ctx->final_sent);
-		if (v) {
-			if (!n)
-				metautils_gba_unref(v);
-			else {
-				if (!headers)
-					headers = g_hash_table_new_full(g_str_hash, g_str_equal,
-							g_free, metautils_gba_unref);
-				g_hash_table_insert(headers, g_strdup(n), v);
-			}
-		}
+		EXTRA_ASSERT(n != NULL);
+		EXTRA_ASSERT(v != NULL);
+		if (!headers)
+			headers = g_hash_table_new_full(g_str_hash, g_str_equal,
+					g_free, metautils_gba_unref);
+		g_hash_table_insert(headers, g_strdup(n), v);
 	}
 	void _no_access (void) {
 		req_ctx->access_disabled = TRUE;
 	}
 	void _add_body(GByteArray *b) {
 		EXTRA_ASSERT(!req_ctx->final_sent);
-		if (body) {
-			metautils_gba_unref(body);
-			body = NULL;
-		}
+		EXTRA_ASSERT(body == NULL);
 		body = b;
 	}
 	void _send_reply(gint code, gchar *msg) {
@@ -615,11 +589,8 @@ _client_call_handler(struct req_ctx_s *req_ctx)
 			GHashTableIter iter;
 			gpointer n, v;
 			g_hash_table_iter_init(&iter, headers);
-			while (g_hash_table_iter_next(&iter, &n, &v)) {
-				if (!n || !v)
-					continue;
+			while (g_hash_table_iter_next(&iter, &n, &v))
 				metautils_message_add_field(answer, (gchar*)n, ((GByteArray*)v)->data, ((GByteArray*)v)->len);
-			}
 		}
 
 		/* encode and send */
@@ -636,31 +607,20 @@ _client_call_handler(struct req_ctx_s *req_ctx)
 	}
 	void _send_error(gint code, GError *e) {
 		EXTRA_ASSERT(!req_ctx->final_sent);
-		if (!e) {
-			_subject ("e=(0) NULL");
-			_send_reply(code, "OK");
-		}
-		else {
-			_add_body(NULL);
-			if (e->code == CODE_REDIRECT)
-				_subject ("e=(%d) redirect to %s", e->code, e->message);
-			else
-				_subject ("e=(%d) %s", e->code, e->message);
-			if (code)
-				e->code = code;
-			if (CODE_IS_NETWORK_ERROR(e->code))
-				e->code = CODE_PROXY_ERROR;
-			else if (CODE_IS_OK(e->code) || CODE_IS_TEMP(e->code))
-				e->code = CODE_INTERNAL_ERROR;
-			_send_reply(e->code, e->message);
-			g_clear_error(&e);
-		}
-	}
-	void _uid(const gchar *fmt, ...) {
-		va_list args;
-		va_start(args, fmt);
-		oio_str_reuse(&req_ctx->uid, g_strdup_vprintf(fmt, args));
-		va_end(args);
+		EXTRA_ASSERT(e != NULL);
+		EXTRA_ASSERT(body == NULL);
+		if (e->code == CODE_REDIRECT)
+			_subject ("e=(%d) redirect to %s", e->code, e->message);
+		else
+			_subject ("e=(%d) %s", e->code, e->message);
+		if (code)
+			e->code = code;
+		if (CODE_IS_NETWORK_ERROR(e->code))
+			e->code = CODE_PROXY_ERROR;
+		else if (CODE_IS_OK(e->code) || CODE_IS_TEMP(e->code))
+			e->code = CODE_INTERNAL_ERROR;
+		_send_reply(e->code, e->message);
+		g_clear_error(&e);
 	}
 	void _register_cnx_data(const gchar *key, gpointer data,
 			GDestroyNotify cleanup) {
@@ -676,15 +636,11 @@ _client_call_handler(struct req_ctx_s *req_ctx)
 		return _ctx_get_cnx_data(req_ctx->clt_ctx, key);
 	}
 
-	gboolean rc = FALSE;
-	struct gridd_request_handler_s *hdl;
-
 	/* reply data */
 	ctx.add_header = _add_header;
 	ctx.add_body = _add_body;
 	ctx.send_reply = _send_reply;
 	ctx.send_error = _send_error;
-	ctx.uid = _uid;
 	ctx.subject = _subject;
 	ctx.register_cnx_data = _register_cnx_data;
 	ctx.forget_cnx_data = _forget_cnx_data;
@@ -693,25 +649,39 @@ _client_call_handler(struct req_ctx_s *req_ctx)
 	/* request data */
 	ctx.client = req_ctx->client;
 	ctx.request = req_ctx->request;
-	ctx.reqname = req_ctx->reqname;
 
-	hdl = g_tree_lookup(req_ctx->disp->tree_requests, req_ctx->reqname);
-	if (!hdl) {
-		rc = _client_reply_fixed(req_ctx, CODE_NOT_FOUND, "No handler found");
-		_notify_request(req_ctx, gq_count_unexpected, gq_time_unexpected);
+	/* check the request wasn't queued for too long */
+	gboolean rc = FALSE;
+	const gint64 now = req_ctx->tv_parsed;
+	if (req_ctx->tv_start < OLDEST(now, meta_queue_max_delay)) {
+		gchar msg[128] = "";
+		g_snprintf(msg, sizeof(msg),
+				"Queued for too long (%" G_GINT64_FORMAT "ms)",
+				(now - req_ctx->tv_start) / G_TIME_SPAN_MILLISECOND);
+		rc = _client_reply_fixed(req_ctx, CODE_EXCESSIVE_LOAD, msg);
+		_notify_request(req_ctx, gq_count_overloaded, gq_time_overloaded);
 	} else {
-		EXTRA_ASSERT(hdl->handler != NULL);
-		rc = hdl->handler(&ctx, hdl->gdata, hdl->hdata);
-		_notify_request(req_ctx, hdl->stat_name_req, hdl->stat_name_time);
+		struct gridd_request_handler_s *hdl =
+			g_tree_lookup(req_ctx->disp->tree_requests, req_ctx->reqname);
+		if (!hdl) {
+			rc = _client_reply_fixed(req_ctx, CODE_NOT_FOUND, "No handler found");
+			_notify_request(req_ctx, gq_count_unexpected, gq_time_unexpected);
+		} else {
+			EXTRA_ASSERT(hdl->handler != NULL);
+			if (hdl->hdata != &_local_variable
+					&& !grid_daemon_is_io_ok(req_ctx->disp)) {
+				rc = _client_reply_fixed(req_ctx, CODE_UNAVAILABLE, "IO errors reported");
+				_notify_request(req_ctx, gq_count_ioerror, gq_time_ioerror);
+			} else {
+				rc = hdl->handler(&ctx, hdl->gdata, hdl->hdata);
+				_notify_request(req_ctx, hdl->stat_name_req, hdl->stat_name_time);
+			}
+		}
 	}
 
-	if (body) {
-		metautils_gba_unref(body);
-		body = NULL;
-	}
-	if (headers) {
+	EXTRA_ASSERT(body == NULL);
+	if (headers)
 		g_hash_table_destroy(headers);
-	}
 	return rc;
 }
 
@@ -751,9 +721,6 @@ _client_manage_l4v(struct network_client_s *client, GByteArray *gba)
 	EXTRA_ASSERT(gba != NULL);
 	EXTRA_ASSERT(client != NULL);
 
-	req_ctx.uid = NULL;
-	req_ctx.subject = NULL;
-	req_ctx.final_sent = FALSE;
 	req_ctx.client = client;
 	req_ctx.transport = &(client->transport);
 	req_ctx.clt_ctx = req_ctx.transport->client_context;
@@ -780,10 +747,14 @@ _client_manage_l4v(struct network_client_s *client, GByteArray *gba)
 	req_ctx.request = request;
 	req_ctx.reqname = _request_get_name(request);
 	req_ctx.uid = _request_get_cid(request);
-	req_ctx.reqid = _req_get_hex_ID(request, hexid, sizeof(hexid));
+	req_ctx.reqid = _req_get_ID(request, hexid, sizeof(hexid));
 	oio_ext_set_reqid(req_ctx.reqid);
 	rc = TRUE;
 
+	/* TODO check the socket is still active, specially if it seems old (~long
+	 * time spent in the queue). */
+
+	/* check the request is well formed */
 	if (!req_ctx.reqname) {
 		_client_reply_fixed(&req_ctx, CODE_BAD_REQUEST, "Invalid/No request name");
 		goto label_exit;
@@ -825,11 +796,9 @@ _stats_runner(gpointer k, gpointer v, gpointer u)
 
 static gboolean
 dispatch_LISTHANDLERS(struct gridd_reply_ctx_s *reply,
-		gpointer gdata, gpointer hdata)
+		gpointer gdata UNUSED, gpointer hdata UNUSED)
 {
-	(void) gdata, (void) hdata;
-
-	GByteArray *body = g_byte_array_new();
+	GByteArray *body = g_byte_array_sized_new(256);
 	g_tree_foreach(reply->client->transport.client_context->dispatcher->tree_requests,
 			_stats_runner, body);
 	reply->add_body(body);
@@ -840,13 +809,12 @@ dispatch_LISTHANDLERS(struct gridd_reply_ctx_s *reply,
 
 static gboolean
 dispatch_LEAN(struct gridd_reply_ctx_s *reply,
-		gpointer gdata, gpointer hdata)
+		gpointer gdata UNUSED, gpointer hdata UNUSED)
 {
 	gchar buf[128] = "Freed:";
-	(void) gdata, (void) hdata;
 
 	if (metautils_message_extract_flag (reply->request, "LIBC", FALSE)) {
-		if (malloc_trim (MALLOC_TRIM_SIZE))
+		if (malloc_trim (malloc_trim_size_ondemand))
 			g_strlcat (buf, " malloc-heap", sizeof(buf));
 	}
 
@@ -864,9 +832,8 @@ dispatch_LEAN(struct gridd_reply_ctx_s *reply,
 
 static gboolean
 dispatch_PING(struct gridd_reply_ctx_s *reply,
-		gpointer gdata, gpointer hdata)
+		gpointer gdata UNUSED, gpointer hdata UNUSED)
 {
-	(void) gdata, (void) hdata;
 	reply->no_access();
 	reply->send_reply(CODE_FINAL_OK, "OK");
 	return TRUE;
@@ -874,9 +841,8 @@ dispatch_PING(struct gridd_reply_ctx_s *reply,
 
 static gboolean
 dispatch_KILL(struct gridd_reply_ctx_s *reply,
-		gpointer gdata, gpointer hdata)
+		gpointer gdata UNUSED, gpointer hdata UNUSED)
 {
-	(void) gdata, (void) hdata;
 	if (reply->client->server->abort_allowed) {
 		abort();
 		reply->send_reply(CODE_FINAL_OK, "OK");
@@ -887,10 +853,48 @@ dispatch_KILL(struct gridd_reply_ctx_s *reply,
 }
 
 static gboolean
+dispatch_SETCFG(struct gridd_reply_ctx_s *reply,
+		gpointer gdata UNUSED, gpointer hdata UNUSED)
+{
+	gsize length = 0;
+	void *body = metautils_message_get_BODY(reply->request, &length);
+
+	json_object *jbody = NULL;
+	GError *err = JSON_parse_buffer(body, length, &jbody);
+	if (err) {
+		reply->send_error(0, err);
+	} else {
+		if (!json_object_is_type(jbody, json_type_object))
+			reply->send_error(0, BADREQ("Object argument expected"));
+		else if (json_object_object_length(jbody) <= 0)
+			reply->send_error(0, BADREQ("Empty object argument"));
+		else {
+			json_object_object_foreach(jbody, k, jv) {
+				oio_var_value_one_with_option(k, json_object_get_string(jv));
+			}
+			reply->send_reply(CODE_FINAL_OK, "OK");
+		}
+	}
+
+	if (jbody)
+		json_object_put (jbody);
+	return TRUE;
+}
+
+static gboolean
+dispatch_GETCFG(struct gridd_reply_ctx_s *reply,
+		gpointer gdata UNUSED, gpointer hdata UNUSED)
+{
+	GString *gstr = oio_var_list_as_json();
+	reply->add_body(g_bytes_unref_to_array(g_string_free_to_bytes(gstr)));
+	reply->send_reply(CODE_FINAL_OK, "OK");
+	return TRUE;
+}
+
+static gboolean
 dispatch_REDIRECT(struct gridd_reply_ctx_s *reply,
-		gpointer gdata, gpointer hdata)
+		gpointer gdata UNUSED, gpointer hdata UNUSED)
 {
-	(void) gdata, (void) hdata;
 	gchar **endpoints = network_server_endpoints(reply->client->server);
 	reply->send_error(0, NEWERROR(CODE_REDIRECT, "%s", endpoints[0]));
 	g_strfreev(endpoints);
@@ -901,20 +905,20 @@ dispatch_REDIRECT(struct gridd_reply_ctx_s *reply,
 
 static gboolean
 dispatch_STATS(struct gridd_reply_ctx_s *reply,
-		gpointer gdata, gpointer hdata)
+		gpointer gdata UNUSED, gpointer hdata UNUSED)
 {
-	(void) gdata, (void) hdata;
-	GByteArray *body = g_byte_array_new();
-
 	GArray *array = network_server_stat_getall(reply->client->server);
-	for (guint i=0; i<array->len ;++i) {
-		struct server_stat_s *st = &g_array_index (array, struct server_stat_s, i);
+	// Rough estimate, will be automatically resized if needed
+	GByteArray *body = g_byte_array_sized_new(array->len * 32);
+	for (guint i = 0; i < array->len; ++i) {
+		struct server_stat_s *st = &g_array_index(
+				array, struct server_stat_s, i);
 		gchar tmp[256];
-		gsize len = g_snprintf (tmp, sizeof(tmp), "%s=%"G_GUINT64_FORMAT"\n",
-				g_quark_to_string (st->which), st->value);
-		g_byte_array_append (body, (guint8*)tmp, len);
+		gsize len = g_snprintf(tmp, sizeof(tmp), "%s=%"G_GUINT64_FORMAT"\n",
+				g_quark_to_string(st->which), st->value);
+		g_byte_array_append(body, (guint8*)tmp, len);
 	}
-	g_array_free (array, TRUE);
+	g_array_free(array, TRUE);
 
 	if (oio_server_volume) {
 		g_byte_array_append (body,
@@ -931,9 +935,8 @@ dispatch_STATS(struct gridd_reply_ctx_s *reply,
 
 static gboolean
 dispatch_VERSION(struct gridd_reply_ctx_s *reply,
-		gpointer gdata, gpointer hdata)
+		gpointer gdata UNUSED, gpointer hdata UNUSED)
 {
-	(void) gdata, (void) hdata;
 	reply->no_access();
 	reply->add_body(metautils_gba_from_string(OIOSDS_PROJECT_VERSION));
 	reply->send_reply(CODE_FINAL_OK, "OK");
@@ -943,14 +946,18 @@ dispatch_VERSION(struct gridd_reply_ctx_s *reply,
 const struct gridd_request_descr_s*
 gridd_get_common_requests(void)
 {
+	/* The marker is used to detect local low-level handlers */
 	static struct gridd_request_descr_s descriptions[] = {
-		{"REQ_LEAN",      dispatch_LEAN,          NULL},
+		/* ping mustwill fail because of I/O errors */
 		{"REQ_PING",      dispatch_PING,          NULL},
 		{"REQ_STATS",     dispatch_STATS,         NULL},
-		{"REQ_VERSION",   dispatch_VERSION,       NULL},
-		{"REQ_HANDLERS",  dispatch_LISTHANDLERS,  NULL},
-		{"REQ_KILL",      dispatch_KILL,          NULL},
-		{"REQ_REDIRECT",  dispatch_REDIRECT,      NULL},
+		{"REQ_VERSION",   dispatch_VERSION,       &_local_variable},
+		{"REQ_HANDLERS",  dispatch_LISTHANDLERS,  &_local_variable},
+		{"REQ_GETCFG",    dispatch_GETCFG,        &_local_variable},
+		{"REQ_SETCFG",    dispatch_SETCFG,        &_local_variable},
+		{"REQ_REDIRECT",  dispatch_REDIRECT,      &_local_variable},
+		{"REQ_LEAN",      dispatch_LEAN,          &_local_variable},
+		{"REQ_KILL",      dispatch_KILL,          &_local_variable},
 		{NULL, NULL, NULL}
 	};
 
@@ -983,3 +990,32 @@ grid_daemon_bind_host(struct network_server_s *server, const gchar *url,
 			(network_transport_factory)transport_gridd_factory);
 }
 
+void
+grid_daemon_notify_io_status(
+		struct gridd_request_dispatcher_s *disp, gboolean ok)
+{
+	EXTRA_ASSERT(disp != NULL);
+	if (ok) {
+		disp->last_io_success = oio_ext_monotonic_time();
+	} else {
+		disp->last_io_error = oio_ext_monotonic_time();
+	}
+}
+
+gboolean
+grid_daemon_is_io_ok(struct gridd_request_dispatcher_s *disp)
+{
+	EXTRA_ASSERT(disp != NULL);
+
+	/* Never touched -> OK */
+	if (!disp->last_io_error && !disp->last_io_success)
+		return TRUE;
+
+	/* The most recent activity is an error -> KO */
+	if (disp->last_io_error > disp->last_io_success)
+		return FALSE;
+
+	/* check the probe thread was not stalled */
+	const gint64 now = oio_ext_monotonic_time();
+	return disp->last_io_success > OLDEST(now, G_TIME_SPAN_MINUTE);
+}
