@@ -1,7 +1,7 @@
 /*
 OpenIO SDS sqliterepo
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -23,6 +23,7 @@ License along with this library.
 #include <unistd.h>
 
 #include <metautils/lib/metautils.h>
+#include <sqliterepo/sqliterepo_variables.h>
 
 #include "sqliterepo.h"
 #include "cache.h"
@@ -32,9 +33,6 @@ License along with this library.
 
 #define BEACON_RESET(B) do { (B)->first = (B)->last = -1; } while (0)
 
-volatile gboolean oio_sqlx_cache_fail_on_heavy_load = SQLX_HEAVYLOAD_FAIL;
-volatile gboolean oio_sqlx_cache_alert_on_heavy_load = SQLX_HEAVYLOAD_ALERT;
-
 struct beacon_s
 {
 	gint first;
@@ -51,6 +49,7 @@ enum sqlx_base_status_e
 						 * to be kept open, and owner tells if the lock
 						 * os currently owned by a thread. */
 	SQLX_BASE_CLOSING, // base being closed, wait for notification and retry on it
+	SQLX_BASE_CLOSING_FOR_DELETION, // base about to be deleted
 };
 
 struct sqlx_base_s
@@ -95,16 +94,6 @@ struct sqlx_cache_s
 	guint bases_max_hard;
 	guint bases_used;
 
-	/* Maximum number of low-priority threads waiting for the base. */
-	guint32 max_waiting;
-
-	guint32 heat_threshold;
-
-	/* @see oio_ext_monotonic_time() */
-	gint64 cool_grace_delay;
-	gint64 hot_grace_delay;
-	gint64 open_timeout;
-
 	/* Doubly linked lists of tables, one by status */
 	struct beacon_s beacon_free;
 	struct beacon_s beacon_idle;
@@ -114,8 +103,6 @@ struct sqlx_cache_s
 	sqlx_cache_close_hook close_hook;
 };
 
-gint64 oio_cache_period_cond_wait = G_TIME_SPAN_SECOND;
-
 /* ------------------------------------------------------------------------- */
 
 static gboolean
@@ -139,6 +126,8 @@ sqlx_status_to_str(enum sqlx_base_status_e status)
 			return "USED";
 		case SQLX_BASE_CLOSING:
 			return "CLOSING";
+		case SQLX_BASE_CLOSING_FOR_DELETION:
+			return "CLOSING_FOR_DELETION";
 		default:
 			return "?";
 	}
@@ -278,6 +267,7 @@ sqlx_base_remove_from_list(sqlx_cache_t *cache, sqlx_base_t *base)
 			SQLX_REMOVE(cache, base, &(cache->beacon_used));
 			return;
 		case SQLX_BASE_CLOSING:
+		case SQLX_BASE_CLOSING_FOR_DELETION:
 			EXTRA_ASSERT(base->link.prev < 0);
 			EXTRA_ASSERT(base->link.next < 0);
 			return;
@@ -307,6 +297,7 @@ sqlx_base_add_to_list(sqlx_cache_t *cache, sqlx_base_t *base,
 			SQLX_UNSHIFT(cache, base, &(cache->beacon_used), SQLX_BASE_USED);
 			return;
 		case SQLX_BASE_CLOSING:
+		case SQLX_BASE_CLOSING_FOR_DELETION:
 			base->status = status;
 			return;
 	}
@@ -391,7 +382,7 @@ _signal_base(sqlx_base_t *base)
  * - The cache-wide lock is still owned
  */
 static void
-_expire_base(sqlx_cache_t *cache, sqlx_base_t *b)
+_expire_base(sqlx_cache_t *cache, sqlx_base_t *b, gboolean deleted)
 {
 	gpointer handle = b->handle;
 
@@ -400,7 +391,8 @@ _expire_base(sqlx_cache_t *cache, sqlx_base_t *b)
 	EXTRA_ASSERT(b->count_open == 0);
 	EXTRA_ASSERT(b->status == SQLX_BASE_USED);
 
-	sqlx_base_move_to_list(cache, b, SQLX_BASE_CLOSING);
+	sqlx_base_move_to_list(cache, b,
+			deleted? SQLX_BASE_CLOSING_FOR_DELETION : SQLX_BASE_CLOSING);
 
 	/* the base is for the given thread, it is time to REALLY close it.
 	 * But this can take a lot of time. So we can release the pool,
@@ -447,7 +439,7 @@ _expire_specific_base(sqlx_cache_t *cache, sqlx_base_t *b, gint64 now,
 	b->owner = g_thread_self();
 	sqlx_base_move_to_list(cache, b, SQLX_BASE_USED);
 
-	_expire_base(cache, b);
+	_expire_base(cache, b, FALSE);
 
 	/* If someone is waiting on the base while it is being closed
 	 * (this arrives when someone tries to read it again after
@@ -466,10 +458,10 @@ sqlx_expire_first_idle_base(sqlx_cache_t *cache, gint64 now)
 	/* Poll the next idle base, and respect the increasing order of the 'heat' */
 	if (0 <= (bd_idle = cache->beacon_idle.last))
 		rc = _expire_specific_base(cache, GET(cache, bd_idle), now,
-				cache->cool_grace_delay);
+				_cache_grace_delay_cool);
 	if (!rc && 0 <= (bd_idle = cache->beacon_idle_hot.last))
 		rc = _expire_specific_base(cache, GET(cache, bd_idle), now,
-				cache->hot_grace_delay);
+				_cache_grace_delay_hot);
 
 	if (rc) {
 		GRID_TRACE("Expired idle base at pos %d", bd_idle);
@@ -480,25 +472,17 @@ sqlx_expire_first_idle_base(sqlx_cache_t *cache, gint64 now)
 
 /* ------------------------------------------------------------------------- */
 
-sqlx_cache_t *
-sqlx_cache_set_max_waiting(sqlx_cache_t *cache, guint32 max_waiting)
-{
-	if (cache) {
-		cache->max_waiting = max_waiting;
-	}
-	return cache;
-}
-
 void
-sqlx_cache_set_max_bases(sqlx_cache_t *cache, guint max)
+sqlx_cache_reconfigure(sqlx_cache_t *cache)
 {
-	GRID_TRACE2("%s(%p,%u)", __FUNCTION__, cache, max);
-	EXTRA_ASSERT(cache != NULL);
-	max = CLAMP(max, 1, cache->bases_max_hard);
-	if (max != cache->bases_max_soft) {
-		GRID_INFO("cache max bases set to [%u]", max);
-		cache->bases_max_soft = max;
-	}
+	if (!cache)
+		return;
+
+	if (sqliterepo_repo_max_bases_soft > 0)
+		cache->bases_max_soft =
+			CLAMP(sqliterepo_repo_max_bases_soft, 1, cache->bases_max_hard);
+	else
+		cache->bases_max_soft = cache->bases_max_hard;
 }
 
 void
@@ -508,32 +492,21 @@ sqlx_cache_set_close_hook(sqlx_cache_t *cache, sqlx_cache_close_hook hook)
 	cache->close_hook = hook;
 }
 
-void
-sqlx_cache_set_open_timeout(sqlx_cache_t *cache, gint64 timeout)
-{
-	EXTRA_ASSERT(cache != NULL);
-	cache->open_timeout = timeout;
-}
-
 sqlx_cache_t *
-sqlx_cache_init(guint max_bases)
+sqlx_cache_init(void)
 {
 	sqlx_cache_t *cache = g_malloc0(sizeof(*cache));
-	cache->cool_grace_delay = SQLX_GRACE_DELAY_COOL * G_TIME_SPAN_SECOND;
-	cache->hot_grace_delay = SQLX_GRACE_DELAY_HOT * G_TIME_SPAN_SECOND;
-	cache->max_waiting = SQLX_MAX_WAITING;
-	cache->heat_threshold = 1;
 	g_mutex_init(&cache->lock);
 	cache->bases_by_name = g_tree_new_full(hashstr_quick_cmpdata,
 			NULL, NULL, NULL);
-	cache->open_timeout = 0;
 	BEACON_RESET(&(cache->beacon_free));
 	BEACON_RESET(&(cache->beacon_idle));
 	BEACON_RESET(&(cache->beacon_idle_hot));
 	BEACON_RESET(&(cache->beacon_used));
 
 	cache->bases_used = 0;
-	cache->bases_max_hard = cache->bases_max_soft = max_bases;
+	cache->bases_max_hard = sqliterepo_repo_max_bases_hard;
+	cache->bases_max_soft = CLAMP(sqliterepo_repo_max_bases_soft, 1, cache->bases_max_hard);
 	cache->bases = g_malloc0(cache->bases_max_hard * sizeof(sqlx_base_t));
 
 	for (guint i=0; i<cache->bases_max_hard ;i++) {
@@ -575,6 +548,7 @@ sqlx_cache_clean(sqlx_cache_t *cache)
 					sqlx_base_debug(__FUNCTION__, base);
 					break;
 				case SQLX_BASE_CLOSING:
+				case SQLX_BASE_CLOSING_FOR_DELETION:
 					GRID_ERROR("Base being closed while the cache is being cleaned");
 					break;
 			}
@@ -606,14 +580,11 @@ sqlx_cache_open_and_lock_base(sqlx_cache_t *cache, const hashstr_t *hname,
 	EXTRA_ASSERT(hname != NULL);
 	EXTRA_ASSERT(result != NULL);
 
-	gint64 start = oio_ext_monotonic_time();
-	gint64 deadline = DEFAULT_CACHE_OPEN_TIMEOUT;
-	if (cache->open_timeout > 0)
-		deadline = cache->open_timeout;
+	const gint64 start = oio_ext_monotonic_time();
+	const gint64 deadline = start + _cache_timeout_open;
 	GRID_TRACE2("%s(%p,%s,%p) delay = %"G_GINT64_FORMAT, __FUNCTION__,
 			(void*)cache, hname ? hashstr_str(hname) : "NULL",
 			(void*)result, deadline);
-	deadline += start;
 
 	g_mutex_lock(&cache->lock);
 retry:
@@ -672,13 +643,18 @@ retry:
 					GRID_DEBUG("Base [%s] in use by another thread (%X), waiting...",
 							hashstr_str(hname), oio_log_thread_id(base->owner));
 
-					if (!urgent && cache->max_waiting > 0 &&
-							base->count_waiting >= cache->max_waiting) {
-						if (oio_sqlx_cache_fail_on_heavy_load) {
-							err = NEWERROR(CODE_EXCESSIVE_LOAD, "Load too high");
+					if (!urgent && _cache_max_waiting > 0 &&
+							base->count_waiting >= _cache_max_waiting) {
+						if (_cache_fail_on_heavy_load) {
+							err = NEWERROR(CODE_EXCESSIVE_LOAD, "Load too high "
+									"(%"G_GUINT32_FORMAT"/%"G_GUINT32_FORMAT")",
+									base->count_waiting, _cache_max_waiting);
 							break;
-						} else if (oio_sqlx_cache_alert_on_heavy_load) {
-							GRID_WARN("Load too high on [%s]", hashstr_str(hname));
+						} else if (_cache_alert_on_heavy_load) {
+							GRID_WARN("Load too high on [%s] "
+									"(%"G_GUINT32_FORMAT"/%"G_GUINT32_FORMAT")",
+									hashstr_str(hname),
+									base->count_waiting, _cache_max_waiting);
 						}
 					}
 
@@ -687,7 +663,7 @@ retry:
 					/* The lock is held by another thread/request.
 					   Do not use 'now' because it can be a fake clock */
 					g_cond_wait_until(wait_cond, &cache->lock,
-							g_get_monotonic_time() + oio_cache_period_cond_wait);
+							g_get_monotonic_time() + _cache_period_cond_wait);
 
 					base->count_waiting --;
 					goto retry;
@@ -702,8 +678,14 @@ retry:
 				/* Just wait for a notification then retry
 				   Do not use 'now' because it can be a fake clock */
 				g_cond_wait_until(wait_cond, &cache->lock,
-						g_get_monotonic_time() + oio_cache_period_cond_wait);
+						g_get_monotonic_time() + _cache_period_cond_wait);
 				goto retry;
+
+			case SQLX_BASE_CLOSING_FOR_DELETION:
+				err = NEWERROR(CODE_UNAVAILABLE,
+						"Base [%s] about to be deleted",
+						hashstr_str(hname));
+				break;
 		}
 	}
 
@@ -720,11 +702,11 @@ retry:
 }
 
 GError *
-sqlx_cache_unlock_and_close_base(sqlx_cache_t *cache, gint bd, gboolean force)
+sqlx_cache_unlock_and_close_base(sqlx_cache_t *cache, gint bd, guint32 flags)
 {
 	GError *err = NULL;
 
-	GRID_TRACE2("%s(%p,%d,%d)", __FUNCTION__, (void*)cache, bd, force);
+	GRID_TRACE2("%s(%p,%d,%d)", __FUNCTION__, (void*)cache, bd, flags);
 
 	EXTRA_ASSERT(cache != NULL);
 	if (base_id_out(cache, bd))
@@ -750,14 +732,14 @@ sqlx_cache_unlock_and_close_base(sqlx_cache_t *cache, gint bd, gboolean force)
 
 		case SQLX_BASE_USED:
 			EXTRA_ASSERT(base->count_open > 0);
-			// held by the current thread
-			if (!(-- base->count_open)) { // to be closed
-				if (force) {
-					_expire_base(cache, base);
+			/* held by the current thread */
+			if (!(-- base->count_open)) {  /* to be closed */
+				if (flags & (SQLX_CLOSE_IMMEDIATELY|SQLX_CLOSE_FOR_DELETION)) {
+					_expire_base(cache, base, flags & SQLX_CLOSE_FOR_DELETION);
 				} else {
 					sqlx_base_debug("CLOSING", base);
 					base->owner = NULL;
-					if (base->heat >= cache->heat_threshold)
+					if (base->heat >= _cache_heat_threshold)
 						sqlx_base_move_to_list(cache, base, SQLX_BASE_IDLE_HOT);
 					else
 						sqlx_base_move_to_list(cache, base, SQLX_BASE_IDLE);
@@ -766,6 +748,7 @@ sqlx_cache_unlock_and_close_base(sqlx_cache_t *cache, gint bd, gboolean force)
 			break;
 
 		case SQLX_BASE_CLOSING:
+		case SQLX_BASE_CLOSING_FOR_DELETION:
 			EXTRA_ASSERT(base->owner != NULL);
 			EXTRA_ASSERT(base->owner != g_thread_self());
 			err = NEWERROR(CODE_INTERNAL_ERROR, "base being closed");
@@ -829,7 +812,7 @@ guint
 sqlx_cache_expire(sqlx_cache_t *cache, guint max, gint64 duration)
 {
 	guint nb = 0;
-	gint64 pivot = oio_ext_monotonic_time () + duration;
+	gint64 deadline = oio_ext_monotonic_time () + duration;
 
 	EXTRA_ASSERT(cache != NULL);
 
@@ -837,7 +820,7 @@ sqlx_cache_expire(sqlx_cache_t *cache, guint max, gint64 duration)
 
 	for (nb=0; !max || nb < max ; nb++) {
 		gint64 now = oio_ext_monotonic_time ();
-		if (now > pivot || !sqlx_expire_first_idle_base(cache, now))
+		if (now > deadline || !sqlx_expire_first_idle_base(cache, now))
 			break;
 	}
 
