@@ -1,7 +1,7 @@
 /*
 OpenIO SDS sqliterepo
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -22,14 +22,6 @@ License along with this library.
 
 # include <metautils/lib/metautils.h>
 
-/* Default timeout when trying to open a base currently in use by another
-   thread. The precision is uniform with oio_ext_monotonic_time(), manage it
-   with G_TIME_SPAN_* macros. */
-#define DEFAULT_CACHE_OPEN_TIMEOUT (20 * G_TIME_SPAN_SECOND)
-
-extern volatile gboolean oio_sqlx_cache_fail_on_heavy_load;
-extern volatile gboolean oio_sqlx_cache_alert_on_heavy_load;
-
 struct hashstr_s;
 
 typedef void (*sqlx_cache_close_hook)(gpointer);
@@ -40,19 +32,14 @@ gpointer sqlx_cache_get_handle(sqlx_cache_t *cache, gint bd);
 
 void sqlx_cache_set_handle(sqlx_cache_t *cache, gint bd, gpointer handle);
 
-sqlx_cache_t * sqlx_cache_init(guint max_bases);
+sqlx_cache_t * sqlx_cache_init(void);
 
 void sqlx_cache_set_close_hook(sqlx_cache_t *cache,
 	sqlx_cache_close_hook hook);
 
-void sqlx_cache_set_max_bases(sqlx_cache_t *cache, guint max);
-
-/* Limit the number of low-priority threads waiting for a lock */
-sqlx_cache_t * sqlx_cache_set_max_waiting(sqlx_cache_t *cache,
-		guint32 max_waiting);
-
-/* timeout in the precision of oio_ext_monotonic_time() */
-void sqlx_cache_set_open_timeout(sqlx_cache_t *cache, gint64 timeout);
+/* Re-set the working variables with the up-to-date values from the central
+ * configuration facility */
+void sqlx_cache_reconfigure(sqlx_cache_t *cache);
 
 void sqlx_cache_clean(sqlx_cache_t *cache);
 
@@ -65,7 +52,7 @@ GError * sqlx_cache_open_and_lock_base(sqlx_cache_t *cache,
 
 /** The invert of sqlx_cache_open_and_lock_base() */
 GError * sqlx_cache_unlock_and_close_base(sqlx_cache_t *cache, gint bd,
-		gboolean force);
+		guint32 flags);
 
 guint sqlx_cache_expire_all(sqlx_cache_t *cache);
 
