@@ -1,7 +1,7 @@
 /*
 OpenIO SDS metautils
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2017 OpenIO, modified as part of OpenIO SDS
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -25,16 +25,17 @@ License along with this library.
 #include <sys/socket.h>
 #include <sys/epoll.h>
 #include <sys/eventfd.h>
-#include <sys/resource.h>
 
 #include <metautils/lib/metautils.h>
+#include <sqliterepo/sqliterepo_variables.h>
+#include <sqliterepo/sqliterepo_remote_variables.h>
+
 #include "gridd_client_pool.h"
 
 #define MAX_ROUND 32
 
 struct gridd_client_pool_s
 {
-	struct gridd_client_pool_vtable_s *vtable;
 	struct event_client_s **active_clients;
 	GAsyncQueue *pending_clients;
 
@@ -44,37 +45,14 @@ struct gridd_client_pool_s
 	/* notifications */
 	int efd; /*!< event file descriptor */
 
-	int active_clients_size;
-	int active_max;
-	int active_count;
+	guint active_clients_size;
+	guint active_clients_max;
+	guint active_count;
 
 	/* if set to any non-zero value, new requests are not started */
 	int closed;
 };
 
-#ifdef HAVE_ENBUG
-gint32 oio_sqlx_request_failure_threshold = 50;
-#endif
-
-static void _destroy (struct gridd_client_pool_s *p);
-
-static guint _get_max (struct gridd_client_pool_s *pool);
-
-static void _set_max (struct gridd_client_pool_s *pool, guint max);
-
-static void _defer (struct gridd_client_pool_s *p, struct event_client_s *ev);
-
-static GError* _round (struct gridd_client_pool_s *p, time_t sec);
-
-static struct gridd_client_pool_vtable_s VTABLE =
-{
-	_destroy,
-	_get_max,
-	_set_max,
-	_defer,
-	_round
-};
-
 struct gridd_client_pool_s *
 gridd_client_pool_create(void)
 {
@@ -93,17 +71,14 @@ gridd_client_pool_create(void)
 		return NULL;
 	}
 
-	/* TODO(jfs): factorize this in metautils */
-	struct rlimit limit = {0};
-	if (0 != getrlimit(RLIMIT_NOFILE, &limit))
-		limit.rlim_cur = limit.rlim_max = 32768;
+	const guint maxfd = metautils_syscall_count_maxfd();
 
 	pool = g_malloc0(sizeof(*pool));
 	pool->pending_clients = g_async_queue_new();
 
 	pool->fdmon = fdmon;
-	pool->active_max = limit.rlim_cur;
-	pool->active_clients_size = limit.rlim_cur;
+	pool->active_clients_size = maxfd;
+	pool->active_clients_max = maxfd;
 	pool->active_clients = g_malloc0(pool->active_clients_size
 			* sizeof(struct event_client_s*));
 
@@ -120,7 +95,7 @@ gridd_client_pool_create(void)
 		return NULL;
 	}
 
-	pool->vtable = &VTABLE;
+	gridd_client_pool_reconfigure(pool);
 	return pool;
 }
 
@@ -188,12 +163,11 @@ event_client_free(struct event_client_s *ec)
 	g_free (ec);
 }
 
-static void
-_destroy(struct gridd_client_pool_s *pool)
+void
+gridd_client_pool_destroy(struct gridd_client_pool_s *pool)
 {
 	if (!pool)
 		return;
-	EXTRA_ASSERT(pool->vtable == &VTABLE);
 
 	pool->closed = ~0;
 
@@ -211,7 +185,7 @@ _destroy(struct gridd_client_pool_s *pool)
 	}
 
 	if (pool->active_clients) {
-		for (int i=0; i<pool->active_clients_size ;i++) {
+		for (guint i=0; i<pool->active_clients_size ;i++) {
 			struct event_client_s *ec = pool->active_clients[i];
 			pool->active_clients[i] = NULL;
 			if (ec)
@@ -241,17 +215,17 @@ _manage_timeouts(struct gridd_client_pool_s *pool)
 		return;
 
 	gint64 now = oio_ext_monotonic_time ();
-	if (now - pool->last_timeout_check < G_TIME_SPAN_SECOND)
+	if (now - pool->last_timeout_check < oio_sqlx_timeout_check_period)
 		return;
 	pool->last_timeout_check = now;
 
-	for (int i=0; i<pool->active_clients_size ;i++) {
+	for (guint i=0; i<pool->active_clients_size ;i++) {
 		struct event_client_s *ec;
 		if (!(ec = pool->active_clients[i]))
 			continue;
 
 		EXTRA_ASSERT(ec->client != NULL);
-		EXTRA_ASSERT(i == gridd_client_fd(ec->client));
+		EXTRA_ASSERT((int)i == gridd_client_fd(ec->client));
 
 		if (gridd_client_expire (ec->client, now)) {
 			GRID_INFO("EXPIRED Client fd=%d [%s]", i, gridd_client_url(ec->client));
@@ -261,7 +235,7 @@ _manage_timeouts(struct gridd_client_pool_s *pool)
 	}
 
 	gint64 elapsed = oio_ext_monotonic_time () - now;
-	if (elapsed > 5 * G_TIME_SPAN_SECOND) {
+	if (elapsed > oio_sqlx_timeout_check_max) {
 		GRID_WARN("Client pool timeout check took %"G_GINT64_FORMAT" ms",
 				elapsed / G_TIME_SPAN_MILLISECOND);
 	} else {
@@ -279,7 +253,7 @@ _manage_requests(struct gridd_client_pool_s *pool)
 	EXTRA_ASSERT(pool != NULL);
 
 	gint64 start = oio_ext_monotonic_time();
-	while (pool->active_count < pool->active_max) {
+	while (pool->active_count < pool->active_clients_max) {
 		ec = g_async_queue_try_pop(pool->pending_clients);
 		if (NULL == ec)
 			break;
@@ -324,7 +298,7 @@ _manage_requests(struct gridd_client_pool_s *pool)
 				"this is a bit too much",
 				elapsed / G_TIME_SPAN_SECOND);
 		gint qlen = g_async_queue_length(pool->pending_clients);
-		if (qlen > pool->active_max)
+		if (qlen > (gint)pool->active_clients_max)
 			GRID_WARN("Client pool still has %d pending requests, "
 					"are we under an election storm?", qlen);
 	}
@@ -369,15 +343,36 @@ _manage_all_events(struct gridd_client_pool_s *pool,
 	}
 }
 
-static GError *
-_round(struct gridd_client_pool_s *pool, time_t sec)
+void
+gridd_client_pool_reconfigure(struct gridd_client_pool_s *pool)
+{
+	EXTRA_ASSERT(pool != NULL);
+	if (sqliterepo_fd_max_active <= 0) {
+		pool->active_clients_max = pool->active_clients_size;
+	} else {
+		pool->active_clients_max =
+			CLAMP(sqliterepo_fd_max_active, 1, pool->active_clients_size);
+	}
+}
+
+GError *
+gridd_client_pool_round(struct gridd_client_pool_s *pool, time_t sec)
 {
 	EXTRA_ASSERT(pool != NULL);
-	EXTRA_ASSERT(pool->vtable == &VTABLE);
 	EXTRA_ASSERT(pool->fdmon >= 0);
+	g_assert(sqliterepo_fd_max_active > 0);
+
+	/* Avoid a possible starvation, when all the events have been
+	 * consumed but some events remain in the queue and slots are
+	 * available to manage them. */
+	long delay = sec * 1000L;
+	if (g_async_queue_length(pool->pending_clients) > 0
+			&& pool->active_count < pool->active_clients_max) {
+		delay = 0L;
+	}
 
 	struct epoll_event ev[MAX_ROUND];
-	int rc = epoll_wait(pool->fdmon, ev, MAX_ROUND, sec * 1000L);
+	int rc = epoll_wait(pool->fdmon, ev, MAX_ROUND, delay);
 
 	if (rc < 0 && errno != EINTR)
 		return NEWERROR(errno, "epoll_wait error: %s", strerror(errno));
@@ -389,11 +384,10 @@ _round(struct gridd_client_pool_s *pool, time_t sec)
 	return NULL;
 }
 
-static void
-_defer(struct gridd_client_pool_s *pool, struct event_client_s *ev)
+void
+gridd_client_pool_defer(struct gridd_client_pool_s *pool, struct event_client_s *ev)
 {
 	EXTRA_ASSERT(pool != NULL);
-	EXTRA_ASSERT(pool->vtable == &VTABLE);
 	EXTRA_ASSERT(ev != NULL);
 	EXTRA_ASSERT(pool->pending_clients != NULL);
 	EXTRA_ASSERT(pool->efd >= 0);
@@ -404,33 +398,18 @@ _defer(struct gridd_client_pool_s *pool, struct event_client_s *ev)
 	} else {
 		gint64 now = oio_ext_monotonic_time();
 		ev->deadline_start = now + 4 * G_TIME_SPAN_SECOND;
-		/* eventfd requires 8-byte integer */
-		guint64 c = 1u;
 		g_async_queue_push(pool->pending_clients, ev);
-		int rc = metautils_syscall_write(pool->efd, &c, 8);
-		if (unlikely(rc < 0)) {
-			GRID_WARN("Failed to signal new deferred requests: (%d) %s",
-					errno, strerror(errno));
-		}
 	}
 }
 
-/* Bias introduced: 1 for epoll, 1 for the eventfd */
-
-static guint
-_get_max(struct gridd_client_pool_s *pool)
+void
+gridd_client_pool_notify(struct gridd_client_pool_s *pool)
 {
-	EXTRA_ASSERT(pool != NULL);
-	EXTRA_ASSERT(pool->vtable == &VTABLE);
-	return pool->active_max + 2;
+	/* eventfd requires 8-byte integer */
+	guint64 c = 1u;
+	int rc = metautils_syscall_write(pool->efd, &c, 8);
+	if (unlikely(rc < 0)) {
+		GRID_WARN("Failed to signal new deferred requests: (%d) %s",
+				errno, strerror(errno));
+	}
 }
-
-static void
-_set_max(struct gridd_client_pool_s *pool, guint max)
-{
-	EXTRA_ASSERT(pool != NULL);
-	EXTRA_ASSERT(pool->vtable == &VTABLE);
-	EXTRA_ASSERT(max > 2);
-	pool->active_max = max - 2;
-}
-
