@@ -1,7 +1,7 @@
 /*
 OpenIO SDS sqliterepo
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -21,18 +21,8 @@ License along with this library.
 #include <unistd.h>
 
 #include <metautils/lib/metautils.h>
-#include <metautils/lib/metacomm.h>
-
-#include <RowFieldSequence.h>
-#include <RowFieldValue.h>
-#include <RowField.h>
-#include <Row.h>
-#include <RowSet.h>
-#include <RowName.h>
-#include <TableHeader.h>
-#include <Table.h>
-#include <TableSequence.h>
-#include <asn_codecs.h>
+#include <metautils/lib/codec.h>
+#include <sqliterepo/sqliterepo_remote_variables.h>
 
 #include "sqliterepo.h"
 #include "election.h"
@@ -58,18 +48,19 @@ struct sqlx_repctx_s
 	GString *errors;
 
 	// Count the explicit changes, those matched
-	gint32 changes;
+	int changes;
+	guint8 local_changes : 1;
 
 	// if set, there is no replication configured, even if a replicated
 	// transaction context had been initiated.
-	char hollow : 1;
+	guint8 hollow : 1;
 
 	// if set, some changes were not matched by the update hook and a resync
 	// will be necessary. This is the case in the SQLX by "DELETE FROM <table>"
 	// queries.
-	char huge : 1;
+	guint8 huge : 1;
 
-	char any_change : 1;
+	guint8 any_change : 1;
 };
 
 static guint
@@ -147,7 +138,7 @@ load_table_header(sqlite3_stmt *stmt, Table_t *t)
 
 	for (i=0,max=sqlite3_data_count(stmt); i<max ;i++) {
 		const char *cname = sqlite3_column_name(stmt, i);
-		struct RowName *rname = calloc(1, sizeof(*rname));
+		struct RowName *rname = ASN1C_CALLOC(1, sizeof(*rname));
 		asn_uint32_to_INTEGER(&(rname->pos), i);
 		OCTET_STRING_fromBuf(&(rname->name), cname, strlen(cname));
 		asn_sequence_add(&(t->header.list), rname);
@@ -163,10 +154,10 @@ load_statement(sqlite3_stmt *stmt, Row_t *row, Table_t *table)
 		load_table_header(stmt, table);
 
 	if (!row->fields) /* Lazy memory allocation */
-		row->fields = calloc(1, sizeof(struct RowFieldSequence));
+		row->fields = ASN1C_CALLOC(1, sizeof(struct RowFieldSequence));
 
 	for (i=0,max=sqlite3_data_count(stmt); i<max ;i++) {
-		struct RowField *rf = calloc(1, sizeof(*rf));
+		struct RowField *rf = ASN1C_CALLOC(1, sizeof(*rf));
 		asn_uint32_to_INTEGER(&(rf->pos), i);
 		rf->value.present = RowFieldValue_PR_n;
 
@@ -237,21 +228,31 @@ load_table_row(sqlite3 *db, const hashstr_t *name, gint64 rowid, Row_t *row,
 static void
 context_pending_inc_versions(struct sqlx_repctx_s *ctx)
 {
-	gboolean _on_table(hashstr_t *k, GTree *rows, gpointer u0) {
-		(void) u0;
+	/* while we iterate on the tables about to be changed, we keep an eye
+	 * on the 'admin' table. We need to increment its version number only
+	 * once. Because that table will always receive a change (the version
+	 * themselves alter the table) but the legit operations on the DB maybe
+	 * already touched that table and cause an increment. */
+	volatile gboolean admin_changed = FALSE;
+	gboolean _on_table(hashstr_t *k, GTree *rows, gpointer u0 UNUSED) {
 		if (g_tree_nnodes(rows) > 0) {
-			if (0 != strcmp(hashstr_str(k), "main.admin")) {
-				gsize max = sizeof("version:")+hashstr_len(k);
-				gchar buf[max];
-				g_snprintf(buf, max, "version:%s", hashstr_str(k));
-				sqlx_admin_inc_version(ctx->sq3, buf, 1);
-			}
+			if (0 == strcmp("main.admin", hashstr_str(k)))
+				admin_changed = TRUE;
+			gchar buf[256];
+			g_snprintf(buf, sizeof(buf), "version:%s", hashstr_str(k));
+			sqlx_admin_inc_version(ctx->sq3, buf, 1);
 		}
 		return FALSE;
 	}
 
-	if (ctx->pending)
-		g_tree_foreach(ctx->pending, (GTraverseFunc)_on_table, NULL);
+	EXTRA_ASSERT(NULL != ctx->pending);
+
+	g_tree_foreach(ctx->pending, (GTraverseFunc)_on_table, NULL);
+
+	/* ... so, if the admin table didn't receive an increment yet, we do it
+	 * now */
+	if (!admin_changed)
+		sqlx_admin_inc_version(ctx->sq3, "version:main.admin", 1);
 }
 
 static void
@@ -269,7 +270,7 @@ context_pending_to_rowset(sqlite3 *db, struct sqlx_repctx_s *ctx)
 			GRID_TRACE2("%s(%s,%"G_GINT64_FORMAT",%d)", __FUNCTION__,
 					hashstr_str(name), rowid, deleted);
 
-			struct Row *row = calloc(1, sizeof(*row));
+			struct Row *row = ASN1C_CALLOC(1, sizeof(*row));
 			asn_int64_to_INTEGER(&(row->rowid), rowid);
 			if (!deleted)
 				load_table_row(db, name, rowid, row, table);
@@ -280,7 +281,7 @@ context_pending_to_rowset(sqlite3 *db, struct sqlx_repctx_s *ctx)
 
 		GRID_TRACE2("%s(%s,%p)", __FUNCTION__, hashstr_str(name), rows);
 
-		table = calloc(1, sizeof(struct Table));
+		table = ASN1C_CALLOC(1, sizeof(struct Table));
 		OCTET_STRING_fromBuf(&(table->name),
 				hashstr_str(name), hashstr_len(name));
 		g_tree_foreach(rows, _on_row, NULL);
@@ -301,20 +302,18 @@ context_pending_to_rowset(sqlite3 *db, struct sqlx_repctx_s *ctx)
 static GError*
 _replicate_on_peers(gchar **peers, struct sqlx_repctx_s *ctx)
 {
-	guint count_errors = 0, count_success = 0;
+	guint count_success = 0;
 
-	dump_request(__FUNCTION__, peers, "SQLX_REPLICATE",
-			sqlx_name_mutable_to_const(&ctx->sq3->name));
+	NAME2CONST(n, ctx->sq3->name);
+	dump_request(__FUNCTION__, peers, "SQLX_REPLICATE", &n);
 
-	GByteArray *encoded = sqlx_pack_REPLICATE(
-			sqlx_name_mutable_to_const(&ctx->sq3->name),
-			&(ctx->sequence));
+	GByteArray *encoded = sqlx_pack_REPLICATE(&n, &(ctx->sequence));
 	struct gridd_client_s **clients =
 		gridd_client_create_many(peers, encoded, NULL, NULL);
 	g_byte_array_unref(encoded);
 
-	gridd_clients_set_timeout_cnx(clients, SQLX_REPLI_TIMEOUT_CNX);
-	gridd_clients_set_timeout(clients, SQLX_REPLI_TIMEOUT);
+	gridd_clients_set_timeout_cnx(clients, oio_election_replicate_timeout_cnx);
+	gridd_clients_set_timeout(clients, oio_election_replicate_timeout_req);
 
 	gridd_clients_start(clients);
 	GError *err = gridd_clients_loop(clients);
@@ -336,14 +335,11 @@ _replicate_on_peers(gchar **peers, struct sqlx_repctx_s *ctx)
 					// whatever the remote problem on *that* peer, the it
 					// is MASTER because the election succeeded, and we won't
 					// restart a whole election
-					++ count_success;
 					g_ptr_array_add(ctx->resync_todo, g_strdup(
 							gridd_client_url(*pc)));
-				} else {
-					++ count_errors;
-					g_string_append_printf (ctx->errors, " [%s/%d/%s]",
-							gridd_client_url(*pc), e->code, e->message);
 				}
+				g_string_append_printf(ctx->errors, " [%s/%d/%s]",
+						gridd_client_url(*pc), e->code, e->message);
 				g_clear_error(&e);
 			}
 		}
@@ -352,11 +348,11 @@ _replicate_on_peers(gchar **peers, struct sqlx_repctx_s *ctx)
 		guint groupsize = 1 + g_strv_length(peers);
 		if (election_manager_get_mode(ctx->sq3->manager) == ELECTION_MODE_GROUP) {
 			if (count_success < groupsize)
-				err = SYSERR("Not enough successes, no group");
+				err = NEWERROR(CODE_UNAVAILABLE, "Not enough successes, no group");
 		}
 		else {
 			if (count_success < group_to_quorum(groupsize))
-				err = SYSERR("Not enough successes, no quorum");
+				err = NEWERROR(CODE_UNAVAILABLE, "Not enough successes, no quorum");
 		}
 	}
 
@@ -367,22 +363,22 @@ _replicate_on_peers(gchar **peers, struct sqlx_repctx_s *ctx)
 static void
 _defer_synchronous_RESYNC(struct sqlx_repctx_s *ctx)
 {
-	gchar **peers = NULL;
+	NAME2CONST(n, ctx->sq3->name);
 
-	GError *err = election_get_peers (ctx->sq3->manager,
-			sqlx_name_mutable_to_const(&ctx->sq3->name), FALSE, &peers);
+	gchar **peers = NULL;
+	GError *err = election_get_peers (ctx->sq3->manager, &n, FALSE, &peers);
+	EXTRA_ASSERT((err != NULL) ^ (peers != NULL));
 
 	if (err != NULL) {
 		GRID_WARN("Replicated transaction started but peers not found "
 				"[%s][%s]: (%d) %s", ctx->sq3->name.base, ctx->sq3->name.type,
 				err->code, err->message);
 		g_clear_error(&err);
-		return;
-	}
-
-	if (peers) {
-		for (gchar **p=peers; *p ;++p)
+	} else if (peers) {
+		for (gchar **p=peers; *p ;++p) {
 			g_ptr_array_add(ctx->resync_todo, *p);
+			*p = NULL;
+		}
 		g_free(peers);
 	}
 }
@@ -390,29 +386,32 @@ _defer_synchronous_RESYNC(struct sqlx_repctx_s *ctx)
 static int
 _perform_REPLICATE(struct sqlx_repctx_s *ctx)
 {
-	gchar **peers = NULL;
+	NAME2CONST(n, ctx->sq3->name);
 
-	GError *err = election_get_peers (ctx->sq3->manager,
-			sqlx_name_mutable_to_const(&ctx->sq3->name), FALSE, &peers);
+	gchar **peers = NULL;
+	GError *err = election_get_peers (ctx->sq3->manager, &n, FALSE, &peers);
+	EXTRA_ASSERT((err != NULL) ^ (peers != NULL));
 
 	if (err != NULL) {
-		GRID_WARN("Replicated transaction started but peers not found "
-				"[%s][%s]: (%d) %s", ctx->sq3->name.base,
-				ctx->sq3->name.type, err->code, err->message);
+		GRID_WARN("Replicated transaction started but peers not found [%s.%s]"
+				": (%d) %s", ctx->sq3->name.base, ctx->sq3->name.type,
+				err->code, err->message);
 		g_clear_error(&err);
 		return 1;
 	}
 
-	if (!peers) {
-		GRID_DEBUG("No peer located, no replication to do");
-		return 0;
+	if (!peers || !oio_str_is_set(*peers)) {
+		GRID_WARN("Replication triggered but no peer found for [%s.%s]",
+				 ctx->sq3->name.base, ctx->sq3->name.type);
+		oio_str_cleanv(&peers);
+		return 1;
 	}
 
 	err = _replicate_on_peers(peers, ctx);
 	g_strfreev(peers);
 	context_flush_rowsets(ctx);
 
-	if (!err)
+	if (likely(err == NULL))
 		return 0;
 
 	GRID_WARN("%s(%p) FAILED: (%d) %s", __FUNCTION__, ctx, err->code,
@@ -475,7 +474,9 @@ hook_update(struct sqlx_repctx_s *ctx, int op, char const *bn, char const *tn,
 		g_free(n);
 	} while (0);
 
-	ctx->changes ++;
+	++ ctx->changes;
+	ctx->local_changes = 1;
+
 	GTree *subtree = context_get_pending_table(ctx->pending, key);
 	guint u = (op == SQLITE_DELETE);
 	g_tree_replace(subtree, g_memdup(&rowid, sizeof(rowid)),
@@ -499,7 +500,8 @@ sqlx_synchronous_resync(struct sqlx_repctx_s *ctx, gchar **peers)
 	}
 
 	// Now send it to the SLAVES
-	peers_restore(peers, sqlx_name_mutable_to_const(&ctx->sq3->name), dump);
+	NAME2CONST(n, ctx->sq3->name);
+	peers_restore(peers, &n, dump);
 	GRID_INFO("RESTORED on SLAVES [%s][%s]", ctx->sq3->name.base,
 			ctx->sq3->name.type);
 }
@@ -540,9 +542,9 @@ sqlx_transaction_prepare(struct sqlx_sqlite3_s *sq3,
 
 	if (!sq3->no_peers &&
 			sqlx_repository_replication_configured(sq3->repo)) {
+		NAME2CONST(n, sq3->name);
 		GError *err = election_has_peers(
-				sqlx_repository_get_elections_manager(sq3->repo),
-				sqlx_name_mutable_to_const(&sq3->name), FALSE, &has);
+				sqlx_repository_get_elections_manager(sq3->repo), &n, FALSE, &has);
 		if (err != NULL) {
 			g_prefix_error(&err, "Peer resolution: ");
 			return err;
@@ -553,6 +555,7 @@ sqlx_transaction_prepare(struct sqlx_sqlite3_s *sq3,
 	repctx->hollow = !has;
 	repctx->sq3 = sq3;
 	repctx->changes = sqlite3_total_changes(sq3->db);
+	repctx->local_changes = 0;
 
 	if (has) {
 		repctx->resync_todo = g_ptr_array_sized_new(4);
@@ -566,7 +569,7 @@ sqlx_transaction_prepare(struct sqlx_sqlite3_s *sq3,
 		sqlite3_update_hook(sq3->db, (sqlite3_update_hook_f)hook_update, repctx);
 	}
 
-	repctx->errors = g_string_new ("");
+	repctx->errors = g_string_sized_new (128);
 
 	*result = repctx;
 	return NULL;
@@ -630,12 +633,16 @@ sqlx_transaction_end(struct sqlx_repctx_s *ctx, GError *err)
 					sqlite_strerror(rc), sqlite3_errmsg(ctx->sq3->db));
 		}
 		sqlx_admin_reload(ctx->sq3);
-	}
-	else {
+	} else {
 		/* Ensure that newly created tables have versions now referenced
 		 * in the admin table. */
 		sqlx_admin_ensure_versions (ctx->sq3);
 
+		/* Ensure any operation happening during the request handler on the
+		 * admin table, will be stored in the replication context and will
+		 * trigger an increment in the version numbers. */
+		sqlx_admin_save_lazy (ctx->sq3);
+
 		/* Special management for the big changes (it is big when we could
 		 * not capture each row change, so that we need to trigger a whole
 		 * resync. */
@@ -652,7 +659,7 @@ sqlx_transaction_end(struct sqlx_repctx_s *ctx, GError *err)
 		if (!ctx->hollow) {
 			if (ctx->huge) {
 				sqlx_admin_inc_all_versions(ctx->sq3, 2);
-			} else if (ctx->changes) {
+			} else if (ctx->local_changes) {
 				context_pending_inc_versions(ctx);
 			}
 		}
@@ -670,22 +677,20 @@ sqlx_transaction_end(struct sqlx_repctx_s *ctx, GError *err)
 		/* Apply the changes on the slaves. */
 		rc = sqlx_exec(ctx->sq3->db, "COMMIT");
 		if (rc != SQLITE_OK && rc != SQLITE_DONE) {
-			err = NEWERROR(rc, "(%s) %s%s", sqlite_strerror(rc),
-					sqlite3_errmsg(ctx->sq3->db), ctx->errors->str);
-			g_prefix_error(&err, "COMMIT failed: ");
+			err = NEWERROR(CODE_UNAVAILABLE, "COMMIT failed: (%s) %s%s",
+					sqlite_strerror(rc), sqlite3_errmsg(ctx->sq3->db),
+					ctx->errors->str);
 			// Restore the in-RAM cache
 			sqlx_admin_reload(ctx->sq3);
 		}
-		else {
-			if (ctx->errors->len > 0) {
-				GRID_WARN("COMMIT errors on [%s.%s]:%s", ctx->sq3->name.base,
-						ctx->sq3->name.type, ctx->errors->str);
-			}
-			if (ctx->resync_todo && ctx->resync_todo->len) {
-				// Detected the need of an explicit RESYNC on some SLAVES.
-				g_ptr_array_add(ctx->resync_todo, NULL);
-				sqlx_synchronous_resync(ctx, (gchar**)ctx->resync_todo->pdata);
-			}
+		if (ctx->errors->len > 0) {
+			GRID_WARN("COMMIT errors on [%s.%s]:%s", ctx->sq3->name.base,
+					ctx->sq3->name.type, ctx->errors->str);
+		}
+		if (ctx->resync_todo && ctx->resync_todo->len) {
+			// Detected the need of an explicit RESYNC on some SLAVES.
+			g_ptr_array_add(ctx->resync_todo, NULL);
+			sqlx_synchronous_resync(ctx, (gchar**)ctx->resync_todo->pdata);
 		}
 	}
 
