@@ -1,7 +1,7 @@
 /*
 OpenIO SDS sqliterepo
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -21,21 +21,10 @@ License along with this library.
 #include <sqlite3.h>
 
 #include <metautils/lib/metautils.h>
-#include <metautils/lib/metacomm.h>
+#include <metautils/lib/codec.h>
+
 #include <server/transport_gridd.h>
-
-#include <RowFieldValue.h>
-#include <RowField.h>
-#include <RowFieldSequence.h>
-#include <Row.h>
-#include <RowSet.h>
-#include <RowName.h>
-#include <TableHeader.h>
-#include <Table.h>
-#include <TableSequence.h>
-
-#include <asn_codecs.h>
-#include <ber_decoder.h>
+#include <sqliterepo/sqliterepo_variables.h>
 
 #include "sqliterepo.h"
 #include "version.h"
@@ -56,29 +45,48 @@ License along with this library.
 	} \
 } while (0)
 
-#define CONST(N) sqlx_name_mutable_to_const(N)
+#define ADMIN "admin"
 
 /* ------------------------------------------------------------------------- */
 
-static gchar *
-_prepare_statement(Table_t *t)
+static void
+version_debug(GTree *current, GTree *expected, GTree *post)
 {
-	gint i;
-	GString *gstr;
+	if (!GRID_TRACE_ENABLED())
+		return;
+	gchar *s0 = version_dump(current); STRING_STACKIFY(s0);
+	gchar *s1 = version_dump(expected); STRING_STACKIFY(s1);
+	gchar *s2 = version_dump(post); STRING_STACKIFY(s2);
+	GRID_TRACE("CURRENT:%s EXPECTED:%s POST:%s", s0, s1, s2);
+}
 
-	gstr = g_string_new("REPLACE INTO ");
+static gchar *
+_prepare_statement(Table_t *t, gboolean is_admin)
+{
+	GString *gstr = g_string_sized_new(256);
+	g_string_append_static(gstr, "REPLACE INTO ");
 	g_string_append_len(gstr, (char*)t->name.buf, t->name.size);
-	g_string_append(gstr, " (ROWID");
-	for (i=0; i < t->header.list.count; i++) {
+	g_string_append_static(gstr, " (");
+	if (!is_admin)
+		g_string_append_static(gstr, "ROWID,");
+
+	for (int i=0; i < t->header.list.count; i++) {
 		RowName_t *r = t->header.list.array[i];
-		g_string_append_c(gstr, ',');
+		if (i > 0)
+			g_string_append_c(gstr, ',');
 		g_string_append_len(gstr, (char*)r->name.buf, r->name.size);
 	}
-	g_string_append(gstr, ") VALUES (?");
-	for (i=0; i < t->header.list.count; i++) {
-		g_string_append_len(gstr, ",?", 2);
+
+	g_string_append_static(gstr, ") VALUES (");
+	if (!is_admin)
+		g_string_append_static(gstr, " ?,");
+
+	for (int i=0; i < t->header.list.count; i++) {
+		if (i > 0)
+			g_string_append_c(gstr, ',');
+		g_string_append_static(gstr, "?");
 	}
-	g_string_append(gstr, ")");
+	g_string_append_c(gstr, ')');
 
 	return g_string_free(gstr, FALSE);
 }
@@ -91,7 +99,14 @@ replicate_table_updates(struct sqlx_sqlite3_s *sq3, Table_t *table)
 	GError *err = NULL;
 	sqlite3_stmt *stmt = NULL;
 
-	sql = _prepare_statement(table);
+	/* if there is no table header, we won't be able to generate a valid
+	 * UPDATE/REPLACE statement. The current table at best only carries
+	 * DELETE commands (with only a ROWID) */
+	if (table->header.list.count <= 0)
+		return NULL;
+
+	gboolean is_admin = !(g_strcmp0((char*)table->name.buf, ADMIN));
+	sql = _prepare_statement(table, is_admin);
 	sqlite3_prepare_debug(rc, sq3->db, sql, -1, &stmt, NULL);
 	g_free(sql);
 
@@ -107,8 +122,8 @@ replicate_table_updates(struct sqlx_sqlite3_s *sq3, Table_t *table)
 			sqlite3_reset(stmt);
 			sqlite3_clear_bindings(stmt);
 
-			sqlite3_bind_int64(stmt, 1, rowid);
-
+			if (!is_admin)
+				sqlite3_bind_int64(stmt, 1, rowid);
 			/* Now apply all the field values */
 			for (j=0; j<row->fields->list.count ;j++) {
 				int pos = 0;
@@ -117,7 +132,7 @@ replicate_table_updates(struct sqlx_sqlite3_s *sq3, Table_t *table)
 
 				field = row->fields->list.array[j];
 				asn_INTEGER2long(&(field->pos), &lpos);
-				pos = lpos + 2;
+				pos = lpos + (is_admin? 1 : 2);
 				switch (field->value.present) {
 					case RowFieldValue_PR_NOTHING:
 					case RowFieldValue_PR_n:
@@ -318,9 +333,7 @@ replicate_body_manage(struct sqlx_sqlite3_s *sq3, TableSequence_t *seq)
 	GTree *oldvers, *expected_version, *postvers;
 
 	oldvers = version_extract_from_admin(sq3);
-	version_debug("CURRENT:", oldvers);
 	expected_version = version_extract_expected(oldvers, seq);
-	version_debug("EXPECTED:", expected_version);
 	postvers = NULL;
 
 	sqlx_exec(sq3->db, "BEGIN");
@@ -343,6 +356,7 @@ label_rollback:
 		/* keep the current version for later */
 		sqlx_admin_reload(sq3);
 		postvers = version_extract_from_admin(sq3);
+		version_debug(oldvers, expected_version, postvers);
 
 		gint64 worst = 0;
 		err = version_validate_diff(postvers, expected_version, &worst);
@@ -593,37 +607,27 @@ enum query_action_e
 	QA_ROLLBACK  = 0x20,
 };
 
-static gboolean
-__is_in_array(register const gchar **p, register const gchar *needle)
-{
-	while (*p) {
-		if (!g_ascii_strcasecmp(needle, *(p++)))
-			return TRUE;
-	}
-	return FALSE;
-}
-
 static gboolean
 _pragma_is_allowed(const gchar *pragma)
 {
-	const gchar *allowed_pragmas[] = {
+	static const gchar * const allowed_pragmas[] = {
 		"table_info", "index_info", "index_list",
 		"page_count", "quick_check", "collation_list",
 		"database_list", "freelist_count",
 		NULL
 	};
-	return pragma ? __is_in_array(allowed_pragmas, pragma) : TRUE;
+	return pragma ? oio_strv_has(allowed_pragmas, pragma) : TRUE;
 }
 
 static gboolean
 _function_is_allowed(const gchar *func)
 {
-	const gchar *forbidden_funcs[] = {
+	static const gchar * const forbidden_funcs[] = {
 		"load_extension", "sqlite_compileoption_get",
 		"sqlite_compileoption_get", "sqlite_source_id",
 		NULL
 	};
-	return func ? !__is_in_array(forbidden_funcs, func) : TRUE;
+	return func ? ! oio_strv_has(forbidden_funcs, func) : TRUE;
 }
 
 enum query_rights_e
@@ -742,7 +746,7 @@ _table_set_error(Table_t *table, GError *err,
 {
 	void _reset_integer (INTEGER_t **pi, gint64 v) {
 		ASN_STRUCT_FREE(asn_DEF_INTEGER, *pi);
-		*pi = calloc(1, sizeof(INTEGER_t));
+		*pi = ASN1C_CALLOC(1, sizeof(INTEGER_t));
 		asn_int64_to_INTEGER(*pi, v);
 	}
 
@@ -1012,7 +1016,7 @@ _execute_next_query(struct sqlx_sqlite3_s *sq3, const gchar *query,
 			for (rc = SQLITE_ROW; rc == SQLITE_ROW ;) {
 				rc = sqlite3_step(stmt);
 				if (rc == SQLITE_ROW) {
-					struct Row *rrow = calloc(1, sizeof(struct Row));
+					struct Row *rrow = ASN1C_CALLOC(1, sizeof(struct Row));
 					load_statement(stmt, rrow, result);
 					asn_sequence_add(&(result->rows.list), rrow);
 				}
@@ -1030,7 +1034,7 @@ _execute_next_query(struct sqlx_sqlite3_s *sq3, const gchar *query,
 		for (rc = SQLITE_ROW; rc == SQLITE_ROW ;) {
 			rc = sqlite3_step(stmt);
 			if (rc == SQLITE_ROW) {
-				struct Row *rrow = calloc(1, sizeof(struct Row));
+				struct Row *rrow = ASN1C_CALLOC(1, sizeof(struct Row));
 				load_statement(stmt, rrow, result);
 				asn_sequence_add(&(result->rows.list), rrow);
 			}
@@ -1114,8 +1118,8 @@ do_query_after_open(struct gridd_reply_ctx_s *reply_ctx,
 		g_free(query);
 
 		if (0 != action) {
-			err = sqlx_repository_status_base(sq3->repo,
-					sqlx_name_mutable_to_const(&sq3->name));
+			NAME2CONST(n, sq3->name);
+			err = sqlx_repository_status_base(sq3->repo, &n);
 			if (NULL != err) {
 				if (err->code != CODE_REDIRECT)
 					g_prefix_error(&err, "Status error: ");
@@ -1136,7 +1140,7 @@ do_query_after_open(struct gridd_reply_ctx_s *reply_ctx,
 		gchar *query;
 
 		req = params->list.array[i32];
-		res = calloc(1, sizeof(struct Table));
+		res = ASN1C_CALLOC(1, sizeof(struct Table));
 		query = _table_to_query(req);
 		replication_ctx = reply_ctx->get_cnx_data("repctx");
 		reply_ctx->forget_cnx_data("repctx");
@@ -1193,7 +1197,8 @@ _check_init_flag(struct sqlx_sqlite3_s *sq3, gboolean autocreate)
 					autocreate? "on":"off", SQLX_ADMIN_INITFLAG);
 			err = NEWERROR(CODE_CONTAINER_NOTFOUND, "Base does not exist");
 		} else {
-			err = sqlx_repository_status_base(sq3->repo, sqlx_name_mutable_to_const(&sq3->name));
+			NAME2CONST(n, sq3->name);
+			err = sqlx_repository_status_base(sq3->repo, &n);
 			if (!err) { /* We are master */
 				struct sqlx_repctx_s *repctx = NULL;
 				GRID_DEBUG("Autocreate %s, inserting %s flag",
@@ -1210,20 +1215,18 @@ _check_init_flag(struct sqlx_sqlite3_s *sq3, gboolean autocreate)
 static GError *
 do_query(struct gridd_reply_ctx_s *reply_ctx, sqlx_repository_t *repo,
 		struct sqlx_name_s *name,
-		TableSequence_t *params, TableSequence_t *result,
-		gboolean autocreate)
+		TableSequence_t *params, TableSequence_t *result)
 {
 	GError *err = NULL;
 	struct sqlx_sqlite3_s *sq3 = NULL;
 
-	GRID_DEBUG("Opening and querying [%s][%s]%s", name->base, name->type,
-			autocreate? " (autocreate)" : "");
+	GRID_DEBUG("Opening and querying [%s][%s]", name->base, name->type);
 
-	err = _checked_open(reply_ctx, repo, name, SQLX_OPEN_MASTERSLAVE, &sq3);
+	err = _checked_open(reply_ctx, repo, name, SQLX_OPEN_CREATE|SQLX_OPEN_MASTERSLAVE, &sq3);
 	if (err != NULL)
 		return err;
 
-	err = _check_init_flag(sq3, autocreate);
+	err = _check_init_flag(sq3, TRUE);
 	if (!err)
 		err = do_query_after_open(reply_ctx, sq3, params, result);
 
@@ -1240,7 +1243,6 @@ do_destroy(struct gridd_reply_ctx_s *reply, struct sqlx_repository_s *repo,
 		struct sqlx_name_s *name, gboolean local)
 {
 	GError *err = NULL;
-	gchar **peers = NULL;
 	struct sqlx_sqlite3_s *sq3 = NULL;
 
 	GRID_DEBUG("Opening for destruction [%s][%s] (%s)",
@@ -1251,13 +1253,15 @@ do_destroy(struct gridd_reply_ctx_s *reply, struct sqlx_repository_s *repo,
 		return err;
 
 	if (!local) {
+		gchar **peers = NULL;
 		err = election_get_peers(sq3->manager, name, FALSE, &peers);
-		if (err)
+		if (err) {
+			EXTRA_ASSERT(peers == NULL);
 			goto end_label;
-		if (NULL != peers) {
+		} else {
+			EXTRA_ASSERT(peers != NULL);
 			err = sqlx_remote_execute_DESTROY_many(peers, NULL, name);
 			g_strfreev(peers);
-			peers = NULL;
 		}
 	}
 
@@ -1275,7 +1279,6 @@ end_label:
 
 /* ------------------------------------------------------------------------- */
 
-#define FLAG_AUTOCREATE 0x01
 #define FLAG_LOCAL      0x02
 #define FLAG_NOCHECK    0x08
 #define FLAG_CHUNKED    0x10
@@ -1283,16 +1286,16 @@ end_label:
 
 static GError *
 _load_sqlx_name (struct gridd_reply_ctx_s *ctx,
-		struct sqlx_name_mutable_s *n, guint32 *pflags)
+		struct sqlx_name_inline_s *n, guint32 *pflags)
 {
 	GError *err;
 	gchar
 		ns[LIMIT_LENGTH_NSNAME],
 		base[LIMIT_LENGTH_BASENAME],
 		type[LIMIT_LENGTH_BASETYPE];
-	gboolean flush, nocheck, local, autocreate, chunked;
+	gboolean flush, nocheck, local, chunked;
 
-	flush = local = autocreate = nocheck = chunked = FALSE;
+	flush = local = nocheck = chunked = FALSE;
 
 	err = metautils_message_extract_string(ctx->request,
 			NAME_MSGKEY_NAMESPACE, ns, sizeof(ns));
@@ -1309,8 +1312,6 @@ _load_sqlx_name (struct gridd_reply_ctx_s *ctx,
 
 	local = metautils_message_extract_flag(ctx->request,
 			NAME_MSGKEY_LOCAL, FALSE);
-	autocreate = metautils_message_extract_flag(ctx->request,
-			NAME_MSGKEY_AUTOCREATE, FALSE);
 	nocheck = metautils_message_extract_flag(ctx->request,
 			NAME_MSGKEY_NOCHECK, FALSE);
 	chunked = metautils_message_extract_flag(ctx->request,
@@ -1320,12 +1321,13 @@ _load_sqlx_name (struct gridd_reply_ctx_s *ctx,
 
 	ctx->subject("%s.%s%s", base, type, local?"|LOCAL":"");
 
-	oio_str_replace(&n->ns, ns);
-	oio_str_replace(&n->base, base);
-	oio_str_replace(&n->type, type);
+	memset(n, 0, sizeof(*n));
+	g_strlcpy(n->ns, ns, sizeof(n->ns));
+	g_strlcpy(n->base, base, sizeof(n->base));
+	g_strlcpy(n->type, type, sizeof(n->type));
+
 	if (pflags) {
 		*pflags = 0;
-		*pflags |= (autocreate ? FLAG_AUTOCREATE : 0);
 		*pflags |= (local ? FLAG_LOCAL : 0);
 		*pflags |= (nocheck ? FLAG_NOCHECK : 0);
 		*pflags |= (chunked ? FLAG_CHUNKED : 0);
@@ -1343,25 +1345,24 @@ _handler_GETVERS(struct gridd_reply_ctx_s *reply,
 	struct sqlx_sqlite3_s *sq3 = NULL;
 	GTree *version = NULL;
 	GError *err = NULL;
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 
 	(void) ignored;
 	if (NULL != (err = _load_sqlx_name(reply, &name, NULL))) {
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
-	/* TODO JFS : trigger an election, useful to reduce the number of
-	   messages during an election. */
-	if (NULL != (err = sqlx_repository_use_base(repo, CONST(&name)))) {
+	/* Kickoff the election, we are already involved in it... */
+	if (NULL != (err = sqlx_repository_use_base(repo, &n0, TRUE, NULL))) {
 		g_prefix_error(&err, "Use: ");
 		reply->send_error(0, err);
 		return TRUE;
 	}
 
-	err = sqlx_repository_open_and_lock(repo, CONST(&name),
-			SQLX_OPEN_LOCAL|SQLX_OPEN_URGENT, &sq3, NULL);
+	err = sqlx_repository_open_and_lock(repo, &n0,
+			SQLX_OPEN_CREATE|SQLX_OPEN_LOCAL|SQLX_OPEN_URGENT, &sq3, NULL);
 	if (NULL != err) {
 		g_prefix_error(&err, "Open/lock: ");
 		reply->send_error(0, err);
@@ -1369,10 +1370,11 @@ _handler_GETVERS(struct gridd_reply_ctx_s *reply,
 	}
 
 	err = sqlx_repository_get_version(sq3, &version);
+	sqlx_repository_unlock_and_close_noerror(sq3);
+
 	if (NULL != err) {
 		reply->send_error(0, err);
-	}
-	else {
+	} else {
 		GByteArray *encoded = version_encode(version);
 		if (!encoded) {
 			err = NEWERROR(CODE_INTERNAL_ERROR, "Encoding error (version)");
@@ -1384,7 +1386,6 @@ _handler_GETVERS(struct gridd_reply_ctx_s *reply,
 		}
 	}
 
-	sqlx_repository_unlock_and_close_noerror(sq3);
 	if (version)
 		g_tree_destroy(version);
 	return TRUE;
@@ -1395,7 +1396,8 @@ _handler_REPLICATE(struct gridd_reply_ctx_s *reply,
 		struct sqlx_repository_s *repo, gpointer ignored)
 {
 	struct sqlx_sqlite3_s *sq3 = NULL;
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 	GError *err = NULL;
 
 	(void) ignored;
@@ -1403,7 +1405,6 @@ _handler_REPLICATE(struct gridd_reply_ctx_s *reply,
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
 	gsize bsize = 0;
 	void *b = metautils_message_get_BODY(reply->request, &bsize);
@@ -1417,15 +1418,19 @@ _handler_REPLICATE(struct gridd_reply_ctx_s *reply,
 	/* Starts an election without being an initiator ... because I receive
 	 * this request from a master, so an election is already running
 	 * somewhere else. */
-	err = sqlx_repository_use_base(repo, CONST(&name));
+	err = sqlx_repository_use_base(repo, &n0, FALSE, NULL);
 	if (NULL != err) {
 		reply->send_error(0, err);
 		return TRUE;
 	}
 
-	err = sqlx_repository_open_and_lock(repo, CONST(&name),
-			SQLX_OPEN_LOCAL|SQLX_OPEN_CREATE|SQLX_OPEN_URGENT, &sq3, NULL);
+	err = sqlx_repository_open_and_lock(repo, &n0,
+			SQLX_OPEN_LOCAL|SQLX_OPEN_URGENT, &sq3, NULL);
 	if (NULL != err) {
+		if (err->code == CODE_CONTAINER_NOTFOUND) {
+			reply->send_error(CODE_PIPEFROM, err);
+			return TRUE;
+		}
 		reply->send_error(0, err);
 		return TRUE;
 	}
@@ -1448,16 +1453,16 @@ _handler_HAS(struct gridd_reply_ctx_s *reply,
 {
     GError *err = NULL;
     gchar *bddname=NULL;
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 
     (void) ignored;
 	if (NULL != (err = _load_sqlx_name(reply, &name, NULL))) {
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
-    if (NULL != (err = sqlx_repository_has_base2(repo, CONST(&name), &bddname))) {
+    if (NULL != (err = sqlx_repository_has_base2(repo, &n0, &bddname))) {
 		reply->send_error(0, err);
 	} else {
 		if (bddname) {
@@ -1477,16 +1482,16 @@ _handler_STATUS(struct gridd_reply_ctx_s *reply,
 		struct sqlx_repository_s *repo, gpointer ignored)
 {
 	GError *err;
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 
 	(void) ignored;
 	if (NULL != (err = _load_sqlx_name(reply, &name, NULL))) {
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
-	if (NULL != (err = sqlx_repository_status_base(repo, CONST(&name))))
+	if (NULL != (err = sqlx_repository_status_base(repo, &n0)))
 		reply->send_error(0, err);
 	else
 		reply->send_reply(CODE_FINAL_OK, "MASTER");
@@ -1499,16 +1504,16 @@ _handler_ISMASTER(struct gridd_reply_ctx_s *reply,
         struct sqlx_repository_s *repo, gpointer ignored)
 {
 	GError *err;
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 
 	(void) ignored;
 	if (NULL != (err = _load_sqlx_name(reply, &name, NULL))) {
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
-	err = sqlx_repository_status_base(repo, CONST(&name));
+	err = sqlx_repository_status_base(repo, &n0);
 	if (NULL == err)
 		reply->send_reply(CODE_FINAL_OK, "MASTER");
 	else {
@@ -1528,19 +1533,21 @@ _handler_DESCR(struct gridd_reply_ctx_s *reply,
 		struct sqlx_repository_s *repo, gpointer ignored)
 {
 	GError *err = NULL;
-	struct sqlx_name_mutable_s name = {0};
-	gchar descr[1024] = "?";
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 
 	(void) ignored;
 	if (NULL != (err = _load_sqlx_name(reply, &name, NULL))) {
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
+	/* Tests made with short base names show JSON lengths around 1780 bytes,
+	 * so 2048 should be enough for most names. */
+	GString *body = g_string_sized_new(2048);
 	election_manager_whatabout(sqlx_repository_get_elections_manager(repo),
-			CONST(&name), descr, sizeof(descr));
-	reply->add_body(metautils_gba_from_string(descr));
+			&n0, body);
+	reply->add_body(g_bytes_unref_to_array(g_string_free_to_bytes(body)));
 	reply->send_reply(CODE_FINAL_OK, "OK");
 	return TRUE;
 }
@@ -1550,16 +1557,16 @@ _handler_USE(struct gridd_reply_ctx_s *reply,
 		struct sqlx_repository_s *repo, gpointer ignored)
 {
 	GError *err;
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 
 	(void) ignored;
 	if (NULL != (err = _load_sqlx_name(reply, &name, NULL))) {
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
-	if (NULL != (err = sqlx_repository_use_base(repo, CONST(&name))))
+	if (NULL != (err = sqlx_repository_use_base(repo, &n0, TRUE, NULL)))
 		reply->send_error(0, err);
 	else
 		reply->send_reply(CODE_FINAL_OK, "OK");
@@ -1571,16 +1578,16 @@ _handler_EXIT(struct gridd_reply_ctx_s *reply,
 		struct sqlx_repository_s *repo, gpointer ignored)
 {
 	GError *err;
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 
 	(void) ignored;
 	if (NULL != (err = _load_sqlx_name(reply, &name, NULL))) {
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
-	if (NULL != (err = sqlx_repository_exit_election(repo, CONST(&name))))
+	if (NULL != (err = sqlx_repository_exit_election(repo, &n0)))
 		reply->send_error(0, err);
 	else
 		reply->send_reply(CODE_FINAL_OK, "OK");
@@ -1594,19 +1601,19 @@ _handler_PIPETO(struct gridd_reply_ctx_s *reply,
 	GError *err = NULL;
 	GByteArray *dump = NULL;
 	gchar target[STRLEN_ADDRINFO];
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 
 	(void) ignored;
 	if (NULL != (err = _load_sqlx_name(reply, &name, NULL))) {
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 	EXTRACT_STRING("DST", target);
 	reply->subject("%s.%s|%s", name.base, name.type, target);
 
 	/* Dump the base in a locked manner */
-	err = _dump(repo, CONST(&name), &dump);
+	err = _dump(repo, &n0, &dump);
 	if (NULL != err) {
 		g_prefix_error(&err, "Dump failed: ");
 		reply->send_error(0, err);
@@ -1616,7 +1623,7 @@ _handler_PIPETO(struct gridd_reply_ctx_s *reply,
 	reply->send_reply(CODE_TEMPORARY, "Dump done");
 
 	/* forward the dump to the target */
-	if (NULL != (err = peer_restore(target, CONST(&name), dump)))
+	if (NULL != (err = peer_restore(target, &n0, dump)))
 		reply->send_error(CODE_INTERNAL_ERROR, err);
 	else
 		reply->send_reply(CODE_FINAL_OK, "OK");
@@ -1629,14 +1636,14 @@ _handler_DUMP(struct gridd_reply_ctx_s *reply,
 {
 	GError *err = NULL;
 	guint32 flags = 0;
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 
 	(void) ignored;
 	if (NULL != (err = _load_sqlx_name(reply, &name, &flags))) {
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
 	void _send_part(GByteArray *part, gint64 remaining)
 	{
@@ -1651,11 +1658,11 @@ _handler_DUMP(struct gridd_reply_ctx_s *reply,
 	}
 
 	if (flags & FLAG_CHUNKED) {
-		err = _dump_chunked(repo, CONST(&name), _send_part);
+		err = _dump_chunked(repo, &n0, _send_part);
 	} else {
 		GByteArray *dump = NULL;
 		/* Open and lock the base */
-		err = _dump(repo, CONST(&name), &dump);
+		err = _dump(repo, &n0, &dump);
 		if (!err) {
 			reply->add_body(dump);
 		}
@@ -1675,14 +1682,14 @@ _handler_RESTORE(struct gridd_reply_ctx_s *reply,
 		struct sqlx_repository_s *repo, gpointer ignored)
 {
 	GError *err;
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 
 	(void) ignored;
 	if (NULL != (err = _load_sqlx_name(reply, &name, NULL))) {
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
 	/* The body is the raw base */
 	gsize dump_size = 0;
@@ -1696,7 +1703,7 @@ _handler_RESTORE(struct gridd_reply_ctx_s *reply,
 		return TRUE;
 	}
 
-	err = _restore(repo, CONST(&name), dump, dump_size);
+	err = _restore(repo, &n0, dump, dump_size);
 	if (NULL != err)
 		reply->send_error(CODE_INTERNAL_ERROR, err);
 	else
@@ -1711,18 +1718,18 @@ _handler_PIPEFROM(struct gridd_reply_ctx_s *reply,
 {
 	GError *err;
 	gchar source[64];
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 
 	(void) ignored;
 	if (NULL != (err = _load_sqlx_name(reply, &name, NULL))) {
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 	EXTRACT_STRING("SRC", source);
 	reply->subject("%s.%s|%s", name.base, name.type, source);
 
-	if (NULL != (err = _pipe_from(source, repo, CONST(&name))))
+	if (NULL != (err = _pipe_from(source, repo, &n0)))
 		reply->send_error(0, err);
 	else
 		reply->send_reply(CODE_FINAL_OK, "OK");
@@ -1737,30 +1744,30 @@ _handler_RESYNC(struct gridd_reply_ctx_s *reply,
 	GError *err = NULL;
 	gboolean has_peers = FALSE;
 	struct election_manager_s *em = NULL;
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 
 	(void) ignored;
 	if (NULL != (err = _load_sqlx_name(reply, &name, NULL))) {
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
 	/* Force refresh of peers from meta1 */
 	em = sqlx_repository_get_elections_manager(repo);
-	err = election_has_peers(em, CONST(&name), TRUE, &has_peers);
+	err = election_has_peers(em, &n0, TRUE, &has_peers);
 	g_clear_error(&err);
 
 	/* Open and lock the base */
-	err = sqlx_repository_use_base(repo, CONST(&name));
+	err = sqlx_repository_use_base(repo, &n0, FALSE, NULL);
 	if (NULL != err) {
 		reply->send_error(0, err);
 		return TRUE;
 	}
 
 	struct sqlx_sqlite3_s *sq3 = NULL;
-	err = sqlx_repository_open_and_lock(repo, CONST(&name),
-			SQLX_OPEN_SLAVEONLY, &sq3, NULL);
+	err = sqlx_repository_open_and_lock(repo, &n0,
+			SQLX_OPEN_CREATE|SQLX_OPEN_SLAVEONLY, &sq3, NULL);
 	if (NULL != err) {
 		reply->send_error(0, err);
 		return TRUE;
@@ -1785,7 +1792,8 @@ _handler_PROPDEL(struct gridd_reply_ctx_s *reply,
 		struct sqlx_repository_s *repo, gpointer ignored)
 {
 	struct sqlx_sqlite3_s *sq3 = NULL;
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 	GError *err = NULL;
 	guint32 flags = 0;
 
@@ -1794,7 +1802,6 @@ _handler_PROPDEL(struct gridd_reply_ctx_s *reply,
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
 	gsize len = 0;
 	void *buf = metautils_message_get_BODY(reply->request, &len);
@@ -1811,10 +1818,9 @@ _handler_PROPDEL(struct gridd_reply_ctx_s *reply,
 	}
 
 	const enum sqlx_open_type_e how = (flags&FLAG_LOCAL)
-		? (SQLX_OPEN_LOCAL|SQLX_OPEN_NOREFCHECK) : SQLX_OPEN_MASTERSLAVE;
-	err = sqlx_repository_open_and_lock(repo, CONST(&name), how, &sq3, NULL);
-	if (NULL != err) {
-		g_prefix_error(&err, "Open/lock: ");
+		? (SQLX_OPEN_LOCAL|SQLX_OPEN_NOREFCHECK) : SQLX_OPEN_MASTERONLY;
+	err = sqlx_repository_open_and_lock(repo, &n0, how, &sq3, NULL);
+	if (err) {
 		reply->send_error(0, err);
 	} else {
 		struct sqlx_repctx_s *repctx = NULL;
@@ -1829,10 +1835,11 @@ _handler_PROPDEL(struct gridd_reply_ctx_s *reply,
 			}
 		}
 
-		if (repctx)
+		if (repctx) {
 			err = sqlx_transaction_end(repctx, err);
-		else
+		} else {
 			sqlx_admin_save_lazy_tnx (sq3);
+		}
 
 		sqlx_repository_unlock_and_close_noerror(sq3);
 		if (err)
@@ -1851,7 +1858,8 @@ _handler_PROPGET(struct gridd_reply_ctx_s *reply,
 		struct sqlx_repository_s *repo, gpointer ignored)
 {
 	struct sqlx_sqlite3_s *sq3 = NULL;
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 	GError *err;
 	guint32 flags = 0;
 
@@ -1861,13 +1869,12 @@ _handler_PROPGET(struct gridd_reply_ctx_s *reply,
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
 	/* Action */
 	const enum sqlx_open_type_e how = (flags&FLAG_LOCAL)
 		? (SQLX_OPEN_LOCAL|SQLX_OPEN_NOREFCHECK) : SQLX_OPEN_MASTERSLAVE;
-	err = sqlx_repository_open_and_lock(repo, CONST(&name), how, &sq3, NULL);
-	if (NULL != err) {
+	err = sqlx_repository_open_and_lock(repo, &n0, how, &sq3, NULL);
+	if (err) {
 		g_prefix_error(&err, "Open/lock: ");
 		reply->send_error(0, err);
 	} else {
@@ -1899,7 +1906,8 @@ _handler_PROPSET(struct gridd_reply_ctx_s *reply,
 		struct sqlx_repository_s *repo, gpointer ignored)
 {
 	struct sqlx_sqlite3_s *sq3 = NULL;
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 	GError *err;
 	guint32 flags = 0;
 
@@ -1909,7 +1917,6 @@ _handler_PROPSET(struct gridd_reply_ctx_s *reply,
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
 	gchar **pairs = NULL;
 	gsize length = 0;
@@ -1934,10 +1941,9 @@ _handler_PROPSET(struct gridd_reply_ctx_s *reply,
 
 	/* Open */
 	const enum sqlx_open_type_e how = (flags&FLAG_LOCAL)
-			? (SQLX_OPEN_LOCAL|SQLX_OPEN_NOREFCHECK) : SQLX_OPEN_MASTERSLAVE;
-	err = sqlx_repository_open_and_lock(repo, CONST(&name), how, &sq3, NULL);
-	if (NULL != err) {
-		g_prefix_error(&err, "Open/lock: ");
+			? (SQLX_OPEN_LOCAL|SQLX_OPEN_NOREFCHECK) : SQLX_OPEN_MASTERONLY;
+	err = sqlx_repository_open_and_lock(repo, &n0, how, &sq3, NULL);
+	if (err) {
 		goto label_exit;
 	}
 
@@ -1957,10 +1963,11 @@ _handler_PROPSET(struct gridd_reply_ctx_s *reply,
 			}
 		}
 
-		if (!(flags&FLAG_LOCAL))
+		if (repctx) {
 			err = sqlx_transaction_end(repctx, err);
-		else
+		} else {
 			sqlx_admin_save_lazy_tnx (sq3);
+		}
 	}
 	sqlx_repository_unlock_and_close_noerror(sq3);
 
@@ -1978,7 +1985,8 @@ _handler_ENABLE(struct gridd_reply_ctx_s *reply,
 		struct sqlx_repository_s *repo, gpointer ignored)
 {
 	struct sqlx_sqlite3_s *sq3 = NULL;
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 	GError *err;
 	guint32 flags = 0;
 
@@ -1987,13 +1995,11 @@ _handler_ENABLE(struct gridd_reply_ctx_s *reply,
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
 	const enum sqlx_open_type_e how = (flags&FLAG_LOCAL)
-		? (SQLX_OPEN_LOCAL|SQLX_OPEN_NOREFCHECK) : SQLX_OPEN_MASTERSLAVE;
-	err = sqlx_repository_open_and_lock(repo, CONST(&name), how, &sq3, NULL);
-	if (NULL != err) {
-		g_prefix_error(&err, "Open/lock: ");
+		? (SQLX_OPEN_LOCAL|SQLX_OPEN_NOREFCHECK) : SQLX_OPEN_MASTERONLY;
+	err = sqlx_repository_open_and_lock(repo, &n0, how, &sq3, NULL);
+	if (err) {
 		reply->send_error(0, err);
 		return TRUE;
 	}
@@ -2007,10 +2013,11 @@ _handler_ENABLE(struct gridd_reply_ctx_s *reply,
 		else
 			err = NEWERROR(CODE_CONTAINER_ENABLED, "Already enabled");
 
-		if (!(flags&FLAG_LOCAL))
+		if (repctx) {
 			err = sqlx_transaction_end(repctx, err);
-		else
+		} else {
 			sqlx_admin_save_lazy_tnx (sq3);
+		}
 	}
 
 	if (NULL != err)
@@ -2027,7 +2034,8 @@ _handler_FREEZE(struct gridd_reply_ctx_s *reply,
 		struct sqlx_repository_s *repo, gpointer ignored)
 {
 	struct sqlx_sqlite3_s *sq3 = NULL;
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 	GError *err;
 	guint32 flags = 0;
 
@@ -2036,13 +2044,11 @@ _handler_FREEZE(struct gridd_reply_ctx_s *reply,
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
 	const enum sqlx_open_type_e how = (flags&FLAG_LOCAL)
-		? (SQLX_OPEN_LOCAL|SQLX_OPEN_NOREFCHECK) : SQLX_OPEN_MASTERSLAVE;
-	err = sqlx_repository_open_and_lock(repo, CONST(&name), how, &sq3, NULL);
-	if (NULL != err) {
-		g_prefix_error(&err, "Open/lock: ");
+		? (SQLX_OPEN_LOCAL|SQLX_OPEN_NOREFCHECK) : SQLX_OPEN_MASTERONLY;
+	err = sqlx_repository_open_and_lock(repo, &n0, how, &sq3, NULL);
+	if (err) {
 		reply->send_error(0, err);
 		return TRUE;
 	}
@@ -2059,10 +2065,11 @@ _handler_FREEZE(struct gridd_reply_ctx_s *reply,
 		else
 			err = NEWERROR(CODE_CONTAINER_DISABLED, "Container disabled");
 
-		if (!(flags&FLAG_LOCAL))
+		if (repctx) {
 			err = sqlx_transaction_end(repctx, err);
-		else
+		} else {
 			sqlx_admin_save_lazy_tnx (sq3);
+		}
 	}
 
 	if (NULL != err)
@@ -2079,7 +2086,8 @@ _handler_DISABLE(struct gridd_reply_ctx_s *reply,
 		struct sqlx_repository_s *repo, gpointer ignored)
 {
 	struct sqlx_sqlite3_s *sq3 = NULL;
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 	GError *err;
 	guint32 flags = 0;
 
@@ -2088,13 +2096,11 @@ _handler_DISABLE(struct gridd_reply_ctx_s *reply,
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
 	const enum sqlx_open_type_e how = (flags&FLAG_LOCAL)
-		? (SQLX_OPEN_LOCAL|SQLX_OPEN_NOREFCHECK) : SQLX_OPEN_MASTERSLAVE;
-	err = sqlx_repository_open_and_lock(repo, CONST(&name), how, &sq3, NULL);
-	if (NULL != err) {
-		g_prefix_error(&err, "Open/lock: ");
+		? (SQLX_OPEN_LOCAL|SQLX_OPEN_NOREFCHECK) : SQLX_OPEN_MASTERONLY;
+	err = sqlx_repository_open_and_lock(repo, &n0, how, &sq3, NULL);
+	if (err) {
 		reply->send_error(0, err);
 		return TRUE;
 	}
@@ -2111,10 +2117,11 @@ _handler_DISABLE(struct gridd_reply_ctx_s *reply,
 		else
 			err = NEWERROR(CODE_CONTAINER_DISABLED, "Container disabled");
 
-		if (!(flags&FLAG_LOCAL))
+		if (repctx) {
 			err = sqlx_transaction_end(repctx, err);
-		else
+		} else {
 			sqlx_admin_save_lazy_tnx (sq3);
+		}
 	}
 
 	if (NULL != err)
@@ -2131,7 +2138,8 @@ _handler_DISABLE2(struct gridd_reply_ctx_s *reply,
 		struct sqlx_repository_s *repo, gpointer ignored)
 {
 	struct sqlx_sqlite3_s *sq3 = NULL;
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 	GError *err;
 	guint32 flags = 0;
 
@@ -2140,13 +2148,11 @@ _handler_DISABLE2(struct gridd_reply_ctx_s *reply,
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
 	const enum sqlx_open_type_e how = (flags&FLAG_LOCAL)
-		? (SQLX_OPEN_LOCAL|SQLX_OPEN_NOREFCHECK) : SQLX_OPEN_MASTERSLAVE;
-	err = sqlx_repository_open_and_lock(repo, CONST(&name), how, &sq3, NULL);
-	if (NULL != err) {
-		g_prefix_error(&err, "Open/lock: ");
+		? (SQLX_OPEN_LOCAL|SQLX_OPEN_NOREFCHECK) : SQLX_OPEN_MASTERONLY;
+	err = sqlx_repository_open_and_lock(repo, &n0, how, &sq3, NULL);
+	if (err) {
 		reply->send_error(0, err);
 		return TRUE;
 	}
@@ -2161,10 +2167,11 @@ _handler_DISABLE2(struct gridd_reply_ctx_s *reply,
 		else
 			sqlx_admin_set_status(sq3, ADMIN_STATUS_DISABLED);
 
-		if (!(flags&FLAG_LOCAL))
+		if (repctx) {
 			err = sqlx_transaction_end(repctx, err);
-		else
+		} else {
 			sqlx_admin_save_lazy_tnx (sq3);
+		}
 	}
 
 	if (NULL != err)
@@ -2183,7 +2190,7 @@ _info_sqlite(GString *gstr)
 {
 	const char *s;
 
-	g_string_append(gstr, "SQLite options:\n");
+	g_string_append_static(gstr, "SQLite options:\n");
 	for (int i=0; NULL != (s = sqlite3_compileoption_get(i)); ++i) {
 		g_string_append_c(gstr, '\t');
 		g_string_append(gstr, s);
@@ -2201,15 +2208,11 @@ _info_repository(struct sqlx_repository_s *r, GString *gstr)
 		g_string_append(gstr, s);
 	}
 
-	g_string_append(gstr, "sqliterepo options:\n");
+	g_string_append_static(gstr, "sqliterepo options:\n");
 	g_string_append_printf(gstr, "\thash: width=%u depth=%u\n",
 			r->hash_width, r->hash_depth);
-	g_string_append_printf(gstr, "\tbases: %u/%u\n",
-			r->bases_count, r->bases_max);
 
-	g_string_append(gstr, "\tflags: ");
-	if (r->flag_autocreate)
-		_append_flag("AUTOCREATE");
+	g_string_append_static(gstr, "\tflags: ");
 	if (r->flag_autovacuum)
 		_append_flag("AUTOVACUUM");
 	if (r->flag_delete_on)
@@ -2224,7 +2227,7 @@ _info_elections(struct sqlx_repository_s *repo, GString *gstr)
 {
 	struct election_counts_s count = election_manager_count(
 			sqlx_repository_get_elections_manager(repo));
-	g_string_append(gstr, "Elections count:\n");
+	g_string_append_static(gstr, "Elections count:\n");
 	g_string_append_printf(gstr, "\ttotal: %u\n", count.total);
 	g_string_append_printf(gstr, "\tnone: %u\n", count.none);
 	g_string_append_printf(gstr, "\tpending: %u\n", count.pending);
@@ -2249,7 +2252,7 @@ _info_replication(struct sqlx_repository_s *repo, GString *gstr)
 		}
 	}
 
-	g_string_append(gstr, "Replication:\n");
+	g_string_append_static(gstr, "Replication:\n");
 	g_string_append_printf(gstr, "\tmode: %s\n",
 			_mode2str(election_manager_get_mode(
 					sqlx_repository_get_elections_manager(repo))));
@@ -2260,7 +2263,7 @@ _info_cache(struct sqlx_repository_s *repo, GString *gstr)
 {
 	struct cache_counts_s count = sqlx_cache_count(
 			sqlx_repository_get_cache(repo));
-	g_string_append(gstr, "Cache count:\n");
+	g_string_append_static(gstr, "Cache count:\n");
 	g_string_append_printf(gstr, "\tmax: %u\n", count.max);
 	g_string_append_printf(gstr, "\thot: %u\n", count.hot);
 	g_string_append_printf(gstr, "\tcold: %u\n", count.cold);
@@ -2297,7 +2300,7 @@ _handler_LEANIFY(struct gridd_reply_ctx_s *reply,
 			NAME_MSGKEY_SIZE, &size);
 	if (err) {
 		g_clear_error (&err);
-		size = SQLITE_RELEASE_SIZE;
+		size = sqliterepo_release_size;
 	}
 	sqlite3_release_memory (size);
 	reply->send_reply(CODE_FINAL_OK, "OK");
@@ -2330,7 +2333,8 @@ _handler_QUERY(struct gridd_reply_ctx_s *reply,
 	GError *err;
 	guint32 flags = 0;
 	TableSequence_t *params = NULL, *result = NULL;
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 
 	/* unpack the parameters */
 	(void) ignored;
@@ -2338,7 +2342,6 @@ _handler_QUERY(struct gridd_reply_ctx_s *reply,
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
 	if (!g_str_has_prefix(name.type, NAME_SRVTYPE_SQLX".") &&
 			strcmp(name.type, NAME_SRVTYPE_SQLX)) {
@@ -2357,9 +2360,8 @@ _handler_QUERY(struct gridd_reply_ctx_s *reply,
 	}
 
 	/* execute the request now */
-	result = calloc(1, sizeof(struct TableSequence));
-	err = do_query(reply, repo, CONST(&name), params,
-			result, flags&FLAG_AUTOCREATE);
+	result = ASN1C_CALLOC(1, sizeof(struct TableSequence));
+	err = do_query(reply, repo, &n0, params, result);
 
 	if (params)
 		asn_DEF_TableSequence.free_struct(&asn_DEF_TableSequence, params, FALSE);
@@ -2395,15 +2397,15 @@ _handler_DESTROY(struct gridd_reply_ctx_s *reply,
 	(void) ignored;
 	GError *err = NULL;
 	guint32 flags = 0;
-	struct sqlx_name_mutable_s name = {0};
+	struct sqlx_name_inline_s name;
+	NAME2CONST(n0, name);
 
 	if (NULL != (err = _load_sqlx_name(reply, &name, &flags))) {
 		reply->send_error(0, err);
 		return TRUE;
 	}
-	SQLXNAME_STACKIFY(name);
 
-	if (NULL != (err = do_destroy(reply, repo, CONST(&name), flags&FLAG_LOCAL)))
+	if (NULL != (err = do_destroy(reply, repo, &n0, flags&FLAG_LOCAL)))
 		reply->send_error(0, err);
 	else
 		reply->send_reply(CODE_FINAL_OK, "OK");
