@@ -1,7 +1,7 @@
 /*
 OpenIO SDS sqliterepo
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2018 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -28,7 +28,7 @@ License along with this library.
 #include <sqlite3.h>
 
 #include <metautils/lib/metautils.h>
-#include <metautils/lib/metacomm.h>
+#include <sqliterepo/sqliterepo_variables.h>
 
 #include "sqliterepo.h"
 #include "hash.h"
@@ -40,17 +40,12 @@ License along with this library.
 #include "restoration.h"
 #include "sqlx_remote.h"
 
+
 #define GSTR_APPEND_SEP(S) do { \
 	if ((S)->str[(S)->len-1]!=G_DIR_SEPARATOR) \
 			g_string_append_c((S), G_DIR_SEPARATOR); \
 } while (0)
 
-static gboolean
-election_manager_configured(const struct election_manager_s *em)
-{
-	return em && (ELECTION_MODE_NONE != election_manager_get_mode (em));
-}
-
 static void
 _close_handle(sqlite3 **pdb)
 {
@@ -116,17 +111,17 @@ label_end:
 static void
 __delete_base(struct sqlx_sqlite3_s *sq3)
 {
-	if (!sq3->path) {
+	if (!sq3->path_inline[0]) {
 		GRID_WARN("DELETE disabled [%s][%s]", sq3->name.base, sq3->name.type);
 		return;
 	}
 
-	if (!unlink(sq3->path))
+	if (!unlink(sq3->path_inline))
 		GRID_DEBUG("DELETE done [%s][%s] (%s)", sq3->name.base,
-			sq3->name.type, sq3->path);
+			sq3->name.type, sq3->path_inline);
 	else
 		GRID_WARN("DELETE failed [%s][%s] (%s) : (%d) %s",
-				sq3->name.base, sq3->name.type, sq3->path,
+				sq3->name.base, sq3->name.type, sq3->path_inline,
 				errno, strerror(errno));
 	sq3->deleted = 0;
 }
@@ -143,7 +138,7 @@ __close_base(struct sqlx_sqlite3_s *sq3)
 			sq3->name.type);
 
 	/* send a vacuum */
-	if (sq3->repo && sq3->repo->flag_autovacuum)
+	if (sq3->repo && sq3->repo->flag_autovacuum && !sq3->deleted)
 		sqlx_exec(sq3->db, "VACUUM");
 
 	sqlx_repository_call_close_callback(sq3);
@@ -151,8 +146,8 @@ __close_base(struct sqlx_sqlite3_s *sq3)
 	/* delete the base */
 	if (sq3->deleted) {
 		if (sq3->repo->election_manager) {
-			GError *err = election_exit(sq3->repo->election_manager,
-					sqlx_name_mutable_to_const(&sq3->name));
+			NAME2CONST(n0, sq3->name);
+			GError *err = election_exit(sq3->repo->election_manager, &n0);
 			if (err) {
 				GRID_WARN("Failed to exit election [%s][%s]: (%d) %s",
 						sq3->name.base, sq3->name.type,
@@ -170,11 +165,9 @@ __close_base(struct sqlx_sqlite3_s *sq3)
 		_close_handle(&(sq3->db));
 
 	/* Clean the structure */
-	sqlx_name_clean(&sq3->name);
-	oio_str_clean (&sq3->path);
+	sq3->path_inline[0] = 0;
 	if (sq3->admin)
 		g_tree_destroy(sq3->admin);
-
 	sq3->bd = -1;
 	SLICE_FREE(struct sqlx_sqlite3_s, sq3);
 }
@@ -322,19 +315,17 @@ sqlx_repository_init(const gchar *vol, const struct sqlx_repo_config_s *cfg,
 
 	repo->schemas = g_tree_new_full(metautils_strcmp3, NULL, g_free, g_free);
 
-	repo->flag_autocreate = BOOL(cfg->flags & SQLX_REPO_AUTOCREATE);
 	repo->flag_autovacuum = BOOL(cfg->flags & SQLX_REPO_VACUUM);
 	repo->flag_delete_on = BOOL(cfg->flags & SQLX_REPO_DELETEON);
 
 	if (!(cfg->flags & SQLX_REPO_NOCACHE)) {
-		repo->cache = sqlx_cache_init(cfg->max_bases ? cfg->max_bases : SQLX_MAX_BASES);
+		repo->cache = sqlx_cache_init();
 		sqlx_cache_set_close_hook(repo->cache,
 				(sqlx_cache_close_hook)__close_base);
 	}
 
 	repo->sync_mode_solo = cfg->sync_solo;
 	repo->sync_mode_repli = cfg->sync_repli;
-	repo->page_size = cfg->page_size;
 
 	repo->locator = _default_locator;
 	repo->locator_data = NULL;
@@ -400,22 +391,6 @@ sqlx_repository_configure_hash(sqlx_repository_t *repo, guint w, guint d)
 			repo->hash_depth, repo->hash_width);
 }
 
-void
-sqlx_repository_configure_maxbases(sqlx_repository_t *repo, guint max)
-{
-	EXTRA_ASSERT(repo != NULL);
-	EXTRA_ASSERT(repo->running);
-	EXTRA_ASSERT(repo->bases_count == 0);
-
-	GRID_TRACE2("%s(%p,%u)", __FUNCTION__, repo, max);
-	EXTRA_ASSERT(max >= 4);
-
-	if (repo->cache)
-		sqlx_cache_set_max_bases(repo->cache, max);
-	repo->bases_count = 0;
-	repo->bases_max = max;
-}
-
 GError*
 sqlx_repository_configure_type(sqlx_repository_t *repo,
 		const char *type, const char *schema)
@@ -434,18 +409,6 @@ sqlx_repository_configure_type(sqlx_repository_t *repo,
 	return NULL;
 }
 
-void
-sqlx_repository_configure_open_timeout(sqlx_repository_t *repo,
-		gint64 timeout)
-{
-	struct sqlx_cache_s *cache = sqlx_repository_get_cache(repo);
-	if (cache) {
-		sqlx_cache_set_open_timeout(cache, timeout);
-	} else {
-		GRID_INFO("Not setting open timeout since there is no cache");
-	}
-}
-
 void
 sqlx_repository_configure_close_callback(sqlx_repository_t *repo,
 		sqlx_repo_close_hook cb, gpointer cb_data)
@@ -609,7 +572,7 @@ _open_fill_args(struct open_args_s *args, struct sqlx_repository_s *repo,
 	args->name.base = n->base;
 	args->name.ns = n->ns;
 
-	GString *fn = g_string_new("");
+	GString *fn = g_string_sized_new(256);
 	repo->locator(repo->locator_data, n, fn);
 	args->realname = hashstr_create_from_gstring(fn);
 	args->realpath = _compute_path_hash(repo, args->realname, NULL);
@@ -659,19 +622,21 @@ sqlx_admin_reload(struct sqlx_sqlite3_s *sq3)
 }
 
 static GError*
-__open_not_cached(struct open_args_s *args, struct sqlx_sqlite3_s **result)
+__open_mkdir(struct open_args_s *args, sqlite3 **db)
 {
 	GError *error = NULL;
-	struct sqlx_sqlite3_s *sq3 = NULL;
 	sqlite3 *handle = NULL;
 	guint attempts = 2;
 	gint rc, flags = 0;
 
+	EXTRA_ASSERT(db != NULL);
+	*db = NULL;
+
 retry:
 	flags |= SQLITE_OPEN_NOMUTEX;
 	flags |= SQLITE_OPEN_PRIVATECACHE;
 	flags |= SQLITE_OPEN_READWRITE;
-	if (args->create || args->repo->flag_autocreate)
+	if (args->create)
 		flags |= SQLITE_OPEN_CREATE;
 	handle = NULL;
 
@@ -684,7 +649,7 @@ retry:
 		case SQLITE_CANTOPEN:
 			GRID_DEBUG("Open soft error [%s] : (%d) %s", args->realpath,
 					rc, sqlite_strerror(rc));
-			if (attempts-- && (args->create || args->repo->flag_autocreate)) {
+			if (attempts-- && args->create) {
 				_close_handle(&handle);
 				if (!(error = __create_directory(args->realpath))) {
 					GRID_TRACE("Directory created, retrying open [%s]", args->realpath);
@@ -705,21 +670,30 @@ retry:
 			return error;
 	}
 
+	*db = handle;
+	return NULL;
+}
+
+static GError*
+__open_not_cached(struct open_args_s *args, struct sqlx_sqlite3_s **result)
+{
+	sqlite3 *handle = NULL;
+	GError *error = __open_mkdir(args, &handle);
+	if (error) return error;
+
 	sqlite3_commit_hook(handle, NULL, NULL);
 	sqlite3_rollback_hook(handle, NULL, NULL);
 	sqlite3_update_hook(handle, NULL, NULL);
 
 	sqlite3_busy_timeout(handle, 30000);
 
-	sq3 = SLICE_NEW0(struct sqlx_sqlite3_s);
+	struct sqlx_sqlite3_s *sq3 = SLICE_NEW0(struct sqlx_sqlite3_s);
 	sq3->db = handle;
 	sq3->bd = -1;
 	sq3->repo = args->repo;
 	sq3->manager = args->repo->election_manager;
-	sq3->name.base = g_strdup(args->name.base);
-	sq3->name.type = g_strdup(args->name.type);
-	sq3->name.ns = g_strdup(args->name.ns);
-	sq3->path = g_strdup(args->realpath);
+	NAMEFILL(sq3->name, args->name);
+	g_strlcpy(sq3->path_inline, args->realpath, sizeof(sq3->path_inline));
 	sq3->admin_dirty = 0;
 	sq3->admin = g_tree_new_full(metautils_strcmp3, NULL, g_free, g_free);
 
@@ -727,10 +701,10 @@ retry:
 	sqlx_exec(handle, "PRAGMA journal_mode = MEMORY");
 	sqlx_exec(handle, "PRAGMA temp_store = MEMORY");
 	if (!_schema_has(sq3->db)) {
-		if (sq3->repo->page_size >= 512) {
+		if (_page_size >= 512) {
 			gchar line[128] = {0};
 			snprintf(line, sizeof(line),
-					"PRAGMA page_size = %u;", sq3->repo->page_size);
+					"PRAGMA page_size = %u;", _page_size);
 			sqlx_exec(sq3->db, line);
 		}
 		sqlx_exec(sq3->db, "PRAGMA synchronous = OFF;");
@@ -784,9 +758,9 @@ __open_maybe_cached(struct open_args_s *args, struct sqlx_sqlite3_s **result)
 		return NULL;
 	}
 
-	GError *e1 = sqlx_cache_unlock_and_close_base(args->repo->cache, bd, FALSE);
+	GError *e1 = sqlx_cache_unlock_and_close_base(args->repo->cache, bd, 0);
 	if (e1) {
-		GRID_WARN("BASE unlock/close error on bd=%d : (%d) %s",
+		GRID_WARN("BASE unlock/close error on bd=%d: (%d) %s",
 				bd, e1->code, e1->message);
 		g_clear_error(&e1);
 	}
@@ -795,6 +769,27 @@ __open_maybe_cached(struct open_args_s *args, struct sqlx_sqlite3_s **result)
 	return e0;
 }
 
+static GError*
+_db_raw_creation(struct open_args_s *args)
+{
+	gint bd = -1;
+	GError *err = sqlx_cache_open_and_lock_base(args->repo->cache, args->realname, TRUE, &bd);
+	if (err) {
+		g_prefix_error(&err, "DB autocreation failed: ");
+		return err;
+	}
+
+	sqlite3 *db = NULL;
+	err = __open_mkdir(args, &db);
+	_close_handle(&db);
+	err = sqlx_cache_unlock_and_close_base(args->repo->cache, bd, 0);
+	if (err) {
+		GRID_WARN("sqlx cache error: (%d) %s", err->code, err->message);
+		g_clear_error(&err);
+	}
+	return NULL;
+}
+
 static GError*
 _open_and_lock_base(struct open_args_s *args, enum election_status_e expected,
 		struct sqlx_sqlite3_s **result, gchar **pmaster)
@@ -802,19 +797,31 @@ _open_and_lock_base(struct open_args_s *args, enum election_status_e expected,
 	GError *err = NULL;
 	enum election_status_e status = 0;
 
-	gboolean election_configured = election_manager_configured(
-			args->repo->election_manager);
+	const gboolean election_configured =
+		election_manager_configured(args->repo->election_manager);
 
 	if (election_configured && !args->no_refcheck) {
-		gboolean has_peers = FALSE;
-		err = election_has_peers(args->repo->election_manager, &args->name,
-				FALSE, &has_peers);
-		if (err != NULL) {
-			g_prefix_error(&err, "Peers resolution error: ");
+
+		gboolean replicated = FALSE;
+		enum election_step_e step = STEP_NONE;
+		err = election_init(args->repo->election_manager, &args->name, &step, &replicated);
+		if (err)
 			return err;
+
+		if (args->create && sqliterepo_election_lazy_recover) {
+			err = sqlx_repository_has_base2(args->repo, &args->name, NULL);
+			if (err) {
+				if (err->code == CODE_CONTAINER_NOTFOUND) {
+					g_clear_error(&err);
+					err = _db_raw_creation(args);
+				}
+			}
 		}
-		if (has_peers)
-			args->is_replicated = TRUE;
+
+		if (!err && args->is_replicated)
+			err = election_start(args->repo->election_manager, &args->name);
+
+		args->is_replicated = BOOL(replicated);
 	}
 
 	/* Now manage the replication status */
@@ -824,7 +831,6 @@ _open_and_lock_base(struct open_args_s *args, enum election_status_e expected,
 				args->is_replicated ? "true" : "false");
 	} else {
 		gchar *url = NULL;
-
 		status = election_get_status(args->repo->election_manager, &args->name, &url);
 		GRID_TRACE("Status got=%d expected=%d master=%s", status, expected, url);
 
@@ -844,7 +850,7 @@ _open_and_lock_base(struct open_args_s *args, enum election_status_e expected,
 					err = NEWERROR(CODE_BADOPFORSLAVE, "not SLAVE");
 				break;
 			case ELECTION_FAILED:
-				err = NEWERROR(CODE_INTERNAL_ERROR, "Election failed [%s][%s]",
+				err = NEWERROR(CODE_UNAVAILABLE, "Election failed [%s][%s]",
 						args->name.base, args->name.type);
 				break;
 		}
@@ -861,6 +867,36 @@ _open_and_lock_base(struct open_args_s *args, enum election_status_e expected,
 			sqlx_alert_dirty_base (*result, "opened with dirty admin");
 		(*result)->election = status;
 	}
+
+	if (!err && args->is_replicated) {
+		NAME2CONST(n, (*result)->name);
+		gchar **peers = NULL;
+		err = sqlx_repository_get_peers((*result)->repo, &n, &peers);
+		if (err) {
+			GRID_WARN("Failed to fetch peers for %s (%d %s), "
+					"the local entry may remain uninitialized",
+					(*result)->path_inline, err->code, err->message);
+			EXTRA_ASSERT(peers == NULL);
+			g_clear_error(&err);
+		} else {
+			EXTRA_ASSERT(peers != NULL);
+			// The list returned by `get_peers` does not include this service
+			gchar *myid = g_strdup(
+					election_manager_get_local((*result)->manager));
+			peers = oio_strv_append(peers, myid);
+			int compare(const void *p0, const void *p1) {
+				return g_strcmp0(*(gchar**)p0, *(gchar**)p1);
+			}
+			qsort(peers, g_strv_length(peers), sizeof(gchar*), compare);
+			if (sqlx_admin_ensure_peers((*result), peers)) {
+				sqlx_admin_save_lazy(*result);
+				GRID_DEBUG("Replications peers saved in %s",
+						(*result)->path_inline);
+			}
+			g_strfreev(peers);
+		}
+	}
+
 	return err;
 }
 
@@ -884,8 +920,10 @@ sqlx_repository_unlock_and_close2(struct sqlx_sqlite3_s *sq3, guint32 flags)
 		sq3->deleted = FALSE;
 
 	if (sq3->repo->cache) {
-		err = sqlx_cache_unlock_and_close_base(sq3->repo->cache, sq3->bd,
-			sq3->deleted || (flags & SQLX_CLOSE_IMMEDIATELY));
+		if (sq3->deleted)
+			flags |= SQLX_CLOSE_FOR_DELETION;
+		err = sqlx_cache_unlock_and_close_base(
+				sq3->repo->cache, sq3->bd, flags);
 	}
 	else {
 		__close_base(sq3);
@@ -907,7 +945,7 @@ sqlx_repository_unlock_and_close_noerror2(struct sqlx_sqlite3_s *sq3,
 	GRID_TRACE2("%s(%p)", __FUNCTION__, sq3);
 	GError *e = sqlx_repository_unlock_and_close2(sq3, flags);
 	if (e) {
-		GRID_WARN("DB closure error : (%d) %s", e->code, e->message);
+		GRID_WARN("DB closure error: (%d) %s", e->code, e->message);
 		g_error_free(e);
 	}
 }
@@ -970,7 +1008,7 @@ sqlx_repository_open_and_lock(sqlx_repository_t *repo,
 		*result = NULL;
 
 	if (!repo->running)
-		return NEWERROR(CODE_INTERNAL_ERROR, "Repository being closed");
+		return NEWERROR(CODE_UNAVAILABLE, "Repository being closed");
 
 	if (NULL != (err = _open_fill_args(&args, repo, n)))
 		return err;
@@ -1045,7 +1083,7 @@ sqlx_repository_has_base2(sqlx_repository_t *repo, const struct sqlx_name_s *n,
 	REPO_CHECK(repo);
 	SQLXNAME_CHECK(n);
 
-	struct open_args_s args;
+	struct open_args_s args = {0};
 
 	if (bddname != NULL)
 		*bddname = NULL;
@@ -1072,57 +1110,22 @@ sqlx_repository_status_base(sqlx_repository_t *repo, const struct sqlx_name_s *n
 	REPO_CHECK(repo);
 	SQLXNAME_CHECK(n);
 
-	GError *err = NULL;
-	gboolean has_peers = FALSE;
-
 	GRID_TRACE2("%s(%p,t=%s,n=%s)", __FUNCTION__, repo, n->type, n->base);
 
-	if (!repo->running)
-		return NEWERROR(CODE_INTERNAL_ERROR, "Repository being shut down");
-
-	if (NULL != (err = _schema_get(repo, n->type, NULL)))
+	/* Kick the election off */
+	gboolean replicated = FALSE;
+	GError *err = sqlx_repository_use_base(repo, n, TRUE, &replicated);
+	if (err)
 		return err;
-
-	if (!election_manager_configured(repo->election_manager)) {
-		GRID_TRACE("Replication disabled by configuration, MASTER de facto");
+	if (!replicated)
 		return NULL;
-	}
-
-	err = election_has_peers(repo->election_manager, n, FALSE, &has_peers);
-	if (err != NULL) {
-		g_prefix_error(&err, "Peers resolution error: ");
-		return err;
-	}
-
-	if (!has_peers) {
-		GRID_TRACE("Unable to find peers for [%s][%s]", n->base, n->type);
-		return NULL;
-	}
 
+	/* Wait for a final status */
 	gchar *url = NULL;
 	enum election_status_e status;
-
 	status = election_get_status(repo->election_manager, n, &url);
 	switch (status) {
 		case ELECTION_LOST:
-			if (GRID_DEBUG_ENABLED()) {
-				gchar **my_peers = NULL;
-				gboolean master_in_peers = FALSE;
-				GError *err2 = election_get_peers(repo->election_manager, n, FALSE, &my_peers);
-				for (gchar **cursor = my_peers;
-						my_peers && *cursor && !master_in_peers;
-						cursor++) {
-					master_in_peers |= (0 == g_strcmp0(url, *cursor));
-				}
-				if (!master_in_peers) {
-					gchar *tmp = g_strjoinv(", ", my_peers);
-					GRID_DEBUG("Redirecting to a bad service (%s not in [%s])",
-							url, tmp);
-					g_free(tmp);
-				}
-				g_strfreev(my_peers);
-				g_clear_error(&err2);
-			}
 			err = NEWERROR(CODE_REDIRECT, "%s", url);
 			break;
 		case ELECTION_LEADER:
@@ -1148,7 +1151,7 @@ sqlx_repository_prepare_election(sqlx_repository_t *repo, const struct sqlx_name
 	GRID_TRACE2("%s(%p,t=%s,n=%s)", __FUNCTION__, repo, n->type, n->base);
 
 	if (!repo->running)
-		return NEWERROR(CODE_INTERNAL_ERROR, "Repository being shut down");
+		return NEWERROR(CODE_UNAVAILABLE, "Repository being shut down");
 
 	if (NULL != (err = _schema_get(repo, n->type, NULL)))
 		return err;
@@ -1158,7 +1161,7 @@ sqlx_repository_prepare_election(sqlx_repository_t *repo, const struct sqlx_name
 		return NULL;
 	}
 
-	return election_init(repo->election_manager, n);
+	return election_init(repo->election_manager, n, NULL, NULL);
 }
 
 GError*
@@ -1171,7 +1174,7 @@ sqlx_repository_exit_election(sqlx_repository_t *repo, const struct sqlx_name_s
 	GRID_TRACE2("%s(%p,t=%s,n=%s)", __FUNCTION__, repo, n->type, n->base);
 
 	if (!repo->running)
-		return NEWERROR(CODE_INTERNAL_ERROR, "Repository being shut down");
+		return NEWERROR(CODE_UNAVAILABLE, "Repository being shut down");
 
 	if (NULL != (err = _schema_get(repo, n->type, NULL)))
 		return err;
@@ -1185,8 +1188,36 @@ sqlx_repository_exit_election(sqlx_repository_t *repo, const struct sqlx_name_s
 	return err;
 }
 
+static GError *
+_base_lazy_recover(sqlx_repository_t *repo, const struct sqlx_name_s *n,
+		enum election_step_e status)
+{
+	GError *err = NULL;
+	if (!(err = sqlx_repository_has_base2(repo, n, NULL)))
+		return NULL;
+
+	g_clear_error(&err);
+
+	if (status == STEP_MASTER || status == STEP_CHECKING_SLAVES) {
+		/* Ensure the election is not MASTER to avoid the DB to be
+		 * recreated empty while MASTER */
+		election_exit(repo->election_manager, n);
+	}
+
+	struct sqlx_sqlite3_s *sq3 = NULL;
+	err = sqlx_repository_open_and_lock(repo, n,
+			SQLX_OPEN_CREATE|SQLX_OPEN_LOCAL|SQLX_OPEN_URGENT,
+			&sq3, NULL);
+	if (err)
+		return err;
+
+	sqlx_repository_unlock_and_close_noerror(sq3);
+	return NULL;
+}
+
 GError*
-sqlx_repository_use_base(sqlx_repository_t *repo, const struct sqlx_name_s *n)
+sqlx_repository_use_base(sqlx_repository_t *repo, const struct sqlx_name_s *n,
+		gboolean allow_autocreate, gboolean *replicated)
 {
 	REPO_CHECK(repo);
 	SQLXNAME_CHECK(n);
@@ -1195,7 +1226,7 @@ sqlx_repository_use_base(sqlx_repository_t *repo, const struct sqlx_name_s *n)
 	GRID_TRACE2("%s(%p,t=%s,n=%s)", __FUNCTION__, repo, n->type, n->base);
 
 	if (!repo->running)
-		return NEWERROR(CODE_INTERNAL_ERROR, "Repository being shut down");
+		return NEWERROR(CODE_UNAVAILABLE, "Repository being shut down");
 
 	if (NULL != (err = _schema_get(repo, n->type, NULL)))
 		return err;
@@ -1205,7 +1236,22 @@ sqlx_repository_use_base(sqlx_repository_t *repo, const struct sqlx_name_s *n)
 		return NULL;
 	}
 
-	return election_start(repo->election_manager, n);
+	/* The initiation of the election will perform the check that the
+	 * election is locally managed. */
+	enum election_step_e status = STEP_NONE;
+	if (!(err = election_init(repo->election_manager, n, &status, replicated))) {
+
+		/* Interleave a DB creation (out of the lock) if explicitely
+		 * allowed by both the request type AND the application */
+		if (allow_autocreate && sqliterepo_election_lazy_recover) {
+			err = _base_lazy_recover(repo, n, status);
+		}
+
+		if (!err)
+			err = election_start(repo->election_manager, n);
+	}
+
+	return err;
 }
 
 /* ------------------------------------------------------------------------- */
@@ -1355,7 +1401,7 @@ sqlx_repository_dump_base_gba(struct sqlx_sqlite3_s *sq3, GByteArray **dump)
 	{
 		GError *_err = NULL;
 		GByteArray **dump2 = arg;
-		GByteArray *_dump = g_byte_array_new();
+		GByteArray *_dump = g_byte_array_sized_new(128 * 1024);
 		_err = _read_file(fd, _dump);
 		if (!_err)
 			*dump2 = _dump;
@@ -1382,7 +1428,7 @@ sqlx_repository_dump_base_chunked(struct sqlx_sqlite3_s *sq3,
 		if (0 > rc)
 			return NEWERROR(errno, "Failed to stat the temporary base");
 		do {
-			GByteArray *gba = g_byte_array_new();
+			GByteArray *gba = g_byte_array_sized_new(128 * 1024);
 			err = _read_file_chunk(fd, chunk_size, gba);
 			if (!err) {
 				bytes_read += gba->len;
@@ -1479,11 +1525,42 @@ GError*
 sqlx_repository_retore_from_master(struct sqlx_sqlite3_s *sq3)
 {
 	EXTRA_ASSERT(sq3 != NULL);
-
+	NAME2CONST(n, sq3->name);
 	return !election_manager_configured(sq3->repo->election_manager)
 		? NEWERROR(CODE_INTERNAL_ERROR, "Replication not configured")
-		: election_manager_trigger_RESYNC(sq3->repo->election_manager,
-				sqlx_name_mutable_to_const(&sq3->name));
+		: election_manager_trigger_RESYNC(sq3->repo->election_manager, &n);
+}
+
+GError*
+sqlx_repository_get_peers(sqlx_repository_t *repo,
+		const struct sqlx_name_s *n, gchar ***result)
+{
+	return election_get_peers(repo->election_manager, n, FALSE, result);
+}
+
+GError*
+sqlx_repository_get_peers2(sqlx_repository_t *repo,
+		const struct sqlx_name_s *n, gchar ***result)
+{
+	REPO_CHECK(repo);
+	SQLXNAME_CHECK(n);
+	EXTRA_ASSERT(result != NULL);
+
+	struct sqlx_sqlite3_s *sq3 = NULL;
+	GError *err = sqlx_repository_open_and_lock(repo, n, SQLX_OPEN_LOCAL|SQLX_OPEN_NOREFCHECK, &sq3, NULL);
+	if (err) {
+		*result = NULL;
+	} else {
+		gchar *tmp = sqlx_admin_get_str(sq3, SQLX_ADMIN_PEERS);
+		sqlx_repository_unlock_and_close_noerror2(sq3, SQLX_CLOSE_IMMEDIATELY);
+		if (tmp) {
+			*result = g_strsplit(tmp, ",", -1);
+			g_free(tmp);
+		} else {
+			*result = g_malloc0(sizeof(gchar*));
+		}
+	}
+	return err;
 }
 
 GError *
@@ -1491,7 +1568,7 @@ sqlx_repository_get_version(struct sqlx_sqlite3_s *sq3, GTree **result)
 {
 	GRID_TRACE2("%s(%p,%p)", __FUNCTION__, sq3, result);
 	if (!sq3 || !result)
-		return NEWERROR(CODE_INTERNAL_ERROR, "Invalid parameter");
+		return BADREQ("Invalid parameter");
 	*result = version_extract_from_admin(sq3);
 	return NULL;
 }
@@ -1525,4 +1602,3 @@ sqlx_repository_get_version2(sqlx_repository_t *repo, const struct sqlx_name_s *
 	*result = version;
 	return NULL;
 }
-
