@@ -1,7 +1,7 @@
 /*
 OpenIO SDS sqliterepo
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -49,8 +49,10 @@ _make_cache_entry(const gchar *buf, gsize len)
 	struct _cache_entry_s *e = g_malloc0(
 			sizeof(struct _cache_entry_s) + _buffer_length(len));
 	e->len = len;
-	if (len > 0)
+	if (buf && len > 0)
 		memcpy(e->buffer, buf, len);
+	if (!buf)
+		e->flag_deleted = 1;
 	return e;
 }
 
@@ -80,6 +82,14 @@ sqlx_exec(sqlite3 *handle, const gchar *sql)
 	return grc;
 }
 
+#ifdef HAVE_EXTRA_DEBUG
+#define _dump_entry(tag,k,v) \
+	GRID_TRACE2("%s: %s <- {del:%d, changed:%d, %s}", tag, \
+			k, v->flag_deleted, v->flag_changed, v->buffer)
+#else
+#define _dump_entry(tag,k,v)
+#endif
+
 gboolean
 sqlx_admin_set_str(struct sqlx_sqlite3_s *sq3, const gchar *k, const gchar *v)
 {
@@ -103,6 +113,7 @@ sqlx_admin_set_str(struct sqlx_sqlite3_s *sq3, const gchar *k, const gchar *v)
 
 	prev->flag_changed = 1;
 	sq3->admin_dirty = 1;
+	_dump_entry("change", k, prev);
 	return TRUE;
 }
 
@@ -118,10 +129,11 @@ void
 sqlx_admin_del(struct sqlx_sqlite3_s *sq3, const gchar *k)
 {
 	struct _cache_entry_s *v = g_tree_lookup(sq3->admin, k);
-	if (v) {
+	if (v && !v->flag_deleted) {
 		v->flag_deleted = 1;
-		v->flag_changed = 0;
-		sq3->admin_dirty = TRUE;
+		v->flag_changed = 1;
+		sq3->admin_dirty = 1;
+		_dump_entry("change", k, v);
 	}
 }
 
@@ -129,9 +141,12 @@ void
 sqlx_admin_del_all_user(struct sqlx_sqlite3_s *sq3)
 {
 	gboolean runner(gchar *k, struct _cache_entry_s *v, gpointer i UNUSED) {
+		if (v->flag_deleted)
+			return FALSE;
 		if (g_str_has_prefix(k, SQLX_ADMIN_PREFIX_USER)) {
 			v->flag_deleted = 1;
-			v->flag_changed = 0;
+			v->flag_changed = 1;
+			_dump_entry("change", k, v);
 		}
 		return FALSE;
 	}
@@ -194,6 +209,8 @@ sqlx_admin_inc_i64(struct sqlx_sqlite3_s *sq3, const gchar *k, const gint64 delt
 
 	v->flag_changed = 1;
 	v->flag_deleted = 0;
+	sq3->admin_dirty = 1;
+	_dump_entry("change", k, v);
 }
 
 static void
@@ -226,7 +243,8 @@ sqlx_admin_inc_version(struct sqlx_sqlite3_s *sq3, const gchar *k, const int del
 	struct _cache_entry_s *prev = g_tree_lookup(sq3->admin, k);
 	if (prev) {
 		_cache_entry_increment_version(prev, delta);
-		sq3->admin_dirty = TRUE;
+		sq3->admin_dirty = 1;
+		_dump_entry("change", k, prev);
 	}
 }
 
@@ -249,6 +267,19 @@ sqlx_admin_ensure_versions (struct sqlx_sqlite3_s *sq3)
 	}
 }
 
+gboolean
+sqlx_admin_ensure_peers(struct sqlx_sqlite3_s *sq3, gchar **peers)
+{
+	EXTRA_ASSERT(peers != NULL);
+	gboolean modified = FALSE;
+	if (!sqlx_admin_has(sq3, SQLX_ADMIN_PEERS)) {
+		gchar *packed_peers = g_strjoinv(",", peers);
+		modified = sqlx_admin_set_str(sq3, SQLX_ADMIN_PEERS, packed_peers);
+		g_free(packed_peers);
+	}
+	return modified;
+}
+
 void
 sqlx_admin_inc_all_versions(struct sqlx_sqlite3_s *sq3, const int delta)
 {
@@ -295,29 +326,28 @@ gchar**
 sqlx_admin_get_keyvalues (struct sqlx_sqlite3_s *sq3)
 {
 	gboolean runner(gchar *k, struct _cache_entry_s *v, GPtrArray *tmp) {
-		g_ptr_array_add (tmp, g_strdup(k));
 		if (v->flag_deleted)
 			return FALSE;
+		g_ptr_array_add(tmp, g_strdup(k));
 		if (!v->buffer)
-			g_ptr_array_add (tmp, g_strdup(""));
+			g_ptr_array_add(tmp, g_strdup(""));
 		else
-			g_ptr_array_add (tmp, g_strndup((gchar*)v->buffer, v->len));
+			g_ptr_array_add(tmp, g_strndup((gchar*)v->buffer, v->len));
 		return FALSE;
 	}
 
-	GPtrArray *tmp = g_ptr_array_new ();
-	g_tree_foreach (sq3->admin, (GTraverseFunc) runner, tmp);
-	return (gchar**) metautils_gpa_to_array (tmp, TRUE);
+	GPtrArray *tmp = g_ptr_array_new();
+	g_tree_foreach(sq3->admin, (GTraverseFunc) runner, tmp);
+	return (gchar**) metautils_gpa_to_array(tmp, TRUE);
 }
 
-guint
+static guint
 sqlx_admin_save (struct sqlx_sqlite3_s *sq3)
 {
 	int rc;
 	guint count = 0;
 	GError *err = NULL;
 	sqlite3_stmt *stmt = NULL;
-	gboolean run_to_delete = FALSE;
 
 	EXTRA_ASSERT(sq3 != NULL);
 
@@ -327,16 +357,18 @@ sqlx_admin_save (struct sqlx_sqlite3_s *sq3)
 		err = SYSERR("DB error: (%d) %s", rc, sqlite3_errmsg(sq3->db));
 	else {
 		gboolean _save (gchar *k, struct _cache_entry_s *v, gpointer i UNUSED) {
-
-			run_to_delete |= BOOL(v->flag_deleted);
-			if (v->flag_deleted || !v->flag_changed)
+			_dump_entry("save", k, v);
+			if (!v->flag_changed)
 				return FALSE;
 
 			sqlite3_reset (stmt);
 			sqlite3_clear_bindings (stmt);
 			sqlite3_bind_text (stmt, 1, k, -1, NULL);
-			if (!v->len) {
+
+			if (v->flag_deleted) {
 				sqlite3_bind_null (stmt, 2);
+			} else if (!v->len) {
+				sqlite3_bind_text (stmt, 2, "", 0, NULL);
 			} else {
 				sqlite3_bind_blob (stmt, 2, (guint8*)v->buffer, v->len, NULL);
 			}
@@ -350,35 +382,6 @@ sqlx_admin_save (struct sqlx_sqlite3_s *sq3)
 		(void) sqlite3_finalize(stmt);
 	}
 
-	if (run_to_delete && !err) {
-		sqlite3_prepare_debug(rc, sq3->db,
-				"DELETE FROM admin WHERE k = ?", -1, &stmt, NULL);
-		if (rc != SQLITE_OK && rc != SQLITE_DONE)
-			err = SYSERR("DB error: (%d) %s", rc, sqlite3_errmsg(sq3->db));
-		else {
-			GSList *deleted = NULL;
-			gboolean _delete (gchar *k, struct _cache_entry_s *v, gpointer i UNUSED) {
-				if (!v->flag_deleted)
-					return FALSE;
-				sqlite3_reset (stmt);
-				sqlite3_clear_bindings (stmt);
-				sqlite3_bind_text (stmt, 1, k, -1, NULL);
-				sqlite3_step_debug_until_end (rc, stmt);
-				if (rc != SQLITE_OK && rc != SQLITE_DONE)
-					err = SYSERR("DB error: (%d) %s", rc, sqlite3_errmsg(sq3->db));
-				else
-					deleted = g_slist_prepend(deleted, k);
-				count ++;
-				return err != NULL;
-			}
-			g_tree_foreach (sq3->admin, (GTraverseFunc)_delete, NULL);
-			(void) sqlite3_finalize(stmt);
-			for (GSList *l = deleted; !err && l; l = l->next)
-				g_tree_remove(sq3->admin, l->data);
-			g_slist_free(deleted);  // values were freed by g_tree_remove
-		}
-	}
-
 	if (err) {
 		GRID_WARN("DB error: failed to save the admin table: (%d) %s",
 				err->code, err->message);
@@ -421,10 +424,14 @@ sqlx_admin_load(struct sqlx_sqlite3_s *sq3)
 		EXTRA_ASSERT(stmt != NULL);
 		while (SQLITE_ROW == (rc = sqlite3_step(stmt))) {
 			const gchar *k = (const gchar*)sqlite3_column_text(stmt, 0);
-			g_tree_replace(sq3->admin, g_strdup(k),
-					_make_cache_entry(
+			struct _cache_entry_s *v = NULL;
+			if (sqlite3_column_type(stmt, 1) == SQLITE_NULL)
+				v = _make_cache_entry(NULL, 0);
+			else
+				v = _make_cache_entry(
 						sqlite3_column_blob(stmt, 1),
-						sqlite3_column_bytes(stmt, 1)));
+						sqlite3_column_bytes(stmt, 1));
+			g_tree_replace(sq3->admin, g_strdup(k), v);
 		}
 		sqlite3_finalize(stmt);
 	}
