@@ -1,7 +1,7 @@
 /*
 OpenIO SDS sqliterepo
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -35,6 +35,10 @@ License along with this library.
 #define SQLX_ADMIN_PREFIX_USER "user."
 #endif
 
+#ifndef SQLX_ADMIN_PEERS
+#define SQLX_ADMIN_PEERS SQLX_ADMIN_PREFIX_SYS "peers"
+#endif
+
 #ifndef SQLX_ADMIN_INITFLAG
 #define SQLX_ADMIN_INITFLAG SQLX_ADMIN_PREFIX_SYS "sqlx.init"
 #endif
@@ -157,10 +161,10 @@ void sqlx_admin_inc_all_versions(struct sqlx_sqlite3_s *sq3, const int delta);
 
 void sqlx_admin_ensure_versions (struct sqlx_sqlite3_s *sq3);
 
-/* Returns the numer of items altered. It doesn't check the <sq3->admin_dirty>
- * flag, and doesn't open/closes a transaction. This is intentional, to let
- * sqlx_admin_save_lazy() take place in a transaction managed by the caller. */
-guint sqlx_admin_save (struct sqlx_sqlite3_s *sq3);
+/* Check if peers have been saved in the database, save them if needed.
+ * Returns TRUE if peers have just been saved,
+ * FALSE if they were already there */
+gboolean sqlx_admin_ensure_peers(struct sqlx_sqlite3_s *sq3, gchar **peers);
 
 /* calls sqlx_admin_save() if the handle is dirty */
 guint sqlx_admin_save_lazy (struct sqlx_sqlite3_s *sq3);
