@@ -1,7 +1,7 @@
 /*
 OpenIO SDS sqliterepo
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -34,7 +34,6 @@ struct sqlx_cache_s;
 struct sqlx_repctx_s;
 struct sqlx_sqlite3_s;
 struct sqlx_name_s;
-struct sqlx_name_mutable_s;
 
 typedef struct sqlx_repository_s sqlx_repository_t;
 
@@ -75,7 +74,10 @@ enum sqlx_open_type_e
 enum sqlx_close_flag_e
 {
 	/** Close the base immediately, don't keep it in the cache */
-	SQLX_CLOSE_IMMEDIATELY = 0x01,
+	SQLX_CLOSE_IMMEDIATELY  = 0x01,
+	/** Close the base immediately,
+	 *  and prevent other threads from reopening it. */
+	SQLX_CLOSE_FOR_DELETION = 0x02,
 };
 
 enum sqlx_repo_flag_e
@@ -83,14 +85,13 @@ enum sqlx_repo_flag_e
 	SQLX_REPO_NOCACHE      = 0x01,
 	SQLX_REPO_VACUUM       = 0x02,
 	SQLX_REPO_DELETEON     = 0x04,
-	SQLX_REPO_AUTOCREATE   = 0x10,
 };
 
 enum sqlx_sync_mode_e
 {
-	SQLX_SYNC_OFF=0,
-	SQLX_SYNC_NORMAL=1,
-	SQLX_SYNC_FULL=2
+	SQLX_SYNC_OFF = 0,
+	SQLX_SYNC_NORMAL = 1,
+	SQLX_SYNC_FULL = 2
 };
 
 enum election_status_e
@@ -104,32 +105,32 @@ struct sqlx_sqlite3_s
 {
 	struct sqlx_repository_s *repo;
 	struct election_manager_s *manager;
-	struct sqlx_name_mutable_s name;
-	sqlite3 *db;
-	gchar *path;
-
 	GTree *admin; // <gchar*,GByteArray*>
-	gint bd; // ID in cache
-	enum election_status_e election; // set at open(), reset at close()
+	sqlite3 *db;
 
-	gboolean admin_dirty : 8;
-	gboolean deleted : 8;
-	gboolean no_peers : 8; // Prevent get_peers()
+	gint bd; // ID in cache
+	enum election_status_e election : 8; // set at open(), reset at close()
+
+	guint8 admin_dirty : 1;
+	guint8 deleted : 1;
+	guint8 no_peers : 1; // Prevent get_peers()
+
+	struct sqlx_name_inline_s name;
+	gchar path_inline[128 + LIMIT_LENGTH_NSNAME + LIMIT_LENGTH_SRVTYPE];
 };
 
 struct sqlx_repo_config_s
 {
-	enum sqlx_repo_flag_e flags; /**< several options. */
+	/** several options. */
+	enum sqlx_repo_flag_e flags;
 
-	enum sqlx_sync_mode_e sync_solo; /**< Which value for pragma synchronous'
-									   for not replicated bases */
+	/** Which value for 'pragma synchronous'
+	 * for not replicated bases */
+	enum sqlx_sync_mode_e sync_solo;
 
-	enum sqlx_sync_mode_e sync_repli; /**< Which value for pragma synchronous'
-									   for replicated bases */
-	guint page_size;
-
-	/* Cache maximum number of items */
-	guint max_bases;
+	/** Which value for 'pragma synchronous'
+	 * for replicated bases */
+	enum sqlx_sync_mode_e sync_repli;
 };
 
 /* ------------------------------------------------------------------------- */
@@ -186,20 +187,10 @@ void sqlx_repository_set_elections(sqlx_repository_t *repo,
 void sqlx_repository_configure_hash(sqlx_repository_t *repo,
 		guint width, guint depth);
 
-/** Calling this function on a repository that already managed a base has no
- * effect. */
-void sqlx_repository_configure_maxbases(sqlx_repository_t *repo,
-		guint max);
-
 /** Register a new DB type with its schema.  */
 GError* sqlx_repository_configure_type(sqlx_repository_t *repo,
 		const char *type, const char *schema);
 
-/* Set open timeout for bases currently in use by another thread.
- * Precision uniform with oio_ext_monotonic_time(). */
-void sqlx_repository_configure_open_timeout(sqlx_repository_t *repo,
-		gint64 timeout);
-
 void sqlx_repository_configure_close_callback(sqlx_repository_t *repo,
 		sqlx_repo_close_hook cb, gpointer cb_data);
 
@@ -234,6 +225,14 @@ void sqlx_repository_unlock_and_close_noerror(struct sqlx_sqlite3_s *sq3);
 void sqlx_repository_unlock_and_close_noerror2(struct sqlx_sqlite3_s *sq3,
 		guint32 flags);
 
+/** Get the replication peers from the higher level service */
+GError* sqlx_repository_get_peers(sqlx_repository_t *repo,
+		const struct sqlx_name_s *n, gchar ***result);
+
+/** Get the replication peers from the local database */
+GError* sqlx_repository_get_peers2(sqlx_repository_t *repo,
+		const struct sqlx_name_s *n, gchar ***result);
+
 GError * sqlx_repository_get_version(struct sqlx_sqlite3_s *sq3,
 		GTree **result);
 
@@ -257,7 +256,9 @@ GError* sqlx_repository_exit_election(sqlx_repository_t *repo,
 
 /** Triggers the global election mechanism on a base given its name */
 GError* sqlx_repository_use_base(sqlx_repository_t *repo,
-		const struct sqlx_name_s *n);
+		const struct sqlx_name_s *n,
+		gboolean allow_autocreate,
+		gboolean *replicated);
 
 /** Triggers the global election mechanism on a base given its name and
  * returns when a final status (or a timeout) has been reached.
