@@ -1,7 +1,7 @@
 /*
 OpenIO SDS sqliterepo
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -20,18 +20,10 @@ License along with this library.
 #include <stddef.h>
 #include <unistd.h>
 
-#include <metautils/lib/metatypes.h>
 #include <metautils/lib/metautils.h>
 #include <metautils/lib/metacomm.h>
-
-#include <RowName.h>
-#include <RowField.h>
-#include <Row.h>
-#include <RowSet.h>
-#include <Table.h>
-#include <TableSequence.h>
-#include <asn_codecs.h>
-#include <der_encoder.h>
+#include <metautils/lib/codec.h>
+#include <sqliterepo/sqliterepo_remote_variables.h>
 
 #include "sqliterepo.h"
 #include "sqlx_macros.h"
@@ -57,7 +49,7 @@ static GByteArray*
 sqlx_encode_ASN1(struct asn_TYPE_descriptor_s *descr, void *s, GError **err)
 {
 	asn_enc_rval_t rv;
-	GByteArray *encoded = g_byte_array_new();
+	GByteArray *encoded = g_byte_array_sized_new(2048);
 	rv = der_encode(descr, s, metautils_asn1c_write_gba, encoded);
 	if (0 >= rv.encoded) {
 		g_byte_array_free(encoded, TRUE);
@@ -69,30 +61,12 @@ sqlx_encode_ASN1(struct asn_TYPE_descriptor_s *descr, void *s, GError **err)
 	return encoded;
 }
 
-GByteArray*
-sqlx_encode_Table(struct Table *table, GError **err)
-{
-	return sqlx_encode_ASN1(&asn_DEF_Table, table, err);
-}
-
 GByteArray*
 sqlx_encode_TableSequence(struct TableSequence *tabseq, GError **err)
 {
 	return sqlx_encode_ASN1(&asn_DEF_TableSequence, tabseq, err);
 }
 
-GByteArray*
-sqlx_encode_Row(struct Row *row, GError **err)
-{
-	return sqlx_encode_ASN1(&asn_DEF_Row, row, err);
-}
-
-GByteArray*
-sqlx_encode_RowSet(struct RowSet *rows, GError **err)
-{
-	return sqlx_encode_ASN1(&asn_DEF_RowSet, rows, err);
-}
-
 /* ------------------------------------------------------------------------- */
 
 GByteArray*
@@ -144,13 +118,6 @@ sqlx_pack_EXITELECTION(const struct sqlx_name_s *name)
 	return message_marshall_gba_and_clean(req);
 }
 
-GByteArray*
-sqlx_pack_ISMASTER(const struct sqlx_name_s *name)
-{
-	MESSAGE req = make_request(NAME_MSGNAME_SQLX_ISMASTER, name);
-	return message_marshall_gba_and_clean(req);
-}
-
 GByteArray*
 sqlx_pack_PIPEFROM(const struct sqlx_name_s *name, const gchar *source)
 {
@@ -219,24 +186,6 @@ sqlx_pack_QUERY(const struct sqlx_name_s *name, const gchar *query,
 	return message_marshall_gba_and_clean(req);
 }
 
-GByteArray*
-sqlx_pack_QUERY_single(const struct sqlx_name_s *name, const gchar *query,
-		gboolean autocreate)
-{
-	EXTRA_ASSERT(name != NULL);
-	EXTRA_ASSERT(query != NULL);
-
-	struct Table *t = calloc(1, sizeof(Table_t));
-	OCTET_STRING_fromBuf(&(t->name), query, strlen(query));
-
-	struct TableSequence *ts = calloc(1, sizeof(TableSequence_t));
-	asn_sequence_add(&(ts->list), t);
-
-	GByteArray *req = sqlx_pack_QUERY(name, query, ts, autocreate);
-	asn_DEF_TableSequence.free_struct(&asn_DEF_TableSequence, ts, FALSE);
-	return req;
-}
-
 GByteArray *
 sqlx_pack_DESTROY(const struct sqlx_name_s *name, gboolean local)
 {
@@ -297,76 +246,42 @@ sqlx_pack_PROPSET_tab(const struct sqlx_name_s *name, gboolean flush, gchar **kv
 }
 
 void
-sqlx_name_clean (struct sqlx_name_mutable_s *n)
-{
-	if (!n) return;
-	oio_str_clean(&n->ns);
-	oio_str_clean(&n->base);
-	oio_str_clean(&n->type);
-}
-
-void
-sqlx_name_free (struct sqlx_name_mutable_s *n)
-{
-	sqlx_name_clean(n);
-	g_free0 (n);
-}
-
-void
-sqlx_name_dup (struct sqlx_name_mutable_s *dst, const struct sqlx_name_s *src)
-{
-	EXTRA_ASSERT(dst != NULL && src != NULL);
-	oio_str_replace (&dst->ns, src->ns);
-	oio_str_replace (&dst->base, src->base);
-	oio_str_replace (&dst->type, src->type);
-}
-
-void
-sqlx_name_fill_type_asis  (struct sqlx_name_mutable_s *n, struct oio_url_s *url,
-		const char *srvtype, gint64 seq)
+sqlx_inline_name_fill_type_asis  (struct sqlx_name_inline_s *n,
+		struct oio_url_s *url, const char *srvtype, gint64 seq)
 {
 	EXTRA_ASSERT (n != NULL);
 	EXTRA_ASSERT (url != NULL);
 	EXTRA_ASSERT (srvtype != NULL);
 
-	n->ns = g_strdup(oio_url_get (url, OIOURL_NS));
+	g_strlcpy(n->ns, oio_url_get (url, OIOURL_NS), sizeof(n->ns));
+	g_strlcpy(n->type, srvtype, sizeof(n->type));
 
 	if (!strcmp(srvtype, NAME_SRVTYPE_META0)) {
-		n->base = g_strdup (oio_url_get (url, OIOURL_NS));
+		const gchar *ns = oio_url_get (url, OIOURL_NS);
+		g_strlcpy(n->base, ns ?: "", sizeof(n->base));
 	} else if (!strcmp(srvtype, NAME_SRVTYPE_META1)) {
-		n->base = g_strndup (oio_url_get(url, OIOURL_HEXID), 4);
+		const gchar *cid = oio_url_get(url, OIOURL_HEXID);
+		g_strlcpy(n->base, cid ?: "", 4+1);
 	} else {
-		n->base = g_strdup_printf ("%s.%"G_GINT64_FORMAT,
-				oio_url_get (url, OIOURL_HEXID), seq);
+		const gchar *cid = oio_url_get(url, OIOURL_HEXID);
+		g_snprintf (n->base, sizeof(n->base), "%s.%"G_GINT64_FORMAT,
+				cid ?: "", seq);
 	}
-
-	n->type = g_strdup (srvtype);
 }
 
 void
-sqlx_name_fill  (struct sqlx_name_mutable_s *n, struct oio_url_s *url,
+sqlx_inline_name_fill (struct sqlx_name_inline_s *n, struct oio_url_s *url,
 		const char *srvtype, gint64 seq)
 {
-	EXTRA_ASSERT (n != NULL);
-	EXTRA_ASSERT (url != NULL);
-	EXTRA_ASSERT (srvtype != NULL);
+	gchar tmp[LIMIT_LENGTH_BASETYPE];
 	const gchar *subtype = oio_url_get (url, OIOURL_TYPE);
 
-	n->ns = g_strdup(oio_url_get (url, OIOURL_NS));
-
-	if (!strcmp(srvtype, NAME_SRVTYPE_META0)) {
-		n->base = g_strdup (oio_url_get (url, OIOURL_NS));
-	} else if (!strcmp(srvtype, NAME_SRVTYPE_META1)) {
-		n->base = g_strndup (oio_url_get(url, OIOURL_HEXID), 4);
-	} else {
-		n->base = g_strdup_printf ("%s.%"G_GINT64_FORMAT,
-				oio_url_get (url, OIOURL_HEXID), seq);
-	}
-
 	if (subtype && 0 != strcmp(subtype, OIOURL_DEFAULT_TYPE))
-		n->type = g_strdup_printf ("%s.%s", srvtype, subtype);
+		g_snprintf(tmp, sizeof(tmp), "%s.%s", srvtype, subtype);
 	else
-		n->type = g_strdup (srvtype);
+		g_strlcpy(tmp, srvtype, sizeof(tmp));
+
+	return sqlx_inline_name_fill_type_asis(n, url, tmp, seq);
 }
 
 gboolean
@@ -397,24 +312,3 @@ sqlx_name_extract (const struct sqlx_name_s *n, struct oio_url_s *url,
 
 	return BOOL(rc);
 }
-
-/* -------------------------------------------------------------------------- */
-
-GByteArray* sqlx_pack_FLUSH (void) {
-	return message_marshall_gba_and_clean (metautils_message_create_named (
-				NAME_MSGNAME_SQLX_FLUSH));
-}
-
-GByteArray* sqlx_pack_RELOAD (void) {
-	return message_marshall_gba_and_clean (metautils_message_create_named (
-				NAME_MSGNAME_SQLX_RELOAD));
-}
-
-GError* sqlx_remote_execute_FLUSH (const char *to) {
-	return gridd_client_exec (to, COMMON_CLIENT_TIMEOUT, sqlx_pack_FLUSH());
-}
-
-GError* sqlx_remote_execute_RELOAD (const char *to) {
-	return gridd_client_exec (to, COMMON_CLIENT_TIMEOUT, sqlx_pack_RELOAD());
-}
-
