@@ -1,7 +1,7 @@
 /*
 OpenIO SDS sqliterepo
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -22,9 +22,17 @@ License along with this library.
 
 #define NAME_CHECK(C) do {\
 	EXTRA_ASSERT((C) != NULL);\
-	EXTRA_ASSERT((C)->base != NULL); \
-	EXTRA_ASSERT((C)->type != NULL); \
-	EXTRA_ASSERT((C)->ns != NULL); \
+	EXTRA_ASSERT(oio_str_is_set((C)->base)); \
+	EXTRA_ASSERT(oio_str_is_set((C)->type)); \
+	EXTRA_ASSERT(oio_str_is_set((C)->ns)); \
+} while (0)
+
+#define NAME2CONST(n, n0) struct sqlx_name_s n = { (n0).ns, (n0).base, (n0).type }
+
+#define NAMEFILL(Name, Src) do { \
+	g_strlcpy((Name).ns, (Src).ns, sizeof((Name).ns)); \
+	g_strlcpy((Name).base, (Src).base, sizeof((Name).base)); \
+	g_strlcpy((Name).type, (Src).type, sizeof((Name).type)); \
 } while (0)
 
 #define SQLXNAME_CHECK(p) do { EXTRA_ASSERT((p) != NULL); NAME_CHECK(p); } while (0)
@@ -38,11 +46,11 @@ struct TableSequence;
 
 // Handy structures avoiding passing too many arguments ------------------------
 
-struct sqlx_name_mutable_s
+struct sqlx_name_inline_s
 {
-	gchar *ns;
-	gchar *base;
-	gchar *type;
+	gchar ns[LIMIT_LENGTH_NSNAME];
+	gchar base[LIMIT_LENGTH_BASENAME];
+	gchar type[LIMIT_LENGTH_BASETYPE];
 };
 
 struct sqlx_name_s
@@ -52,39 +60,16 @@ struct sqlx_name_s
 	const char *type;
 };
 
-void sqlx_name_clean (struct sqlx_name_mutable_s *n);
-void sqlx_name_free  (struct sqlx_name_mutable_s *n);
-
-void sqlx_name_dup   (struct sqlx_name_mutable_s *dst,
-		const struct sqlx_name_s *src);
-
-void sqlx_name_fill  (struct sqlx_name_mutable_s *n, struct oio_url_s *url,
-		const char *srvtype, gint64 seq);
-
-void sqlx_name_fill_type_asis  (struct sqlx_name_mutable_s *n,
+void sqlx_inline_name_fill  (struct sqlx_name_inline_s *n,
 		struct oio_url_s *url, const char *srvtype, gint64 seq);
 
+void sqlx_inline_name_fill_type_asis  (struct sqlx_name_inline_s *n,
+		struct oio_url_s *url, const char *srvtype, gint64 seq);
+
+
 gboolean sqlx_name_extract (const struct sqlx_name_s *n, struct oio_url_s *url,
 		const char *srvtype, gint64 *pseq);
 
-static inline struct sqlx_name_s *
-sqlx_name_mutable_to_const(struct sqlx_name_mutable_s *mut)
-{
-	return (struct sqlx_name_s*)mut;
-}
-
-#define SQLXNAME_STACKIFY(N) do { \
-	if ((N).ns) STRING_STACKIFY((N).ns); \
-	if ((N).base) STRING_STACKIFY((N).base); \
-	if ((N).type) STRING_STACKIFY((N).type); \
-} while (0)
-
-GByteArray* sqlx_pack_FLUSH (void);
-GByteArray* sqlx_pack_RELOAD (void);
-
-GError* sqlx_remote_execute_FLUSH (const char *to);
-GError* sqlx_remote_execute_RELOAD (const char *to);
-
 // sqliterepo-related requests coders ------------------------------------------
 
 GByteArray* sqlx_pack_ENABLE (const struct sqlx_name_s *name);
@@ -108,7 +93,6 @@ GByteArray* sqlx_pack_USE(const struct sqlx_name_s *name);
 GByteArray* sqlx_pack_DESCR(const struct sqlx_name_s *name);
 GByteArray* sqlx_pack_STATUS(const struct sqlx_name_s *name);
 GByteArray* sqlx_pack_GETVERS(const struct sqlx_name_s *name);
-GByteArray* sqlx_pack_ISMASTER(const struct sqlx_name_s *name);
 
 GByteArray* sqlx_pack_PIPEFROM(const struct sqlx_name_s *name, const gchar *source);
 GByteArray* sqlx_pack_PIPETO(const struct sqlx_name_s *name, const gchar *target);
@@ -127,19 +111,10 @@ GByteArray* sqlx_pack_INFO(void);
 GByteArray* sqlx_pack_QUERY(const struct sqlx_name_s *name,
 		const gchar *query, struct TableSequence *params, gboolean autocreate);
 
-GByteArray* sqlx_pack_QUERY_single(const struct sqlx_name_s *name,
-		const gchar *query, gboolean autocreate);
-
 GByteArray* sqlx_pack_DESTROY(const struct sqlx_name_s *name, gboolean local);
 
 // sqlx-related elements coders ------------------------------------------------
 
-GByteArray* sqlx_encode_Row(struct Row *row, GError **err);
-
-GByteArray* sqlx_encode_RowSet(struct RowSet *rows, GError **err);
-
-GByteArray* sqlx_encode_Table(struct Table *table, GError **err);
-
 GByteArray* sqlx_encode_TableSequence(struct TableSequence *tabseq,
 		GError **err);
 
@@ -151,9 +126,6 @@ void peers_restore(gchar **targets, struct sqlx_name_s *name,
 GError * peer_restore(const gchar *target, struct sqlx_name_s *name,
 		GByteArray *dump);
 
-GError * peer_dump_gba(const gchar *target, struct sqlx_name_s *name,
-		GByteArray **result);
-
 typedef GError* (*peer_dump_cb)(GByteArray *part, gint64 remaining, gpointer arg);
 
 GError * peer_dump(const gchar *target, struct sqlx_name_s *name, gboolean chunked,
