@@ -1,7 +1,7 @@
 /*
 OpenIO SDS sqliterepo
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -23,17 +23,6 @@ License along with this library.
 #include <metautils/lib/metautils.h>
 #include <sqliterepo/sqlx_remote.h>
 
-/**
- * Destroy an SQLX database.
- *
- * @param target One of the services managing the database.
- * @param sid Unused
- * @param name The name of the database
- * @param local TRUE to destroy only the database local to the service
- */
-GError* sqlx_remote_execute_DESTROY(const gchar *target, GByteArray *sid,
-		struct sqlx_name_s *name, gboolean local);
-
 /**
  * Locally destroy an SQLX database on several services.
  *
