@@ -1,7 +1,7 @@
 /*
 OpenIO SDS sqliterepo
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -25,9 +25,7 @@ License along with this library.
 
 #include <metautils/lib/metautils.h>
 #include <metautils/lib/metacomm.h>
-
-#include <Table.h>
-#include <TableSequence.h>
+#include <metautils/lib/codec.h>
 
 #include "sqliterepo.h"
 #include "sqlx_remote.h"
@@ -86,40 +84,12 @@ gchar*
 version_dump(GTree *t)
 {
 	EXTRA_ASSERT(t != NULL);
-	GString *gstr = g_string_new("");
+	GString *gstr = g_string_sized_new(128);
 	if (t)
 		g_tree_foreach(t, hook_dump, gstr);
 	return g_string_free(gstr, FALSE);
 }
 
-void
-version_debug(const gchar *tag, GTree *versions)
-{
-	if (!GRID_TRACE_ENABLED())
-		return;
-
-	(void) tag;
-	gchar *s = version_dump(versions);
-	GRID_TRACE("%s %s (%s)", tag, s, __FUNCTION__);
-	g_free(s);
-}
-
-static gboolean
-hook_increment(gpointer k, gpointer v, gpointer u)
-{
-	(void) k; (void) u;
-	OV(v)->version ++;
-	OV(v)->when = oio_ext_real_time() / G_TIME_SPAN_SECOND;
-	return FALSE;
-}
-
-void
-version_increment_all(GTree *t)
-{
-	if (t)
-		g_tree_foreach(t, hook_increment, NULL);
-}
-
 #include <TableVersion.h>
 #include <BaseVersion.h>
 #include <asn_codecs.h>
@@ -137,7 +107,7 @@ version_encode(GTree *t)
 		(void) _u;
 		if (_k && _v && hashstr_len(_k) > 0) {
 			struct object_version_s *v = _v;
-			struct TableVersion *tv = calloc(1, sizeof(*tv));
+			struct TableVersion *tv = ASN1C_CALLOC(1, sizeof(*tv));
 			OCTET_STRING_fromBuf(&(tv->name), hashstr_str(_k), hashstr_len(_k));
 			asn_int64_to_INTEGER(&(tv->version), v->version);
 			asn_int64_to_INTEGER(&(tv->when), v->when);
@@ -149,7 +119,7 @@ version_encode(GTree *t)
 	GRID_TRACE2("%s(%p)", __FUNCTION__, t);
 	g_tree_foreach(t, runner, NULL);
 
-	encoded = g_byte_array_new();
+	encoded = g_byte_array_sized_new(128);
 	rv = der_encode(&asn_DEF_BaseVersion, &bv, metautils_asn1c_write_gba, encoded);
 	asn_DEF_BaseVersion.free_struct(&asn_DEF_BaseVersion, &bv, TRUE);
 
@@ -212,12 +182,9 @@ version_extract_effective_diff(TableSequence_t *seq)
 
 	for (i=0; i<seq->list.count ;i++) {
 		Table_t *table = seq->list.array[i];
-		if (table->name.size != sizeof("main.admin")-1 ||
-				memcmp(table->name.buf, "main.admin", table->name.size-1)) {
-			struct object_version_s *o = version_getslen(1, t,
-					table->name.buf, table->name.size);
-			o->version = 1;
-		}
+		struct object_version_s *o = version_getslen(1, t,
+				table->name.buf, table->name.size);
+		o->version = 1;
 	}
 
 	return t;
