@@ -1,7 +1,7 @@
 /*
 OpenIO SDS sqlx
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -25,6 +25,10 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <sys/resource.h>
 
 #include <metautils/lib/metautils.h>
+#include <metautils/lib/common_variables.h>
+#include <server/server_variables.h>
+#include <sqliterepo/sqliterepo_variables.h>
+
 #include <cluster/lib/gridcluster.h>
 #include <events/oio_events_queue.h>
 #include <events/oio_events_queue_zmq.h>
@@ -48,16 +52,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "sqlx_service.h"
 
-#ifndef SQLX_MAX_TIMER_PER_ROUND
-# define SQLX_MAX_TIMER_PER_ROUND 100
-#endif
-
-#ifndef SQLX_SHUTDOWN_TIMEOUT
-# define SQLX_SHUTDOWN_TIMEOUT (10 * G_TIME_SPAN_SECOND)
-#endif
-
-static volatile gboolean udp_allowed = FALSE;
-
 // common_main hooks
 static struct grid_main_option_s * sqlx_service_get_options(void);
 static const char * sqlx_service_usage(void);
@@ -68,13 +62,16 @@ static void sqlx_service_specific_fini(void);
 static void sqlx_service_specific_stop(void);
 
 // Periodic tasks & thread's workers
+static void _task_probe_repository(gpointer p);
 static void _task_malloc_trim(gpointer p);
 static void _task_expire_bases(gpointer p);
 static void _task_expire_resolver(gpointer p);
-static void _task_react_elections(gpointer p);
+static void _task_react_NONE(gpointer p);
+static void _task_react_TIMERS(gpointer p);
 static void _task_reload_nsinfo(gpointer p);
-static void _task_reload_workers(gpointer p);
-static void _task_reconfigure_events(gpointer p);
+static void _task_reload_peers(gpointer p);
+static void _task_update_stats(gpointer p);
+static GQuark gq_events_health = 0;
 
 static gpointer _worker_queue (gpointer p);
 static gpointer _worker_clients (gpointer p);
@@ -107,6 +104,11 @@ static struct grid_main_option_s common_options[] =
 	{"Announce", OT_STRING, {.str = &SRV.announce},
 		"Announce this IP:PORT couple instead of the TCP endpoint"},
 
+	{"SysConfig", OT_BOOL, {.b = &SRV.config_system},
+		"Load the system configuration and overload the central variables"},
+	{"Config", OT_LIST, {.lst = &SRV.config_paths},
+		"Load the given file and overload the central variables"},
+
 	{"Replicate", OT_BOOL, {.b = &SRV.flag_replicable},
 		"DO NOT USE THIS. This might disable the replication"},
 
@@ -117,37 +119,13 @@ static struct grid_main_option_s common_options[] =
 		"SYNC mode to be applied on non-replicated bases after open "
 			"(0=NONE,1=NORMAL,2=FULL)"},
 
-	{"OpenTimeout", OT_INT64, {.i64=&SRV.open_timeout},
-		"Timeout when opening bases in use by another thread "
-			"(milliseconds). -1 means wait forever, 0 return "
-			"immediately." },
-
-	{"MaxBaseWaiters", OT_UINT, {.u = &SRV.cfg_max_waiters},
-		"Limits the number of threads waiting on a single base" },
-
-	{"MaxBasesHard", OT_UINT, {.u = &SRV.cfg_max_bases_hard},
-		"Absolute max number of cached bases. Won't ever be increased." },
-	{"MaxBases", OT_UINT, {.u = &SRV.cfg_max_bases_soft},
-		"Limits the number of concurrent open bases (0=automatic), bounded "
-			"to MaxBasesHard naturally"	},
-
-	{"MaxPassive", OT_UINT, {.u = &SRV.cfg_max_passive},
-		"Limits the number of concurrent passive connections (0=automatic)" },
-	{"MaxActive", OT_UINT, {.u = &SRV.cfg_max_active},
-		"Limits the number of concurrent active connections (0=automatic)" },
-	{"MaxWorkers", OT_UINT, {.u=&SRV.cfg_max_workers},
-		"Limits the number of worker threads" },
-
-	{"PageSize", OT_UINT, {.u=&SRV.cfg_page_size},
-		"Page size of SQLite databases (0=use sqlite default)" },
-
 	{"CacheEnabled", OT_BOOL, {.b = &SRV.flag_cached_bases},
 		"If set, each base will be cached in a way it won't be accessed"
 			" by several requests in the same time."},
 	{"DeleteEnabled", OT_BOOL, {.b = &SRV.flag_delete_on},
 		"If not set, prevents deleting database files from disk"},
 
-	{NULL, 0, {.i=0}, NULL}
+	{NULL, 0, {.any=0}, NULL}
 };
 
 // repository hooks ------------------------------------------------------------
@@ -196,12 +174,13 @@ _configure_with_arguments(struct sqlx_service_s *ss, int argc, char **argv)
 		GRID_ERROR("Invalid URL to be announced [%s]", ss->announce->str);
 		return FALSE;
 	}
+
+	/* Positional argument: NS */
+
 	if (argc < 2) {
 		GRID_ERROR("Not enough options, see usage.");
 		return FALSE;
 	}
-
-	// Positional arguments
 	gsize s = g_strlcpy(ss->ns_name, argv[0], sizeof(ss->ns_name));
 	if (s >= sizeof(ss->ns_name)) {
 		GRID_WARN("Namespace name too long (given=%"G_GSIZE_FORMAT" max=%u)",
@@ -210,8 +189,7 @@ _configure_with_arguments(struct sqlx_service_s *ss, int argc, char **argv)
 	}
 	GRID_DEBUG("NS configured to [%s]", ss->ns_name);
 
-	ss->lb_world = oio_lb_local__create_world();
-	ss->lb = oio_lb__create();
+	/* Positional argument: VOLUME */
 
 	s = g_strlcpy(ss->volume, argv[1], sizeof(ss->volume));
 	if (s >= sizeof(ss->volume)) {
@@ -221,10 +199,23 @@ _configure_with_arguments(struct sqlx_service_s *ss, int argc, char **argv)
 	}
 	GRID_DEBUG("Volume configured to [%s]", ss->volume);
 
-	/* Load the default ZK url */
+	/* Before loading what remains, first populate the central configuration
+	 * facility. This is a pure side effect but the value might have an impact
+	 * even on the subsequent structure creations. */
+	if (!oio_var_value_with_files(SRV.ns_name, SRV.config_system, SRV.config_paths)) {
+		GRID_ERROR("Unknown NS [%s]", SRV.ns_name);
+		return FALSE;
+	}
+
+	/* Ensure we cache the NS configuration */
+	struct oio_cfg_handle_s *ns_conf = oio_cfg_cache_create();
+	oio_cfg_set_handle(ns_conf);
+
+	/* Load the ZK url.
+	 * We first look for an URL common to all the services, and we will maybe
+	 * override it with a value specific for the file. */
 	ss->zk_url = oio_cfg_get_value(ss->ns_name, OIO_CFG_ZOOKEEPER);
 
-	/* if any, use a specific ZK url for the current service type */
 	do {
 		gchar k[sizeof(OIO_CFG_ZOOKEEPER)+2+LIMIT_LENGTH_SRVTYPE];
 		g_snprintf(k, sizeof(k), "%s.%s", OIO_CFG_ZOOKEEPER, ss->service_config->srvtype);
@@ -236,104 +227,126 @@ _configure_with_arguments(struct sqlx_service_s *ss, int argc, char **argv)
 		if (str) oio_str_reuse(&ss->zk_url, str);
 	} while (0);
 
-	/* Check if UDP is allowed for servers in the /etc/oio/sds.conf files */
-	gchar *str_udp_allowed = oio_cfg_get_value (ss->ns_name, OIO_CFG_UDP_ALLOWED);
-	if (str_udp_allowed) {
-		udp_allowed = oio_str_parse_bool(str_udp_allowed, FALSE);
-		GRID_NOTICE("UDP %s", udp_allowed ? "allowed" : "forbidden");
-		g_free(str_udp_allowed);
+#define BSTR(B) (BOOL(B) ? "on" : "off")
+	GRID_NOTICE("TFO[%d] UDP[%d] OUT[%d]",
+			oio_socket_fastopen, oio_udp_allowed, oio_log_outgoing);
+	return TRUE;
+}
+
+static void
+_patch_configuration_fd(void)
+{
+	/* By design, this function never returns less than 1024. */
+	const guint maxfd = metautils_syscall_count_maxfd();
+	EXTRA_ASSERT(maxfd >= 1024);
+
+	// We keep some FDs for unexpected cases (sqlite sometimes uses
+	// temporary files, even when we ask for memory journals) and for
+	// internal mechanics (notifications, epoll, etc).
+	const guint total = maxfd - 32;
+
+	const guint reserved = sqliterepo_repo_max_bases_soft
+		+ server_fd_max_passive + sqliterepo_fd_max_active;
+
+	// The operator already reserved to many connections, and we cannot
+	// promise these numbers. we hope he/she is aware of his/her job.
+	if (reserved > total) {
+		GRID_WARN("Too many descriptors have been reserved (%u), "
+				"please reconfigure the service or extend the system limit "
+				"(currently set to %u).", reserved, maxfd);
+		if (!sqliterepo_repo_max_bases_soft) {
+			sqliterepo_repo_max_bases_soft = MIN(1024, sqliterepo_repo_max_bases_hard);
+			GRID_WARN("maximum # of bases not set, arbitrarily set to %u",
+					sqliterepo_repo_max_bases_soft);
+		}
+		if (!server_fd_max_passive) {
+			server_fd_max_passive = 64;
+			GRID_WARN("maximum # of incoming cnx not set, arbitrarily set to %u",
+					server_fd_max_passive);
+		}
+		if (!sqliterepo_fd_max_active) {
+			sqliterepo_fd_max_active = 64;
+			GRID_WARN("maximum # of outgoing cnx not set, arbitrarily set to %u",
+					sqliterepo_fd_max_active);
+		}
+	} else {
+		guint available = total - reserved;
+		guint *to_be_set[4] = {NULL, NULL, NULL, NULL};
+		guint limits[3] = {G_MAXUINT, G_MAXUINT, G_MAXUINT};
+		do {
+			guint i = 0;
+			/* When automatically set, keep it low to avoid DoS. */
+			if (!sqliterepo_fd_max_active) {
+				to_be_set[i] = &sqliterepo_fd_max_active;
+				limits[i] = (2 * total) / 100;
+				i++;
+			}
+			if (sqliterepo_repo_max_bases_soft <= 0) {
+				to_be_set[i] = &sqliterepo_repo_max_bases_soft;
+				limits[i] = CLAMP(limits[i],
+						((48 * total) / 100), sqliterepo_repo_max_bases_hard);
+				i++;
+			}
+			if (!server_fd_max_passive) {
+				to_be_set[i] = &server_fd_max_passive;
+				limits[i] = (50 * total) / 100;
+				i++;
+			}
+		} while (0);
+
+		// Fan out all the available FD on each slot that did not reach its max
+		while (available > 0) {
+			gboolean any = FALSE;
+			for (guint i = 0; to_be_set[i] && available > 0; i++) {
+				if (*to_be_set[i] < limits[i]) {
+					(*to_be_set[i]) ++;
+					available--;
+					any = TRUE;
+				}
+			}
+			if (!any)
+				break;
+		}
 	}
 
-	/* Check if the logging of outgoing requests has been activated */
-	gchar *str_log_out = oio_cfg_get_value(ss->ns_name, OIO_CFG_LOG_OUTGOING);
-	if (str_log_out) {
-		oio_log_outgoing = oio_str_parse_bool(str_log_out, FALSE);
-		g_free(str_log_out);
-	}
-	if (oio_log_outgoing)
-		GRID_NOTICE("Outgoing requests log [ON]");
+	GRID_INFO("FD limits set to ACTIVES[%u] PASSIVES[%u] BASES[%u/%u] SYS[%u]",
+			sqliterepo_fd_max_active, server_fd_max_passive,
+			sqliterepo_repo_max_bases_soft, sqliterepo_repo_max_bases_hard,
+			maxfd);
+}
+
+static gboolean
+_patch_and_apply_configuration(void)
+{
+	_patch_configuration_fd();
+
+	if (SRV.server)
+		network_server_reconfigure(SRV.server);
+	if (SRV.repository)
+		sqlx_cache_reconfigure(sqlx_repository_get_cache(SRV.repository));
+	if (SRV.clients_pool)
+		gridd_client_pool_reconfigure(SRV.clients_pool);
 
 	return TRUE;
 }
 
-static gboolean
-_configure_limits(struct sqlx_service_s *ss)
+static void
+_reconfigure_on_SIGHUP(void)
 {
-	guint newval = 0, max = 0, total = 0, available = 0, min = 0;
-	struct rlimit limit = {0, 0};
-#define CONFIGURE_LIMIT(cfg,real) do { \
-	max = MIN(max, available); \
-	newval = (cfg > 0 && cfg < max) ? cfg : max; \
-	newval = newval > min? newval : min; \
-	real = newval; \
-	available -= real; \
-} while (0)
-
-	if (0 != getrlimit(RLIMIT_NOFILE, &limit)) {
-		GRID_ERROR("Max file descriptor unknown: getrlimit error "
-				"(errno=%d) %s", errno, strerror(errno));
-		return FALSE;
-	}
-	if (limit.rlim_cur < 64) {
-		GRID_ERROR("Not enough file descriptors allowed [%lu], "
-				"minimum 64 required", (unsigned long) limit.rlim_cur);
-		return FALSE;
-	}
-
-	// We keep 20 FDs for unexpected cases (sqlite sometimes uses
-	// temporary files, even when we ask for memory journals).
-	total = (limit.rlim_cur - 20);
-	// If user sets outstanding values for the first 2 parameters,
-	// there is still 2% available for the 3rd.
-	max = total * 49 / 100;
-	// Hardcoded in sqlx_repository_configure_maxbases()
-	min = 4;
-
-	available = total;
-
-	/* max_bases cannot be changed at runtime, so we set it first and
-	 * clamp the other limits accordingly.
-	 * For backward compatibility purposes, we configure the SOFT limit
-	 * instead of the hard one, when only "MaxBases" is set, and we deduce
-	 * the HARD limit. This is the limit the users will see in action, but
-	 * with a room for expansion though.
-	 */
-	CONFIGURE_LIMIT(
-			(ss->cfg_max_bases_soft > 0?
-				ss->cfg_max_bases_soft : SQLX_MAX_BASES_PERCENT(total)),
-			ss->max_bases_soft);
-	ss->max_bases_hard = ss->cfg_max_bases_hard > 0 ?
-				ss->cfg_max_bases_hard : SQLX_MAX_BASES;
-
-	// max_passive > max_active permits answering to clients while
-	// managing internal procedures (elections, replications...).
-	CONFIGURE_LIMIT(
-			(ss->cfg_max_passive > 0?
-				ss->cfg_max_passive : SQLX_MAX_PASSIVE_PERCENT(total)),
-			ss->max_passive);
-	CONFIGURE_LIMIT(
-			(ss->cfg_max_active > 0?
-				ss->cfg_max_active : SQLX_MAX_ACTIVE_PERCENT(total)),
-			ss->max_active);
-
-	GRID_INFO("Limits set to ACTIVES[%u] PASSIVES[%u] BASES[%u/%u] "
-			"fd=%u/%u page_size=%u",
-			ss->max_active, ss->max_passive,
-			ss->max_bases_soft, ss->max_bases_hard,
-			ss->max_active + ss->max_passive + ss->max_bases_soft,
-			(guint)limit.rlim_cur, ss->cfg_page_size);
-
-	return TRUE;
-#undef CONFIGURE_LIMIT
+	GRID_NOTICE("SIGHUP! Reconfiguring...");
+	oio_var_reset_all();
+	oio_var_value_with_files(SRV.ns_name, SRV.config_system, SRV.config_paths);
+	_patch_and_apply_configuration();
 }
 
 static gboolean
 _init_configless_structures(struct sqlx_service_s *ss)
 {
-	if (!(ss->server = network_server_init())
+	if (!(ss->lb_world = oio_lb_local__create_world())
+			|| !(ss->lb = oio_lb__create())
+			|| !(ss->server = network_server_init())
 			|| !(ss->dispatcher = transport_gridd_build_empty_dispatcher())
 			|| !(ss->clients_pool = gridd_client_pool_create())
-			|| !(ss->clients_factory = gridd_client_factory_create())
 			|| !(ss->resolver = hc_resolver_create())
 			|| !(ss->gtq_admin = grid_task_queue_create("admin"))
 			|| !(ss->gtq_reload = grid_task_queue_create("reload"))) {
@@ -347,8 +360,66 @@ _init_configless_structures(struct sqlx_service_s *ss)
 static gboolean
 _configure_peering (struct sqlx_service_s *ss)
 {
-	ss->peering = sqlx_peering_factory__create_direct
-		(ss->clients_pool, ss->clients_factory);
+	ss->peering = sqlx_peering_factory__create_direct(ss->clients_pool);
+	return TRUE;
+}
+
+static gboolean
+_configure_zk_shard(struct sqlx_service_s *ss,
+		const char *realprefix, const char *zk_url)
+{
+	struct sqlx_sync_s *ssync = sqlx_sync_create(zk_url);
+	if (!ssync)
+		return FALSE;
+
+	g_ptr_array_add(ss->sync_tab, ssync);
+
+	sqlx_sync_set_prefix(ssync, realprefix);
+
+	sqlx_sync_set_hash(ssync,
+										 ss->service_config->zk_hash_width,
+										 ss->service_config->zk_hash_depth);
+
+	GError *err = sqlx_sync_open(ssync);
+	if (err != NULL) {
+		GRID_WARN("SYNC init error [%s]: (%d) %s",
+							zk_url, err->code, err->message);
+		g_clear_error(&err);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+static gboolean
+_configure_zk_shard_muxed(struct sqlx_service_s *ss,
+		guint mux_factor, const char *realprefix, const char *zk_url)
+{
+	if (mux_factor <= 0)
+		return FALSE;
+
+	for (guint i=0; i<mux_factor ;++i) {
+		if (!_configure_zk_shard(ss, realprefix, zk_url))
+			return FALSE;
+	}
+	return TRUE;
+}
+
+static gboolean
+_configure_multi_zk(struct sqlx_service_s *ss,
+		guint mux_factor, const char *realprefix, const char *zk_url)
+{
+	if (mux_factor <= 0)
+		return FALSE;
+
+	gchar **shards = g_strsplit(zk_url, OIO_CSV_SEP2, -1);
+	STRINGV_STACKIFY(shards);
+	for (gchar **shard=shards; shards && *shard ;++shard) {
+		if (!oio_str_is_set(*shard))
+			continue;
+		if (!_configure_zk_shard_muxed(ss, mux_factor, realprefix, *shard))
+			return FALSE;
+	}
 	return TRUE;
 }
 
@@ -360,28 +431,152 @@ _configure_synchronism(struct sqlx_service_s *ss)
 		return TRUE;
 	}
 
-	gboolean shuffle = oio_cfg_get_bool(ss->ns_name, OIO_CFG_ZK_SHUFFLED, TRUE);
-	GRID_INFO("ZK cnx string shuffling [%s]", shuffle ? "ON" : "OFF");
-	ss->sync = sqlx_sync_create(ss->zk_url, shuffle);
-	if (!ss->sync)
-		return FALSE;
+	ss->sync_tab = g_ptr_array_new();
 
 	gchar *realprefix = g_strdup_printf("/hc/ns/%s/%s", ss->ns_name,
 			ss->service_config->zk_prefix);
-	sqlx_sync_set_prefix(ss->sync, realprefix);
-	g_free(realprefix);
+	STRING_STACKIFY(realprefix);
 
-	sqlx_sync_set_hash(ss->sync, ss->service_config->zk_hash_width,
-			ss->service_config->zk_hash_depth);
+	return _configure_multi_zk(ss, sqliterepo_zk_mux_factor,
+																	 realprefix, ss->zk_url);
+}
 
-	GError *err = sqlx_sync_open(ss->sync);
-	if (err != NULL) {
-		GRID_WARN("SYNC init error: (%d) %s", err->code, err->message);
-		g_clear_error(&err);
-		return FALSE;
+static gchar **
+filter_services(struct sqlx_service_s *ss, gchar **s, const gchar *type)
+{
+	gboolean matched = FALSE;
+	GPtrArray *tmp = g_ptr_array_new();
+	for (; *s ;s++) {
+		struct meta1_service_url_s *u = meta1_unpack_url(*s);
+		gboolean srvtype_matched;
+		gchar *host;
+		if (u) {
+			srvtype_matched = !strcmp(type, u->srvtype);
+			host = u->host;
+		} else {
+			srvtype_matched = TRUE;
+			host = *s;
+		}
+		if (srvtype_matched) {
+			if (!g_ascii_strcasecmp(host, ss->url->str))
+				matched = TRUE;
+			else
+				g_ptr_array_add(tmp, g_strdup(host));
+		}
+		meta1_service_url_clean(u);
 	}
 
-	return TRUE;
+	gchar **out = (gchar**)metautils_gpa_to_array (tmp, TRUE);
+	if (matched)
+		return out;
+	g_strfreev (out);
+	return NULL;
+}
+
+GError *
+sqlx_service_resolve_peers(struct sqlx_service_s *ss,
+		const struct sqlx_name_s *n, gboolean nocache, gchar ***result)
+{
+	EXTRA_ASSERT(ss != NULL);
+	EXTRA_ASSERT(result != NULL);
+
+	GError *err = NULL;
+	gint64 seq = 1;
+	gint retry = 1;
+
+	*result = NULL;
+
+	struct oio_url_s *u = oio_url_empty ();
+	oio_url_set(u, OIOURL_NS, ss->ns_name);
+
+	if (!sqlx_name_extract(n, u, ss->service_config->srvtype, &seq)) {
+		err = BADREQ("Invalid type name: '%s'", n->type);
+	} else {
+label_retry:
+		if (nocache)
+			hc_decache_reference_service(ss->resolver, u, n->type);
+
+		gchar **peers = NULL;
+		err = hc_resolve_reference_service(ss->resolver, u, n->type, &peers);
+		if (err == NULL) {
+			EXTRA_ASSERT(peers != NULL);
+			*result = peers;
+			peers = NULL;
+		} else {
+			EXTRA_ASSERT(peers == NULL);
+			if (retry > 0 && err->code == CODE_RANGE_NOTFOUND) {
+				// We may have asked the wrong meta1
+				hc_decache_reference(ss->resolver, u);
+				retry --;
+				goto label_retry;
+			}
+			g_prefix_error(&err, "Peer resolution error: ");
+		}
+	}
+
+	oio_url_pclean (&u);
+	return err;
+}
+
+// TODO: replace `nocache` by flags
+static GError *
+_get_peers_wrapper(struct sqlx_service_s *ss, const struct sqlx_name_s *name,
+		gboolean nocache, gchar ***result)
+{
+	EXTRA_ASSERT(ss != NULL);
+	EXTRA_ASSERT(result != NULL);
+
+	gchar **peers = NULL;
+	GError *err = NULL;
+
+	*result = NULL;
+
+	// Try to read peers from the database
+	if (!nocache) {
+		err = sqlx_repository_get_peers2(ss->repository, name, &peers);
+		if (err) {
+			if (err->code != CODE_CONTAINER_NOTFOUND) {
+				GRID_INFO("Failed to get_peers() from local database: (%d) %s",
+						err->code, err->message);
+			}
+			g_clear_error(&err);
+			EXTRA_ASSERT(peers == NULL);
+		}
+	}
+
+label_retry:
+	// Try to read peers from the upper-level service
+	if (!peers || !oio_str_is_set(*peers)) {
+		oio_str_cleanv(&peers);
+		err = ss->service_config->get_peers(ss, name, nocache, &peers);
+	}
+
+	if (!err) {
+		EXTRA_ASSERT(peers != NULL);
+		*result = filter_services(ss, peers, name->type);
+		oio_str_cleanv(&peers);
+		if (!*result) {
+			// If cache was enabled, we can retry without cache
+			if (!nocache) {
+				nocache = TRUE;
+				goto label_retry;
+			} else {
+				err = NEWERROR(CODE_CONTAINER_NOTFOUND, "Base not managed");
+			}
+		}
+	} else {
+		EXTRA_ASSERT(peers == NULL);
+		if (err->code == CODE_RANGE_NOTFOUND && !nocache) {
+			// We may have asked the wrong upper-level service
+			// TODO: call hc_decache_reference(ss->resolver, url);
+			// and remove the retry loop from meta2
+			nocache = TRUE;
+			goto label_retry;
+		}
+	}
+
+	EXTRA_ASSERT((err != NULL) ^ (*result != NULL));
+	return err;
 }
 
 static gboolean
@@ -393,8 +588,9 @@ _configure_replication(struct sqlx_service_s *ss)
 	replication_config.ctx = ss;
 	replication_config.get_local_url = _get_url;
 	replication_config.get_version = _get_version;
-	replication_config.get_peers = (GError* (*)(gpointer, const struct sqlx_name_s*,
-				gboolean, gchar ***)) ss->service_config->get_peers;
+	replication_config.get_peers = (GError* (*)(gpointer,
+			const struct sqlx_name_s*, gboolean, gchar ***))
+			_get_peers_wrapper;
 
 	GError *err = election_manager_create(&replication_config,
 			&ss->election_manager);
@@ -405,6 +601,7 @@ _configure_replication(struct sqlx_service_s *ss)
 		return FALSE;
 	}
 
+	election_manager_dump_delays();
 	return TRUE;
 }
 
@@ -415,16 +612,9 @@ _configure_backend(struct sqlx_service_s *ss)
 	repository_config.flags = 0;
 	repository_config.flags |= ss->flag_delete_on ? SQLX_REPO_DELETEON : 0;
 	repository_config.flags |= ss->flag_cached_bases ? 0 : SQLX_REPO_NOCACHE;
-	repository_config.flags |= ss->flag_autocreate ? SQLX_REPO_AUTOCREATE : 0;
 	repository_config.sync_solo = ss->sync_mode_solo;
 	repository_config.sync_repli = ss->sync_mode_repli;
 
-	repository_config.page_size = SQLX_DEFAULT_PAGE_SIZE;
-	if (ss->cfg_page_size >= 512)
-		repository_config.page_size = ss->cfg_page_size;
-
-	repository_config.max_bases = ss->max_bases_hard;
-
 	GError *err = sqlx_repository_init(ss->volume, &repository_config,
 			&ss->repository);
 	if (err) {
@@ -434,10 +624,6 @@ _configure_backend(struct sqlx_service_s *ss)
 		return FALSE;
 	}
 
-	sqlx_repository_configure_maxbases(SRV.repository, SRV.max_bases_soft);
-	sqlx_cache_set_max_waiting(sqlx_repository_get_cache(SRV.repository),
-			SRV.cfg_max_waiters);
-
 	err = sqlx_repository_configure_type(ss->repository,
 			ss->service_config->srvtype, ss->service_config->schema);
 
@@ -447,9 +633,6 @@ _configure_backend(struct sqlx_service_s *ss)
 		return FALSE;
 	}
 
-	sqlx_repository_configure_open_timeout (ss->repository,
-			ss->open_timeout * G_TIME_SPAN_MILLISECOND);
-
 	sqlx_repository_configure_hash (ss->repository,
 			ss->service_config->repo_hash_width,
 			ss->service_config->repo_hash_depth);
@@ -462,13 +645,15 @@ static gboolean
 _configure_tasks(struct sqlx_service_s *ss)
 {
 	grid_task_queue_register(ss->gtq_reload, 5, _task_reload_nsinfo, NULL, ss);
-	grid_task_queue_register(ss->gtq_reload, 5, _task_reload_workers, NULL, ss);
-	grid_task_queue_register(ss->gtq_reload, 5, _task_reconfigure_events, NULL, ss);
+	grid_task_queue_register(ss->gtq_reload, 5, _task_reload_peers, NULL, ss);
+	grid_task_queue_register(ss->gtq_reload, 5, _task_probe_repository, NULL, ss);
 
 	grid_task_queue_register(ss->gtq_admin, 1, _task_expire_bases, NULL, ss);
 	grid_task_queue_register(ss->gtq_admin, 1, _task_expire_resolver, NULL, ss);
-	grid_task_queue_register(ss->gtq_admin, 1, _task_react_elections, NULL, ss);
-	grid_task_queue_register(ss->gtq_admin, 3600, _task_malloc_trim, NULL, ss);
+	grid_task_queue_register(ss->gtq_admin, 1, _task_react_NONE, NULL, ss);
+	grid_task_queue_register(ss->gtq_admin, 1, _task_react_TIMERS, NULL, ss);
+	grid_task_queue_register(ss->gtq_admin, 1, _task_malloc_trim, NULL, ss);
+	grid_task_queue_register(ss->gtq_admin, 5, _task_update_stats, NULL, ss);
 
 	return TRUE;
 }
@@ -481,7 +666,7 @@ _configure_events_queue (struct sqlx_service_s *ss)
 		return TRUE;
 	}
 
-	gchar *url =  oio_cfg_get_eventagent (SRV.ns_name);
+	gchar *url = oio_cfg_get_eventagent (SRV.ns_name);
 	STRING_STACKIFY (url);
 
 	if (!url) {
@@ -527,7 +712,7 @@ sqlx_service_action(void)
 {
 	GError *err = NULL;
 
-	if (oio_cache_avoid_on_error)
+	if (oio_client_cache_errors)
 		GRID_NOTICE("Faulty peers avoidance: ENABLED");
 
 	if (!SRV.flag_nolock) {
@@ -539,12 +724,11 @@ sqlx_service_action(void)
 
 	oio_server_volume = SRV.volume;
 
-	gridd_client_pool_set_max(SRV.clients_pool, SRV.max_active);
-	network_server_set_maxcnx(SRV.server, SRV.max_passive);
-
 	election_manager_set_peering(SRV.election_manager, SRV.peering);
-	if (SRV.sync)
-		election_manager_set_sync(SRV.election_manager, SRV.sync);
+	if (SRV.sync_tab && SRV.sync_tab->len > 0) {
+		for (guint i=0; i<SRV.sync_tab->len ;++i)
+			election_manager_add_sync(SRV.election_manager, SRV.sync_tab->pdata[i]);
+	}
 	sqlx_repository_set_elections(SRV.repository, SRV.election_manager);
 
 	grid_task_queue_fire(SRV.gtq_reload);
@@ -581,14 +765,14 @@ sqlx_service_action(void)
 	if (!grid_main_is_running())
 		return;
 
-	if (udp_allowed) {
+	if (oio_udp_allowed) {
 		int fd_udp = network_server_first_udp(SRV.server);
 		GRID_DEBUG("UDP socket fd=%d", fd_udp);
 		sqlx_peering_direct__set_udp(SRV.peering, fd_udp);
 	}
 
 	/* SERVER/GRIDD main run loop */
-	if (NULL != (err = network_server_run(SRV.server)))
+	if (NULL != (err = network_server_run(SRV.server, _reconfigure_on_SIGHUP)))
 		return _action_report_error(err, "GRIDD run failure");
 }
 
@@ -607,10 +791,10 @@ sqlx_service_specific_stop(void)
 static gboolean
 sqlx_service_configure(int argc, char **argv)
 {
-	return _configure_limits(&SRV)
-		&& _init_configless_structures(&SRV)
+	const gboolean rc = _init_configless_structures(&SRV)
 		&& _configure_with_arguments(&SRV, argc, argv)
 		/* NS now known! */
+		&& _patch_and_apply_configuration()
 		&& _configure_synchronism(&SRV)
 		&& _configure_peering(&SRV)
 		&& _configure_replication(&SRV)
@@ -620,23 +804,15 @@ sqlx_service_configure(int argc, char **argv)
 		&& _configure_events_queue(&SRV)
 		&& (!SRV.service_config->post_config
 				|| SRV.service_config->post_config(&SRV));
+	_patch_and_apply_configuration();
+	return rc;
 }
 
 static void
 sqlx_service_set_defaults(void)
 {
-	SRV.max_elections_timers_per_round = SQLX_MAX_TIMER_PER_ROUND;
-	SRV.open_timeout = DEFAULT_CACHE_OPEN_TIMEOUT / G_TIME_SPAN_MILLISECOND;
-
-	SRV.cfg_max_waiters = SQLX_MAX_WAITING;
-	SRV.cfg_max_bases_soft = 0;
-	SRV.cfg_max_bases_hard = SQLX_MAX_BASES;
-	SRV.cfg_max_passive = 0;
-	SRV.cfg_max_active = 0;
-	SRV.cfg_max_workers = 200;
-	SRV.cfg_page_size = SQLX_DEFAULT_PAGE_SIZE;
+	SRV.config_system = TRUE;
 	SRV.flag_replicable = TRUE;
-	SRV.flag_autocreate = TRUE;
 	SRV.flag_delete_on = TRUE;
 	SRV.flag_cached_bases = TRUE;
 
@@ -645,6 +821,8 @@ sqlx_service_set_defaults(void)
 
 	if (SRV.service_config->set_defaults)
 		SRV.service_config->set_defaults(&SRV);
+
+	gq_events_health = g_quark_from_static_string("gauge event.health");
 }
 
 static void
@@ -677,9 +855,11 @@ sqlx_service_specific_fini(void)
 	}
 	if (election_manager_is_operational(SRV.election_manager))
 		election_manager_exit_all(SRV.election_manager,
-				SQLX_SHUTDOWN_TIMEOUT, TRUE);
-	if (SRV.sync)
-		sqlx_sync_close(SRV.sync);
+				sqliterepo_server_exit_ttl, TRUE);
+	if (SRV.sync_tab) {
+		for (guint i=0; i<SRV.sync_tab->len ;++i)
+			sqlx_sync_close(SRV.sync_tab->pdata[i]);
+	}
 	if (SRV.peering)
 		sqlx_peering__destroy(SRV.peering);
 
@@ -730,19 +910,16 @@ sqlx_service_specific_fini(void)
 		SRV.clients_pool = NULL;
 	}
 
-	if (SRV.clients_factory) {
-		gridd_client_factory_clean(SRV.clients_factory);
-		SRV.clients_factory = NULL;
-	}
-
 	// Must be freed after SRV.clients_pool
 	if (SRV.election_manager) {
 		election_manager_clean(SRV.election_manager);
 		SRV.election_manager = NULL;
 	}
-	if (SRV.sync) {
-		sqlx_sync_clear(SRV.sync);
-		SRV.sync = NULL;
+	if (SRV.sync_tab) {
+		for (guint i=0; i<SRV.sync_tab->len ;++i)
+			sqlx_sync_clear(SRV.sync_tab->pdata[i]);
+		g_ptr_array_free(SRV.sync_tab, TRUE);
+		SRV.sync_tab = NULL;
 	}
 
 	if (SRV.lb)
@@ -766,6 +943,8 @@ sqlx_service_specific_fini(void)
 		g_free (all_options);
 		all_options = NULL;
 	}
+
+	oio_cfg_set_handle(NULL);
 }
 
 static guint
@@ -853,11 +1032,58 @@ _worker_clients(gpointer p)
 	return p;
 }
 
+#define HEXA "0123456789ABCDEF"
+
 static void
-_task_malloc_trim(gpointer p)
+_task_probe_repository(gpointer p)
 {
-	(void) p;
-	malloc_trim (PERIODIC_MALLOC_TRIM_SIZE);
+	struct sqlx_service_s *ss = PSRV(p);
+	int rc, errsav;
+	GError *err = NULL;
+	gchar path[PATH_MAX], subdir[16], filename[16];
+
+	oio_str_randomize(subdir, sizeof(subdir), HEXA);
+	oio_str_randomize(filename, sizeof(filename), HEXA);
+
+	/* Try a directory creation */
+	g_strlcpy(path, ss->volume, sizeof(path));
+	g_strlcat(path, "/probe-", sizeof(path));
+	g_strlcat(path, subdir, sizeof(path));
+	GRID_DEBUG("Probing directory %s", path);
+	rc = g_mkdir(path, 0755);
+	errsav = errno;
+	(void) g_rmdir(path);
+	if (rc != 0) {
+		GRID_WARN("I/O error on %s: (%d) %s", path, errsav, strerror(errsav));
+		grid_daemon_notify_io_status(ss->dispatcher, FALSE);
+		return;
+	}
+
+	/* Try a file creation */
+	g_strlcpy(path, ss->volume, sizeof(path));
+	g_strlcat(path, "/probe-", sizeof(path));
+	g_strlcat(path, filename, sizeof(path));
+	GRID_DEBUG("Probing file %s", path);
+	rc = g_file_set_contents(path, "", 0, &err);
+	(void) g_unlink(path);
+	if (!rc) {
+		GRID_WARN("I/O error on %s: (%d) %s", path, err->code, err->message);
+		g_clear_error(&err);
+		grid_daemon_notify_io_status(ss->dispatcher, FALSE);
+		return;
+	}
+
+	grid_daemon_notify_io_status(ss->dispatcher, TRUE);
+}
+
+static void
+_task_malloc_trim(gpointer p UNUSED)
+{
+	VARIABLE_PERIOD_DECLARE();
+	if (VARIABLE_PERIOD_SKIP(sqlx_periodic_malloctrim_period))
+		return;
+
+	malloc_trim (sqlx_periodic_malloctrim_size);
 }
 
 static void
@@ -868,7 +1094,13 @@ _task_expire_bases(gpointer p)
 
 	struct sqlx_cache_s *cache = sqlx_repository_get_cache(PSRV(p)->repository);
 	if (cache != NULL) {
-		guint count = sqlx_cache_expire(cache, 100, 500 * G_TIME_SPAN_MILLISECOND);
+
+		VARIABLE_PERIOD_DECLARE();
+		if (VARIABLE_PERIOD_SKIP(sqlx_periodic_decache_period))
+			return;
+
+		guint count = sqlx_cache_expire(cache,
+				sqlx_periodic_decache_max_bases, sqlx_periodic_decache_max_delay);
 		if (count)
 			GRID_DEBUG("Expired %u bases", count);
 	}
@@ -880,28 +1112,46 @@ _task_expire_resolver(gpointer p)
 	if (!grid_main_is_running ())
 		return;
 
-	guint count = hc_resolver_expire(PSRV(p)->resolver);
-	if (count)
-		GRID_DEBUG("Expired %u entries from the resolver cache", count);
+	guint count_expire = hc_resolver_expire(PSRV(p)->resolver);
+	guint count_purge = hc_resolver_purge (PSRV(p)->resolver);
+	if (count_expire || count_purge) {
+		GRID_DEBUG ("Resolver: expired %u, purged %u",
+				count_expire, count_purge);
+	}
+}
+
+#define _task_alert_message(action) \
+	"Action %s on %u elections took %"G_GINT64_FORMAT"ms", \
+	#action, count, t / G_TIME_SPAN_MILLISECOND
+
+#define _task_timed_action(action,period,delay) do { \
+	if (!grid_main_is_running () || !PSRV(p)->flag_replicable) return; \
+	VARIABLE_PERIOD_DECLARE(); \
+	if (VARIABLE_PERIOD_SKIP(sqliterepo_election_task_##period)) return; \
+	gint64 t = oio_ext_monotonic_time(); \
+	guint count = election_manager_play_##action (PSRV(p)->election_manager); \
+	t = oio_ext_monotonic_time() - t; \
+	if (t > sqliterepo_election_task_##delay) { \
+		GRID_WARN(_task_alert_message(action)); \
+	} else { \
+		GRID_DEBUG(_task_alert_message(action)); \
+	} \
+} while (0)
+
+static void
+_task_react_NONE(gpointer p)
+{
+	_task_timed_action(exits, EXIT_period, EXIT_alert);
 }
 
 static void
-_task_react_elections(gpointer p)
+_task_react_TIMERS(gpointer p)
 {
-	if (!grid_main_is_running ())
-		return;
-	if (!PSRV(p)->flag_replicable)
-		return;
-
-	gint64 t = oio_ext_monotonic_time();
-	guint count = election_manager_play_timers (PSRV(p)->election_manager,
-			PSRV(p)->max_elections_timers_per_round);
-	t = t - oio_ext_monotonic_time();
-
-	if (count || t > (500*G_TIME_SPAN_MILLISECOND)) {
-		GRID_DEBUG("Reacted %u elections in %"G_GINT64_FORMAT"ms",
-				count, t / G_TIME_SPAN_MILLISECOND);
-	}
+	_task_timed_action(timers_FAILED, TIMER_period, TIMER_alert);
+	_task_timed_action(timers_DELAYED_MASTER, TIMER_period, TIMER_alert);
+	_task_timed_action(timers_DELAYED_SLAVE, TIMER_period, TIMER_alert);
+	_task_timed_action(timers_MASTER, TIMER_period, TIMER_alert);
+	_task_timed_action(timers_SLAVE, TIMER_period, TIMER_alert);
 }
 
 static void
@@ -926,51 +1176,87 @@ _task_reload_nsinfo(gpointer p)
 }
 
 static void
-_task_reload_workers(gpointer p)
+_task_update_stats(gpointer p)
 {
-	if (!grid_main_is_running ())
-		return;
-	if (!PSRV(p)->nsinfo)
+	if (!grid_main_is_running() || !PSRV(p)->events_queue)
 		return;
 
-	gint64 max_workers = namespace_info_get_srv_param_i64 (PSRV(p)->nsinfo,
-			NULL, PSRV(p)->service_config->srvtype, "max_workers",
-			SRV.cfg_max_workers);
-	network_server_set_max_workers(PSRV(p)->server, (guint) max_workers);
+	gint64 health = oio_events_queue__get_health(PSRV(p)->events_queue);
+	network_server_stat_push2(PSRV(p)->server, FALSE,
+			gq_events_health, health, 0, 0);
+}
+
+static gboolean
+_srv_is_down(struct service_info_s *si)
+{
+	EXTRA_ASSERT(si != NULL);
+
+	if (!si->tags)
+		return FALSE;
+
+	struct service_tag_s *tag =
+		service_info_get_tag(si->tags, NAME_TAGNAME_RAWX_UP);
+	if (!tag)
+		return FALSE;
+
+	gboolean up = TRUE;
+	service_tag_get_value_boolean(tag, &up, NULL);
+	return !up;
+}
+
+static gchar *
+_srv_url (struct service_info_s *si)
+{
+	EXTRA_ASSERT(si != NULL);
+	gchar tmp[STRLEN_ADDRINFO];
+	grid_addrinfo_to_string(&si->addr, tmp, sizeof(tmp));
+	return g_strdup(tmp);
+}
+
+static gchar **
+_filter_down_hosts(GSList *l)
+{
+	GPtrArray *tmp = g_ptr_array_new();
+	for (; l ;l=l->next) {
+		if (_srv_is_down(l->data))
+			g_ptr_array_add(tmp, _srv_url(l->data));
+	}
+	g_ptr_array_add(tmp, NULL);
+	return (gchar**) g_ptr_array_free(tmp, FALSE);
 }
 
 static void
-_task_reconfigure_events (gpointer p)
+_task_reload_peers(gpointer p)
 {
+	VARIABLE_PERIOD_DECLARE();
+
 	if (!grid_main_is_running ())
 		return;
-	if (!p || !PSRV(p)->events_queue)
-		return;
 
-	struct namespace_info_s *ni = PSRV(p)->nsinfo;
-	if (!ni || !ni->options)
-		return;
-
-	gint64 i64 = namespace_info_get_srv_param_i64(ni, NULL,
-			PSRV(p)->service_config->srvtype,
-			OIO_CFG_EVTQ_MAXPENDING,
-			OIO_EVTQ_MAXPENDING);
-	GRID_TRACE("Looking for [%s]: %"G_GINT64_FORMAT,
-			OIO_CFG_EVTQ_MAXPENDING, i64);
-
-	if (i64 >= 0 && i64 < G_MAXUINT) {
-		guint u = (guint) i64;
-		oio_events_queue__set_max_pending (PSRV(p)->events_queue, u);
+	const char * nsname = PSRV(p)->ns_name;
+	const char * srvtype = PSRV(p)->service_config->srvtype;
+	GSList *allsrv = NULL;
+	GError *err = conscience_get_services(nsname, srvtype, FALSE, &allsrv);
+	if (err) {
+		GRID_WARN("Failed to reload the list of [%s/%s]: (%d) %s",
+				nsname, srvtype, err->code, err->message);
+		g_clear_error(&err);
+	} else {
+		gchar **down = _filter_down_hosts(allsrv);
+		gridd_client_learn_peers_down((const char * const *)down);
+		if (down && *down) {
+			const gsize len = g_strv_length(down);
+			if (VARIABLE_PERIOD_SKIP(180)) {
+				GRID_DEBUG("Loaded %" G_GSIZE_FORMAT " down %s", len, srvtype);
+			} else {
+				/* once per 15 minutes */
+				GRID_NOTICE("Loaded %" G_GSIZE_FORMAT " down %s", len, srvtype);
+			}
+		}
+		g_strfreev(down);
 	}
 
-	i64 = namespace_info_get_srv_param_i64(ni, NULL,
-			PSRV(p)->service_config->srvtype,
-			OIO_CFG_EVTQ_BUFFER_DELAY,
-			OIO_EVTQ_BUFFER_DELAY);
-	if (i64 >= 0 && i64 < 3600) {
-		oio_events_queue__set_buffering(PSRV(p)->events_queue,
-				i64 * G_TIME_SPAN_SECOND);
-	}
+	g_slist_free_full(allsrv, (GDestroyNotify)service_info_clean);
 }
 
 static void
@@ -978,7 +1264,7 @@ _reload_lb_service_types(struct oio_lb_world_s *lbw, struct oio_lb_s *lb,
 		gchar **srvtypes, GPtrArray *tabsrv, GPtrArray *taberr)
 {
 	struct service_update_policies_s *pols = service_update_policies_create();
-	gchar *pols_cfg = gridcluster_get_service_update_policy(SRV.nsinfo);
+	gchar *pols_cfg = oio_var_get_string(oio_ns_service_update_policy);
 	service_update_reconfigure(pols, pols_cfg);
 	g_free(pols_cfg);
 
@@ -1065,7 +1351,7 @@ _reload_lb_world(struct oio_lb_world_s *lbw, struct oio_lb_s *lb)
 		GSList *srv = NULL;
 		GError *e = conscience_get_services(SRV.ns_name, srvtype, FALSE, &srv);
 		if (e) {
-			GRID_WARN("Failed to load the list of [%s] in NS=%s", SRV.ns_name, srvtype);
+			GRID_WARN("Failed to load the list of [%s] in NS=%s", srvtype, SRV.ns_name);
 			any_loading_error = TRUE;
 		}
 		g_ptr_array_add(tabsrv, srv);
@@ -1111,7 +1397,7 @@ sqlx_task_reload_lb (struct sqlx_service_s *ss)
 		GRID_WARN("Failed to reload LB world: %s", err->message);
 		g_clear_error(&err);
 	} else {
-		ADAPTIVE_PERIOD_ONSUCCESS(10);
+		ADAPTIVE_PERIOD_ONSUCCESS(oio_sqlx_lb_refresh_period);
 	}
 }
 
