@@ -1,7 +1,7 @@
 /*
 OpenIO SDS sqlx
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -22,12 +22,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #define PSRV(P) ((struct sqlx_service_s*)(P))
 
-#define SQLX_MAX_BASES_PERCENT(max)   (max * 30) / 100
-#define SQLX_MAX_PASSIVE_PERCENT(max) (max * 40) / 100
-#define SQLX_MAX_ACTIVE_PERCENT(max)  (max * 30) / 100
-
 struct election_manager_s;
-struct gridd_client_factory_s;
 struct gridd_client_pool_s;
 struct gridd_request_dispatcher_s;
 struct grid_single_rrd_s;
@@ -88,8 +83,17 @@ struct sqlx_service_s
 	GString *announce;
 	gchar *zk_url;
 
+	/* Should the service load an additional list of configuration files to
+	 * supersede the values in place in the central configuration facility */
+	GSList *config_paths;
+
+	/* Should the service load the system configuration and feed the central
+	 * configuration facility. */
+	gboolean config_system;
+
+	GPtrArray *sync_tab;
+
 	struct sqlx_repository_s *repository;
-	struct sqlx_sync_s *sync;
 	struct sqlx_peering_s *peering;
 	struct election_manager_s *election_manager;
 	struct network_server_s *server;
@@ -97,7 +101,6 @@ struct sqlx_service_s
 	struct hc_resolver_s *resolver;
 	struct oio_lb_s *lb;
 	struct oio_lb_world_s *lb_world;
-	GHashTable *lb_pools;
 
 	struct oio_events_queue_s *events_queue;
 	GThread *thread_queue;
@@ -113,34 +116,14 @@ struct sqlx_service_s
 	struct grid_task_queue_s *gtq_admin;
 	GThread *thread_admin;
 
-	struct gridd_client_factory_s *clients_factory;
 	struct gridd_client_pool_s *clients_pool;
 	GThread *thread_client;
 
-	/* This is configured during the "configure" step, and can be overriden
-	   in the _post_config hook. */
-	gint64 open_timeout;
-	guint max_bases_soft;
-	guint max_bases_hard;
-	guint max_passive;
-	guint max_active;
-	guint max_elections_timers_per_round;
-
 	//-------------------------------------------------------------------
 	// Variables used during the startup time of the server, but not used
 	// anymore after that.
 	//-------------------------------------------------------------------
 
-	guint cfg_max_waiters;
-	guint cfg_max_bases_hard;
-	guint cfg_max_bases_soft;
-
-	guint cfg_max_passive;
-	guint cfg_max_active;
-	guint cfg_max_workers;
-
-	guint cfg_page_size;
-
 	guint sync_mode_repli;
 	guint sync_mode_solo;
 
@@ -150,9 +133,6 @@ struct sqlx_service_s
 	// Are DB deletions allowed ?
 	gboolean flag_delete_on;
 
-	// Are DB autocreations enabled?
-	gboolean flag_autocreate;
-
 	// Turn to TRUE to avoid locking the repository volume
 	gboolean flag_nolock;
 
@@ -173,6 +153,14 @@ extern int sqlite_service_main(int argc, char **argv,
 
 void sqlx_service_set_custom_options (struct grid_main_option_s *options);
 
+GError* sqlx_reload_lb_service_types(struct oio_lb_world_s *lbw,
+		struct oio_lb_s *lb, GSList *list_srvtypes);
+
+/** Use the resolver to find peers.
+ *  Only for services registered in meta1 (e.g. meta2 and sqlx). */
+GError * sqlx_service_resolve_peers(struct sqlx_service_s *ss,
+		const struct sqlx_name_s *n, gboolean nocache, gchar ***result);
+
 // FIXME: this is only used in meta1
 /** Reloads the optional (oio_lb_s*). Exposed to let the
  * server enable it in its post-config hook. This is destined to
