@@ -1,21 +1,20 @@
 #!/usr/bin/env python
 
-# test_oiosds.py, a script performing functional tests of OpenIO SDS's C SDK
-# Copyright (C) 2015 OpenIO, original work as part of
-# OpenIO Software Defined Storage
+# OpenIO SDS functional tests
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 #
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License as
-# published by the Free Software Foundation, either version 3 of the
-# License, or (at your option) any later version.
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3.0 of the License, or (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful,
+# This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Affero General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU Lesser General Public
+# License along with this library.
 
 import sys
 import json
@@ -242,14 +241,16 @@ def test_list_ok(lib):
 
         (("/v3.0/NS/container/list?acct=ACCT&ref=JFS", {}, ""),
          (200, {}, json.dumps({
-                "objects": [{"name": x, "hash": "0000", "size": 0, "ver": 1}
+                "objects": [{"name": x, "hash": "0000",
+                             "size": 0, "version": 1}
                             for x in names],
                 "prefixes": [],
                               }))),
 
         (("/v3.0/NS/container/list?acct=ACCT&ref=JFS&prefix=pla", {}, ""),
          (200, {}, json.dumps({
-                "objects": [{"name": x, "hash": "0000", "size": 0, "ver": 1}
+                "objects": [{"name": x, "hash": "0000",
+                             "size": 0, "version": 1}
                             for x in names if x.startswith("pla")],
                 "prefixes": [],
                               }))),
@@ -258,7 +259,7 @@ def test_list_ok(lib):
          (200, {"X-Oio-list-truncated": False, "X-Oio-list-next": "plep", },
             json.dumps({
                 "objects": [{"name": "plep", "hash": "0000",
-                             "size": 0, "ver": 1}],
+                             "size": 0, "version": 1}],
                 "prefixes": [],
                        }))),
 
@@ -266,8 +267,8 @@ def test_list_ok(lib):
          (200, {"X-Oio-list-truncated": True, "X-Oio-list-next": "plep", },
             json.dumps({
                 "objects": [
-                    {"name": "plap", "hash": "0000", "size": 0, "ver": 1},
-                    {"name": "plep", "hash": "0000", "size": 0, "ver": 1},
+                    {"name": "plap", "hash": "0000", "size": 0, "version": 1},
+                    {"name": "plep", "hash": "0000", "size": 0, "version": 1},
                 ],
                 "prefixes": [],
                        }))),
@@ -276,8 +277,8 @@ def test_list_ok(lib):
          (200, {"X-Oio-list-truncated": True, "X-Oio-list-next": "plep", },
             json.dumps({
                 "objects": [
-                    {"name": "plap", "hash": "0000", "size": 0, "ver": 1},
-                    {"name": "plep", "hash": "0000", "size": 0, "ver": 1},
+                    {"name": "plap", "hash": "0000", "size": 0, "version": 1},
+                    {"name": "plep", "hash": "0000", "size": 0, "version": 1},
                 ],
                 "prefixes": [],
                        }))),
@@ -287,7 +288,7 @@ def test_list_ok(lib):
          (200, {"X-Oio-list-truncated": False, "X-Oio-list-next": "plep", },
             json.dumps({
                 "objects": [{"name": "plep", "hash": "0000",
-                             "size": 0, "ver": 1}],
+                             "size": 0, "version": 1}],
                 "prefixes": [],
                        }))),
     ]
