@@ -1,6 +1,6 @@
 /*
-OpenIO SDS core library
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+OpenIO SDS functional tests
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -22,6 +22,7 @@ License along with this library.
 #include <glib.h>
 #include <json.h>
 
+#include <core/client_variables.h>
 #include <core/oiolog.h>
 #include <core/oiourl.h>
 #include <core/oio_sds.h>
@@ -166,7 +167,6 @@ _test_wrap (const char *strcfg, const char *ns,
 	oio_sds_pfree (&sds);
 	g_assert_null (sds);
 	oio_cfg_set_handle (NULL);
-	oio_cfg_handle_clean (cfg);
 }
 
 static void
