@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2015-2017 OpenIO, original work as part of
-# OpenIO Software Defined Storage
+
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -18,9 +18,12 @@
 from time import sleep, time
 
 import redis
+import random
 from oio.account.backend import AccountBackend
 from oio.common.utils import Timestamp
-from tests.utils import BaseTestCase
+from tests.utils import BaseTestCase, random_str
+from werkzeug.exceptions import Conflict
+from testtools.testcase import ExpectedException
 
 
 class TestAccountBackend(BaseTestCase):
@@ -76,6 +79,18 @@ class TestAccountBackend(BaseTestCase):
         self.assertEqual(metadata['a'], '1b')
         self.assert_('b' not in metadata)
 
+    def test_list_account(self):
+        backend = AccountBackend({}, self.conn)
+
+        # Create and check if in list
+        account_id = 'test_list'
+        backend.create_account(account_id)
+        account_list = backend.list_account()
+        self.assertTrue(account_id in account_list)
+
+        # Check the result of a nonexistent account
+        self.assertFalse("Should_not_exist" in account_list)
+
     def test_info_account(self):
         backend = AccountBackend({}, self.conn)
         account_id = 'test'
@@ -153,6 +168,7 @@ class TestAccountBackend(BaseTestCase):
         # Container is flushed, but the event is deferred
         flush_timestamp = Timestamp(time()).normal
 
+        sleep(.00001)
         # Container delete event, sent immediately after deletion
         backend.update_container(account_id, 'c1',
                                  None, Timestamp(time()).normal,
@@ -190,14 +206,16 @@ class TestAccountBackend(BaseTestCase):
             self.conn.ttl('container:%s:%s' % (account_id, name)) >= 1)
 
         # same event
-        backend.update_container(account_id, name, mtime, dtime, 0, 0)
+        with ExpectedException(Conflict):
+            backend.update_container(account_id, name, mtime, dtime, 0, 0)
         res = self.conn.zrangebylex('containers:%s' % account_id, '-', '+')
         self.assertEqual(len(res), 0)
         self.assertTrue(
             self.conn.ttl('container:%s:%s' % (account_id, name)) >= 1)
 
         # old event
-        backend.update_container(account_id, name, old_mtime, 0, 0, 0)
+        with ExpectedException(Conflict):
+            backend.update_container(account_id, name, old_mtime, 0, 0, 0)
         res = self.conn.zrangebylex('containers:%s' % account_id, '-', '+')
         self.assertEqual(len(res), 0)
         self.assertTrue(
@@ -230,7 +248,8 @@ class TestAccountBackend(BaseTestCase):
             self.conn.ttl('container:%s:%s' % (account_id, name)) >= 1)
 
         # ensure it has been removed
-        backend.update_container(account_id, name, 0, dtime, 0, 0)
+        with ExpectedException(Conflict):
+            backend.update_container(account_id, name, 0, dtime, 0, 0)
         res = self.conn.zrangebylex('containers:%s' % account_id, '-', '+')
         self.assertEqual(len(res), 0)
         self.assertTrue(
@@ -253,7 +272,8 @@ class TestAccountBackend(BaseTestCase):
                                         (account_id, name), 'mtime'), mtime)
 
         # same event
-        backend.update_container(account_id, name, mtime, 0, 0, 0)
+        with ExpectedException(Conflict):
+            backend.update_container(account_id, name, mtime, 0, 0, 0)
 
         res = self.conn.zrangebylex('containers:%s' % account_id, '-', '+')
         self.assertEqual(res[0], name)
@@ -274,7 +294,8 @@ class TestAccountBackend(BaseTestCase):
 
         # Old event
         old_mtime = Timestamp(time() - 1).normal
-        backend.update_container(account_id, name, old_mtime, 0, 0, 0)
+        with ExpectedException(Conflict):
+            backend.update_container(account_id, name, old_mtime, 0, 0, 0)
 
         res = self.conn.zrangebylex('containers:%s' % account_id, '-', '+')
         self.assertEqual(res[0], name)
@@ -284,7 +305,8 @@ class TestAccountBackend(BaseTestCase):
 
         # Old delete event
         dtime = Timestamp(time() - 1).normal
-        backend.update_container(account_id, name, 0, dtime, 0, 0)
+        with ExpectedException(Conflict):
+            backend.update_container(account_id, name, 0, dtime, 0, 0)
         res = self.conn.zrangebylex('containers:%s' % account_id, '-', '+')
         self.assertEqual(res[0], name)
         self.assertEqual(self.conn.hget('container:%s:%s' %
@@ -437,3 +459,136 @@ class TestAccountBackend(BaseTestCase):
         self.assertEqual(len(listing), 2)
         self.assertEqual([c[0] for c in listing],
                          ['3-0049-', '3-0049-0049'])
+
+    def test_refresh_account(self):
+        backend = AccountBackend({}, self.conn)
+        account_id = random_str(16)
+        account_key = 'account:%s' % account_id
+
+        self.assertEqual(backend.create_account(account_id), account_id)
+
+        total_bytes = 0
+        total_objects = 0
+
+        # 10 containers with bytes and objects
+        for i in range(10):
+            name = "container%d" % i
+            mtime = Timestamp(time()).normal
+            nb_bytes = random.randrange(100)
+            total_bytes += nb_bytes
+            nb_objets = random.randrange(100)
+            total_objects += nb_objets
+            backend.update_container(account_id, name, mtime, 0,
+                                     nb_objets, nb_bytes)
+
+        # change values
+        self.conn.hset(account_key, 'bytes', 1)
+        self.conn.hset(account_key, 'objects', 2)
+        self.assertEqual(self.conn.hget(account_key, 'bytes'), '1')
+        self.assertEqual(self.conn.hget(account_key, 'objects'), '2')
+
+        backend.refresh_account(account_id)
+        self.assertEqual(self.conn.hget(account_key, 'bytes'),
+                         str(total_bytes))
+        self.assertEqual(self.conn.hget(account_key, 'objects'),
+                         str(total_objects))
+
+    def test_update_container_wrong_timestamp_format(self):
+        backend = AccountBackend({}, self.conn)
+        account_id = 'test'
+        self.assertEqual(backend.create_account(account_id), account_id)
+
+        # initial container
+        name = '"{<container \'&\' name>}"'
+        mtime = "12456.0000076"
+        backend.update_container(account_id, name, mtime, 0, 0, 0)
+
+        res = self.conn.zrangebylex('containers:%s' % account_id, '-', '+')
+        self.assertEqual(res[0], name)
+
+        # same event
+        with ExpectedException(Conflict):
+            backend.update_container(account_id, name, mtime, 0, 0, 0)
+
+        res = self.conn.zrangebylex('containers:%s' % account_id, '-', '+')
+        self.assertEqual(res[0], name)
+
+        mtime = "0000012456.00005"
+        backend.update_container(account_id, name, mtime, 0, 0, 0)
+
+        res = self.conn.zrangebylex('containers:%s' % account_id, '-', '+')
+        self.assertEqual(res[0], name)
+
+        self.assertEqual(self.conn.hget('container:%s:%s' %
+                                        (account_id, name), 'mtime'), mtime)
+
+        mtime = "0000012456.00035"
+        backend.update_container(account_id, name, mtime, 0, 0, 0)
+
+        res = self.conn.zrangebylex('containers:%s' % account_id, '-', '+')
+        self.assertEqual(res[0], name)
+
+        self.assertEqual(self.conn.hget('container:%s:%s' %
+                                        (account_id, name), 'mtime'), mtime)
+
+    def test_is_sup(self):
+        backend = AccountBackend({}, self.conn)
+        compare = (backend.lua_is_sup +
+                   """
+            if (is_sup(KEYS[1], KEYS[2])) then
+              return redis.status_reply('IS SUP');
+            else
+              return redis.error_reply('IS NOT SUP');
+            end;
+                   """)
+
+        compare_script = backend.register_script(compare)
+        self.assertRaises(redis.exceptions.ResponseError, compare_script,
+                          keys=["12457.00245", "12457.00245"],
+                          client=self.conn)
+        self.assertEqual(compare_script(keys=["42457.00245", "12457.00245"],
+                                        client=self.conn), "IS SUP")
+        self.assertRaises(redis.exceptions.ResponseError, compare_script,
+                          keys=["12457.00245", "12457.002450"],
+                          client=self.conn)
+        self.assertEqual(compare_script(keys=["42457.00245", "12457.002450"],
+                                        client=self.conn), "IS SUP")
+        self.assertRaises(redis.exceptions.ResponseError, compare_script,
+                          keys=["12457.00245", "12457.002456"],
+                          client=self.conn)
+        self.assertEqual(compare_script(keys=["42457.00245", "12457.002456"],
+                                        client=self.conn), "IS SUP")
+        self.assertEqual(compare_script(keys=["42457.00246", "42457.002458"],
+                                        client=self.conn), "IS SUP")
+
+    def test_flush_account(self):
+        backend = AccountBackend({}, self.conn)
+        account_id = random_str(16)
+        account_key = 'account:%s' % account_id
+
+        self.assertEqual(backend.create_account(account_id), account_id)
+
+        total_bytes = 0
+        total_objects = 0
+
+        # 10 containers with bytes and objects
+        for i in range(10):
+            name = "container%d" % i
+            mtime = Timestamp(time()).normal
+            nb_bytes = random.randrange(100)
+            total_bytes += nb_bytes
+            nb_objets = random.randrange(100)
+            total_objects += nb_objets
+            backend.update_container(account_id, name, mtime, 0,
+                                     nb_objets, nb_bytes)
+
+        self.assertEqual(self.conn.hget(account_key, 'bytes'),
+                         str(total_bytes))
+        self.assertEqual(self.conn.hget(account_key, 'objects'),
+                         str(total_objects))
+
+        backend.flush_account(account_id)
+        self.assertEqual(self.conn.hget(account_key, 'bytes'), '0')
+        self.assertEqual(self.conn.hget(account_key, 'objects'), '0')
+        self.assertEqual(self.conn.zcard("containers:%s" % account_id), 0)
+        self.assertEqual(self.conn.exists("container:test:*"), 0)
