@@ -1,3 +1,18 @@
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
+#
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3.0 of the License, or (at your option) any later version.
+#
+# This library is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public
+# License along with this library.
+
 import hashlib
 
 import os
@@ -9,6 +24,7 @@ from oio.container.client import ContainerClient
 from oio.blob.client import BlobClient
 from oio.common.constants import chunk_xattr_keys
 from tests.utils import BaseTestCase, random_str, random_id
+from oio.common.constants import OIO_VERSION
 
 
 class TestContent(object):
@@ -77,6 +93,9 @@ class TestBlobAuditorFunctional(BaseTestCase):
                       'chunk_id': self.chunk.id_chunk,
                       'chunk_pos': self.chunk.pos,
                       'chunk_hash': self.chunk.md5,
+                      'full_path': ['%s/%s/%s' % (self.account, self.ref,
+                                                  self.content.path)],
+                      'oio_version': OIO_VERSION
                       }
         self.blob_c.chunk_put(self.chunk_url, chunk_meta, self.data)
 
@@ -117,7 +136,7 @@ class TestBlobAuditorFunctional(BaseTestCase):
                           self.chunk_path)
 
     def test_container_deleted(self):
-        self.container_c.container_destroy(self.account, self.ref)
+        self.container_c.container_delete(self.account, self.ref)
 
         self.assertRaises(exc.OrphanChunk, self.auditor.chunk_audit,
                           self.chunk_path)
