@@ -1,10 +1,29 @@
+# -*- coding: utf-8 -*-
+
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
+#
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3.0 of the License, or (at your option) any later version.
+#
+# This library is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public
+# License along with this library.
+
 import string
 import random
 import os
 import os.path
 from hashlib import md5
 from urlparse import urlparse
+from urllib import quote_plus
 from oio.common.http import http_connect
+from oio.common.constants import OIO_VERSION
 
 from tests.utils import BaseTestCase
 
@@ -37,6 +56,9 @@ class TestBlobFunctional(BaseTestCase):
             'x-oio-chunk-meta-chunk-size': len(data),
             'x-oio-chunk-meta-chunk-hash': md5(data).hexdigest().upper(),
             'x-oio-chunk-meta-chunk-pos': 0,
+            'x-oio-chunk-meta-full-path': ('test/test/test' +
+                                           ',test1/test1/test1'),
+            'x-oio-chunk-meta-oio-version': OIO_VERSION
         }
 
     def _rawx_url(self, chunkid):
@@ -86,7 +108,45 @@ class TestBlobFunctional(BaseTestCase):
         conn.close()
         return resp, body
 
-    def _cycle_put(self, length, expected, remove_headers=None):
+    def test_copy_errors(self):
+        length = 100
+        chunkid = self.chunkid()
+        chunkdata = random_buffer(string.printable, length)
+        chunkurl = self._rawx_url(chunkid)
+        self._check_not_present(chunkurl)
+        headers = self._chunk_attr(chunkid, chunkdata)
+        metachunk_size = 9 * length
+        # TODO take random legit value
+        metachunk_hash = md5().hexdigest()
+        # TODO should also include meta-chunk-hash
+        trailers = {'x-oio-chunk-meta-metachunk-size': metachunk_size,
+                    'x-oio-chunk-meta-metachunk-hash': metachunk_hash}
+        # Initial put that must succeed
+        resp, body = self._http_request(chunkurl, 'PUT', chunkdata, headers,
+                                        trailers)
+        expected = 201
+        self.assertEqual(resp.status, expected)
+        copyid = self.chunkid()
+        copyid = chunkid[:-60] + copyid[-60:]
+        copyurl = self._rawx_url(copyid)
+        headers = {}
+        headers["Destination"] = copyurl
+        resp, _ = self._http_request(chunkurl, 'COPY', '', headers)
+        self.assertEqual(resp.status, 403)
+
+        headers = {}
+        headers["Destination"] = chunkurl
+        headers['x-oio-chunk-meta-full-path'] = self._generate_fullpath(
+                "account-snapshot", "container-snapshot", "test"+"-snapshot",
+                'x-oio-chunk-meta-content-version')
+        resp, _ = self._http_request(chunkurl, 'COPY', '', headers)
+        self.assertEqual(resp.status, 403)
+
+        headers = {}
+        resp, _ = self._http_request(chunkurl, 'COPY', '', headers)
+        self.assertEqual(resp.status, 400)
+
+    def _cycle_put(self, length, expected, remove_headers=None, path=None):
         chunkid = self.chunkid()
         chunkdata = random_buffer(string.printable, length)
         chunkurl = self._rawx_url(chunkid)
@@ -96,6 +156,9 @@ class TestBlobFunctional(BaseTestCase):
             for h in remove_headers:
                 del headers[h]
 
+        if path:
+            headers['x-oio-chunk-meta-full-path'] = self._generate_fullpath(
+                path, path, path, headers['x-oio-chunk-meta-content-version'])
         # we do not really care about the actual value
         metachunk_size = 9 * length
         # TODO take random legit value
@@ -127,6 +190,10 @@ class TestBlobFunctional(BaseTestCase):
         resp, body = self._http_request(chunkurl, 'GET', '', {})
         self.assertEqual(resp.status, 200)
         self.assertEqual(body, chunkdata)
+        fp = set(headers.pop('x-oio-chunk-meta-full-path').split(','))
+        fp2 = set(resp.getheader('x-oio-chunk-meta-full-path').split(','))
+        self.assertSetEqual(fp2, fp)
+
         headers['x-oio-chunk-meta-metachunk-size'] = metachunk_size
         headers['x-oio-chunk-meta-metachunk-hash'] = metachunk_hash.upper()
         for k, v in headers.items():
@@ -247,3 +314,65 @@ class TestBlobFunctional(BaseTestCase):
     def test_bad_chunkid(self):
         self._check_bad_headers(
             32, bad_headers={'x-oio-chunk-meta-chunk-id': '00'*32})
+
+    def _generate_fullpath(self, account, container_name, path, version):
+        return '{0}/{1}/{2}/{3}'.format(quote_plus(account),
+                                        quote_plus(container_name),
+                                        quote_plus(path),
+                                        version)
+
+    def _cycle_copy(self, path):
+        chunkid = self.chunkid()
+        chunkdata = random_buffer(string.printable, 1)
+        chunkurl = self._rawx_url(chunkid)
+        # chunkpath = self._chunk_path(chunkid)
+        headers = self._chunk_attr(chunkid, chunkdata)
+        metachunk_hash = md5().hexdigest()
+        trailers = {'x-oio-chunk-meta-metachunk-size': 1,
+                    'x-oio-chunk-meta-metachunk-hash': metachunk_hash}
+
+        self._check_not_present(chunkurl)
+        headers['x-oio-chunk-meta-full-path'] = self._generate_fullpath(
+                path, path, path, headers['x-oio-chunk-meta-content-version'])
+        resp, _ = self._http_request(chunkurl, 'PUT', chunkdata, headers,
+                                     trailers)
+        self.assertEqual(resp.status, 201)
+
+        copyid = self.chunkid()
+        copyid = chunkid[:-60] + copyid[-60:]
+        copyurl = self._rawx_url(copyid)
+
+        headers = {}
+        headers["Destination"] = copyurl
+        headers['x-oio-chunk-meta-full-path'] = self._generate_fullpath(
+                "account-snapshot", "container-snapshot", path+"-snapshot",
+                'x-oio-chunk-meta-content-version')
+        resp, _ = self._http_request(chunkurl, 'COPY', '', headers)
+        self.assertEqual(resp.status, 201)
+        resp, body = self._http_request(copyurl, 'GET', '', {})
+        self.assertEqual(resp.status, 200)
+        fp = set([headers['x-oio-chunk-meta-full-path'],
+                  self._generate_fullpath(path, path, path,
+                                          '1456938361143740')])
+        fp2 = set(resp.getheader('x-oio-chunk-meta-full-path').split(","))
+        self.assertSetEqual(fp, fp2)
+
+    def test_strange_path(self):
+        strange_paths = [
+                "Annual report.txt",
+                "foo+bar=foobar.txt",
+                "100%_bug_free.c",
+                "forward/slash/allowed",
+                "I\\put\\backslashes\\and$dollar$signs$in$file$names",
+                "Je suis tombé sur la tête, mais ça va bien.",
+                "%s%f%u%d%%",
+                "{1},{0},{3}",
+                "carriage\rreturn",
+                "line\nfeed",
+                "ta\tbu\tla\ttion",
+                "controlchars",
+                "//azeaze\\//azeaz\\//azea"
+                ]
+        for path in strange_paths:
+            self._cycle_put(1, 201, path=path)
+            self._cycle_copy(path)
