@@ -1,6 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2015 OpenIO, original work as part of
-# OpenIO Software Defined Storage
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -34,6 +33,7 @@ from oio.content.factory import ContentFactory
 from oio.content.plain import PlainContent
 from oio.content.ec import ECContent
 from tests.utils import BaseTestCase, ec
+from urllib import quote_plus
 
 
 def md5_stream(stream):
@@ -64,8 +64,8 @@ class TestContentFactory(BaseTestCase):
         self.container_name = "TestContentFactory%f" % time.time()
         self.blob_client = BlobClient()
         self.container_client = ContainerClient(self.gridconf)
-        self.container_client.container_create(acct=self.account,
-                                               ref=self.container_name)
+        self.container_client.container_create(account=self.account,
+                                               reference=self.container_name)
         self.container_id = cid_from_name(self.account,
                                           self.container_name).upper()
         self.stgpol = "SINGLE"
@@ -76,6 +76,12 @@ class TestContentFactory(BaseTestCase):
     def tearDown(self):
         super(TestContentFactory, self).tearDown()
 
+    def _generate_fullpath(self, account, container_name, path, version):
+        return ['{0}/{1}/{2}/{3}'.format(quote_plus(account),
+                                         quote_plus(container_name),
+                                         quote_plus(path),
+                                         version)]
+
     def test_get_ec(self):
         meta = {
             "chunk_method": "ec/algo=liberasurecode_rs_vand,k=6,m=2",
@@ -88,7 +94,8 @@ class TestContentFactory(BaseTestCase):
             "mime_type": "application/octet-stream",
             "name": "tox.ini",
             "policy": self.stgpol_ec,
-            "version": "1450176946676289"
+            "version": "1450176946676289",
+            "oio_version": "4.0",
         }
         chunks = [
             {
@@ -108,13 +115,20 @@ class TestContentFactory(BaseTestCase):
                 "pos": "0.3", "size": 512,
                 "hash": "DA9D7F72AEEA5791565724424CE45C16"}
         ]
-        self.content_factory.container_client.content_show = Mock(
+        self.content_factory.container_client.content_locate = Mock(
             return_value=(meta, chunks))
-        c = self.content_factory.get("xxx_container_id", "xxx_content_id")
+        c = self.content_factory.get("xxx_container_id", "xxx_content_id",
+                                     account=self.account,
+                                     container_name=self.container_name)
         self.assertEqual(type(c), ECContent)
         self.assertEqual(c.content_id, "3FA2C4A1ED2605005335A276890EC458")
         self.assertEqual(c.length, 658)
         self.assertEqual(c.path, "tox.ini")
+        self.assertEqual(c.full_path,
+                         self._generate_fullpath(self.account,
+                                                 self.container_name,
+                                                 "tox.ini",
+                                                 meta['version']))
         self.assertEqual(c.version, "1450176946676289")
         # TODO test storage method
         self.assertEqual(len(c.chunks), 4)
@@ -135,7 +149,8 @@ class TestContentFactory(BaseTestCase):
             "mime_type": "application/octet-stream",
             "name": "tox.ini",
             "policy": self.stgpol_twocopies,
-            "version": "1450176946676289"
+            "version": "1450176946676289",
+            "oio_version": "4.0",
         }
         chunks = [
             {
@@ -147,14 +162,21 @@ class TestContentFactory(BaseTestCase):
                 "pos": "0", "size": 658,
                 "hash": "E952A419957A6E405BFC53EC65483F73"}
         ]
-        self.content_factory.container_client.content_show = Mock(
+        self.content_factory.container_client.content_locate = Mock(
             return_value=(meta, chunks))
-        c = self.content_factory.get("xxx_container_id", "xxx_content_id")
+        c = self.content_factory.get("xxx_container_id", "xxx_content_id",
+                                     account=self.account,
+                                     container_name=self.container_name)
         self.assertEqual(type(c), PlainContent)
         self.assertEqual(c.content_id, "3FA2C4A1ED2605005335A276890EC458")
         self.assertEqual(c.length, 658)
         self.assertEqual(c.path, "tox.ini")
         self.assertEqual(c.version, "1450176946676289")
+        self.assertEqual(c.full_path,
+                         self._generate_fullpath(self.account,
+                                                 self.container_name,
+                                                 "tox.ini",
+                                                 meta['version']))
         # TODO test storage_method
         self.assertEqual(len(c.chunks), 2)
         self.assertEqual(c.chunks[0].raw(), chunks[0])
@@ -176,7 +198,8 @@ class TestContentFactory(BaseTestCase):
             "mime_type": "application/octet-stream",
             "name": "titi",
             "policy": self.stgpol_ec,
-            "version": "1450341162332663"
+            "version": "1450341162332663",
+            "oio_version": "4.0",
         }
         chunks = [
             {
@@ -199,7 +222,9 @@ class TestContentFactory(BaseTestCase):
         self.content_factory.container_client.content_prepare = Mock(
             return_value=(meta, chunks))
         c = self.content_factory.new("xxx_container_id", "titi",
-                                     1000, self.stgpol_ec)
+                                     1000, self.stgpol_ec,
+                                     account=self.account,
+                                     container_name=self.container_name)
         self.assertEqual(type(c), ECContent)
         self.assertEqual(c.content_id, "F4B1C8DD132705007DE8B43D0709DAA2")
         self.assertEqual(c.length, 1000)
@@ -212,9 +237,16 @@ class TestContentFactory(BaseTestCase):
         self.assertEqual(c.chunks[2].raw(), chunks[1])
         self.assertEqual(c.chunks[3].raw(), chunks[0])
 
-    def _new_content(self, stgpol, data, path="titi"):
+    def _new_content(self, stgpol, data, path="titi", account=None,
+                     container_name=None, mime_type=None, properties=None):
         old_content = self.content_factory.new(self.container_id, path,
-                                               len(data), stgpol)
+                                               len(data), stgpol,
+                                               account=account,
+                                               container_name=container_name)
+        if properties:
+            old_content.properties = properties
+        if mime_type:
+            old_content.mime_type = mime_type
         old_content.create(BytesIO(data))
         return self.content_factory.get(self.container_id,
                                         old_content.content_id)
@@ -227,7 +259,10 @@ class TestContentFactory(BaseTestCase):
             self.stgpol_threecopies: PlainContent,
             self.stgpol_ec: ECContent
         }
-        old_content = self._new_content(old_policy, data)
+        mime_type = "application/test"
+        props = {'a': 'b'}
+        old_content = self._new_content(
+            old_policy, data, mime_type=mime_type, properties=props)
         self.assertEqual(type(old_content), obj_type[old_policy])
 
         changed_content = self.content_factory.change_policy(
@@ -245,6 +280,8 @@ class TestContentFactory(BaseTestCase):
         downloaded_data = "".join(new_content.fetch())
 
         self.assertEqual(downloaded_data, data)
+        self.assertEqual(mime_type, new_content.mime_type)
+        self.assertDictEqual(props, new_content.properties)
 
     @ec
     def test_change_content_0_byte_policy_single_to_ec(self):
@@ -329,7 +366,7 @@ class TestContentFactory(BaseTestCase):
         hosts = []
         for c in content_updated.chunks.filter(metapos=0):
             self.assertThat(hosts, Not(Contains(c.host)))
-            self.assertNotEquals(c.id, chunk_id)
+            self.assertNotEqual(c.id, chunk_id)
             hosts.append(c.host)
 
         new_chunk_meta, new_chunk_stream = self.blob_client.chunk_get(
@@ -367,30 +404,39 @@ class TestContentFactory(BaseTestCase):
                 "I\\put\\backslashes\\and$dollar$signs$in$file$names",
                 "Je suis tombé sur la tête, mais ça va bien.",
                 "%s%f%u%d%%",
+                "{1},{0},{3}",
                 "carriage\rreturn",
                 "line\nfeed",
                 "ta\tbu\tla\ttion",
                 "controlchars",
+                "//azeaze\\//azeaz\\//azea"
                 ]
         answers = dict()
         for cname in strange_paths:
             content = self._new_content(self.stgpol, "nobody cares", cname)
             answers[cname] = content
-        listing = self.container_client.container_list(self.account,
-                                                       self.container_name)
+
+        _, listing = self.container_client.content_list(self.account,
+                                                        self.container_name)
         obj_set = {k["name"].encode("utf8", "ignore")
                    for k in listing["objects"]}
         try:
             # Ensure the saved path is the one we gave the object
             for cname in answers:
                 self.assertEqual(cname, answers[cname].path)
+                full_path = self._generate_fullpath(self.account,
+                                                    self.container_name,
+                                                    cname,
+                                                    answers[cname].version)
+                self.assertEqual(answers[cname].full_path, full_path)
             # Ensure all objects appear in listing
             for cname in strange_paths:
                 self.assertIn(cname, obj_set)
+
         finally:
             # Cleanup
             for cname in answers:
                 try:
                     content.delete()
-                except:
+                except Exception:
                     pass
