@@ -1,5 +1,4 @@
-# Copyright (C) 2015-2017 OpenIO, original work as part of
-# OpenIO Software Defined Storage
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -24,7 +23,7 @@ from oio.common.utils import cid_from_name
 from oio.container.client import ContainerClient
 from oio.content.factory import ContentFactory
 from oio.crawler.storage_tierer import StorageTiererWorker
-from oio import ObjectStorageApi
+from oio.api.object_storage import ObjectStorageApi
 from tests.functional.content.test_content import random_data
 from tests.utils import BaseTestCase
 
@@ -49,33 +48,38 @@ class TestStorageTierer(BaseTestCase):
         self.container_0_id = cid_from_name(
             self.test_account, self.container_0_name)
         self.container_client.container_create(
-            acct=self.test_account, ref=self.container_0_name)
+            account=self.test_account, reference=self.container_0_name)
 
         self.container_1_name = "container_with_1_content"
         self.container_1_id = cid_from_name(
             self.test_account, self.container_1_name)
         self.container_client.container_create(
-            acct=self.test_account, ref=self.container_1_name)
+            account=self.test_account, reference=self.container_1_name)
         self.container_1_content_0_name = "container_1_content_0"
         self.container_1_content_0 = self._new_content(
-            self.container_1_id, self.container_1_content_0_name, "SINGLE")
+            self.container_1_id, self.container_1_content_0_name, "SINGLE",
+            self.test_account, self.container_0_name)
 
         self.container_2_name = "container_with_2_contents"
         self.container_2_id = cid_from_name(
             self.test_account, self.container_2_name)
         self.container_client.container_create(
-            acct=self.test_account, ref=self.container_2_name)
+            account=self.test_account, reference=self.container_2_name)
         self.container_2_content_0_name = "container_2_content_0"
         self.container_2_content_0 = self._new_content(
-            self.container_2_id, self.container_2_content_0_name, "SINGLE")
+            self.container_2_id, self.container_2_content_0_name, "SINGLE",
+            self.test_account, self.container_0_name)
         self.container_2_content_1_name = "container_2_content_1"
         self.container_2_content_1 = self._new_content(
-            self.container_2_id, self.container_2_content_1_name, "TWOCOPIES")
+            self.container_2_id, self.container_2_content_1_name, "TWOCOPIES",
+            self.test_account, self.container_0_name)
 
-    def _new_content(self, container_id, content_name, stgpol):
+    def _new_content(self, container_id, content_name, stgpol, account,
+                     reference):
         data = random_data(10)
         content = self.content_factory.new(container_id, content_name,
-                                           len(data), stgpol)
+                                           len(data), stgpol, account=account,
+                                           container_name=reference)
 
         content.create(BytesIO(data))
         return content
@@ -86,7 +90,7 @@ class TestStorageTierer(BaseTestCase):
     def test_iter_container_list(self):
         worker = StorageTiererWorker(self.gridconf, Mock())
         api = ObjectStorageApi(self.namespace)
-        actual = [x[0] for x in api.container_list(self.test_account)[0]]
+        actual = [x[0] for x in api.container_list(self.test_account)]
         if len(actual) < 3:
             print "Slow event propagation!"
             # account events have not yet propagated
@@ -117,7 +121,8 @@ class TestStorageTierer(BaseTestCase):
         # add a new content created after the three previous contents
         now = int(time.time())
         time.sleep(2)
-        self._new_content(self.container_2_id, "titi", "TWOCOPIES")
+        self._new_content(self.container_2_id, "titi", "TWOCOPIES",
+                          self.test_account, self.container_2_name)
 
         self.gridconf["outdated_threshold"] = 2
         worker = StorageTiererWorker(self.gridconf, Mock())
