@@ -1,4 +1,20 @@
 # -*- coding: utf-8 -*-
+
+# Copyright (C) 2016-2017 OpenIO SAS, as part of OpenIO SDS
+#
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3.0 of the License, or (at your option) any later version.
+#
+# This library is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public
+# License along with this library.
+
 import os
 import mock
 
@@ -8,6 +24,7 @@ from oio.blob.utils import chunk_xattr_keys
 from oio.common.exceptions import FaultyChunk
 from oio.rdir.client import RdirClient
 from tests.utils import BaseTestCase, random_id
+from oio.common.constants import OIO_VERSION
 
 
 class TestIndexerCrawler(BaseTestCase):
@@ -60,6 +77,8 @@ class TestIndexerCrawler(BaseTestCase):
             'plain/nb_copy=3')
         xattr.setxattr(
             chunk_path, 'user.' + chunk_xattr_keys['content_version'], '0')
+        xattr.setxattr(
+            chunk_path, 'user.' + chunk_xattr_keys['oio_version'], OIO_VERSION)
 
         return chunk_path, container_id, content_id, chunk_id
 
@@ -124,3 +143,33 @@ class TestIndexerCrawler(BaseTestCase):
         indexer = BlobIndexer(self.conf)
 
         self.assertRaises(FaultyChunk, indexer.update_index, chunk_path)
+        os.remove(chunk_path)
+
+    def test_index_chunk_with_wrong_paths(self):
+        indexer = BlobIndexer(self.conf)
+
+        # try to index the chunks
+        indexer.index_pass()
+        successes = indexer.successes
+        errors = indexer.errors
+
+        # create fake chunks
+        chunk_path1, _, _, _ = self._create_chunk(
+            self.rawx_conf['path'] + ".pending")
+        chunk_path2, _, _, _ = self._create_chunk(
+            self.rawx_conf['path'][:-1] + 'G')
+        chunk_path3, _, _, _ = self._create_chunk(
+            self.rawx_conf['path'] + '0')
+        chunk_path4, _, _, _ = self._create_chunk(
+            self.rawx_conf['path'][:-1])
+
+        # try to index the chunks
+        indexer.index_pass()
+
+        self.assertEqual(indexer.successes, successes)
+        self.assertEqual(indexer.errors, errors)
+
+        os.remove(chunk_path1)
+        os.remove(chunk_path2)
+        os.remove(chunk_path3)
+        os.remove(chunk_path4)
