@@ -1,18 +1,35 @@
+# Copyright (C) 2015-2018 OpenIO SAS, as part of OpenIO SDS
+#
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3.0 of the License, or (at your option) any later version.
+#
+# This library is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public
+# License along with this library.
+
 import unittest
 import random
 from io import BytesIO
 from collections import defaultdict
-from hashlib import md5
+import hashlib
 from copy import deepcopy
 from eventlet import Timeout
+from mock import patch
 from oio.common.storage_method import STORAGE_METHODS
-from oio.api.ec import ECChunkWriteHandler, ECChunkDownloadHandler, \
+from oio.api.ec import EcMetachunkWriter, ECChunkDownloadHandler, \
     ECRebuildHandler
-from oio.common import exceptions as exc
+from oio.common import exceptions as exc, green
 from oio.common.constants import chunk_headers
 from tests.unit.api import empty_stream, decode_chunked_body, \
-    FakeResponse, CHUNK_SIZE, EMPTY_CHECKSUM
+    FakeResponse, CHUNK_SIZE, EMPTY_MD5, EMPTY_SHA256
 from tests.unit import set_http_connect, set_http_requests
+from oio.common.constants import OIO_VERSION
 
 
 class TestEC(unittest.TestCase):
@@ -29,6 +46,8 @@ class TestEC(unittest.TestCase):
             'container_id': self.cid,
             'policy': 'EC',
             'content_path': 'test',
+            'full_path': ['account/container/test'],
+            'oio_version': OIO_VERSION
         }
         self._meta_chunk = [
                 {'url': 'http://127.0.0.1:7000/0', 'pos': '0.0', 'num': 0},
@@ -48,7 +67,7 @@ class TestEC(unittest.TestCase):
         return deepcopy(self._meta_chunk)
 
     def checksum(self, d=''):
-        return md5(d)
+        return hashlib.md5(d)
 
     def test_write_simple(self):
         checksum = self.checksum()
@@ -57,12 +76,12 @@ class TestEC(unittest.TestCase):
         nb = self.storage_method.ec_nb_data + self.storage_method.ec_nb_parity
         resps = [201] * nb
         with set_http_connect(*resps):
-            handler = ECChunkWriteHandler(self.sysmeta, self.meta_chunk(),
-                                          checksum, self.storage_method)
+            handler = EcMetachunkWriter(self.sysmeta, self.meta_chunk(),
+                                        checksum, self.storage_method)
             bytes_transferred, checksum, chunks = handler.stream(source, size)
         self.assertEqual(len(chunks), nb)
         self.assertEqual(bytes_transferred, 0)
-        self.assertEqual(checksum, EMPTY_CHECKSUM)
+        self.assertEqual(checksum, EMPTY_MD5)
 
     def test_write_exception(self):
         checksum = self.checksum()
@@ -71,8 +90,8 @@ class TestEC(unittest.TestCase):
         nb = self.storage_method.ec_nb_data + self.storage_method.ec_nb_parity
         resps = [500] * nb
         with set_http_connect(*resps):
-            handler = ECChunkWriteHandler(self.sysmeta, self.meta_chunk(),
-                                          checksum, self.storage_method)
+            handler = EcMetachunkWriter(self.sysmeta, self.meta_chunk(),
+                                        checksum, self.storage_method)
             self.assertRaises(exc.OioException, handler.stream, source, size)
 
     def test_write_quorum_success(self):
@@ -84,8 +103,8 @@ class TestEC(unittest.TestCase):
         resps = [201] * quorum_size
         resps += [500] * (nb - quorum_size)
         with set_http_connect(*resps):
-            handler = ECChunkWriteHandler(self.sysmeta, self.meta_chunk(),
-                                          checksum, self.storage_method)
+            handler = EcMetachunkWriter(self.sysmeta, self.meta_chunk(),
+                                        checksum, self.storage_method)
             bytes_transferred, checksum, chunks = handler.stream(source, size)
         self.assertEqual(len(chunks), nb)
 
@@ -95,7 +114,7 @@ class TestEC(unittest.TestCase):
             self.assertEqual(chunks[i].get('error'), 'resp: HTTP 500')
 
         self.assertEqual(bytes_transferred, 0)
-        self.assertEqual(checksum, EMPTY_CHECKSUM)
+        self.assertEqual(checksum, EMPTY_MD5)
 
     def test_write_quorum_error(self):
         checksum = self.checksum()
@@ -106,14 +125,15 @@ class TestEC(unittest.TestCase):
         resps = [500] * quorum_size
         resps += [201] * (nb - quorum_size)
         with set_http_connect(*resps):
-            handler = ECChunkWriteHandler(self.sysmeta, self.meta_chunk(),
-                                          checksum, self.storage_method)
+            handler = EcMetachunkWriter(self.sysmeta, self.meta_chunk(),
+                                        checksum, self.storage_method)
             # TODO use specialized Exception
             self.assertRaises(exc.OioException, handler.stream, source, size)
 
     def test_write_connect_errors(self):
         test_cases = [
-                {'error': Timeout(1.0), 'msg': 'connect: Timeout 1.0 second'},
+                {'error': green.ConnectionTimeout(1.0),
+                 'msg': 'connect: Connection timeout 1.0 second'},
                 {'error': Exception('failure'), 'msg': 'connect: failure'},
         ]
         for test in test_cases:
@@ -127,9 +147,9 @@ class TestEC(unittest.TestCase):
             err_pos = random.randint(0, nb)
             resps.insert(err_pos, test['error'])
             with set_http_connect(*resps):
-                handler = ECChunkWriteHandler(self.sysmeta,
-                                              self.meta_chunk_copy(),
-                                              checksum, self.storage_method)
+                handler = EcMetachunkWriter(self.sysmeta,
+                                            self.meta_chunk_copy(),
+                                            checksum, self.storage_method)
                 bytes_transferred, checksum, chunks = handler.stream(
                     source, size)
 
@@ -139,11 +159,12 @@ class TestEC(unittest.TestCase):
             self.assertEqual(chunks[nb - 1].get('error'), test['msg'])
 
             self.assertEqual(bytes_transferred, 0)
-            self.assertEqual(checksum, EMPTY_CHECKSUM)
+            self.assertEqual(checksum, EMPTY_MD5)
 
     def test_write_response_error(self):
         test_cases = [
-                {'error': Timeout(1.0), 'msg': 'resp: Timeout 1.0 second'},
+                {'error': green.ChunkWriteTimeout(1.0),
+                 'msg': 'resp: Chunk write timeout 1.0 second'},
                 {'error': Exception('failure'), 'msg': 'resp: failure'},
         ]
         for test in test_cases:
@@ -155,9 +176,9 @@ class TestEC(unittest.TestCase):
             resps = [201] * (nb - 1)
             resps.append((100, test['error']))
             with set_http_connect(*resps):
-                handler = ECChunkWriteHandler(self.sysmeta,
-                                              self.meta_chunk_copy(),
-                                              checksum, self.storage_method)
+                handler = EcMetachunkWriter(self.sysmeta,
+                                            self.meta_chunk_copy(),
+                                            checksum, self.storage_method)
                 bytes_transferred, checksum, chunks = handler.stream(
                     source, size)
 
@@ -167,7 +188,7 @@ class TestEC(unittest.TestCase):
             self.assertEqual(chunks[nb - 1].get('error'), test['msg'])
 
             self.assertEqual(bytes_transferred, 0)
-            self.assertEqual(checksum, EMPTY_CHECKSUM)
+            self.assertEqual(checksum, EMPTY_MD5)
 
     def test_write_error_source(self):
         class TestReader(object):
@@ -180,8 +201,8 @@ class TestEC(unittest.TestCase):
         nb = self.storage_method.ec_nb_data + self.storage_method.ec_nb_parity
         resps = [201] * nb
         with set_http_connect(*resps):
-            handler = ECChunkWriteHandler(self.sysmeta, self.meta_chunk(),
-                                          checksum, self.storage_method)
+            handler = EcMetachunkWriter(self.sysmeta, self.meta_chunk(),
+                                        checksum, self.storage_method)
             self.assertRaises(exc.SourceReadError, handler.stream, source,
                               size)
 
@@ -195,10 +216,10 @@ class TestEC(unittest.TestCase):
         nb = self.storage_method.ec_nb_data + self.storage_method.ec_nb_parity
         resps = [201] * nb
         with set_http_connect(*resps):
-            handler = ECChunkWriteHandler(self.sysmeta, self.meta_chunk(),
-                                          checksum, self.storage_method)
-            self.assertRaises(Timeout, handler.stream, source,
-                              size)
+            handler = EcMetachunkWriter(self.sysmeta, self.meta_chunk(),
+                                        checksum, self.storage_method)
+            self.assertRaises(
+                exc.OioTimeout, handler.stream, source, size)
 
     def test_write_exception_source(self):
         class TestReader(object):
@@ -210,8 +231,8 @@ class TestEC(unittest.TestCase):
         nb = self.storage_method.ec_nb_data + self.storage_method.ec_nb_parity
         resps = [201] * nb
         with set_http_connect(*resps):
-            handler = ECChunkWriteHandler(self.sysmeta, self.meta_chunk(),
-                                          checksum, self.storage_method)
+            handler = EcMetachunkWriter(self.sysmeta, self.meta_chunk(),
+                                        checksum, self.storage_method)
             # TODO specialize exception
             self.assertRaises(Exception, handler.stream, source,
                               size)
@@ -234,8 +255,8 @@ class TestEC(unittest.TestCase):
         # TODO test headers
 
         with set_http_connect(*resps, cb_body=cb_body):
-            handler = ECChunkWriteHandler(self.sysmeta, self.meta_chunk(),
-                                          checksum, self.storage_method)
+            handler = EcMetachunkWriter(self.sysmeta, self.meta_chunk(),
+                                        checksum, self.storage_method)
             bytes_transferred, checksum, chunks = handler.stream(source, size)
 
         self.assertEqual(len(test_data), bytes_transferred)
@@ -273,6 +294,54 @@ class TestEC(unittest.TestCase):
         self.assertEqual(
             test_data_checksum, self.checksum(final_data).hexdigest())
 
+    def _test_write_checksum_algo(self, expected_checksum, **kwargs):
+        global_checksum = self.checksum()
+        source = empty_stream()
+        size = CHUNK_SIZE * self.storage_method.ec_nb_data
+        nb = self.storage_method.ec_nb_data + self.storage_method.ec_nb_parity
+        resps = [201] * nb
+        with set_http_connect(*resps):
+            handler = EcMetachunkWriter(
+                self.sysmeta, self.meta_chunk(),
+                global_checksum, self.storage_method,
+                **kwargs)
+            bytes_transferred, checksum, chunks = handler.stream(source, size)
+        self.assertEqual(nb, len(chunks))
+        self.assertEqual(0, bytes_transferred)
+        self.assertEqual(expected_checksum, checksum)
+
+    def test_write_default_checksum_algo(self):
+        with patch('hashlib.new', wraps=hashlib.new) as algo_new:
+            self._test_write_checksum_algo(EMPTY_MD5)
+            algo_new.assert_called_with('md5')
+            # Should be called once for the metachunk
+            # and once for each chunk.
+            self.assertEqual(
+                (self.storage_method.ec_nb_data +
+                 self.storage_method.ec_nb_parity +
+                 1),
+                len(algo_new.call_args_list))
+
+    def test_write_custom_checksum_algo(self):
+        with patch('hashlib.new', wraps=hashlib.new) as algo_new:
+            self._test_write_checksum_algo(
+                EMPTY_SHA256, chunk_checksum_algo='sha256')
+            algo_new.assert_called_with('sha256')
+            # Should be called once for the metachunk
+            # and once for each chunk.
+            self.assertEqual(
+                (self.storage_method.ec_nb_data +
+                 self.storage_method.ec_nb_parity +
+                 1),
+                len(algo_new.call_args_list))
+
+    def test_write_no_checksum_algo(self):
+        with patch('hashlib.new', wraps=hashlib.new) as algo_new:
+            self._test_write_checksum_algo(
+                EMPTY_MD5, chunk_checksum_algo=None)
+            # Should be called only once for the metachunk
+            algo_new.assert_called_once_with('md5')
+
     def _make_ec_chunks(self, data):
         segment_size = self.storage_method.ec_segment_size
 
