@@ -1,16 +1,34 @@
+# Copyright (C) 2015-2018 OpenIO SAS, as part of OpenIO SDS
+#
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3.0 of the License, or (at your option) any later version.
+#
+# This library is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public
+# License along with this library.
+
 import unittest
 from collections import defaultdict
 from io import BytesIO
-from hashlib import md5
+import hashlib
 from eventlet import Timeout
+from mock import patch
+
 from oio.common import exceptions as exc
 from oio.common import green
-from oio.api.replication import ReplicatedChunkWriteHandler
+from oio.api.replication import ReplicatedMetachunkWriter
 from oio.common.storage_method import STORAGE_METHODS
-from tests.unit.api import CHUNK_SIZE, EMPTY_CHECKSUM, empty_stream, \
-    decode_chunked_body, FakeResponse
+from tests.unit.api import CHUNK_SIZE, EMPTY_MD5, EMPTY_SHA256, \
+    empty_stream, decode_chunked_body, FakeResponse
 from oio.api import io
 from tests.unit import set_http_connect, set_http_requests
+from oio.common.constants import OIO_VERSION
 
 
 class TestReplication(unittest.TestCase):
@@ -27,6 +45,8 @@ class TestReplication(unittest.TestCase):
             'container_id': self.cid,
             'policy': 'REPLI3',
             'content_path': 'test',
+            'full_path': ['account/container/test'],
+            'oio_version': OIO_VERSION,
         }
 
         self._meta_chunk = [
@@ -39,7 +59,7 @@ class TestReplication(unittest.TestCase):
         return self._meta_chunk
 
     def checksum(self, d=''):
-        return md5(d)
+        return hashlib.md5(d)
 
     def test_write_simple(self):
         checksum = self.checksum()
@@ -48,13 +68,13 @@ class TestReplication(unittest.TestCase):
         size = CHUNK_SIZE
         resps = [201] * len(meta_chunk)
         with set_http_connect(*resps):
-            handler = ReplicatedChunkWriteHandler(
+            handler = ReplicatedMetachunkWriter(
                 self.sysmeta, meta_chunk, checksum, self.storage_method)
             bytes_transferred, checksum, chunks = handler.stream(
                 source, size)
             self.assertEqual(len(chunks), len(meta_chunk))
             self.assertEqual(bytes_transferred, 0)
-            self.assertEqual(checksum, EMPTY_CHECKSUM)
+            self.assertEqual(checksum, EMPTY_MD5)
 
     def test_write_exception(self):
         checksum = self.checksum()
@@ -63,7 +83,7 @@ class TestReplication(unittest.TestCase):
         size = CHUNK_SIZE
         resps = [500] * len(meta_chunk)
         with set_http_connect(*resps):
-            handler = ReplicatedChunkWriteHandler(
+            handler = ReplicatedMetachunkWriter(
                 self.sysmeta, meta_chunk, checksum, self.storage_method)
             self.assertRaises(exc.OioException, handler.stream, source, size)
 
@@ -76,7 +96,7 @@ class TestReplication(unittest.TestCase):
         resps = [201] * quorum_size
         resps += [500] * (len(meta_chunk) - quorum_size)
         with set_http_connect(*resps):
-            handler = ReplicatedChunkWriteHandler(
+            handler = ReplicatedMetachunkWriter(
                 self.sysmeta, meta_chunk, checksum, self.storage_method)
             bytes_transferred, checksum, chunks = handler.stream(source, size)
 
@@ -91,7 +111,7 @@ class TestReplication(unittest.TestCase):
             #     self.assertEqual(chunks[i].get('error'), 'HTTP 500')
 
             self.assertEqual(bytes_transferred, 0)
-            self.assertEqual(checksum, EMPTY_CHECKSUM)
+            self.assertEqual(checksum, EMPTY_MD5)
 
     def test_write_quorum_error(self):
         checksum = self.checksum()
@@ -102,7 +122,7 @@ class TestReplication(unittest.TestCase):
         resps = [500] * quorum_size
         resps += [201] * (len(meta_chunk) - quorum_size)
         with set_http_connect(*resps):
-            handler = ReplicatedChunkWriteHandler(
+            handler = ReplicatedMetachunkWriter(
                 self.sysmeta, meta_chunk, checksum, self.storage_method)
             self.assertRaises(exc.OioException, handler.stream, source, size)
 
@@ -114,7 +134,7 @@ class TestReplication(unittest.TestCase):
         resps = [201] * (len(meta_chunk) - 1)
         resps.append(Timeout(1.0))
         with set_http_connect(*resps):
-            handler = ReplicatedChunkWriteHandler(
+            handler = ReplicatedMetachunkWriter(
                 self.sysmeta, meta_chunk, checksum, self.storage_method)
             bytes_transferred, checksum, chunks = handler.stream(source, size)
 
@@ -129,7 +149,7 @@ class TestReplication(unittest.TestCase):
         #     chunks[len(meta_chunk) - 1].get('error'), '1.0 second')
 
         self.assertEqual(bytes_transferred, 0)
-        self.assertEqual(checksum, EMPTY_CHECKSUM)
+        self.assertEqual(checksum, EMPTY_MD5)
 
     def test_write_partial_exception(self):
         checksum = self.checksum()
@@ -140,7 +160,7 @@ class TestReplication(unittest.TestCase):
         resps = [201] * (len(meta_chunk) - 1)
         resps.append(Exception("failure"))
         with set_http_connect(*resps):
-            handler = ReplicatedChunkWriteHandler(
+            handler = ReplicatedMetachunkWriter(
                 self.sysmeta, meta_chunk, checksum, self.storage_method)
             bytes_transferred, checksum, chunks = handler.stream(source, size)
         self.assertEqual(len(chunks), len(meta_chunk)-1)
@@ -151,7 +171,7 @@ class TestReplication(unittest.TestCase):
         # self.assertEqual(chunks[len(meta_chunk) - 1].get('error'), 'failure')
 
         self.assertEqual(bytes_transferred, 0)
-        self.assertEqual(checksum, EMPTY_CHECKSUM)
+        self.assertEqual(checksum, EMPTY_MD5)
 
     def test_write_error_source(self):
         class TestReader(object):
@@ -165,7 +185,7 @@ class TestReplication(unittest.TestCase):
         nb = len(meta_chunk)
         resps = [201] * nb
         with set_http_connect(*resps):
-            handler = ReplicatedChunkWriteHandler(
+            handler = ReplicatedMetachunkWriter(
                 self.sysmeta, meta_chunk, checksum, self.storage_method)
             self.assertRaises(exc.SourceReadError, handler.stream, source,
                               size)
@@ -182,10 +202,10 @@ class TestReplication(unittest.TestCase):
         nb = len(meta_chunk)
         resps = [201] * nb
         with set_http_connect(*resps):
-            handler = ReplicatedChunkWriteHandler(
+            handler = ReplicatedMetachunkWriter(
                 self.sysmeta, meta_chunk, checksum, self.storage_method)
-            self.assertRaises(Timeout, handler.stream, source,
-                              size)
+            self.assertRaises(
+                exc.OioTimeout, handler.stream, source, size)
 
     def test_write_exception_source(self):
         class TestReader(object):
@@ -199,7 +219,7 @@ class TestReplication(unittest.TestCase):
         nb = len(meta_chunk)
         resps = [201] * nb
         with set_http_connect(*resps):
-            handler = ReplicatedChunkWriteHandler(
+            handler = ReplicatedMetachunkWriter(
                 self.sysmeta, meta_chunk, checksum, self.storage_method)
             # TODO specialize exception
             self.assertRaises(Exception, handler.stream, source,
@@ -220,7 +240,7 @@ class TestReplication(unittest.TestCase):
             put_reqs[conn_id]['parts'].append(part)
 
         with set_http_connect(*resps, cb_body=cb_body):
-            handler = ReplicatedChunkWriteHandler(
+            handler = ReplicatedMetachunkWriter(
                 self.sysmeta, meta_chunk, checksum, self.storage_method)
             bytes_transferred, checksum, chunks = handler.stream(source, size)
 
@@ -240,6 +260,43 @@ class TestReplication(unittest.TestCase):
             self.assertEqual(len(test_data), len(body))
             self.assertEqual(self.checksum(body).hexdigest(), final_checksum)
 
+    def _test_write_checksum_algo(self, expected_checksum, **kwargs):
+        global_checksum = self.checksum()
+        source = empty_stream()
+        meta_chunk = self.meta_chunk()
+        size = CHUNK_SIZE
+        resps = [201] * len(meta_chunk)
+        with set_http_connect(*resps, headers=kwargs.get('headers')):
+                handler = ReplicatedMetachunkWriter(
+                    self.sysmeta, self.meta_chunk(), global_checksum,
+                    self.storage_method, **kwargs)
+                bytes_transferred, checksum, chunks = \
+                    handler.stream(source, size)
+        self.assertEqual(len(meta_chunk), len(chunks))
+        self.assertEqual(0, bytes_transferred)
+        self.assertEqual(expected_checksum, checksum)
+
+    def test_write_default_checksum_algo(self):
+        with patch('hashlib.new', wraps=hashlib.new) as algo_new:
+            self._test_write_checksum_algo(EMPTY_MD5)
+            # Called only once for the metachunk
+            algo_new.assert_called_once_with('md5')
+
+    def test_write_custom_checksum_algo(self):
+        with patch('hashlib.new', wraps=hashlib.new) as algo_new:
+            self._test_write_checksum_algo(
+                EMPTY_SHA256, chunk_checksum_algo='sha256')
+            # Called only once for the metachunk
+            algo_new.assert_called_once_with('sha256')
+
+    def test_write_no_checksum_algo(self):
+        from oio.common.constants import CHUNK_HEADERS
+        headers = {CHUNK_HEADERS['chunk_hash']: EMPTY_MD5}
+        with patch('hashlib.new', wraps=hashlib.new) as algo_new:
+            self._test_write_checksum_algo(
+                EMPTY_MD5, chunk_checksum_algo=None, headers=headers)
+            algo_new.assert_not_called()
+
     def test_read(self):
         test_data = ('1234' * 1024)[:-10]
         data_checksum = self.checksum(test_data).hexdigest()
