@@ -1,7 +1,7 @@
 /*
-OpenIO SDS meta2v2
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
+OpenIO SDS unit tests
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -20,10 +20,10 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 #include <glib.h>
 
+#include <metautils/lib/common_variables.h>
 #include <meta2v2/meta2_macros.h>
 #include <meta2v2/meta2_utils.h>
 #include <meta2v2/meta2_backend_internals.h>
-#include <meta2v2/meta2v2_remote.h>
 #include <meta2v2/generic.h>
 #include <meta2v2/autogen.h>
 #include <resolver/hc_resolver.h>
@@ -37,7 +37,6 @@ typedef void (*container_test_f) (struct meta2_backend_s *m2,
 		struct oio_url_s *url, gint64 maxver);
 
 static guint64 container_counter = 0;
-static gint64 chunk_size = 3000;
 static gint64 chunks_count = 3;
 static volatile gint64 CLOCK_START = 0;
 static volatile gint64 CLOCK = 0;
@@ -57,12 +56,19 @@ static gint64 _get_monotonic (void) { return CLOCK; }
 
 static gint64 _get_real (void) { return CLOCK; }
 
-static gboolean
-_versioned(struct meta2_backend_s *m2, struct oio_url_s *u)
+static gint64
+_version(struct meta2_backend_s *m2, struct oio_url_s *u)
 {
 	gint64 v = 0;
 	GError *err = meta2_backend_get_max_versions(m2, u, &v);
 	g_assert_no_error(err);
+	return v;
+}
+
+static gboolean
+_versioned(struct meta2_backend_s *m2, struct oio_url_s *u)
+{
+	const gint64 v = _version(m2, u);
 	return VERSIONS_ENABLED(v);
 }
 
@@ -106,9 +112,10 @@ _props_generate(struct oio_url_s *url, gint64 v, guint count)
 	return result;
 }
 
+
 static GSList*
-_create_alias(struct meta2_backend_s *m2b, struct oio_url_s *url,
-		const gchar *polname)
+_create_alias2(struct meta2_backend_s *m2b, struct oio_url_s *url,
+		const gchar *polname, int chunks_per_metachunks)
 {
 	void _onbean(gpointer u, gpointer bean) {
 		*((GSList**)u) = g_slist_prepend(*((GSList**)u), bean);
@@ -120,10 +127,13 @@ _create_alias(struct meta2_backend_s *m2b, struct oio_url_s *url,
 
 	g_assert(chunks_count > 1);
 	err = meta2_backend_generate_beans(m2b, url,
-			(chunk_size*(chunks_count-1))+1, polname, FALSE, _onbean, &beans);
+			(oio_ns_chunk_size*(chunks_count-1))+1,
+			polname, FALSE, _onbean, &beans);
 	generated = g_slist_length(beans);
-	expected = 1 + 1 + chunks_count;
-	GRID_DEBUG("BEANS generated=%u expected=%u", generated, expected);
+	expected = 1 + 1 + chunks_count * chunks_per_metachunks;
+	GRID_DEBUG("BEANS policy=%s generated=%u expected=%u",
+			polname, generated, expected);
+	_bean_debugl2(__FUNCTION__, beans);
 	g_assert_no_error(err);
 	g_assert(generated == expected);
 
@@ -131,6 +141,13 @@ _create_alias(struct meta2_backend_s *m2b, struct oio_url_s *url,
 	return beans;
 }
 
+static GSList*
+_create_alias(struct meta2_backend_s *m2b, struct oio_url_s *url,
+		const gchar *polname)
+{
+	return _create_alias2(m2b, url, polname, 1);
+}
+
 static void
 check_list_count(struct meta2_backend_s *m2, struct oio_url_s *url,
 		guint expected)
@@ -157,18 +174,14 @@ static struct namespace_info_s *
 _init_nsinfo(const gchar *ns, gint64 maxvers)
 {
 	struct namespace_info_s *nsinfo;
-	gchar str[512];
 
 	nsinfo = g_malloc0 (sizeof(*nsinfo));
 	namespace_info_init (nsinfo);
-	nsinfo->chunk_size = chunk_size;
 	g_strlcpy (nsinfo->name, ns, sizeof(nsinfo->name));
 
-	g_snprintf (str, sizeof(str), "%"G_GINT64_FORMAT, maxvers);
-	g_hash_table_insert(nsinfo->options, g_strdup("meta2_max_versions"),
-			metautils_gba_from_string(str));
-	g_hash_table_insert(nsinfo->options, g_strdup("storage_policy"),
-			metautils_gba_from_string("NONE"));
+	gchar tmp[32];
+	g_snprintf(tmp, sizeof(tmp), "%" G_GINT64_FORMAT, maxvers);
+	g_assert_true(oio_var_value_one_with_option("meta2.max_versions", tmp));
 
 	g_hash_table_insert(nsinfo->storage_policy, g_strdup("classic"),
 			metautils_gba_from_string("NONE:DUPONETWO"));
@@ -176,6 +189,12 @@ _init_nsinfo(const gchar *ns, gint64 maxvers)
 			metautils_gba_from_string("NONE:DUPONETHREE"));
 	g_hash_table_insert(nsinfo->storage_policy, g_strdup("secure"),
 			metautils_gba_from_string("NONE:DUP_SECURE"));
+	g_hash_table_insert(nsinfo->storage_policy, g_strdup("THREECOPIES"),
+			metautils_gba_from_string("rawx3:DUPONETHREE"));
+	g_hash_table_insert(nsinfo->storage_policy, g_strdup("TWOCOPIES"),
+			metautils_gba_from_string("rawx2:DUPONETWO"));
+	g_hash_table_insert(nsinfo->storage_policy, g_strdup("EC"),
+			metautils_gba_from_string("EC:EC"));
 
 	g_hash_table_insert(nsinfo->data_security, g_strdup("DUPONETWO"),
 			metautils_gba_from_string("plain/distance=1,nb_copy=2"));
@@ -183,6 +202,8 @@ _init_nsinfo(const gchar *ns, gint64 maxvers)
 			metautils_gba_from_string("plain/distance=1,nb_copy=3"));
 	g_hash_table_insert(nsinfo->data_security, g_strdup("DUP_SECURE"),
 			metautils_gba_from_string("plain/distance=4,nb_copy=2"));
+	g_hash_table_insert(nsinfo->data_security, g_strdup("EC"),
+			metautils_gba_from_string("ec/k=6,m=3,algo=liberasurecode_rs_vand,distance=1"));
 
 	return nsinfo;
 }
@@ -202,6 +223,7 @@ _init_lb(int nb_services)
 		g_snprintf(item->id, LIMIT_LENGTH_SRVID, "127.0.0.1:%d", 6000+i);
 		oio_lb_world__feed_slot(lb_world, "*", item);
 	}
+	oio_lb_world__purge_old_generations(lb_world);
 	oio_lb_world__debug(lb_world);
 
 	struct oio_lb_pool_s *pool = oio_lb_world__create_pool(lb_world,
@@ -212,6 +234,21 @@ _init_lb(int nb_services)
 	return lb;
 }
 
+static void
+_init_pool_ec_2cpy_3cpy(struct meta2_backend_s *m2)
+{
+	struct oio_lb_pool_s *rawx3 = oio_lb_world__create_pool(lb_world, "rawx3");
+	oio_lb_world__add_pool_targets(rawx3, "3,*");
+	oio_lb__force_pool(m2->lb, rawx3);
+
+	struct oio_lb_pool_s *rawx2 = oio_lb_world__create_pool(lb_world, "rawx2");
+	oio_lb_world__add_pool_targets(rawx2, "2,*");
+	oio_lb__force_pool(m2->lb, rawx2);
+
+	struct oio_lb_pool_s *ec = oio_lb_world__create_pool(lb_world, "EC");
+	oio_lb_world__add_pool_targets(ec, "9,*");
+	oio_lb__force_pool(m2->lb, ec);
+}
 static void
 _repo_wrapper(const gchar *ns, gint64 maxvers, repo_test_f fr)
 {
@@ -233,7 +270,7 @@ _repo_wrapper(const gchar *ns, gint64 maxvers, repo_test_f fr)
 			g_get_home_dir(), getpid());
 	g_mkdir_with_parents(repodir, 0755);
 
-	lb = _init_lb(6);
+	lb = _init_lb(9);
 	g_assert_nonnull(lb);
 
 	resolver = hc_resolver_create();
@@ -242,7 +279,6 @@ _repo_wrapper(const gchar *ns, gint64 maxvers, repo_test_f fr)
 	cfg.flags = SQLX_REPO_DELETEON;
 	cfg.sync_solo = SQLX_SYNC_OFF;
 	cfg.sync_repli = SQLX_SYNC_OFF;
-	cfg.max_bases = 64; /* no need for many slots */
 	err = sqlx_repository_init(repodir, &cfg, &repository);
 	g_assert_no_error(err);
 
@@ -284,7 +320,6 @@ _repo_failure(const gchar *ns)
 	g_assert_nonnull(resolver);
 
 	cfg.flags = SQLX_REPO_DELETEON;
-	cfg.max_bases = 64;
 	err = sqlx_repository_init(repodir, &cfg, &repository);
 	g_assert_no_error(err);
 	err = meta2_backend_init(&backend, repository, ns, lb, resolver);
@@ -301,7 +336,7 @@ static void
 _container_wraper(const char *ns, gint64 maxvers, container_test_f cf)
 {
 	void test(struct meta2_backend_s *m2) {
-		struct m2v2_create_params_s params = {NULL, NULL, NULL, FALSE};
+		struct m2v2_create_params_s params = {0};
 		struct oio_url_s *url;
 		GError *err;
 
@@ -332,7 +367,6 @@ _container_wraper(const char *ns, gint64 maxvers, container_test_f cf)
 static void
 _container_wraper_allversions (const char *ns, container_test_f cf)
 {
-	_container_wraper (ns, -2, cf);
 	_container_wraper (ns, -1, cf);
 	_container_wraper (ns, 0, cf);
 	_container_wraper (ns, 1, cf);
@@ -374,7 +408,8 @@ static void
 test_container_create_destroy(void)
 {
 	void test(struct meta2_backend_s *m2, struct oio_url_s *u, gint64 maxver) {
-		g_assert (VERSIONS_ENABLED(maxver) == _versioned(m2, u));
+		g_assert_cmpint(maxver, ==, _version(m2, u));
+		g_assert_cmpint(VERSIONS_ENABLED(maxver), ==, _versioned(m2, u));
 	}
 	_container_wraper_allversions("NS", test);
 }
@@ -403,6 +438,243 @@ test_content_put_no_beans(void)
 	_container_wraper_allversions("NS", test);
 }
 
+static void
+_remove_bean(GSList **beans, guint nb, gchar *pos)
+{
+	for (guint i = 0; i < nb ; i++) {
+		for (GSList *l = *beans; l; l = l->next) {
+			gpointer bean = l->data;
+			if (DESCR(bean) == &descr_struct_CHUNKS) {
+				if (pos) {
+					GString *pos_bean = CHUNKS_get_position(bean);
+					if (!g_strcmp0(pos_bean->str, pos)) {
+						*beans = g_slist_remove(*beans, bean);
+						_bean_clean(bean);
+						break;
+					}
+				} else  {
+					*beans = g_slist_remove(*beans, bean);
+					_bean_clean(bean);
+					break;
+				}
+			}
+		}
+	}
+}
+
+static void
+test_content_check_all_beans_correct(void)
+{
+	void test(struct meta2_backend_s * m2, struct oio_url_s * u, gint64 maxver) {
+		(void) maxver;
+		_init_pool_ec_2cpy_3cpy(m2);
+		GError *err;
+		GSList *beans_3cpy = _create_alias2(m2, u, "THREECOPIES", 3);
+		GString *message = g_string_new("");
+		err = meta2_backend_check_content(m2, beans_3cpy, message, FALSE);
+		g_string_free(message, TRUE);
+		_bean_cleanl2(beans_3cpy);
+		g_assert_no_error(err);
+
+		GSList *beans_2cpy = _create_alias2(m2, u, "TWOCOPIES", 2);
+		message = g_string_new("");
+		err = meta2_backend_check_content(m2, beans_2cpy, message, FALSE);
+		g_string_free(message, TRUE);
+		_bean_cleanl2(beans_2cpy);
+		g_assert_no_error(err);
+
+		message = g_string_new("");
+		GSList *beans_ec = _create_alias2(m2, u, "EC", 3);
+		err = meta2_backend_check_content(m2, beans_ec, message, FALSE);
+		g_string_free(message, TRUE);
+		_bean_cleanl2(beans_ec);
+		g_assert_no_error(err);
+
+	}
+	_container_wraper_allversions("NS", test);
+}
+
+static void
+test_content_check_1_missing_bean_plain_irreparable(void)
+{
+	void test(struct meta2_backend_s * m2, struct oio_url_s * u, gint64 maxver) {
+		(void) maxver;
+		GError *err;
+		GSList *beans = _create_alias(m2, u, NULL);
+		GRID_DEBUG("TEST nb_beans=%u", g_slist_length(beans));
+		for (GSList *l = beans; l; l = l->next) {
+			gpointer bean = l->data;
+			if(DESCR(bean) == &descr_struct_CHUNKS) {
+				beans = g_slist_remove(beans, bean);
+				_bean_clean(bean);
+				break;
+			}
+		}
+		GRID_DEBUG("TEST nb_beans=%u", g_slist_length(beans));
+		GString *message = g_string_new("");
+		err = meta2_backend_check_content(m2, beans, message, FALSE);
+		g_assert_error(err, GQ(), CODE_CONTENT_CORRUPTED);
+		g_error_free(err);
+		g_string_free(message, TRUE);
+		_bean_cleanl2(beans);
+	}
+	_container_wraper_allversions("NS", test);
+}
+
+static void
+test_content_check_1_missing_bean_plain_copy_reparable(void)
+{
+	void test(struct meta2_backend_s * m2, struct oio_url_s * u, gint64 maxver) {
+		(void) maxver;
+		_init_pool_ec_2cpy_3cpy(m2);
+		GError *err;
+		GSList *beans_2cpy = _create_alias2(m2, u, "TWOCOPIES", 2);
+		GString *message_2cpy = g_string_new("");
+		_remove_bean(&beans_2cpy, 1, NULL);
+		err = meta2_backend_check_content(m2, beans_2cpy, message_2cpy, FALSE);
+		g_assert_error(err, GQ(), CODE_CONTENT_UNCOMPLETE);
+		g_error_free(err);
+		gchar *missing_chunks = g_strrstr(message_2cpy->str, "\"missing_chunks\":[2]");
+		g_assert_nonnull(missing_chunks);
+		_bean_cleanl2(beans_2cpy);
+		g_string_free(message_2cpy, TRUE);
+
+		GSList *beans_3cpy = _create_alias2(m2, u, "THREECOPIES", 3);
+		GString *message_3cpy = g_string_new("");
+		_remove_bean(&beans_3cpy, 1, NULL);
+		err = meta2_backend_check_content(m2, beans_3cpy, message_3cpy, FALSE);
+		g_assert_error(err, GQ(), CODE_CONTENT_UNCOMPLETE);
+		g_error_free(err);
+		missing_chunks = g_strrstr(message_3cpy->str, "\"missing_chunks\":[2]");
+		g_assert_nonnull(missing_chunks);
+		_bean_cleanl2(beans_3cpy);
+		g_string_free(message_3cpy, TRUE);
+	}
+	_container_wraper_allversions("NS", test);
+
+}
+
+static void
+test_content_check_2_missing_bean_plain_copy_reparable(void)
+{
+	void test(struct meta2_backend_s * m2, struct oio_url_s * u, gint64 maxver) {
+		(void) maxver;
+		_init_pool_ec_2cpy_3cpy(m2);
+		GError *err;
+		GSList *beans_3cpy = _create_alias2(m2, u, "THREECOPIES", 3);
+		GString *message_3cpy = g_string_new("");
+		_remove_bean(&beans_3cpy, 2, NULL);
+		err = meta2_backend_check_content(m2, beans_3cpy, message_3cpy, FALSE);
+		g_assert_error(err, GQ(), CODE_CONTENT_UNCOMPLETE);
+		g_error_free(err);
+		gchar *missing_chunks = g_strrstr(message_3cpy->str, "\"missing_chunks\":[2,2]");
+		g_assert_nonnull(missing_chunks);
+		g_slist_free_full(beans_3cpy, _bean_clean);
+		g_string_free(message_3cpy, TRUE);
+
+	}
+	_container_wraper_allversions("NS", test);
+}
+
+static void
+test_content_check_missing_bean_plain_copy_irreparable(void)
+{
+	void test(struct meta2_backend_s * m2, struct oio_url_s * u, gint64 maxver) {
+		(void) maxver;
+		_init_pool_ec_2cpy_3cpy(m2);
+		GError *err;
+		GSList *beans_2cpy = _create_alias2(m2, u, "TWOCOPIES", 2);
+		GString *message_2cpy = g_string_new("");
+		_remove_bean(&beans_2cpy, 2, NULL);
+		err = meta2_backend_check_content(m2, beans_2cpy, message_2cpy, FALSE);
+		g_assert_error(err, GQ(), CODE_CONTENT_CORRUPTED);
+		g_error_free(err);
+		g_string_free(message_2cpy, TRUE);
+		_bean_cleanl2(beans_2cpy);
+
+		GSList *beans_3cpy = _create_alias2(m2, u, "THREECOPIES", 3);
+		GString *message_3cpy = g_string_new("");
+		_remove_bean(&beans_3cpy, 3, NULL);
+		err = meta2_backend_check_content(m2, beans_3cpy, message_3cpy, FALSE);
+		g_assert_error(err, GQ(), CODE_CONTENT_CORRUPTED);
+		g_error_free(err);
+		_bean_cleanl2(beans_3cpy);
+		g_string_free(message_3cpy, TRUE);
+	}
+	_container_wraper_allversions("NS", test);
+}
+
+static void
+test_content_check_missing_first_pos(void)
+{
+	void test(struct meta2_backend_s * m2, struct oio_url_s * u, gint64 maxver) {
+		(void) maxver;
+		_init_pool_ec_2cpy_3cpy(m2);
+		GError *err;
+		GString *message_nocpy = g_string_new("");
+		GSList *beans_nocpy = _create_alias(m2, u, NULL);
+		_remove_bean(&beans_nocpy, 1, "0");
+		err = meta2_backend_check_content(m2, beans_nocpy, message_nocpy, FALSE);
+		g_assert_error(err, GQ(), CODE_CONTENT_CORRUPTED);
+		g_error_free(err);
+		_bean_cleanl2(beans_nocpy);
+		g_string_free(message_nocpy, TRUE);
+
+		GSList *beans_2cpy = _create_alias2(m2, u, "TWOCOPIES", 2);
+		GString *message_2cpy = g_string_new("");
+		_remove_bean(&beans_2cpy, 1, "0");
+		err = meta2_backend_check_content(m2, beans_2cpy, message_2cpy, FALSE);
+		g_assert_error(err, GQ(), CODE_CONTENT_UNCOMPLETE);
+		g_error_free(err);
+		_remove_bean(&beans_2cpy, 1, "0");
+		err = meta2_backend_check_content(m2, beans_2cpy, message_2cpy, FALSE);
+		g_assert_error(err, GQ(), CODE_CONTENT_CORRUPTED);
+		g_error_free(err);
+		_bean_cleanl2(beans_2cpy);
+		g_string_free(message_2cpy, TRUE);
+	}
+	_container_wraper_allversions("NS", test);
+}
+
+static void
+test_content_check_ec_missing_1_chunk(void)
+{
+	void test(struct meta2_backend_s * m2, struct oio_url_s * u, gint64 maxver) {
+		(void) maxver;
+		_init_pool_ec_2cpy_3cpy(m2);
+		GError *err;
+		GString *message_ec1 = g_string_new("");
+		GSList *beans_ec1 = _create_alias2(m2, u, "EC", 3);
+		_remove_bean(&beans_ec1, 1, NULL);
+		err = meta2_backend_check_content(m2, beans_ec1, message_ec1, FALSE);
+		g_assert_error(err, GQ(), CODE_CONTENT_UNCOMPLETE);
+		g_error_free(err);
+		_bean_cleanl2(beans_ec1);
+		g_string_free(message_ec1, TRUE);
+
+		GString *message_ecm  = g_string_new("");
+		GSList *beans_ecm = _create_alias2(m2, u, "EC", 3);
+		int m = 3;
+		_remove_bean(&beans_ecm, m, NULL);
+		err = meta2_backend_check_content(m2, beans_ecm, message_ecm, FALSE);
+		g_assert_error(err, GQ(), CODE_CONTENT_UNCOMPLETE);
+		g_error_free(err);
+		_bean_cleanl2(beans_ecm);
+		g_string_free(message_ecm, TRUE);
+
+		GString *message_ecm1  = g_string_new("");
+		GSList *beans_ecm1 = _create_alias2(m2, u, "EC", 3);
+		int m1 = m + 1;
+		_remove_bean(&beans_ecm1, m1, NULL);
+		err = meta2_backend_check_content(m2, beans_ecm1, message_ecm1, FALSE);
+		g_assert_error(err, GQ(), CODE_CONTENT_CORRUPTED);
+		g_error_free(err);
+		_bean_cleanl2(beans_ecm1);
+		g_string_free(message_ecm1, TRUE);
+	}
+	_container_wraper_allversions("NS", test);
+}
+
 static void
 test_content_put_prop_get(void)
 {
@@ -447,7 +719,7 @@ test_content_put_prop_get(void)
 		expected = 2 + chunks_count;
 		GRID_DEBUG("TEST count=%u expected=%u", tmp->len, expected);
 		_debug_beans_array(tmp);
-		g_assert(tmp->len == expected);
+		g_assert_cmpuint(tmp->len, ==, expected);
 		_bean_cleanv2(tmp);
 
 		/* idem, but with the properties */
@@ -458,7 +730,7 @@ test_content_put_prop_get(void)
 		expected = 10 + 2 + chunks_count;
 		GRID_DEBUG("TEST count=%u expected=%u", tmp->len, expected);
 		_debug_beans_array(tmp);
-		g_assert(tmp->len == expected);
+		g_assert_cmpuint(tmp->len, ==, expected);
 		_bean_cleanv2(tmp);
 
 		/* delete the bean */
@@ -479,7 +751,7 @@ test_content_put_prop_get(void)
 		g_clear_error (&err);
 		expected = 0;
 		GRID_DEBUG("TEST count=%u expected=%u", tmp->len, expected);
-		g_assert(tmp->len == expected);
+		g_assert_cmpuint(tmp->len, ==, expected);
 		_bean_cleanv2(tmp);
 	}
 
@@ -514,7 +786,7 @@ test_content_put_get_delete(void)
 				_bean_buffer_cb, tmp);
 		g_assert_no_error(err);
 		expected = 2+chunks_count;
-		g_assert(tmp->len == expected);
+		g_assert_cmpuint(tmp->len, ==, expected);
 		_bean_cleanv2(tmp);
 
 		check_list_count(m2,u,1);
@@ -536,7 +808,7 @@ test_content_put_get_delete(void)
 				_bean_buffer_cb, tmp);
 		g_assert_error(err, GQ(), CODE_CONTENT_NOTFOUND);
 		g_clear_error (&err);
-		g_assert(tmp->len == 0);
+		g_assert_cmpuint(tmp->len, ==, 0);
 		_bean_cleanv2(tmp);
 
 		if (VERSIONS_ENABLED(maxver)) {
@@ -556,11 +828,11 @@ test_content_put_get_delete(void)
 			GRID_DEBUG("TEST Got %u beans for all versions, expected %u"
 					" (chunks count: %"G_GINT64_FORMAT")",
 					tmp->len, expected, chunks_count);
-			g_assert(tmp->len == expected);
+			g_assert_cmpuint(tmp->len, ==, expected);
 		} else {
 			g_assert_error(err, GQ(), CODE_CONTENT_NOTFOUND);
 			g_clear_error (&err);
-			g_assert(tmp->len == 0);
+			g_assert_cmpuint(tmp->len, ==, 0);
 		}
 		_bean_cleanv2(tmp);
 
@@ -579,6 +851,44 @@ test_content_put_get_delete(void)
 	_container_wraper_allversions("NS", test);
 }
 
+static void
+test_content_put_lower_version(void)
+{
+	void test(struct meta2_backend_s *m2, struct oio_url_s *u, gint64 maxver) {
+		(void) maxver;
+		GSList *beans = NULL;
+		GError *err;
+
+		CLOCK_START = CLOCK = oio_ext_rand_int();
+
+		/* generate the beans for an alias of 3 chunks */
+		beans = _create_alias(m2, u, NULL);
+
+		/* first PUT */
+		err = meta2_backend_put_alias(m2, u, beans, NULL, NULL);
+		g_assert_no_error(err);
+		CHECK_ALIAS_VERSION(m2, u, _get_real());
+		check_list_count(m2, u, 1);
+		_bean_cleanl2(beans);
+
+		/* second PUT, with lower version */
+		CLOCK--;
+		beans = _create_alias(m2, u, NULL);
+		CLOCK++;
+		err = meta2_backend_put_alias(m2, u, beans, NULL, NULL);
+		g_assert_error(err, GQ(), CODE_CONTENT_PRECONDITION);
+		_bean_cleanl2(beans);
+		g_clear_error(&err);
+
+		CHECK_ALIAS_VERSION(m2, u, CLOCK_START);
+	}
+	_container_wraper("NS", -1, test);
+	/* Would fail for another reason */
+	// _container_wraper (ns, 0, cf);
+	_container_wraper("NS", 1, test);
+	_container_wraper("NS", 2, test);
+}
+
 static void
 test_content_append_empty(void)
 {
@@ -620,13 +930,13 @@ test_content_append(void)
 		expected = 1 + 1 + chunks_count;
 		GRID_DEBUG("Put -> %u beans (ALLVERSION)", tmp->len);
 		//_debug_beans_array (tmp);
-		g_assert(tmp->len == expected);
+		g_assert_cmpuint(tmp->len, ==, expected);
 		_bean_cleanv2(tmp);
 
 		/* append th same chunks */
 		tmp = g_ptr_array_new ();
 		err = meta2_backend_append_to_alias(m2, u, beans, _bean_buffer_cb, tmp);
-		g_assert(err != NULL);
+		g_assert_nonnull(err);
 		g_clear_error (&err);
 		_bean_cleanv2 (tmp);
 
@@ -653,7 +963,7 @@ test_content_append(void)
 			+ chunks_count  /* original chunks */
 			+ chunks_count; /* new chunks appended */
 		GRID_DEBUG("TEST After the append, got %u, expected %u", tmp->len, expected);
-		g_assert(tmp->len == expected);
+		g_assert_cmpuint(tmp->len, ==, expected);
 		_bean_cleanv2(tmp);
 
 		/* delete the alias */
@@ -724,7 +1034,7 @@ test_content_append_not_found(void)
 		CLOCK ++;
 		g_assert_no_error(err);
 		expected = 1 + 1 + chunks_count;
-		g_assert(tmp->len == expected);
+		g_assert_cmpuint(tmp->len, ==, expected);
 		_bean_cleanv2(tmp);
 
 		/* re-APPEND */
@@ -747,7 +1057,7 @@ test_content_append_not_found(void)
 		expected = 1 /* alias */ + 1 /* headers */
 			+ chunks_count /* original chunks+contents */
 			+ chunks_count; /* new chunks appended */
-		g_assert(tmp->len == expected);
+		g_assert_cmpuint(tmp->len, ==, expected);
 		_bean_cleanv2(tmp);
 
 		/* delete the bean */
@@ -765,7 +1075,7 @@ test_content_append_not_found(void)
 		err = meta2_backend_get_alias(m2, u, M2V2_FLAG_NODELETED, _bean_buffer_cb, tmp);
 		g_assert_error(err, GQ(), CODE_CONTENT_NOTFOUND);
 		g_clear_error (&err);
-		g_assert(tmp->len == 0);
+		g_assert_cmpuint(tmp->len, ==, 0);
 		_bean_cleanv2(tmp);
 
 		/* check we can get both deleted and previous versions */
@@ -923,6 +1233,8 @@ main(int argc, char **argv)
 			test_content_delete_not_found);
 	g_test_add_func("/meta2v2/backend/content/put_get_delete",
 			test_content_put_get_delete);
+	g_test_add_func("/meta2v2/backend/content/put_lower_version",
+			test_content_put_lower_version);
 	g_test_add_func("/meta2v2/backend/content/put_prop_get",
 			test_content_put_prop_get);
 	g_test_add_func("/meta2v2/backend/content/append_empty",
@@ -937,7 +1249,22 @@ main(int argc, char **argv)
 			test_content_append_not_found);
 	g_test_add_func("/meta2v2/backend/content/dedup",
 			test_content_dedup);
+	g_test_add_func("/meta2v2/backend/content/check_all_beans_correct",
+			test_content_check_all_beans_correct);
+	g_test_add_func("/meta2v2/backend/content/check_1_missing_bean_irreparable",
+			test_content_check_1_missing_bean_plain_irreparable);
+	g_test_add_func("/meta2v2/backend/content/check_1_missing_bean_reparable",
+			test_content_check_1_missing_bean_plain_copy_reparable);
+	g_test_add_func("/meta2v2/backend/content/check_2_missing_bean_reparable",
+			test_content_check_2_missing_bean_plain_copy_reparable);
+	g_test_add_func("/meta2v2/backend/content/check_missing_last_pos",
+			test_content_check_missing_bean_plain_copy_irreparable);
+	g_test_add_func("/meta2v2/backend/content/check_missing_first_pos",
+			test_content_check_missing_first_pos);
+	g_test_add_func("/meta2v2/backend/content/check_missing_ec_1_chunk",
+			test_content_check_ec_missing_1_chunk);
 
 	return g_test_run();
 }
 
+
