@@ -1,7 +1,7 @@
 /*
-OpenIO SDS sqliterepo
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+OpenIO SDS unit tests
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -26,6 +26,7 @@ License along with this library.
 #include <sqliterepo/sqliterepo.h>
 #include <sqliterepo/cache.h>
 #include <sqliterepo/internals.h>
+#include <sqliterepo/sqliterepo_variables.h>
 
 #undef GQ
 #define GQ() g_quark_from_static_string("oio.sqlite")
@@ -57,10 +58,10 @@ _round_lock(sqlx_cache_t *cache)
 		g_assert_cmpint(id0, ==, id);
 	}
 	for (int i=0; i<6 ;i++) {
-		err = sqlx_cache_unlock_and_close_base(cache, id0, FALSE);
+		err = sqlx_cache_unlock_and_close_base(cache, id0, 0);
 		g_assert_no_error (err);
 	}
-	err = sqlx_cache_unlock_and_close_base(cache, id0, FALSE);
+	err = sqlx_cache_unlock_and_close_base(cache, id0, 0);
 	g_assert_error (err, GQ(), CODE_INTERNAL_ERROR);
 	g_clear_error (&err);
 
@@ -68,7 +69,7 @@ _round_lock(sqlx_cache_t *cache)
 		gint id = oio_ext_rand_int ();
 		err = sqlx_cache_open_and_lock_base(cache, hn1, FALSE, &id);
 		g_assert_no_error (err);
-		err = sqlx_cache_unlock_and_close_base(cache, id, FALSE);
+		err = sqlx_cache_unlock_and_close_base(cache, id, 0);
 		g_assert_no_error (err);
 	}
 }
@@ -76,9 +77,8 @@ _round_lock(sqlx_cache_t *cache)
 static void
 test_lock (void)
 {
-	sqlx_cache_t *cache = sqlx_cache_init(16);
+	sqlx_cache_t *cache = sqlx_cache_init();
 	g_assert(cache != NULL);
-	sqlx_cache_set_max_bases (cache, 8);
 	sqlx_cache_set_close_hook(cache, sqlite_close);
 	for (int j=g_random_int_range(3,5); j>0 ;j--) {
 		for (int i=g_random_int_range(5,7); i>0 ;i--)
@@ -92,10 +92,9 @@ test_lock (void)
 static void
 _round_init (void)
 {
-	sqlx_cache_t *cache = sqlx_cache_init(16);
+	sqlx_cache_t *cache = sqlx_cache_init();
 	g_assert(cache != NULL);
 	for (int j=g_random_int_range(3,5); j>0 ;j--) {
-		sqlx_cache_set_max_bases (cache, 8);
 		sqlx_cache_set_close_hook(cache, sqlite_close);
 		sqlx_cache_debug(cache);
 		sqlx_cache_expire(cache, 0, 0);
@@ -139,7 +138,7 @@ _test_cache_limit (sqlx_cache_t *cache, guint max)
 
 	// 3 a bit of cleanup
 	for (guint i=0; i<max ;++i) {
-		GError *err = sqlx_cache_unlock_and_close_base(cache, ids[i], FALSE);
+		GError *err = sqlx_cache_unlock_and_close_base(cache, ids[i], 0);
 		g_assert_no_error(err);
 	}
 }
@@ -150,18 +149,22 @@ test_limit (void)
 	guint all_maxes[] = {1, 2, 4, 8, 0};
 	for (guint imax=0; all_maxes[imax] != 0 ;++imax) {
 
-		guint hardmax = all_maxes[imax];
-		sqlx_cache_t *cache = sqlx_cache_init(hardmax);
+		sqliterepo_repo_max_bases_hard = all_maxes[imax];
+		sqliterepo_repo_max_bases_soft = all_maxes[imax];
+		sqlx_cache_t *cache = sqlx_cache_init();
 		g_assert_nonnull(cache);
-		_test_cache_limit (cache, hardmax);
-		if (hardmax > 1) {
-			guint softmax = hardmax / 2;
-			sqlx_cache_set_max_bases(cache, softmax);
-			_test_cache_limit (cache, softmax);
+		_test_cache_limit (cache, sqliterepo_repo_max_bases_hard);
+		if (sqliterepo_repo_max_bases_hard > 1) {
+			sqliterepo_repo_max_bases_soft = sqliterepo_repo_max_bases_hard / 2;
+			sqlx_cache_reconfigure(cache);
+			_test_cache_limit (cache, sqliterepo_repo_max_bases_soft);
 		}
 		sqlx_cache_debug(cache);
 		sqlx_cache_expire(cache, 0, 0);
 		sqlx_cache_clean(cache);
+
+		// restore a value that is high-enough
+		sqliterepo_repo_max_bases_hard = 8192;
 	}
 }
 
