@@ -1,13 +1,30 @@
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
+#
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+# License as published by the Free Software Foundation; either
+# version 3.0 of the License, or (at your option) any later version.
+#
+# This library is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public
+# License along with this library.
+
 from __future__ import print_function
+import logging
 import sys
 import os
-import json
 import yaml
 import testtools
-import requests
 import random
 import string
 from functools import wraps
+from oio.common.http import get_pool_manager
+from urllib import urlencode
+from oio.common.utils import json as jsonlib
 
 random_chars = string.ascii_letters + string.digits
 random_chars_id = 'ABCDEF' + string.digits
@@ -70,7 +87,12 @@ def get_config(defaults=None):
     return conf
 
 
-class BaseTestCase(testtools.TestCase):
+class CommonTestCase(testtools.TestCase):
+
+    def is_running_on_public_ci(self):
+        from os import getenv
+        clues = (getenv("TRAVIS"), getenv("CIRCLECI"))
+        return any(clue is not None for clue in clues)
 
     def _random_user(self):
         return "user-" + random_str(16, "0123456789ABCDEF")
@@ -112,65 +134,120 @@ class BaseTestCase(testtools.TestCase):
     def param_content(self, ref, path):
         return {'ref': ref, 'acct': self.account, 'path': path}
 
+    @staticmethod
+    def static_request(method, url, data=None, params=None, headers=None,
+                       json=None, http_pool=None):
+        if not http_pool:
+            http_pool = get_pool_manager()
+        # Add query string
+        if params:
+            out_param = []
+            for k, v in params.items():
+                if v is not None:
+                    if isinstance(v, unicode):
+                        v = unicode(v).encode('utf-8')
+                    out_param.append((k, v))
+            encoded_args = urlencode(out_param)
+            url += '?' + encoded_args
+
+        # Convert json and add Content-Type
+        headers = headers if headers else {}
+        if json:
+            headers["Content-Type"] = "application/json"
+            data = jsonlib.dumps(json)
+
+        out_kwargs = {}
+        out_kwargs['headers'] = headers
+        out_kwargs['body'] = data
+
+        return http_pool.request(method, url, **out_kwargs)
+
+    def request(self, method, url,
+                data=None, params=None, headers=None, json=None):
+        return self.static_request(method, url, data=data, params=params,
+                                   headers=headers, json=json,
+                                   http_pool=self.http_pool)
+
+    @classmethod
+    def setUpClass(cls):
+        cls._cls_conf = get_config()
+        cls._cls_account = cls._cls_conf['account']
+        cls._cls_ns = cls._cls_conf['namespace']
+        cls._cls_uri = 'http://' + cls._cls_conf['proxy']
+
     def setUp(self):
-        super(BaseTestCase, self).setUp()
+        super(CommonTestCase, self).setUp()
         self.conf = get_config()
         self.uri = 'http://' + self.conf['proxy']
         self.ns = self.conf['namespace']
         self.account = self.conf['account']
-        self.session = requests.session()
-        self._flush_cs('echo')
+        self.http_pool = get_pool_manager()
 
     def tearDown(self):
-        super(BaseTestCase, self).tearDown()
-        self._flush_cs('echo')
+        super(CommonTestCase, self).tearDown()
 
     @classmethod
     def tearDownClass(cls):
-        pass
+        super(CommonTestCase, cls).tearDownClass()
 
     def _flush_cs(self, srvtype):
         params = {'type': srvtype}
-        resp = self.session.post(self._url_cs("deregister"), params=params)
-        self.assertEqual(resp.status_code / 100, 2)
-        resp = self.session.post(self._url_cs("flush"), params=params)
-        self.assertEqual(resp.status_code / 100, 2)
+        resp = self.request('POST', self._url_cs("flush"),
+                            params=params)
+        self.assertEqual(resp.status / 100, 2)
 
     def _register_srv(self, srv):
-        resp = self.session.post(self._url_cs("register"), json.dumps(srv))
-        self.assertEqual(resp.status_code, 200)
+        resp = self.request('POST', self._url_cs("register"),
+                            jsonlib.dumps(srv))
+        self.assertIn(resp.status, (200, 204))
 
     def _lock_srv(self, srv):
-        resp = self.session.post(self._url_cs("lock"), json.dumps(srv))
-        self.assertEqual(resp.status_code, 200)
+        resp = self.request('POST', self._url_cs("lock"),
+                            jsonlib.dumps(srv))
+        self.assertIn(resp.status, (200, 204))
 
     def _unlock_srv(self, srv):
-        resp = self.session.post(self._url_cs("unlock"), json.dumps(srv))
-        self.assertEqual(resp.status_code, 200)
+        resp = self.request('POST', self._url_cs("unlock"),
+                            jsonlib.dumps(srv))
+        self.assertIn(resp.status, (200, 204))
 
     def _flush_proxy(self):
         url = self.uri + '/v3.0/cache/flush/local'
-        resp = self.session.post(url, '')
-        self.assertEqual(resp.status_code / 100, 2)
+        resp = self.request('POST', url, '')
+        self.assertEqual(resp.status / 100, 2)
+
+    @classmethod
+    def _cls_reload_proxy(cls):
+        url = '{0}/v3.0/{1}/lb/reload'.format(cls._cls_uri, cls._cls_ns)
+        cls.static_request('POST', url, '')
 
     def _reload_proxy(self):
         url = '{0}/v3.0/{1}/lb/reload'.format(self.uri, self.ns)
-        resp = self.session.post(url, '')
-        self.assertEqual(resp.status_code / 100, 2)
+        resp = self.request('POST', url, '')
+        self.assertEqual(resp.status / 100, 2)
 
     def _flush_meta(self):
         for srvtype in ('meta1', 'meta2'):
             for t in self.conf['services'][srvtype]:
                 url = self.uri + '/v3.0/forward/flush'
-                resp = self.session.post(url, params={'id': t['addr']})
-                self.assertEqual(resp.status_code, 204)
+                resp = self.request('POST', url,
+                                    params={'id': t['addr']})
+                self.assertEqual(resp.status, 204)
+
+    @classmethod
+    def _cls_reload_meta(cls):
+        for srvtype in ('meta1', 'meta2'):
+            for t in cls._cls_conf['services'][srvtype]:
+                url = cls._cls_uri + '/v3.0/forward/reload'
+                cls.static_request('POST', url, params={'id': t['addr']})
 
     def _reload_meta(self):
         for srvtype in ('meta1', 'meta2'):
             for t in self.conf['services'][srvtype]:
                 url = self.uri + '/v3.0/forward/reload'
-                resp = self.session.post(url, params={'id': t['addr']})
-                self.assertEqual(resp.status_code, 204)
+                resp = self.request('POST', url,
+                                    params={'id': t['addr']})
+                self.assertEqual(resp.status, 204)
 
     def _reload(self):
         self._flush_proxy()
@@ -186,7 +263,7 @@ class BaseTestCase(testtools.TestCase):
         outd = {'ns': self.ns,
                 'type': str(srvtype),
                 'addr': self._addr(low=lowport, high=highport, ip=ip),
-                'score': random.randint(0, 100),
+                'score': random.randint(1, 100),
                 'tags': {'stat.cpu': 1, 'tag.vol': 'test', 'tag.up': True}}
         if extra_tags:
             outd["tags"].update(extra_tags)
@@ -199,5 +276,28 @@ class BaseTestCase(testtools.TestCase):
         self.assertEqual(body['status'], expected_code_oio)
 
     def assertError(self, resp, code_http, expected_code_oio):
-        self.assertEqual(resp.status_code, code_http)
-        self.assertIsError(resp.json(), expected_code_oio)
+        self.assertEqual(resp.status, code_http)
+        self.assertIsError(self.json_loads(resp.data), expected_code_oio)
+
+    @classmethod
+    def json_loads(cls, data):
+        try:
+            return jsonlib.loads(data)
+        except ValueError:
+            logging.info("Unparseable data: %s", str(data))
+            raise
+
+
+class BaseTestCase(CommonTestCase):
+
+    def setUp(self):
+        super(BaseTestCase, self).setUp()
+        self._flush_cs('echo')
+
+    def tearDown(self):
+        super(BaseTestCase, self).tearDown()
+        self._flush_cs('echo')
+
+    @classmethod
+    def tearDownClass(cls):
+        super(BaseTestCase, cls).tearDownClass()
