@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
-# oio-bootstrap.py, a CLI tool of OpenIO SDS
-# Copyright (C) 2015-2016 OpenIO, original work as part of OpenIO SDS
+# oio-bootstrap.py
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -80,43 +80,52 @@ hz 10
 aof-rewrite-incremental-fsync yes
 """
 
-template_redis_gridinit = """
+template_gridinit_redis = """
 [service.${NS}-${SRVTYPE}-${SRVNUM}]
 group=${NS},localhost,${SRVTYPE},${IP}:${PORT}
-on_die=respawn
+on_die=cry
 enabled=true
 start_at_boot=false
 command=redis-server ${CFGDIR}/${NS}-${SRVTYPE}-${SRVNUM}.conf
 """
 
-template_account_gridinit = """
+template_gridinit_beanstalkd = """
 [service.${NS}-${SRVTYPE}-${SRVNUM}]
 group=${NS},localhost,${SRVTYPE},${IP}:${PORT}
 on_die=respawn
 enabled=true
+start_at_boot=true
+command=beanstalkd -l ${IP} -p ${PORT} -b ${DATADIR}/${NS}-${SRVTYPE}-${SRVNUM} -f 1000 -s 10240000
+env.PYTHONPATH=${CODEDIR}/@LD_LIBDIR@/python2.7/site-packages
+"""
+
+template_gridinit_account = """
+[service.${NS}-${SRVTYPE}-${SRVNUM}]
+group=${NS},localhost,${SRVTYPE},${IP}:${PORT}
+on_die=cry
+enabled=true
 start_at_boot=false
 command=${EXE_PREFIX}-${SRVTYPE}-server ${CFGDIR}/${NS}-${SRVTYPE}-${SRVNUM}.conf
 env.PYTHONPATH=${CODEDIR}/@LD_LIBDIR@/python2.7/site-packages
 """
 
-template_rdir_gridinit = """
+template_gridinit_rdir = """
 [service.${NS}-${SRVTYPE}-${SRVNUM}]
 group=${NS},localhost,${SRVTYPE},${IP}:${PORT}
-on_die=respawn
+on_die=cry
 enabled=true
 start_at_boot=false
 command=${EXE_PREFIX}-${SRVTYPE}-server ${CFGDIR}/${NS}-${SRVTYPE}-${SRVNUM}.conf
-env.PYTHONPATH=${CODEDIR}/@LD_LIBDIR@/python2.7/site-packages
 """
 
-template_proxy_gridinit = """
+template_gridinit_proxy = """
 [service.${NS}-proxy]
 group=${NS},localhost,proxy,${IP}:${PORT}
-on_die=respawn
+on_die=cry
 enabled=true
 start_at_boot=false
-#command=${EXE_PREFIX}-proxy -s OIO,${NS},proxy -O Bind=${RUNDIR}/${NS}-proxy.sock ${IP}:${PORT} ${NS}
-command=${EXE_PREFIX}-proxy -O Cache=off -s OIO,${NS},proxy ${IP}:${PORT} ${NS}
+#command=${EXE} -s OIO,${NS},proxy -O Bind=${RUNDIR}/${NS}-proxy.sock ${IP}:${PORT} ${NS}
+command=${EXE} -s OIO,${NS},proxy ${IP}:${PORT} ${NS}
 """
 
 template_blob_indexer_service = """
@@ -147,7 +156,6 @@ LoadModule dav_rawx_module @APACHE2_MODULES_DIRS@/mod_dav_rawx.so
 <IfModule !mod_logio.c>
   LoadModule logio_module ${APACHE2_MODULES_SYSTEM_DIR}modules/mod_logio.so
 </IfModule>
-
 <IfModule !unixd_module>
   LoadModule unixd_module ${APACHE2_MODULES_SYSTEM_DIR}modules/mod_unixd.so
 </IfModule>
@@ -199,11 +207,12 @@ MaxSpareServers 40
 </IfModule>
 
 <IfModule worker.c>
-StartServers 2
-MaxClients 40
-MinSpareThreads 2
-MaxSpareThreads 40
-ThreadsPerChild 20
+MaxClients 512
+MaxRequestWorkers 512
+ThreadsPerChild 256
+StartServers 1
+MinSpareThreads 8
+MaxSpareThreads 32
 MaxRequestsPerChild 0
 </IfModule>
 
@@ -223,7 +232,7 @@ grid_hash_depth        1
 grid_fsync             disabled
 
 # At the end of an upload, perform a fsync() on the directory holding the chunk
-grid_fsync_dir         enabled
+grid_fsync_dir         disabled
 
 # Preallocate space for the chunk file (enabled by default)
 #grid_fallocate enabled
@@ -258,6 +267,9 @@ LoadModule wsgi_module ${APACHE2_MODULES_SYSTEM_DIR}modules/mod_wsgi.so
 <IfModule !mod_logio.c>
   LoadModule logio_module ${APACHE2_MODULES_SYSTEM_DIR}modules/mod_logio.so
 </IfModule>
+<IfModule !unixd_module>
+  LoadModule unixd_module ${APACHE2_MODULES_SYSTEM_DIR}modules/mod_unixd.so
+</IfModule>
 <IfModule !log_config_module>
   LoadModule log_config_module ${APACHE2_MODULES_SYSTEM_DIR}modules/mod_log_config.so
 </IfModule>
@@ -283,7 +295,8 @@ ErrorLog ${SDSDIR}/logs/${NS}-${SRVTYPE}-${SRVNUM}-errors.log
 CustomLog ${SDSDIR}/logs/${NS}-${SRVTYPE}-${SRVNUM}-access.log log/common env=!nolog
 LogLevel info
 
-WSGIDaemonProcess ${SRVTYPE}-${SRVNUM} processes=2 threads=1 user=${USER} group=${GROUP}
+#WSGIDaemonProcess ${SRVTYPE}-${SRVNUM} processes=8 threads=1 response-buffer-size=8388608 send-buffer-size=8388608 receive-buffer-size=8388608 user=${USER} group=${GROUP}
+WSGIDaemonProcess ${SRVTYPE}-${SRVNUM} processes=8 threads=1 user=${USER} group=${GROUP}
 #WSGIProcessGroup ${SRVTYPE}-${SRVNUM}
 WSGIApplicationGroup ${SRVTYPE}-${SRVNUM}
 WSGIScriptAlias / ${CFGDIR}/${NS}-${SRVTYPE}-${SRVNUM}.wsgi
@@ -302,15 +315,33 @@ from oio.${SRVTYPE}.app import create_app
 application = create_app(conf)
 """
 
+template_wsgi_service_coverage_start = """
+import atexit
+import os
+import coverage
+
+cov = coverage.coverage(data_file='@CMAKE_BINARY_DIR@/.coverage.wsgi',
+                        data_suffix=True, concurrency="thread")
+cov.start()
+"""
+
+template_wsgi_service_coverage_stop = """
+def save_coverage():
+    cov.stop()
+    cov.save()
+
+atexit.register(save_coverage)
+"""
+
 template_meta_watch = """
 host: ${IP}
 port: ${PORT}
 type: ${SRVTYPE}
-location: abcd.hem.oio.vol${SRVNUM}
+location: ${LOC}
 slots:
     - ${SRVTYPE}
 checks:
-    - {type: tcp}
+    - {type: asn1}
 
 stats:
     - {type: volume, path: ${VOLUME}}
@@ -335,7 +366,7 @@ template_rawx_watch = """
 host: ${IP}
 port: ${PORT}
 type: rawx
-location: abcd.hem.oio.vol${SRVNUM}
+location: ${LOC}
 checks:
     - {type: http, uri: /info}
 slots:
@@ -351,7 +382,7 @@ template_rdir_watch = """
 host: ${IP}
 port: ${PORT}
 type: rdir
-location: abcd.hem.oio.vol${SRVNUM}
+location: ${LOC}
 checks:
     - {type: tcp}
 slots:
@@ -366,7 +397,7 @@ template_redis_watch = """
 host: ${IP}
 port: ${PORT}
 type: redis
-location: hem.oio.db${SRVNUM}
+location: localhost.db${SRVNUM}
 checks:
     - {type: tcp}
 slots:
@@ -412,43 +443,17 @@ path=${LIBDIR}/grid/msg_fallback.so
 [Plugin.conscience]
 path=${LIBDIR}/grid/msg_conscience.so
 param_namespace=${NS}
-param_chunk_size=${CHUNK_SIZE}
 
+# Multi-conscience
 param_hub.me=tcp://${IP}:${PORT_HUB}
 param_hub.group=${CS_ALL_HUB}
 
-param_option.events-max-pending=10000
-param_option.meta2.events-max-pending=1000
-param_option.sqlx.events-max-pending=1000
-param_option.meta1.events-max-pending=100
-param_option.meta2.events-buffer-delay=5
-param_option.state=${STATE}
-param_option.worm=${WORM}
-
-param_option.service_update_policy=meta2=KEEP|${M2_REPLICAS}|${M2_DISTANCE};sqlx=KEEP|${SQLX_REPLICAS}|${SQLX_DISTANCE}|;rdir=KEEP|1|1|user_is_a_service=rawx
-
-param_option.meta2_max_versions=${VERSIONING}
-param_option.meta2_keep_deleted_delay=86400
-param_option.compression=on
-param_option.container_max_size=50000000
-param_option.flat_hash_offset=0
-param_option.flat_hash_size=0
-param_option.flat_bitlength=17
-param_option.storage_policy=${STGPOL}
-
 # Storage policies definitions
 param_storage_conf=${CFGDIR}/${NS}-policies.conf
 
 # Service scoring and pools definitions
 param_service_conf=${CFGDIR}/${NS}-service-{pool,type}*.conf
 
-# For an easy transition, it is still possible to define
-# service score expression, variation, timeout and lock here.
-#
-#param_service.meta0.lock_at_first_register=false
-#param_service.meta0.score_timeout=3600
-#param_service.meta0.score_variation_bound=5
-#param_service.meta0.score_expr=((num stat.cpu)>0) * ((num stat.io)>0) * ((num stat.space)>1) * root(3,((num stat.cpu)*(num stat.space)*(num stat.io)))
 """
 
 template_conscience_policies = """
@@ -499,7 +504,7 @@ template_service_pools = """
 # Service pools declarations
 # ----------------------------
 #
-# Pool are automatically created if not defined in configuration,
+# Pools are automatically created if not defined in configuration,
 # according to storage policy or service update policy rules.
 #
 # "targets" is a ';'-separated list.
@@ -512,14 +517,22 @@ template_service_pools = """
 #
 # "nearby_mode" is a boolean telling to find services close to each other.
 #
-#### power user options, don't modify it unless you know what you are doing
-# "mask" is a 64 bits hexadecimal mask used to check service distance.
-# It defaults to FFFFFFFFFFFF0000. It can also be specified as "/48".
+# "min_dist" is the absolute minimum distance between services returned
+# by the pool. It defaults to 1, which is the minimum. If you set it too
+# high, there is a risk the pool fails to find a service set matching
+# all the criteria.
 #
-# "mask_max_shift" is the maximum number of bits to shift the mask
-# to degrade it when distance requirement are not satisfiable.
-# It defaults to 16.
+# "max_dist" is the distance between services that the pool will try to
+# ensure. This option defaults to 4, which is the maximum. If you know
+# that all the services are close together, you can reduce this number
+# to accelerate the research.
 #
+# "warn_dist" is the distance between services at which the pool will emit
+# a warning, for further improvement.
+#
+
+[pool:meta1]
+targets=${M1_REPLICAS},meta1
 
 [pool:meta2]
 targets=${M2_REPLICAS},meta2
@@ -552,13 +565,13 @@ targets=1,rawx-europe,rawx;1,rawx-usa,rawx;1,rawx-asia,rawx
 
 [pool:rawx3nearby]
 targets=3,rawx
-mask=/62
 nearby_mode=true
+warn_dist=2
 
 [pool:rawx3faraway]
 targets=3,rawx
-mask=FFFFFFFF00000000
-mask_max_shift=24
+min_dist=2
+warn_dist=2
 
 """
 
@@ -625,6 +638,7 @@ inherit_env=1
 #env.PATH=${PATH}:${HOME}/.local/bin:${CODEDIR}/bin:/bin:/usr/bin:/usr/local/bin
 env.LD_LIBRARY_PATH=${HOME}/.local/@LD_LIBDIR@:${LIBDIR}
 
+
 limit.core_size=-1
 #limit.max_files=2048
 #limit.stack_size=256
@@ -635,27 +649,19 @@ limit.core_size=-1
 
 template_gridinit_ns = """
 
-[service.${NS}-event-agent]
-group=${NS},localhost,event
-on_die=respawn
-enabled=true
-start_at_boot=false
-command=${EXE_PREFIX}-event-agent ${CFGDIR}/event-agent.conf
-env.PYTHONPATH=${CODEDIR}/@LD_LIBDIR@/python2.7/site-packages
-
 [service.${NS}-conscience-agent]
 group=${NS},localhost,conscience,conscience-agent
-on_die=respawn
+on_die=cry
 enabled=true
 start_at_boot=true
 command=${EXE_PREFIX}-conscience-agent ${CFGDIR}/conscience-agent.yml
 env.PYTHONPATH=${CODEDIR}/@LD_LIBDIR@/python2.7/site-packages
 """
 
-template_conscience_gridinit = """
+template_gridinit_conscience = """
 [service.${NS}-conscience-${SRVNUM}]
 group=${NS},localhost,conscience,${IP}:${PORT}
-on_die=respawn
+on_die=cry
 enabled=true
 start_at_boot=true
 command=${EXE_PREFIX}-daemon -s OIO,${NS},cs,${SRVNUM} ${CFGDIR}/${NS}-conscience-${SRVNUM}.conf
@@ -664,7 +670,7 @@ command=${EXE_PREFIX}-daemon -s OIO,${NS},cs,${SRVNUM} ${CFGDIR}/${NS}-conscienc
 template_gridinit_meta = """
 [service.${NS}-${SRVTYPE}-${SRVNUM}]
 group=${NS},localhost,${SRVTYPE},${IP}:${PORT}
-on_die=respawn
+on_die=cry
 enabled=true
 start_at_boot=false
 command=${EXE} -s OIO,${NS},${SRVTYPE},${SRVNUM} -O Endpoint=${IP}:${PORT} ${EXTRA} ${NS} ${DATADIR}/${NS}-${SRVTYPE}-${SRVNUM}
@@ -673,7 +679,7 @@ command=${EXE} -s OIO,${NS},${SRVTYPE},${SRVNUM} -O Endpoint=${IP}:${PORT} ${EXT
 template_gridinit_sqlx = """
 [service.${NS}-${SRVTYPE}-${SRVNUM}]
 group=${NS},localhost,${SRVTYPE},${IP}:${PORT}
-on_die=respawn
+on_die=cry
 enabled=true
 start_at_boot=false
 command=${EXE} -s OIO,${NS},${SRVTYPE},${SRVNUM} -O DirectorySchemas=${CFGDIR}/sqlx/schemas -O Endpoint=${IP}:${PORT} ${EXTRA} ${NS} ${DATADIR}/${NS}-${SRVTYPE}-${SRVNUM}
@@ -685,16 +691,16 @@ group=${NS},localhost,${SRVTYPE},${IP}:${PORT}
 command=${EXE_PREFIX}-blob-indexer ${CFGDIR}/${NS}-${SRVTYPE}-${SRVNUM}.conf
 enabled=true
 start_at_boot=false
-on_die=respawn
+on_die=cry
 """
 
 template_gridinit_httpd = """
 [Service.${NS}-${SRVTYPE}-${SRVNUM}]
 group=${NS},localhost,${SRVTYPE},${IP}:${PORT}
-command=${HTTPD_BINARY} -D FOREGROUND -f ${CFGDIR}/${NS}-${SRVTYPE}-${SRVNUM}.conf
+command=${HTTPD_BINARY} -D FOREGROUND -f ${CFGDIR}/${NS}-${SRVTYPE}-${SRVNUM}.httpd.conf
 enabled=true
 start_at_boot=false
-on_die=respawn
+on_die=cry
 """
 
 template_local_header = """
@@ -713,22 +719,26 @@ ${NOZK}zookeeper.sqlx= ${ZK_CNXSTRING}
 
 #proxy-local=${RUNDIR}/${NS}-proxy.sock
 proxy=${IP}:${PORT_PROXYD}
-ecd=${IP}:${PORT_ECD}
-event-agent=beanstalk://127.0.0.1:11300
-#event-agent=ipc://${RUNDIR}/event-agent.sock
 conscience=${CS_ALL_PUB}
-
-log_outgoing=yes
-avoid_faulty_services=${AVOID_FAULTY}
-
-# Allow the sqliterepo election mechanism to use UDP requests
-udp_allowed=${UDP_ALLOWED}
+ecd=${IP}:${PORT_ECD}
+${NOBS}event-agent=${BEANSTALKD_CNXSTRING}
 
 meta1_digits=${M1_DIGITS}
-zk_shuffled=${ZK_SHUFFLED}
 
-# Allow rawx to send chunk events (enabled by default)
-#rawx_events=yes
+admin=${IP}:${PORT_ADMIN}
+
+"""
+
+template_gridinit_event_agent = """
+
+[service.${NS}-${SRVTYPE}-${SRVNUM}]
+group=${NS},localhost,event
+on_die=respawn
+enabled=true
+start_at_boot=false
+command=${EXE_PREFIX}-event-agent ${CFGDIR}/${NS}-${SRVTYPE}-${SRVNUM}.conf
+env.PYTHONPATH=${CODEDIR}/@LD_LIBDIR@/python2.7/site-packages
+
 """
 
 template_event_agent = """
@@ -738,11 +748,12 @@ namespace = ${NS}
 user = ${USER}
 workers = 2
 concurrency = 5
-handlers_conf = ${CFGDIR}/event-handlers.conf
+handlers_conf = ${CFGDIR}/event-handlers-${SRVNUM}.conf
 log_facility = LOG_LOCAL0
 log_level = INFO
 log_address = /dev/log
-syslog_prefix = OIO,${NS},event-agent
+syslog_prefix = OIO,${NS},${SRVTYPE},${SRVNUM}
+queue_url=${QUEUE_URL}
 """
 
 template_event_agent_handlers = """
@@ -754,10 +765,17 @@ pipeline = noop
 # pipeline = replication
 pipeline = noop
 
+[handler:storage.content.broken]
+pipeline = content_rebuild
+
 [handler:storage.content.deleted]
 # pipeline = content_cleaner replication
 pipeline = content_cleaner
 
+[handler:storage.content.drained]
+# pipeline = content_cleaner replication
+pipeline = content_cleaner
+
 [handler:storage.container.new]
 pipeline = account_update
 
@@ -774,12 +792,17 @@ pipeline = volume_index
 pipeline = volume_index
 
 [handler:account.services]
-pipeline = noop
+pipeline = account_update
 
 [filter:content_cleaner]
 use = egg:oio#content_cleaner
 key_file = ${KEY_FILE}
 
+[filter:content_rebuild]
+use = egg:oio#notify
+tube = oio-rebuild
+queue_url = ${QUEUE_URL}
+
 [filter:account_update]
 use = egg:oio#account_update
 
@@ -789,7 +812,7 @@ use = egg:oio#volume_index
 [filter:replication]
 use = egg:oio#notify
 tube = oio-repli
-queue_url = beanstalk://127.0.0.1:11300
+queue_url = ${QUEUE_URL}
 
 [filter:noop]
 use = egg:oio#noop
@@ -845,6 +868,18 @@ log_address = /dev/log
 syslog_prefix = OIO,${NS},rdir,${SRVNUM}
 """
 
+template_admin = """
+[admin-server]
+bind_addr = ${IP}
+bind_port = ${PORT}
+namespace = ${NS}
+log_facility = LOG_LOCAL0
+log_level = INFO
+log_address = /dev/log
+syslog_prefix = OIO,${NS},admin,${SRVNUM}
+redis_host = ${IP}
+"""
+
 sqlx_schema_dovecot = """
 CREATE TABLE IF NOT EXISTS box (
    name TEXT NOT NULL PRIMARY KEY,
@@ -925,17 +960,16 @@ CFGDIR = SDSDIR + '/conf'
 RUNDIR = SDSDIR + '/run'
 LOGDIR = SDSDIR + '/logs'
 SPOOLDIR = SDSDIR + '/spool'
-DATADIR = SDSDIR + '/data'
 WATCHDIR = SDSDIR + '/conf/watch'
 TMPDIR = '/tmp'
 CODEDIR = '@CMAKE_INSTALL_PREFIX@'
 LIBDIR = CODEDIR + '/@LD_LIBDIR@'
 PATH = HOME+"/.local/bin:@CMAKE_INSTALL_PREFIX@/bin:/usr/sbin"
-port = 6000
 
 # Constants for the configuration of oio-bootstrap
 NS = 'ns'
 IP = 'ip'
+SVC_HOSTS = 'hosts'
 SVC_NB = 'count'
 SVC_PARAMS = 'params'
 ALLOW_REDIS = 'redis'
@@ -950,25 +984,17 @@ M2_STGPOL = 'storage_policy'
 SQLX_REPLICAS = 'sqlx_replicas'
 PROFILE = 'profile'
 PORT_START = 'port_start'
-CHUNK_SIZE = 'chunk_size'
 ACCOUNT_ID = 'account_id'
 BUCKET_NAME = 'bucket_name'
 COMPRESSION = 'compression'
 APPLICATION_KEY = 'application_key'
 KEY_FILE='key_file'
 META_HEADER='x-oio-chunk-meta'
-WORMED="worm"
-NS_STATE="state"
-MASTER_VALUE="master"
-SLAVE_VALUE="slave"
-STANDALONE_VALUE="standalone"
-AVOID_FAULTY="avoid_faulty_services"
-UDP_ALLOWED="udp_allowed"
-ZK_SHUFFLED="zk_shuffled"
+COVERAGE=os.getenv('PYTHON_COVERAGE')
 
 defaults = {
     'NS': 'OPENIO',
-    'IP': '127.0.0.1',
+    SVC_HOSTS: ('127.0.0.1',),
     'ZK': '127.0.0.1:2181',
     'NB_CS': 1,
     'NB_M0': 1,
@@ -982,9 +1008,8 @@ defaults = {
     'REPLI_M2': 1,
     'REPLI_M1': 1,
     'COMPRESSION': "off",
-    M1_DIGITS: 4,
-    AVOID_FAULTY: "off",
-    UDP_ALLOWED: "off"}
+    MONITOR_PERIOD: 1,
+    M1_DIGITS: 4}
 
 # XXX When /usr/sbin/httpd is present we suspect a Redhat/Centos/Fedora
 # environment. If not, we consider being in a Ubuntu/Debian environment.
@@ -1001,6 +1026,8 @@ if not os.path.exists('/usr/sbin/httpd'):
 def config(env):
     return '{CFGDIR}/{NS}-{SRVTYPE}-{SRVNUM}.conf'.format(**env)
 
+def httpd_config(env):
+    return '{CFGDIR}/{NS}-{SRVTYPE}-{SRVNUM}.httpd.conf'.format(**env)
 
 def watch(env):
     return '{WATCHDIR}/{NS}-{SRVTYPE}-{SRVNUM}.yml'.format(**env)
@@ -1026,26 +1053,27 @@ def type2exe(t):
     return EXE_PREFIX + '-' + str(t) + '-server'
 
 
-def next_port():
-    global port
-    res, port = port, port + 1
-    return res
-
-
 def generate(options):
-    def getint(v, default):
-        if v is None:
-            return int(default)
-        return int(v)
+    global first_port
 
-    global port
-    port = getint(options.get('port-start'), 6000)
+    def ensure(v, default):
+        if v is None:
+            return default
+        return v
+
+    def getint(v, default):
+        try:
+            return int(ensure(v, default))
+        except:
+            return default
 
     final_conf = {}
     final_services = {}
 
-    port_proxy = next_port()
-    port_ecd = next_port()
+    ports = (x for x in xrange(options['port'],60000))
+    port_proxy = next(ports)
+    port_ecd = next(ports)
+    port_admin = next(ports)
 
     versioning = 1
     stgpol = "SINGLE"
@@ -1059,24 +1087,20 @@ def generate(options):
         versioning = options[M2_VERSIONS]
     if M2_STGPOL in options:
         stgpol = options[M2_STGPOL]
+    options['config']['ns.storage_policy'] = stgpol
+
+    # `options` already holds the YAML values overriden by the CLI values
+    hosts = options.get(SVC_HOSTS) or defaults[SVC_HOSTS]
 
     ns = options.get('ns') or defaults['NS']
-    ip = options.get('ip') or defaults['IP']
     backblaze_account_id = options.get('backblaze', {}).get(ACCOUNT_ID)
     backblaze_bucket_name = options.get('backblaze', {}).get(BUCKET_NAME)
     backblaze_app_key = options.get('backblaze', {}).get(APPLICATION_KEY)
-    is_wormed = options.get('worm', False)
-    worm = '1' if is_wormed else '0'
-    state = options.get("state", None)
-    avoid_faulty_services = str(options.get(AVOID_FAULTY, "off")).lower()
-    udp_allowed = str(options.get(UDP_ALLOWED, "off")).lower()
-    zk_shuffled = str(options.get(ZK_SHUFFLED, "off")).lower()
 
-    if state not in [MASTER_VALUE, SLAVE_VALUE, STANDALONE_VALUE]:
-        state = STANDALONE_VALUE
+    DATADIR = options.get('DATADIR', SDSDIR + '/data')
+
     key_file = options.get(KEY_FILE, CFGDIR + '/' + 'application_keys.cfg')
-    ENV = dict(IP=ip,
-               ZK_CNXSTRING=options.get('ZK'),
+    ENV = dict(ZK_CNXSTRING=options.get('ZK'),
                NS=ns,
                HOME=HOME,
                EXE_PREFIX=EXE_PREFIX,
@@ -1097,10 +1121,11 @@ def generate(options):
                USER=str(pwd.getpwuid(os.getuid()).pw_name),
                GROUP=str(grp.getgrgid(os.getgid()).gr_name),
                VERSIONING=versioning,
-               STGPOL=stgpol,
                PORT_PROXYD=port_proxy,
                PORT_ECD=port_ecd,
+               PORT_ADMIN=port_admin,
                M1_DIGITS=meta1_digits,
+               M1_REPLICAS=meta1_replicas,
                M2_REPLICAS=meta2_replicas,
                M2_DISTANCE=str(1),
                SQLX_REPLICAS=sqlx_replicas,
@@ -1111,12 +1136,7 @@ def generate(options):
                BACKBLAZE_APPLICATION_KEY=backblaze_app_key,
                KEY_FILE=key_file,
                HTTPD_BINARY=HTTPD_BINARY,
-               META_HEADER=META_HEADER,
-               STATE=state,
-               WORM=worm,
-               AVOID_FAULTY=avoid_faulty_services,
-               UDP_ALLOWED=udp_allowed,
-               ZK_SHUFFLED=zk_shuffled)
+               META_HEADER=META_HEADER)
 
     def merge_env(add):
         env = dict(ENV)
@@ -1130,6 +1150,14 @@ def generate(options):
             env['env.ORIG_EXE'] = orig_exe
             env['EXE'] = new_exe
             env['env.G_DEBUG'] = "gc-friendly"
+        elif options.get(PROFILE) == "callgrind":
+            orig_exe = env.get('EXE', env['EXE_PREFIX'])
+            new_exe = "valgrind --tool=callgrind --collect-jumps=yes\
+ --collect-systime=yes --trace-children=yes\
+ --callgrind-out-file=/tmp/callgrind.out.%q{ORIG_EXE}.%p " + orig_exe
+            env['env.ORIG_EXE'] = orig_exe
+            env['EXE'] = new_exe
+            del env['env.G_SLICE']
         return env
 
     def subenv(add):
@@ -1137,8 +1165,8 @@ def generate(options):
         env['VOLUME'] = '{DATADIR}/{NS}-{SRVTYPE}-{SRVNUM}'.format(**env)
         return env
 
-    ENV['CHUNK_SIZE'] = getint(options.get(CHUNK_SIZE), 1024*1024)
-    ENV['MONITOR_PERIOD'] = getint(options.get(MONITOR_PERIOD), 1)
+    ENV['MONITOR_PERIOD'] = getint(
+            options.get(MONITOR_PERIOD), defaults[MONITOR_PERIOD])
     if options.get(ZOOKEEPER):
         ENV['NOZK'] = ''
     else:
@@ -1157,7 +1185,15 @@ def generate(options):
         if t not in final_services:
             final_services[t] = []
 
-        out = {'num': env['SRVNUM']}
+        num = int(env['SRVNUM'])
+        out = {'num': str(num)}
+        if 'IP' not in env:
+            _h = tuple(hosts)
+            if t in options and isinstance(options[t], dict):
+                _h = ensure(options[t].get(SVC_HOSTS), hosts)
+            env['IP'] = _h[(num-1) % len(_h)]
+        if 'LOC' not in env:
+            env['LOC'] = "srv%s.vol%d" % (env['IP'].rsplit('.', 1)[-1], num)
         if 'PORT' in env:
             out['addr'] = '%s:%s' % (env['IP'], env['PORT'])
         if 'VOLUME' in env:
@@ -1185,28 +1221,58 @@ def generate(options):
             f.write(tpl.safe_substitute(ENV))
         # Prepare a list of consciences
         for num in range(nb_conscience):
-            cs.append((num + 1, next_port(), next_port()))
+            h = hosts[num % len(hosts)]
+            cs.append((num + 1, h, next(ports), next(ports)))
         ENV.update({
             'CS_ALL_PUB': ','.join(
-                [str(ip)+':'+str(pub) for _, pub, _ in cs]),
+                [str(host)+':'+str(port) for _, host, port, _ in cs]),
             'CS_ALL_HUB': ','.join(
-                ['tcp://'+str(ip)+':'+str(hub) for _, _, hub in cs]),
+                ['tcp://'+str(host)+':'+str(hub) for _, host, _, hub in cs]),
         })
-
-        for num, pub, hub in cs:
+        # generate the conscience files
+        for num, host, port, hub in cs:
             env = subenv({'SRVTYPE': 'conscience', 'SRVNUM': num,
-                          'PORT': pub, 'PORT_HUB': hub})
+                          'PORT': port, 'PORT_HUB': hub})
             add_service(env)
             with open(gridinit(env), 'a+') as f:
-                tpl = Template(template_conscience_gridinit)
+                tpl = Template(template_gridinit_conscience)
                 f.write(tpl.safe_substitute(env))
             with open(config(env), 'w+') as f:
                 tpl = Template(template_conscience_service)
                 f.write(tpl.safe_substitute(env))
 
+    # beanstalkd
+    all_beanstalkd = list()
+    nb_beanstalkd = getint(options['beanstalkd'].get(SVC_NB), 1)
+    if nb_beanstalkd:
+        # prepare a list of all the beanstalkd
+        for num in range(nb_beanstalkd):
+            h = hosts[num % len(hosts)]
+            all_beanstalkd.append((num + 1, h, next(ports)))
+        # generate the files
+        for num, host, port in all_beanstalkd:
+            env = subenv({'SRVTYPE': 'beanstalkd', 'SRVNUM': num,
+                          'IP': host, 'PORT': port,
+                          'EXE': 'beanstalkd'})
+            add_service(env)
+            # gridinit config
+            tpl = Template(template_gridinit_beanstalkd)
+            with open(gridinit(env), 'a+') as f:
+                f.write(tpl.safe_substitute(env))
+                for key in (k for k in env.iterkeys() if k.startswith("env.")):
+                    f.write("%s=%s\n" % (key, env[key]))
+
+        beanstalkd_cnxstring = ';'.join(
+                "beanstalk://" + str(h) + ":" + str(p)
+                for _, h, p in all_beanstalkd)
+        ENV.update({'BEANSTALKD_CNXSTRING': beanstalkd_cnxstring,
+                    'NOBS':''})
+    else:
+        ENV.update({'BEANSTALKD_CNXSTRING': '***disabled***', 'NOBS':'#'})
+
     # meta* + sqlx
     def generate_meta(t, n, tpl, ext_opt=""):
-        env = subenv({'SRVTYPE': t, 'SRVNUM': n, 'PORT': next_port(),
+        env = subenv({'SRVTYPE': t, 'SRVNUM': n, 'PORT': next(ports),
                       'EXE': ENV['EXE_PREFIX'] + '-' + t + '-server',
                       'EXTRA': ext_opt})
         add_service(env)
@@ -1223,7 +1289,7 @@ def generate(options):
 
     # meta0
     nb_meta0 = max(getint(options['meta0'].get(SVC_NB), defaults['NB_M0']),
-                   defaults['NB_M0'])
+                   meta1_replicas)
     if nb_meta0:
         for i in range(nb_meta0):
             generate_meta('meta0', i + 1, template_gridinit_meta,
@@ -1253,15 +1319,16 @@ def generate(options):
                           options['sqlx'].get(SVC_PARAMS, ""))
 
     # RAWX
-    nb_rawx = getint(options['rawx'].get(SVC_NB), defaults['NB_RAWX'])
-    compression = options['rawx'].get(COMPRESSION, "off")
+    srvtype = 'rawx'
+    nb_rawx = getint(options[srvtype].get(SVC_NB), defaults['NB_RAWX'])
+    compression = options[srvtype].get(COMPRESSION, "off")
     if nb_rawx:
-        for num in range(nb_rawx):
-            env = subenv({'SRVTYPE': 'rawx',
-                          'SRVNUM': num + 1,
-                          'PORT': next_port(),
+        for i in range(nb_rawx):
+            env = subenv({'SRVTYPE': srvtype,
+                          'SRVNUM': i + 1,
+                          'PORT': next(ports),
                           'COMPRESSION': compression,
-                          'EXTRASLOT': ('rawx-even' if num % 2 else 'rawx-odd')
+                          'EXTRASLOT': ('rawx-even' if i % 2 else 'rawx-odd')
                           })
             add_service(env)
             # gridinit (rawx)
@@ -1273,7 +1340,7 @@ def generate(options):
             to_write = tpl.safe_substitute(env)
             if options.get(OPENSUSE, None):
                 to_write = re.sub(r"LoadModule.*mpm_worker.*", "", to_write)
-            with open(config(env), 'w+') as f:
+            with open(httpd_config(env), 'w+') as f:
                 f.write(to_write)
             # watcher
             tpl = Template(template_rawx_watch)
@@ -1294,11 +1361,12 @@ def generate(options):
                 f.write(tpl.safe_substitute(env))
 
     # redis
-    env = subenv({'SRVTYPE': 'redis', 'SRVNUM': 1, 'PORT': 6379})
+    srvtype = 'redis'
+    env = subenv({'SRVTYPE': srvtype, 'SRVNUM': 1, 'PORT': 6379})
     add_service(env)
     if options.get(ALLOW_REDIS):
         with open(gridinit(env), 'a+') as f:
-            tpl = Template(template_redis_gridinit)
+            tpl = Template(template_gridinit_redis)
             f.write(tpl.safe_substitute(env))
         with open(config(env), 'w+') as f:
             tpl = Template(template_redis)
@@ -1308,11 +1376,14 @@ def generate(options):
             f.write(tpl.safe_substitute(env))
 
     # proxy
-    env = subenv({'SRVTYPE': 'proxy', 'SRVNUM': 1, 'PORT': port_proxy})
+    env = subenv({'SRVTYPE': 'proxy', 'SRVNUM': 1, 'PORT': port_proxy,
+                  'EXE': ENV['EXE_PREFIX'] + '-' + 'proxy'})
     add_service(env)
     with open(gridinit(env), 'a+') as f:
-        tpl = Template(template_proxy_gridinit)
+        tpl = Template(template_gridinit_proxy)
         f.write(tpl.safe_substitute(env))
+        for key in (k for k in env.iterkeys() if k.startswith("env.")):
+            f.write("%s=%s\n" % (key, env[key]))
 
     # ecd
     env = subenv({'SRVTYPE': 'ecd', 'SRVNUM': 1, 'PORT': port_ecd})
@@ -1325,7 +1396,7 @@ def generate(options):
     to_write = tpl.safe_substitute(env)
     if options.get(OPENSUSE, False):
         to_write = re.sub(r"LoadModule.*mpm_worker.*", "", to_write)
-    with open(config(env), 'w+') as f:
+    with open(httpd_config(env), 'w+') as f:
         f.write(to_write)
     # service desc
     tpl = Template(template_wsgi_service_descr)
@@ -1333,11 +1404,36 @@ def generate(options):
     with open(wsgi(env), 'w+') as f:
         f.write(to_write)
 
+    # container
+    env = subenv({'SRVTYPE': 'container', 'SRVNUM': 1, 'PORT': port_admin})
+    add_service(env)
+    tpl = Template(template_gridinit_httpd)
+    with open(gridinit(env), 'a+') as f:
+        f.write(tpl.safe_substitute(env))
+    # service
+    tpl = Template(template_wsgi_service_host)
+    to_write = tpl.safe_substitute(env)
+    if options.get(OPENSUSE, False):
+        to_write = re.sub(r"LoadModule.*mpm_worker.*", "", to_write)
+    with open(httpd_config(env), 'w+') as f:
+        f.write(to_write)
+    # service desc
+    tpl = Template(template_wsgi_service_descr)
+    to_write = tpl.safe_substitute(env, SRVTYPE='container',
+                                        KEY_FILE=config(env))
+    with open(wsgi(env), 'w+') as f:
+        f.write(to_write)
+    # service configuration
+    tpl = Template(template_admin)
+    to_write = tpl.safe_substitute(env)
+    with open(config(env), 'w+') as f:
+        f.write(to_write)
+
     # account
-    env = subenv({'SRVTYPE': 'account', 'SRVNUM': 1, 'PORT': next_port()})
+    env = subenv({'SRVTYPE': 'account', 'SRVNUM': 1, 'PORT': next(ports)})
     add_service(env)
     with open(gridinit(env), 'a+') as f:
-        tpl = Template(template_account_gridinit)
+        tpl = Template(template_gridinit_account)
         f.write(tpl.safe_substitute(env))
     with open(config(env), 'w+') as f:
         tpl = Template(template_account)
@@ -1351,10 +1447,10 @@ def generate(options):
     for num in range(nb_rdir):
         env = subenv({'SRVTYPE': 'rdir',
                       'SRVNUM': num + 1,
-                      'PORT': next_port()})
+                      'PORT': next(ports)})
         add_service(env)
         with open(gridinit(env), 'a+') as f:
-            tpl = Template(template_rdir_gridinit)
+            tpl = Template(template_gridinit_rdir)
             f.write(tpl.safe_substitute(env))
         with open(config(env), 'w+') as f:
             tpl = Template(template_rdir)
@@ -1363,15 +1459,21 @@ def generate(options):
             tpl = Template(template_rdir_watch)
             f.write(tpl.safe_substitute(env))
 
-    # Event agent configuration
-    env = subenv({'SRVTYPE': 'event-agent', 'SRVNUM': 1})
-    add_service(env)
-    with open(CFGDIR + '/' + 'event-agent.conf', 'w+') as f:
-        tpl = Template(template_event_agent)
-        f.write(tpl.safe_substitute(env))
-    with open(CFGDIR + '/' + 'event-handlers.conf', 'w+') as f:
-        tpl = Template(template_event_agent_handlers)
-        f.write(tpl.safe_substitute(env))
+    # Event agent configuration -> one per beanstalkd
+    for num, host, port in all_beanstalkd:
+        bnurl = 'beanstalk://{0}:{1}'.format(host, port)
+        env = subenv({'SRVTYPE': 'event-agent', 'SRVNUM': num,
+                      'QUEUE_URL': bnurl})
+        add_service(env)
+        with open(gridinit(env), 'a+') as f:
+            tpl = Template(template_gridinit_event_agent)
+            f.write(tpl.safe_substitute(env))
+        with open(config(env), 'w+') as f:
+            tpl = Template(template_event_agent)
+            f.write(tpl.safe_substitute(env))
+        with open(CFGDIR + '/' + 'event-handlers-'+str(num)+'.conf', 'w+') as f:
+            tpl = Template(template_event_agent_handlers)
+            f.write(tpl.safe_substitute(env))
 
     # Conscience agent configuration
     env = subenv({'SRVTYPE': 'conscience-agent', 'SRVNUM': 1})
@@ -1392,16 +1494,23 @@ def generate(options):
         f.write(tpl.safe_substitute(ENV))
     # system config
     with open('{OIODIR}/sds.conf'.format(**ENV), 'w+') as f:
+        env = merge_env({'IP':hosts[0]})
         tpl = Template(template_local_header)
-        f.write(tpl.safe_substitute(ENV))
+        f.write(tpl.safe_substitute(env))
         tpl = Template(template_local_ns)
-        f.write(tpl.safe_substitute(ENV))
+        f.write(tpl.safe_substitute(env))
+        # Now dump the configuration
+        for k, v in options['config'].iteritems():
+            strv = str(v)
+            if isinstance(v,bool):
+                strv = strv.lower()
+            f.write('{0}={1}\n'.format(k, strv))
 
     with open('{KEY_FILE}'.format(**ENV), 'w+') as f:
         tpl = Template(template_credentials)
         f.write(tpl.safe_substitute(ENV))
 
-    # ensure volumes
+    # ensure volumes for srvtype in final_services:
     for srvtype in final_services:
         for rec in final_services[srvtype]:
             if 'path' in rec:
@@ -1409,17 +1518,23 @@ def generate(options):
 
     final_conf["services"] = final_services
     final_conf["namespace"] = ns
-    final_conf["chunk_size"] = ENV['CHUNK_SIZE']
     final_conf["storage_policy"] = stgpol
     final_conf["account"] = 'test_account'
     final_conf["sds_path"] = SDSDIR
-    final_conf[AVOID_FAULTY] = avoid_faulty_services
-    final_conf[UDP_ALLOWED] = udp_allowed
-    final_conf[ZK_SHUFFLED] = zk_shuffled
+    # TODO(jfs): remove this line only required by some tests cases
+    final_conf["chunk_size"] = options['config']['ns.chunk_size']
     final_conf["proxy"] = final_services['proxy'][0]['addr']
     final_conf[M2_REPLICAS] = meta2_replicas
     final_conf[M1_REPLICAS] = meta1_replicas
     final_conf[M1_DIGITS] = meta1_digits
+    for k in (APPLICATION_KEY, COMPRESSION, BUCKET_NAME,
+              ACCOUNT_ID, PORT_START, PROFILE,
+              MONITOR_PERIOD):
+        if k in ENV:
+            final_conf[k] = ENV[k]
+        elif k in defaults:
+            final_conf[k] = defaults[k]
+    final_conf['config'] = options['config']
     with open('{CFGDIR}/test.yml'.format(**ENV), 'w+') as f:
         f.write(yaml.dump(final_conf))
     return final_conf
@@ -1432,37 +1547,84 @@ def dump_config(conf):
     print 'M1_DIGITS=%s' % conf[M1_DIGITS]
 
 
-def main():
-    parser = argparse.ArgumentParser(description='OpenIO bootstrap tool')
+def merge_config(base, inc):
+    for k, v in inc.iteritems():
+        if isinstance(v, dict):
+            if k not in base:
+                base[k] = v
+            elif isinstance(base[k], dict):
+                base[k] = merge_config(base[k], v)
+            else:
+                raise Exception("What the fuck!? You fucking basterd!")
+        else:
+            base[k] = v
+    return base
 
-    parser.add_argument("namespace", help="Namespace name")
-    parser.add_argument("ip", help="IP to use")
-    parser.add_argument("-c", "--conf", action="append", dest='config',
+
+def main():
+    if COVERAGE:
+        global template_wsgi_service_descr
+        template_wsgi_service_descr = "".join([template_wsgi_service_coverage_start,
+                                               template_wsgi_service_descr,
+                                               template_wsgi_service_coverage_stop])
+    parser = argparse.ArgumentParser(description='OpenIO bootstrap tool')
+    parser.add_argument("-c", "--conf",
+                        action="append", dest='config',
                         help="Bootstrap configuration file")
-    parser.add_argument("-d", "--dump", action="store_true", default=False,
+    parser.add_argument("-d", "--dump",
+                        action="store_true", default=False,
                         dest='dump_config', help="Dump results")
+    parser.add_argument("-p", "--port",
+                        type=int, default=6000,
+                        help="Specify the first port of the range")
+    parser.add_argument("-D", "--data",
+                        action="store", type=str, default=None,
+                        help="Specify a DATA directory")
+    parser.add_argument("--profile", choices=['default', 'valgrind', 'callgrind'],
+                        help="Launch SDS with specific tool")
+    parser.add_argument("namespace",
+                        action='store', type=str, default=None,
+                        help="Namespace name")
+    parser.add_argument("ip",
+                        metavar='<ip>', nargs='*',
+                        help="set of IP to use (repeatable option)")
 
     opts = {}
+    opts['config'] = dict()
+    opts['config']['proxy.cache.enabled'] = False
+    opts['config']['ns.chunk_size'] = 1024 * 1024
     opts[ZOOKEEPER] = False
-    opts['conscience'] = {SVC_NB: None}
-    opts['meta0'] = {SVC_NB: None}
-    opts['meta1'] = {SVC_NB: None}
-    opts['meta2'] = {SVC_NB: None}
-    opts['sqlx'] = {SVC_NB: None}
-    opts['rawx'] = {SVC_NB: None}
-    opts['rdir'] = {SVC_NB: None}
+    opts['conscience'] = {SVC_NB: None, SVC_HOSTS: None}
+    opts['meta0'] = {SVC_NB: None, SVC_HOSTS: None}
+    opts['meta1'] = {SVC_NB: None, SVC_HOSTS: None}
+    opts['meta2'] = {SVC_NB: None, SVC_HOSTS: None}
+    opts['sqlx'] = {SVC_NB: None, SVC_HOSTS: None}
+    opts['rawx'] = {SVC_NB: None, SVC_HOSTS: None}
+    opts['rdir'] = {SVC_NB: None, SVC_HOSTS: None}
+    opts['beanstalkd'] = {SVC_NB: None, SVC_HOSTS: None}
 
     options = parser.parse_args()
+
+    if options.data:
+        opts['DATADIR'] = options.data
+
     if options.config:
         for path in options.config:
             with open(path, 'r') as f:
                 data = yaml.load(f)
                 if data:
-                    opts.update(data)
+                    opts = merge_config(opts, data)
+
+    opts['port'] = int(options.port)
+
+    # Remove empty strings, then apply the default if no value remains
+    options.ip = [str(x) for x in options.ip if x]
+    if len(options.ip) > 0:
+        opts[SVC_HOSTS] = tuple(options.ip)
 
     opts['ZK'] = os.environ.get('ZK', defaults['ZK'])
     opts['ns'] = options.namespace
-    opts['ip'] = options.ip
+    opts[PROFILE] = options.profile
     final_conf = generate(opts)
     if options.dump_config:
         dump_config(final_conf)
