@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 
-# oio-reset.sh, a CLI tool of OpenIO SDS
-# Copyright (C) 2015-2016 OpenIO, original work as part of OpenIO SDS
+# oio-reset.sh
+# Copyright (C) 2015-2017 OpenIO SAS, original work as part of OpenIO SDS
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -15,24 +15,28 @@
 #
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
-set -e
 
+set -e
 
 PREFIX="@EXE_PREFIX@"
 NS=OPENIO
-IP=127.0.0.1
+IP=
+PORT=
 OIO=$HOME/.oio
 SDS=$OIO/sds
 GRIDINIT_SOCK=${SDS}/run/gridinit.sock
 BOOTSTRAP_CONFIG=
+PROFILE=
+DATADIR=
 
 ZKSLOW=0
-ZKRESET=0
 verbose=0
 OPENSUSE=`grep -i opensuse /etc/*release || echo -n ''`
 
-while getopts "I:N:f:Z:Cvb" opt; do
+while getopts "D:P:I:N:f:Z:p:Cvb" opt; do
     case $opt in
+        D) DATADIR="${OPTARG}" ;;
+        P) PORT="${OPTARG}" ;;
         I) IP="${OPTARG}" ;;
         N) NS="${OPTARG}" ;;
         f) if [ -n "$OPTARG" ]; then
@@ -43,7 +47,7 @@ while getopts "I:N:f:Z:Cvb" opt; do
 			fi
 		fi ;;
         Z) ZKSLOW=1 ;;
-        C) ZKRESET=1 ;;
+        p) PROFILE="${OPTARG}" ;;
         v) ((verbose=verbose+1)) ;;
         \?) exit 1 ;;
     esac
@@ -66,27 +70,20 @@ timeout () {
     ((count=count+1))
 }
 
-wait_for_srvtype () {
-    echo "Waiting for the $2 $1 to get a score"
-    $PREFIX-wait-scored.sh -u -N "$2" -n "$NS" -s "$1" -t 15
-}
-
-timestamp () {
-    echo
-    date '+%Y-%m-%d %H:%M:%S.%N'
-}
 
 #-------------------------------------------------------------------------------
 
+cmd_openio="openio --oio-ns $NS"
 G_DEBUG_LEVEL=WARN
 if [ $verbose != 0 ] ; then
     G_DEBUG_LEVEL=TRACE
     echo "# $0" \
         "-I \"${IP}\"" \
+        "-P \"${PORT}\"" \
         "-N \"${NS}\"" \
         "-Z \"${ZKSLOW}\"" \
-        "-C \"${ZKRESET}\"" \
         "${BOOTSTRAP_CONFIG}"
+	cmd_openio="$cmd_openio -v --debug"
 fi
 export G_DEBUG_LEVEL
 
@@ -96,7 +93,7 @@ if [ $verbose -ge 1 ] ; then set -x ; fi
 pgrep -u "$UID" --full gridinit | while read pidof_gridinit ; do
     # First try a clean stop of gridinit's children
     if [ -e "$GRIDINIT_SOCK" ] ; then
-        if ! gridinit_cmd -S "$GRIDINIT_SOCK" stop ; then
+        if ! gridinit_cmd -S "$GRIDINIT_SOCK" stop >/dev/null ; then
             echo "Failed to send 'stop' to gridinit"
         fi
     fi
@@ -119,11 +116,12 @@ done
 # Generate a new configuration and start the new gridinit
 
 mkdir -p "$OIO" && cd "$OIO" && (rm -rf sds.conf sds/{conf,data,run,logs})
-${PREFIX}-bootstrap.py "$NS" "$IP" -d ${BOOTSTRAP_CONFIG} > /tmp/oio-bootstrap.$$
+bootstrap_opt=
+if [[ -n "${PORT}" ]] ; then bootstrap_opt="${bootstrap_opt} --port ${PORT}" ; fi
+if [[ -n "${DATADIR}" ]] ; then bootstrap_opt="${bootstrap_opt} --data ${DATADIR}" ; fi
+if [[ -n "${PROFILE}" ]] ; then bootstrap_opt="${bootstrap_opt} --profile ${PROFILE}" ; fi
+${PREFIX}-bootstrap.py $bootstrap_opt -d ${BOOTSTRAP_CONFIG} "$NS" "$IP" > /tmp/oio-bootstrap.$$
 
-# Variables
-# PROXY
-# REPLI_DIRECTORY
 
 . /tmp/oio-bootstrap.$$
 rm -f /tmp/oio-bootstrap.$$
@@ -134,13 +132,11 @@ gridinit -s OIO,gridinit -d ${SDS}/conf/gridinit.conf
 # Initiate Zookeeper (if necessary)
 ZK=$(${PREFIX}-cluster --local-cfg | grep "$NS/zookeeper" ; exit 0)
 if [ -n "$ZK" ] ; then
-    opts=
+    opts=--lazy
     for srvtype in ${AVOID} ; do opts="${opts} --avoid=${srvtype}" ; done
     if [ $ZKSLOW -ne 0 ] ; then opts="${opts} --slow" ; fi
-    if [ $ZKRESET -ne 0 ] ; then
-        zk-reset.py "$NS" ;
-    fi
-    zk-bootstrap.py $opts "$NS"
+	zk-reset.py "$NS" ;
+    zk-bootstrap.py --lazy $opts "$NS"
 fi
 
 
@@ -154,31 +150,29 @@ while ! [ -e "$GRIDINIT_SOCK" ] ; do
 done
 pidof_gridinit=$(pgrep -u "$UID" --full gridinit)
 
-timestamp
+
+echo -e "\n### Start gridinit and wait for the services to register"
 gridinit_cmd -S "$GRIDINIT_SOCK" reload >/dev/null
-gridinit_cmd -S "$GRIDINIT_SOCK" start "@${NS}"
-timestamp
-COUNT=$(${PREFIX}-test-config.py -c -t meta2 -t rawx -t sqlx)
-wait_for_srvtype "(sqlx|rawx|meta2)" $((2 + COUNT))
-timestamp
-COUNT=$(${PREFIX}-test-config.py -c -t meta0 -t meta1)
-wait_for_srvtype "(meta0|meta1)" $((1 + COUNT))
+gridinit_cmd -S "$GRIDINIT_SOCK" start "@${NS}" >/dev/null
 
-timestamp
+COUNT=$(${PREFIX}-test-config.py -c -t meta2 -t rawx -t sqlx -t meta0 -t meta1 -t rdir)
+$cmd_openio cluster wait -d 30 -u -n "$COUNT" sqlx rawx meta2 meta0 meta1 rdir
 
-openio \
-	--oio-ns "$NS" -v directory bootstrap \
+
+echo -e "\n### Init the meta0/meta1 directory"
+$cmd_openio directory bootstrap --check --no-rdir \
 	--replicas $(${PREFIX}-test-config.py -v directory_replicas)
 
-timestamp
-# unlock all services
-openio --oio-ns "$NS" cluster unlockall
-$PREFIX-wait-scored.sh -n "$NS" -t 60
-$PREFIX-flush-all.sh -n "$NS"
+echo -e "\n### Assign rdir services"
+$cmd_openio volume admin bootstrap
 
-timestamp
+echo -e "\n### Wait for the services to have a score"
+$cmd_openio cluster unlockall
+$PREFIX-flush-all.sh -n "$NS" >/dev/null
+
+echo -e "\n### Congrats, it's a NS"
 find $SDS -type d | xargs chmod a+rx
 gridinit_cmd -S "$GRIDINIT_SOCK" status2
 $PREFIX-cluster -r "$NS"
 
-timestamp
+echo -e "\nexport OIO_NS=$NS OIO_ACCT=ACCT-$RANDOM"
