@@ -1,20 +1,20 @@
 /*
 oio-tool, a CLI tool of OpenIO
-Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015-2016 OpenIO, modified as part of OpenIO SDS
+Copyright (C) 2014 Worldline, as part of Redcurrant
+Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 
-This library is free software; you can redistribute it and/or
-modify it under the terms of the GNU Lesser General Public
-License as published by the Free Software Foundation; either
-version 3.0 of the License, or (at your option) any later version.
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU Affero General Public License as
+published by the Free Software Foundation, either version 3 of the
+License, or (at your option) any later version.
 
-This library is distributed in the hope that it will be useful,
+This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Lesser General Public License for more details.
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU Affero General Public License for more details.
 
-You should have received a copy of the GNU Lesser General Public
-License along with this library.
+You should have received a copy of the GNU Affero General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <glib.h>
@@ -24,17 +24,8 @@ License along with this library.
 static void
 _dump_cid (const char *s)
 {
-	static gchar bad[] = {
-		'-', '-', '-', '-', '-', '-', '-', '-',
-		'-', '-', '-', '-', '-', '-', '-', '-',
-		'-', '-', '-', '-', '-', '-', '-', '-',
-		'-', '-', '-', '-', '-', '-', '-', '-',
-		'-', '-', '-', '-', '-', '-', '-', '-',
-		'-', '-', '-', '-', '-', '-', '-', '-',
-		'-', '-', '-', '-', '-', '-', '-', '-',
-		'-', '-', '-', '-', '-', '-', '-', '-',
-		0
-	};
+	static gchar bad[] =
+			"----------------------------------------------------------------";
 
 	char str_id[128];
 	const char *dst;
@@ -168,11 +159,13 @@ _ping(gchar *dest, gchar *to)
 	GError *err = gridd_client_exec(dest, timeout, encoded);
 	gint64 end = oio_ext_monotonic_time();
 	if (err) {
-		g_print("KO (%d) %s\n", err->code, err->message);
+		g_print("PING KO (%d) %s\n", err->code, err->message);
 		g_clear_error(&err);
 		return 1;
 	} else {
-		g_print("OK %lfs\n", (end - start) / (gdouble)G_TIME_SPAN_SECOND);
+		gdouble ping_ms = (end - start) / (gdouble)G_TIME_SPAN_MILLISECOND;
+		g_print("PING OK %.3lfms | time=%.3lfms;;;0.0;%.3lf\n",
+				ping_ms, ping_ms, timeout * 1000.0);
 		return 0;
 	}
 }
@@ -198,6 +191,36 @@ _info(const char *dest)
 	if (out) g_byte_array_free(out, TRUE);
 }
 
+static int
+_config(const char *ns, gboolean raw, int nbfiles, char **pfiles)
+{
+	GSList *files = NULL;
+	for (int i=0; i<nbfiles; i++)
+		files = g_slist_append(files, pfiles[i]);
+	oio_var_reset_all();
+	const gboolean known = oio_var_value_with_files(ns, TRUE, files);
+	g_slist_free(files);
+
+	if (!known) {
+		g_printerr("Unknown namespace: [%s]\n", ns);
+		return 1;
+	}
+
+	if (raw) {
+		void _on_var(const char *k, const char *v) {
+			g_print("%s=%s\n", k, v);
+		}
+		oio_var_list_all(_on_var);
+	} else {
+		GString *out = oio_var_list_as_json();
+		g_print("%s", out->str);
+		g_string_free(out, TRUE);
+	}
+
+	return 0;
+}
+
+
 static void
 _print_loc(const char *dotted_loc)
 {
@@ -205,31 +228,49 @@ _print_loc(const char *dotted_loc)
 	g_print("%s\t%"OIO_LOC_FORMAT"\n", dotted_loc, loc);
 }
 
+static void
+_print_usage(const char *name)
+{
+	g_printerr ("Usage:\n");
+	g_printerr ("\nDump a view of all the variables known in the central "
+			"configurationn\n");
+	g_printerr (" %s config NS [--raw] PATH...\n", name);
+	g_printerr ("\nPrint hex representation of the address\n");
+	g_printerr (" %s addr IP:PORT\n", name);
+	g_printerr ("\nPrint hex representation of container ID\n");
+	g_printerr (" %s cid  OIOURL\n", name);
+	g_printerr ("\nGenerate container names with same hexadecimal prefix\n");
+	g_printerr (" %s hash ACCOUNT [PREFIX]\n", name);
+	g_printerr ("\nPing a service (timeout is 10.0 seconds by default)\n");
+	g_printerr (" %s ping IP:PORT [TIMEOUT]\n", name);
+	g_printerr ("\nGet free CPU, IO and space statistics\n");
+	g_printerr (" %s stat [path]...\n", name);
+	g_printerr ("\nCompute 64b integer location from dotted string\n");
+	g_printerr (" %s location DOTTED_STRING...\n", name);
+	g_printerr ("\nCall a handler that always redirects, and succeeds if the "
+			"client ends up with a\n'Too many redirections' error\n");
+	g_printerr ("  %s redirection IP:PORT\n", name);
+}
+
 int
 main (int argc, char **argv)
 {
 	if (argc < 2) {
-		g_printerr ("Usage:\n");
-		g_printerr ("\nPrint hex representation of the address\n");
-		g_printerr ("  %s addr IP:PORT\n", argv[0]);
-		g_printerr ("\nPrint hex representation of container ID\n");
-		g_printerr ("  %s cid  OIOURL\n", argv[0]);
-		g_printerr ("\nGenerate container names with same hexadecimal prefix\n");
-		g_printerr ("  %s hash ACCOUNT [PREFIX]\n", argv[0]);
-		g_printerr ("\nPing a service\n");
-		g_printerr ("  %s ping IP:PORT [TIMEOUT]\n", argv[0]);
-		g_printerr ("\nGet free CPU, IO and space statistics\n");
-		g_printerr ("  %s stat [path]...\n", argv[0]);
-		g_printerr ("\nCompute 64b integer location from dotted string\n");
-		g_printerr ("  %s location DOTTED_STRING...\n", argv[0]);
-		g_printerr ("\nCalls a handler that always redirect, and succeeds if the "
-				"client ends up with a\n'Too many redirections' error\n");
-		g_printerr ("  %s redirection IP:PORT\n", argv[0]);
+		_print_usage(argv[0]);
 		return 2;
 	}
+	oio_log_lazy_init();
 	oio_ext_set_random_reqid ();
 
-	if (!strcmp("addr", argv[1])) {
+	if (!strcmp("config", argv[1])) {
+		gboolean raw = FALSE;
+		int idx_files = 3;
+		if (argc > idx_files && !strcmp("--raw", argv[idx_files])) {
+			idx_files ++;
+			raw = TRUE;
+		}
+		return _config(argv[2], raw, argc - idx_files, argv + idx_files);
+	} else if (!strcmp("addr", argv[1])) {
 		for (int i=2; i<argc ;++i)
 			_dump_addr (argv[i]);
 		return 0;
@@ -264,6 +305,11 @@ main (int argc, char **argv)
 			return 1;
 		}
 		return _redirect(argv[2], "5.0");
+	} else if (!strcmp("-h", argv[1]) ||
+			!strcmp("help", argv[1]) ||
+			!strcmp("--help", argv[1])) {
+		_print_usage(argv[0]);
+		return 0;
 	}
 
 	return 1;
