@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 
-# zk-bootstrap.py, a script initating a Zookeeper instance for OpenIO SDS.
-# Copyright (C) 2014 Worldine, original work as part of Redcurrant
-# Copyright (C) 2015-2016 OpenIO, modified as part of OpenIO SDS
+# zk-bootstrap.py
+# Copyright (C) 2014 Worldline, as part of Redcurrant
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -60,15 +60,8 @@ def batch_create(zh, batch):
             rc, zrc, ignored = args
             if rc != 0:
                 print "zookeeper.acreate() error"
-            else:
-                if zrc == 0:
-                    # print 'create/set('+path+') : OK'
-                    pass
-                elif zrc == zookeeper.NODEEXISTS:
-                    # print 'create/set('+path+') : ALREADY'
-                    pass
-                else:
-                    print 'create/set('+path+') : FAILED'
+            elif zrc != 0 and zrc != zookeeper.NODEEXISTS:
+                print 'create/set('+path+') : FAILED'
             sem.release()
         zookeeper.acreate(zh, path, data, acl_openbar, 0, completion)
     for path, data in batch:
@@ -82,7 +75,7 @@ def batch_create(zh, batch):
 def create_tree(zh, nodes, options):
     N, ok, ko = 2048, 0, 0
     if options.SLOW is not None and options.SLOW:
-        N = 256
+        N = 8
     for batch in batch_split(nodes, N):
         pre = now()
         o, k = batch_create(zh, batch)
@@ -128,11 +121,15 @@ def main():
     parser = OptionParser()
     parser.add_option(
             '-v', '--verbose',
-            action="store_true", dest="flag_verbose",
+            action="store_true", dest="flag_verbose", default=False,
             help='Triggers debugging traces')
+    parser.add_option(
+            '--lazy',
+            action="store_true", dest="LAZY", default=False,
+            help='Quickly check if things seem OK.')
     parser.add_option(
             '--slow',
-            action="store_true", dest="SLOW",
+            action="store_true", dest="SLOW", default=False,
             help='Send small batches to avoid timeouts on slow hosts.')
     parser.add_option(
             '--avoid',
@@ -159,15 +156,29 @@ def main():
     ns = args[1]
     cnxstr = load_namespace_conf(ns)['zookeeper']
     zookeeper.set_debug_level(zookeeper.LOG_LEVEL_INFO)
-    zh = zookeeper.init(cnxstr)
+    for shard in cnxstr.split(";"):
+        logging.info("ZK=%s", shard)
+        zh = zookeeper.init(shard)
 
-    # synchronous creation of the root
-    try:
-        zookeeper.create(zh, PREFIX, '', acl_openbar, 0)
-    except zookeeper.NodeExistsException:
-        pass
-    create_tree(zh, namespace_tree(ns, options), options)
-    zookeeper.close(zh)
+        # synchronous creation of the root
+        try:
+            zookeeper.create(zh, PREFIX, '', acl_openbar, 0)
+        except zookeeper.NodeExistsException:
+            pass
+
+        missing = True
+        if options.LAZY:
+            _m = False
+            for t, _, _ in SRVTYPES:
+                try:
+                    _, _ = zookeeper.get(zh, PREFIX_NS + '/' + ns + '/el/' + t)
+                except Exception:
+                    _m = True
+            missing = _m
+
+        if missing:
+            create_tree(zh, namespace_tree(ns, options), options)
+        zookeeper.close(zh)
 
 
 if __name__ == '__main__':
