@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 
 # zk-reset.py, a script resetting a Zookeeper instance for OpenIO SDS.
-# Copyright (C) 2014 Worldine, original work as part of Redcurrant
-# Copyright (C) 2015-2016 OpenIO, modified as part of OpenIO SDS
+# Copyright (C) 2014 Worldline, as part of Redcurrant
+# Copyright (C) 2015-2017 OpenIO SAS, as part of OpenIO SDS
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as
@@ -30,7 +30,7 @@ def delete_children(zh, path):
             p = path + '/' + n
             delete_children(zh, p)
             zookeeper.delete(zh, p)
-    except:
+    except Exception:
         pass
 
 
@@ -66,16 +66,18 @@ def main():
 
     ns = args[1]
     cnxstr = load_namespace_conf(ns)['zookeeper']
-
     zookeeper.set_debug_level(zookeeper.LOG_LEVEL_INFO)
-    zh = zookeeper.init(cnxstr)
-    if options.flag_all:
-        logging.warn("FLUSHING all the oio-sds entries in the ZK server")
-        delete_children(zh, "/hc")
-    else:
-        logging.info("Cleaning only the meta0 registrations in ZK server")
-        delete_children(zh, "/hc/ns/"+ns+"/srv/meta0")
-    zookeeper.close(zh)
+
+    for shard in cnxstr.split(";"):
+        logging.info("ZK=%s", shard)
+        zh = zookeeper.init(shard)
+        if options.flag_all:
+            logging.warn("FLUSHING all the oio-sds entries in the ZK server")
+            delete_children(zh, "/hc")
+        else:
+            logging.info("Cleaning only the meta0 registrations in ZK server")
+            delete_children(zh, "/hc/ns/"+ns+"/srv/meta0")
+        zookeeper.close(zh)
 
 
 if __name__ == '__main__':
